package com.itopview.wm.cm.customplugin;

import org.reflections.Reflections;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * 工厂类
 */
public class YemFactory {
    private static final Map<String, YemInterface> implementations = new HashMap<>();

    public YemFactory(String path){
        // 扫描带有@YemImplementation注解的类，并将其实例化并放入implementations Map中
        Reflections reflections = new Reflections(path);
        Set<Class<?>> annotatedClasses = reflections.getTypesAnnotatedWith(YemImplementation.class);
        for (Class<?> annotatedClass : annotatedClasses) {
            YemImplementation annotation = annotatedClass.getAnnotation(YemImplementation.class);
            String implementationKey = annotation.value();
            try {
                YemInterface implementation = (YemInterface) annotatedClass.newInstance();
                implementations.put(implementationKey, implementation);
            } catch (InstantiationException | IllegalAccessException e) {
                e.printStackTrace();
            }
        }
    }

    public static YemInterface getImplementation(String key) {
        return implementations.get(key);
    }
}

