package com.itopview.wm.cm.formplugin.form;

import java.util.EventObject;
import java.util.HashMap;
import java.util.Map;

import com.itopview.wm.sys.utils.MetaPropertyHelper;
import com.itopview.wm.utils.YEM;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.CodeEdit;
import kd.bos.form.control.Control;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;

/**
 * 条款对照
 * 1、选择字段标识带出字段名称和显示格式
 *
 * @author zhangh
 */
public class ClauseContrastFormPlugin extends AbstractBillPlugIn {

    @Override
    public void registerListener(EventObject e) {
        this.addClickListeners(new String[]{"tpv_fieldkey"});
        this.addClickListeners(new String[]{"tpv_codeedit"});
    }

    @Override
    public void click(EventObject evt) {
        String key = ((Control) evt.getSource()).getKey();
        switch (key) {
            case "tpv_fieldkey":
                this.clickBillCol(evt, key, "tpv_billkey", "number");
                break;
            case "tpv_codeedit":
                this.clickCodeEdit();
                break;
        }
    }

    /**
     * @Title: clickCodeEdit
     * @Descrition: 打开代码编辑器
     * @eturn: a
     */
    public void clickCodeEdit() {
        IFormView view = this.getView();
        FormShowParameter showParameter = new FormShowParameter();
        showParameter.setFormId("tpv_tpl_codeforms");
        Map<String, Object> map = new HashMap();
        Object jsCode = this.getModel().getValue("tpv_bcode_tag");
        map.put("tpv_codeeditap", jsCode);
        showParameter.setCustomParams(map);
        showParameter.setStatus(OperationStatus.ADDNEW);
        showParameter.setCloseCallBack(new CloseCallBack(this, "tpv_updatestatus"));
        showParameter.getOpenStyle().setShowType(ShowType.Modal);
        this.getView().showForm(showParameter);
    }

    /**
     * 点击问本框按钮，弹字段选择器动态表单
     *
     * @param evt
     * @param key      文本框按钮的字段标识
     * @param baseCol  需求单据字段的标识
     * @param childCol 需求单据编码标识
     */
    private void clickBillCol(EventObject evt, String key, String baseCol, String childCol) {
        DynamicObject requireBill = MetaPropertyHelper.getAndCheckF7Value(this.getView(), baseCol);
        if (requireBill != null) {//进行判空处理
            String formKey = requireBill.getString(childCol);
            MetaPropertyHelper.showPropertySelect(this, key, formKey, false);
        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent e) {
        String key = e.getActionId();
        switch (key) {
            case "tpv_fieldkey":
                this.callBackEntryData(e);
                break;
            case "tpv_updatestatus":
                HashMap<String, Object> values = (HashMap<String, Object>) e.getReturnData();
                if (YEM.isNotEmpty(values)){
                    String fullname = (String) values.get("tpv_codeeditap");
                    this.getModel().setValue("tpv_bcode_tag", fullname);
                    if (YEM.isNotEmpty(fullname)){
                        this.getModel().setValue("tpv_codeedit", "公式");
                    }
                }
            default:
                break;
        }
    }


    /**
     * 将数据返回条款对照
     *
     * @param e
     */
    private void callBackEntryData(ClosedCallBackEvent e) {
        DynamicObjectCollection data = (DynamicObjectCollection) e.getReturnData();
        if (YEM.isNotEmpty(data)) {
            for (int i = 0; i < data.size(); ++i) {
                DynamicObject info = data.get(i);
                String fullnumber = info.getString("tpv_number");
                String fullname = info.getString("tpv_name");
                String classSimpleName = info.getString("tpv_datatype");
                this.getModel().setValue("tpv_fieldkey", fullnumber);//字段标识
                this.getModel().setValue("name", fullname);//字段名称
                this.getModel().setValue("tpv_format", classSimpleName);//字段类型
                this.getModel().setValue("tpv_field", info.getString("tpv_fullnumber"));//字段全称
            }
        }
    }

    @Override
    public void afterBindData(EventObject e) {
        this.mustInputFieldType();
    }

    /**
     * 根据条件进行必录，如果字段标识为空，字段类型基础资料必录
     */
    private void mustInputFieldType() {
        String tpv_fieldkey = (String) this.getModel().getValue("tpv_fieldkey");
        BasedataEdit tpv_fieldtype = this.getControl("tpv_fieldtype");
        if (StringUtils.isNotEmpty(tpv_fieldkey)) {
            tpv_fieldtype.setMustInput(false);
        } else {
            tpv_fieldtype.setMustInput(true);
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        String name = e.getProperty().getName();
        switch (name) {
            case "tpv_fieldkey":
                this.mustInputFieldType();
                break;

            case "tpv_billkey"://选择需求单据之后，字段标识等字段清空，重新选择
                this.getModel().setValue("tpv_fieldkey", "");
                this.getModel().setValue("tpv_format", "");
                this.getModel().setValue("tpv_field", "");
				break;
            case "tpv_codeedit":
				Object value = this.getModel().getValue("tpv_codeedit");
				if (YEM.isEmpty(value)){
					this.getModel().setValue("tpv_bcode_tag", null);
                    this.getModel().setValue("tpv_codeedit", null);
				}
                break;
            default:
                break;
        }
    }

}
