package com.itopview.wm.cm.formplugin.form;

import java.util.*;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.sys.utils.RuleSettingsHelper;
import com.itopview.wm.utils.NumberChineseUtil;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.Control;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;

/**
 * 合同模板(主插件)
 * @author zhangh
 *
 */

public class ContractTemplateFromPlugin extends AbstractBillPlugIn implements BeforeF7SelectListener{

	@Override
	public void beforeF7Select(BeforeF7SelectEvent e) {
 		String name = e.getProperty().getName();
		switch (name) {
		case "tpv_org":
//			this.setOrgF7Filter(e);
			break;
		case "tpv_termname":
//			this.setTermF7Filter(e);
			break;
		default:
			break;
		}		
	}

	/**
	 * 对条款名称进行过滤(作废)
	 * @param e
	 */
	public void setTermF7Filter(BeforeF7SelectEvent e) {
		ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
		showParameter.getListFilterParameter().getQFilters().clear();
		long currentUserId = UserServiceHelper.getCurrentUserId();
		DynamicObject bos_user = BusinessDataServiceHelper.loadSingleFromCache(currentUserId, "bos_user");
		if (bos_user != null) {
			List<Long> dept = new ArrayList<>();
			DynamicObjectCollection entryentity = bos_user.getDynamicObjectCollection("entryentity");
			for (DynamicObject row : entryentity) {
				dept.add(row.getDynamicObject("dpt").getLong("id"));
			}
			
			if (dept.size() > 0) {
				QFilter qFilter = new QFilter("tpv_dept", QCP.in, dept).or(new QFilter("tpv_sharedscope.fbasedataid.id", QCP.in, dept));
				showParameter.getListFilterParameter().setFilter(qFilter);
			}
		}
	}

	/**
	 * 所属团队过滤(作废)
	 * @param e
	 */
	public void setOrgF7Filter(BeforeF7SelectEvent e) {
		ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
		showParameter.getListFilterParameter().getQFilters().clear();
		showParameter.getListFilterParameter().setFilter(new QFilter("orgpattern.name", QCP.in, new String[] {"事业部","公司"}));
	}
	
	@Override
	public void registerListener(EventObject e) {
		((BasedataEdit) this.getControl("tpv_org")).addBeforeF7SelectListener(this);
		((BasedataEdit) this.getControl("tpv_dept")).addBeforeF7SelectListener(this);
		((BasedataEdit) this.getControl("tpv_termname")).addBeforeF7SelectListener(this);
		((BasedataEdit) this.getControl("tpv_sharedscope")).addBeforeF7SelectListener(this);
		this.addItemClickListeners(new String[]{"advcontoolbarap"});
		//LinRui -add
		this.addClickListeners(new String[]{"tpv_callcond"});
	}
	
	@Override
	public void propertyChanged(PropertyChangedArgs e) {
		String name = e.getProperty().getName();
		int rowIndex = this.getModel().getEntryCurrentRowIndex("tpv_contractterms");
		switch (name) {
		case "tpv_termname":
			this.setTermData(e);
			break;
		case "tpv_clausetype":
//			this.setNewTermBillno();
			break;
		case "tpv_contentbefore":
		case "tpv_clausescontent":
//			this.setIsChange();
			String tpv_clausescontent = (String) this.getModel().getValue("tpv_clausescontent", rowIndex);
			this.getModel().setValue("tpv_clausescontent_tag", tpv_clausescontent, rowIndex);
			break;
		case "tpv_clausescontent_tag":
			String tpv_clausescontent_tag = (String) this.getModel().getValue("tpv_clausescontent_tag", rowIndex);
			this.getModel().setValue("tpv_clausescontent", tpv_clausescontent_tag, rowIndex);
			break;
		//begin LinRui
		case "tpv_callcond":
			this.getModel().setValue("tpv_checkjson",null, rowIndex);
			break;
		//end LinRui
		default:
			break;
		}
	}

	/**
	 * 修改前条款和修改后条款不一致，是否调整改为true(作废)
	 */
	public void setIsChange() {
		int row = this.getModel().getEntryCurrentRowIndex("tpv_contractterms");
		String tpv_contentbefore = (String) this.getModel().getValue("tpv_contentbefore", row);
		String tpv_clausescontent = (String) this.getModel().getValue("tpv_clausescontent", row);
		if (StringUtils.equals(tpv_contentbefore, tpv_clausescontent)) {
			this.getModel().setValue("tpv_adjust", false, row);
		}else {
			this.getModel().setValue("tpv_adjust", true, row);
		}
	}
	
	/**
	 * 根据序号和条款类型给条款正式编号赋值
	 */
	private void setNewTermBillno() {
		DynamicObjectCollection tpv_contractterms = this.getModel().getEntryEntity("tpv_contractterms");
		for (int i = 0; i < tpv_contractterms.size(); i++) {
			DynamicObject contractterms = tpv_contractterms.get(i);
			DynamicObject tpv_termname = contractterms.getDynamicObject("tpv_termname");
			if (tpv_termname != null) {
				tpv_termname = BusinessDataServiceHelper.loadSingleFromCache(tpv_termname.getLong("id"), "tpv_cm_contractterms");
				if (i == 0) {
					this.getModel().setValue("tpv_termbillno", "二、" + tpv_termname.getString("tpv_clausechina"), i);
				}else {
					DynamicObject contractterms1 = tpv_contractterms.get(i - 1);
					String tpv_termbillno = contractterms1.getString("tpv_termbillno");
					if (StringUtils.isNotEmpty(tpv_termbillno)) {
						String termbillno = tpv_termbillno.split("、")[0];
						int num = NumberChineseUtil.chineseToNumber(termbillno) + 1;
						String format = NumberChineseUtil.format(num, false, false);
						this.getModel().setValue("tpv_termbillno", format + "、" + tpv_termname.getString("tpv_clausechina"), i);
					}
				}
			}
		}
	}

	/**
	 * 将条款名称中的信息携带到分录信息中
	 * @param e 
	 */
	private void setTermData(PropertyChangedArgs e) {
		ChangeData[] changeSet = e.getChangeSet();
		int index = changeSet[0].getRowIndex();
		DynamicObject tpv_termname = (DynamicObject) this.getModel().getValue("tpv_termname", index);
		if (tpv_termname != null) {
			tpv_termname = BusinessDataServiceHelper.loadSingleFromCache(tpv_termname.getPkValue(),
					tpv_termname.getDynamicObjectType().getName());
			this.getModel().setValue("tpv_clausetype", tpv_termname.getDynamicObject("tpv_clausetype"), index);
			this.getModel().setValue("tpv_clausescontent", tpv_termname.getString("tpv_clausecontentchina"), index);
			this.getModel().setValue("tpv_clausescontent_tag", tpv_termname.getString("tpv_clausecontentchina"), index);
			this.getModel().setValue("tpv_contentbefore", tpv_termname.getString("tpv_clausecontentchina"), index);
			this.getModel().setValue("tpv_contentbefore_tag", tpv_termname.getString("tpv_clausecontentchina"), index);
			this.getModel().setValue("tpv_describe", tpv_termname.getString("tpv_textfield"), index);
			LocaleString locStr = new LocaleString();
			locStr.setLocaleValue_zh_CN(tpv_termname.getString("tpv_clausechina"));
			locStr.setLocaleValue_en(tpv_termname.getString("tpv_clauseenglish"));
			this.getModel().setValue("tpv_clausename", locStr, index);
		}
		
		setNewTermBillno();
	}
	
	@Override
	public void itemClick(ItemClickEvent evt) {
		String key = evt.getItemKey();
		switch (key) {
		case "tb_del"://删行
		case "tpv_moveup"://上移
		case "tpv_shiftdown"://下移
			setNewTermBillno();
			break;

		default:
			break;
		}
	}
	
	@Override
	public void afterBindData(EventObject e) {
		/*String name = this.getModel().getDataEntityType().getName();
		NewVersionHelper newVersionHelper = new NewVersionHelper(name);
		String billstatus = (String) this.getModel().getValue("billstatus");
		if (newVersionHelper.isChanged() && "A".equals(billstatus)) {
			this.getModel().setValue("tpv_newtemplate", false);
			DynamicObjectCollection tpv_clauseimport = this.getModel().getEntryEntity("tpv_contractterms");
			for (int i = 0; i < tpv_clauseimport.size(); i++) {
				this.getModel().setValue("tpv_adjust", false, i);
			}
		}*/
	}
	
	@Override
	public void afterDoOperation(AfterDoOperationEventArgs e) {
		switch (e.getOperateKey()) {
		case "unaudit":
			if (e.getOperationResult().isSuccess()) {
				this.getView().invokeOperation("refresh");
			}
			break;

		default:
			break;
		}
	}

	//LinRui -add
	@Override
	public void click(EventObject evt) {
		super.click(evt);
		IFormView view = this.getView();
		IDataModel model = view.getModel();
		String key = ((Control) evt.getSource()).getKey();
		switch (key) {
			case "tpv_callcond" ://调用条件
				int index = this.getModel().getEntryCurrentRowIndex("tpv_entryentity"); //选择当行索引
				FormShowParameter showParameter = new FormShowParameter();
				showParameter.setStatus(OperationStatus.ADDNEW);
				showParameter.getOpenStyle().setShowType(ShowType.Modal);
				showParameter.setFormId("tpv_cm_concallcond"); //合同模板调用条件
				Map<String, Object> map = new HashMap<>();
				String value = (String) model.getValue("tpv_checkjson");
				map.put("data",value);
				showParameter.setCustomParams(map);
				showParameter.setCloseCallBack(new CloseCallBack(this, "tpv_callcond"));
				this.getView().showForm(showParameter);
				break;
		}
	}

	//LinRui -add
	@Override
	public void closedCallBack(ClosedCallBackEvent e) {
		super.closedCallBack(e);
		String key = e.getActionId();
		switch (key) {
			case "tpv_callcond":
				this.callBack4CheckRule(e);
				break;
		}
	}
	//LinRui -add
	private void callBack4CheckRule(ClosedCallBackEvent e) {
		String actionId = e.getActionId();
		Object returnData = e.getReturnData();
		if("tpv_callcond".equals(actionId) && returnData != null) {
			JSONObject jsonObject = (JSONObject) returnData;
			JSONArray jsonArray = (JSONArray) jsonObject.get("listCallcond");
			if (null != jsonArray && jsonArray.size() > 0) {
				IDataModel model = this.getModel();
				JSONObject jsonObj = (JSONObject) returnData;
				int rowIndex = model.getEntryCurrentRowIndex("tpv_contractterms");
//				model.beginInit();
				model.setValue("tpv_callcond", "请查看详细", rowIndex);
//				model.endInit();
				model.setValue("tpv_checkjson", jsonObj.toJSONString(), rowIndex);
			}
		} else {
			return;
		}
	}
}
