package com.itopview.wm.cm.formplugin.form;

import kd.bos.base.AbstractBasePlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.resource.ResManager;
import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.designer.botp.BillTreeBuildParameter;
import kd.bos.entity.EntityType;
import kd.bos.entity.EntityTypeUtil;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.filter.FilterCondition;
import kd.bos.entity.tree.TreeNode;
import kd.bos.exception.KDBizException;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.ShowType;
import kd.bos.form.control.Control;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.FilterGrid;
import kd.bos.form.control.events.RowClickEvent;
import kd.bos.form.control.events.RowClickEventListener;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.plugin.IFormPlugin;
import kd.bos.servicehelper.MetadataServiceHelper;
import kd.bos.util.StringUtils;
import kd.mpscmm.msplan.mrp.business.helper.BillFieldEntityHelper;
import kd.sdk.plugin.Plugin;

import java.util.*;

/**
 * 计息策略 表单插件
 */
public class InterestBearStrategyEdit extends AbstractBasePlugIn implements Plugin, RowClickEventListener {
    private static final String RECEIVABLE_ENTRY = "tpv_ibs_receivableentry";//应收项目分录标识
    private static final String RECEIVED_ENTRY = "tpv_ibs_receivedentry";//已收项目分录标识

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
    }

    @Override
    public void beforeBindData(EventObject e) {
        super.beforeBindData(e);
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        //应收和已收分录行增加监听
        EntryGrid tpv_ibs_receivableentry = this.getView().getControl(RECEIVABLE_ENTRY);
        tpv_ibs_receivableentry.addRowClickListener(this);

        EntryGrid tpv_ibs_receivedentry = this.getView().getControl(RECEIVED_ENTRY);
        tpv_ibs_receivedentry.addRowClickListener(this);

        this.addClickListeners(new String[]{"tpv_ablesrcfieldname", "tpv_abledate", "tpv_ablecurrency", "tpv_ableamt",
                "tpv_edsrcfieldname", "tpv_eddate", "tpv_edcurrency", "tpv_edamt"});
    }

    @Override
    public void afterDoOperation(AfterDoOperationEventArgs e) {
        super.afterDoOperation(e);
        String key = e.getOperateKey();
        IDataModel model = this.getModel();
        switch (key) {
            case "ablesave"://应收过滤条件保存
                int rowIndex = model.getEntryCurrentRowIndex(RECEIVABLE_ENTRY);
                filterGridToConditionJson("tpv_ablefiltergridap", "tpv_ablefiltersjson", rowIndex);
                break;
            case "edsave"://已收过滤条件保存
                int rowIndex2 = model.getEntryCurrentRowIndex(RECEIVED_ENTRY);
                filterGridToConditionJson("tpv_edfiltergridap", "tpv_edfiltersjson", rowIndex2);
                break;
            default:
                break;
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String key = e.getProperty().getName();
        IDataModel model = this.getModel();
        int rowIndex = model.getEntryCurrentRowIndex(RECEIVABLE_ENTRY);
        int rowIndex2 = model.getEntryCurrentRowIndex(RECEIVED_ENTRY);
        switch (key) {
            case "tpv_ablebill"://应收单据
                model.setValue("tpv_ablesrcfieldname", null, rowIndex);
                model.setValue("tpv_abledate", null, rowIndex);
                model.setValue("tpv_ableamt", null, rowIndex);
                break;
            case "tpv_ablesrcfieldname"://应收合同关联字段
                String tpv_ablesrcfieldname = null == model.getValue("tpv_ablesrcfieldname", rowIndex) ? "" : model.getValue("tpv_ablesrcfieldname", rowIndex).toString();
                if ("".equals(tpv_ablesrcfieldname)) {
                    model.setValue("tpv_ablesrcfield", null, rowIndex);
                }
                break;
            case "tpv_abledate"://应收日期字段
                String tpv_abledate = null == model.getValue("tpv_abledate", rowIndex) ? "" : model.getValue("tpv_abledate", rowIndex).toString();
                if ("".equals(tpv_abledate)) {
                    model.setValue("tpv_abledatefield", null, rowIndex);
                }
                break;
            case "tpv_ablecurrency"://应收币别字段
                String tpv_ablecurrency = null == model.getValue("tpv_ablecurrency", rowIndex) ? "" : model.getValue("tpv_ablecurrency", rowIndex).toString();
                if ("".equals(tpv_ablecurrency)) {
                    model.setValue("tpv_ablecurrencyfield", null, rowIndex);
                }
                break;
            case "tpv_ableamt"://应收金额字段
                String tpv_ableamt = null == model.getValue("tpv_ableamt", rowIndex) ? "" : model.getValue("tpv_ableamt", rowIndex).toString();
                if ("".equals(tpv_ableamt)) {
                    model.setValue("tpv_ableamtfield", null, rowIndex);
                }
                break;
            case "tpv_edbill"://已收单据
                model.setValue("tpv_edsrcfieldname", null, rowIndex2);
                model.setValue("tpv_eddate", null, rowIndex2);
                model.setValue("tpv_edamt", null, rowIndex2);
                break;
            case "tpv_edsrcfieldname"://已收合同关联字段
                String tpv_edsrcfieldname = null == model.getValue("tpv_edsrcfieldname", rowIndex) ? "" : model.getValue("tpv_edsrcfieldname", rowIndex).toString();
                if ("".equals(tpv_edsrcfieldname)) {
                    model.setValue("tpv_edsrcfield", null, rowIndex);
                }
                break;
            case "tpv_eddate"://已收日期字段
                String tpv_eddate = null == model.getValue("tpv_eddate", rowIndex2) ? "" : model.getValue("tpv_eddate", rowIndex2).toString();
                if ("".equals(tpv_eddate)) {
                    model.setValue("tpv_eddatefield", null, rowIndex2);
                }
                break;
            case "tpv_edcurrency"://已收币别字段
                String tpv_edcurrency = null == model.getValue("tpv_edcurrency", rowIndex2) ? "" : model.getValue("tpv_edcurrency", rowIndex2).toString();
                if ("".equals(tpv_edcurrency)) {
                    model.setValue("tpv_edcurrencyfield", null, rowIndex2);
                }
                break;
            case "tpv_edamt"://已收金额字段
                String tpv_edamt = null == model.getValue("tpv_edamt", rowIndex2) ? "" : model.getValue("tpv_edamt", rowIndex2).toString();
                if ("".equals(tpv_edamt)) {
                    model.setValue("tpv_edamtfield", null, rowIndex2);
                }
                break;
            case "tpv_abledatatype"://应收取值类型
                break;
            case "tpv_eddatatype"://已收取值类型
                break;
            default:
                break;
        }
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        Control c = (Control) evt.getSource();
        String key = c.getKey().toLowerCase();
        switch (key) {
            case "tpv_ablesrcfieldname":
                this.openBillFieldList(RECEIVABLE_ENTRY, "tpv_ablebill", "tpv_ablesrcfieldname");
                break;
            case "tpv_abledate":
                this.openBillFieldList(RECEIVABLE_ENTRY, "tpv_ablebill", "tpv_abledate");
                break;
            case "tpv_ablecurrency":
                this.openBillFieldList(RECEIVABLE_ENTRY, "tpv_ablebill", "tpv_ablecurrency");
                break;
            case "tpv_ableamt":
                this.openBillFieldList(RECEIVABLE_ENTRY, "tpv_ablebill", "tpv_ableamt");
                break;
            case "tpv_edsrcfieldname":
                this.openBillFieldList(RECEIVED_ENTRY, "tpv_edbill", "tpv_edsrcfieldname");
                break;
            case "tpv_eddate":
                this.openBillFieldList(RECEIVED_ENTRY, "tpv_edbill", "tpv_eddate");
                break;
            case "tpv_edcurrency":
                this.openBillFieldList(RECEIVED_ENTRY, "tpv_edbill", "tpv_edcurrency");
                break;
            case "tpv_edamt":
                this.openBillFieldList(RECEIVED_ENTRY, "tpv_edbill", "tpv_edamt");
                break;
            default:
                break;
        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent e) {
        super.closedCallBack(e);
        String actionId = e.getActionId();
        Object returnData = e.getReturnData();
        switch (actionId) {
            case "tpv_ablesrcfieldname":
                setfieldvalue(returnData, RECEIVABLE_ENTRY, "tpv_ablebill", "tpv_ablesrcfieldname", "tpv_ablesrcfield");
                break;
            case "tpv_abledate":
                setfieldvalue(returnData, RECEIVABLE_ENTRY, "tpv_ablebill", "tpv_abledate", "tpv_abledatefield");
                break;
            case "tpv_ablecurrency":
                setfieldvalue(returnData, RECEIVABLE_ENTRY, "tpv_ablebill", "tpv_ablecurrency", "tpv_ablecurrencyfield");
                break;
            case "tpv_ableamt":
                setfieldvalue(returnData, RECEIVABLE_ENTRY, "tpv_ablebill", "tpv_ableamt", "tpv_ableamtfield");
                break;
            case "tpv_edsrcfieldname":
                setfieldvalue(returnData, RECEIVED_ENTRY, "tpv_edbill", "tpv_edsrcfieldname", "tpv_edsrcfield");
                break;
            case "tpv_eddate":
                setfieldvalue(returnData, RECEIVED_ENTRY, "tpv_edbill", "tpv_eddate", "tpv_eddatefield");
                break;
            case "tpv_edcurrency":
                setfieldvalue(returnData, RECEIVED_ENTRY, "tpv_edbill", "tpv_edcurrency", "tpv_edcurrencyfield");
                break;
            case "tpv_edamt":
                setfieldvalue(returnData, RECEIVED_ENTRY, "tpv_edbill", "tpv_edamt", "tpv_edamtfield");
                break;
            default:
                break;
        }
    }

    /**
     * 字段赋值
     *
     * @param returnData
     * @param entry
     * @param bill
     * @param fieldname
     * @param field
     */
    public void setfieldvalue(Object returnData, String entry, String bill, String fieldname, String field) {
        IDataModel model = this.getModel();
        int rowIndex = model.getEntryCurrentRowIndex(entry);
        if (returnData == null || kd.bos.dataentity.utils.StringUtils.isBlank((Object) returnData)) {
            return;
        }
        String fieldId = (String) returnData;
        MainEntityType billMainType = this.getMainEntityType(bill, rowIndex);
        String fieldCaption = BillFieldEntityHelper.buildPropFullCaption((EntityType) billMainType, (String) fieldId);
        this.getModel().setValue(fieldname, (Object) fieldCaption, rowIndex);
        this.getModel().setValue(field, (Object) fieldId, rowIndex);
    }

    @Override
    public void entryRowClick(RowClickEvent e) {
        EntryGrid entryGrid = (EntryGrid) e.getSource();
        String key = entryGrid.getEntryKey();
        int rowIndex = e.getRow();
        if ("tpv_ibs_receivableentry".equals(key)) {
            init_filtergridap("tpv_ablefiltergridap", "tpv_ablebill", rowIndex);
            conditionJsonToFilterGrid("tpv_ablefiltergridap", "tpv_ablefiltersjson", rowIndex);
        } else if ("tpv_ibs_receivedentry".equals(key)) {
            init_filtergridap("tpv_edfiltergridap", "tpv_edbill", rowIndex);
            conditionJsonToFilterGrid("tpv_edfiltergridap", "tpv_edfiltersjson", rowIndex);
        }

    }

    /**
     * 初始化通用过滤
     *
     * @param filtergridap
     * @param bill
     * @param rowIndex
     */
    public void init_filtergridap(String filtergridap, String bill, int rowIndex) {
        DynamicObject accEntity = (DynamicObject) this.getModel().getValue(bill, rowIndex);
        if (accEntity != null) {
            String entityName = accEntity.getPkValue().toString();
            FilterGrid filterGrid = this.getView().getControl(filtergridap);
            FilterCondition condition = new FilterCondition();
            filterGrid.SetValue(condition);
            MainEntityType entityType = MetadataServiceHelper.getDataEntityType(entityName);
            EntityTypeUtil helper = new EntityTypeUtil();
            List<Map<String, Object>> cols = helper.getFilterColumns(entityType, false);
            filterGrid.setFilterColumns(cols);
            filterGrid.setEntityNumber(entityName);
            this.getView().updateView(filtergridap);
        }
    }

    /**
     * 保存通用过滤至文本字段
     *
     * @param filtergridap
     * @param conditionJson_tag
     * @param rowIndex
     */
    public void filterGridToConditionJson(String filtergridap, String conditionJson_tag, int rowIndex) {
        FilterGrid filterGrid = this.getView().getControl(filtergridap);
        FilterCondition condition = filterGrid.getFilterGridState().getFilterCondition();
        String conditionJson = SerializationUtils.toJsonString(condition);
        this.getModel().setValue(conditionJson_tag, conditionJson, rowIndex);
    }

    /**
     * 文本字段转通用过滤
     *
     * @param filtergridap
     * @param conditionJson_tag
     * @param rowIndex
     */
    public void conditionJsonToFilterGrid(String filtergridap, String conditionJson_tag, int rowIndex) {
        String conditionJson = (String) this.getModel().getValue(conditionJson_tag, rowIndex);
        FilterCondition condition = null;
        if (StringUtils.isBlank(conditionJson)) {
            condition = new FilterCondition();
        } else {
            condition = SerializationUtils.fromJsonString(conditionJson, FilterCondition.class);
        }
        FilterGrid filterGrid = this.getView().getControl(filtergridap);
        filterGrid.SetValue(condition);
    }

    /**
     * @param entry:
     * @param bill:
     * @param field:
     * @return void
     * @author: chen
     * @date: 2023/12/15 16:27
     * @description: 开启单据字段选择面板
     */
    private void openBillFieldList(String entry, String bill, String field) {
        int rowIndex = this.getModel().getEntryCurrentRowIndex(entry);
        MainEntityType mainType = this.getMainEntityType(bill, rowIndex);
        BillTreeBuildParameter parameter = new BillTreeBuildParameter(mainType);
        parameter.setIncludePKField(true);
        parameter.setOnlyPhysicsField(false);
        TreeNode BillNode = this.buildBillTreeNodes(parameter, bill);
        String nodesJson = SerializationUtils.toJsonString((Object) BillNode);
        this.showBillFieldForm(nodesJson, field);
    }

    private MainEntityType getMainEntityType(String entityname, int rowIndex) {
        DynamicObject accEntity = (DynamicObject) this.getModel().getValue(entityname, rowIndex);
        if (accEntity != null) {
            Object billentityPkValue = accEntity.getPkValue();
            MainEntityType dataEntityType = MetadataServiceHelper.getDataEntityType((String) billentityPkValue.toString());
            return dataEntityType;
        }
        throw new KDBizException(ResManager.loadKDString((String) "单据为空，请先给单据赋值。", (String) "BillFieldTransferEdit_25", (String) "mpscmm-msplan-formplugin", (Object[]) new Object[0]));
    }

    private TreeNode buildBillTreeNodes(BillTreeBuildParameter parameter, String field) {
        MainEntityType mainType = parameter.getMainType();
        TreeNode billNode = new TreeNode("", mainType.getName(), mainType.getDisplayName().toString());
        billNode.setIsOpened(true);
        List<TreeNode> fldNodes = BillFieldEntityHelper.buildFldTreeNodes((BillTreeBuildParameter) parameter);
        HashMap<String, TreeNode> nodeMap = new HashMap<String, TreeNode>(16);
        for (TreeNode node : fldNodes) {
            nodeMap.put(node.getId(), node);
        }
        for (TreeNode node : fldNodes) {
            TreeNode parentNode = (TreeNode) nodeMap.get(node.getParentid());
            if (parentNode == null) {
                node.setParentid(billNode.getId());
                billNode.addChild(node);
                continue;
            }
            parentNode.addChild(node);
        }
        return billNode;
    }

    private void showBillFieldForm(String nodesJson, String custumparam) {
        FormShowParameter showParameter = new FormShowParameter();
        showParameter.setFormId("msplan_billfieldselect");
        showParameter.getCustomParams().put("treenodes", nodesJson);
        showParameter.setCloseCallBack(new CloseCallBack((IFormPlugin) this, custumparam));
        showParameter.getOpenStyle().setShowType(ShowType.Modal);
        this.getView().showForm(showParameter);
    }
}