package com.itopview.wm.cm.formplugin.form;

import com.itopview.wm.sys.utils.BothCardHelper;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

/*
    质押收货单
 */
public class PledgereceiptFormPlugin extends AbstractBillPlugIn implements BeforeF7SelectListener {

    //供应商添加
    private static final String ADD_GYS = "tpv_flex_addgys";
    //供应商信息
    private static final String INFO_GYS = "tpv_flex_gysinfo1";

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        BasedataEdit tpv_purchasecode = (BasedataEdit) this.getView().getControl("tpv_purcontractno");
        tpv_purchasecode.addBeforeF7SelectListener((BeforeF7SelectListener) this);
        BasedataEdit tpv_salescode = (BasedataEdit) this.getView().getControl("tpv_salecontractno");
        tpv_salescode.addBeforeF7SelectListener((BeforeF7SelectListener) this);
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String name = e.getProperty().getName();
        switch (name) {
            case "tpv_purcontractno":  //采购合同号
            case "tpv_salecontractno":  //销售合同号
                qFilterThisOrgName(e);  //过滤当前组织
                break;
        }
    }

    //过滤当前组织
    private void qFilterThisOrgName(BeforeF7SelectEvent e) {
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        List<QFilter> QFilters = new ArrayList<QFilter>();
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        DynamicObject org = dataEntity.getDynamicObject("org");
        if (org == null) {
            return;
        }
        Long id = (Long) org.get("id");
        QFilters.add(new QFilter("org", "=", id)); //设置过滤条件：组织
        QFilters.add(new QFilter("billstatus", "=", "C")); //单据状态：已审核
        showParameter.getListFilterParameter().setQFilters(QFilters);
    }

    //  银行账户bankaccount  银行信息单据体  bizparther_phone 工商登记电话  linkman 联系人  postal_code 电子邮箱  tx_register_no 纳税人识别号  bizparther_address  注册地址
    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        ChangeData cd = e.getChangeSet()[0];
        ChangeData[] changeData = e.getChangeSet();
        int rowIndex = changeData[0].getRowIndex();
        switch (name) {
            case "tpv_purcontractno":  //采购合同号
                getBillSupplierInfo();  //获取供应商信息
                break;
            case "tpv_salecontractno":  //销售合同号
                getBillCustomerInfo();  //获取客户信息
                break;
            case "tpv_asstacttype": // 往来户类型
                clearInfo();
            default:
                break;
        }
        this.getView().updateView();
    }

    private void clearInfo() {
        String value = (String) this.getModel().getValue("tpv_asstacttype");
        if ("0".equals(value)) {
            this.getModel().setValue("tpv_customer", null);  //清空字段[p
            this.getModel().setValue("tpv_customername", null);
            this.getModel().setValue("tpv_customertin", null);
            this.getModel().setValue("tpv_customerbank", null);
            this.getModel().setValue("tpv_customeracct", null);
            this.getModel().setValue("tpv_customerlinkman", null);
            this.getModel().setValue("tpv_customertel", null);
            this.getModel().setValue("tpv_customeraddr", null);
            this.getModel().setValue("tpv_customeremail", null);
            BothCardHelper.setCustomerLabel(this);  //抬头客户赋值
        } else if ("1".equals(value)) {
            this.getModel().setValue("tpv_supplier", null);  //清空字段
            this.getModel().setValue("tpv_suppliername", null);
            this.getModel().setValue("tpv_suppliertin", null);
            this.getModel().setValue("tpv_supplierbank", null);
            this.getModel().setValue("tpv_supplieracct", null);
            this.getModel().setValue("tpv_supplierlinkman", null);
            this.getModel().setValue("tpv_suppliertel", null);
            this.getModel().setValue("tpv_supplieraddr", null);
            this.getModel().setValue("tpv_supplieremail", null);
            BothCardHelper.setSupplierLabel(this);  //抬头供应商赋值
        }
    }

    //获取选择合同号对应单据里的客户信息，并赋值给当前单据页面抬头客户内容块
    private void getBillCustomerInfo() {
        this.getModel().setValue("tpv_customer", null);  //清空字段
        this.getModel().setValue("tpv_customername", null);
        this.getModel().setValue("tpv_customertin", null);
        this.getModel().setValue("tpv_customerbank", null);
        this.getModel().setValue("tpv_customeracct", null);
        this.getModel().setValue("tpv_customerlinkman", null);
        this.getModel().setValue("tpv_customertel", null);
        this.getModel().setValue("tpv_customeraddr", null);
        this.getModel().setValue("tpv_customeremail", null);
        DynamicObject tpv_salescode = (DynamicObject) this.getModel().getValue("tpv_salecontractno");
        //合同号为空返回
        if (tpv_salescode == null) {
            return;
        } else {
            DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle("tpv_dt_salescontract", new QFilter[]{new QFilter("id", QCP.equals, tpv_salescode.getLong("id"))});
            DynamicObject tpv_customer = dynamicObject.getDynamicObject("tpv_customer"); //客户
            if (tpv_customer == null) {
                this.getView().showMessage("选择的合同号里没有填写客户信息");
            } else {
                String tpv_customername = dynamicObject.getString("tpv_customername");  //客户名称
                String tpv_customertin = dynamicObject.getString("tpv_customertin");  //纳税人识别号(供应商)
                String tpv_customerbank = dynamicObject.getString("tpv_customerbank");  //开户行(供应商)
                String tpv_customeracct = dynamicObject.getString("tpv_customeracct");  //账号(供应商)
                String tpv_customerlinkman = dynamicObject.getString("tpv_customerlinkman");  //联系人(供应商)
                String tpv_customertel = dynamicObject.getString("tpv_customertel");  //电话(供应商)
                String tpv_customeraddr = dynamicObject.getString("tpv_customeraddr");  //地址(供应商)
                String tpv_customeremail = dynamicObject.getString("tpv_customeremail"); //E-Mail(供应商)
                this.getModel().setValue("tpv_customer", tpv_customer);
                this.getModel().setValue("tpv_customername", tpv_customername);
                this.getModel().setValue("tpv_customertin", tpv_customertin);
                this.getModel().setValue("tpv_customerbank", tpv_customerbank);
                this.getModel().setValue("tpv_customeracct", tpv_customeracct);
                this.getModel().setValue("tpv_customerlinkman", tpv_customerlinkman);
                this.getModel().setValue("tpv_customertel", tpv_customertel);
                this.getModel().setValue("tpv_customeraddr", tpv_customeraddr);
                this.getModel().setValue("tpv_customeremail", tpv_customeremail);
            }
            BothCardHelper.setCustomerLabel(this);  //抬头赋值

        }
    }

    //获取选择合同号对应单据里的供应商信息，并赋值给当前单据页面抬头供应商内容块
    private void getBillSupplierInfo() {
        /*clearFormInfo();*/
        this.getModel().setValue("tpv_supplier", null);  //清空字段
        this.getModel().setValue("tpv_suppliername", null);
        this.getModel().setValue("tpv_suppliertin", null);
        this.getModel().setValue("tpv_supplierbank", null);
        this.getModel().setValue("tpv_supplieracct", null);
        this.getModel().setValue("tpv_supplierlinkman", null);
        this.getModel().setValue("tpv_suppliertel", null);
        this.getModel().setValue("tpv_supplieraddr", null);
        this.getModel().setValue("tpv_supplieremail", null);
        DynamicObject tpv_purchasecode = (DynamicObject) this.getModel().getValue("tpv_purcontractno");
        //合同号为空返回
        if (tpv_purchasecode == null) {
            return;
        } else {
            DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle("tpv_dt_purchase", new QFilter[]{new QFilter("id", QCP.equals, tpv_purchasecode.getLong("id"))});
            DynamicObject tpv_supplier = dynamicObject.getDynamicObject("tpv_supplier"); //供应商

            if (tpv_supplier == null) {
                this.getView().showMessage("选择的合同号里没有填写供应商信息");
            } else {
                String tpv_suppliername = dynamicObject.getString("tpv_suppliername");  //供应商名称
                String tpv_suppliertin = dynamicObject.getString("tpv_suppliertin");  //纳税人识别号(供应商)
                String tpv_supplierbank = dynamicObject.getString("tpv_supplierbank");  //开户行(供应商)
                String tpv_supplieracct = dynamicObject.getString("tpv_supplieracct");  //账号(供应商)
                String tpv_supplierlinkman = dynamicObject.getString("tpv_supplierlinkman");  //联系人(供应商)
                String tpv_suppliertel = dynamicObject.getString("tpv_suppliertel");  //电话(供应商)
                String tpv_supplieraddr = dynamicObject.getString("tpv_supplieraddr");  //地址(供应商)
                String tpv_supplieremail = dynamicObject.getString("tpv_supplieremail"); //E-Mail(供应商)
                this.getModel().setValue("tpv_supplier", tpv_supplier);
                this.getModel().setValue("tpv_suppliername", tpv_suppliername);
                this.getModel().setValue("tpv_suppliertin", tpv_suppliertin);
                this.getModel().setValue("tpv_supplierbank", tpv_supplierbank);
                this.getModel().setValue("tpv_supplieracct", tpv_supplieracct);
                this.getModel().setValue("tpv_supplierlinkman", tpv_supplierlinkman);
                this.getModel().setValue("tpv_suppliertel", tpv_suppliertel);
                this.getModel().setValue("tpv_supplieraddr", tpv_supplieraddr);
                this.getModel().setValue("tpv_supplieremail", tpv_supplieremail);
            }
            BothCardHelper.setSupplierLabel(this);  //抬头赋值

        }
    }
}
