package com.itopview.wm.cm.formplugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.resource.ResManager;
import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.designer.botp.BillTreeBuildParameter;
import kd.bos.entity.EntityType;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.tree.TreeNode;
import kd.bos.exception.KDBizException;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.ShowType;
import kd.bos.form.control.Control;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.plugin.IFormPlugin;
import kd.bos.servicehelper.MetadataServiceHelper;
import kd.mpscmm.msplan.mrp.business.helper.BillFieldEntityHelper;
import kd.sdk.plugin.Plugin;

import java.util.*;

/**
 * 作价策略 表单插件
 */
public class PricingStrategyEdit extends AbstractBillPlugIn implements Plugin {

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
    }

    @Override
    public void afterDoOperation(AfterDoOperationEventArgs e) {
        super.afterDoOperation(e);
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners(new String[]{"tpv_srcfieldname"});
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String key = e.getProperty().getName();
        IDataModel model = this.getModel();
        int rowIndex = model.getEntryCurrentRowIndex("tpv_ps_entry");
        int rowCount = model.getEntryRowCount("tpv_ps_entry");
        switch (key) {
            case "tpv_bill":
                for (int i = 0; i < rowCount; i++) {
                    model.setValue("tpv_srcfieldname", null, i);
                    model.setValue("tpv_srcfield", null, i);
                }
                break;
            case "tpv_srcfieldname":
                String tpv_srcfieldname = null == model.getValue("tpv_srcfieldname", rowIndex) ? "" : model.getValue("tpv_srcfieldname", rowIndex).toString();
                if ("".equals(tpv_srcfieldname)) {
                    model.setValue("tpv_srcfield", null, rowIndex);
                }
                break;
            default:
                break;
        }
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        Control c = (Control) evt.getSource();
        String key = c.getKey().toLowerCase();
        int rowIndex = this.getModel().getEntryCurrentRowIndex("tpv_ps_entry");
        switch (key) {
            case "tpv_srcfieldname":
                this.openBillFieldList();
                break;
            default:
                break;
        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent e) {
        super.closedCallBack(e);
        String actionId = e.getActionId();
        Object returnData = e.getReturnData();
        int rowIndex = this.getModel().getEntryCurrentRowIndex("tpv_ps_entry");
        switch (actionId) {
            case "tpv_srcfield":
                if (returnData == null || StringUtils.isBlank((Object) returnData)) {
                    return;
                }
                String fieldId = (String) returnData;
                MainEntityType billMainType = this.getMainEntityType("tpv_bill", rowIndex);
                String fieldCaption = BillFieldEntityHelper.buildPropFullCaption((EntityType) billMainType, (String) fieldId);
                this.getModel().setValue("tpv_srcfieldname", (Object) fieldCaption, rowIndex);
                this.getModel().setValue("tpv_srcfield", (Object) fieldId, rowIndex);
                break;
            default:
                break;
        }
    }

    private void openBillFieldList() {
        int rowIndex = this.getModel().getEntryCurrentRowIndex("tpv_ps_entry");
        MainEntityType mainType = this.getMainEntityType("tpv_bill", rowIndex);
        BillTreeBuildParameter parameter = new BillTreeBuildParameter(mainType);
        parameter.setIncludePKField(true);
        parameter.setOnlyPhysicsField(false);
        TreeNode BillNode = this.buildBillTreeNodes(parameter, "tpv_bill");
        String nodesJson = SerializationUtils.toJsonString((Object) BillNode);
        this.showBillFieldForm(nodesJson, "tpv_srcfield");
    }

    private MainEntityType getMainEntityType(String entityname, int rowIndex) {
        DynamicObject accEntity = (DynamicObject) this.getModel().getValue(entityname, rowIndex);
        if (accEntity != null) {
            Object billentityPkValue = accEntity.getPkValue();
            MainEntityType dataEntityType = MetadataServiceHelper.getDataEntityType((String) billentityPkValue.toString());
            return dataEntityType;
        }
        throw new KDBizException(ResManager.loadKDString((String) "单据为空，请先给单据赋值。", (String) "BillFieldTransferEdit_25", (String) "mpscmm-msplan-formplugin", (Object[]) new Object[0]));
    }

    private void showBillFieldForm(String nodesJson, String custumparam) {
        FormShowParameter showParameter = new FormShowParameter();
        showParameter.setFormId("msplan_billfieldselect");
        showParameter.getCustomParams().put("treenodes", nodesJson);
        showParameter.setCloseCallBack(new CloseCallBack((IFormPlugin) this, custumparam));
        showParameter.getOpenStyle().setShowType(ShowType.Modal);
        this.getView().showForm(showParameter);
    }

    private TreeNode buildBillTreeNodes(BillTreeBuildParameter parameter, String field) {
        MainEntityType mainType = parameter.getMainType();
        TreeNode billNode = new TreeNode("", mainType.getName(), mainType.getDisplayName().toString());
        billNode.setIsOpened(true);
        List<TreeNode> fldNodes = BillFieldEntityHelper.buildFldTreeNodes((BillTreeBuildParameter) parameter);
        HashMap<String, TreeNode> nodeMap = new HashMap<String, TreeNode>(16);
        for (TreeNode node : fldNodes) {
            nodeMap.put(node.getId(), node);
        }
        for (TreeNode node : fldNodes) {
            TreeNode parentNode = (TreeNode) nodeMap.get(node.getParentid());
            if (parentNode == null) {
                node.setParentid(billNode.getId());
                billNode.addChild(node);
                continue;
            }
            parentNode.addChild(node);
        }
        return billNode;
    }

    private List<String> getExitField(String sign) {
        IDataModel model = this.getModel();
        DynamicObjectCollection rowData_s = model.getEntryEntity("tpv_ps_entry");
        ArrayList<String> list = new ArrayList<String>(8);
        for (DynamicObject rowData : rowData_s) {
            String message = rowData.getString(sign);
            list.add(message);
        }
        return list;
    }
}