package com.itopview.wm.cm.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.*;

/**
 * ClassName: AltimeterVerificationOp
 * Description: 高低计核销单反写金额到高低计单   高低计规划单反写金额到高低计单
 * date: 2023/11/15 17:15
 *
 * @author: Guo jiaChen
 */
public class AltimeterVerificationOp extends AbstractOperationServicePlugIn {

    private static final String algoKey = "AltimeterVerificationOp";
    private static final String VerificationEntryDetail = "tpv_verificatedetail";
    private static final String SettleEntryDetail = "tpv_settledetail";

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_settledetail");
        fieldKeys.add("tpv_srcbillno");
        fieldKeys.add("tpv_verificatedetail");
        fieldKeys.add("tpv_amountnotax1");
        fieldKeys.add("tpv_sourcebillid");
        fieldKeys.add("tpv_verificationtype");
        fieldKeys.add("tpv_srcbillno");
        fieldKeys.add("tpv_srcbillno1");
        fieldKeys.add("tpv_srcbillentryid");
        fieldKeys.add("tpv_srcbillentryid1");
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        String key = e.getOperationKey();
        DynamicObject[] entities = e.getDataEntities();
        for (int i = 0; i < entities.length; ++i) {
            changeData(entities[i], key);
        }
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] bills = this.getDataEntities();
                for (int i = 0; i < bills.length; ++i) {
                    ExtendedDataEntity bill = bills[i];
                    switch (this.getOperateKey()) {
                        case "save":
                        case "submit":
                            checkTaxamounts(bill);
                            break;
                    }
                }
            }

            public void checkTaxamounts(ExtendedDataEntity bill) {
                DynamicObject dataEntity = bill.getDataEntity();
                DynamicObjectCollection tpvSettledetails = dataEntity.getDynamicObjectCollection("tpv_settledetail");//结算明细
                for (int i = 0; i < tpvSettledetails.size(); i++) {
                    BigDecimal tpvAltveramt = tpvSettledetails.get(i).getBigDecimal("tpv_unm").setScale(2, BigDecimal.ROUND_HALF_UP);//核销金额（不含税） 结算明细
                    DynamicObjectCollection tpvVerificatedetails = tpvSettledetails.get(i).getDynamicObjectCollection("tpv_verificatedetail");//核销明细
                    BigDecimal decimal = BigDecimal.ZERO;
                    for (int j = 0; j < tpvVerificatedetails.size(); j++) {
                        BigDecimal tpvAmountnotax1 = tpvVerificatedetails.get(j).getBigDecimal("tpv_amountnotax1").setScale(2, BigDecimal.ROUND_HALF_UP);//核销金额（不含税）
                        BigDecimal tpvAltverunm = tpvVerificatedetails.get(j).getBigDecimal("tpv_altverunm").setScale(2, BigDecimal.ROUND_HALF_UP);//未核销金额（不含税）
                        if (tpvAmountnotax1.compareTo(tpvAltverunm) > 0) {
                            this.addErrorMessage(bill, "核销明细第" + (j + 1) + "行的核销金额（不含税）大于未核销金额（不含税）");
                        }
                        decimal = decimal.add(tpvAmountnotax1);
                    }
                    if (decimal.compareTo(tpvAltveramt) > 0) {
                        this.addErrorMessage(bill, "结算明细第" + (i + 1) + "行的结算明细核销金额（不含税）的汇总大于未核销金额（不含税）");
                    }
                }
            }
        });
    }

    public static String changeData(DynamicObject dataEntity, String key) {
        String verificationData = updateAltimeterVerificationData(dataEntity, key);
        return verificationData;
    }

    /**
     * @Title: updateAltimeterVerificationData
     * @Description: 反写高低计单核销金额
     */
    public static String updateAltimeterVerificationData(DynamicObject dataEntity, String operateKey) {

        DynamicObjectCollection tpvSettledetails = dataEntity.getDynamicObjectCollection("tpv_settledetail");

        HashMap<Long, BigDecimal> hashMapTax = new HashMap<>();//核销金额（含税）
        HashMap<Long, BigDecimal> hashMap = new HashMap<>();//核销金额（不含税）
        List<Long> idList = new ArrayList<>();
        List<Long> idListEnt = new ArrayList<>();
        List<DynamicObject> tpvCmAltverifications = new ArrayList<>();
        if (!"delete".equals(operateKey)) {
            tpvCmAltverifications.add(dataEntity);
        }
        for (DynamicObject tpvSettledetail : tpvSettledetails) {
            Long tpvSrcbillno = Long.valueOf(tpvSettledetail.getString("tpv_srcbillno"));//来源单据id
            Long tpvSrcbillentryid = Long.valueOf(tpvSettledetail.getString("tpv_srcbillentryid"));//来源单据分录id
            idList.add(tpvSrcbillno);
            idListEnt.add(tpvSrcbillentryid);
            DynamicObjectCollection tpvVerificatedetails = tpvSettledetail.getDynamicObjectCollection("tpv_verificatedetail");//核销明细
            for (DynamicObject tpvVerificatedetail : tpvVerificatedetails) {
                Long srcbillno = Long.valueOf(tpvVerificatedetail.getString("tpv_srcbillno1"));
                Long tpvSrcbillentryid1 = Long.valueOf(tpvVerificatedetail.getString("tpv_srcbillentryid1"));
                idList.add(srcbillno);
                idListEnt.add(tpvSrcbillentryid1);
            }
        }

        DynamicObject[] tpvCmAltve = BusinessDataServiceHelper.load("tpv_cm_altverification",
                "tpv_verificationtype,tpv_settledetail,tpv_srcbillentryid,tpv_amounttax,tpv_altveramt,tpv_verificatedetail,tpv_srcbillentryid1,tpv_amounttax1,tpv_amountnotax1",
                new QFilter[]{new QFilter("id", QCP.not_equals, dataEntity.getPkValue())});

        for (DynamicObject dynamicObject : tpvCmAltve) {
            tpvCmAltverifications.add(dynamicObject);
        }

        for (DynamicObject tpvCmAltverification : tpvCmAltverifications) {
            String tpvVerificationtype = tpvCmAltverification.getString("tpv_verificationtype");//核销类型
            DynamicObjectCollection settledetails = tpvCmAltverification.getDynamicObjectCollection("tpv_settledetail");
            for (DynamicObject settledetail : settledetails) {
                Long srcbillentryid = Long.valueOf(settledetail.getString("tpv_srcbillentryid"));//来源单据分录id
                BigDecimal tpvAmounttax = settledetail.getBigDecimal("tpv_amounttax");//核销金额（含税）
                BigDecimal tpvAltveramt = settledetail.getBigDecimal("tpv_altveramt");//核销金额（不含税）
                if (!hashMap.containsKey(srcbillentryid)) {//核销金额（不含税）
                    hashMap.put(srcbillentryid, tpvAltveramt);
                } else {
                    BigDecimal altveramt = hashMap.get(srcbillentryid);
                    hashMap.put(srcbillentryid, altveramt.add(tpvAltveramt));
                }
                if (!hashMapTax.containsKey(srcbillentryid)) {//核销金额（含税）
                    hashMapTax.put(srcbillentryid, tpvAmounttax);
                } else {
                    BigDecimal altveramt = hashMapTax.get(srcbillentryid);
                    hashMapTax.put(srcbillentryid, altveramt.add(tpvAmounttax));
                }
                if ("A".equals(tpvVerificationtype)) {
                    DynamicObjectCollection tpvVerificatedetails = settledetail.getDynamicObjectCollection("tpv_verificatedetail");//核销明细
                    for (DynamicObject tpvVerificatedetail : tpvVerificatedetails) {
                        Long srcbillentryid1 = Long.valueOf(tpvVerificatedetail.getString("tpv_srcbillentryid1"));//来源单据分录id
                        BigDecimal amounttax = tpvVerificatedetail.getBigDecimal("tpv_amounttax1");//核销金额（含税）
                        BigDecimal amountnotax1 = tpvVerificatedetail.getBigDecimal("tpv_amountnotax1");//核销金额（不含税）
                        if (!hashMap.containsKey(srcbillentryid1)) {//核销金额（不含税）
                            hashMap.put(srcbillentryid1, amountnotax1);
                        } else {
                            BigDecimal altveramt = hashMap.get(srcbillentryid);
                            hashMap.put(srcbillentryid1, altveramt.add(amountnotax1));
                        }
                        if (!hashMapTax.containsKey(srcbillentryid1)) {//核销金额（含税）
                            hashMapTax.put(srcbillentryid1, amounttax);
                        } else {
                            BigDecimal altveramt = hashMapTax.get(srcbillentryid);
                            hashMapTax.put(srcbillentryid1, altveramt.add(amounttax));
                        }
                    }
                }
            }
        }
        DynamicObject[] tpvCmUnevens = BusinessDataServiceHelper.load("tpv_cm_uneven",
                "tpv_advconap,tpv_taxamounts,tpv_notaxamounts,tpv_planqty,tpv_cancelqtys,tpv_unwrittenqtys,tpv_unwtax,tpv_unw,tpv_unplanqty", null);
        for (DynamicObject tpvCmUneven : tpvCmUnevens) {
            DynamicObjectCollection tpvAdvconaps = tpvCmUneven.getDynamicObjectCollection("tpv_advconap");//结算明细
            for (DynamicObject tpvAdvconap : tpvAdvconaps) {
                long id = tpvAdvconap.getLong("id");
                if (hashMapTax.containsKey(id)) {
                    BigDecimal tpvTaxamounts = tpvAdvconap.getBigDecimal("tpv_taxamounts");//含税金额
                    BigDecimal tpvNotaxamounts = tpvAdvconap.getBigDecimal("tpv_notaxamounts");//不含税金额
                    BigDecimal tpvPlanqty = tpvAdvconap.getBigDecimal("tpv_planqty");//规划金额（含税）

                    BigDecimal amounttax = hashMapTax.get(id);//核销金额（含税）
                    BigDecimal amountnotax1 = hashMap.get(id);//核销金额（不含税）

                    tpvAdvconap.set("tpv_cancelqtys", amounttax);//核销金额（含税）
                    tpvAdvconap.set("tpv_unwrittenqtys", amountnotax1);//核销金额（不含税）
                    tpvAdvconap.set("tpv_unwtax", tpvTaxamounts.subtract(amounttax));//未核销金额（含税）
                    tpvAdvconap.set("tpv_unw", tpvNotaxamounts.subtract(amountnotax1));//未核销金额（不含税）
                    tpvAdvconap.set("tpv_unplanqty", tpvTaxamounts.subtract(tpvPlanqty).subtract(amounttax));//未规划金额
                } else {
                    BigDecimal tpvTaxamounts = tpvAdvconap.getBigDecimal("tpv_taxamounts");//含税金额
                    BigDecimal tpvNotaxamounts = tpvAdvconap.getBigDecimal("tpv_notaxamounts");//不含税金额
                    BigDecimal tpvPlanqty = tpvAdvconap.getBigDecimal("tpv_planqty");//规划金额（含税）

                    tpvAdvconap.set("tpv_cancelqtys", BigDecimal.ZERO);//核销金额（含税）
                    tpvAdvconap.set("tpv_unwrittenqtys", BigDecimal.ZERO);//核销金额（不含税）
                    tpvAdvconap.set("tpv_unwtax", tpvTaxamounts);//未核销金额（含税）
                    tpvAdvconap.set("tpv_unw", tpvNotaxamounts);//未核销金额（不含税）
                    tpvAdvconap.set("tpv_unplanqty", tpvTaxamounts.subtract(tpvPlanqty));//未规划金额
                }
            }
        }
        SaveServiceHelper.save(tpvCmUnevens);
        return null;
    }

    /**
     * @Title: updatePlanBillStatus
     * @Description: 反写高低计单规划金额, 控制规划单是否禁止
     */
    public static String updatePlanBillStatus(DynamicObject dataEntity, String operateKey) {
        DynamicObjectCollection tpvSettledetails = dataEntity.getDynamicObjectCollection("tpv_settlement");
        for (DynamicObject tpvSettledetail : tpvSettledetails) {
            Long tpvSrcbillno = dataEntity.getLong("tpv_sourcebillid");//来源单据id
            DynamicObject cmUneven = BusinessDataServiceHelper.loadSingle("tpv_cm_uneven", new QFilter[]{new QFilter("id", "=", tpvSrcbillno)});
            Boolean isflag = true;
            if (cmUneven != null) {
                List<DynamicObject> arrayList = new ArrayList<>();
//                boolean exists = QueryServiceHelper.exists("tpv_cm_plannedbills", dataEntity.getPkValue());
//                if (!exists) {
//                    arrayList.add(dataEntity);
//                }
                QFilter qFilter = new QFilter("tpv_sourcebillid", "=", cmUneven.getPkValue());
                if ("delete".equals(operateKey) || "save".equals(operateKey)) {
                    qFilter.and("id", QCP.not_equals, dataEntity.getPkValue());
                    if (!arrayList.contains(dataEntity) && "save".equals(operateKey)) {
                        arrayList.add(dataEntity);
                    }
                }
                DynamicObject[] altverifiCation = BusinessDataServiceHelper.load("tpv_cm_plannedbills", "tpv_cancelqtys,tpv_sridid,tpv_amount,tpv_settlement,tpv_entryids", qFilter.toArray());

                for (DynamicObject altver : altverifiCation) {
                    arrayList.add(altver);
                }
                BigDecimal uncwn = BigDecimal.ZERO;
                BigDecimal uncwnRate = BigDecimal.ZERO;
                DynamicObjectCollection tpvAdvconaps = cmUneven.getDynamicObjectCollection("tpv_advconap");//高低计 分录
                for (DynamicObject tpvAdvconap : tpvAdvconaps) {//高低计 分录
                    BigDecimal tpvTaxamounts = tpvAdvconap.getBigDecimal("tpv_taxamounts");//（高低计分录）含税金额
                    BigDecimal tpvCancelqtys = tpvAdvconap.getBigDecimal("tpv_cancelqtys");//（高低计分录）核销金额
                    uncwn = uncwn.add(tpvCancelqtys);
                    for (DynamicObject altver : arrayList) {//高低计对应的所有高低规划销单
                        DynamicObjectCollection settledetails = altver.getDynamicObjectCollection("tpv_settlement");
                        for (DynamicObject settledetail : settledetails) {//高低规划销单 结算明细分录
                            Long srcbillentryid = settledetail.getLong("tpv_sridid");//来源单据分录id
                            Long advconapLong = tpvAdvconap.getLong("id");
                            if (advconapLong.equals(srcbillentryid)) {
                                DynamicObjectCollection tpvVerificatedetail = settledetail.getDynamicObjectCollection("tpv_programme");
                                for (DynamicObject object : tpvVerificatedetail) {
                                    uncwn = uncwn.add(object.getBigDecimal("tpv_amount"));
                                    uncwnRate = uncwnRate.add(object.getBigDecimal("tpv_amount"));
                                }
                            }
                        }
                    }
                    tpvAdvconap.set("tpv_planqty", uncwnRate);//高低计单 规划金额
                    tpvAdvconap.set("tpv_unplanqty", tpvTaxamounts.subtract(uncwn));//高低计单 未规划金额
                    if (tpvTaxamounts.subtract(uncwn).compareTo(BigDecimal.ZERO) < 0) {
                        isflag = false;
                    }
                }
            }
            if (isflag || "delete".equals(operateKey)) {
                return saveData(new DynamicObject[]{cmUneven}, "tpv_cm_uneven");
            } else {
                return "tpv_cm_uneven";
            }
        }
        return "";
    }

    /**
     * @Title: saveData
     * @Description: 保存数据
     */
    private static String saveData(DynamicObject[] plan, String entityID) {
        OperationResult saveOperate = OperationServiceHelper.executeOperate("save", entityID,
                plan, YEM.getOptionNoMsg());  //调用界面的保存按钮校验【回填单据的界面】
        if (!saveOperate.isSuccess()) {
            return "自动保存失败。" + YEM.getOpMsg(saveOperate);
        }
        return "";
    }
}
