package com.itopview.wm.cm.opplugin.op;


import com.itopview.wm.cm.utils.ClauseContractUtil;

import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.fi.cas.helper.OperateServiceHelper;

/**
 * 四大合同审核生成合同模板和合同条款库
 * @author zhangh
 *
 */
public class Contract2ClauseAuditOp extends AbstractOperationServicePlugIn {
	
	@Override
	public void afterExecuteOperationTransaction(AfterOperationArgs e) {
		switch (e.getOperationKey()) {
		case "audit":
			if (this.getOperationResult().isSuccess()) {
				DynamicObject[] dataentity = e.getDataEntities();
				for (DynamicObject data : dataentity) {
					data = BusinessDataServiceHelper.loadSingleFromCache(data.getPkValue(), data.getDataEntityType().getName());
					createContractClauseData(data);
					createContractTemplateData(data);
				}
			}
			break;

		default:
			break;
		}
	}

	/**
	 * 审核生成(修改合同模板)
	 * @param data
	 */
	private void createContractTemplateData(DynamicObject data) {
		String tpv_revise = data.getString("tpv_revise");
		if (StringUtils.isNotEmpty(tpv_revise)) {
			if (tpv_revise.equals("A")) {
				this.createTemplateData(data);
			}else if (tpv_revise.equals("B")) {
				this.updateTemplateData(data);
			}
		}
	}

	/**
	 * 修改合同模板
	 * @param data
	 */
	private void updateTemplateData(DynamicObject data) {
		DynamicObject tpv_template = data.getDynamicObject("tpv_template");
		if (tpv_template != null) {
			tpv_template = BusinessDataServiceHelper.loadSingleFromCache(tpv_template.getPkValue(), "tpv_cm_contracttemplate");
			produceTemplateData(data, tpv_template, false);
		}		
	}
	
	/**
	 *  新增合同模板
	 * @param data
	 */
	private void createTemplateData(DynamicObject data) {
		String tpv_templatename = data.getString("tpv_templatename");
		DynamicObject contracttemplate = BusinessDataServiceHelper.loadSingle("tpv_cm_contracttemplate", "id", new QFilter[] {new QFilter("tpv_templatename", QCP.equals, tpv_templatename)});
		if (contracttemplate == null) {
			DynamicObject tpv_template = BusinessDataServiceHelper.newDynamicObject("tpv_cm_contracttemplate");
			this.produceTemplateData(data, tpv_template, true);
		}
	}

	/**
	 * 创建合同模板数据
	 * @param data  合同实体
	 * @param tpv_template 合同模板实体
	 * @param result 新增还是修改
	 */
	private void produceTemplateData(DynamicObject data, DynamicObject tpv_template, boolean result) {
		tpv_template.set("billstatus", "C");
		tpv_template.set("org", data.getDynamicObject("org"));
		tpv_template.set("tpv_org", data.getDynamicObject("tpv_department"));  //需要修改
		tpv_template.set("tpv_dept", data.getDynamicObject("tpv_department"));
		tpv_template.set("tpv_templatetype", data.getString("tpv_templatetype"));
		tpv_template.set("tpv_templatename", data.getString("tpv_templatename"));
		tpv_template.set("tpv_contracttype", data.getString("tpv_contracttype"));
		tpv_template.set("tpv_status", "A");
		String name = data.getDataEntityType().getName();
		String entryKey = "tpv_materialentry"; 
		if (name.equals("pm_purorderbill") || name.equals("sm_salorder") || name.equals("tpv_pm_purorderbill_inh") || name.equals("tpv_sm_salorder_inh")) {
			entryKey = "billentry";
		}
		DynamicObjectCollection materialList = ClauseContractUtil.getMaterialList(entryKey, tpv_template, data, name);
		tpv_template.set("tpv_material", materialList);
		
		tpv_template.getDynamicObjectCollection("tpv_contractterms").clear();//清除原分录
		this.templateAdjustmentEntry(data, tpv_template, result);
		
		OperateServiceHelper.execOperate("save", "tpv_cm_contracttemplate",
				new DynamicObject[] { tpv_template }, OperateOption.create());
		
		data.set("tpv_templateid", tpv_template.getPkValue());
		SaveServiceHelper.save(new DynamicObject[] {data});		
	}

	/**
	 * 合同模板分录修改和生成
	 * @param data
	 * @param tpv_template
	 * @param result
	 */
	private void templateAdjustmentEntry(DynamicObject data, DynamicObject tpv_template, boolean result) {
		DynamicObjectCollection tpv_clauseimport = data.getDynamicObjectCollection("tpv_clauseimport");
		DynamicObjectCollection tpv_contractterms = tpv_template.getDynamicObjectCollection("tpv_contractterms");
		for (DynamicObject clauseimport : tpv_clauseimport) {
			DynamicObject tpv_clausetype = clauseimport.getDynamicObject("tpv_clausetype");
			tpv_clausetype = BusinessDataServiceHelper.loadSingleFromCache(tpv_clausetype.getPkValue(), "tpv_cm_contractterms");
			DynamicObject contractterms = new DynamicObject(tpv_contractterms.getDynamicObjectType());//获取单据体数据类型
			contractterms.set("tpv_termname", tpv_clausetype);
			DynamicObject clausetype = tpv_clausetype.getDynamicObject("tpv_clausetype");
			contractterms.set("tpv_clausetype", clausetype);
			contractterms.set("tpv_termbillno", clauseimport.getString("tpv_termtype"));
			contractterms.set("tpv_clausescontent", clauseimport.getString("tpv_updatebefore_tag"));
			contractterms.set("tpv_clausescontent_tag", clauseimport.getString("tpv_updatebefore_tag"));
			tpv_contractterms.add(contractterms);
		}
	}

	/**
	 * 审核生成(修改)合同条款库
	 * @param data
	 */
	private void createContractClauseData(DynamicObject data) {
		String tpv_updateterm = data.getString("tpv_updateterm");
		if (StringUtils.isNotEmpty(tpv_updateterm)) {
			if (tpv_updateterm.equals("A")) {
				this.createClausedata(data);
			}else if (tpv_updateterm.equals("B")) {
				this.updateClauseData(data);
			}
		}
	}

	/**
	 * 修改合同条款库
	 * @param data
	 */
	private void updateClauseData(DynamicObject data) {
		DynamicObjectCollection tpv_clauseimport = data.getDynamicObjectCollection("tpv_clauseimport");
		for (DynamicObject clauseimport : tpv_clauseimport) {
			if (clauseimport.getBoolean("tpv_change")) {
				DynamicObject tpv_clausetype = clauseimport.getDynamicObject("tpv_clausetype");
				if (tpv_clausetype == null) {
					DynamicObject newDynamicObject = BusinessDataServiceHelper.newDynamicObject("tpv_cm_contractterms");
					produceClauseData(data,clauseimport, newDynamicObject, true);
				}else {
					tpv_clausetype = BusinessDataServiceHelper.loadSingleFromCache(tpv_clausetype.getPkValue(), "tpv_cm_contractterms");
					produceClauseData(data,clauseimport, tpv_clausetype, false);
				}
			}
		}
	}

	/**
	 * 生成合同条款库
	 * @param data
	 */
	private void createClausedata(DynamicObject data) {
		DynamicObjectCollection tpv_clauseimport = data.getDynamicObjectCollection("tpv_clauseimport");
		for (DynamicObject clauseimport : tpv_clauseimport) {
			if (clauseimport.getBoolean("tpv_change")) {
				DynamicObject tpv_cm_contractterms = BusinessDataServiceHelper.newDynamicObject("tpv_cm_contractterms");
				produceClauseData(data, clauseimport, tpv_cm_contractterms, true);
			}
		}		
	}

	/**
	 * 创建合同条款库数据
	 * @param data  合同实体
	 * @param clauseimport  合同合同条款库分录实体
	 * @param tpv_clausetype  合同条款库实体 
	 * @param result 新增还是修改
	 */
	private void produceClauseData(DynamicObject data, DynamicObject clauseimport, DynamicObject tpv_clausetype,
			boolean result) {
		tpv_clausetype.set("billstatus", "C");
		tpv_clausetype.set("org", data.getDynamicObject("org"));
		tpv_clausetype.set("tpv_org", data.getDynamicObject("tpv_department"));
		tpv_clausetype.set("tpv_dept", data.getDynamicObject("tpv_department"));
		
		DynamicObject oldclausetype = clauseimport.getDynamicObject("tpv_clausetype");
		if (oldclausetype == null) {
			String tpv_termtype = clauseimport.getString("tpv_termtype");
			DynamicObject bd_typeclause = BusinessDataServiceHelper.loadSingle("tpv_bd_typeclause", "id", new QFilter[] {new QFilter("name", QCP.equals, tpv_termtype)});
			if (bd_typeclause == null) {
				//如果条款类型为空，先生成条款类型
				bd_typeclause = BusinessDataServiceHelper.newDynamicObject("tpv_bd_typeclause");//合同条款库为空，说名点击新增行生成
				if (StringUtils.isNotEmpty(tpv_termtype) && tpv_termtype.contains("、")) {
					bd_typeclause.set("tpv_chinanum", tpv_termtype.split("、")[0]);
					bd_typeclause.set("tpv_clausename", tpv_termtype.split("、")[1]);
					bd_typeclause.set("name", tpv_termtype);
					bd_typeclause.set("enable", "1");
					bd_typeclause.set("status", "C");
					OperateServiceHelper.execOperate("save", "tpv_bd_typeclause", new DynamicObject[] {bd_typeclause}, OperateOption.create());
				}
			}
			tpv_clausetype.set("tpv_clausetype", bd_typeclause);
			
		}else {
			oldclausetype = BusinessDataServiceHelper.loadSingleFromCache(oldclausetype.getPkValue(), "tpv_cm_contractterms");
			tpv_clausetype.set("tpv_clausetype", oldclausetype.getDynamicObject("tpv_clausetype"));
		}
		
		tpv_clausetype.set("tpv_clausecontentchina", clauseimport.getString("tpv_updatebefore"));
		tpv_clausetype.set("tpv_clausecontentchina_tag", clauseimport.getString("tpv_updatebefore"));
		tpv_clausetype.set("tpv_clausechina", clauseimport.getLocaleString("tpv_clausename").getLocaleValue_zh_CN());
		
		OperateServiceHelper.execOperate("save", "tpv_cm_contractterms",
				new DynamicObject[] { tpv_clausetype }, OperateOption.create());

		clauseimport.set("tpv_clausetype", tpv_clausetype);
		SaveServiceHelper.save(new DynamicObject[] { data });
	}
}
