package com.itopview.wm.cm.opplugin.op;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.itopview.wm.sys.utils.NewVersionHelper;
import com.itopview.wm.utils.DynamicObjectUtil;

import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.exception.KDBizException;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.coderule.CodeRuleServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;
import kd.fi.cas.helper.OperateServiceHelper;
/**
 * 合同模板反写合同条款库
 * @author zhangh
 *
 */
public class ContractTemplateAuditOp extends AbstractOperationServicePlugIn {
	
	@Override
	public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
		switch (e.getOperationKey()) {
		case "audit":
			for (DynamicObject data : e.getDataEntities()) {
				String name = data.getDataEntityType().getName();
				data = BusinessDataServiceHelper.loadSingleFromCache(data.getPkValue(), name);
				this.updateContractTermData(data);//合同条款库
				NewVersionHelper newVersionHelper = new NewVersionHelper(data.getDataEntityType().getName());
				boolean tpv_newtemplate = data.getBoolean("tpv_newtemplate");
				if (newVersionHelper.isChanged() && tpv_newtemplate) {//合同模板为变更单据并且合同模板新增打钩
					createTemplate(data);
				}else if (newVersionHelper.isChanged() && !tpv_newtemplate) {
					//合同模板为变更并且不需要生成新的合同模板，执行合同模板变更生效操作，改变源单的单据状态
					OperateServiceHelper.execOperate("effectchange", name, new DynamicObject[] { data }, OperateOption.create());
					DynamicObject bill = BusinessDataServiceHelper.loadSingleFromCache(data.getLong("tpv_masterid"), newVersionHelper.getRequireKey());
					bill.set("billstatus", "C");
					SaveServiceHelper.save(new DynamicObject[] { bill });
				}
			}
			break;

		default:
			break;
		}
	}

	/**
	 * 审核新增合同模板
	 * 
	 * @param data
	 */
	private void createTemplate(DynamicObject data) {
		List<QFilter> qFilters = new ArrayList<>();
		qFilters.add(new QFilter("tpv_templatename", QCP.equals, data.getString("tpv_templatename")));
		NewVersionHelper newVersionHelper = new NewVersionHelper(data.getDataEntityType().getName());
		if (newVersionHelper.isChanged()) {
			qFilters.add(new QFilter("id", QCP.not_equals, data.getLong("tpv_masterid")));
		} else {
			qFilters.add(new QFilter("id", QCP.not_equals, data.getPkValue()));
		}	
		
		DynamicObject contracttemplate = BusinessDataServiceHelper.loadSingle("tpv_cm_contracttemplate",
				"id,billno", qFilters.toArray(new QFilter[] {}));
		if (contracttemplate == null) {
			DynamicObject newDynamicObject = BusinessDataServiceHelper.newDynamicObject("tpv_cm_contracttemplate");
			DynamicObjectUtil.copyDynamicObject(data, newDynamicObject);
			newDynamicObject.set("billstatus", "C");
			String number = CodeRuleServiceHelper.getNumber("tpv_cm_contracttemplate", newDynamicObject, null);
			newDynamicObject.set("billno", number);
			newDynamicObject.set("tpv_version", 1);
			newDynamicObject.set("tpv_masterid", null);
			newDynamicObject.set("tpv_newtemplate", false);
			DynamicObjectCollection tpv_contractterms = newDynamicObject.getDynamicObjectCollection("tpv_contractterms");
			for (DynamicObject contractterms : tpv_contractterms) {
				contractterms.set("tpv_adjust", false);
			}
			
			OperateServiceHelper.execOperate("save", "tpv_cm_contracttemplate",
					new DynamicObject[] { newDynamicObject }, OperateOption.create());

			data.set("tpv_effective", true);
			
			data.set("tpv_changedate", new Date());
			data.set("tpv_effectdate", new Date());
			data.set("tpv_changeman", UserServiceHelper.getCurrentUserId());
			data.set("tpv_effector", UserServiceHelper.getCurrentUserId());
			SaveServiceHelper.save(new DynamicObject[] { data });
			
			DynamicObject bill = BusinessDataServiceHelper.loadSingleFromCache(data.getLong("tpv_masterid"), newVersionHelper.getRequireKey());
			bill.set("tpv_version", data.getBigDecimal("tpv_version"));
			SaveServiceHelper.save(new DynamicObject[] { bill });
		} else {
			throw new KDBizException(String.format("合同模板审核生成新的合同模板时，模板名称不允许重复，请变更模板名称。"));
		}
	}

	/**
	 * 合同条款库新增或者覆盖逻辑
	 * @param data
	 */
	private void updateContractTermData(DynamicObject data) {
		String tpv_amend = data.getString("tpv_amend");//条款修改控制
		if (StringUtils.isNotEmpty(tpv_amend)) {
			DynamicObjectCollection tpv_contractterms = data.getDynamicObjectCollection("tpv_contractterms");
			for (DynamicObject contractterms : tpv_contractterms) {
				boolean tpv_adjust = contractterms.getBoolean("tpv_adjust");//是否调整
				if (tpv_amend.equals("A") && tpv_adjust) {//条款修改控制为新增并且是否调整打钩
					this.createTermData(data, contractterms);
				}else if (tpv_amend.equals("B") && tpv_adjust) {//条款修改控制为覆盖并且是否调整打钩
					updateTermData(data, contractterms);
					updateHistoryTemplate(data, contractterms);
				}
			}
		}
	}

	/**
	 * 合同条款覆盖之后，修复历史的合同模板
	 * @param data
	 * @param contractterms 
	 */
	private void updateHistoryTemplate(DynamicObject data, DynamicObject contractterms) {
		DynamicObject tpv_termname = contractterms.getDynamicObject("tpv_termname");
		if (tpv_termname != null) {
			Long termname = tpv_termname.getLong("id");
			tpv_termname = BusinessDataServiceHelper.loadSingleFromCache(termname, tpv_termname.getDynamicObjectType().getName());
			List<QFilter> qFilters = new ArrayList<>();
			qFilters.add(new QFilter("tpv_contractterms.tpv_termname", QCP.equals, termname));
			qFilters.add(new QFilter("billstatus", QCP.equals, "C"));
			qFilters.add(new QFilter("id", QCP.not_equals, data.getLong("id")));
			DynamicObject[] cm_contracttemplate = BusinessDataServiceHelper.load("tpv_cm_contracttemplate", "id", qFilters.toArray(new QFilter[] {}));
			for (DynamicObject contracttemplate : cm_contracttemplate) {
				contracttemplate = BusinessDataServiceHelper.loadSingleFromCache(contracttemplate.getPkValue(),contracttemplate.getDynamicObjectType().getName());
				DynamicObjectCollection tpv_contractterms = contracttemplate.getDynamicObjectCollection("tpv_contractterms");
				for (DynamicObject oldcontractterms : tpv_contractterms) {
					DynamicObject tpv_oldtermname = oldcontractterms.getDynamicObject("tpv_termname");
					if (tpv_oldtermname != null && tpv_oldtermname.getLong("id") == termname) {
						oldcontractterms.set("tpv_clausetype", tpv_termname.getDynamicObject("tpv_clausetype"));
						LocaleString locStr = new LocaleString();
						locStr.setLocaleValue_zh_CN(tpv_termname.getString("tpv_clausechina"));
						locStr.setLocaleValue_en(tpv_termname.getString("tpv_clauseenglish"));
						oldcontractterms.set("tpv_clausename", locStr);
						DynamicObject tpv_clausetype = tpv_termname.getDynamicObject("tpv_clausetype");
						oldcontractterms.set("tpv_termbillno", tpv_clausetype.getString("tpv_clausename"));
						oldcontractterms.set("tpv_clausescontent", tpv_termname.getString("tpv_clausecontentchina"));
						oldcontractterms.set("tpv_clausescontent_tag", tpv_termname.getString("tpv_clausecontentchina"));
						oldcontractterms.set("tpv_contentbefore", tpv_termname.getString("tpv_clausecontentchina"));
						oldcontractterms.set("tpv_contentbefore_tag", tpv_termname.getString("tpv_clausecontentchina"));
					}
				}
			}
			SaveServiceHelper.save(cm_contracttemplate);
		}
	}

	/**
	 * 修改合同条款库
	 * 
	 * @param data
	 * @param contractterms
	 */
	private void updateTermData(DynamicObject data, DynamicObject contractterms) {
		DynamicObject tpv_termname = contractterms.getDynamicObject("tpv_termname");// 获取合同条款库
		if (tpv_termname != null) {
			tpv_termname = BusinessDataServiceHelper.loadSingleFromCache(tpv_termname.getPkValue(), "tpv_cm_contractterms");
			setData(data, contractterms, tpv_termname);
		}
	}

	/**
	 * 创建新的合同条款库数据
	 * @param data
	 * @param contractterms
	 */
	private void createTermData(DynamicObject data, DynamicObject contractterms) {
		DynamicObject cm_contractterms = BusinessDataServiceHelper.newDynamicObject("tpv_cm_contractterms");
		setData(data, contractterms, cm_contractterms);
	}

	/**
	 * 给新增/修改的合同条款库数据赋值
	 * @param data
	 *            合同模板主实体
	 * @param contractterms
	 *            合同模板分录实体
	 * @param tpv_termname
	 *            合同条款库实体
	 */
	private void setData(DynamicObject data, DynamicObject contractterms, DynamicObject tpv_termname) {
		tpv_termname.set("org", data.getDynamicObject("org"));
		tpv_termname.set("tpv_org", data.getDynamicObject("tpv_org"));
		tpv_termname.set("tpv_dept", data.getDynamicObject("tpv_dept"));
		String tpv_tradetype = data.getString("tpv_tradetype");
		tpv_termname.set("tpv_foreigntrade", tpv_tradetype);
		tpv_termname.set("tpv_clausetype", contractterms.getDynamicObject("tpv_clausetype"));
		String tpv_clauseschina = contractterms.getString("tpv_clausescontent");
		tpv_termname.set("tpv_clausecontentchina", tpv_clauseschina);
		tpv_termname.set("tpv_clausecontentchina_tag", tpv_clauseschina);
		tpv_termname.set("tpv_clausechina",
				contractterms.getLocaleString("tpv_clausename").getLocaleValue_zh_CN());
		tpv_termname.set("billstatus", "C");
		OperateServiceHelper.execOperate("save", "tpv_cm_contractterms", new DynamicObject[] { tpv_termname },
				OperateOption.create());

		contractterms.set("tpv_termname", tpv_termname);//将新生成的合同条款库赋值给合同模板
		SaveServiceHelper.save(new DynamicObject[] { data });

	}
}
