package com.itopview.wm.cm.opplugin.op;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.DeleteServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * 采购价格测算 保存操作插件
 */
public class SalePriceCalculationSaveOp extends AbstractOperationServicePlugIn implements Plugin {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("org");
        keys.add("tpv_bizdate");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
    }

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
        if ("save".equals(e.getOperationKey()) && null != e.getDataEntities()) {
            DynamicObject[] bills = e.getDataEntities();
            for (int i = 0; i < bills.length; ++i) {
                DynamicObject bill = bills[i];
                delete(bill);
                delete_old_data(bill);
            }
        }
    }

    /**
     * 删除组织、业务日期重复数据
     * @param bill
     */
    public void delete(DynamicObject bill){
        DynamicObject org = bill.getDynamicObject("org");
        Date tpv_bizdate = bill.getDate("tpv_bizdate");
        QFilter qFilter = new QFilter("org", QCP.equals, org.getPkValue());
        qFilter.and(new QFilter("tpv_bizdate", QCP.equals, tpv_bizdate));
        long id = bill.getLong("id");
        if(0L != id){
            qFilter.and(new QFilter("id", QCP.not_equals, id));
        }
        DynamicObject [] tpv_salepricecalculat_s = BusinessDataServiceHelper.load("tpv_salepricecalculat",
                "id,billno,billstatus", new QFilter[] { qFilter });
        if(null != tpv_salepricecalculat_s && tpv_salepricecalculat_s.length>0){
            List del_id_s = new ArrayList();
            for(int i=0;i<tpv_salepricecalculat_s.length;i++){
                DynamicObject tpv_salepricecalculat = tpv_salepricecalculat_s[i];
                del_id_s.add(tpv_salepricecalculat.getPkValue());
            }
            qFilter = new QFilter("id", QCP.in, del_id_s);
            DeleteServiceHelper.delete("tpv_salepricecalculat",new QFilter[] { qFilter });
        }
    }

    /**
     * 删除历史数据中合同已完结记录
     * @param bill
     */
    public void delete_old_data(DynamicObject bill){
        DynamicObject org = bill.getDynamicObject("org");
        Date tpv_bizdate = bill.getDate("tpv_bizdate");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(tpv_bizdate);
        calendar.add(Calendar.DAY_OF_MONTH,-1);

        QFilter qFilter = new QFilter("org", QCP.equals, org.getPkValue());
        qFilter.and(new QFilter("tpv_bizdate", QCP.equals, calendar.getTime()));
        DynamicObject [] tpv_purpricecalculation_s = BusinessDataServiceHelper.load("tpv_purpricecalculation",
                "id,billno,billstatus", new QFilter[] { qFilter });
        if(null != tpv_purpricecalculation_s && tpv_purpricecalculation_s.length>0){
            DynamicObject tpv_purpricecalculation = tpv_purpricecalculation_s[0];
            tpv_purpricecalculation = BusinessDataServiceHelper.loadSingle(tpv_purpricecalculation.getPkValue(),tpv_purpricecalculation.getDynamicObjectType().getName());
            Date tpv_bizdate_old = tpv_purpricecalculation.getDate("tpv_bizdate");
            List index_s = new ArrayList();
            DynamicObjectCollection tpv_spc_entry_s = tpv_purpricecalculation.getDynamicObjectCollection("tpv_spc_entry");
            for (int i=0;i<tpv_spc_entry_s.size();i++) {
                DynamicObject tpv_spc_entry = tpv_spc_entry_s.get(i);
                DynamicObject tpv_dt_purchase = tpv_spc_entry.getDynamicObject("tpv_dt_purchase");
                tpv_dt_purchase  = BusinessDataServiceHelper.loadSingle(tpv_dt_purchase.getPkValue(),tpv_dt_purchase.getDynamicObjectType().getName());
                if(false){
                    index_s.add(i);
                }
            }
            if(index_s.size()>0){
                for(int i=index_s.size()-1;i>=0;i--){
                    int index = new Integer(index_s.get(i).toString());
                    tpv_spc_entry_s.remove(index);
                }
                SaveServiceHelper.save(new DynamicObject[]{tpv_purpricecalculation});
            }
        }
    }
}