package com.itopview.wm.cm.opplugin.validator;

import java.util.ArrayList;
import java.util.List;

import com.itopview.wm.utils.YEM;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

public class ContractTermsSubmitValidator extends AbstractValidator {

	@Override
	public void validate() {
		ExtendedDataEntity[] dataEntity = this.getDataEntities();
		for (ExtendedDataEntity extendedDataEntity : dataEntity) {
			this.checkHollowing(extendedDataEntity);
		}
	}

	/**
	 * 校验挖空字段
	 * @param extendedDataEntity
	 */
	private void checkHollowing(ExtendedDataEntity extendedDataEntity) {
		DynamicObject dataEntity = extendedDataEntity.getDataEntity();
		String tpv_foreigntrade = dataEntity.getString("tpv_foreigntrade");
		if (StringUtils.isNotEmpty(tpv_foreigntrade)) {
			if (tpv_foreigntrade.equals("A")) {
//				this.checkChineseAndEnlishSame(dataEntity, extendedDataEntity);
			}
		}
		
		this.queryFieldControl(extendedDataEntity, dataEntity);
	}

	/**
	 * 根据条款内容挖空字段判断是否在【字段对照】表中存在
	 * @param extendedDataEntity
	 * @param dataEntity 
	 */
	private void queryFieldControl(ExtendedDataEntity extendedDataEntity, DynamicObject dataEntity) {
		String tpv_clauseschina = dataEntity.getString("tpv_clausecontentchina");
		List<String> clauseschina = new ArrayList<>();
		if (StringUtils.isNotEmpty(tpv_clauseschina)) {
			for (int i = 0; i < tpv_clauseschina.length(); i++) {
				//根据条款内容字符串的字符长度，进行循环截取挖空字段
				if (tpv_clauseschina.contains("【")) {
					tpv_clauseschina = this.cutOutString(tpv_clauseschina, clauseschina);
				}
			}
		}
		if (clauseschina.size() > 0) {
			for (String str : clauseschina) {
				DynamicObject clausecontrast = BusinessDataServiceHelper.loadSingle("tpv_bd_clausecontrast", "id", new QFilter[] {new QFilter("tpv_clausekey", QCP.equals, str)});
				if (clausecontrast == null) {
					this.addErrorMessage(extendedDataEntity, String.format("挖空字段: %s在字段对照中不存在!!!", str));
				}
			}
		}
	}

	/**
	 * 贸易类型为外贸，判断中文挖空字段和英文挖空字段一致(长度，内容一致)【作废】
	 * @param dataEntity
	 * @param extendedDataEntity 
	 */
	public void checkChineseAndEnlishSame(DynamicObject dataEntity, ExtendedDataEntity extendedDataEntity) {
		String tpv_clauseschina = dataEntity.getString("tpv_clausecontentchina");
		String tpv_clausesenglish = dataEntity.getString("tpv_clausecontentenglish");
		List<String> clauseschina = new ArrayList<>();
		List<String> clausesenglish = new ArrayList<>();
		if (YEM.isNotEmpty(tpv_clauseschina) && YEM.isNotEmpty(tpv_clausesenglish)) {
			if (StringUtils.isNotEmpty(tpv_clauseschina)) {
				for (int i = 0; i < tpv_clauseschina.length(); i++) {
					if (tpv_clauseschina.contains("【")) {
						tpv_clauseschina = this.cutOutString(tpv_clauseschina, clauseschina);
					}
				}
			}

			if (StringUtils.isNotEmpty(tpv_clausesenglish)) {
				for (int i = 0; i < tpv_clausesenglish.length(); i++) {
					if (tpv_clausesenglish.contains("【")) {
						tpv_clausesenglish = this.cutOutString(tpv_clausesenglish, clausesenglish);
					}
				}
			}
			
			if (clauseschina.size() == clausesenglish.size()) {
				for (String str : clauseschina) {
					if (!clausesenglish.contains(str)) {
						this.addErrorMessage(extendedDataEntity, String.format("中文挖空字段: %s在英文挖空字段中不存在!!!", str));
						return;
					}
				}
				
				for (String str : clausesenglish) {
					if (!clauseschina.contains(str)) {
						this.addErrorMessage(extendedDataEntity, String.format("英文挖空字段: %s在中文挖空字段中不存在!!!", str));
						return;
					}
				}
			}else {
				this.addErrorMessage(extendedDataEntity, "中文内容挖空字段和英文挖空字段长度不一致!!!");
				return;
			}
		}
	}

	/**
	 * 根据挖空字段截取字符串
	 * @param tpv_clauseschina     字符串截取之后的内容
	 * @param clauseschina         挖空字段的集合
	 */
	private String cutOutString(String tpv_clauseschina, List<String> clauseschina) {
		String substring = tpv_clauseschina.substring(tpv_clauseschina.indexOf("【") + 1, tpv_clauseschina.indexOf("】"));
		String str = tpv_clauseschina.substring(0, tpv_clauseschina.indexOf("【")) + 
				tpv_clauseschina.substring(tpv_clauseschina.indexOf("】") + 1);
		clauseschina.add(substring);
		return str;
	}

}
