package com.itopview.wm.cm.opplugin.validator;

import com.itopview.wm.cm.opplugin.op.AltimeterVerificationOp;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.entity.validate.AbstractValidator;

import java.math.BigDecimal;
import java.util.List;

/**
 * 提交校验规划明细分录
 *
 * @author LiuHB
 * @version 1.0
 */
public class PlannedValidator extends AbstractOperationServicePlugIn {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_entryids");
        fieldKeys.add("tpv_settlement");
        fieldKeys.add("tpv_amount");
        fieldKeys.add("tpv_programme");
        fieldKeys.add("tpv_sourcebillid");
        fieldKeys.add("tpv_sridid");
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        String key = e.getOperationKey();
        DynamicObject[] entities = e.getDataEntities();
        if ("submit".equals(key) || "delete".equals(key) || "save".equals(key)) {
            for (DynamicObject entity : entities) {
                AltimeterVerificationOp.updatePlanBillStatus(entity, key);
            }
        }

    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        DynamicObject[] entities = e.getDataEntities();
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] bills = this.getDataEntities();
                String key = this.getOperateKey();
                for (int i = 0; i < bills.length; i++) {
                    ExtendedDataEntity bill = bills[i];
                    switch (key) {
                        case "submit":
                            DynamicObject[] entities = e.getDataEntities();
                            for (DynamicObject entry : entities) {
                                // 获取单据体
                                DynamicObjectCollection entryEntity = entry.getDynamicObjectCollection("tpv_settlement");
                                for (DynamicObject settlement : entryEntity) {
                                    DynamicObjectCollection programme = settlement.getDynamicObjectCollection("tpv_programme");
                                    for (DynamicObject tpvProgramme : programme) {
                                        int indexOf = programme.indexOf(tpvProgramme);
                                        indexOf = indexOf + 1;
                                        String documenttype = tpvProgramme.getString("tpv_documenttype");
                                        String tpvPurchasecontr = tpvProgramme.getString("tpv_purchasecontr");
                                        String tpvContractnu = tpvProgramme.getString("tpv_contractnu");
                                        if (documenttype == null) {
                                            break;
                                        } else if (documenttype.equals("C")) {
                                            if (tpvPurchasecontr == null) {
                                                this.addMessage(bill, "第" + indexOf + "行的规划采购合同号为空");
                                            }
                                        } else if (documenttype.equals("A") || documenttype.equals("B")) {
                                            if (tpvContractnu == null) {
                                                this.addMessage(bill, "第" + indexOf + "行的规划销售合同号为空");
                                            }
                                        }
                                    }
                                }
                            }
                            updatePlanBillStatus(bill, key);
                            break;
                        case "save":
                            DynamicObject[] entitie = e.getDataEntities();
                            BigDecimal tpvAmount = BigDecimal.ZERO;
                            BigDecimal tpvPlannedamt = BigDecimal.ZERO;
                            for (DynamicObject entry : entitie) {

                                // 获取单据体
                                DynamicObjectCollection entryEntitys = entry.getDynamicObjectCollection("tpv_settlement");
                                for (DynamicObject dynamicObject : entryEntitys) {
                                    tpvAmount = BigDecimal.ZERO;
                                    int indexOf = entryEntitys.indexOf(dynamicObject);
                                    indexOf = ++indexOf;
                                    DynamicObjectCollection entryEntity = dynamicObject.getDynamicObjectCollection("tpv_programme");
                                    for (DynamicObject entryEntityd : entryEntity) {

                                        tpvAmount = tpvAmount.add(entryEntityd.getBigDecimal("tpv_amount"));
                                    }
                                    tpvPlannedamt = dynamicObject.getBigDecimal("tpv_plannedamt");
                                    if (tpvAmount.compareTo(tpvPlannedamt) > -1) {
                                        this.addMessage(bill, "结算明细第" + indexOf + "行的规划金额大于待规划金额");
                                    }

                                }
                            }
                            updatePlanBillStatus(bill, key);
                            break;
                    }
                }
            }

            public void updatePlanBillStatus(ExtendedDataEntity bill, String key) {
                DynamicObject[] entitied = e.getDataEntities();
                for (DynamicObject entitiedynamic : entitied) {
                    String updatePlanBillStatus = AltimeterVerificationOp.updatePlanBillStatus(entitiedynamic, key);
                    if (updatePlanBillStatus.contains("tpv_cm_uneven") && !"delete".equals(this.getOperateKey())) {
                        this.addMessage(bill, "反写规划金额到高低计单失败，请检查数据");
                    }
                }
            }
        });
    }
}




