package com.itopview.wm.cm.report;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QFilter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 串拣配成本置换工作台-查询插件
 *
 * @author WangShuGuang
 * @description
 * @date 2023年11月17日
 */
public class PlacemenRptQuery extends AbstractReportListDataPlugin {

    private static final String algoKey = PlacemenRptQuery.class.getName();
    private static List<String> fieldKeys_ship = new ArrayList<>();// 销售发货

    private static List<String> fieldKeys_purchase = new ArrayList<>();//采购合同

    private static List<String> fieldKeys_hwlz = new ArrayList<>();// 货物流转

    private static List<String> fieldKeys_purinbill = new ArrayList<>();// 采购入库

    private static List<String> fieldKeys_otherinbill = new ArrayList<>();// 其他入库

    private static List<String> fieldKeys_transinbill = new ArrayList<>();// 分布调入

    private static List<String> fieldKeys_transdirbill = new ArrayList<>();// 直接调拨

    @Override
    public DataSet query(ReportQueryParam param, Object obj) throws Throwable {
        String whereFilter = createWhereStr(param);
        DataSet instockDataSet = query(whereFilter);
        return instockDataSet;
    }

    public static DataSet query(String whereFilter) {
        DataSet mainDataSet = mainDataSet(whereFilter);
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
        return createDataSet.where("tpv_qty > 0 and tpv_loansign = true").orderBy(new String[]{"tpv_shipmentsdate desc"});
    }

    private static DataSet mainDataSet(String whereFilter) {
        initAllFields();
        QFilter[] qFilters = null;
        DataSet tpv_dt_salesshipping = ORM.create().queryDataSet(algoKey, "tpv_dt_salesshipping",
                String.join(",", fieldKeys_ship), qFilters);
        if (YEM.isNotEmpty(whereFilter)) {
            tpv_dt_salesshipping = tpv_dt_salesshipping.where(whereFilter);
        }
        return tpv_dt_salesshipping;
    }

    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        return whereFilter;
    }

    private static void initAllFields() {
        fieldKeys_ship.clear();
        addFields("tpv_company", "org");//公司
        addFields("tpv_salesman", "tpv_operator");//业务员
        addFields("tpv_delivergoodsid", "tpv_materialentry.tpv_dt_salepick_in");//销售发货单拣配库存子分录id
        addFields("tpv_shipments", "billno");//销售发货单号
        addFields("tpv_department", "tpv_department");//部门
        addFields("tpv_salescontract", "tpv_contractnumber");//销售合同
        addFields("tpv_combination", "tpv_combina");//组合号
        addFields("tpv_strategy", "tpv_policyber");//策略号
        addFields("tpv_tradenature", "tpv_tradeproperty");//贸易性质
        addFields("tpv_contractdistribution", "tpv_materialentry.tpv_dt_salepick_in.tpv_contractmark");//配货合同号
        addFields("tpv_distributioncombinati", "tpv_materialentry.tpv_dt_salepick_in.tpv_combination_in");//配货组合号
        addFields("tpv_distributionstrategy", "tpv_materialentry.tpv_dt_salepick_in.tpv_strategy_in");//配货策略号
        addFields("tpv_distributionnature", "tpv_materialentry.tpv_dt_salepick_in.tpv_tradeproperty_in");//配货贸易性质
        addFields("tpv_supplier", "tpv_materialentry.tpv_dt_salepick_in.tpv_supplier_in");//配货供应商
        addFields("tpv_distribution", "tpv_materialentry.tpv_dt_salepick_in.tpv_department_in");//配货部门
        addFields("tpv_fentryid", "tpv_materialentry.tpv_dt_salepick_in.tpv_fentryid_in");//配货单据分录id
        addFields("tpv_material", "tpv_materialentry.tpv_dt_salepick_in.tpv_material_in");//商品编码
        addFields("tpv_prodname", "tpv_materialentry.tpv_dt_salepick_in.tpv_prodname_in");//品名
        addFields("tpv_shader", "tpv_materialentry.tpv_dt_salepick_in.tpv_shader_in");//材质
        addFields("tpv_origin", "tpv_materialentry.tpv_dt_salepick_in.tpv_origin_in");//产地
        addFields("tpv_specation", "tpv_materialentry.tpv_dt_salepick_in.tpv_specation_in");//规格
        addFields("tpv_ofmeasure", "tpv_materialentry.tpv_dt_salepick_in.tpv_measure");//计量单位
        addFields("tpv_volumenumber", "tpv_materialentry.tpv_dt_salepick_in.tpv_volumenumber_in");//卷号
        addFields("tpv_qty", "tpv_materialentry.tpv_dt_salepick_in.tpv_pickqty_in");//数量
        addFields("tpv_pickqty", "tpv_materialentry.tpv_dt_salepick_in.tpv_pickqty_in");//本次配货数量
        addFields("tpv_alreadycost", "tpv_materialentry.tpv_dt_salepick_in.tpv_alreadycost");//已成本置换数量
        addFields("tpv_invoiceqty", "tpv_materialentry.tpv_dt_salepick_in.tpv_invoiceqty");//已开票数量
        addFields("tpv_pickbill", "tpv_materialentry.tpv_dt_salepick_in.tpv_pickbill_in");//配货单据
        addFields("tpv_billno", "tpv_materialentry.tpv_dt_salepick_in.tpv_billno_in");//配货单据编号
        //addFields("tpv_fentryid", "tpv_materialentry.tpv_dt_salepick_in.tpv_fentryid_in");//配货单据分录id
        addFields("tpv_loansign", "tpv_materialentry.tpv_dt_salepick_in.tpv_loansign");//借货标记
        addFields("tpv_shipmentsdate", "tpv_bizdate");//发货日期
        addFields("tpv_marketcontract", "tpv_contractnumber");//销售合同号
        addFields("tpv_correspondent", "tpv_customer");//客户
        /*addFields("tpv_qty", "tpv_materialentry.tpv_dt_salepick_zy.tpv_pickqty_zy");// 数量
        addFields("tpv_ofpackages", "tpv_materialentry.tpv_dt_salepick_zy.tpv_packnum_zy");// 包装数量
        addFields("tpv_ofmeasure", "tpv_materialentry.tpv_dt_salepick_zy.tpv_ofmeasure");// 计量单位
        addFields("tpv_realsubmit", "tpv_materialentry.tpv_dt_salepick_zy.tpv_practicalok_zy");// 实提完毕
        addFields("tpv_danjutype", "billstatus");//单据状态*/
        //addFields("tpv_ofmeasure", "true");// 计量单位
    }

    static {

    }

    private static void addFields(String fieldKey, String fieldKey_bill) {
        fieldKeys_ship.add(fieldKey_bill + " " + fieldKey);
    }
}
