package com.itopview.wm.cm.utils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.itopview.wm.sys.utils.NewVersionHelper;
import com.itopview.wm.utils.YEM;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.entity.ILocaleString;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.IDataEntityType;
import kd.bos.dataentity.metadata.dynamicobject.DynamicProperty;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.ValueMapItem;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.property.ComboProp;
import kd.bos.entity.property.EntryProp;
import kd.bos.entity.property.LinkEntryProp;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.CodeEdit;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.metadata.dao.MetaCategory;
import kd.bos.metadata.dao.MetadataDao;
import kd.bos.metadata.entity.EntityItem;
import kd.bos.metadata.entity.EntityMetadata;
import kd.bos.metadata.entity.commonfield.DateRangeField;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.MetadataServiceHelper;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

/**
 * 合同条款帮助类
 *
 * @author zhangh
 */
public class ClauseContractUtil {

    /**
     * 给挖空字段添加背景值
     *
     * @param billentrykey
     * @param color
     * @param view
     */
    public static void backgroundColor(DynamicObject billentrykey, String color, IFormView view) {
        String tpv_bill = billentrykey.getString("tpv_bill");//本单标识
        String idByNumber = MetadataDao.getIdByNumber(view.getEntityId(), MetaCategory.Entity);
        EntityMetadata meta = (EntityMetadata) MetadataDao.readRuntimeMeta(idByNumber, MetaCategory.Entity);
        List<EntityItem<?>> items = meta.getItems();
        for (EntityItem<?> entityItem : items) {
            if (entityItem instanceof DateRangeField) {
                DateRangeField item = (DateRangeField) entityItem;
                String startDateFieldKey = item.getStartDateFieldKey();
                String endDateFieldKey = item.getEndDateFieldKey();
                if (tpv_bill.equals(startDateFieldKey) || tpv_bill.equals(endDateFieldKey)) {
                    tpv_bill = item.getKey();
                }
            }
        }
        String tpv_entrykey = billentrykey.getString("tpv_entrykey");
        if (!"".equals(tpv_entrykey)) {
            YEM.changeCellColor(view, tpv_entrykey, tpv_bill, 0, color);//改变控件颜色
        } else {
            YEM.changeCtlColor(view, tpv_bill, color);
        }
    }

    /**
     * 从文本字段中截取挖空字段
     *
     * @param view
     * @return
     */
    public static List<String> getHollowFieldList(IFormView view) {
        IDataModel model = view.getModel();
        DynamicObject tpv_template = (DynamicObject) model.getValue("tpv_template");//我方合同模板
        List<String> scoopField = new ArrayList<>();
        if (tpv_template != null) {
            tpv_template = BusinessDataServiceHelper.loadSingleFromCache(tpv_template.getPkValue(),
                    "tpv_cm_contracttemplate");//合同模板
            DynamicObjectCollection tpv_contractterms = tpv_template.getDynamicObjectCollection("tpv_contractterms");
            for (DynamicObject contractterms : tpv_contractterms) {
                DynamicObject tpv_termname = contractterms.getDynamicObject("tpv_termname");//条款库
                if (tpv_termname != null) {
                    String tpv_clausechina = contractterms.getString("tpv_clausescontent");
                    if (StringUtils.isNotEmpty(tpv_clausechina)) {
                        for (int i = 0; i < tpv_clausechina.length(); i++) {
                            if (tpv_clausechina.contains("【")) {
                                tpv_clausechina = cutOutString(tpv_clausechina, scoopField);
                            }
                        }
                    }
                }
            }
        } else {
            model.deleteEntryData("tpv_dynamicfield");
        }
        return scoopField;
    }

    /**
     * 根据挖空字段截取字符串
     *
     * @param tpv_clausechina
     * @param scoopField
     * @return
     */
    public static String cutOutString(String tpv_clausechina, List<String> scoopField) {
        String str = "";
        if (tpv_clausechina.contains("【")) {
            String substring = tpv_clausechina.substring(tpv_clausechina.indexOf("【") + 1,
                    tpv_clausechina.indexOf("】"));
            str = tpv_clausechina.substring(0, tpv_clausechina.indexOf("【"))
                    + tpv_clausechina.substring(tpv_clausechina.indexOf("】") + 1);
            if (!scoopField.contains(substring)) {
                scoopField.add(substring);
            }
        }
        return str;
    }

    /**
     * 判读挖空字段是否在本单据中是否存在，如果存在，在挖空字段集合中删除 获取动态挖空字段集合
     *
     * @param field         挖空字段的条款字段标识
     * @param scoopFieldNew 挖空字段集合(动态的挖空字段)
     * @param view
     */
    public static void hollowFieldHasOrNo(String field, List<String> scoopFieldNew, IFormView view) {
        IDataModel model = view.getModel();
        String name = model.getDataEntityType().getName();
        //根据当前单据标识，获取MainEntityType实体
        MainEntityType entityType = EntityMetadataCache.getDataEntityType(name);
        List<QFilter> qFilters = new ArrayList<>();
        qFilters.add(new QFilter("tpv_clausekey", QCP.equals, field));//条款字段标识
        qFilters.add(new QFilter("tpv_billkey.number", QCP.equals, name));//根据需求单据编码
        // 查询本单的字段对照
        DynamicObject bd_clausecontrast = BusinessDataServiceHelper.loadSingle("tpv_bd_clausecontrast",
                "id,tpv_fieldkey", qFilters.toArray(new QFilter[]{}));
        if (bd_clausecontrast != null) {
            // 获取字段对照中的字段标识，如果为空说明是动态挖空字段
            String tpv_fieldkey = bd_clausecontrast.getString("tpv_fieldkey");
            if (StringUtils.isNotEmpty(tpv_fieldkey)) {
                boolean whether = false;//进行判断是否是本单的挖空字段，如果是本单的挖空字段值为true ，否则，值为false
                List<IDataEntityType> entryTypes = new ArrayList<>();//获取分录的集合
                for (IDataEntityProperty item : entityType.getProperties()) {
                    //单据体控件
                    if (item instanceof EntryProp && !(item instanceof LinkEntryProp)) {
                        entryTypes.add(((EntryProp) item).getItemType());
                    }
                    String fieldname = item.getName();
                    if (fieldname.equals(tpv_fieldkey)) {// 判断挖空字段是否是表头字段
                        whether = true;
                    }
                }

                if (entryTypes.size() > 0) {
                    for (IDataEntityType iDataEntityType : entryTypes) {
                        for (IDataEntityProperty property : iDataEntityType.getProperties()) {
                            String fieldname = property.getName();
                            if (fieldname.equals(tpv_fieldkey)) {// 判断挖空字段是否是分录字段
                                whether = true;
                            }
                        }
                    }

                }
                if (!whether) {
                    scoopFieldNew.add(field);
                }
            } else {
                scoopFieldNew.add(field);
            }

        } else {
            scoopFieldNew.add(field);
        }
    }

    /**
     * 构建挖空分录
     *
     * @param field 条款字段标识
     */
    public static void createHollowEntry(String field, IFormView view) {
        IDataModel model = view.getModel();
        List<QFilter> qFilters = new ArrayList<>();
        String name = model.getDataEntityType().getName();
        qFilters.add(new QFilter("tpv_billkey.number", QCP.equals, name));
        qFilters.add(new QFilter("tpv_clausekey", QCP.equals, field));
        // 查询字段类型表，获取动态挖空字段实体
        DynamicObject clausecontrast = BusinessDataServiceHelper.loadSingle("tpv_bd_clausecontrast", "id",
                qFilters.toArray(new QFilter[]{}));
        if (clausecontrast != null) {
            clausecontrast = BusinessDataServiceHelper.loadSingleFromCache(clausecontrast.getPkValue(),
                    "tpv_bd_clausecontrast");
            int row = model.createNewEntryRow("tpv_dynamicfield");
            model.setValue("tpv_fieldname", clausecontrast, row);//tpv_fieldname
            model.setValue("tpv_fieldtype", clausecontrast.getDynamicObject("tpv_fieldtype"), row);//tpv_fieldtype
            model.setValue("tpv_format", clausecontrast.getString("tpv_format"), row);
            model.setValue("tpv_fieldkey", clausecontrast.getString("tpv_clausekey"), row);//字段标识
        }
    }

    /**
     * 构建本单挖空字段分录
     *
     * @param map  <条款字段标识， 条款字段对照实体>
     * @param key  条款字段标识
     * @param view
     */
    public static void setEntryBillkey(Map<String, DynamicObject> map, String key, IFormView view) {
        IDataModel model = view.getModel();
        DynamicObject clausecontrast = map.get(key);// 获取字段对照实体
        clausecontrast = BusinessDataServiceHelper.loadSingleFromCache(clausecontrast.getPkValue(),
                clausecontrast.getDataEntityType().getName());
        int row = model.createNewEntryRow("tpv_billentrykey");
        String tpv_field = clausecontrast.getString("tpv_field");
        boolean isFlag = clausecontrast.getBoolean("tpv_startgs");
        String field = "";
        String fieldaf = "";
        if (tpv_field.contains(".")) {
            field = tpv_field.substring(0, tpv_field.indexOf("."));
            fieldaf = tpv_field.substring(tpv_field.indexOf(".") + 1, tpv_field.length());
            model.setValue("tpv_bill", fieldaf, row);
        } else {
            model.setValue("tpv_bill", tpv_field, row);
        }
        if (field.equals("")) {
            DynamicProperty property = EntityMetadataCache.getDataEntityType(view.getEntityId()).getProperty(tpv_field);
            model.setValue("tpv_billfieldname", property.getDisplayName(), row);
        } else {
            DynamicObjectCollection coll = model.getDataEntity(true).getDynamicObjectCollection(field);
            DynamicProperty property = coll.getDynamicObjectType().getProperty(fieldaf);
            model.setValue("tpv_billfieldname", property.getDisplayName(), row);
        }

        model.setValue("tpv_entrykey", field, row);
        model.setValue("tpv_format1", clausecontrast.getString("tpv_format"), row);
        model.setValue("tpv_clausekey", clausecontrast.getString("tpv_clausekey"), row);
    }

    /**
     * js代码执行
     *
     * @param jsCode
     * @return
     */
    public static Object getRunResult(String jsCode) {
        ScriptEngineManager manager = new ScriptEngineManager();
        ScriptEngine engine = manager.getEngineByName("nashorn");// nashorn -JavaScript引擎
        Object result = null;
        try {
            result = engine.eval(jsCode);
        } catch (ScriptException e) {
            e.printStackTrace();
        }
        return result;
    }

    /***
     * 获取挖空字段的内容值
     *
     * @param tpv_entrykey    本单挖空字段分录标识
     * @param bill            分录实体
     * @param entityId        当前单据标识
     * @param dataEntity      当前单据实体
     * @return
     */


    public static String getScoopValue(String tpv_entrykey, DynamicObject bill, String entityId,
                                       DynamicObject dataEntity) {
        Object fieldValue = null;
        String tpv_bill = bill.getString("tpv_bill");
        NewVersionHelper newVersionHelper = new NewVersionHelper(entityId);
        if (newVersionHelper.isChanged()) {
            entityId = newVersionHelper.getRequireKey();
        }

        if (StringUtils.isNotEmpty(tpv_entrykey)) {
            DynamicObjectCollection entryEntity = dataEntity.getDynamicObjectCollection(tpv_entrykey);
            if (entryEntity.size() > 0) {
                fieldValue = entryEntity.get(0).get(tpv_bill);
            }
        } else {
            fieldValue = dataEntity.get(tpv_bill);
        }
        String value = ""; // 中文
        String tpv_format1 = bill.getString("tpv_format1");
        if (StringUtils.isNotEmpty(tpv_format1)) {
            if (tpv_format1.equals("PriceProp") || tpv_format1.equals("QtyProp") || tpv_format1.equals("AmountProp")
                    || tpv_format1.equals("DecimalProp")) {
                BigDecimal bigvalue = ((BigDecimal) fieldValue).setScale(2, BigDecimal.ROUND_HALF_UP);
                if (bigvalue.compareTo(BigDecimal.ZERO) > 0) {
                    value = bigvalue + "";
                }
            } else if (tpv_format1.equals("BasedataProp") || tpv_format1.equals("UserProp")
                    || tpv_format1.equals("OrgProp") || tpv_format1.equals("CurrencyProp")
                    || tpv_format1.equals("UnitProp") || tpv_format1.equals("CreaterProp")
                    || tpv_format1.equals("ModifierProp")) {
                DynamicObject basevalue = (DynamicObject) fieldValue;
                if (basevalue != null) {
                    value = basevalue.getLocaleString("name").getLocaleValue_zh_CN();
                }
            } else if (tpv_format1.equals("BooleanProp")) {
                Boolean boolValue = (Boolean) fieldValue;
                if (boolValue) {
                    value = "是";
                } else {
                    value = "否";
                }
            } else if (tpv_format1.equals("ComboProp")) {
                String comvalue = (String) fieldValue;
                if (StringUtils.isNotEmpty(comvalue)) {
                    MainEntityType type = MetadataServiceHelper
                            .getDataEntityType(dataEntity.getDataEntityType().getName());
                    ComboProp ComboProp = (ComboProp) type.findProperty(tpv_bill);
                    List<ValueMapItem> comboItems = ComboProp.getComboItems();
                    for (ValueMapItem comboItem : comboItems) {
                        if (comboItem.getValue().equals(comvalue)) {
                            value = comboItem.getName().getLocaleValue_zh_CN();
                        }
                    }
                }
            } else if (tpv_format1.equals("TextProp")) {
                if (StringUtils.isNotEmpty((String) fieldValue)) {
                    value = (String) fieldValue;
                }
            } else if (tpv_format1.equals("MuliLangTextProp")) {
                value = ((ILocaleString) fieldValue).getLocaleValue_zh_CN();
            } else if (tpv_format1.equals("DateTimeProp") || tpv_format1.equals("DateProp")
                    || tpv_format1.equals("ModifyDateProp") || tpv_format1.equals("CreateDateProp")) {
                Date date = (Date) fieldValue;
                if (date != null) {
                    ZoneId zoneId = ZoneId.systemDefault();
                    ZonedDateTime zonedDateTime = date.toInstant().atZone(zoneId);
                    LocalDate localDate = zonedDateTime.toLocalDate();
                    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy年MM月dd日");
                    value = localDate.format(formatter);
                }

            } else if (tpv_format1.equals("MulBasedataProp")) {
                DynamicObjectCollection dycollValue = (DynamicObjectCollection) fieldValue;
                List<String> chineseValue = new ArrayList<>();
                List<String> engValue = new ArrayList<>();
                for (DynamicObject dycoll : dycollValue) {
                    String cnValue = dycoll.getDynamicObject("fbasedataid").getLocaleString("name")
                            .getLocaleValue_zh_CN();
                    String enValue = dycoll.getDynamicObject("fbasedataid").getLocaleString("name").getLocaleValue_en();
                    if (!chineseValue.contains(cnValue)) {
                        chineseValue.add(cnValue);
                    }

                    if (!engValue.contains(enValue)) {
                        engValue.add(enValue);
                    }
                }

                value = String.join("、", chineseValue);
            } else if (tpv_format1.equals("IntegerProp")) {
                Integer integer = (Integer) fieldValue;
                if (integer != 0) {
                    value = integer + "";
                }
            }
        }
        return value;
    }

    /**
     * 用获取到的挖空字段内容替换条款内容中的挖空字段
     *
     * @param tpv_content     挖空字段的内容值
     * @param tpv_updateafter 条款内容
     * @param tpv_fieldkey    条款字段标识
     * @param dataEntity      当前单据实体
     * @return
     */
    public static String setupdateValue(String tpv_content, String tpv_updateafter, String tpv_fieldkey,
                                        DynamicObject dataEntity) {
        if (StringUtils.isNotEmpty(tpv_content)) {
            tpv_updateafter = tpv_updateafter.replace("【" + tpv_fieldkey + "】", tpv_content);
        }
        return tpv_updateafter;
    }

    public static String setValue(List<String> list, Map<String, DynamicObject> map,
                                  Map<String, DynamicObject> dynamicKey, IFormView view, String context) {
        DynamicObject dataEntity = view.getModel().getDataEntity(true);
        for (String key : list) {
            if (map.containsKey(key)) {
                DynamicObject bill = map.get(key);
                String tpv_clausekey = bill.getString("tpv_clausekey");
                String tpv_entrykey = bill.getString("tpv_entrykey");
                String scoopValue = getScoopValue(tpv_entrykey, bill, view.getEntityId(), dataEntity);
                context = setupdateValue(scoopValue, context, tpv_clausekey, dataEntity);
            }

            if (dynamicKey.containsKey(key)) {
                DynamicObject bill = dynamicKey.get(key);
                String tpv_contentvalue = bill.getString("tpv_contentvalue");
                if (YEM.isNotEmpty(tpv_contentvalue)) {
                    String value = bill.getLocaleString("tpv_contentvalue").getLocaleValue_zh_CN();
                    context = setupdateValue(value, context, key, dataEntity);
                }
            }
        }
        return context;
    }

    /**
     * 挖空字段值改变给修改后条款赋值
     *
     * @param iFormView
     * @param fieldkey
     * @param billentrykey
     */
    public static void getFieldValue2UpdateAfter(String fieldkey, DynamicObject billentrykey, IFormView view) {
        IDataModel model = view.getModel();
        DynamicObject dataEntity = model.getDataEntity(true);
        String name = model.getDataEntityType().getName();
        DynamicObjectCollection tpv_clauseimport = dataEntity.getDynamicObjectCollection("tpv_clauseimport");
        for (int i = 0; i < tpv_clauseimport.size(); i++) {
            DynamicObject clauseimport = tpv_clauseimport.get(i);
            String tpv_updatebefore = clauseimport.getString("tpv_updatebefore");// 修改前条款内容
            List<String> list = new ArrayList<>();//所有挖空字段的集合
            Map<String, DynamicObject> map = getKey(model);
            Map<String, DynamicObject> dynamicKey = getDynamicKey(model);
            String updatebefore = tpv_updatebefore;
            for (int j = 0; j < updatebefore.length(); j++) {
                updatebefore = cutOutString(updatebefore, list);
            }
            String context = tpv_updatebefore;
            // 处理当前行所有挖空字段的值
            for (String key : list) {
                if (map.containsKey(key)) {
                    DynamicObject bill = map.get(key);
                    String tpv_clausekey = bill.getString("tpv_clausekey");
                    String tpv_entrykey = bill.getString("tpv_entrykey");

                    String scoopValue = getScoopValue(tpv_entrykey, bill, view.getEntityId(), dataEntity);
                    context = setupdateValue(scoopValue, context, tpv_clausekey, dataEntity);
                    DynamicObject bDclausecontrast = BusinessDataServiceHelper.loadSingle("tpv_bd_clausecontrast", "tpv_startgs,tpv_bcode_tag", new QFilter[]{new QFilter("tpv_billkey.number", "=", name), new QFilter("tpv_clausekey", "=", tpv_clausekey)});
                    if (bDclausecontrast!=null){
                        boolean tpvStartgs = bDclausecontrast.getBoolean("tpv_startgs");
                        if (tpvStartgs) {
                            String jsCode = (String) bDclausecontrast.get("tpv_bcode_tag");
                            jsCode = jsCode.replaceAll("dynamic", "'" + context + "'");
                            Object runResult = getRunResult(jsCode);
                            if (runResult != null) {
                                context = context + (String) getRunResult(jsCode);
                            }
                        }
                    }

                    if (map.containsKey(key)) {
                        model.setValue("tpv_billcontent", scoopValue, map.get(key).getInt("seq") - 1);
                    }
                }
                if (dynamicKey.containsKey(key)) {
                    DynamicObject bill = dynamicKey.get(key);
                    String tpv_contentvalue = bill.getString("tpv_contentvalue");
                    DynamicObject bDclausecontrast = BusinessDataServiceHelper.loadSingle("tpv_bd_clausecontrast", "tpv_startgs,tpv_bcode_tag", new QFilter[]{new QFilter("tpv_billkey.number", "=", name), new QFilter("tpv_clausekey", "=", key)});
                    if (bDclausecontrast!=null){
                        boolean tpvStartgs = bDclausecontrast.getBoolean("tpv_startgs");
                        if (tpvStartgs) {
                            String jsCode = (String) bDclausecontrast.get("tpv_bcode_tag");
                            jsCode = jsCode.replaceAll("dynamic", "'" + context + "'");
                            Object runResult = getRunResult(jsCode);
                            if (runResult != null) {
                                context = context + (String) getRunResult(jsCode);
                            }
                        }
                    }
                    if (YEM.isNotEmpty(tpv_contentvalue)) {
                        String value = bill.getLocaleString("tpv_contentvalue").getLocaleValue_zh_CN();
                        context = setupdateValue(value, context, key, dataEntity);
                    }
                }

            }

            // 最终结果赋值，给修改后条款赋值
            model.setValue("tpv_updateafter", context, i);
            model.setValue("tpv_updateafter_tag", context, i);
        }
    }

    /**
     * 获取本单挖空字段集合
     *
     * @param model
     */
    public static Map<String, DynamicObject> getKey(IDataModel model) {
        Map<String, DynamicObject> map = new HashMap<>();//Map<条款字段标识,分录实体>
        DynamicObjectCollection tpv_billentrykey = model.getEntryEntity("tpv_billentrykey");
        for (DynamicObject billentrykey : tpv_billentrykey) {
            map.put(billentrykey.getString("tpv_clausekey"), billentrykey);
        }
        return map;
    }

    /**
     * 获取动态挖空字段的集合
     *
     * @return
     */
    public static Map<String, DynamicObject> getDynamicKey(IDataModel model) {
        Map<String, DynamicObject> map = new HashMap<>();//Map<条款字段标识,分录实体>
        DynamicObjectCollection tpv_billentrykey = model.getEntryEntity("tpv_dynamicfield");
        for (DynamicObject billentrykey : tpv_billentrykey) {
            map.put(billentrykey.getString("tpv_fieldkey"), billentrykey);
        }
        return map;
    }

    public static void showForm(IFormView view, AbstractBillPlugIn plugin) {
        IDataModel model = view.getModel();
        int index = model.getEntryCurrentRowIndex("tpv_dynamicfield");
        DynamicObject tpv_dynamicfield = model.getEntryRowEntity("tpv_dynamicfield", index);
        DynamicObject tpv_fieldtype = tpv_dynamicfield.getDynamicObject("tpv_fieldtype");
        if (tpv_fieldtype != null) {
            tpv_fieldtype = BusinessDataServiceHelper.loadSingleFromCache(tpv_fieldtype.getPkValue(),
                    "tpv_bd_fieldtype");
            String tpv_keytype = tpv_fieldtype.getString("tpv_keytype");
            if (StringUtils.isNotEmpty(tpv_keytype)) {
                switch (tpv_keytype) {
                    case "ComboProp":
                        skipPullDownList(tpv_fieldtype, view, plugin, "tpv_cm_comboitemssel");
                        break;

                    case "BasedataProp":
                    case "UnitProp":
                    case "UserProp":
                    case "OrgProp":
                    case "CurrencyProp":
                    case "MulBasedataProp":
                        skipInitialValue(tpv_fieldtype, view, plugin, "tpv_cm_basedata");
                        break;

                    case "DateTimeProp":
                        skipDateControl(tpv_fieldtype, view, plugin, "md_forexquote_date");
                        break;

                    default:
                        break;
                }
            }
        }
    }

    /**
     * 跳转日期控件
     *
     * @param tpv_fieldtype
     */
    private static void skipDateControl(DynamicObject tpv_fieldtype, IFormView view, AbstractBillPlugIn plugin,
                                        String formKey) {
        FormShowParameter showParameter = new FormShowParameter();
        showParameter.setStatus(OperationStatus.ADDNEW);
        showParameter.getOpenStyle().setShowType(ShowType.Modal);
        showParameter.setFormId(formKey);
        showParameter.setCloseCallBack(new CloseCallBack(plugin, formKey));
        view.showForm(showParameter);
    }

    /**
     * 跳转基础资料
     *
     * @param tpv_fieldtype
     */
    private static void skipInitialValue(DynamicObject tpv_fieldtype, IFormView view, AbstractBillPlugIn plugin,
                                         String formKey) {
        DynamicObject tpv_billkey = tpv_fieldtype.getDynamicObject("tpv_billkey");
        if (tpv_billkey != null) {
            FormShowParameter showParameter = new FormShowParameter();
            showParameter.setStatus(OperationStatus.ADDNEW);
            showParameter.getOpenStyle().setShowType(ShowType.Modal);
            showParameter.setCustomParam("billkey", tpv_billkey.getString("number"));
            showParameter.setFormId(formKey);
            showParameter.setCloseCallBack(new CloseCallBack(plugin, formKey));
            view.showForm(showParameter);
        }
    }

    /**
     * 跳转下拉列表控件
     *
     * @param tpv_fieldtype
     */
    private static void skipPullDownList(DynamicObject tpv_fieldtype, IFormView view, AbstractBillPlugIn plugin,
                                         String formKey) {
        DynamicObjectCollection tpv_pulldownentry = tpv_fieldtype.getDynamicObjectCollection("tpv_pulldownentry");
        List<Map<String, Object>> list = new ArrayList<>();

        for (DynamicObject pulldownentry : tpv_pulldownentry) {
            Map<String, Object> map = new HashMap<>();
            map.put("EnName", pulldownentry.getLocaleString("tpv_fcaption").getLocaleValue_en());
            map.put("CnName", pulldownentry.getLocaleString("tpv_fcaption").getLocaleValue_zh_CN());
            map.put("Value", pulldownentry.getString("tpv_fvalue"));
            map.put("ImageKey", "");
            if (map.size() > 0) {
                list.add(map);
            }
        }

        if (list.size() > 0) {
            FormShowParameter showParameter = new FormShowParameter();
            showParameter.setStatus(OperationStatus.ADDNEW);
            showParameter.getOpenStyle().setShowType(ShowType.Modal);
            showParameter.setFormId(formKey);
            showParameter.setCustomParam("comboProp", list);
            showParameter.setCloseCallBack(new CloseCallBack(plugin, formKey));
            view.showForm(showParameter);
        }
    }

    public static void backValue(IFormView view, ClosedCallBackEvent e) {
        String key = e.getActionId();
        Object returnData = e.getReturnData();
        switch (key) {
            case "tpv_cm_basedata":
                backBasedValue(returnData, view);
                break;
            case "md_forexquote_date":
                backDateValue(returnData, view);
                break;
            case "tpv_cm_comboitemssel":
                backPullDownValue(returnData, view);
                break;

            default:
                break;
        }
    }

    private static void backPullDownValue(Object returnData, IFormView view) {
        DynamicObject data = (DynamicObject) returnData;
        IDataModel model = view.getModel();
        if (data != null) {
            int index = model.getEntryCurrentRowIndex("tpv_dynamicfield");
            LocaleString local = new LocaleString();
            String tpv_fcaptionCn = data.getLocaleString("tpv_fcaption").getLocaleValue_zh_CN();
            local.setLocaleValue_zh_CN(tpv_fcaptionCn);
            DynamicObject tpv_dynamicfield = model.getEntryRowEntity("tpv_dynamicfield", index);
            tpv_dynamicfield.set("tpv_contentvalue", local);
            view.updateView("tpv_dynamicfield");
            view.setEnable(false, index, "tpv_contentvalue");
            updateCasule(tpv_dynamicfield, tpv_fcaptionCn, view);
        }
    }

    /**
     * 跳转日期控件，返回数据
     *
     * @param returnData
     * @param view
     */
    private static void backDateValue(Object returnData, IFormView view) {
        Date data = (Date) returnData;
        IDataModel model = view.getModel();
        int index = model.getEntryCurrentRowIndex("tpv_dynamicfield");
        if (data != null) {
            LocaleString local = new LocaleString();
            String tpv_fcaptionCn = "";
            ZoneId zoneId = ZoneId.systemDefault();
            ZonedDateTime zonedDateTime = data.toInstant().atZone(zoneId);
            LocalDate localDate = zonedDateTime.toLocalDate();
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy年MM月dd日");
            tpv_fcaptionCn = localDate.format(formatter);
            local.setLocaleValue_zh_CN(tpv_fcaptionCn);
            DynamicObject tpv_dynamicfield = model.getEntryRowEntity("tpv_dynamicfield", index);
            tpv_dynamicfield.set("tpv_contentvalue", local);
            view.updateView("tpv_dynamicfield");
            view.setEnable(false, index, "tpv_contentvalue");
            updateCasule(tpv_dynamicfield, tpv_fcaptionCn, view);
        }
    }

    /**
     * backPullDownValue 跳转基础资料控件，返回数据
     *
     * @param returnData
     */
    @SuppressWarnings("unchecked")
    private static void backBasedValue(Object returnData, IFormView view) {
        IDataModel model = view.getModel();
        int index = model.getEntryCurrentRowIndex("tpv_dynamicfield");
        List<DynamicObject> list = (List<DynamicObject>) returnData;
        if (list != null) {
            List<String> CnValue = new ArrayList<>();
            List<String> basedid = new ArrayList<>();
            for (DynamicObject bill : list) {
                CnValue.add(bill.getLocaleString("tpv_fcaption").getLocaleValue_zh_CN());
                basedid.add(bill.getString("tpv_baseid"));
            }
            LocaleString local = new LocaleString();
            local.setLocaleValue_zh_CN(String.join("、", CnValue));

            DynamicObject tpv_dynamicfield = model.getEntryRowEntity("tpv_dynamicfield", index);
            tpv_dynamicfield.set("tpv_contentvalue", local);
            tpv_dynamicfield.set("tpv_baseid", String.join("、", basedid));
            view.updateView("tpv_dynamicfield");
            view.setEnable(false, index, "tpv_contentvalue");
            updateCasule(tpv_dynamicfield, String.join("、", CnValue), view);
        }

    }

    /**
     * 合同动态分录中点击文本框按钮，弹出框回填信息之后，修改合同条款分录中的值
     *
     * @param tpv_dynamicfield 动态字段分录实体
     * @param tpv_fcaptionCn   中文内容值
     * @param view
     * @param tpv_fcaptionEn   英文内容值
     */
    public static void updateCasule(DynamicObject tpv_dynamicfield, String tpv_fcaptionCn, IFormView view) {
        String tpv_fieldkey = tpv_dynamicfield.getString("tpv_fieldkey");
        IDataModel model = view.getModel();
        DynamicObject dataEntity = model.getDataEntity(true);
        DynamicObjectCollection tpv_clauseimport = model.getEntryEntity("tpv_clauseimport");
        for (int i = 0; i < tpv_clauseimport.size(); i++) {
            DynamicObject clauseimport = tpv_clauseimport.get(i);
            String tpv_updatebefore = clauseimport.getString("tpv_updatebefore");
            if (tpv_updatebefore.contains(tpv_fieldkey)) {
                List<String> list = new ArrayList<>();
                Map<String, DynamicObject> map = getKey(model);
                Map<String, DynamicObject> dynamicKey = getDynamicKey(model);
                String updatebefore = tpv_updatebefore;
                for (int j = 0; j < updatebefore.length(); j++) {
                    updatebefore = cutOutString(updatebefore, list);
                }
                String context = tpv_updatebefore;
                for (String key : list) {
                    if (map.containsKey(key)) {
                        DynamicObject bill = map.get(key);
                        String tpv_clausekey = bill.getString("tpv_clausekey");
                        String tpv_entrykey = bill.getString("tpv_entrykey");
                        String scoopValue = getScoopValue(tpv_entrykey, bill, view.getEntityId(), dataEntity);
                        context = setupdateValue(scoopValue, context, tpv_clausekey, dataEntity);
                    }

                    if (dynamicKey.containsKey(key)) {
                        DynamicObject bill = dynamicKey.get(key);
                        String tpv_contentvalue = bill.getString("tpv_contentvalue");
                        if (YEM.isNotEmpty(tpv_contentvalue)) {
                            String value = bill.getLocaleString("tpv_contentvalue").getLocaleValue_zh_CN();
                            context = setupdateValue(value, context, key, dataEntity);
                        }
                    }
                }
                context = setupdateValue(tpv_fcaptionCn, context, tpv_fieldkey, dataEntity);

                model.setValue("tpv_updateafter", context, i);
                model.setValue("tpv_updateafter_tag", context, i);
            }
        }
    }

    /**
     * 获取多选基础资料的值
     *
     * @param entryKey         分录标识
     * @param newDynamicObject 创建的辅助资料
     * @param data             单据元数据
     * @param name             单据标识
     * @return
     */
    public static DynamicObjectCollection getMaterialList(String entryKey, DynamicObject newDynamicObject,
                                                          DynamicObject data, String name) {
        DynamicObjectCollection entrydata = data.getDynamicObjectCollection(entryKey);
        DynamicObjectCollection materialList = newDynamicObject.getDynamicObjectCollection("tpv_material");

        for (DynamicObject entry : entrydata) {
            if (name.equals("pm_purorderbill") || name.equals("sm_salorder") || name.equals("tpv_pm_purorderbill_inh") || name.equals("tpv_sm_salorder_inh")) {
                DynamicObject tpv_material = entry.getDynamicObject("material");
                if (tpv_material != null) {
                    DynamicObject material = tpv_material.getDynamicObject("masterid");
                    DynamicObject newMaterial = new DynamicObject(materialList.getDynamicObjectType());
                    newMaterial.set("fbasedataId", material);
                    materialList.add(newMaterial);
                }
            } else {
                DynamicObject tpv_material = entry.getDynamicObject("tpv_material");
                if (tpv_material != null) {
                    DynamicObject newMaterial = new DynamicObject(materialList.getDynamicObjectType());
                    newMaterial.set("fbasedataId", tpv_material);
                    materialList.add(newMaterial);
                }
            }
        }
        return materialList;
    }
}
