package com.itopview.wm.cm.utils;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.taxc.common.util.StringUtil;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 成本处理报表，返回数据帮助类
 * @Author LinRui
 * @Date 2023/11/17 17:19
 */
public class CostDisposeRptUtils {
    private static final String algoKey = CostDisposeRptUtils.class.getName();

    /**
     * @Description: //成本处理可红冲数量查询表
     * @param: storageid - [java.lang.String] - 入库分录id
     * @param: tentauth - [java.lang.String] - 暂估/真实
     * @param: costype - [java.lang.String] - 陈本类型
     * @return: kd.bos.algo.DataSet
     **/
    public static DataSet getCostdisposeredRptQuery(String storageid, String tentauth, String costype) {
        QFilter[] qFilters = new QFilter[]{new QFilter("billstatus", QCP.equals, "C")};

        DataSet tpvCmCostdispose = ORM.create().queryDataSet(algoKey, "tpv_cm_costdispose",
                "id,billno,tpv_itemcode,tpv_itemcode.name,tpv_conprodname,tpv_conorigin,tpv_conspecation,tpv_bizdate," +
                        "tpv_conshader,createtime,modifytime,tpv_bluered,tpv_costdirec,tpv_tentauth," +
                        "tpv_costype,tpv_currency,tpv_origprice,tpv_rate,tpv_origampintax,tpv_origampnotax," +
                        "tpv_origamptax,tpv_basampintax,tpv_basampnotax,tpv_basamptax,tpv_dollarintax," +
                        "tpv_dollarnotax,tpv_dollartax,tpv_soudocumtype,tpv_soudocumnum,tpv_soudocumid," +
                        "tpv_tiggdoutype,tpv_tiggdounum,tpv_tiggdouid,tpv_storagenum,tpv_storageid,tpv_supstoragenum," +
                        "tpv_supstorageid,tpv_movegoodsnum,tpv_movegoodsid,tpv_msungoodsid,tpv_quantity," +
                        "tpv_elimination,tpv_purconnum,tpv_combinbun,tpv_vendor,tpv_logserpro,tpv_saleconnum," +
                        "tpv_client,tpv_tradeproperty,tpv_drawback,tpv_refundrate",
                qFilters);
        DataSet tpvEliminationId = tpvCmCostdispose.copy();  //id = tpv_elimination
        List<String> tpvElimination_s = new ArrayList<>();
        for (Row row : tpvEliminationId) {
            String tpvElimination = row.getString("tpv_elimination");
            if (StringUtils.isNotBlank(tpvElimination)) {
                tpvElimination_s.add(tpvElimination);
            }
        }
        DataSet redQty = tpvCmCostdispose.copy();
        if (tpvElimination_s.size() > 0) {
            redQty = redQty.where(String.format("id in (%s)", StringUtil.strip(tpvElimination_s.toString(), "[]")))
                    .groupBy(new String[]{"id", "tpv_elimination"})
                    .sum("tpv_quantity").finish().select("id", "tpv_quantity redQty");
        } else {
            redQty = redQty.select("id", "0 redQty");
        }

        String[] selectFiled = new String[]{"id tpv_id", "billno tpv_billno", "tpv_itemcode", "tpv_conprodname", "tpv_conorigin", "tpv_conspecation", "tpv_bizdate",
                "tpv_conshader", "createtime tpv_createtime", "modifytime tpv_modifytime", "tpv_bluered", "tpv_costdirec", "tpv_tentauth",
                "tpv_costype", "tpv_currency", "tpv_origprice", "tpv_rate", "tpv_origampintax", "tpv_origampnotax",
                "tpv_origamptax", "tpv_basampintax", "tpv_basampnotax", "tpv_basamptax", "tpv_dollarintax", "tpv_dollarnotax",
                "tpv_dollartax", "tpv_soudocumtype", "tpv_soudocumnum", "tpv_soudocumid", "tpv_tiggdoutype", "tpv_tiggdounum",
                "tpv_tiggdouid", "tpv_storagenum", "tpv_storageid", "tpv_supstoragenum", "tpv_supstorageid", "tpv_movegoodsnum",
                "tpv_movegoodsid", "tpv_msungoodsid", "tpv_quantity + redQty tpv_redqty", "tpv_elimination", "tpv_purconnum",
                "tpv_combinbun", "tpv_vendor", "tpv_logserpro", "tpv_saleconnum", "tpv_client", "tpv_tradeproperty", "tpv_drawback",
                "tpv_refundrate"};

        DataSet isPointprice = tpvCmCostdispose.where("tpv_bluered = 'A'").leftJoin(redQty).on("id", "id")
                .select(selectFiled).finish().where("tpv_ispointpriceed = 'A'").orderBy(new String[]{"tpv_bizdate desc"});
        DataSet notPointprice = tpvCmCostdispose.where("tpv_bluered = 'A'").leftJoin(redQty).on("id", "id")
                .select(selectFiled).finish().where("tpv_ispointpriceed = 'B'").orderBy(new String[]{"tpv_bizdate desc"});
        DataSet dataSet = isPointprice.union(notPointprice).select(selectFiled);
        String whereFilter = "1 = 1";
        // 入库分录id
        if (YEM.isNotEmpty(storageid)) {
            whereFilter = String.format("%s and tpv_storageid = '%s'", whereFilter, storageid);
        }
        // 暂估/真实
        if (YEM.isNotEmpty(storageid)) {
            whereFilter = String.format("%s and tpv_storageid = '%s'", whereFilter, tentauth);
        }
        // 成本类型
        if (YEM.isNotEmpty(storageid)) {
            whereFilter = String.format("%s and tpv_storageid = '%s'", whereFilter, costype);
        }
        dataSet = dataSet.where(whereFilter);
        return dataSet;
    }
}
