package com.itopview.wm.crm.utils;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.GroupbyDataSet;
import kd.bos.algo.JoinDataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

public class CusCreditLimitUtils {

    private static final String algoKey = CusCreditLimitUtils.class.getName();

    /**
     * 客户赊销额度-合同明细
     *
     * @param depId      部门ID
     * @param customerId 客户ID
     * @param contractNo 合同号
     * @param isZeroHide 占用余额全部为0不显示
     * @return
     */

    public static DynamicObjectCollection getCusCreditLimit(long depId, long customerId, String contractNo, boolean isZeroHide) {

        //销售合同
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
        qFilter1.and(new QFilter("tpv_isselloncredit", QCP.equals, true));  //【是否赊销】=“是”
        if (0L != depId) {
            qFilter1.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter1.and(new QFilter("tpv_customer.id", QCP.equals, customerId));
        }
        if (StringUtils.isNotEmpty(contractNo)) {
            qFilter1.and(new QFilter("tpv_contractnos", QCP.ILIKE, "%" + contractNo + "%"));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salescontract",
                        "id contractId,tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
                                "tpv_customer.id tpv_customerId,tpv_customer.name tpv_customerName,tpv_contractnos,tpv_bizdate,tpv_paymentstatus," +
                                "tpv_materialentry.tpv_effectivequantity*tpv_materialentry.tpv_taxprice contractAmt",
                        new QFilter[]{qFilter1}, null)
                .groupBy(new String[]{"contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
                        "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus"})
                .sum("contractAmt")
                .finish();

        //销售发货
        QFilter qFilter2 = new QFilter("billstatus", QCP.equals, "C");
        if (0L != depId) {
            qFilter2.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter2.and(new QFilter("tpv_customer.id", QCP.equals, customerId));
        }
        if (StringUtils.isNotEmpty(contractNo)) {
            qFilter1.and(new QFilter("tpv_contractnumber", QCP.ILIKE, "%" + contractNo + "%"));
        }
        DataSet dataSet2 = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salesshipping",
                        "tpv_sourcebillid,tpv_materialentry.tpv_qty*tpv_materialentry.tpv_taxprice shippingAmt",
                        new QFilter[]{qFilter2}, null)
                .groupBy(new String[]{"tpv_sourcebillid"})
                .sum("shippingAmt")
                .finish();

        DataSet dataSet3 = dataSet1.leftJoin(dataSet2).on("contractId", "tpv_sourcebillid")
                .select("contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
                        "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus", "contractAmt", "shippingAmt")
                .finish();

        //占用减项  暂时为0，待可放余额表完成后，联查取值
        DataSet cuscreditBalance = getDeliverableBalanceConData(0L, 0L, contractNo,false)
                .groupBy(new String[]{"contractId"}).sum("tpv_amt14").finish().select("contractId contractNo", "tpv_amt14 e_actamt");
//        DataSet cuscreditBalance = dataSet3.select("contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
//                "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus", "contractAmt", "shippingAmt", "0.0 e_actamt");
        DataSet dataSet4 = dataSet3.leftJoin(cuscreditBalance).on("contractId", "contractNo")
                .select("contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
                        "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus", "contractAmt", "shippingAmt", "(case when e_actamt == null then 0.0 else e_actamt end) e_actamt")
                .finish();
        //(签约)占用余额
        dataSet4 = dataSet4.select("contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
                "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus", "contractAmt", "shippingAmt", "e_actamt", "contractAmt - e_actamt uncontractAmt");
        //(送货)占用余额
        dataSet4 = dataSet4.select("contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
                        "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus", "contractAmt", "shippingAmt", "e_actamt", "uncontractAmt", "shippingAmt - e_actamt unshippingAmt")
                .orderBy(new String[]{"tpv_bizdate desc"});
        if (isZeroHide) {
            dataSet4 = dataSet4.where("uncontractAmt<>0 and unshippingAmt<>0");
        }
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(dataSet4);
        return col_s;
    }

    public static DataSet getCusCreditLimits(long depId, long customerId, long contractId, boolean isZeroHide) {
        //销售合同
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
        qFilter1.and(new QFilter("tpv_isselloncredit", QCP.equals, true));
        if (0L != depId) {
            qFilter1.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter1.and(new QFilter("tpv_customer.id", QCP.equals, customerId));
        }
        if (0L != contractId) {
            qFilter1.and(new QFilter("id", QCP.equals, contractId));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salescontract",
                        "id contractId,tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
                                "tpv_customer.id tpv_customerId,tpv_customer.name tpv_customerName,tpv_contractnos,tpv_bizdate,tpv_paymentstatus," +
                                "tpv_materialentry.tpv_effectivequantity*tpv_materialentry.tpv_taxprice contractAmt",
                        new QFilter[]{qFilter1}, null)
                .groupBy(new String[]{"contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
                        "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus"})
                .sum("contractAmt")
                .finish();

        //销售发货
        QFilter qFilter2 = new QFilter("billstatus", QCP.equals, "C");
        if (0L != depId) {
            qFilter2.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter2.and(new QFilter("tpv_customer.id", QCP.equals, customerId));
        }
        if (0L != contractId) {
            qFilter2.and(new QFilter("tpv_sourcebillid", QCP.equals, contractId));
        }
        DataSet dataSet2 = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salesshipping",
                        "tpv_sourcebillid,tpv_materialentry.tpv_qty*tpv_materialentry.tpv_taxprice shippingAmt",
                        new QFilter[]{qFilter2}, null)
                .groupBy(new String[]{"tpv_sourcebillid"})
                .sum("shippingAmt")
                .finish();

        DataSet dataSet3 = dataSet1.leftJoin(dataSet2).on("contractId", "tpv_sourcebillid")
                .select("contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
                        "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus", "contractAmt", "shippingAmt")
                .finish();

        //占用减项
        DataSet cuscreditBalance = getDeliverableBalanceConData(0L, 0L, "",false)
                .groupBy(new String[]{"contractId"}).sum("tpv_amt14").finish().select("contractId contractNo", "tpv_amt14 e_actamt");
        DataSet dataSet4 = dataSet3.leftJoin(cuscreditBalance).on("contractId", "contractNo")
                .select("contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
                        "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus", "contractAmt", "shippingAmt", "e_actamt")
                .finish();
        //(签约)占用余额
        dataSet4 = dataSet4.select("contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
                "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus", "contractAmt", "shippingAmt", "e_actamt", "contractAmt - e_actamt uncontractAmt");
        //(送货)占用余额
        dataSet4 = dataSet4.select("contractId", "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName",
                "tpv_contractnos", "tpv_bizdate", "tpv_paymentstatus", "contractAmt", "shippingAmt", "e_actamt", "uncontractAmt tpv_uncontractamt", "shippingAmt - e_actamt tpv_unshippingamt");
        if (isZeroHide) {
            dataSet4 = dataSet4.where("tpv_uncontractamt<>0 and tpv_unshippingamt<>0");
        }
        return dataSet4;
    }

    /**
     * 客户赊销额度使用情况表
     *
     * @param tpv_cusgroup_f 客户集团
     * @param tpv_custom_f   客户ID
     * @param orgId_f        部门ID
     * @return
     */
    public static DynamicObjectCollection getCusCreditLimitUse(String tpv_cusgroup_f, long tpv_custom_f, long orgId_f) {
        //客户额度维护
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
//        qFilter1.and(new QFilter("tpv_invalids", QCP.equals, false));
//        qFilter1.and(new QFilter("tpv_crm_cuslimit.tpv_frozenstate", QCP.equals, false));
        if (StringUtils.isNotEmpty(tpv_cusgroup_f)) {
            qFilter1.and(new QFilter("tpv_cusgroup", QCP.equals, tpv_cusgroup_f));
        }
        if (0L != orgId_f) {
            qFilter1.and(new QFilter("org", QCP.equals, orgId_f));
        }
        if (0L != tpv_custom_f) {
            qFilter1.and(new QFilter("tpv_crm_cuslimit.tpv_custom", QCP.equals, tpv_custom_f));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_crm_custmain",
                        "id,tpv_orgcredit,tpv_organization.id orgId,tpv_organization.name orgName,tpv_cusgroup,tpv_groupconamt,tpv_groupdelim," +
                                "tpv_crm_cuslimit.tpv_custom.id tpv_customId,tpv_crm_cuslimit.tpv_custom.name tpv_customName," +
                                "tpv_crm_cuslimit.tpv_quotatype tpv_quotatype,tpv_crm_cuslimit.tpv_limitamt tpv_limitamt",
                        new QFilter[]{qFilter1}, null)
                .groupBy(new String[]{"id", "tpv_orgcredit", "orgId", "orgName", "tpv_cusgroup", "tpv_customId", "tpv_customName", "tpv_quotatype"})
                .sum("tpv_limitamt") //tpv_groupconamt、tpv_groupdelim
                .finish()
                .select("id", "tpv_orgcredit", "orgId", "orgName", "tpv_cusgroup", "tpv_customId", "tpv_customName",
                        "(case when tpv_quotatype == 'A' then tpv_limitamt else 0.0 end) signingAmt",
                        "(case when tpv_quotatype == 'B' then tpv_limitamt else 0.0 end) shippingAmt")
                .groupBy(new String[]{"id", "tpv_orgcredit", "orgId", "orgName", "tpv_cusgroup", "tpv_customId", "tpv_customName"})
                .sum("signingAmt").sum("shippingAmt").finish();

//        DynamicObjectCollection cusCreditLimit_s = getCusCreditLimit(orgId_f, tpv_custom_f, 0L, true);
//
//        String[] FIELDS = {"tpv_creditType", "tpv_orgcredit", "orgId", "orgName", "tpv_cusgroup", "tpv_customId", "tpv_customName",
//                "signingAmt", "shippingAmt", "signingUseAmt", "shippingUseAmt", "signingResidueAmt", "shippingResidueAmt"};
//        DataType[] DATATYPES = {DataType.StringType, DataType.StringType, DataType.StringType, DataType.StringType, DataType.StringType,
//                DataType.StringType, DataType.StringType, DataType.BigDecimalType, DataType.BigDecimalType, DataType.BigDecimalType,
//                DataType.BigDecimalType, DataType.BigDecimalType, DataType.BigDecimalType};
//        Collection<Object[]> coll = new ArrayList<>();
//        RowMeta createRowMeta = RowMetaFactory.createRowMeta(FIELDS, DATATYPES);
//        CollectionInput collectionInput = new CollectionInput(createRowMeta, coll);
//        DataSet createDataSet = Algo.create(algoKey).createDataSet(collectionInput);
//        List<HashMap<String, Object>> resultList = new ArrayList<>();
//        for (Row row : dataSet1) {
//            String tpv_orgcredit = row.getString("tpv_orgcredit");//公司/部门授信
//            long orgId = row.getLong("orgId");//组织ID
//            String orgName = row.getString("orgName");//组织名称
//            String tpv_cusgroup = row.getString("tpv_cusgroup");//客户集团
//            BigDecimal tpv_groupconamt = row.getBigDecimal("tpv_groupconamt");//集团签约额度（万元）
//            BigDecimal tpv_groupdelim = row.getBigDecimal("tpv_groupdelim");//集团送货额度（万元）
//            long tpv_customId = row.getLong("tpv_customId");//客户ID
//            String tpv_customName = row.getString("tpv_customName");//客户名称
//            String tpv_quotatype = row.getString("tpv_quotatype");//额度类型
//            BigDecimal tpv_limitamt = row.getBigDecimal("tpv_limitamt");//额度金额
//            String tpv_creditType = "A";//授信类型 枚举（总额度/单独授信）
//            BigDecimal signingAmt = tpv_groupconamt;//额度-签约
//            BigDecimal shippingAmt = tpv_groupdelim;//额度-送货
//            if (tpv_limitamt.compareTo(BigDecimal.ZERO) > 0) {
//                tpv_creditType = "B";
//                if ("Signing".equals(tpv_quotatype)) {
//                    signingAmt = tpv_limitamt;
//                } else if ("delivergoods".equals(tpv_quotatype)) {
//                    shippingAmt = tpv_limitamt;
//                }
//            }
//            BigDecimal signingUseAmt = BigDecimal.ZERO;//占用-签约
//            BigDecimal shippingUseAmt = BigDecimal.ZERO;//占用-送货
//            for (DynamicObject cusCreditLimit : cusCreditLimit_s) {
//                long tpv_departmentId = cusCreditLimit.getLong("tpv_departmentId");//部门ID
//                long tpv_customerId = cusCreditLimit.getLong("tpv_customerId");//客户ID
//                BigDecimal uncontractAmt = cusCreditLimit.getBigDecimal("uncontractAmt");//占用余额-签约
//                BigDecimal unshippingAmt = cusCreditLimit.getBigDecimal("unshippingAmt");//占用余额-送货
//                if ("A".equals(tpv_orgcredit) && tpv_customId == tpv_customerId) {
//                    signingUseAmt = BigDecimalUtils.add(signingUseAmt, uncontractAmt, 10);
//                    shippingUseAmt = BigDecimalUtils.add(shippingUseAmt, unshippingAmt, 10);
//                } else if ("B".equals(tpv_orgcredit) && 0L != orgId
//                        && tpv_customId == tpv_customerId && orgId == tpv_departmentId) {
//                    signingUseAmt = BigDecimalUtils.add(signingUseAmt, uncontractAmt, 10);
//                    shippingUseAmt = BigDecimalUtils.add(shippingUseAmt, unshippingAmt, 10);
//                }
//            }
//            BigDecimal signingResidueAmt = BigDecimalUtils.sub(signingAmt, signingUseAmt, 10);//剩余额度-签约
//            BigDecimal shippingResidueAmt = BigDecimalUtils.sub(shippingAmt, shippingUseAmt, 10);//剩余额度-送货
//            HashMap<String, Object> hashMap = new HashMap<String, Object>();
//            String key = tpv_creditType + "_" + tpv_orgcredit + "_" + orgId + "_" + tpv_cusgroup + "_" + tpv_customId;
//            int index = -1;
//            for (int i = 0; i < resultList.size(); i++) {
//                HashMap<String, Object> hashMap_old = resultList.get(i);
//                String key_old = hashMap_old.get("key").toString();
//                if (key.equals(key_old)) {
//                    index = i;
//                    hashMap = hashMap_old;
//                }
//            }
//            hashMap.put("tpv_creditType", tpv_creditType);
//            hashMap.put("tpv_orgcredit", tpv_orgcredit);
//            hashMap.put("orgId", orgId);
//            hashMap.put("orgName", orgName);
//            hashMap.put("tpv_cusgroup", tpv_cusgroup);
//            hashMap.put("tpv_customId", tpv_customId);
//            hashMap.put("tpv_customName", tpv_customName);
//            if ("Signing".equals(tpv_quotatype)) {
//                hashMap.put("signingAmt", signingAmt);
//                hashMap.put("signingUseAmt", signingUseAmt);
//                hashMap.put("signingResidueAmt", signingResidueAmt);
//            } else if ("delivergoods".equals(tpv_quotatype)) {
//                hashMap.put("shippingAmt", shippingAmt);
//                hashMap.put("shippingUseAmt", shippingUseAmt);
//                hashMap.put("shippingResidueAmt", shippingResidueAmt);
//            }
//            hashMap.put("key", key);
//            if (-1 != index) {
//                resultList.set(index, hashMap);
//            } else {
//                resultList.add(hashMap);
//            }
//        }
//        for (int i = 0; i < resultList.size(); i++) {
//            HashMap<String, Object> hashMap = resultList.get(i);
//            Object[] tempData = new Object[FIELDS.length];
//            Object tpv_creditType = hashMap.get("tpv_creditType");
//            Object tpv_orgcredit = hashMap.get("tpv_orgcredit");
//            Object orgId = hashMap.get("orgId");
//            Object orgName = hashMap.get("orgName");
//            Object tpv_cusgroup = hashMap.get("tpv_cusgroup");
//            Object tpv_customId = hashMap.get("tpv_customId");
//            Object tpv_customName = hashMap.get("tpv_customName");
//            Object signingAmt = hashMap.containsKey("signingAmt") ? hashMap.get("signingAmt") : BigDecimal.ZERO;
//            Object shippingAmt = hashMap.containsKey("shippingAmt") ? hashMap.get("shippingAmt") : BigDecimal.ZERO;
//            Object signingUseAmt = hashMap.containsKey("signingUseAmt") ? hashMap.get("signingUseAmt") : BigDecimal.ZERO;
//            Object shippingUseAmt = hashMap.containsKey("shippingUseAmt") ? hashMap.get("shippingUseAmt") : BigDecimal.ZERO;
//            Object signingResidueAmt = hashMap.containsKey("signingResidueAmt") ? hashMap.get("signingResidueAmt") : BigDecimal.ZERO;
//            Object shippingResidueAmt = hashMap.containsKey("shippingResidueAmt") ? hashMap.get("shippingResidueAmt") : BigDecimal.ZERO;
//            tempData[0] = tpv_creditType;
//            tempData[1] = tpv_orgcredit;
//            tempData[2] = orgId;
//            tempData[3] = orgName;
//            tempData[4] = tpv_cusgroup;
//            tempData[5] = tpv_customId;
//            tempData[6] = tpv_customName;
//            tempData[7] = signingAmt;
//            tempData[8] = shippingAmt;
//            tempData[9] = signingUseAmt;
//            tempData[10] = shippingUseAmt;
//            tempData[11] = signingResidueAmt;
//            tempData[12] = shippingResidueAmt;
//            coll.add(tempData);
//        }
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(dataSet1);
        return col_s;
    }

    public static DataSet getCusCreditLimitUses(String tpv_orgcredit_filter, String tpv_credittype_filter, String tpv_cusgroup_f, long tpv_custom_f, long orgId_f) {
        //客户额度维护
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
//        qFilter1.and(new QFilter("tpv_invalids", QCP.equals, false));
//        qFilter1.and(new QFilter("tpv_crm_cuslimit.tpv_frozenstate", QCP.equals, false));
        if (StringUtils.isNotEmpty(tpv_cusgroup_f)) {
            qFilter1.and(new QFilter("tpv_cusgroup", QCP.equals, tpv_cusgroup_f));
        }
        if (0L != orgId_f) {
            qFilter1.and(new QFilter("tpv_organization", QCP.equals, orgId_f));
        }
        if (0L != tpv_custom_f) {
            qFilter1.and(new QFilter("tpv_crm_cuslimit.tpv_custom", QCP.equals, tpv_custom_f));
        }

        // 单独授信
        DataSet queryNoTotal;
        if ("A".equals(tpv_orgcredit_filter)) {// 全司
            queryNoTotal = selectcustomerlimitsEntry(qFilter1.toArray(), new String[]{"tpv_customerId"}, tpv_custom_f, orgId_f);
        } else if ("B".equals(tpv_orgcredit_filter)) {// 部门
            queryNoTotal = selectcustomerlimitsEntry(qFilter1.toArray(), new String[]{"tpv_customerId", "tpv_departmentId"}, tpv_custom_f, orgId_f);
        } else {// 全司+部门
            queryNoTotal = selectcustomerlimitsEntry(new QFilter("tpv_orgcredit", QCP.equals, "A").and(qFilter1).toArray(),
                    new String[]{"tpv_customerId"}, tpv_custom_f, orgId_f).union(
                    selectcustomerlimitsEntry(new QFilter("tpv_orgcredit", QCP.equals, "B").and(qFilter1).toArray(),
                            new String[]{"tpv_customerId", "tpv_departmentId"}, tpv_custom_f, orgId_f));
        }

        //总额度
        DataSet queryIsTotal = selectCustomerlimits(queryNoTotal);
        DataSet union = queryIsTotal.union(queryNoTotal).select(new String[]{"custmainId", "tpv_creditType", "tpv_orgcredit",
                        "tpv_departmentId", "tpv_dep", "tpv_cusgroup", "tpv_customerId", "tpv_customer",
                        " tpv_signingamt", "tpv_shippingamt", "tpv_signinguseamt", "tpv_shippinguseamt",
                        "tpv_signingamt - tpv_signinguseamt tpv_signingresidueamt", "tpv_shippingamt - tpv_shippinguseamt tpv_shippingresidueamt",
                        "tpv_groupconamt", "tpv_groupdelim", "tpv_isfreezetext"})
                .select(new String[]{"custmainId", "tpv_creditType", "tpv_orgcredit",
                        "tpv_departmentId", "tpv_dep", "tpv_cusgroup", "tpv_customerId", "tpv_customer",
                        "(case when tpv_isfreezetext == '' then tpv_signingamt else 0.0 end) tpv_signingamt", "(case when tpv_isfreezetext == '' then tpv_signingamt else 0.0 end) tpv_shippingamt",
                        "(case when tpv_isfreezetext == '' then tpv_signinguseamt else 0.0 end) tpv_signinguseamt", "(case when tpv_isfreezetext == '' then tpv_shippinguseamt else 0.0 end) tpv_shippinguseamt",
                        "(case when tpv_isfreezetext == '' then tpv_signingresidueamt else 0.0 end) tpv_signingresidueamt", "(case when tpv_isfreezetext == '' then tpv_shippingresidueamt else 0.0 end) tpv_shippingresidueamt",
                        "Round(tpv_signingamt,2) + tpv_isfreezetext tpv_signingamts", "Round(tpv_shippingamt,2) + tpv_isfreezetext tpv_shippingamts",
                        "Round(tpv_signinguseamt,2) + tpv_isfreezetext tpv_signinguseamts", "Round(tpv_shippinguseamt,2) + tpv_isfreezetext tpv_shippinguseamts",
                        "Round(tpv_signingresidueamt,2) + tpv_isfreezetext tpv_signingresidueamts", "Round(tpv_shippingresidueamt,2) + tpv_isfreezetext tpv_shippingresidueamts",
                        "tpv_groupconamt", "tpv_groupdelim", "(case when tpv_isfreezetext == '' then '(正常)' else tpv_isfreezetext end) tpv_isfreezetext"});
        DataSet orderBy = union.orderBy(new String[]{"tpv_cusgroup", "custmainId"});
        if (YEM.isNotEmpty(tpv_credittype_filter)) {
            orderBy = orderBy.where(String.format("tpv_credittype = '%s'", tpv_credittype_filter));// 授信类型
        }
        return orderBy;
    }

    private static DataSet selectCustomerlimits(DataSet queryNoTotal) {
        DataSet customerlimits = queryNoTotal.groupBy(new String[]{"custmainId", "'A' tpv_creditType", "tpv_orgcredit",
                        "tpv_departmentId", "tpv_dep", "tpv_cusgroup", "tpv_invalids", "tpv_frozenstate", "null tpv_customerId", "null tpv_customer",
                        "tpv_groupconamt tpv_signingamt", "tpv_groupdelim tpv_shippingamt", "tpv_groupconamt", "tpv_groupdelim"})
                .sum("tpv_signinguseamt").sum("tpv_shippinguseamt")
                .finish();
        customerlimits = customerlimits.select(new String[]{"custmainId", "tpv_creditType", "tpv_orgcredit",
                "tpv_departmentId", "tpv_dep", "tpv_cusgroup", "tpv_invalids", "tpv_frozenstate", "(case when tpv_invalids == false then '' else '(已失效)' end) tpv_isfreezetext", "tpv_customerId", "tpv_customer",
                "tpv_signingamt", "tpv_shippingamt", "tpv_signinguseamt", "tpv_shippinguseamt",
                "tpv_signingamt - tpv_signinguseamt tpv_signingresidueamt", "tpv_shippingamt - tpv_shippinguseamt tpv_shippingresidueamt",
                "tpv_groupconamt", "tpv_groupdelim",});
        return customerlimits;
    }

    private static DataSet selectcustomerlimitsEntry(QFilter[] qFilters, String[] keys, long tpv_custom_f, long orgId_f) {
        DataSet customerlimitsEntry = QueryServiceHelper.queryDataSet(algoKey, "tpv_crm_custmain",
                        "id custmainId,tpv_orgcredit,tpv_organization.id tpv_departmentId,tpv_organization.name tpv_dep,tpv_cusgroup,tpv_groupconamt  * 10000 tpv_groupconamt,tpv_groupdelim  * 10000 tpv_groupdelim," +
                                "tpv_crm_cuslimit.tpv_custom.id tpv_customerId,tpv_crm_cuslimit.tpv_custom.name tpv_customName," +
                                "tpv_crm_cuslimit.tpv_quotatype tpv_quotatype,tpv_crm_cuslimit.tpv_limitamt tpv_limitamt,tpv_invalids,tpv_crm_cuslimit.tpv_frozenstate tpv_frozenstate",
                        qFilters, null)
                .groupBy(new String[]{"custmainId", "tpv_orgcredit", "tpv_departmentId", "tpv_dep", "tpv_cusgroup", "tpv_customerId", "tpv_customName", "tpv_quotatype", "tpv_groupconamt", "tpv_groupdelim", "tpv_invalids", "tpv_frozenstate"})
                .sum("tpv_limitamt")
                .finish()
                .select("custmainId", "'B' tpv_creditType", "tpv_orgcredit", "tpv_departmentId", "tpv_dep", "tpv_cusgroup", "tpv_customerId", "tpv_customName tpv_customer", "tpv_groupconamt", "tpv_groupdelim",
                        "(case when tpv_quotatype == 'A' then tpv_limitamt else 0.0 end) tpv_signingamt",
                        "(case when tpv_quotatype == 'B' then tpv_limitamt else 0.0 end) tpv_shippingamt", "tpv_invalids", "tpv_frozenstate")
                .groupBy(new String[]{"custmainId", "tpv_creditType", "tpv_orgcredit", "tpv_departmentId", "tpv_dep", "tpv_cusgroup", "tpv_customerId", "tpv_customer", "tpv_groupconamt", "tpv_groupdelim",
                        "tpv_invalids", "tpv_frozenstate"})
                .sum("tpv_signingamt").sum("tpv_shippingamt").finish().orderBy(new String[]{"tpv_cusgroup"});

        DataSet query = getCusCreditLimits(0L, 0L, 0L, true);
        GroupbyDataSet groupbyDataSet;
        if (keys == null) {
            groupbyDataSet = query.groupBy();
        } else {
            groupbyDataSet = query.groupBy(keys);
        }
        DataSet finish = groupbyDataSet.sum("tpv_uncontractamt").sum("tpv_unshippingamt")
                .finish(); // 查询供应商预付款占用-合同明细表

        JoinDataSet leftJoin = customerlimitsEntry.leftJoin(finish);
        for (String key : keys) {
            leftJoin.on(key, key);
        }
        DataSet select = leftJoin.select(new String[]{"custmainId", "tpv_creditType", "tpv_orgcredit", "tpv_departmentId", "tpv_dep",
                "tpv_cusgroup", "tpv_customerId", "tpv_customer", "tpv_groupconamt", "tpv_groupdelim", "tpv_signingamt", "tpv_shippingamt",
                "(case when tpv_uncontractamt != null then tpv_uncontractamt else 0.0 end) tpv_signinguseamt",
                "(case when tpv_unshippingamt != null then tpv_unshippingamt else 0.0 end) tpv_shippinguseamt", "tpv_invalids", "tpv_frozenstate"}).finish();
        select = select.select(new String[]{"custmainId", "tpv_creditType", "tpv_orgcredit", "tpv_departmentId", "tpv_dep",
                "tpv_cusgroup", "tpv_invalids", "tpv_frozenstate", "(case when tpv_frozenstate == 'A' then '(已冻结)' else '' end) tpv_isfreezetext",
                "tpv_customerId", "tpv_customer", "tpv_signingamt * 10000 tpv_signingamt", "tpv_shippingamt * 10000 tpv_shippingamt",
                "tpv_signinguseamt", "tpv_shippinguseamt", "tpv_signingamt * 10000 - tpv_signinguseamt tpv_signingresidueamt",
                "tpv_shippingamt * 10000 - tpv_shippinguseamt tpv_shippingresidueamt", "tpv_groupconamt", "tpv_groupdelim"});
        return select;
    }

    /**
     * @Description: //可放货余额表 -非配供业务
     * @param: depId - 部门
     * @param: customerId - 客户
     * @param: contractId - 销售合同号
     * @return: DynamicObjectCollection
     **/
    public static DynamicObjectCollection getDeliverableBalanceCon(long depId, long customerId, String contractNo) {
        //销售合同
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
        if (StringUtils.isNotEmpty(contractNo)) {
            qFilter1.and(new QFilter("tpv_contractnos", QCP.ILIKE, "%" + contractNo + "%"));
        }
        if (0L != depId) {
            qFilter1.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter1.and(new QFilter("tpv_customer.id", QCP.equals, customerId));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salescontract",
                "id contractId,org.id orgId,org.name orgName,tpv_bizdate," +
                        "tpv_documenttypes.id tpv_documenttypesId,tpv_documenttypes.name tpv_documenttypesName,tpv_documenttypes.number tpv_documenttypesNumber," +
                        "tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
                        "tpv_customer.id tpv_customerId,tpv_customer.name tpv_customerName,tpv_projectabb,tpv_contractnos contractNo",
                new QFilter[]{qFilter1}, null);
        List mainIds_s = new ArrayList();
        DataSet dataSet1_1 = dataSet1.copy();
        for (Row row : dataSet1_1) {
            mainIds_s.add(row.getLong("contractId"));
        }
        String con = "Con";
        DataSet lefted = leftJoinUtil(depId, customerId, contractNo, dataSet1, mainIds_s, con).orderBy(new String[]{"tpv_bizdate desc"});
//        DataSet dataSet = ORM.create().queryDataSet("", "", null);
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(lefted);
        return col_s;
    }

    public static DataSet getDeliverableBalanceConData(long depId, long customerId, String contractNo) {
        //销售合同
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
        if (StringUtils.isNotEmpty(contractNo)) {
            qFilter1.and(new QFilter("tpv_contractnos", QCP.like, "%" + contractNo + "%"));
        }
        if (0L != depId) {
            qFilter1.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter1.and(new QFilter("tpv_customer.id", QCP.equals, customerId));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salescontract",
                "id contractId,org.id orgId,org.name orgName,tpv_bizdate," +
                        "tpv_documenttypes.id tpv_documenttypesId,tpv_documenttypes.name tpv_documenttypesName,tpv_documenttypes.number tpv_documenttypesNumber," +
                        "tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
                        "tpv_customer.id tpv_customerId,tpv_customer.name tpv_customerName,tpv_projectabb,tpv_contractnos contractNo",
                new QFilter[]{qFilter1}, null);
        List mainIds_s = new ArrayList();
        DataSet dataSet1_1 = dataSet1.copy();
        for (Row row : dataSet1_1) {
            mainIds_s.add(row.getLong("contractId"));
        }
        String con = "Con";
        DataSet lefted = leftJoinUtil(depId, customerId, contractNo, dataSet1, mainIds_s, con);
        lefted = lefted.select("contractId", "tpv_bizdate", "orgId tpv_orgid", "orgName tpv_org", "tpv_departmentId tpv_departmentid", "tpv_departmentName tpv_dep",
                "tpv_customerId tpv_customerid", "tpv_customerName tpv_customer", "contractNo tpv_contractno", "shippingAmt tpv_amt2",
                "amtSum1 tpv_amt5", "e_actamt3 tpv_amt12 ", "amtSum2 tpv_amt14", "unAmtSum1 tpv_amt15", "e_actamt8 tpv_amt23",
                "e_actamt8 tpv_amt25", "tpv_amt19", "amtSum4 tpv_amt27").orderBy(new String[]{"tpv_bizdate desc"});
        return lefted;
    }

    public static DataSet getDeliverableBalanceConData(long depId, long customerId, String contractNo, Boolean flag) {
        //销售合同
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
        if (StringUtils.isNotEmpty(contractNo)) {
            qFilter1.and(new QFilter("tpv_contractnos", QCP.like, "%" + contractNo + "%"));
        }
        if (0L != depId) {
            qFilter1.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter1.and(new QFilter("tpv_customer.id", QCP.equals, customerId));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salescontract",
                "id contractId,org.id orgId,org.name orgName,tpv_bizdate," +
                        "tpv_documenttypes.id tpv_documenttypesId,tpv_documenttypes.name tpv_documenttypesName,tpv_documenttypes.number tpv_documenttypesNumber," +
                        "tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
                        "tpv_customer.id tpv_customerId,tpv_customer.name tpv_customerName,tpv_projectabb,tpv_contractnos contractNo",
                new QFilter[]{qFilter1}, null);
        List mainIds_s = new ArrayList();
        DataSet dataSet1_1 = dataSet1.copy();
        for (Row row : dataSet1_1) {
            mainIds_s.add(row.getLong("contractId"));
        }
        String con = "Con";
        //2发货总金额
        DataSet leftshippingAmt = leftshippingAmt(depId, customerId, contractNo, dataSet1, mainIds_s, con);
        //============================================================================================
        //3供应链金融（暂无）
        //============================================================================================
        //4已收货款【总计金额】
        DataSet leftReceiveAmt = leftReceiveAmt(depId, customerId, contractNo, leftshippingAmt, mainIds_s, con);
        DataSet dataSet = leftReceiveAmt.select("contractId", "tpv_bizdate", "orgId tpv_orgid", "orgName tpv_org", "tpv_departmentId tpv_departmentid", "tpv_departmentName tpv_dep",
                "tpv_customerId tpv_customerid", "tpv_customerName tpv_customer", "contractNo tpv_contractno", "shippingAmt tpv_amt2",
                "amtSum1 tpv_amt5","e_actamt3 tpv_amt12","amtSum2 tpv_amt14").orderBy(new String[]{"tpv_bizdate desc"});
        return dataSet;
    }

    /**
     * @Description: //可放货余额表 -项目配供业务
     * @param: depId - 部门
     * @param: customerId - 客户
     * @param: project - 项目
     * @return: DynamicObjectCollection
     **/
    public static DynamicObjectCollection getDeliverableBalanceXM(long depId, long customerId, String project) {
        //建档管理
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
        qFilter1.and(new QFilter("tpv_framename", QCP.equals, ""));  //框架简称
        qFilter1.and(new QFilter("tpv_project", QCP.not_equals, ""));  //项目简称
        //qFilter1.and(new QFilter("tpv_filetypes", QCP.equals, "")); //建档类型 -项目建档 2 标识tpv_item
        if (StringUtils.isNotEmpty(project)) {
            qFilter1.and(new QFilter("tpv_project", QCP.like, "%" + project + "%"));
        }
        if (0L != depId) {
            qFilter1.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter1.and(new QFilter("tpv_custom.id", QCP.equals, customerId));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_pw_filing",
                "id contractId, tpv_bizdate,org.id orgId,org.name orgName,tpv_project tpv_projectabb," +
                        "tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
                        "tpv_custom.id tpv_customerId,tpv_custom.name tpv_customerName",
                new QFilter[]{qFilter1}, null);
        List mainIds_s = new ArrayList();
        DataSet dataSet1_1 = dataSet1.copy();
        for (Row row : dataSet1_1) {
            mainIds_s.add(row.getLong("contractId"));
        }
        String xm = "XM";
        DataSet lefted = leftJoinUtil(depId, customerId, "", dataSet1, mainIds_s, xm).orderBy(new String[]{"tpv_bizdate desc"});
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(lefted);
        return col_s;
    }

    /**
     * @Description: //可放货余额表 -框架配供业务
     * @param: depId - 部门
     * @param: customerId - 客户
     * @param: project - 项目
     * @return: DynamicObjectCollection
     **/
    public static DynamicObjectCollection getDeliverableBalanceKJ(long depId, long customerId, String project) {
        //建档管理
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
        qFilter1.and(new QFilter("tpv_frameworkname", QCP.not_equals, ""));  //框架简称
        //qFilter1.and(new QFilter("tpv_filetypes", QCP.equals, "")); //建档类型 -框架建档 1 标识tpv_frame
        if (StringUtils.isNotEmpty(project)) {
            qFilter1.and(new QFilter("tpv_frameworkname", QCP.like, "%" + project + "%"));
        }
        if (0L != depId) {
            qFilter1.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter1.and(new QFilter("tpv_custom.id", QCP.equals, customerId));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_pw_filing",
                "id contractId, tpv_bizdate,org.id orgId,org.name orgName,tpv_frameworkname tpv_projectabb," +
                        "tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
                        "tpv_custom.id tpv_customerId,tpv_custom.name tpv_customerName",
                new QFilter[]{qFilter1}, null);
//        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_pw_filing",
//                "id contractId,org.id orgId,org.name orgName,tpv_framename,tpv_frameworkname," +
//                        "tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
//                        "tpv_custom.id tpv_customerId,tpv_custom.name tpv_customerName",
//                new QFilter[]{qFilter1}, null).where("tpv_framename != '' or tpv_frameworkname != ''");
//        dataSet1 = dataSet1.select("contractId", "orgId", "orgName", "(case when tpv_framename != '' then tpv_framename else tpv_frameworkname end) tpv_projectabb",
//                "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName");
        List mainIds_s = new ArrayList();
        DataSet dataSet1_1 = dataSet1.copy();
        for (Row row : dataSet1_1) {
            mainIds_s.add(row.getLong("contractId"));
        }
        String kj = "KJ";
        DataSet lefted = leftJoinUtil(depId, customerId, project, dataSet1, mainIds_s, kj).orderBy(new String[]{"tpv_bizdate desc"});
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(lefted);
        return col_s;
    }

    //根据客户、公司、部门、项目号/销售合同号 ，进行分组汇总
    public static DataSet getDeliverableBalanceCons(long depId, long customerId, String contractNo) {
        //销售合同
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
        if (StringUtils.isNotEmpty(contractNo)) {
            qFilter1.and(new QFilter("tpv_contractnos", QCP.like, "%" + contractNo + "%"));
        }
        if (0L != depId) {
            qFilter1.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter1.and(new QFilter("tpv_customer.id", QCP.equals, customerId));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salescontract",
                "id contractId,org.id orgId,org.name orgName,tpv_bizdate," +
                        "tpv_documenttypes.id tpv_documenttypesId,tpv_documenttypes.name tpv_documenttypesName,tpv_documenttypes.number tpv_documenttypesNumber," +
                        "tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
                        "tpv_customer.id tpv_customerId,tpv_customer.name tpv_customerName,tpv_projectabb,tpv_contractnos contractNo",
                new QFilter[]{qFilter1}, null);
        List mainIds_s = new ArrayList();
        DataSet dataSet1_1 = dataSet1.copy();
        for (Row row : dataSet1_1) {
            mainIds_s.add(row.getLong("contractId"));
        }
        String con = "Con";
        DataSet lefted = leftJoinUtil(depId, customerId, contractNo, dataSet1, mainIds_s, con).orderBy(new String[]{"tpv_bizdate desc"});
        lefted = lefted.groupBy(new String[]{"contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                        "tpv_customerId", "tpv_customerName", "contractNo", "tpv_amt19"})
                .sum("shippingAmt").sum("amtSum1").sum("e_actamt3").sum("amtSum2").sum("unAmtSum1").sum("e_actamt8")
                .sum("amtSum3").sum("amtSum4").finish();
        lefted = lefted.select("contractId", "tpv_bizdate", "orgId tpv_orgid", "orgName tpv_org", "tpv_departmentId tpv_departmentid", "tpv_departmentName tpv_dep",
                "tpv_customerId tpv_customerid", "tpv_customerName tpv_customer", "contractNo tpv_contractno", "shippingAmt tpv_amt2",
                "amtSum1 tpv_amt5", "e_actamt3 tpv_amt12 ", "amtSum2 tpv_amt14", "unAmtSum1 tpv_amt15", "e_actamt8 tpv_amt23",
                "e_actamt8 tpv_amt25", "tpv_amt19", "amtSum4 tpv_amt27").orderBy(new String[]{"tpv_bizdate desc"});
        return lefted;
    }

    public static DataSet getDeliverableBalanceXMs(long depId, long customerId, String project) {
        //建档管理
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
        qFilter1.and(new QFilter("tpv_framename", QCP.equals, ""));  //框架简称
        qFilter1.and(new QFilter("tpv_project", QCP.not_equals, ""));  //项目简称
        //qFilter1.and(new QFilter("tpv_filetypes", QCP.equals, "")); //建档类型 -项目建档 2 标识tpv_item
        if (StringUtils.isNotEmpty(project)) {
            qFilter1.and(new QFilter("tpv_project", QCP.like, "%" + project + "%"));
        }
        if (0L != depId) {
            qFilter1.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter1.and(new QFilter("tpv_custom.id", QCP.equals, customerId));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_pw_filing",
                "id contractId, tpv_bizdate,org.id orgId,org.name orgName,tpv_project tpv_projectabb," +
                        "tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
                        "tpv_custom.id tpv_customerId,tpv_custom.name tpv_customerName",
                new QFilter[]{qFilter1}, null);
        List mainIds_s = new ArrayList();
        DataSet dataSet1_1 = dataSet1.copy();
        for (Row row : dataSet1_1) {
            mainIds_s.add(row.getLong("contractId"));
        }
        String xm = "XM";
        DataSet lefted = leftJoinUtil(depId, customerId, project, dataSet1, mainIds_s, xm).orderBy(new String[]{"tpv_bizdate desc"});
        lefted = lefted.groupBy(new String[]{"contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                        "tpv_customerId", "tpv_customerName", "tpv_projectabb"})
                .sum("shippingAmt").sum("amtSum1").sum("e_actamt3").sum("amtSum2").sum("unAmtSum1").sum("e_actamt8")
                .sum("amtSum3").sum("tpv_amt19").sum("amtSum4").finish();
        lefted = lefted.select("contractId", "tpv_bizdate", "orgId tpv_orgid", "orgName tpv_org", "tpv_departmentId tpv_departmentid", "tpv_departmentName tpv_dep",
                "tpv_customerId tpv_customerid", "tpv_customerName tpv_customer", "tpv_projectabb", "shippingAmt tpv_amt2",
                "amtSum1 tpv_amt5", "e_actamt3 tpv_amt12 ", "amtSum2 tpv_amt14", "unAmtSum1 tpv_amt15", "e_actamt8 tpv_amt23",
                "e_actamt8 tpv_amt25", "tpv_amt19", "amtSum4 tpv_amt27").orderBy(new String[]{"tpv_bizdate desc"});
        return lefted;
    }

    public static DataSet getDeliverableBalanceKJs(long depId, long customerId, String project) {
        //建档管理
        QFilter qFilter1 = new QFilter("billstatus", QCP.equals, "C");
        qFilter1.and(new QFilter("tpv_frameworkname", QCP.not_equals, ""));  //框架简称
        //qFilter1.and(new QFilter("tpv_filetypes", QCP.equals, "")); //建档类型 -框架建档 1 标识tpv_frame
        if (StringUtils.isNotEmpty(project)) {
            qFilter1.and(new QFilter("tpv_frameworkname", QCP.like, "%" + project + "%"));
        }
        if (0L != depId) {
            qFilter1.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter1.and(new QFilter("tpv_custom.id", QCP.equals, customerId));
        }
        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_pw_filing",
                "id contractId, tpv_bizdate,org.id orgId,org.name orgName,tpv_frameworkname tpv_projectabb," +
                        "tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
                        "tpv_custom.id tpv_customerId,tpv_custom.name tpv_customerName",
                new QFilter[]{qFilter1}, null);
//        DataSet dataSet1 = QueryServiceHelper.queryDataSet(algoKey, "tpv_pw_filing",
//                "id contractId,org.id orgId,org.name orgName,tpv_framename,tpv_frameworkname," +
//                        "tpv_department.id tpv_departmentId,tpv_department.name tpv_departmentName," +
//                        "tpv_custom.id tpv_customerId,tpv_custom.name tpv_customerName",
//                new QFilter[]{qFilter1}, null).where("tpv_framename != '' or tpv_frameworkname != ''");
//        dataSet1 = dataSet1.select("contractId", "orgId", "orgName", "(case when tpv_framename != '' then tpv_framename else tpv_frameworkname end) tpv_projectabb",
//                "tpv_departmentId", "tpv_departmentName", "tpv_customerId", "tpv_customerName");
        List mainIds_s = new ArrayList();
        DataSet dataSet1_1 = dataSet1.copy();
        for (Row row : dataSet1_1) {
            mainIds_s.add(row.getLong("contractId"));
        }
        String kj = "KJ";
        DataSet lefted = leftJoinUtil(depId, customerId, project, dataSet1, mainIds_s, kj).orderBy(new String[]{"tpv_bizdate desc"});
        lefted = lefted.groupBy(new String[]{"contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                        "tpv_customerId", "tpv_customerName", "tpv_projectabb"})
                .sum("shippingAmt").sum("amtSum1").sum("e_actamt3").sum("amtSum2").sum("unAmtSum1").sum("e_actamt8")
                .sum("amtSum3").sum("tpv_amt19").sum("amtSum4").finish();
        lefted = lefted.select("contractId", "tpv_bizdate", "orgId tpv_orgid", "orgName tpv_org", "tpv_departmentId tpv_departmentid", "tpv_departmentName tpv_dep",
                "tpv_customerId tpv_customerid", "tpv_customerName tpv_customer", "tpv_projectabb", "shippingAmt tpv_amt2",
                "amtSum1 tpv_amt5", "e_actamt3 tpv_amt12 ", "amtSum2 tpv_amt14", "unAmtSum1 tpv_amt15", "e_actamt8 tpv_amt23",
                "e_actamt8 tpv_amt25", "tpv_amt19", "amtSum4 tpv_amt27").orderBy(new String[]{"tpv_bizdate desc"});
        return lefted;
    }

    /**
     * @Description: //根据可放余额表主维度 -左连接查询
     * @param: depId - 部门
     * @param: customerId - 客户
     * @param: mainBillId - 主维度单据ID
     * @param: dataSet1 - 主维度数据
     * @param: mainIds_s - 主维度ID集合
     * @param: mainBill - 查询纬度
     * @return: kd.bos.algo.DataSet
     **/
    private static DataSet leftJoinUtil(Long depId, Long customerId, String contractNo,
                                        DataSet dataSet1, List mainIds_s, String mainBill) {
        /**
         * 联查，拼接字段，自增方法方式
         * RptUtil.getDataSetFiledAlias(queryDataSet2),new String[] {"tpv_ckyx","tpv_cktradeproperty"})
         * */

        //2发货总金额
        DataSet leftshippingAmt = leftshippingAmt(depId, customerId, contractNo, dataSet1, mainIds_s, mainBill);
        //============================================================================================
        //3供应链金融（暂无）
        //============================================================================================
        //4已收货款【总计金额】
        DataSet leftReceiveAmt = leftReceiveAmt(depId, customerId, contractNo, leftshippingAmt, mainIds_s, mainBill);
        //============================================================================================
        //5货物余额
        //============================================================================================
        //6未到期商业承兑汇票（暂无）
        //============================================================================================
        //7含商业承兑汇票货款余额（暂无）
        //============================================================================================
        //8财务货款余额（暂无）
        //============================================================================================
        //9已申请额度 ——赊销/项目/框架
        DataSet leftApplyLimited = leftApplyLimited(depId, customerId, contractNo, leftReceiveAmt, mainIds_s, mainBill);
        //============================================================================================
        //10额度余额
        //============================================================================================
        //11已收保证金
        DataSet leftReceiptBond = leftReceiptBond(depId, customerId, contractNo, leftApplyLimited, mainIds_s, mainBill);
        //============================================================================================
        //12总计金额
        //============================================================================================
        DataSet dataSet = leftReceiptBond;
        return dataSet;
    }

    //11已收保证金
    private static DataSet leftReceiptBond(Long depId, Long customerId, String contractNo, DataSet leftApplyLimited, List mainIds_s, String mainBill) {

        //11.4质押货物金额
        QFilter qFilter16 = new QFilter("billstatus", QCP.equals, "C");
        qFilter16.and(new QFilter("tpv_stakepurpose", QCP.equals, "B"));

        //11.5退款总金额
        //11.6【总计金额】
        //dataSet19 = dataSet19.addField("e_actamt8", "amtSum3");
        DataSet dataSet19;
        if ("XM".equals(mainBill) || "KJ".equals(mainBill)) {
            DataSet leftApplyLimited_1 = leftApplyLimited.copy();
            List contractNoIdList = new ArrayList();
            for (Row row : leftApplyLimited_1) {
                contractNoIdList.add(row.getLong("contractNoId"));
            }
            if (contractNoIdList.size() > 0) {
                qFilter16.and(new QFilter("tpv_salecontractno.id", QCP.in, contractNoIdList));
            }
            DataSet dataSet16 = QueryServiceHelper.queryDataSet(algoKey, "tpv_pledgereceipt",
                    "id,tpv_salecontractno.id contractNoId,tpv_amount e_actamt1",
                    new QFilter[]{qFilter16}, null);
            //11.4.1货物解押单
            QFilter qFilter17 = new QFilter("billstatus", QCP.equals, "C");
            DataSet dataSet17 = QueryServiceHelper.queryDataSet(algoKey, "tpv_cargodecompression",
                    "tpv_sourcebillid id,tpv_amount e_actamt2",
                    new QFilter[]{qFilter17}, null);

            DataSet dataSet18 = dataSet16.leftJoin(dataSet17).on("id", "id")
                    .select("contractNoId", "e_actamt1-e_actamt2 e_actamt8").finish();
            dataSet19 = leftApplyLimited.leftJoin(dataSet18).on("contractNoId", "contractNoId")
                    .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                            "tpv_customerId", "tpv_customerName", "tpv_projectabb", "shippingAmt",
                            "amtSum1", "e_actamt3", "amtSum2", "unAmtSum1", "e_actamt8",
                            "e_actamt8 amtSum3", "tpv_amt19").finish()
                    .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                            "tpv_customerId", "tpv_customerName", "tpv_projectabb", "shippingAmt",
                            "amtSum1", "e_actamt3", "amtSum2", "unAmtSum1", "e_actamt8",
                            "amtSum3", "tpv_amt19", "unAmtSum1 + amtSum3 amtSum4");
        } else {
            if (mainIds_s.size() > 0) {
                qFilter16.and(new QFilter("tpv_sourcebillid", QCP.in, mainIds_s));
            }
            DataSet dataSet16 = QueryServiceHelper.queryDataSet(algoKey, "tpv_pledgereceipt",
                    "id,tpv_sourcebillid contractId,tpv_amount e_actamt1",
                    new QFilter[]{qFilter16}, null);
            //11.4.1货物解押单
            QFilter qFilter17 = new QFilter("billstatus", QCP.equals, "C");
            DataSet dataSet17 = QueryServiceHelper.queryDataSet(algoKey, "tpv_cargodecompression",
                    "tpv_sourcebillid id,tpv_amount e_actamt2",
                    new QFilter[]{qFilter17}, null);

            DataSet dataSet18 = dataSet16.leftJoin(dataSet17).on("id", "id")
                    .select("contractId", "e_actamt1-e_actamt2 e_actamt8").finish()
                    .groupBy(new String[]{"contractId"})
                    .sum("e_actamt8")
                    .finish();
            dataSet19 = leftApplyLimited.leftJoin(dataSet18).on("contractId", "contractId")
                    .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                            "tpv_customerId", "tpv_customerName", "contractNo", "shippingAmt",
                            "amtSum1", "e_actamt3", "amtSum2", "unAmtSum1", "e_actamt8",
                            "e_actamt8 amtSum3", "tpv_amt19").finish()
                    .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                            "tpv_customerId", "tpv_customerName", "contractNo", "shippingAmt",
                            "amtSum1", "e_actamt3", "amtSum2", "unAmtSum1", "e_actamt8",
                            "amtSum3", "tpv_amt19", "unAmtSum1 + amtSum3 amtSum4");
        }
        return dataSet19;
    }

    //9已申请额度 ——赊销/项目/框架
    private static DataSet leftApplyLimited(Long depId, Long customerId, String contractNo, DataSet leftReceiveAmt, List mainIds_s, String mainBill) {
        QFilter qFilter9 = new QFilter("billstatus", QCP.equals, "C");
        DataSet dataSet10 = null;
        if ("XM".equals(mainBill) || "KJ".equals(mainBill)) {
            if ("XM".equals(mainBill)) {
                //过滤单据状态 项目
                if (mainIds_s.size() > 0) {
                    qFilter9.and(new QFilter("tpv_sourcebillid", QCP.in, mainIds_s));
                }
            } else if ("KJ".equals(mainBill)) {
                //过滤单据状态 框架
                if (mainIds_s.size() > 0) {
                    qFilter9.and(new QFilter("tpv_sourcebillid", QCP.in, mainIds_s));
                }
            }
            DataSet tpv_pw_administrative = QueryServiceHelper.queryDataSet(algoKey, "tpv_pw_administrative",
                            "id saleIdb,tpv_sourcebillid contractId,tpv_endtime,tpv_projectmoney, (case when tpv_endtime >= now() then tpv_temporarylift else 0.0 end) tpv_temporarylift",
                            new QFilter[]{qFilter9}, null)
                    .select("saleIdb", "contractId", "tpv_projectmoney + tpv_temporarylift projectmoney");
            dataSet10 = leftReceiveAmt.leftJoin(tpv_pw_administrative).on("contractId", "contractId")
                    .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                            "tpv_customerId", "tpv_customerName", "tpv_projectabb", "contractNoId", "shippingAmt",
                            "amtSum1", "e_actamt3", "amtSum2", "unAmtSum1", "projectmoney tpv_amt19")
                    .finish();
        } else {
            //DataSet createDataSet = cusCreditData(); //客户赊销额度使用情况表
            DataSet cusCreditLimitUses = getCusCreditLimitUses("", "", "", customerId, depId)
                    .groupBy(new String[]{"tpv_customerId", "tpv_departmentId"})
                    .sum("tpv_signingresidueamt").sum("tpv_shippingresidueamt")
                    .finish();
            DataSet finish = leftReceiveAmt.leftJoin(cusCreditLimitUses).on("tpv_customerId", "tpv_customerId").on("tpv_departmentId", "tpv_departmentId")
                    .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                            "tpv_customerId", "tpv_customerName", "contractNo", "shippingAmt",
                            "amtSum1", "e_actamt3", "amtSum2", "unAmtSum1", "tpv_signingresidueamt - tpv_shippingresidueamt cusCreditamt")
                    .finish();
            dataSet10 = finish.select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                    "tpv_customerId", "tpv_customerName", "contractNo", "shippingAmt",
                    "amtSum1", "e_actamt3", "amtSum2", "unAmtSum1", "(case when cusCreditamt > 0 then 'A' else '' end) tpv_amt19");
        }
        return dataSet10;
    }

    //4已收货款【总计金额】
    private static DataSet leftReceiveAmt(Long depId, Long customerId, String contractNo, DataSet leftshippingAmt, List mainIds_s, String mainBill) {

        //4.5质押货物金额
        QFilter qFilter6 = new QFilter("billstatus", QCP.equals, "C");
        qFilter6.and(new QFilter("tpv_stakepurpose", QCP.equals, "A"));

        DataSet dataSet11_2;
        if ("XM".equals(mainBill) || "KJ".equals(mainBill)) {
            DataSet leftshippingAmt_1 = leftshippingAmt.copy();
            List contractNoIdList = new ArrayList();
            for (Row row : leftshippingAmt_1) {
                contractNoIdList.add(row.getLong("contractNoId"));
            }
            if (contractNoIdList.size() > 0) {
                qFilter6.and(new QFilter("tpv_salecontractno.id", QCP.in, contractNoIdList));
            }

            DataSet dataSet6 = QueryServiceHelper.queryDataSet(algoKey, "tpv_pledgereceipt",
                    "id saleIda,tpv_salecontractno.id contractNoId,tpv_amount e_actamt1",
                    new QFilter[]{qFilter6}, null);
            //4.5.1货物解押单
            DataSet dataSet6_1 = dataSet6.copy();
            List<Long> pledgerList = new ArrayList();
            for (Row row : dataSet6_1) {
                pledgerList.add(row.getLong("saleIda"));
            }
            QFilter qFilter10 = new QFilter("billstatus", QCP.equals, "C");
            qFilter10.and(new QFilter("tpv_sourcebillid", QCP.in, pledgerList));
            DataSet dataSet10 = QueryServiceHelper.queryDataSet(algoKey, "tpv_cargodecompression",
                    "tpv_sourcebillid saleIdb,tpv_amount e_actamt2",
                    new QFilter[]{qFilter10}, null);

            DataSet dataSet11 = dataSet6.leftJoin(dataSet10).on("saleIda", "saleIdb")
                    .select("saleIda", "contractNoId", "e_actamt1-e_actamt2 e_actamt3").finish()
                    .groupBy(new String[]{"saleIda", "contractNoId"})
                    .sum("e_actamt3")
                    .finish();

            DataSet dataSet11_1 = dataSet11.select("contractNoId", "e_actamt3", "e_actamt3 amtSum2");

            dataSet11_2 = leftshippingAmt.leftJoin(dataSet11_1).on("contractNoId", "contractNoId")
                    .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                            "tpv_customerId", "tpv_customerName", "tpv_projectabb", "contractNoId", "shippingAmt",
                            "amtSum1", "e_actamt3", "amtSum2", "amtSum2 - amtSum1 unAmtSum1").finish();
        } else {
            if (mainIds_s.size() > 0) {
                qFilter6.and(new QFilter("tpv_sourcebillid", QCP.in, mainIds_s));
            }
            DataSet dataSet6 = QueryServiceHelper.queryDataSet(algoKey, "tpv_pledgereceipt",
                    "id saleIda,tpv_sourcebillid contractId,tpv_amount e_actamt1",
                    new QFilter[]{qFilter6}, null);
            //4.5.1货物解押单
            DataSet dataSet6_1 = dataSet6.copy();
            List<Long> pledgerList = new ArrayList();
            for (Row row : dataSet6_1) {
                pledgerList.add(row.getLong("saleIda"));
            }
            QFilter qFilter10 = new QFilter("billstatus", QCP.equals, "C");
            qFilter10.and(new QFilter("tpv_sourcebillid", QCP.in, pledgerList));
            DataSet dataSet10 = QueryServiceHelper.queryDataSet(algoKey, "tpv_cargodecompression",
                    "tpv_sourcebillid saleIdb,tpv_amount e_actamt2",
                    new QFilter[]{qFilter10}, null);

            DataSet dataSet11 = dataSet6.leftJoin(dataSet10).on("saleIda", "saleIdb")
                    .select("contractId", "e_actamt1-e_actamt2 e_actamt3").finish()
                    .groupBy(new String[]{"contractId"})
                    .sum("e_actamt3")
                    .finish();

            DataSet dataSet11_1 = dataSet11.select("contractId", "e_actamt3", "e_actamt3 amtSum2");
            dataSet11_2 = leftshippingAmt.leftJoin(dataSet11_1).on("contractId", "contractId")
                    .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                            "tpv_customerId", "tpv_customerName", "contractNo", "shippingAmt",
                            "amtSum1", "e_actamt3", "amtSum2", "amtSum2 - amtSum1 unAmtSum1").finish();
        }
        return dataSet11_2;
    }

    //2发货总金额
    private static DataSet leftshippingAmt(Long depId, Long customerId, String contractNo, DataSet dataSet1, List mainIds_s, String mainBill) {
        //============================================================================================
        //2.1初始化（暂无）
        //2.2发货总金额
        QFilter qFilter2 = new QFilter("billstatus", QCP.equals, "C");
        if (0L != depId) {
            qFilter2.and(new QFilter("tpv_department.id", QCP.equals, depId));
        }
        if (0L != customerId) {
            qFilter2.and(new QFilter("tpv_customer.id", QCP.equals, customerId));
        }

        DataSet dataSet2;
        DataSet dataSet3;
        if ("XM".equals(mainBill) || "KJ".equals(mainBill)) {
            QFilter qFilter3 = new QFilter("billstatus", QCP.equals, "C");
            if ("XM".equals(mainBill)) {
//                if (StringUtils.isNotEmpty(contractNo)) {  //项目简称
//                    qFilter3.and(new QFilter("tpv_project", QCP.like, "%" + contractNo + "%"));
//                }
                if (mainIds_s.size() > 0) {  //销售合同id【销售合同号】 或者 建档管理id【项目简称】
                    qFilter3.and(new QFilter("tpv_projectabrefs.id", QCP.in, mainIds_s)); //项目简称
                }
            }
            if ("KJ".equals(mainBill)) {
//                if (StringUtils.isNotEmpty(contractNo)) {  //框架简称
//                    qFilter3.and(new QFilter("tpv_frameworkname", QCP.like, "%" + contractNo + "%"));
//                }
                if (mainIds_s.size() > 0) {  //销售合同id【销售合同号】 或者 建档管理id【框架简称】
                    qFilter3.and(new QFilter("tpv_framebbrefs.id", QCP.in, mainIds_s)); //框架简称
                }
            }
            //根据 基础资料销售合同号的【框架简称】查询所有销售合同id
            DataSet tpv_dt_salescontract = QueryServiceHelper.queryDataSet(algoKey, "tpv_bd_salescontract",
                    "id contractNoId,tpv_framebbrefs.id filingId,tpv_projectabrefs.id projectId", new QFilter[]{qFilter3}, null);
            List conIds_s = new ArrayList();
            //copy遍历，防止联查时迭代(关闭后再次遍历)报错
            DataSet tpv_dt_salescontract_1 = tpv_dt_salescontract.copy();
            for (Row row : tpv_dt_salescontract_1) {
                conIds_s.add(row.getLong("contractNoId"));
            }
            if ("XM".equals(mainBill)) {
                dataSet1 = dataSet1.leftJoin(tpv_dt_salescontract).on("contractId", "projectId")
                        .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                                "tpv_customerId", "tpv_customerName", "tpv_projectabb", "contractNoId").finish();
            }
            if ("KJ".equals(mainBill)) {
                dataSet1 = dataSet1.leftJoin(tpv_dt_salescontract).on("contractId", "filingId")
                        .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                                "tpv_customerId", "tpv_customerName", "tpv_projectabb", "contractNoId").finish();
            }
            //已生成销售合同id
            qFilter2.and(new QFilter("tpv_sourcebillid", QCP.in, conIds_s));
            DataSet tpv_dt_salepricebill = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salepricebill",
                    "id billid,tpv_sourcebillid contractNoId,tpv_contractnumber,tpv_materialentry,tpv_materialentry.tpv_qty_final*tpv_materialentry.tpv_taxamt_final shippingAmt",
                    new QFilter[]{qFilter2}, null).groupBy(new String[]{"billid", "contractNoId"}).sum("shippingAmt").finish();

            DataSet tpv_dt_salesshipping = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salesshipping",
                    "id billid,tpv_sourcebillid contractNoId,tpv_contractnumber,tpv_materialentry," +
                            "(tpv_materialentry.tpv_qty - tpv_materialentry.tpv_confirmed - tpv_materialentry.tpv_cargobatchqty) * tpv_materialentry.tpv_taxprice shippingAmt",
                    new QFilter[]{qFilter2}, null).groupBy(new String[]{"billid", "contractNoId"}).sum("shippingAmt").finish();
            DataSet union = tpv_dt_salepricebill.union(tpv_dt_salesshipping);

            dataSet2 = dataSet1.leftJoin(union).on("contractNoId", "contractNoId")
                    .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                            "tpv_customerId", "tpv_customerName", "tpv_projectabb", "contractNoId", "shippingAmt").finish();
        } else {
            if (StringUtils.isNotEmpty(contractNo)) {
                qFilter2.and(new QFilter("tpv_contractnumber", QCP.like, "%" + contractNo + "%"));
            }
            if (mainIds_s.size() > 0) {  //销售合同id【销售合同号】 或者 建档管理id【项目简称】
                qFilter2.and(new QFilter("tpv_sourcebillid", QCP.in, mainIds_s));
            }
            DataSet tpv_dt_salepricebill = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salepricebill",
                    "id billid,tpv_sourcebillid contractId,tpv_contractnumber,tpv_materialentry,tpv_materialentry.tpv_qty_final*tpv_materialentry.tpv_taxamt_final shippingAmt",
                    new QFilter[]{qFilter2}, null).groupBy(new String[]{"billid", "contractId"}).sum("shippingAmt").finish();

            DataSet tpv_dt_salesshipping = QueryServiceHelper.queryDataSet(algoKey, "tpv_dt_salesshipping",
                    "id billid,tpv_sourcebillid contractId,tpv_contractnumber,tpv_materialentry," +
                            "(tpv_materialentry.tpv_qty - tpv_materialentry.tpv_confirmed - tpv_materialentry.tpv_cargobatchqty) * tpv_materialentry.tpv_taxprice shippingAmt",
                    new QFilter[]{qFilter2}, null).groupBy(new String[]{"billid", "contractId"}).sum("shippingAmt").finish();
            DataSet union = tpv_dt_salepricebill.union(tpv_dt_salesshipping);
            dataSet2 = dataSet1.leftJoin(union).on("contractId", "contractId")
                    .select("contractId", "tpv_bizdate", "orgId", "orgName", "tpv_departmentId", "tpv_departmentName",
                            "tpv_customerId", "tpv_customerName", "contractNo", "shippingAmt").finish();
        }
        //2.3销售退货总金额（暂无）
        //2.4开票核销高低计单（暂无）
        //2.5【总计金额】
        dataSet3 = dataSet2.addField("shippingAmt", "amtSum1");
        return dataSet3;
    }


    public static DynamicObjectCollection getDeliverableBalance(String tpvProjectFilter, long depId, long customerId, long contractId) {
        return new DynamicObjectCollection();
    }
}
