package com.itopview.wm.dt.api.clients;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.sys.utils.Api;
import com.itopview.wm.sys.utils.ApiHelper;
import com.itopview.wm.sys.utils.ApiLog;
import com.itopview.wm.utils.HttpHelper;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.control.events.BeforeItemClickEvent;
import kd.bos.form.events.AfterDoOperationEventArgs;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * ClassName: DHLApi
 * Description:
 * date: 2023/9/7 9:30
 *
 * @author: Guo jiaChen
 * @version:
 * @since JDK 1.8
 */
public class DHLApi extends AbstractBillPlugIn {
    public final static String TRACKING_NUMBER = "shipmentTrackingNumber";//发货编号
    public final static String REFERENCE_TYPE = "shipmentReferenceType";//运输类型
    public final static String TRACKING_VIEW = "trackingView";//轨迹视图
    public final static String LEVEL_DETAIL = "levelOfDetail";//详细级别
    public final static String FORMAT_DATE = "yyyy-MM-dd\'T\'HH:mm:ss";//时间转换格式


    @Override
    public void beforeItemClick(BeforeItemClickEvent evt) {
        super.beforeItemClick(evt);
        String key = evt.getItemKey();
        if ("bar_save".equals(key)) {
            setShipmentDesc();
        }
    }

    @Override
    public void afterDoOperation(AfterDoOperationEventArgs eventArgs) {
        super.afterDoOperation(eventArgs);
        if ("save".equals(eventArgs.getOperateKey())) {
            this.getView().updateView();
        }
    }


    /**
     * 获取船只信息
     */
    private void setShipmentDesc() {
        String errorMsg = "";//异常信息
        DynamicObjectCollection entries = this.getModel().getEntryEntity("tpv_breakdown");//分录
        int rowIndex = 1;
        for (DynamicObject entry : entries) {
            String courier = (String) entry.get("tpv_courier");//快递单号

            if (!courier.isEmpty()) {
                HashMap<String, Object> map = new HashMap<>();
                map.put(TRACKING_NUMBER, courier);
                //获取返回数据
                String result = runContract(map);
                JSONObject body = JSON.parseObject(result);//将结果转换为JSONObject
                String status = body.getString("status");//获取状态
                if (null == status) {//请求正常
                    JSONArray shipments = body.getJSONArray("shipments");
                    JSONObject shipment = (JSONObject) shipments.get(0);
                    String shipmentStatus = shipment.getString("status");//获取状态
                    String timestamp = shipment.getString("shipmentTimestamp");//货物装运时间戳
                    String deliveryDate = shipment.getString("estimatedDeliveryDate");//货物送达时间
                    if ("Success".equals(shipmentStatus)) {//请求成功!
                        Map<String, String> shipmentDesc = getShipmentStatus(shipment);//获取当前快递状态
                        if (shipmentDesc != null) {
                            String typeCode = shipmentDesc.get("typeCode");
                            if (typeCode.equals("OK")) {//已签收
                                String nowStatus = shipmentDesc.get("description");//当前快递状态
                                String date = shipmentDesc.get("date");//年-月-日
                                String time = shipmentDesc.get("time");//时-分-秒
                                entry.set("tpv_courierstatus", nowStatus);
                            }
                        }
                        Date signDate = parseDate(deliveryDate, FORMAT_DATE);//签收日期
                        Date sendDate = parseDate(timestamp, FORMAT_DATE);//寄出日期
                        //分录赋值
                        entry.set("tpv_sendoutdate", sendDate);
                        entry.set("tpv_receipteddate", signDate);
                    }
                } else {//请求失败!
                    if (status.equals("400")) {
                        //请求异常,订单追踪码格式错误
                        errorMsg += "分录第" + rowIndex + "行:请确认快递单号格式!" + "\n";
                    }
                    if (status.equals("404")) {
                        //请求失败,订单追踪码输入有误
                        errorMsg += "分录第" + rowIndex + "行:请输入正确的快递单号!" + "\n";
                    }
                    rowIndex++;
                }
            }
        }
        if (!errorMsg.isEmpty()) {
            this.getView().showTipNotification(errorMsg);
        }

    }


    /**
     * @param map
     */
    private static String runContract(Map<String, Object> map) {
        String number = (String) map.get(TRACKING_NUMBER);
        String type = (String) map.get(REFERENCE_TYPE);
        String trackingView = (String) map.get(TRACKING_VIEW);
        String levelOfDetail = (String) map.get(LEVEL_DETAIL);
//        number = "5980623180";
        type = "CU";
        trackingView = "all-checkpoints";
        levelOfDetail = "all";
        //请求参数
        String property = TRACKING_NUMBER + "=" + number +
                "&" + REFERENCE_TYPE + "=" + type +
                "&" + TRACKING_VIEW + "=" + trackingView +
                "&" + LEVEL_DETAIL + "=" + levelOfDetail;

        ApiHelper apiHelper = new ApiHelper("api.dhl");//api
        if (apiHelper.isExist()) {//api
            String secretId = apiHelper.getParam("secretId");//api
            Api api = apiHelper.getApi("dhl");//api
            if (api.isExist()) {//api
                ApiLog log = apiHelper.log(api);//获取日志
                //请求地址+请求参数
                String url = "https://express.api.dhl.com/mydhlapi/test/tracking?" + property;

                //Get请求(请求头信息)
                HashMap<String, String> headers = new HashMap<>();
                headers.put("Message-Reference", "13713c7bba4c16fab1ac826923640645");
                headers.put("Message-Reference-Date", "Sat,02 Apr 2022 10:13:22 GM");
                headers.put("Accept-Language", "chi");
                headers.put("Authorization", "Basic YXZpY2ludGxtaW5DTjpSQDlnQSM5ZlZAM3BHQDR3");

                String result = "";//返回结果
                try {
                    result = HttpHelper.httpGet(url, headers);//获取请求返回的结果

                } catch (Exception e) {
                    e.printStackTrace();
                    log.setErrorMsg(YEM.exceptionToStr(e));
                }
                //log
                log.setRequestData(url);
                log.setResponseData(result);
                log.setSuccess(true);
                log.write();
                return result;
            }
        }
        return "";
    }
    /**
     * 获取当前快递状态
     *
     * @param shipment
     */
    private static Map<String, String> getShipmentStatus(JSONObject shipment) {
        Map<String, String> map = new HashMap<>();
        JSONArray pieces = shipment.getJSONArray("pieces");
        JSONObject piece = (JSONObject) pieces.get(0);
        JSONArray events = piece.getJSONArray("events");
        if (events.size() > 0) {
            JSONObject event = (JSONObject) events.get(events.size() - 1);//获取快递最后的状态
            //map.put("trackingNumber",piece.getString("trackingNumber"));//获取快递单号
            map.put("date", event.getString("date"));//获取当前节点时间(年-月-日)
            map.put("time", event.getString("time"));//获取当前节点时间(时-分-秒)
            map.put("typeCode", event.getString("typeCode"));//获取当前节点状态码
            map.put("description", event.getString("description"));//获取当前节点状态描述
            return map;
        }
        return null;
    }

    /**
     * 日期格式转换
     *
     * @param date
     * @param formatDate
     * @return
     */
    private static Date parseDate(String date, String formatDate) {
        SimpleDateFormat format = new SimpleDateFormat(formatDate);
        try {
            return format.parse(date);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }
}
