package com.itopview.wm.dt.formplugin.form;

import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.DynamicObjectUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.control.AttachmentPanel;
import kd.bos.form.control.events.UploadEvent;
import kd.bos.form.control.events.UploadListener;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.Date;
import java.util.EventObject;

/**
 * @Description 协议单  实习查询生成下游单全部数据、去除协议号字段中所有的空格
 * @Author LinRui
 * Date 2023/8/11 11:47
 **/
public class AgreementFormPlugin extends AbstractBillPlugIn implements UploadListener {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        AttachmentPanel attachmentPanel = this.getView().getControl("attachmentpanel");
        attachmentPanel.addUploadListener(this);
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        addYearDay();
    }

    @Override
    public void afterRemove(UploadEvent evt) {
        outdoublesignature();
    }

    @Override
    public void afterUpload(UploadEvent evt) {
        IDataModel model = this.getModel();
        String jsonStr = (String) model.getValue("tpv_attachmentcon_tag");
        JSONObject json = YEM.isEmpty(jsonStr) ? new JSONObject() : JSONObject.parseObject(jsonStr);
        model.setValue("tpv_attachmentcon_tag", json.toJSONString());//附件状态
    }

    //值更新，去除字符串中所有的空格
    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        ChangeData cd = e.getChangeSet()[0];
        ChangeData[] changeData = e.getChangeSet();
        int rowIndex = changeData[0].getRowIndex();
        switch (name) {
            case "tpv_protocol":
                String tpv_protocol = (String) this.getModel().getValue("tpv_protocol");
                String replaceAll = tpv_protocol.replace(" ", "");//去除空格
                this.getModel().setValue("tpv_protocol", replaceAll);
                break;
            case "tpv_attachmenttype"://附件类型
                outdoublesignature();//带出双签状态
                break;
            case "tpv_signdate"://签约日期
                addYearDay();
                break;
        }
    }

    private void addYearDay() {
        Date signingdate = (Date) this.getModel().getValue("tpv_signdate");//签约日期
        if (signingdate != null) {
            Date date = DateUtil.addDays(signingdate, -1);
            int year = date.getYear() + 1;
            date.setYear(year);
            this.getModel().setValue("tpv_ctvaliddate", date);//合同有效期限
        }
    }

    private void outdoublesignature() {
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        DynamicObjectCollection attachmentdetail = dataEntity.getDynamicObjectCollection("tpv_attachmentdetail");//附件分录
        if (attachmentdetail.size() > 0) {
            for (DynamicObject attachment : attachmentdetail) {
                String tpv_attachmenttype = attachment.getString("tpv_attachmenttype.name");
                if (tpv_attachmenttype != null && tpv_attachmenttype.contains("双签")) {
                    this.getModel().setValue("tpv_desestatus", "B");//双签状态
                } else {
                    this.getModel().setValue("tpv_desestatus", "A");
                }
            }
        } else {
            this.getModel().setValue("tpv_desestatus", "A");
        }
    }

    //单据打开或刷新后，实时查询下游单信息，汇总到分录
    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        IDataModel model = this.getView().getModel();
        String value = (String) model.getValue("billstatus");
        if ("C".equals(value)) {
            dynamicEntry();//动态展示分录
        }
    }

    //实时动态查询--查询已生成下游单数据
    private void dynamicEntry() {
        String billno = (String) this.getModel().getValue("billno");

        QFilter[] qFilter = new QFilter[]{
                new QFilter("tpv_sourcebillno", QCP.in, billno)}; //根据来源单据编号过滤

        //查询下游月协
        DynamicObject[] queryYX = BusinessDataServiceHelper.load("tpv_dt_agreement", "id,billno", qFilter);
        if (queryYX != null) {
            for (DynamicObject dynamcObject : queryYX) {
                String billnoYX = dynamcObject.getString("billno");
                botpDynamicObjectEntry(billnoYX);//查询下游数据集
            }
        }
        botpDynamicObjectEntry(billno);//查询下游数据集

    }

    //查询下游数据集
    private void botpDynamicObjectEntry(String billno) {
//        QFilter[] qFilter = new QFilter[] {
//                new QFilter("tpv_sourcebillno", QCP.in, billno)}; //根据来源单据编号过滤
        QFilter qFilter = new QFilter("tpv_sourcebillno", QCP.in, billno); //根据来源单据编号过滤
        qFilter.or(new QFilter("tpv_longtermag.number", QCP.equals, billno));
        String selectStrxs = DynamicObjectUtil.getSelectfields("tpv_dt_salescontract", false);
        selectStrxs = DynamicObjectUtil.getEntrySelectfields(selectStrxs, "tpv_dt_salescontract", "tpv_materialentry", false);

        //查询下游销售合同
        DynamicObject[] queryXS = BusinessDataServiceHelper.load("tpv_dt_salescontract", selectStrxs, new QFilter[]{qFilter});

        //查询对应的单据主实体,过滤条件-标识跟单据主实体编号
        DynamicObject contractTypexs = BusinessDataServiceHelper.loadSingle("bos_billmainentity", "id"
                , new QFilter[]{new QFilter("number", QCP.equals, "tpv_dt_salescontract")});
        for (DynamicObject objxs : queryXS) {
            DynamicObjectCollection entryxs = objxs.getDynamicObjectCollection("tpv_materialentry");
            BigDecimal sumQtyxs = BigDecimal.ZERO;
            BigDecimal sumAmtxs = BigDecimal.ZERO;
            for (DynamicObject entityxs : entryxs) {
                sumQtyxs = sumQtyxs.add(entityxs.getBigDecimal("tpv_qty"));  //签约数量
                sumAmtxs = sumAmtxs.add(entityxs.getBigDecimal("tpv_taxpramount"));  //含税金额
            }
            //往分录上标识，添加行数据
            DynamicObjectCollection entryentityxs = this.getView().getModel().getEntryEntity("tpv_dt_agreement_w");

            // 新增行数据，赋值
            DynamicObject entityxs = entryentityxs.addNew();
            //entityxs.set("tpv_contractnr",objxs.getString("billno")); //单据编号
            entityxs.set("tpv_contractnr", objxs.getString("tpv_contractnos"));  //合同号
            entityxs.set("tpv_entrybillstatuss", objxs.getString("billstatus")); //单据状态
            entityxs.set("tpv_eycontractdate", objxs.getDate("tpv_signingdate")); //签约日期
            entityxs.set("tpv_eycurrency", objxs.getDynamicObject("tpv_currency")); //币别
            entityxs.set("tpv_typeofreward", objxs.getDynamicObject("tpv_awardtype"));  //提奖类型
            entityxs.set("tpv_businesstype", objxs.getDynamicObject("tpv_businesstype"));  //业务类型
            entityxs.set("tpv_contracttype", contractTypexs);  //合同类型
            entityxs.set("tpv_etstate", objxs.getString("tpv_contractstatus"));  //合同状态
            entityxs.set("tpv_ctquantitys", sumQtyxs); //签约数量
            entityxs.set("tpv_eytaxamount", sumAmtxs); //含税金额
            entityxs.set("tpv_eyhouseholds", objxs.getDynamicObject("tpv_customer")); //客户
            this.getView().updateView("tpv_dt_agreement_w");
        }


        String selectStrcg = DynamicObjectUtil.getSelectfields("tpv_dt_purchase", false);
        selectStrcg = DynamicObjectUtil.getEntrySelectfields(selectStrcg, "tpv_dt_purchase", "tpv_materialentry", false);

        //查询下游采购合同
        DynamicObject[] queryCG = BusinessDataServiceHelper.load("tpv_dt_purchase", selectStrcg, new QFilter[]{qFilter});

        //查询对应的单据主实体,过滤条件-标识跟单据主实体编号
        DynamicObject contractTypecg = BusinessDataServiceHelper.loadSingle("bos_billmainentity", "id"
                , new QFilter[]{new QFilter("number", QCP.equals, "tpv_dt_purchase")});
        for (DynamicObject objcg : queryCG) {
            DynamicObjectCollection entrycg = objcg.getDynamicObjectCollection("tpv_materialentry");
            BigDecimal sumQtycg = BigDecimal.ZERO;
            BigDecimal sumAmtcg = BigDecimal.ZERO;
            for (DynamicObject entitycg : entrycg) {
                sumQtycg = sumQtycg.add(entitycg.getBigDecimal("tpv_qty"));  //签约数量
                sumAmtcg = sumAmtcg.add(entitycg.getBigDecimal("tpv_taxpramount")); //含税金额
            }
            //往分录上标识，添加行数据
            DynamicObjectCollection entryentitycg = this.getView().getModel().getEntryEntity("tpv_dt_agreement_w");

            // 新增行数据，赋值
            DynamicObject entitycg = entryentitycg.addNew();
            //entitycg.set("tpv_contractnr",objcg.getString("billno")); //单据编号
            entitycg.set("tpv_contractnr", objcg.getString("tpv_contractnumber"));  //合同号
            entitycg.set("tpv_entrybillstatuss", objcg.getString("billstatus")); //单据状态
            entitycg.set("tpv_eycontractdate", objcg.getDate("tpv_dateofcontracting")); //签约日期
            entitycg.set("tpv_eycurrency", objcg.getDynamicObject("tpv_currency")); //币别
            entitycg.set("tpv_typeofreward", objcg.getDynamicObject("tpv_typeofreward"));  //提奖类型
            entitycg.set("tpv_businesstype", objcg.getDynamicObject("tpv_businesstype"));  //业务类型
            entitycg.set("tpv_contracttype", contractTypecg);  //合同类型
            entitycg.set("tpv_etstate", objcg.getString("tpv_contractstatusx"));  //合同状态
            entitycg.set("tpv_ctquantitys", sumQtycg); //签约数量
            entitycg.set("tpv_eytaxamount", sumAmtcg); //含税金额
            entitycg.set("tpv_eyhouseholds", objcg.getDynamicObject("tpv_supplier")); //供应商
            this.getView().updateView("tpv_dt_agreement_w");
        }
    }
}
