package com.itopview.wm.dt.formplugin.form;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.YEM;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.form.CloseCallBack;
import kd.bos.form.IFormView;
import kd.bos.form.ShowFormHelper;
import kd.bos.form.ShowType;
import kd.bos.form.control.Control;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.events.HyperLinkClickEvent;
import kd.bos.form.events.HyperLinkClickListener;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.IListView;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

/**
 * 合并审批-通用插件
 *
 * @author longh
 * @description 表单插件:采销合同、特殊合同、关联交易
 * @date 2023年8月29日
 */
public class ApprovalTogetherBillPlugIn extends AbstractBillPlugIn
        implements BeforeF7SelectListener, HyperLinkClickListener {

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        setEnable();
    }

    private void setEnable() {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        String tpv_hbsptype_apt = (String) model.getValue("tpv_hbsptype_apt");// 合并审批
        Long tpv_mainbillid_apt = (Long) model.getValue("tpv_mainbillid_apt");// 审批主题单据ID
        boolean active = "B".equals(tpv_hbsptype_apt);// 合并审批-主动
        boolean passive = "C".equals(tpv_hbsptype_apt);// 合并审批-被动
        view.setEnable(active, "tpv_newentry_app", "tpv_deleteentry_app");
        view.setVisible(YEM.isNotEmpty(tpv_mainbillid_apt) && passive, "tpv_btn_apt");
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners("tpv_btn_apt", "tpv_billno_apt", "tpv_connum_apt");
        EntryGrid grid = (EntryGrid) this.getControl("tpv_approvaltogether");
        grid.addHyperClickListener(this);
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        IDataModel model = this.getModel();
        String name = e.getProperty().getName();
        ChangeData[] changeData = e.getChangeSet();
        int rowIndex = changeData[0].getRowIndex();
        switch (name) {
            case "tpv_hbsptype_apt":
                setEnable();
                model.deleteEntryData("tpv_approvaltogether");
                model.setValue("tpv_mainbillkey_apt", null);
                model.setValue("tpv_mainbillno_apt", null);
                model.setValue("tpv_mainbillid_apt", null);
                hbsptypeApt();
                break;
            case "tpv_billkey_apt":
                model.setValue("tpv_billno_apt", "", rowIndex);
                model.setValue("tpv_connum_apt", "", rowIndex);
                break;
            default:
                break;
        }
    }

    private void hbsptypeApt() {
        if ("B".equals(this.getModel().getValue("tpv_hbsptype_apt"))) {
            this.getModel().setValue("tpv_mainbillno_apt", this.getModel().getValue("billno"));
            this.getModel().setValue("tpv_mainbillkey_apt", this.getModel().getDataEntity(true).getDataEntityType().getName());  //审批主题单据
        } else {
            this.getModel().setValue("tpv_mainbillid_apt", null);
            this.getModel().setValue("tpv_mainbillno_apt", null);
            this.getModel().setValue("tpv_mainbillkey_apt", null);
        }
    }

    @Override
    public void afterDoOperation(AfterDoOperationEventArgs e) {
        super.afterDoOperation(e);
        String operateKey = e.getOperateKey();
        OperationResult operationResult = e.getOperationResult();
        if (operationResult != null && operationResult.isSuccess()) {
            switch (operateKey) {
                case "save":
                case "submit":
                    if (operationResult != null && operationResult.isSuccess()) {
                        this.getView().invokeOperation("refresh");
                    }
                    break;
                default:
                    break;
            }
            setEnable();
        }
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        String key = ((Control) evt.getSource()).getKey();
        switch (key) {
            case "tpv_btn_apt":
                mainbillnoClick();
                break;
            case "tpv_billno_apt":
            case "tpv_connum_apt":
                billnoClick();
                break;
            default:
                break;
        }
    }

    private void mainbillnoClick() {
        IDataModel model = this.getModel();
        String tpv_mainbillkey_apt = (String) model.getValue("tpv_mainbillkey_apt");
        Object tpv_mainbillid_apt = model.getValue("tpv_mainbillid_apt");
        showBill(tpv_mainbillkey_apt, tpv_mainbillid_apt);
    }

    private void billnoClick() {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        int index = model.getEntryCurrentRowIndex("tpv_approvaltogether");
        String billkey = (String) model.getValue("tpv_billkey_apt", index);
        if (YEM.isNotEmpty(billkey)) {
            List<Long> fids = new ArrayList<>();
            DynamicObjectCollection entryEntity = model.getEntryEntity("tpv_approvaltogether");
            for (DynamicObject row : entryEntity) {
                String tpv_billkey_apt = row.getString("tpv_billkey_apt");
                if (billkey.equals(tpv_billkey_apt)) {
                    fids.add(row.getLong("tpv_billid_apt"));
                }
            }
            QFilter qFilter = new QFilter("billstatus", QCP.equals, "B");
            qFilter.and(new QFilter("tpv_hbsptype_apt", QCP.equals, "C"));
            qFilter.and(new QFilter("tpv_mainbillid_apt", QCP.equals, 0L));
            qFilter.and(new QFilter("id", QCP.not_in, fids));

            ListShowParameter createShowListForm = ShowFormHelper.createShowListForm(billkey, true, 0);
            createShowListForm.setCloseCallBack(new CloseCallBack(this, "tpv_billno_apt"));
            createShowListForm.getListFilterParameter().getQFilters().add(qFilter);
            view.showForm(createShowListForm);
        } else {
            view.showTipNotification("请先选择查询单据。");
        }
    }

    private void showBill(String formkey, Object formid) {
        if (YEM.isNotEmpty(formkey) || YEM.isNotEmpty(formid)) {
            BillShowParameter showParameter = new BillShowParameter();
            showParameter.setFormId(formkey);
            showParameter.setPkId(formid);
            showParameter.setStatus(OperationStatus.VIEW);
            showParameter.getOpenStyle().setShowType(ShowType.MainNewTabPage);
            this.getView().showForm(showParameter);
        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        String actionId = closedCallBackEvent.getActionId();
        Object returnData = closedCallBackEvent.getReturnData();
        switch (actionId) {
            case "tpv_billno_apt":
                billF7CallBack(returnData);
                break;
            default:
                break;
        }
    }

    private void billF7CallBack(Object returnData) {
        IDataModel model = this.getModel();
        if (returnData instanceof ListSelectedRowCollection) {
            ListSelectedRowCollection selectRows = (ListSelectedRowCollection) returnData;
            if (selectRows != null && selectRows.size() > 0) {
                Object[] ids = new Object[selectRows.size()];
                for (int i = 0; i < selectRows.size(); i++) {
                    ids[i] = selectRows.get(i).getPrimaryKeyValue();
                }
                int index = model.getEntryCurrentRowIndex("tpv_approvaltogether");
                String billkey = (String) model.getValue("tpv_billkey_apt", index);
                if (YEM.isNotEmpty(billkey)) {
                    MainEntityType type = EntityMetadataCache.getDataEntityType(billkey);
                    DynamicObject[] bills = BusinessDataServiceHelper.load(ids, type);
                    for (int i = 0; i < bills.length; i++) {
                        DynamicObject bill = bills[i];
                        int index2;
                        if (i == 0) {
                            index2 = index;
                        } else {
                            index2 = model.createNewEntryRow("tpv_approvaltogether");
                        }
                        model.setValue("tpv_billkey_apt", billkey, index2);// 单据
                        model.setValue("tpv_billno_apt", bill.get("billno"), index2);// 单据号
                        String name = this.getModel().getDataEntity(true).getDataEntityType().getName();
                        String num = "billno";
                        if ("tpv_dt_purchase".equals(billkey)) {
                            num = "tpv_contractnumber";
                        } else if ("tpv_dt_salescontract".equals(billkey)) {
                            num = "tpv_contractnos";
                        }
                        model.setValue("tpv_connum_apt", bill.get(num), index2);// 合同号
                        model.setValue("tpv_billid_apt", bill.get("id"), index2);// 单据ID
                        model.setValue("tpv_supplier_apt", bill.get("tpv_supplier"), index2);// 供应商
                        model.setValue("tpv_customer_apt", bill.get("tpv_customer"), index2);// 客户

                        BigDecimal qty = BigDecimal.ZERO;
                        BigDecimal amt = BigDecimal.ZERO;
                        List<String> materialList = new ArrayList<>();
                        DynamicObjectCollection tpv_materialentry = bill
                                .getDynamicObjectCollection("tpv_materialentry");
                        for (DynamicObject row : tpv_materialentry) {
                            qty = BigDecimalUtils.add(qty, row.getBigDecimal("tpv_qty"), 10);
                            amt = BigDecimalUtils.add(amt, row.getBigDecimal("tpv_taxpramount"), 10);
                            DynamicObject tpv_material = row.getDynamicObject("tpv_material");
                            if (tpv_material != null) {
                                materialList.add(tpv_material.getLocaleString("name").getLocaleValue());
                            }
                        }
                        model.setValue("tpv_materials_apt", String.join(",", materialList), index2);// 商品
                        model.setValue("tpv_qty_apt", qty, index2);// 数量
                        model.setValue("tpv_price_apt", BigDecimalUtils.div(amt, qty, 2), index2);// 单价
                        model.setValue("tpv_amt_apt", amt, index2);// 金额

                        String billTypeKey = "";
                        switch (billkey) {
                            case "tpv_dt_purchase":
                                billTypeKey = "tpv_document";
                                break;
                            case "tpv_dt_salescontract":
                                billTypeKey = "tpv_documenttypes";
                                break;
                            default:
                                break;
                        }
                        if (YEM.isNotEmpty(billTypeKey)) {
                            model.setValue("tpv_billtype_apt", bill.get(billTypeKey), index2);// 单据类型
                        }
                    }
                }
            }
        }
    }

    @Override
    public void hyperLinkClick(HyperLinkClickEvent e) {
        IDataModel model = this.getModel();
        String fieldName = e.getFieldName();
        int rowIndex = e.getRowIndex();
        switch (fieldName) {
            case "tpv_billno_apt":
			case "tpv_connum_apt":
                String tpv_billkey_apt = (String) model.getValue("tpv_billkey_apt", rowIndex);
                Object tpv_billid_apt = model.getValue("tpv_billid_apt");
                showBill(tpv_billkey_apt, tpv_billid_apt);
                break;
            default:
                break;
        }
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String name = e.getProperty().getName();
        switch (name) {
            case "":

                break;
        }
    }

}
