package com.itopview.wm.dt.formplugin.form;

import com.itopview.wm.dt.utils.PickShipsHelper;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.property.AmountProp;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.StyleCss;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.control.events.RowClickEvent;
import kd.bos.form.control.events.RowClickEventListener;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.events.BeforeDoOperationEventArgs;
import kd.bos.form.field.AmountEdit;
import kd.bos.form.operate.FormOperate;
import kd.bos.report.ReportShowParameter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.util.StringUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author ljw
 * @version 1.0
 * @description: 销售发货表表单插件
 * @date 2023/8/10 14:51
 * ljwxj
 */
public class SalesshippingBillPlug extends AbstractBillPlugIn implements RowClickEventListener {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addItemClickListeners(new String[]{"tpv_tbmainentry", "tbmain"});
        EntryGrid entryGrid = this.getView().getControl("tpv_materialentry");
        entryGrid.addRowClickListener(this);
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        IFormView view = this.getView();
        setQtyEnableByPick(view);
        setQtyEnableByPrice(view);
        Object sourcebillid = this.getModel().getValue("tpv_sourcebillid");
        if (YEM.isNotEmpty(sourcebillid)){
            this.getView().setEnable(false, "tpv_billtype");
        }
    }

    /**
     * 货物批次后设置数量锁定
     *
     * @param view
     */
    private void setQtyEnableByPrice(IFormView view) {
        String entryKey = "tpv_materialentry";
        IDataModel model = view.getModel();
        DynamicObjectCollection entryEntity = model.getEntryEntity(entryKey);
        for (int i = 0; i < entryEntity.size(); i++) {
            DynamicObject row = entryEntity.get(i);
            String tpv_selectiondocu = row.getString("tpv_selectiondocu") == null ? "" : row.getString("tpv_selectiondocu");
            String tpvDocument = row.getString("tpv_document");
            if (StringUtils.isNotEmpty(tpv_selectiondocu)) {
                if (!"A".equals(tpvDocument)) {
                    this.getView().setEnable(false, i, "tpv_qty");
                }
            } else {
                this.getView().setEnable(true, i, "tpv_qty");
            }
        }
    }

    /**
     * 拣配后设置数量锁定
     *
     * @param view
     */
    public static void setQtyEnableByPick(IFormView view) {
        // String entityKey = view.getEntityId();
        String entryKey = "tpv_materialentry";
        String qtyKey = "tpv_qty";
        String packnumKey = "tpv_packnum";
        // String materialKey = "tpv_material";
        IDataModel model = view.getModel();
        DynamicObjectCollection entryEntity = model.getEntryEntity(entryKey);
        for (int i = 0; i < entryEntity.size(); i++) {
            DynamicObject row = entryEntity.get(i);
            DynamicObjectCollection salepick_zy = row.getDynamicObjectCollection("tpv_dt_salepick_zy");
            DynamicObjectCollection salepick_in = row.getDynamicObjectCollection("tpv_dt_salepick_in");
            if (salepick_zy.size() > 0 || salepick_in.size() > 0) {
                view.setEnable(false, i, qtyKey);
                view.setEnable(false, i, packnumKey);
            }
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        ChangeData[] changeData = e.getChangeSet();
        int rowIndex = changeData[0].getRowIndex();
        switch (name) {
            case "tpv_inventorytype":
                inventoryTypeChanged(changeData);
                break;
            case "tpv_whether":
            case "tpv_business":
                business(changeData);
                break;
//            case "tpv_practicalok_zy":
//                practicalok(changeData);
//                break;
        }
    }
//
//    /**
//     * 根据子分录的值更新父分录数据
//     * @param changeData
//     */
//    private void practicalok(ChangeData[] changeData) {
//        DynamicObjectCollection tpv_materialentry = (DynamicObjectCollection) this.getModel().getEntryEntity("tpv_materialentry");
//        for (DynamicObject dynamicObject : tpv_materialentry) {
//            Boolean flag = true;
//            DynamicObjectCollection tpv_dt_salepick_in = dynamicObject.getDynamicObjectCollection("tpv_dt_salepick_in");
//            DynamicObjectCollection tpv_dt_salepick_zy = dynamicObject.getDynamicObjectCollection("tpv_dt_salepick_zy");
//            if (tpv_dt_salepick_in.size()>0) {
//                for (DynamicObject dynamicObjects : tpv_dt_salepick_in) {
//                    Boolean tpv_practicalok_in = dynamicObjects.getBoolean("tpv_practicalok_in");
//                    if (!tpv_practicalok_in) {
//                        flag = false;
//                    }
//                }
//            }
//            if (tpv_dt_salepick_zy.size()>0) {
//                for (DynamicObject dynamicObjects : tpv_dt_salepick_zy) {
//                    Boolean tpv_practicalok_zy = dynamicObjects.getBoolean("tpv_practicalok_zy");
//                    if (!tpv_practicalok_zy) {
//                        flag = false;
//                    }
//                }
//            }
//            if (flag) {
//                dynamicObject.set("tpv_practicalok",true);
//            }
//        }
//
//    }

    private void business(ChangeData[] changeData) {
        Boolean tpv_whether = (Boolean) this.getModel().getValue("tpv_whether");
        DynamicObject tpv_business = (DynamicObject) this.getModel().getValue("tpv_business");
        String number = null;
        if (null != tpv_business) {
            number = tpv_business.getString("number");

        }
        if ("YWLX-230902-0001".equals(number) || tpv_whether) {
            AmountEdit textEdit = this.getView().getControl("tpv_thirdpartyprice");
            textEdit.setMustInput(true);
            AmountProp textProp = (AmountProp) textEdit.getProperty();
            textProp.setMustInput(true);
        } else {
            AmountEdit textEdit = this.getView().getControl("tpv_thirdpartyprice");
            textEdit.setMustInput(false);
            AmountProp textProp = (AmountProp) textEdit.getProperty();
            textProp.setMustInput(false);
        }
    }

    /**
     * 库存/直运-修改校验
     *
     * @param changeData
     * @author longh
     */
    private void inventoryTypeChanged(ChangeData[] changeData) {
        IDataModel model = this.getModel();
        String oldValue = (String) changeData[0].getOldValue();
        if (YEM.isNotEmpty(oldValue)) {
            String subEnd = "A".equals(oldValue) ? "_in" : "_zy";
            int count = model.getEntryRowCount("tpv_dt_salepick" + subEnd);
            if (count > 0) {
                model.setValue("tpv_inventorytype", oldValue);
                this.getView().showTipNotification("已有拣配信息，不允许修改。");
            }
        }
    }

    @Override
    public void afterDoOperation(AfterDoOperationEventArgs e) {
        super.afterDoOperation(e);
        IFormView view = this.getView();
        // String operateKey = e.getOperateKey();
        OperationResult operationResult = e.getOperationResult();
        if (operationResult != null && operationResult.isSuccess()) {
            setQtyEnableByPick(view);
        }
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String itemKey = evt.getItemKey().toLowerCase();
        String rpt = "";
        Long cusId = 0L;
        Long deptId = 0L;
        DynamicObject tpv_customer = (DynamicObject) this.getModel().getValue("tpv_customer");
        if (null != tpv_customer) {
            cusId = tpv_customer.getLong("id");
        }
        DynamicObject tpv_department = (DynamicObject) this.getModel().getValue("tpv_department");
        if (null != tpv_department) {
            deptId = tpv_department.getLong("id");
        }
        switch (itemKey) {
            case "tpv_pick":
                IFormView view = this.getView();
                IDataModel model = this.getModel();
                DynamicObject dataEntity = model.getDataEntity(true);
                EntryGrid entry = view.getControl("tpv_materialentry");
                int selectRows = entry.getSelectRows().length;
                if (selectRows > 1) {
                    view.showTipNotification("只能选择一行数据进行操作。");
                }
                if (selectRows == 0) {
                    view.showTipNotification("未选择数据。");
                }
                if (selectRows == 1) {
                    int index = entry.getSelectRows()[0];
                    DynamicObject row = model.getEntryRowEntity("tpv_materialentry", index);
                    DynamicObject tpvDtSalescontract = BusinessDataServiceHelper.loadSingle(row.getLong("tpv_cobillid"), "tpv_dt_salescontract");

                    if (null == row.getString("tpv_selectiondocu") || "".equals(row.getString("tpv_selectiondocu"))) {
                        view.showTipNotification("第 " + row.get("seq") + "行 未货物批次选择，或者货物批次单号为空，不允许拣配。");
                        return;
                    }
                }
                PickShipsHelper.pickClick(this.getView());
                break;
            case "tpv_inseriespick": //串拣配
                DynamicObject dataEntityInseries = this.getModel().getDataEntity(true);
                dataEntityInseries.set("tpv_yesnoinseries","1");
                OperationResult saveOperate = this.getView().invokeOperation("save", YEM.getOptionNoMsg());
                SaveServiceHelper.update(dataEntityInseries);// 写入数据库
                PickShipsHelper.pickClick(this.getView());
                break;
            case "tpv_cargobatch": // 货物批次选择
                cargobatchClick();
                break;
            case "tpv_contractinquiries": //合同可放货余额查询
                rpt = "tpv_cuscreditbalancerpta";
                cusCreditRptDataSet(rpt, cusId, deptId);
                break;
            case "tpv_projectqueries": //项目可放货余额查询
                rpt = "tpv_cuscreditbalancerptb";
                cusCreditRptDataSet(rpt, cusId, deptId);
                break;
            case "tpv_frameworkqueries": //框架可放货余额查询
                rpt = "tpv_cuscreditbalancerptc";
                cusCreditRptDataSet(rpt, cusId, deptId);
                break;
            case "tpv_customerinquiries": //客户额度使用情况查询
                rpt = "tpv_cuscredituserpt";
                cusCreditRptDataSet(rpt, cusId, deptId);
                break;
            default:
                break;
        }
    }

    private void cusCreditRptDataSet(String rptbill, Long tpvCustomerId, Long tpvDepartmentId) {
        //打开报表页面
        ReportShowParameter reportShowParameter = new ReportShowParameter();
        reportShowParameter.getOpenStyle().setShowType(ShowType.Modal);
        reportShowParameter.setFormId(rptbill);
        StyleCss styleCss = new StyleCss();
        styleCss.setHeight("720");
        styleCss.setWidth("1200");
        //设置弹出表单的样式，宽，高等
        reportShowParameter.getOpenStyle().setInlineStyleCss(styleCss);
        // 设置报表过滤条件
        ReportQueryParam reportQueryParam = new ReportQueryParam();
        FilterInfo filterInfo = new FilterInfo();
        filterInfo.addFilterItem("tpv_customer_filter", tpvCustomerId);
        filterInfo.addFilterItem("tpv_dep_filter", tpvDepartmentId);
        reportQueryParam.setFilter(filterInfo);
        reportShowParameter.setQueryParam(reportQueryParam);
        getView().showForm(reportShowParameter);
    }

    /**
     * 按钮操作执行前监听
     * 阻止复制行
     *
     * @param args
     */
    @Override
    public void beforeDoOperation(BeforeDoOperationEventArgs args) {
        super.beforeDoOperation(args);
        FormOperate operate = (FormOperate) args.getSource();
        String key = operate.getOperateKey();
        if ("copyentryrow".equals(key)) {
            //获取单据体控件  校验
            EntryGrid entry = this.getView().getControl("tpv_materialentry");
            int selectRows[] = entry.getSelectRows();
            //获取单据体数据集合
            DynamicObjectCollection entity = this.getModel().getEntryEntity("tpv_materialentry");
            Map<Long, DynamicObject> map = new HashMap<>();
            if (null != selectRows && selectRows.length > 0) {
                for (int selectRow : selectRows) {
                    DynamicObject dynamicObject = entity.get(selectRow);  //获取选中行的单据体数据
                    map.put(Long.valueOf(selectRow), dynamicObject);
                }
            } else {
                this.getView().showTipNotification("未选择行数据。");
                return;
            }
            List<String> settleSeqList = new ArrayList();
            if (null != map && map.size() > 0) {
                for (Map.Entry<Long, DynamicObject> dynamicObject : map.entrySet()) {
                    DynamicObject dynamicObject1 = dynamicObject.getValue();
                    String tpv_selectiondocu = dynamicObject1.getString("tpv_selectiondocu");//货物批次单号
                    if (null != tpv_selectiondocu && "" != tpv_selectiondocu) {
                        String settleSeq = dynamicObject1.getString("seq");
                        settleSeqList.add(settleSeq);
                    }
                }
                if (settleSeqList.size() > 0) {
                    args.setCancel(true);
                    this.getView().showTipNotification("分录选中行 " + String.join(",", settleSeqList.toString()) + " 已货物批次价格确认，不允许再次复制！");
                }
            }
        }
    }

    /**
     * @author: LinRui
     * @Description: //货物批次选择
     **/
    private void cargobatchClick() {
        IFormView view = this.getView();
        IDataModel model = this.getModel();
        DynamicObject dataEntity = model.getDataEntity(true);
        EntryGrid entry = view.getControl("tpv_materialentry");
        int selectRows = entry.getSelectRows().length;
        if (selectRows > 1) {
            view.showTipNotification("只能选择一行数据进行操作。");
        }
        if (selectRows == 0) {
            view.showTipNotification("未选择数据。");
        }
        if (selectRows == 1) {
            int index = entry.getSelectRows()[0];
            DynamicObject row = model.getEntryRowEntity("tpv_materialentry", index);
            DynamicObject tpvDtSalescontract = BusinessDataServiceHelper.loadSingle(row.getLong("tpv_cobillid"), "tpv_dt_salescontract");
            BigDecimal tpvOverloadingrates = tpvDtSalescontract.getBigDecimal("tpv_overloadingrates"); //溢装率(%)

            if (row.getDynamicObjectCollection("tpv_dt_salepick_zy").size() > 0
                    || row.getDynamicObjectCollection("tpv_dt_salepick_in").size() > 0) {
                view.showTipNotification("第 " + row.get("seq") + " 已拣配过，不允许再次批次选择。");
                return;
            }
//			if (YEM.isNotEmpty(row.getString("tpv_document")) || YEM.isNotEmpty(row.getString("tpv_selectiondocu"))) {
//				view.showTipNotification("第 " + row.get("seq") + " 行已成功操作过一下货物批次选择，不允许再次批次选择。");
//				return;
//			}

            OperationResult saveOperate = view.invokeOperation("save", YEM.getOptionNoMsg());
            if (!saveOperate.isSuccess()) {
                view.showTipNotification("自动保存失败。" + YEM.getOpMsg(saveOperate));
                return;
            }
            FormShowParameter showParameter = new FormShowParameter();
            showParameter.setStatus(OperationStatus.ADDNEW);
            showParameter.getOpenStyle().setShowType(ShowType.Modal);
            showParameter.setFormId("tpv_dt_cargobatchs");

            List<String> notInputs = new ArrayList<>();
            Map<String, Object> param = new HashMap<>();
            param.put("pkid", row.getPkValue());
            param.put("tpv_material", PickShipsHelper.getNotNull(view, notInputs, row, "tpv_material")); // 物料
            param.put("tpv_qty", PickShipsHelper.getNotNull(view, notInputs, row, "tpv_qty")); // 数量
            param.put("tpv_cobillno", row.get("tpv_cobillno")); // 核心单据编码
            param.put("tpv_cobillid", row.get("tpv_cobillid")); // 核心单据ID
            param.put("tpv_coentryid", row.get("tpv_coentryid")); // 核心单据分录ID
            param.put("tpv_srcbillkey", row.get("tpv_srcbillkey")); // 来源单据标识
            param.put("tpv_srcbillno", row.get("tpv_srcbillno")); // 来源单据编码
            param.put("tpv_srcbillid", row.get("tpv_srcbillid")); // 来源单据ID
            param.put("tpv_srcentryid", row.get("tpv_srcentryid")); // 来源单据分录ID
            DynamicObject tpvUnit = (DynamicObject) row.get("tpv_unit");
            Long tpvUnitLong = 0L;
            if (null != tpvUnit) {
                tpvUnitLong = tpvUnit.getLong("id");
            }
            param.put("tpv_units", tpvUnitLong); // 计量单位
            param.put("tpv_document", row.get("tpv_document")); // 货物批次选择单据
            param.put("tpv_selectiondocu", row.get("tpv_selectiondocu")); // 货物批次选择单据号
            param.put("tpv_overloadingrates", tpvOverloadingrates); // 核心单据的溢装率(%)
            if (notInputs.size() > 0) {
                view.showTipNotification(String.format("请先填写【%s】。", String.join("、", notInputs)));
            } else {
                showParameter.setCustomParams(param);
                view.showForm(showParameter);
            }
        }
    }

    @Override
    public void entryRowClick(RowClickEvent evt) {
        RowClickEventListener.super.entryRowClick(evt);
        int row = evt.getRow();
        DynamicObject rowData = this.getModel().getEntryRowEntity("tpv_materialentry", row);
        if (null != rowData) {
            String tpvDocument = rowData.getString("tpv_document");
            String tpv_selectiondocu = rowData.getString("tpv_selectiondocu") == null ? "" : rowData.getString("tpv_selectiondocu");
            if (StringUtils.isNotEmpty(tpv_selectiondocu)) {
                if (!"A".equals(tpvDocument)) {
                    this.getView().setEnable(false, "tpv_advconbaritemap");
                    this.getView().setEnable(false, row, "tpv_qty");
                }
            } else {
                this.getView().setEnable(true, "tpv_advconbaritemap");
                this.getView().setEnable(true, row, "tpv_qty");
            }
        } else {
            return;
        }
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);

    }
}
