package com.itopview.wm.dt.formplugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.form.ConfirmCallBackListener;
import kd.bos.form.MessageBoxOptions;
import kd.bos.form.MessageBoxResult;
import kd.bos.form.control.events.BeforeItemClickEvent;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.MessageBoxClosedEvent;

import java.util.EventObject;

/**
 * @Description: 测试生成成本按钮弹框确认提示
 * @Author LinRui
 * @Date 2023/12/21 10:25
 */
public class TestCostFormPlugin extends AbstractBillPlugIn {
    //工具栏已投标按钮的标识
    private static final String KEY_COSTSETOP = "tpv_costsetop";
    //工具栏已投标按钮的标识
    private static final String KEY_TAKEEFFECT = "tpv_takeeffect";
    //工具栏未投标按钮的标识
    private static final String KEY_UNTAKEEFFECT = "tpv_untakeeffect";
    //工具栏未投标按钮的标识
    private static final String KEY_RED = "tpv_red"; //采购发票、货票发票红冲按钮
    //工具栏未投标按钮的标识
    private static final String KEY_REDPUNCH = "tpv_redpunch";  //费用发票红冲按钮
    //单据中的工具栏标识
    private static final String KEY_TOOlBAR = "tbmain";

    /**
     * 注册工具栏监听
     *
     * @param e
     */
    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        //监听工具栏按钮点击事件
        this.addItemClickListeners(KEY_TOOlBAR);
    }

    @Override
    public void beforeItemClick(BeforeItemClickEvent evt) {
        super.beforeItemClick(evt);
        //工具栏上的所有按钮的点击都会激活itemClick方法，需要开发人员实现不同按钮的逻辑
        if (StringUtils.equals(KEY_COSTSETOP, evt.getItemKey())) {
            //作废操作前让用户选择是否继续作废操作，在用户点击确认框上的按钮后，系统会调用confirmCallBack方法
            ConfirmCallBackListener confirmCallBackListener = new ConfirmCallBackListener(KEY_COSTSETOP, this);
            //设置页面确认框，参数为：标题，选项框类型，回调监听
            this.getView().showConfirm("是否确认生成成本？", MessageBoxOptions.YesNo, confirmCallBackListener);
            evt.setCancel(true);
        }
        if (StringUtils.equals(KEY_TAKEEFFECT, evt.getItemKey())) {
            //作废操作前让用户选择是否继续作废操作，在用户点击确认框上的按钮后，系统会调用confirmCallBack方法
            ConfirmCallBackListener confirmCallBackListener = new ConfirmCallBackListener(KEY_TAKEEFFECT, this);
            //设置页面确认框，参数为：标题，选项框类型，回调监听
            this.getView().showConfirm("是否确认生成成本？", MessageBoxOptions.YesNo, confirmCallBackListener);
            evt.setCancel(true);
        }
        if (StringUtils.equals(KEY_UNTAKEEFFECT, evt.getItemKey())) {
            //作废操作前让用户选择是否继续作废操作，在用户点击确认框上的按钮后，系统会调用confirmCallBack方法
            ConfirmCallBackListener confirmCallBackListener = new ConfirmCallBackListener(KEY_UNTAKEEFFECT, this);
            //设置页面确认框，参数为：标题，选项框类型，回调监听
            this.getView().showConfirm("是否确认删除成本？", MessageBoxOptions.YesNo, confirmCallBackListener);
            evt.setCancel(true);
        }
        if (StringUtils.equals(KEY_RED, evt.getItemKey())) {
            //作废操作前让用户选择是否继续作废操作，在用户点击确认框上的按钮后，系统会调用confirmCallBack方法
            ConfirmCallBackListener confirmCallBackListener = new ConfirmCallBackListener(KEY_RED, this);
            //设置页面确认框，参数为：标题，选项框类型，回调监听
            this.getView().showConfirm("是否红冲", MessageBoxOptions.YesNo, confirmCallBackListener);
            evt.setCancel(true);
        }
        if (StringUtils.equals(KEY_REDPUNCH, evt.getItemKey())) {
            //作废操作前让用户选择是否继续作废操作，在用户点击确认框上的按钮后，系统会调用confirmCallBack方法
            ConfirmCallBackListener confirmCallBackListener = new ConfirmCallBackListener(KEY_REDPUNCH, this);
            //设置页面确认框，参数为：标题，选项框类型，回调监听
            this.getView().showConfirm("是否红冲", MessageBoxOptions.YesNo, confirmCallBackListener);
            evt.setCancel(true);
        }
    }

    @Override
    public void confirmCallBack(MessageBoxClosedEvent messageBoxClosedEvent) {
        super.confirmCallBack(messageBoxClosedEvent);
        //判断是否是对应确认框的点击回调事件
        if (StringUtils.equals(KEY_COSTSETOP, messageBoxClosedEvent.getCallBackId())) {
            if (MessageBoxResult.Yes.equals(messageBoxClosedEvent.getResult())) {
                this.getView().invokeOperation("costsetop");
                this.getView().showSuccessNotification("生成成功！");
            } else if (MessageBoxResult.No.equals(messageBoxClosedEvent.getResult())) {
                // 点击否的相关处理逻辑。。。。
            }
        }
        //判断是否是对应确认框的点击回调事件
        if (StringUtils.equals(KEY_TAKEEFFECT, messageBoxClosedEvent.getCallBackId())) {
            if (MessageBoxResult.Yes.equals(messageBoxClosedEvent.getResult())) {
                this.getView().invokeOperation("takeeffect");
                this.getView().showSuccessNotification("生成成功！");
            } else if (MessageBoxResult.No.equals(messageBoxClosedEvent.getResult())) {
                // 点击否的相关处理逻辑。。。。
            }
        }
        //判断是否是对应确认框的点击回调事件
        if (StringUtils.equals(KEY_UNTAKEEFFECT, messageBoxClosedEvent.getCallBackId())) {
            if (MessageBoxResult.Yes.equals(messageBoxClosedEvent.getResult())) {
                this.getView().invokeOperation("untakeeffect");
            } else if (MessageBoxResult.No.equals(messageBoxClosedEvent.getResult())) {
                // 点击否的相关处理逻辑。。。。
            }
        }
        //判断是否是对应确认框的点击回调事件
        if (StringUtils.equals(KEY_RED, messageBoxClosedEvent.getCallBackId())) {
            if (MessageBoxResult.Yes.equals(messageBoxClosedEvent.getResult())) {
                this.getView().invokeOperation("red");
            } else if (MessageBoxResult.No.equals(messageBoxClosedEvent.getResult())) {
                // 点击否的相关处理逻辑。。。。
            }
        }
        //判断是否是对应确认框的点击回调事件
        if (StringUtils.equals(KEY_REDPUNCH, messageBoxClosedEvent.getCallBackId())) {
            if (MessageBoxResult.Yes.equals(messageBoxClosedEvent.getResult())) {
                this.getView().invokeOperation("redpunch");
            } else if (MessageBoxResult.No.equals(messageBoxClosedEvent.getResult())) {
                // 点击否的相关处理逻辑。。。。
            }
        }
    }

    /**
     * 工具栏按钮点击事件
     *
     * @param evt
     */
    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);

    }
}
