package com.itopview.wm.dt.formplugin.form;

import com.itopview.wm.dt.utils.PickShipsHelper;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.datamodel.RowDataEntity;
import kd.bos.entity.datamodel.events.AfterAddRowEventArgs;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.Date;
import java.util.EventObject;
import java.util.HashMap;
import java.util.Map;

/**
 * 分步调出单-表单插件
 *
 * @author longh
 * @description
 * @date 2023年9月22日
 */
public class TransOutBillPlugin extends AbstractBillPlugIn implements BeforeF7SelectListener {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addItemClickListeners(new String[]{"tpv_tbmainentry", "tpv_advcontoolbarap1","tbmain"});
        ((BasedataEdit) this.getControl("tpv_operator")).addBeforeF7SelectListener(this);
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        this.getModel().setValue("tpv_department",null);
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String itemKey = evt.getItemKey().toLowerCase();
        switch (itemKey) {
            case "tpv_pick":
                PickShipsHelper.pickClick(this.getView());
                break;
            case "tpv_bulkentrybutton": //客户额度使用情况查询
                showInfoForm(this.getView());
                break;
            case "tpv_upwarehouse":
                showDyFormBill();
                break;
            default:
                break;
        }
    }

    private void showDyFormBill() {
        // 创建弹出动态表单页面对象
        FormShowParameter showParameter = new FormShowParameter();
        // 设置弹出页面的标识
        showParameter.setFormId("tpv_iscb_advconap");
        Map<String, Object> map = new HashMap<>();
        DynamicObject entrepot = (DynamicObject) this.getModel().getValue("tpv_entrepot");
        if (YEM.isEmpty(entrepot)) {
            this.getView().showTipNotification("请先填写仓库");
            return;
        }
        map.put("tpv_entrepot",entrepot.getLong("id"));
        // 存入获取到的动态表单数据
        showParameter.setCustomParams(map);
        // 状态
        showParameter.setStatus(OperationStatus.ADDNEW);
        // 设置页面关闭回调方法
        // CloseCallBack参数：回调插件，回调标识
        showParameter.setCloseCallBack(new CloseCallBack(this, "tpv_upwarehouse"));
        // 设置弹出页面打开方式
        showParameter.getOpenStyle().setShowType(ShowType.Modal);
        // 打开动态表单
        this.getView().showForm(showParameter);
    }

    /**
     * 设置打开批量录入金额动态表单
     */
    private void showInfoForm(IFormView view) {
        // 获取单据体
        FormShowParameter showParameter = new FormShowParameter();
        showParameter.setStatus(OperationStatus.ADDNEW);
        showParameter.getOpenStyle().setShowType(ShowType.Modal);
        showParameter.setFormId("tpv_bulkentryexpenses");
        showParameter.setCloseCallBack(new CloseCallBack(this, "tpv_bulkentrybutton"));
        view.showForm(showParameter);
    }

    /**
     * @param e 回调，接收子页面传来的值。
     */
    @Override
    public void closedCallBack(ClosedCallBackEvent e) {
        String actionId = e.getActionId();
        Object returnData = e.getReturnData();
        switch (actionId) {
            case "tpv_bulkentrybutton":
                DynamicObjectCollection data = (DynamicObjectCollection) returnData;
                setValue(data);
                break;
            case "tpv_upwarehouse":
                if (null != returnData) {
                    HashMap<String, Object> values = (HashMap<String, Object>) returnData;
                    this.setInwardInfo(values);
                }
                break;
            case "tpv_operator":
                operatorCallBack(e);
                break;
            default:
                break;
        }
    }

    private void setInwardInfo(HashMap<String, Object> values) {
        IDataModel model = this.getModel();
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        model.setValue("tpv_sendingpath", values.get("tpv_sendingpaths"));
        // 回填动态表单的接口方
        model.setValue("tpv_interface", values.get("tpv_interface") != null ? Long.valueOf((String) values.get("tpv_interface")) : null);
        // 回填动态表单的传真号
        model.setValue("tpv_fax", values.get("tpv_faxs"));
        // 回填动态表单的邮箱地址
        model.setValue("tpv_eaddress", values.get("tpv_eaddresss"));
        // 回填动态表单的用途
        model.setValue("tpv_specification", values.get("tpv_specifications"));
        model.setValue("tpv_default", values.get("tpv_defaults"));
        SaveServiceHelper.update( dataEntity );//写入数据库
        SaveServiceHelper.save(new DynamicObject[] {dataEntity});
    }

    private void setValue(DynamicObjectCollection data) {
        if (data != null && data.size() > 0) {
            DynamicObject dataEntity = this.getModel().getDataEntity(true);
            DynamicObjectCollection materialentry = dataEntity.getDynamicObjectCollection("billentry");//商品明细分录
            for (int i = 0; materialentry.size() > i; i++) {
                DynamicObject dynamicObject = materialentry.get(i);
                BigDecimal tpvQty = dynamicObject.getBigDecimal("qty");
                DynamicObjectCollection dynamicObjects = dynamicObject.getDynamicObjectCollection("tpv_dt_transoutbill_f");
                for (DynamicObject freigroambill : data) {
                    DynamicObject addNewDy = dynamicObjects.addNew();
                    addNewDy.set("tpv_costitem",freigroambill.getDynamicObject("tpv_expenseitems"));
                    addNewDy.set("tpv_supplier",freigroambill.getDynamicObject("tpv_supplierf"));
                    addNewDy.set("tpv_reimbursement",freigroambill.getDynamicObject("tpv_costcurrencys"));
                    addNewDy.set("tpv_chargequantity",tpvQty);
                    addNewDy.set("tpv_univalence",freigroambill.getBigDecimal("tpv_unitprices"));
                    BigDecimal tpvMoneys= tpvQty.multiply(freigroambill.getBigDecimal("tpv_unitprices"));
                    addNewDy.set("tpv_amt",tpvMoneys.setScale(2, BigDecimal.ROUND_HALF_UP));
                    addNewDy.set("tpv_taxrate",freigroambill.getBigDecimal("tpv_taxrates"));
                    addNewDy.set("tpv_desc",freigroambill.getString("tpv_comments"));
                    DynamicObject tpv_currency = freigroambill.getDynamicObject("tpv_costcurrencys");
                    if (YEM.isNotEmpty(tpv_currency)) {
                        Object tpv_exratetable = this.getModel().getValue("tpv_exratetable");
                        Object tpv_currencybase = this.getModel().getValue("settlescurrency"); //本位币
                        Date tpv_exratedate = (Date) this.getModel().getValue("tpv_exratedate");
                        BigDecimal exchangeRateCny = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 1L, tpv_exratedate);
                        BigDecimal exchangeRateUsd = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 6L, tpv_exratedate);
                        BigDecimal exchangeRate = YEM.getExchangeRate(tpv_exratetable, tpv_currency, tpv_currencybase, tpv_exratedate);
                        addNewDy.set("tpv_costrate",exchangeRate);
                    }
                }
            }
            this.getView().updateView("tpv_dt_transoutbill_f");
        }
    }
    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String key = e.getProperty().getName();
        ChangeData cd = e.getChangeSet()[0];
        switch (key) {
            case "tpv_reimbursement":
                this.setExchangeRate();
                break;
            case "tpv_exratetable":
            case "tpv_exratedate":
            case "settlescurrency":
                setSubEntryExchangeRate();
                break;
            case "tpv_entrepot":
                entrepot(cd);
                break;
        }
    }

    private void entrepot(ChangeData cd) {
        IDataModel model = this.getModel();
        DynamicObject dataEntity = model.getDataEntity(true);
        DynamicObjectCollection billentry = dataEntity.getDynamicObjectCollection("billentry");
        DynamicObject tpvEntrepotObject = (DynamicObject) model.getValue("tpv_entrepot");
        model.deleteEntryData("billentry");
        if (YEM.isEmpty(tpvEntrepotObject)){
            model.setValue("tpv_sendstate", null);
            model.setValue("tpv_sendingpath",null);
            model.setValue("tpv_interface",null);
            model.setValue("tpv_fax",null);
            model.setValue("tpv_eaddress",null);
            model.setValue("tpv_specification",null);
            model.setValue("tpv_default",null);
            return;
        }
        long entrepotId = (long) ((DynamicObject) cd.getNewValue()).getPkValue();
        DynamicObject entrepot = BusinessDataServiceHelper.loadSingle(entrepotId, "bd_warehouse");
        DynamicObjectCollection sendmode = entrepot.getDynamicObjectCollection("tpv_sendmode");//发货指令发送方式
        Boolean flag = false;
        if (sendmode.size() > 0) {
            for (DynamicObject obj:sendmode) {
                boolean tpvDefault = obj.getBoolean("tpv_default");
                String sendingpath = obj.getString("tpv_sendingpath");//发送途径
                DynamicObject api = obj.getDynamicObject("tpv_interface");//API
                String fax = obj.getString("tpv_fax");//传真号
                String eaddress = obj.getString("tpv_eaddress");//邮箱地址
                String specification = obj.getString("tpv_specification");//途径说明
                if (tpvDefault) {
                    if ("E".equals(sendingpath)) {
                        model.setValue("tpv_sendstate", "A");
                    }
                    model.setValue("tpv_sendingpath",sendingpath);
                    model.setValue("tpv_interface",api);
                    model.setValue("tpv_fax",fax);
                    model.setValue("tpv_eaddress",eaddress);
                    model.setValue("tpv_specification",specification);
                    model.setValue("tpv_default",tpvDefault);
                    flag = true;
                }
            }
        }
        if (sendmode.size() <= 0 | !flag) {
            model.setValue("tpv_sendstate", null);
            model.setValue("tpv_sendingpath",null);
            model.setValue("tpv_interface",null);
            model.setValue("tpv_fax",null);
            model.setValue("tpv_eaddress",null);
            model.setValue("tpv_specification",null);
            model.setValue("tpv_default",null);
        }
    }

    @Override
    public void afterAddRow(AfterAddRowEventArgs e) {
        super.afterAddRow(e);
        String name = e.getEntryProp().getName();
        int i = 0;
        RowDataEntity[] rowdata = e.getRowDataEntities();
        for (RowDataEntity rdata : rowdata) {
            i = rdata.getRowIndex();
        }
        switch (name) {
            case "tpv_dt_transoutbill_f"://费用分录
                EntryGrid entryGrid = this.getControl("billentry");//商品明细
                int selectRows[] = entryGrid.getSelectRows();//获取选中的行号
                DynamicObject dataEntity = this.getModel().getDataEntity(true);
                DynamicObjectCollection billentry = dataEntity.getDynamicObjectCollection("billentry");//商品分录
                int entryCurrentRowIndex = this.getModel().getEntryCurrentRowIndex("billentry");
//                for (int row : selectRows) {
                DynamicObject billentryRow = billentry.get(entryCurrentRowIndex);
                DynamicObjectCollection tpvDtPurinbill = billentryRow.getDynamicObjectCollection("tpv_dt_transoutbill_f");//费用分录
                for (DynamicObject dynamicObject : tpvDtPurinbill) {
                    DynamicObject tpv_currency = dynamicObject.getDynamicObject("tpv_reimbursement");  //结算币别
                    if (null != tpv_currency) {
                        Object tpv_exratetable = this.getModel().getValue("tpv_exratetable");
                        Object tpv_currencybase = this.getModel().getValue("settlescurrency"); //本位币
                        Date tpv_exratedate = (Date) this.getModel().getValue("tpv_exratedate");
                        BigDecimal exchangeRateCny = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 1L, tpv_exratedate);
                        BigDecimal exchangeRateUsd = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 6L, tpv_exratedate);
                        BigDecimal exchangeRate = YEM.getExchangeRate(tpv_exratetable, tpv_currency, tpv_currencybase, tpv_exratedate);
                        this.getModel().setValue("tpv_costrate", exchangeRate, i);
                    }
                }
//                }
                break;
        }
    }

    /**
     * 汇率获取
     */
    private void setExchangeRate() {
        int index = this.getModel().getEntryCurrentRowIndex("tpv_reimbursement");
        Object tpv_exratetable = this.getModel().getValue("tpv_exratetable");
        Object tpv_currency = this.getModel().getValue("tpv_reimbursement", index);  //结算币别
        Object tpv_currencybase = this.getModel().getValue("settlescurrency"); //本位币
        Date tpv_exratedate = (Date) this.getModel().getValue("tpv_exratedate");
        BigDecimal exchangeRateCny = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 1L, tpv_exratedate);
        BigDecimal exchangeRateUsd = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 6L, tpv_exratedate);
        BigDecimal exchangeRate = YEM.getExchangeRate(tpv_exratetable, tpv_currency, tpv_currencybase, tpv_exratedate);
        this.getModel().setValue("tpv_costrate", exchangeRate, index);
    }

    private void setSubEntryExchangeRate() {
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        DynamicObjectCollection billentry = dataEntity.getDynamicObjectCollection("billentry");
        if (billentry.size() > 0) {
            for (int i = 0; i < billentry.size(); i++) {
                DynamicObject dynamicObject = billentry.get(i);
                DynamicObjectCollection tpvDtPurinbillF = dynamicObject.getDynamicObjectCollection("tpv_dt_transdirbill_f");
                if (tpvDtPurinbillF.size() > 0) {
                    this.getModel().setEntryCurrentRowIndex("billentry", i);
                    for (int i1 = 0; i1 < tpvDtPurinbillF.size(); i1++) {
                        DynamicObject object = tpvDtPurinbillF.get(i1);
                        DynamicObject tpv_currency = object.getDynamicObject("tpv_reimbursement");
                        if (null != tpv_currency) {
                            Object tpv_exratetable = this.getModel().getValue("tpv_exratetable");
                            Object tpv_currencybase = this.getModel().getValue("settlescurrency"); //本位币
                            Date tpv_exratedate = (Date) this.getModel().getValue("tpv_exratedate");
                            BigDecimal exchangeRateCny = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 1L, tpv_exratedate);
                            BigDecimal exchangeRateUsd = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 6L, tpv_exratedate);
                            BigDecimal exchangeRate = YEM.getExchangeRate(tpv_exratetable, tpv_currency, tpv_currencybase, tpv_exratedate);
                            this.getModel().setValue("tpv_costrate", exchangeRate, i1);
                        }
                    }
                }
            }
        }
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String name = e.getProperty().getName();
        switch (name) {
            case "tpv_operator":
                operatorF7(e, name);
                break;
            default:
                break;
        }
    }

    private void operatorF7(BeforeF7SelectEvent e, String key) {
        FormShowParameter showParameter = e.getFormShowParameter();
        CloseCallBack closeCallBack = new CloseCallBack(this, key);
        showParameter.setCloseCallBack(closeCallBack);
    }

    private void operatorCallBack(ClosedCallBackEvent e) {
        IDataModel model = this.getModel();
        ListSelectedRowCollection selectedRows = (ListSelectedRowCollection) e.getReturnData();
        if(selectedRows != null && selectedRows.size() > 0){
            ListSelectedRow selectedRow = selectedRows.get(0);
            long userId = (long) selectedRow.getPrimaryKeyValue();//人员id
            long userEntryId = (long) selectedRow.getEntryPrimaryKeyValue();//部门分录id
            model.beginInit();
            model.setValue("tpv_operator", userId);
            DynamicObjectCollection entry = getUserDepts(userId);
            for (DynamicObject row : entry) {
                if(userEntryId == row.getLong("id")){
                    model.setValue("tpv_department", row.get("org"));
                }
            }
            model.endInit();
            this.getView().updateView("tpv_operator");
            this.getView().updateView("tpv_department");
        }
    }

    private static DynamicObjectCollection getUserDepts(long userId) {
        QFilter qFilter = new QFilter("user", QCP.equals, userId);
//        qFilter.and(new QFilter("org.orgpattern.number", QCP.equals, "Orgform06"));
        return QueryServiceHelper.query("bos_userposition", "id,org", qFilter.toArray());
    }
}
