package com.itopview.wm.dt.opplugin.op;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.sys.formplugin.dynamic.ApptoSetsTestEdit;
import com.itopview.wm.sys.utils.NewVersionHelper;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 合并审批-通用插件
 *
 * @author longh
 * @description 服务插件
 * @date 2023年8月29日
 */
public class ApprovalTogetherOp extends AbstractOperationServicePlugIn {
    private static Log log = LogFactory.getLog(ApprovalTogetherOp.class);
    private static final String opKey = "tpv_approvaltogether";

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("billno");
        fieldKeys.add("tpv_hbsptype_apt");
        fieldKeys.add("tpv_mainbillkey_apt");
        fieldKeys.add("tpv_mainbillno_apt");
        fieldKeys.add("tpv_mainbillid_apt");
        fieldKeys.add("tpv_approvaltogether");
        fieldKeys.add("tpv_billkey_apt");
        fieldKeys.add("tpv_billno_apt");
        fieldKeys.add("tpv_billid_apt");
        fieldKeys.add("tpv_connum_apt");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {

            @Override
            public void validate() {
                String operateKey = this.getOperateKey();
                ExtendedDataEntity[] dataEntities = this.getDataEntities();
                for (ExtendedDataEntity extendedDataEntity : dataEntities) {
                    switch (operateKey) {
                        case "save":
                        case "submit":
                            validateRepeat(extendedDataEntity);
                            approvalTogether(operateKey,extendedDataEntity);
                            break;
                        default:
                            break;
                    }
                }
            }

            private void approvalTogether(String operateKey, ExtendedDataEntity extendedDataEntity) {
                if ("submit".equals(operateKey)){
                    DynamicObject dataEntity = extendedDataEntity.getDataEntity();
                    String entityNumber = this.getEntityKey();
                    NewVersionHelper vHelper = new NewVersionHelper(entityNumber);
                    if (vHelper.isChanged()) {
                        return;
                    }
                    DynamicObjectCollection tpv_approvaltogether = dataEntity
                            .getDynamicObjectCollection("tpv_approvaltogether");
                    String tpvHbsptypeApt = dataEntity.getString("tpv_hbsptype_apt");
                    if (tpv_approvaltogether.size()==0 && "B".equals(tpvHbsptypeApt)){
                        this.addErrorMessage(extendedDataEntity,"本单的【审批方式】选择了合并审批-主动，请在提交前将对应的合并审批被动单据挂上，方可带其一起生效。");
                    }
                }
            }

            private void validateRepeat(ExtendedDataEntity extendedDataEntity) {
                DynamicObject dataEntity = extendedDataEntity.getDataEntity();
                Object pkValue = dataEntity.getPkValue();
                String entityNumber = this.getEntityKey();
                NewVersionHelper vHelper = new NewVersionHelper(entityNumber);
                if (vHelper.isChanged()) {
                    return;
                }
                DynamicObjectCollection tpv_approvaltogether = dataEntity
                        .getDynamicObjectCollection("tpv_approvaltogether");
                for (int i = 0; i < tpv_approvaltogether.size(); i++) {
                    DynamicObject row = tpv_approvaltogether.get(i);
                    String tpv_billno_apt = row.getString("tpv_billno_apt");// 单据号
                    String tpv_billkey_apt = row.getString("tpv_billkey_apt");// 单据
                    long tpv_billid_apt = row.getLong("tpv_billid_apt");// 单据ID
                    if (YEM.isEmpty(tpv_billkey_apt) || YEM.isEmpty(tpv_billid_apt)) {
                        this.addErrorMessage(extendedDataEntity, String.format("合并审批第【%s】行数据缺失，请修改。", i + 1));
                        continue;
                    }
                    QFilter qFilter = new QFilter("tpv_billid_apt", QCP.equals, tpv_billid_apt);
                    qFilter.and(new QFilter("tpv_billkey_apt", QCP.equals, tpv_billkey_apt));
                    if (YEM.isNotEmpty(pkValue)) {
                        qFilter.and(new QFilter("tpv_srcbillid_apt", QCP.not_equals, pkValue));
                    }
                    if (QueryServiceHelper.exists("tpv_approvaltogether", qFilter.toArray())) {
                        MainEntityType dataEntityType = YEM.getDataEntityType(entityNumber);
                        String billName = dataEntityType.getDisplayName().getLocaleValue();
                        this.addErrorMessage(extendedDataEntity,
                                String.format("合并审批第【%s】行，%s【%s】已协同审批，请修改。", i + 1, billName, tpv_billno_apt));
                    }
                }
                if (YEM.isNotEmpty(pkValue)) {
                    JSONArray jsonArray = new JSONArray();
                    QFilter qFilter = new QFilter("tpv_srcbillid_apt", QCP.equals, pkValue);
                    DynamicObjectCollection query = QueryServiceHelper.query("tpv_approvaltogether",
                            "tpv_billkey_apt,tpv_billid_apt", qFilter.toArray());
                    for (DynamicObject obj : query) {
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.put("tpv_billkey_apt", obj.get("tpv_billkey_apt"));
                        jsonObject.put("tpv_billid_apt", obj.get("tpv_billid_apt"));
                        jsonArray.add(jsonObject);
                    }
                    OperateOption option = this.getOption();
                    option.setVariableValue(opKey + String.valueOf(pkValue), jsonArray.toJSONString());
                }
            }
        });
    }

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
        String operationKey = e.getOperationKey();
        for (DynamicObject dataEntity : e.getDataEntities()) {
            switch (operationKey) {
                case "save":
                    saveInfo(dataEntity);
                    break;
                case "submit":
                    submitInfo(dataEntity);
                    break;
                case "audit":
                    auditInfo(dataEntity);
                    break;
                default:
                    break;
            }
        }
    }

    private void submitInfo(DynamicObject dataEntity) {
        String entityKey = dataEntity.getDynamicObjectType().getName();
        NewVersionHelper vHelper = new NewVersionHelper(entityKey);
        if (vHelper.isChanged()) {
            return;
        }
        String tpvHbsptypeApt = dataEntity.getString("tpv_hbsptype_apt");
        StringBuilder builder = new StringBuilder();
        int purchase = 0; //采购合同
        int salescontract = 0;//销售合同
        if ("B".equals(tpvHbsptypeApt)) {
            DynamicObjectCollection collections = dataEntity.getDynamicObjectCollection("tpv_approvaltogether");
            if (collections.size() == 0) {
                return;
            }
            String number = dataEntity.getDataEntityType().getName();
            DynamicObject apptogetersets = BusinessDataServiceHelper.loadSingle("tpv_tpl_apptogetersets", new QFilter[]{new QFilter("tpv_requirebill", QCP.equals, number)});
            if (apptogetersets == null) {
                return;
            }
            DynamicObjectCollection collection = dataEntity.getDynamicObjectCollection("tpv_approvaltogether");
            for (DynamicObject approvaltogether : collection) {
                String billkey = approvaltogether.getString("tpv_billkey_apt");
                if ("tpv_dt_purchase".equals(billkey) || "tpv_dt_purchase_inh".equals(billkey)) {
                    purchase++;
                }
                if ("tpv_dt_salescontract".equals(billkey) || "tpv_dt_salescontract_ext".equals(billkey)) {
                    salescontract++;
                }
                String tpvBillnoApt = approvaltogether.getString("tpv_billno_apt");
                DynamicObject loadSingle1 = BusinessDataServiceHelper.loadSingle(billkey, new QFilter[]{new QFilter("billno", QCP.equals, tpvBillnoApt)});
                Object pkValue = loadSingle1.getPkValue();
                String richText = ApptoSetsTestEdit.bulidRichText(apptogetersets, pkValue, billkey);
                builder.append(richText);
            }
            StringBuilder join = new StringBuilder();
            join.append("本次审批将附带" + purchase + "个采购合同，" + salescontract + "个销售合同合并生效；");
            join.append(builder);
            dataEntity.set("tpv_join_tag", join);
            SaveServiceHelper.update(dataEntity);
        }
    }

    private void saveInfo(DynamicObject dataEntity) {
        String entityKey = dataEntity.getDynamicObjectType().getName();
        NewVersionHelper vHelper = new NewVersionHelper(entityKey);
        if (vHelper.isChanged()) {
            return;
        }
        DynamicObjectCollection tpv_approvaltogether = dataEntity.getDynamicObjectCollection("tpv_approvaltogether");
        Map<String, Map<Long, Boolean>> map = new HashMap<>();
        for (DynamicObject row : tpv_approvaltogether) {
            String tpv_billkey_apt = row.getString("tpv_billkey_apt");// 单据
            long tpv_billid_apt = row.getLong("tpv_billid_apt");// 单据ID
            Map<Long, Boolean> map2;
            if (map.containsKey(tpv_billkey_apt)) {
                map2 = map.get(tpv_billkey_apt);
            } else {
                map2 = new HashMap<>();
            }
            map2.put(tpv_billid_apt, true);
            map.put(tpv_billkey_apt, map2);
        }
        OperateOption option = this.getOption();
        Object pkValue = dataEntity.getPkValue();
        if (option.containsVariable(opKey + String.valueOf(pkValue))) {
            String variableValue = option.getVariableValue(opKey + String.valueOf(pkValue));
            JSONArray parseArray = JSON.parseArray(variableValue);
            if (parseArray != null) {
                for (int i = 0; i < parseArray.size(); i++) {
                    JSONObject jsonObject = parseArray.getJSONObject(i);
                    String tpv_billkey_apt = jsonObject.getString("tpv_billkey_apt");
                    Long tpv_billid_apt = jsonObject.getLong("tpv_billid_apt");
                    Map<Long, Boolean> map2;
                    if (map.containsKey(tpv_billkey_apt)) {
                        map2 = map.get(tpv_billkey_apt);
                    } else {
                        map2 = new HashMap<>();
                    }
                    if (!map2.containsKey(tpv_billid_apt)) {
                        map2.put(tpv_billid_apt, false);
                    } else {
                        map2.remove(tpv_billid_apt);
                    }
                    map.put(tpv_billkey_apt, map2);
                }
            }
        }
        for (String billKey : map.keySet()) {
            Map<Long, Boolean> map2 = map.get(billKey);
            if (!map2.isEmpty()) {
                QFilter qFilter = new QFilter("id", QCP.in, map2.keySet());
                DynamicObject[] load = BusinessDataServiceHelper.load(billKey,
                        "tpv_mainbillkey_apt,tpv_mainbillno_apt,tpv_mainbillid_apt", qFilter.toArray());
                for (DynamicObject bill : load) {
                    long pkId = bill.getLong("id");
                    if (map2.getOrDefault(pkId, false)) {
                        bill.set("tpv_mainbillkey_apt", entityKey);
                        bill.set("tpv_mainbillno_apt", dataEntity.get("billno"));
                        bill.set("tpv_mainbillid_apt", pkValue);
                    } else {
                        bill.set("tpv_mainbillkey_apt", null);
                        bill.set("tpv_mainbillno_apt", null);
                        bill.set("tpv_mainbillid_apt", null);
                    }
                }
                SaveServiceHelper.save(load);
            }
        }
        String tpv_hbsptype_apt = dataEntity.getString("tpv_hbsptype_apt");
        if ("B".equals(tpv_hbsptype_apt)) {
            dataEntity.set("tpv_mainbillkey_apt", entityKey);
            dataEntity.set("tpv_mainbillno_apt", dataEntity.get("billno"));
            dataEntity.set("tpv_mainbillid_apt", pkValue);
            SaveServiceHelper.update(dataEntity);
        }
    }

    private void auditInfo(DynamicObject dataEntity) {
        String entityKey = dataEntity.getDynamicObjectType().getName();
        NewVersionHelper vHelper = new NewVersionHelper(entityKey);
        if (vHelper.isChanged()) {
            return;
        }
        DynamicObjectCollection tpv_approvaltogether = dataEntity.getDynamicObjectCollection("tpv_approvaltogether");
        for (DynamicObject row : tpv_approvaltogether) {
            String tpv_billno_apt = row.getString("tpv_billno_apt");// 单据号
            String tpv_connum_apt = row.getString("tpv_connum_apt");// 合同号
            String tpv_billkey_apt = row.getString("tpv_billkey_apt");// 单据
            long tpv_billid_apt = row.getLong("tpv_billid_apt");// 单据ID
            if (YEM.isNotEmpty(tpv_billkey_apt) && YEM.isNotEmpty(tpv_billid_apt)) {
                OperationResult executeOperate = OperationServiceHelper.executeOperate("audit", tpv_billkey_apt,
                        new Object[]{tpv_billid_apt}, YEM.getOption());
                if (!executeOperate.isSuccess()) {
                    MainEntityType dataEntityType = YEM.getDataEntityType(tpv_billkey_apt);
                    String billName = dataEntityType.getDisplayName().getLocaleValue();
                    log.error(String.format("%s【%s】审核失败。%s", billName, tpv_connum_apt, YEM.getOpMsg(executeOperate)));
                }
            }
        }
    }

}