package com.itopview.wm.dt.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;

import java.util.List;

/**
 * @Description: 内贸结算单（客户） -审核
 * @Author LinRui
 * @Date 2024/4/25 10:57
 */
public class DomSettlsCusAuditOp extends AbstractOperationServicePlugIn implements Plugin {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_purchasecontracts");
        fieldKeys.add("tpv_billingtype");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] bills = this.getDataEntities();
                String key = this.getOperateKey();
                for (int i = 0; i < bills.length; i++) {
                    ExtendedDataEntity bill = bills[i];
                    switch (key) {
                        case "audit":
                            DynamicObject[] entities = e.getDataEntities();
                            for (DynamicObject entry : entities) {
                                DynamicObject tpvPurcontractno = entry.getDynamicObject("tpv_purchasecontracts");
                                if (YEM.isNotEmpty(tpvPurcontractno)) {
                                    QFilter qFilter = new QFilter("tpv_purchasecontracts.tpv_settlementstatus", QCP.equals,"B");
                                    qFilter.and(new QFilter("id",QCP.not_equals,entry.getLong("id")));
                                    DynamicObject[] dynamicObject = BusinessDataServiceHelper.load("tpv_dt_domsettc","id",qFilter.toArray());
                                    if (dynamicObject.length > 0) {
                                        this.addErrorMessage(bill,"本单销售合同号，已存在历史最终结算单！");
                                    }
                                }
                            }
                    }
                }
            }
        });
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        String name=e.getOperationKey();
        DynamicObject[] bills = e.getDataEntities();
        if("unaudit".equals(name)){
            for (int i = 0; i < bills.length; ++i) {
                DynamicObject bill = bills[i];
                backContratSettlementStatus(bill, false);  //反写合同的结算状态
            }
        }
        if("audit".equals(name)){
            for (int i = 0; i < bills.length; ++i) {
                DynamicObject bill = bills[i];
                backContratSettlementStatus(bill, true);  //反写合同的结算状态
            }
        }
    }

    private void backContratSettlementStatus(DynamicObject bill, Boolean flag) {
        DynamicObject entity = BusinessDataServiceHelper.loadSingle(bill.getPkValue(), bill.getDataEntityType().getName());
        DynamicObject tpvPurcontractno = entity.getDynamicObject("tpv_purchasecontracts");
        if (YEM.isNotEmpty(tpvPurcontractno)) {
            String tpvBillingtype = entity.getString("tpv_billingtype");
            if (YEM.isNotEmpty(tpvBillingtype) & flag) {
                if ("B".equals(tpvBillingtype)) {
                    DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle(tpvPurcontractno.getPkValue(), "tpv_dt_salescontract");
                    dynamicObject.set("tpv_settlementstatus","B");
                    SaveServiceHelper.save(new DynamicObject[]{dynamicObject});
                }
            }
            if (YEM.isNotEmpty(tpvBillingtype) & !flag) {
                if ("B".equals(tpvBillingtype)) {
                    DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle(tpvPurcontractno.getPkValue(), "tpv_dt_salescontract");
                    dynamicObject.set("tpv_settlementstatus","A");
                    SaveServiceHelper.save(new DynamicObject[]{dynamicObject});
                }
            }
        }

    }
}
