package com.itopview.wm.dt.opplugin.op;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.exception.KDBizException;
import kd.sdk.plugin.Plugin;

import java.math.BigDecimal;
import java.util.List;

/**
 * @ClassName ProcurementAuditOp
 * @Description 卷号总数量不能小于商品数量
 * @Author yhj
 * @Date 2023/8/22 16:43
 * @Version 1.0
 */
public class ProcurementAuditOp extends AbstractOperationServicePlugIn implements Plugin {

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                if ("submit".equals(this.getOperateKey())) {
                    ExtendedDataEntity[] bills = this.getDataEntities();
                    for (int i = 0; i < bills.length; ++i) {
                        ExtendedDataEntity bill = bills[i];
                        onsubmitDate(bill);
                    }
                }
            }

            private void onsubmitDate(ExtendedDataEntity dataEntities) {
                DynamicObject dy = dataEntities.getDataEntity();
                BigDecimal tpvBillablequantitys = BigDecimal.ZERO;//费用子分录数量合计
                DynamicObjectCollection billentry = dy.getDynamicObjectCollection("billentry");//商品明细
                if (billentry.size() > 0) {
                    for (DynamicObject mater : billentry) {
                        BigDecimal tpvQty = mater.getBigDecimal("qty");//商品分录数量
                        DynamicObjectCollection tpvDtPurinbillf = mater.getDynamicObjectCollection("tpv_dt_purinbill_f");//费用明细
                        if (tpvDtPurinbillf.size() > 0) {
                            for (DynamicObject purchase : tpvDtPurinbillf) {
                                DynamicObject tpvCostitem = purchase.getDynamicObject("tpv_costitem");//费用项目
                                BigDecimal tpv_amt = purchase.getBigDecimal("tpv_amt");//金额
                                int row = purchase.getInt("Seq");
                                if (tpvCostitem != null && tpv_amt.compareTo(BigDecimal.ZERO) == 0) {
                                    this.addMessage(dataEntities, "费用信息分录第" + row + "行金额不能为空!");
                                }
                                tpvBillablequantitys = BigDecimal.ZERO;
                                for (DynamicObject freigroambill : tpvDtPurinbillf) {
                                    tpvBillablequantitys = tpvBillablequantitys.add(freigroambill.getBigDecimal("tpv_chargequantity"));
                                }
//                                if (tpvBillablequantitys.compareTo(tpvQty) > 0) {
//                                    this.addMessage(dataEntities, "商品明细第" + row + "行（{费用信息}子分录【计费数量】之和不能大于{商品信息}分录行【数量】）!");
//                                }
                            }
                        }
                    }
                }
            }
        });
    }

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_dt_purinbill_j");
        keys.add("tpv_dt_purinbill_j.tpv_quantity");
        keys.add("tpv_dt_purinbill_f.tpv_chargequantity");
        keys.add("tpv_dt_purinbill_f.seq");
        keys.add("tpv_dt_purinbill_f.tpv_costitem");
        keys.add("tpv_dt_purinbill_f.tpv_amt");
        keys.add("billentry");
        keys.add("billentry.Seq");
        keys.add("billentry.qty");
    }

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
        if ("submit".equals(e.getOperationKey()) && null != e.getDataEntities()) {
            DynamicObject[] bills = e.getDataEntities();
            for (int i = 0; i < bills.length; ++i) {
                DynamicObject bill = bills[i];
                submitDate(bill);
            }
        }
    }

    public void submitDate(DynamicObject e) {
        BigDecimal tpvQuantity = BigDecimal.ZERO;//卷号数量合计
        BigDecimal tpv_billablequantitys = BigDecimal.ZERO;//费用子分录数量合计
        DynamicObjectCollection materialentry = e.getDynamicObjectCollection("billentry");//商品明细
        for (DynamicObject matertrty : materialentry) {
            DynamicObjectCollection subentryentity = matertrty.getDynamicObjectCollection("tpv_dt_purinbill_j");//卷号管理
            DynamicObjectCollection tpvDtPurinbillf = matertrty.getDynamicObjectCollection("tpv_dt_purinbill_f");//费用信息
            int row = matertrty.getInt("Seq");
            BigDecimal tpvQty = matertrty.getBigDecimal("qty");//商品分录数量
            if (subentryentity.size() > 0) {
                tpvQuantity = BigDecimal.ZERO;
                for (DynamicObject subentrty : subentryentity) {
                    tpvQuantity = tpvQuantity.add(subentrty.getBigDecimal("tpv_quantity"));
                }
                if (tpvQuantity.compareTo(tpvQty) < 0) {
                    throw new KDBizException("商品明细第" + row + "行（{卷号管理}子分录【数量】之和应等于{商品信息}分录行【数量】）!");
                }
            }
        }
    }
}
