package com.itopview.wm.dt.opplugin.op;

import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.scm.common.util.DynamicObjectUtil;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author GX
 * 关联交易单-服务插件
 */
public class ReleTradeOP extends AbstractOperationServicePlugIn {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_newpurchase");
        keys.add("tpv_newselechase");
        keys.add("tpv_policynums");
        keys.add("tpv_positionnumspur");
        keys.add("tpv_openpositionpur");
        keys.add("tpv_policynumsele");
        keys.add("tpv_positionnumssale");
        keys.add("tpv_openpositionsale");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] bills = this.getDataEntities();
                if ("submit".equals(this.getOperateKey())||"audit".equals(this.getOperateKey())) {
                    for (int i = 0; i < bills.length; ++i) {
                        ExtendedDataEntity bill = bills[i];
                        onsubmitDate(bill);
                    }
                }
            }

            public void onsubmitDate(ExtendedDataEntity dataEntities) {
                DynamicObject dy = dataEntities.getDataEntity();
                String tpv_newpurchase=dy.getString("tpv_newpurchase");//新采购合同号
                String tpv_newselechase=dy.getString("tpv_newselechase");//新销售合同号
                if (!tpv_newpurchase.isEmpty()){
                    String selectFieid2 = DynamicObjectUtil.getSelectfields("tpv_dt_purchase", false);// 获取单据头所有字段标识
                    DynamicObject[] purchasebills= BusinessDataServiceHelper.load("tpv_dt_purchase", selectFieid2, new QFilter[] {
                            new QFilter("tpv_contractnumber",QCP.equals,tpv_newpurchase)});
                    if (purchasebills.length>0){
                        this.addMessage(dataEntities,"采购合同号已存在，请重新填写！");
                    }
                }if (!tpv_newselechase.isEmpty()){
                    String selectFieid2 = DynamicObjectUtil.getSelectfields("tpv_dt_salescontract", false);// 获取单据头所有字段标识
                    DynamicObject[] purchasebills= BusinessDataServiceHelper.load("tpv_dt_salescontract", selectFieid2, new QFilter[] {
                            new QFilter("tpv_contractnos",QCP.equals,tpv_newpurchase)});
                    if (purchasebills.length>0){
                        this.addMessage(dataEntities,"销售合同号已存在，请重新填写！");
                    }
                }
            }
        });
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            setPositionQty("tpv_policynums", "tpv_positionnumspur", "tpv_openpositionpur", bill);
            setPositionQty("tpv_policynumsele", "tpv_positionnumssale", "tpv_openpositionsale", bill);
            OperationServiceHelper.executeOperate("save", bill.getDataEntityType().getName(),
                    new DynamicObject[]{bill}, YEM.getOptionNoMsg());
        }
    }

    private void setPositionQty(String name, String positionnumsKey, String openpositionKey, DynamicObject bill) {
        DynamicObject tpvStrategynum = bill.getDynamicObject(name);
        if (YEM.isNotEmpty(tpvStrategynum)) {
            long id = tpvStrategynum.getLong("id");
            DataSet purchase = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_purchase", "id,tpv_contractnumber,tpv_policynumber.id policyid," +
                            "tpv_policynumber.number policynumber,tpv_materialentry,tpv_materialentry.tpv_qty qty",
                    new QFilter[]{new QFilter("tpv_contractstatusx", QCP.in, new String[]{"A", "B", "C", "D", "F", "G"})
                            .and(new QFilter("tpv_policynumber.id", QCP.in, id))}).select("policyid policyid","qty purqty","0.0 saleqty");
            DataSet salescontract = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_salescontract", "id,tpv_contractnos,tpv_strategynum.id strategyid," +
                            "tpv_strategynum.number strategynum,tpv_materialentry,tpv_materialentry.tpv_qty qty",
                    new QFilter[]{new QFilter("tpv_contractstatus", QCP.in, new String[]{"A","B","C","D","G","H"})
                            .and(new QFilter("tpv_strategynum.id", QCP.in, id))}).select("strategyid policyid","qty saleqty","0.0 purqty");
            DataSet dataSet = purchase.union(salescontract).groupBy(new String[]{"policyid"}).sum("saleqty").sum("purqty").finish();
            dataSet = RptUtil.zeroNullNums(dataSet);
            dataSet = dataSet.addField("(purqty - saleqty) qty", "qty");
            BigDecimal qty = BigDecimal.ZERO;
            if (!dataSet.isEmpty()) {
                for (Row row : dataSet) {
                    qty = row.getBigDecimal("qty").setScale(2, BigDecimal.ROUND_HALF_UP).abs();
                }
            }
            BigDecimal tpvReplylimit = BigDecimalUtils.toBigDecimal(tpvStrategynum.getBigDecimal("tpv_replylimit")).setScale(2, BigDecimal.ROUND_HALF_UP);
            bill.set(positionnumsKey,qty);
            bill.set(openpositionKey,tpvReplylimit.subtract(qty));
        } else {
            bill.set(positionnumsKey, BigDecimal.ZERO);
            bill.set(openpositionKey, BigDecimal.ZERO);
        }
    }
}
