package com.itopview.wm.dt.opplugin.op;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.dt.report.PickShipsRptQuery;
import com.itopview.wm.sys.utils.NewVersionHelper;
import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 销售发货-服务插件
 *
 * @author longh
 * @description 保存、提交、删除，反写拣配信息
 * @date 2023年9月11日
 */
public class SaleShippingPickOp extends AbstractOperationServicePlugIn {
    private static final String opValueKey = "tpv_pick";

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_materialentry");
        fieldKeys.add("tpv_dt_salepick_zy");
        fieldKeys.add("tpv_pickbill_zy");
        fieldKeys.add("tpv_fid_zy");
        fieldKeys.add("tpv_dt_salepick_in");
        fieldKeys.add("tpv_pickbill_in");
        fieldKeys.add("tpv_fid_in");
        fieldKeys.add("billentry");
        fieldKeys.add("tpv_pickbill_in");
        fieldKeys.add("tpv_fid_in");
        fieldKeys.add("tpv_directconsignmentid");
    }


    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            paramPick(bill);
        }
    }

    private void paramPick(DynamicObject bill) {
        Object pkValue = bill.getPkValue();
        String entityNumber = bill.getDynamicObjectType().getName();
        NewVersionHelper vHelper = new NewVersionHelper(entityNumber);
        if (vHelper.isChanged()) {
            return;
        }
        if (YEM.isNotEmpty(pkValue)) {
            JSONArray jsonArray = new JSONArray();
            QFilter qFilter = new QFilter("id", QCP.equals, pkValue);
            DynamicObject[] query;
            switch (entityNumber) {
                case "tpv_dt_salesshipping":
                    query = BusinessDataServiceHelper.load(entityNumber,
                            "tpv_materialentry,tpv_dt_salepick_zy,tpv_pickbill_zy,tpv_fid_zy,tpv_dt_salepick_in,tpv_pickbill_in,tpv_fid_in",
                            qFilter.toArray());
                    break;
                case "im_otheroutbill":
                    query = BusinessDataServiceHelper.load(entityNumber, "billentry,tpv_pickbill_in,tpv_fid_in,tpv_directconsignmentid",
                            qFilter.toArray());
                    break;
                case "im_transoutbill":
                case "im_transdirbill":
                    query = BusinessDataServiceHelper.load(entityNumber, "billentry,tpv_pickbill_in,tpv_fid_in",
                            qFilter.toArray());
                    break;
                case "tpv_dt_noconfiguration":
                    query = BusinessDataServiceHelper.load(entityNumber,
                            "tpv_sonentryentity,tpv_pickbill_in,tpv_fid_in",
                            qFilter.toArray());
                    break;
                case "tpv_dt_relevancedealgs":
                    query = BusinessDataServiceHelper.load(entityNumber,
                            "tpv_dt_goodstransfe_zh,tpv_dt_transferdist_ph,tpv_pickbill,tpv_fids",
                            qFilter.toArray());
                    break;
                default:
                    return;
            }
            for (DynamicObject obj : query) {
                addPickBackBill(jsonArray, obj);
            }
            OperateOption option = this.getOption();
            option.setVariableValue(opValueKey, jsonArray.toJSONString());
        }
    }

    private void addPickBackBill(JSONArray jsonArray, DynamicObject bill) {
        String entityNumber = bill.getDynamicObjectType().getName();
        switch (entityNumber) {
            case "tpv_dt_salesshipping":
                DynamicObjectCollection entry = bill.getDynamicObjectCollection("tpv_materialentry");
                for (DynamicObject row : entry) {
                    DynamicObjectCollection tpv_dt_salepick_zy = row.getDynamicObjectCollection("tpv_dt_salepick_zy");
                    for (DynamicObject bill_s : tpv_dt_salepick_zy) {
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.put("tpv_pickbill1", bill_s.getString("tpv_pickbill_zy"));
                        jsonObject.put("tpv_fid1", bill_s.getLong("tpv_fid_zy"));
                        jsonArray.add(jsonObject);
                    }
                    DynamicObjectCollection tpv_dt_salepick_in = row.getDynamicObjectCollection("tpv_dt_salepick_in");
                    for (DynamicObject bill_s : tpv_dt_salepick_in) {
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.put("tpv_pickbill1", bill_s.getString("tpv_pickbill_in"));
                        jsonObject.put("tpv_fid1", bill_s.getLong("tpv_fid_in"));
                        jsonArray.add(jsonObject);
                    }
                }
                break;
            case "im_otheroutbill":
                DynamicObjectCollection billentrys = bill.getDynamicObjectCollection("billentry");
                for (DynamicObject row : billentrys) {
                    JSONObject jsonObject = new JSONObject();
//                    long tpv_directconsignmentid = row.getLong("tpv_directconsignmentid");
//                    if (tpv_directconsignmentid <= 0) {
                        jsonObject.put("tpv_pickbill1", row.getString("tpv_pickbill_in"));
                        jsonObject.put("tpv_fid1", row.getLong("tpv_fid_in"));
                        jsonArray.add(jsonObject);
//                    }
                }
                break;
            case "im_transoutbill":
            case "im_transdirbill":
                DynamicObjectCollection billentry = bill.getDynamicObjectCollection("billentry");
                for (DynamicObject row : billentry) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("tpv_pickbill1", row.getString("tpv_pickbill_in"));
                    jsonObject.put("tpv_fid1", row.getLong("tpv_fid_in"));
                    jsonArray.add(jsonObject);
                }
                break;
            case "tpv_dt_noconfiguration":
                DynamicObjectCollection tpv_dt_salepick_in = bill.getDynamicObjectCollection("tpv_sonentryentity");
                for (DynamicObject row : tpv_dt_salepick_in) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("tpv_pickbill1", row.getString("tpv_pickbill_in"));
                    jsonObject.put("tpv_fid1", row.getLong("tpv_fid_in"));
                    jsonArray.add(jsonObject);
                }
                break;
            case "tpv_dt_relevancedealgs":
                if (null != bill){
                    DynamicObjectCollection tpvDtGoodstransfeZh = bill.getDynamicObjectCollection("tpv_dt_goodstransfe_zh");
                    for (DynamicObject row : tpvDtGoodstransfeZh) {
                        DynamicObjectCollection tpv_DtTransferdistPh = row.getDynamicObjectCollection("tpv_dt_transferdist_ph");
                        for (DynamicObject bill_s : tpv_DtTransferdistPh) {
                            JSONObject jsonObject = new JSONObject();
                            jsonObject.put("tpv_pickbill1", bill_s.getString("tpv_pickbill"));
                            jsonObject.put("tpv_fid1", bill_s.getLong("tpv_fids"));
                            jsonArray.add(jsonObject);
                        }
                    }
                }
                break;
            default:
                return;
        }
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            backPick(bill);
        }
    }

    private void backPick(DynamicObject bill) {
        OperateOption option = this.getOption();
        JSONArray jsonArray = new JSONArray();
        if (option.containsVariable(opValueKey)) {
            jsonArray = JSON.parseArray(option.getVariableValue(opValueKey));
        }
        if (!bill.getDynamicObjectType().getName().equals("tpv_dt_relevancedealgs")){
            addPickBackBill(jsonArray, bill);
        }
        Map<String, Map<Long, List<DynamicObject>>> pickMap = PickShipsRptQuery.pickDataCols();
        Map<String, List<JSONObject>> backMap = jsonArray.stream().map(x -> (JSONObject) x)
                .filter(x -> YEM.isNotEmpty(x.getString("tpv_pickbill1")))
                .collect(Collectors.groupingBy(x -> x.getString("tpv_pickbill1")));
        for (String billKey : backMap.keySet()) {
            List<Long> ids = backMap.get(billKey).stream().map(x -> x.getLong("tpv_fid1")).distinct()
                    .collect(Collectors.toList());
            String entryKey;
            String pickKey;
            String packnumKey;
            String subEntryKey = "";
            String subPickKey = "";
            boolean von = false;
            switch (billKey) {
                case "tpv_dt_purchase":
                    entryKey = "tpv_materialentry";
                    pickKey = "tpv_pickedquantity";
                    packnumKey = "tpv_ispacknumqty";
                    break;
                case "tpv_dt_freigroambill":
                    entryKey = "tpv_materialentry";
                    pickKey = "tpv_direcroomtqty";
                    packnumKey = "tpv_ispacknumqty";
                    break;
                case "im_purinbill":
                    entryKey = "billentry";
                    pickKey = "tpv_quantitypicked";
                    subEntryKey = "tpv_dt_purinbill_j";
                    subPickKey = "tpv_quantitypickeds";
                    packnumKey = "tpv_ispacknumqty";
                    von = true;
                    break;
                case "im_otherinbill":
                    entryKey = "billentry";
                    pickKey = "tpv_quantitypicked";
                    packnumKey = "tpv_ispacknumqty";
                    break;
                case "im_transinbill":
                    entryKey = "billentry";
                    pickKey = "tpv_quantitypicked";
                    packnumKey = "tpv_ispacknumqty";
                    break;
                case "im_transdirbill":
                    entryKey = "billentry";
                    pickKey = "tpv_quantitypicked";
                    packnumKey = "tpv_ispacknumqty";
                    break;
                default:
                    continue;
            }
            String selectFields = "id," + entryKey + "," + pickKey + "," + packnumKey;
            if (von) {
                selectFields += "," + subEntryKey + "," + subPickKey;
            }
            QFilter qFilter = new QFilter("id", QCP.in, ids);
            DynamicObject[] bills = BusinessDataServiceHelper.load(billKey, selectFields, qFilter.toArray());
            for (DynamicObject data : bills) {
                DynamicObjectCollection entry = data.getDynamicObjectCollection(entryKey);
                for (DynamicObject row : entry) {
                    BigDecimal tpv_pickqty_s = BigDecimal.ZERO;// 已拣配数量
                    BigDecimal tpv_packnum_s = BigDecimal.ZERO;// 已拣配包装数量
                    long entryId = row.getLong("id");
                    Map<Long, List<DynamicObject>> map = pickMap.get(billKey);
                    if (von) {
                        DynamicObjectCollection vonEntry = row.getDynamicObjectCollection(subEntryKey);
                        for (DynamicObject subRow : vonEntry) {
                            long tpv_vid = subRow.getLong("id");
                            BigDecimal tpv_pickqty_sub = BigDecimal.ZERO;// 已拣配数量-卷号分录
                            if (map != null) {
                                List<DynamicObject> list = map.get(entryId);
                                if (list != null) {
                                    for (DynamicObject obj : list) {
                                        long tpv_vid1 = obj.getLong("tpv_vid");
                                        if (tpv_vid == tpv_vid1) {
                                            tpv_pickqty_sub = BigDecimalUtils.add(tpv_pickqty_sub,
                                                    obj.getBigDecimal("tpv_pickqty_s"), 10);
                                        }
                                    }
                                }
                            }
                            subRow.set(subPickKey, tpv_pickqty_sub);
                        }
                    }
                    if (map != null) {
                        List<DynamicObject> list = map.get(entryId);
                        if (list != null) {
                            for (DynamicObject obj : list) {
                                tpv_pickqty_s = BigDecimalUtils.add(tpv_pickqty_s, obj.getBigDecimal("tpv_pickqty_s"),
                                        10);
                                tpv_packnum_s = BigDecimalUtils.add(tpv_packnum_s, obj.getBigDecimal("tpv_packnum_s"),
                                        10);
                            }
                        }
                    }
                    row.set(pickKey, tpv_pickqty_s);
                    row.set(packnumKey, tpv_packnum_s);
                }
            }
            if (bills.length > 0) {
                SaveServiceHelper.save(bills);
            }
        }
    }
}