package com.itopview.wm.dt.opplugin.op;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 销售发货删除货物批次确认价格校验 -服务插件
 * @Author LinRui
 * @Date 2023/9/22 19:01
 */
public class SaleShippingPriceOp extends AbstractOperationServicePlugIn {

    private static final String opValueKey = "tpv_cargobatch";

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_materialentry");  //分录标识
        fieldKeys.add("tpv_cargobatchqty");  //价格发货确认数量
        fieldKeys.add("tpv_qty");  //数量
        fieldKeys.add("tpv_document");  //货物批次选择单据
        fieldKeys.add("tpv_selectiondocu");  //货物批次选择单号
        fieldKeys.add("tpv_cobillid");  //核心单据ID
        fieldKeys.add("tpv_coentryid");  //核心单据分录ID
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        e.getOperationKey();
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            paramSalePrice(bill);
        }
    }

    private void paramSalePrice(DynamicObject bill) {
        Object pkValue = bill.getPkValue();
        if (YEM.isNotEmpty(pkValue)) {
            JSONObject jsonObject = new JSONObject();
            if (QueryServiceHelper.exists("tpv_dt_salesshipping", bill.getPkValue())) {
                addPriceBackBill(jsonObject, bill); //记录单据界面入库前的临时数据
            }
            OperateOption option = this.getOption();
            option.setVariableValue(opValueKey, jsonObject.toJSONString());
        }
    }

    private void addPriceBackBill(JSONObject jsonObject, DynamicObject bill) {
        DynamicObject tpvDtSalesshipping = BusinessDataServiceHelper.loadSingle(bill.getPkValue(), "tpv_dt_salesshipping");
        List<Long> entryId = new ArrayList<>();
        DynamicObjectCollection tpvMaterialentry = tpvDtSalesshipping.getDynamicObjectCollection("tpv_materialentry");
        for (DynamicObject dynamicObject : tpvMaterialentry) {
            String tpvDocument = dynamicObject.getString("tpv_document"); //货物批次选择单据

            if (null != tpvDocument && "B".equals(tpvDocument)) {
//                JSONObject jsonObject = new JSONObject();
//                jsonObject.put("tpv_selectiondocu", dynamicObject.get("tpv_selectiondocu")); //货物批次选择单据号
//                jsonObject.put("tpv_cargobatchqty", dynamicObject.get("tpv_cargobatchqty")); //价格确认货物批次数量
//                jsonObject.put("tpv_coentryid", dynamicObject.get("tpv_coentryid")); //核心单据分录ID
//                jsonObject.put("entryid",dynamicObject.getPkValue()); //分录ID
                String tpvSelectiondocu = dynamicObject.getString("tpv_selectiondocu");
                entryId.add((Long) dynamicObject.get("tpv_coentryid"));
                if (jsonObject.size() > 0) {
                    if (jsonObject.entrySet().contains(tpvSelectiondocu)) {
                        entryId.add((Long) dynamicObject.get("tpv_coentryid"));
                    }
                }
                jsonObject.put(tpvSelectiondocu, entryId); //货物批次选择单据号
                //jsonArray.add(jsonObject);
            }
        }
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            backSalePrice(bill);  //反写销售价格确认单
        }
    }

    private void backSalePrice(DynamicObject bill) {
        OperateOption option = this.getOption();
        JSONObject jsonObject = new JSONObject();
        if (option.containsVariable(opValueKey)) {
            jsonObject = JSON.parseObject(option.getVariableValue(opValueKey));
        }
        if (QueryServiceHelper.exists("tpv_dt_salesshipping", bill.getPkValue())) {
            addPriceBackBill(jsonObject, bill);//记录单据界面入库后的数据
        }
        Map<String, List<Long>> entrypriceBillnos = new HashMap<>();
        //List<String> entrypriceBillno = new ArrayList<>();  //分录批次价格单号
        List<Long> entryid = new ArrayList<>();
        jsonObject.entrySet().stream().forEach(x -> {  //存入分录行数据，货物批次是价格确认单单号、及核心单据分录行id或来源分录行id
            String key = x.getKey();
            if (!entrypriceBillnos.containsKey(key)) {
                List<Long> value = (List<Long>) x.getValue();
                for (Long aLong : value) {
                    entryid.add(aLong);
                    entrypriceBillnos.put(key, entryid);
                }
            }
        });

        Map<String, Map<Long, BigDecimal>> salePriceBillMap = new HashMap<>(); //利用hashmap键不能重复，覆盖累加赋值

        DynamicObject[] saleShippPriceDatas = BusinessDataServiceHelper.load("tpv_dt_salesshipping",
                "id,billno,tpv_materialentry,tpv_materialentry.tpv_srcentryid,tpv_materialentry.tpv_cargobatchqty,tpv_materialentry.tpv_selectiondocu,tpv_materialentry.tpv_coentryid",
                new QFilter[]{new QFilter("tpv_materialentry.tpv_selectiondocu", QCP.in, entrypriceBillnos.keySet())});
        //遍历所有关联销售价格确认的  单销售发货单分录行数据
        List<String> billnoList = new ArrayList<>();
        if (saleShippPriceDatas.length > 0) {
            for (DynamicObject saleShippPriceData : saleShippPriceDatas) {
                String billno = saleShippPriceData.getString("billno");
                billnoList.add(billno);
                DynamicObjectCollection tpvMaterialentry = saleShippPriceData.getDynamicObjectCollection("tpv_materialentry");
                for (DynamicObject dynamicObject : tpvMaterialentry) {
                    long tpvSrcentryid = dynamicObject.getLong("tpv_srcentryid");  //来源分录ID
                    long tpvCoentryid = dynamicObject.getLong("tpv_coentryid");  //核心单分录ID
                    BigDecimal tpvCargobatchqty = dynamicObject.getBigDecimal("tpv_cargobatchqty"); //价格确认货物批次数量
                    String tpvSelectiondocu = dynamicObject.getString("tpv_selectiondocu");  //货物批次单号
                    //long tpvSelectiondocuid = dynamicObject.getLong("tpv_selectiondocuid");  ////货物批次单据ID
                    if (entrypriceBillnos.keySet().contains(tpvSelectiondocu)) {
                        if (!salePriceBillMap.containsKey(tpvSelectiondocu)) {
                            Map<Long, BigDecimal> backPriceMap = new HashMap<>();
                            backPriceMap.put(tpvCoentryid, tpvCargobatchqty);
                            salePriceBillMap.put(tpvSelectiondocu, backPriceMap);
                        } else {
                            if (salePriceBillMap.containsKey(tpvSelectiondocu)) {
                                salePriceBillMap.entrySet().stream().forEach(x -> {
                                    if (x.getKey().equals(tpvSelectiondocu)) {
                                        if (!x.getValue().containsKey(tpvCoentryid)) {
                                            x.getValue().put(tpvCoentryid, tpvCargobatchqty);
                                        } else {
                                            BigDecimal add = x.getValue().get(tpvCoentryid).add(tpvCargobatchqty);
                                            x.getValue().put(tpvCoentryid, add);
                                        }
                                    }
                                });
                            }
                        }
                    }
                }
            }
        } else {
            Map<String, List<Map.Entry<String, List<Long>>>> listMap = entrypriceBillnos.entrySet().stream().collect(Collectors.groupingBy(x -> x.getKey()));
            for (String x : listMap.keySet()) {
                if (!salePriceBillMap.containsKey(x)) {
                    Map<Long, BigDecimal> backPriceMap = new HashMap<>();
                    for (Map.Entry<String, List<Long>> stringListEntry : listMap.get(x)) {
                        for (Long aLong : stringListEntry.getValue()) {
                            backPriceMap.put(aLong, BigDecimal.ZERO);
                        }
                        salePriceBillMap.put(x, backPriceMap);
                    }
                }
            }
        }

        for (String s : entrypriceBillnos.keySet()) {
            if (!billnoList.contains(s)) {
                if (!salePriceBillMap.containsKey(s)) {
                    Map<Long, BigDecimal> backPriceMap = new HashMap<>();
                    for (Long aLong : entrypriceBillnos.get(s)) {
                        backPriceMap.put(aLong, BigDecimal.ZERO);
                    }
                    salePriceBillMap.put(s, backPriceMap);
                }
            }
        }

        Set<String> priceBillId = salePriceBillMap.keySet();
        DynamicObject[] tpvDtSalepricebills = BusinessDataServiceHelper.load("tpv_dt_salepricebill",
                "id,billno,tpv_materialentry,tpv_materialentry.tpv_srcentryid,tpv_materialentry.tpv_cargobatchqty",
                new QFilter[]{new QFilter("billno", QCP.in, priceBillId)});
        if (null != tpvDtSalepricebills) {
            for (DynamicObject tpvDtSalepricebill : tpvDtSalepricebills) {
                DynamicObjectCollection tpvMaterialentry = tpvDtSalepricebill.getDynamicObjectCollection("tpv_materialentry");
                String billno = tpvDtSalepricebill.getString("billno");
                for (DynamicObject dynamicObject : tpvMaterialentry) {
                    if (salePriceBillMap.containsKey(billno)) {
                        Long tpvSrcentryid = dynamicObject.getLong("tpv_srcentryid");
                        salePriceBillMap.get(billno).entrySet().stream().filter(x -> x.getKey() != null).forEach(x -> {
                            if (x.getKey().equals(tpvSrcentryid)) {
                                BigDecimal decimal = salePriceBillMap.get(billno).get(tpvSrcentryid);
                                dynamicObject.set("tpv_cargobatchqty", decimal);
                            }
                        });
                    }
                }
            }
            SaveServiceHelper.save(tpvDtSalepricebills);
        }

    }
}
