package com.itopview.wm.dt.opplugin.op;

import com.itopview.wm.utils.DynamicObjectUtil;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.exception.KDBizException;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.sdk.plugin.Plugin;

import java.math.BigDecimal;
import java.util.List;

/**
 * @ClassName StorageSubmitOp
 * @Description 保存、提交校验采购合同数量 * 溢装率 小于 入库数量
 * @Author yhj
 * @Date 2023/9/18 15:11
 * @Version 1.0
 */
public class StorageSubmitOp extends AbstractOperationServicePlugIn implements Plugin {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_overfillrate");
        keys.add("tpv_contractqty");
        keys.add("tpv_selectsum");
        keys.add("tpv_contract");
    }
    //自定义校验器
    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] dataEntities = this.getDataEntities();
                if ("save".equals(this.getOperateKey()) && null != e.getDataEntities()) {
                    submitDate(dataEntities);
                }
                if ("submit".equals(this.getOperateKey()) && null != e.getDataEntities()) {
                    submitDate(dataEntities);
                    //saveDate(dataEntities);
                    //onSumbit(dataEntities);
                }
            }

            private void onSumbit(ExtendedDataEntity[] dataEntities) {
                for (ExtendedDataEntity data : dataEntities) {
                    DynamicObject dy = data.getDataEntity();
                    BigDecimal contractqty = BigDecimal.ZERO;//合同数量
                    BigDecimal overfillrate = BigDecimal.ZERO;//溢装率
                    BigDecimal one = BigDecimal.ONE;//1
                    BigDecimal tpv_billablequantitys = BigDecimal.ZERO;//费用子分录数量
                    BigDecimal million = BigDecimal.valueOf(100);//100
                    BigDecimal selectsum = BigDecimal.ZERO;//入库数量
                    Long conbillid = dy.getLong("tpv_corebillid");//合同id
                    String billno =dy.getString("billno");//单据编号
                    DynamicObjectCollection materialentry=dy.getDynamicObjectCollection("billentry");//商品分录
                    QFilter[] qFilter = new QFilter[]{new QFilter("tpv_dt_purchase.id", QCP.equals, conbillid)};
                    String selectfieldsFast = DynamicObjectUtil.getSelectfieldsFast("tpv_dt_purchase", "tpv_materialentry");
                    DynamicObject[] queryHW = BusinessDataServiceHelper.load("tpv_dt_purchase", selectfieldsFast, qFilter);//采购合同
                    if (queryHW != null) {
                        for (DynamicObject obj : queryHW) {
                            overfillrate = obj.getBigDecimal("tpv_overfillrate");//溢装率
                        }
                    }
                    for (DynamicObject materia:materialentry){
                        DynamicObjectCollection tpv_dt_freigroambill_f = materia.getDynamicObjectCollection("tpv_dt_purinbill_f");//费用信息
                        int row = materia.getInt("Seq");
                        BigDecimal qty = materia.getBigDecimal("qty");//商品分录数量
                        if (tpv_dt_freigroambill_f.size() > 0) {
                            for (DynamicObject freigroambill : tpv_dt_freigroambill_f) {
                                tpv_billablequantitys = freigroambill.getBigDecimal("tpv_chargequantity");
                            }
                            BigDecimal dive = overfillrate.divide(million);
                            BigDecimal add = dive.add(one);
                            BigDecimal sum = qty.multiply(add);
//                            if (tpv_billablequantitys.compareTo(sum) > 0) {
//                                this.addMessage(data, "商品明细第" + row + "行（{费用信息}子分录【计费数量】之和不能大于{商品分录}【数量】*（1+溢装率）!");
//                            }
                        }
                    }
                }
            }

            //提交校验
            public void submitDate(ExtendedDataEntity[] dataEntities) {
//                ExtendedDataEntity bill=null;
//                for (int i = 0; i < dataEntities.length; ++i) {
//                     bill = dataEntities[i];
//                }
                BigDecimal sumqty = BigDecimal.ZERO;//数量汇总
                BigDecimal sumpurqty=BigDecimal.ZERO;//合同分录未流转数量汇总
                BigDecimal sumflowstoc=BigDecimal.ZERO;//已入库加已流转数量汇总
                for (ExtendedDataEntity data : dataEntities) {
                    DynamicObject dy = data.getDataEntity();
                    BigDecimal contractqty = BigDecimal.ZERO;//合同数量
                    BigDecimal overfillrate = BigDecimal.ZERO;//溢装率
                    BigDecimal one = BigDecimal.ONE;//1
                    BigDecimal tpv_billablequantitys = BigDecimal.ZERO;//费用子分录数量
                    BigDecimal million = BigDecimal.valueOf(100);//100
                    BigDecimal selectsum = BigDecimal.ZERO;//入库数量
                    Long conbillid = dy.getLong("tpv_corebillid");//合同id
                    String billno =dy.getString("billno");//单据编号
                    DynamicObjectCollection materialentry=dy.getDynamicObjectCollection("billentry");//商品分录

                    QFilter[] qFilter = new QFilter[]{new QFilter("tpv_dt_purchase.id", QCP.equals, conbillid)};
                    String selectfieldsFast = DynamicObjectUtil.getSelectfieldsFast("tpv_dt_purchase", "tpv_materialentry");
                    DynamicObject[] queryHW = BusinessDataServiceHelper.load("tpv_dt_purchase", selectfieldsFast, qFilter);//采购合同
                    QFilter[] qFilters = new QFilter[]{new QFilter("tpv_contract", QCP.equals, conbillid)
                            ,new QFilter("billno", QCP.not_equals, billno)};
                    String selectfields = DynamicObjectUtil.getSelectfieldsFast("im_purinbill", "billentry");
                    DynamicObject[] queryRK = BusinessDataServiceHelper.load("im_purinbill", selectfields, qFilters);//采购入库
                    if (queryHW != null) {
                        for (DynamicObject obj : queryHW) {
                            overfillrate = obj.getBigDecimal("tpv_overfillrate");//溢装率
                            DynamicObjectCollection materialentryus=obj.getDynamicObjectCollection("tpv_materialentry");//商品分录
                            for (DynamicObject materi:materialentryus){
                                contractqty=contractqty.add(materi.getBigDecimal("tpv_qty"));//合同数量
                                BigDecimal qty=materi.getBigDecimal("tpv_qty");//合同分录数量
                                BigDecimal flowedquantity=materi.getBigDecimal("tpv_flowedquantity");//合同分录已流转数量
                                BigDecimal tpv_stockednum=materi.getBigDecimal("tpv_stockednum");//合同分录已入库数量
                                sumflowstoc=sumflowstoc.add(flowedquantity.add(tpv_stockednum)).setScale(4,BigDecimal.ROUND_HALF_UP);
                                sumpurqty=sumpurqty.add(qty.subtract(flowedquantity).subtract(tpv_stockednum));
                            }
                        }
                    }
                    for (DynamicObject materia:materialentry){
                        sumqty=sumqty.add(materia.getBigDecimal("qty"));
                        DynamicObjectCollection tpv_dt_freigroambill_f = materia.getDynamicObjectCollection("tpv_dt_purinbill_f");//费用信息
                        int row = materia.getInt("Seq");
//                        BigDecimal qty = materia.getBigDecimal("qty");//商品分录数量
//                        if (tpv_dt_freigroambill_f.size() > 0) {
//                            for (DynamicObject freigroambill : tpv_dt_freigroambill_f) {
//                                tpv_billablequantitys = freigroambill.getBigDecimal("tpv_chargequantity");
//                            }
//                            BigDecimal dive = overfillrate.divide(million);
//                            BigDecimal add = dive.add(one);
//                            BigDecimal sum = qty.multiply(add);
//                            if (tpv_billablequantitys.compareTo(sum) > 0) {
//                                this.addMessage(data, "商品明细第" + row + "行（{费用信息}子分录【计费数量】之和不能大于{商品分录}【数量】*（1+溢装率）!");
//                            }
//                        }
                    }
                    if (queryRK != null) {
                        for (DynamicObject objs : queryRK) {
                            DynamicObjectCollection materialentrys=objs.getDynamicObjectCollection("billentry");//商品分录
                            for (DynamicObject materias:materialentrys){
                                selectsum = selectsum.add(materias.getBigDecimal("qty"));//入库数量
                            }
                        }
                    }
                    BigDecimal dive = overfillrate.divide(million);
                    BigDecimal add = dive.add(one);
                    BigDecimal sum = contractqty.multiply(add);
                    if (sum.compareTo(selectsum.add(sumqty)) == -1) {
                        this.addErrorMessage(data, "当前累计入库已达" + selectsum.add(sumqty) + ",大于合同【数量】*（1+溢装率）!");
                    }
                }
            }

            public void saveDate(ExtendedDataEntity[] dataEntities) {
                BigDecimal sumqty = BigDecimal.ZERO;//数量汇总
                BigDecimal sumpurqty=BigDecimal.ZERO;//合同分录未流转数量汇总
                BigDecimal sumflowstoc=BigDecimal.ZERO;//已入库加已流转数量汇总
                for (ExtendedDataEntity data : dataEntities) {
                    DynamicObject dy = data.getDataEntity();
                    Long conbillid = dy.getLong("tpv_corebillid");//合同id
                    DynamicObjectCollection materialentry=dy.getDynamicObjectCollection("billentry");//商品分录
                    for (DynamicObject materia:materialentry){
                        sumqty=sumqty.add(materia.getBigDecimal("qty"));
                    }
                    QFilter[] qFilter = new QFilter[]{new QFilter("tpv_dt_purchase.id", QCP.equals, conbillid)};
                    String selectfieldsFast = DynamicObjectUtil.getSelectfieldsFast("tpv_dt_purchase", "tpv_materialentry");
                    DynamicObject[] queryHW = BusinessDataServiceHelper.load("tpv_dt_purchase", selectfieldsFast, qFilter);//采购合同
                    if (queryHW != null) {
                        for (DynamicObject obj : queryHW) {
                            DynamicObjectCollection materialentryus=obj.getDynamicObjectCollection("tpv_materialentry");//商品分录
                            for (DynamicObject materi:materialentryus){
                                BigDecimal qty=materi.getBigDecimal("tpv_qty");//合同分录数量
                                BigDecimal flowedquantity=materi.getBigDecimal("tpv_flowedquantity");//合同分录已流转数量
                                BigDecimal tpv_stockednum=materi.getBigDecimal("tpv_stockednum");//合同分录已入库数量
                                sumflowstoc=sumflowstoc.add(flowedquantity.add(tpv_stockednum)).setScale(4,BigDecimal.ROUND_HALF_UP);
                                sumpurqty=sumpurqty.add(qty.subtract(flowedquantity).subtract(tpv_stockednum));
                            }
                        }
                        if (sumpurqty.compareTo(sumqty)<0){
                            this.addWarningMessage(data,"当前累计流转/入库已达"+sumflowstoc.add(sumqty)+"吨，大于原货物流转单数量，是否继续？");
                        }
                    }
                }
            }
        });
    }
}

