package com.itopview.wm.dt.opplugin.validator;

import com.itopview.wm.sys.utils.NewVersionHelper;
import com.itopview.wm.utils.BigDecimalUtils;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;

import java.math.BigDecimal;
import java.util.List;

/**
 * @Description: 销售合同提交校检插件
 * @Auther: zbh
 */
public class SaleChaseSubmitValidator extends AbstractOperationServicePlugIn implements Plugin {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_customer");
        keys.add("tpv_approvaltogether");
        keys.add("tpv_supplier_apt");
        keys.add("tpv_documenttypes");
        keys.add("tpv_supplychain");
        keys.add("tpv_contractnos");
        keys.add("tpv_paymentplans");
        keys.add("tpv_proportion");
        keys.add("tpv_thirdparty");
        keys.add("tpv_contractstatus");
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        String key = e.getOperationKey();
        DynamicObject[] entities = e.getDataEntities();
        switch (key) {
            case "submit":
                submit(entities,key);
                break;
            case "unsubmit":
                submit(entities,key);
                break;
            case "audit":
                submit(entities,key);
                break;
            case "unaudit":
                submit(entities,key);
                break;
        }
    }

    private void submit(DynamicObject[] entities, String key) {
        for (DynamicObject entity : entities) {
            if ("submit".equals(key)){
                entity.set("tpv_contractstatus","B");
                SaveServiceHelper.save(new DynamicObject[]{entity});
            }else if ("audit".equals(key)){
                entity.set("tpv_contractstatus","D");
                SaveServiceHelper.save(new DynamicObject[]{entity});
            }else if ("unsubmit".equals(key)||"unaudit".equals(key)){
                entity.set("tpv_contractstatus","A");
                SaveServiceHelper.save(new DynamicObject[]{entity});
            }
        }
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        DynamicObject[] entities = e.getDataEntities();
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] bills = this.getDataEntities();
                String key = this.getOperateKey();
                for (int i = 0; i < bills.length; i++) {
                    switch (key) {
                        case "submit":
                            submitSupplychain(bills[i], bills);
                            subThirdparty(bills[i], bills);
                            proportionSub(bills[i], bills);
                            cutomerNameSup(bills[i], bills);
                            break;
                        case "audit":
                            break;
                        default:
                            break;
                    }
                }
            }

            /**
             * 收款计划分录的比例之和是否为100
             *
             * @param bill
             * @param bills
             */
            private void proportionSub(ExtendedDataEntity bill, ExtendedDataEntity[] bills) {
                DynamicObject dataEntity = bill.getDataEntity();
                DynamicObjectCollection tpvPaymentplans = dataEntity.getDynamicObjectCollection("tpv_paymentplans");
                BigDecimal decimal = BigDecimal.ZERO;
                if (tpvPaymentplans.size() == 0) {
                    this.addMessage(bill, "收款计划“比例(%)”不允许为空！");
                }
                for (DynamicObject tpvPaymentplan : tpvPaymentplans) {
                    BigDecimal proportion = tpvPaymentplan.getBigDecimal("tpv_proportion");
                    decimal = decimal.add(proportion);
                }
                if (decimal.compareTo(BigDecimalUtils.one_handred) != 0) {
                    this.addMessage(bill, "收款计划“比例(%)”必须等于100");
                }
            }

            /*
           {合并审批-分录}存在任意一行的【供应商】与本《销售合同》【客户】重名时，单据不允许提交，阻止语“采销两端存在同一客商，不允许同时提交审批，请检查。”；
            */
            private void cutomerNameSup(ExtendedDataEntity extendedDataEntity, ExtendedDataEntity[] operateKey) {
                String entityNumber = this.getEntityKey();
                NewVersionHelper vHelper = new NewVersionHelper(entityNumber);
                if (vHelper.isChanged()) {
                    return;
                }
                DynamicObject dataEntity = extendedDataEntity.getDataEntity();
                DynamicObject customer = dataEntity.getDynamicObject("tpv_customer");
                if (customer == null) {
                    return;
                }
                String cusname = customer.getString("name");
                DynamicObjectCollection approvaltogether = dataEntity.getDynamicObjectCollection("tpv_approvaltogether");
                for (DynamicObject approval : approvaltogether) {
                    String supname = "";
                    DynamicObject supplier = approval.getDynamicObject("tpv_supplier_apt");
                    if (supplier != null) {
                        supname = supplier.getString("name");
                    }
                    if (cusname.equals(supname)) {
                        this.addMessage(extendedDataEntity, "采销两端存在同一客商，不允许同时提交审批，请检查！");
                    }
                }
            }

            /*
            是否第三方销售为"是"时，校验《第三方客户备案》中是否存在当前【部门】对本【客户】的生效状态的单据，如无，则不允许提交，
             */
            private void subThirdparty(ExtendedDataEntity dataEntities, ExtendedDataEntity[] bills) {
                DynamicObject dataEntity = dataEntities.getDataEntity();
                Boolean thirdparty = dataEntity.getBoolean("tpv_thirdparty"); //是否第三方销售
                if (thirdparty) {
                    DynamicObject customer = dataEntity.getDynamicObject("tpv_customer");//客户
                    if (customer == null) {
                        return;
                    }
                    Object id = customer.get("id");
                    DynamicObject[] load = BusinessDataServiceHelper.load("tpv_cm_thirdclirecor", "", new QFilter[]{new QFilter("billstatus", QCP.equals, "C"), new QFilter("tpv_customer.id", QCP.equals, id)});
                    if (0 == load.length) {
                        this.addMessage(dataEntities, "当前客户在贵部门未进行第三方客户备案，请先申请备案。");
                    }
                }
            }

            /*
             【是否供应链金融】=“是”时，校验《链捷贷客户合作审批单》中是否存在当前【客户】的生效状态的单据，如无，则不允许提交，
             */
            private void submitSupplychain(ExtendedDataEntity dataEntities, ExtendedDataEntity[] bills) {
                DynamicObject dataEntity = dataEntities.getDataEntity();
                Boolean supplychain = dataEntity.getBoolean("tpv_supplychain");//是否供应链金融
                if (supplychain) {
                    DynamicObject customer = dataEntity.getDynamicObject("tpv_customer");//客户
                    if (customer == null) {
                        return;
                    }
                    Object id = customer.get("id");
                    DynamicObject[] load = BusinessDataServiceHelper.load("tpv_dt_chaijieloancus", "billstatus", new QFilter[]{new QFilter("billstatus", QCP.equals, 'C'), new QFilter("tpv_customer.id", QCP.equals, id)});
                    if (0 == load.length) {
                        this.addMessage(dataEntities, "供应链金融业务下，客户必须先通过《链捷贷客户合作审批单》备案。");
                    }
                }
            }
        });
    }
}
