package com.itopview.wm.dt.report;

import kd.bos.algo.*;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

import static kd.fi.er.report.query.TripOrderRpt.algoKey;

/**
 * @author LiuHB
 * @version 1.0
 */
public class CushisdetailsaPlugin extends AbstractReportListDataPlugin {

    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        DynamicObject dd = (DynamicObject) o; //查询数据保存在o中
        String string = dd.getString("tpv_contractnos");
        QFilter QFilter = new QFilter("tpv_contractnos" ,QCP.equals, string);
        String selelctFiles =
                        "tpv_materialentry.id  tpv_intercode," +
                        "tpv_contractnos  tpv_salescontnum," +
                        "tpv_materialentry.tpv_prodname    tpv_brand," +
                        "tpv_materialentry.tpv_material.name  tpv_goodsname," +
                        "tpv_materialentry.tpv_qty  tpv_quantgoods," +
                        "tpv_materialentry.tpv_taxprice  tpv_pricegoods," +
                        "tpv_materialentry.tpv_taxpramount  tpv_totalgoods," +
                        "tpv_materialentry.tpv_unit  tpv_unita," +
                        "tpv_bizdate   tpv_creationdate," +
                        "tpv_materialentry.tpv_shippedquantity  tpv_shipmentq,"+
                        "tpv_materialentry.tpv_taxpramount   tpv_totalshprices"
                        ;
        DataSet dataSet = QueryServiceHelper.queryDataSet(this.getClass().getName(), "tpv_dt_salescontract", selelctFiles,
                QFilter.toArray(), null);
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(dataSet);//转化
        List<Field> fields = new ArrayList<>();
        List<Object[]> allrows = new ArrayList<>();
        fields.add(new Field("tpv_intercode", DataType.StringType));// 销售合同商品明细内码(主键)
        fields.add(new Field("tpv_salescontnum", DataType.StringType));// 销售合同号
        fields.add(new Field("tpv_brand", DataType.StringType));// 品牌
        fields.add(new Field("tpv_goodsname", DataType.StringType));// 货物名称
        fields.add(new Field("tpv_quantgoods", DataType.BigDecimalType));// 货物数量
        fields.add(new Field("tpv_pricegoods", DataType.BigDecimalType));// 货物单价
        fields.add(new Field("tpv_totalgoods", DataType.BigDecimalType));// 货物总价
        fields.add(new Field("tpv_unita", DataType.StringType));// 计量单位
        fields.add(new Field("tpv_creationdate", DataType.DateType)); //创建日期
        fields.add(new Field("tpv_shipmentq", DataType.BigDecimalType)); //发货数量
        fields.add(new Field("tpv_totalshprices",DataType.BigDecimalType));//发货总价
        int size = fields.size();// DataSet字段数
        if (size > 0) {
            allrows = toallrows(col_s, size);
        }
        //创建dataset
        CollectionInput collectionInput = new CollectionInput(new RowMeta(fields.toArray(new Field[0])), allrows);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(collectionInput);
        return createDataSet;

    }

    private List<Object[]> toallrows(DynamicObjectCollection col_s, int size) {
        List<Object[]> allrows = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject map = col_s.get(i);
            Object[] array = new Object[size];
            BigDecimal quantgoods = map.getBigDecimal("tpv_quantgoods");
            BigDecimal pricegoods = map.getBigDecimal("tpv_pricegoods");
            BigDecimal totalgoods = map.getBigDecimal("tpv_totalgoods");
            BigDecimal shipmentqs = map.getBigDecimal("tpv_shipmentq");
            BigDecimal totalshprices = pricegoods.multiply(shipmentqs);
            array[0] = map.getString("tpv_intercode");// 销售合同商品明细内码(主键)
            array[1] = map.getString("tpv_salescontnum");// 销售合同号
            array[2] = map.getString("tpv_brand");// 品牌
            array[3] = map.getString("tpv_goodsname");// 货物名称
            array[4] = quantgoods;// 货物数量
            array[5] = pricegoods;// 货物单价
            array[6] = totalgoods;// 货物总价
            array[7] = map.getString("tpv_unita");// 单位
            array[8] = map.getDate("tpv_creationdate");// 创建日期
            array[9] = shipmentqs;// 发货数量
            array[10] = totalshprices;// 发货总价
            allrows.add(array);
        }
        return allrows;
    }

}
