package com.itopview.wm.dt.report;


import kd.bd.sbd.utils.StringUtils;
import kd.bos.algo.*;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static kd.fi.er.report.query.TripOrderRpt.algoKey;

/**
 * @author ljw
 * @date 2023/8/15 10:20
 * @className 运价报表插件
 * @description ljwxj
 */
public class FreightrateQueryRpt extends AbstractReportListDataPlugin {
    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        FilterInfo filterInfo = param.getFilter();//获取所有的过滤条件
        String billnum = filterInfo.getString("tpv_billnum");//单据编号
        String combillnum = filterInfo.getString("tpv_combillnum");//对比单号
        String pricetrantype = filterInfo.getString("tpv_pricetrantype");//运价表类型

        // 根据过滤条件单据编号 查询运价单
        QFilter QFilter = new QFilter("billno", QCP.equals, billnum);//查询单据编号相等的
        QFilter.and("billstatus", QCP.equals, "C");//查询单据状态为已审核的
        String selelctFiles = "tpv_freightbill_djt.tpv_routestart  tpv_airrouteorigin," +
                "tpv_freightbill_djt.tpv_endofroute tpv_airrouteterminus," +
                "tpv_freightbill_djt.tpv_form tpv_modality," +
                "tpv_freightbill_djt.tpv_unit tpv_unit," +
                "tpv_freightbill_djt.tpv_periodfrom tpv_nowpricec," +
                "tpv_freightbill_djt.tpv_currentprice tpv_nowpricez," +
                "tpv_freightbill_djt.tpv_remark tpv_remark"; // 查询字段
        DataSet dataSet = QueryServiceHelper.queryDataSet(this.getClass().getName(), "tpv_freightbill", selelctFiles,
                QFilter.toArray(), null);//根据单据编号查询
        //查询全部 不包含单据编号字段
        String selelctFile =
                "tpv_freightbill_djt.tpv_routestart tpv_airrouteorigin," +
                        "tpv_freightbill_djt.tpv_endofroute tpv_airrouteterminus," +
                        "tpv_freightbill_djt.tpv_form tpv_modality," +
                        "tpv_freightbill_djt.tpv_unit tpv_unit," +
                        "tpv_freightbill_djt.tpv_periodfrom tpv_lastpricec," +
                        "tpv_freightbill_djt.tpv_currentprice tpv_lastpricez," +
                        "tpv_freightbill_djt.tpv_remark tpv_remark"; // 查询字段
        QFilter QFilterss = new QFilter("billno", QCP.not_equals, billnum);//查询不等于单据编号的
        QFilterss.and("billstatus", QCP.equals, "C");//查询单据状态为已审核的
        QFilterss.and("tpv_tarifftype", QCP.equals, pricetrantype);//查询运价表相同的
        if (!StringUtils.isEmpty(combillnum)) {
            QFilterss.and("billno", QCP.equals, combillnum);
        }
        DataSet data = QueryServiceHelper.queryDataSet(this.getClass().getName(), "tpv_freightbill", selelctFile,
                QFilterss.toArray(), "tpv_bizdate desc");//根据单据编号查询
//        reDataSet = new String().add("tpv_comfallrise",null);
//        String tpv_comfallrise = new String("tpv_comfallrise");
        DataSet reDataSet = dataSet.leftJoin(data)
                .on("tpv_airrouteorigin", "tpv_airrouteorigin")
                .on("tpv_airrouteterminus", "tpv_airrouteterminus")
                .on("tpv_modality", "tpv_modality")
                .on("tpv_unit", "tpv_unit")
                .select(
                        new String[]{"tpv_airrouteorigin", "tpv_airrouteterminus", "tpv_modality", "tpv_unit", "tpv_nowpricec", "tpv_nowpricez", "tpv_remark"}
                        , new String[]{"tpv_lastpricec", "tpv_lastpricez"})
                .finish();

        reDataSet = reDataSet.select("tpv_airrouteorigin", "tpv_airrouteterminus", "tpv_modality", "tpv_unit",
                "tpv_nowpricec", "tpv_nowpricez", "tpv_remark", "tpv_lastpricec", "tpv_lastpricez"
        );
        //转换类型
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(reDataSet);
        List<Field> fields = new ArrayList<>();
        List<Object[]> allrows = new ArrayList<>();
        fields.add(new Field("tpv_airrouteorigin", DataType.LongType));// 航线起点
        fields.add(new Field("tpv_airrouteterminus", DataType.LongType));// 航线终点
        fields.add(new Field("tpv_modality", DataType.StringType));// 形式
        fields.add(new Field("tpv_unit", DataType.LongType));// 单位
        fields.add(new Field("tpv_nowpricec", DataType.BigDecimalType));// 本期价格从
        fields.add(new Field("tpv_nowpricez", DataType.BigDecimalType));// 本期价格至
        fields.add(new Field("tpv_lastpricec", DataType.BigDecimalType));// 上期价格从
        fields.add(new Field("tpv_lastpricez", DataType.BigDecimalType));// 上期价格至
        fields.add(new Field("tpv_change", DataType.StringType));// 涨跌
        fields.add(new Field("tpv_comfallrise", DataType.StringType));// 较上期涨跌
        fields.add(new Field("tpv_remark", DataType.StringType));// 备注
        int size = fields.size();// DataSet字段数
        if (size > 0) {
            allrows = toallrows(col_s, size);
        }
        //创建dataset
        CollectionInput collectionInput = new CollectionInput(new RowMeta(fields.toArray(new Field[0])), allrows);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(collectionInput);
        return createDataSet.distinct();
    }

    /**
     * 循环取值并进行赋值
     * @param col_s
     * @param size
     * @return
     */
    private List<Object[]> toallrows(DynamicObjectCollection col_s, int size) {
        List<Object[]> allrows = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject map = col_s.get(i);
            Object[] array = new Object[size];
            if (map.getLong("tpv_airrouteorigin") > 0) {
                array[0] = map.get("tpv_airrouteorigin");
            }
            if (map.getLong("tpv_airrouteterminus") > 0) {
                array[1] = map.get("tpv_airrouteterminus");
            }
            if (map.getString("tpv_modality") != null) {
                array[2] = map.get("tpv_modality");
            }
            if (map.getLong("tpv_unit") > 0) {
                array[3] = map.get("tpv_unit");
            }
            BigDecimal tpvNowpricec = map.getBigDecimal("tpv_nowpricec");
            BigDecimal tpvNowpricez = map.getBigDecimal("tpv_nowpricez");
            BigDecimal tpvLastpricec = map.getBigDecimal("tpv_lastpricec");
            BigDecimal tpvLastpricez = map.getBigDecimal("tpv_lastpricez");
            array[4] = tpvNowpricec;
            array[5] = tpvNowpricez;
            array[6] = tpvLastpricec;
            array[7] = tpvLastpricez;
            array[8] = comfallrise(tpvNowpricec, tpvNowpricez, tpvLastpricec, tpvLastpricez, "tpv_change");
            array[9] = comfallrise(tpvNowpricec, tpvNowpricez, tpvLastpricec, tpvLastpricez, "tpv_comfallrise");
            array[10] = map.getString("tpv_remark");
            allrows.add(array);
        }
        return allrows;
    }

    /**
     * 确认涨跌平，并计算要赋予的值
     * @param nowpricec
     * @param nowpricez
     * @param lastpricec
     * @param lastpricez
     * @param type
     * @return
     */
    public String comfallrise(BigDecimal nowpricec, BigDecimal nowpricez, BigDecimal lastpricec, BigDecimal lastpricez, String type) {
        String comfa = "";
        BigDecimal A = nowpricec.subtract(lastpricec);
        BigDecimal B = nowpricez.subtract(lastpricez);
        BigDecimal C = (BigDecimal.ZERO);
        if (A.compareTo(C) == 1 && B.compareTo(C) == 1) {
            if (A.equals(B)){
                BigDecimal D = A.abs();
                BigDecimal E = B.abs();
                if (type.equals("tpv_change")) {
                    comfa = "涨";
                } else {
                    comfa = ("涨 " + D );
                }
            }else {
                if (A.compareTo(B) == 1) {
                    BigDecimal D = A.abs();
                    BigDecimal E = B.abs();
                    if (type.equals("tpv_change")) {
                        comfa = "涨";
                    } else {
                        comfa = ("涨 " + E + "~" + D);
                    }
                } else if (A.compareTo(B) == -1) {
                    BigDecimal D = A.abs();
                    BigDecimal E = B.abs();
                    if (type.equals("tpv_change")) {
                        comfa = "涨";
                    } else {
                        comfa = "涨 " + D + "~" + E;
                    }
                }
            }
        } else if (A.compareTo(C) == 1 && B.compareTo(C) == 0) {
            if (type.equals("tpv_change")) {
                comfa = "涨";
            } else {
                comfa = "涨 " + A;
            }
        } else if (A.compareTo(C) == 0 && B.compareTo(C) == 1) {
            if (type.equals("tpv_change")) {
                comfa = "涨";
            } else {
                comfa = "涨 " + B;
            }
        } else if (A.compareTo(C) == -1 && B.compareTo(C) == -1) {
            if (A.equals(B)){
                BigDecimal D = A.abs();
                if (type.equals("tpv_change")) {
                    comfa = "跌";
                } else {
                    comfa = "跌 " + D ;
                }
            }else {
                if (A.compareTo(B) == 1) {
                    BigDecimal D = A.abs();
                    BigDecimal E = B.abs();
                    if (type.equals("tpv_change")) {
                        comfa = "跌";
                    } else {
                        comfa = "跌 " + D + "~" + E;
                    }
                } else if (A.compareTo(B) == -1) {
                    BigDecimal D = A.abs();
                    BigDecimal E = B.abs();
                    if (type.equals("tpv_change")) {
                        comfa = "跌";
                    } else {
                        comfa = "跌 " + E + "~" + D;
                    }
                }
            }
        } else if (A.compareTo(C) == -1 && B.compareTo(C) == 0) {
            BigDecimal D = A.abs();
            if (type.equals("tpv_change")) {
                comfa = "跌";
            } else {
                comfa = "跌 " + D;
            }
        } else if (A.compareTo(C) == 0 && B.compareTo(C) == -1) {
            BigDecimal D = B.abs();
            if (type.equals("tpv_change")) {
                comfa = "跌";
            } else {
                comfa = "跌 " + D;
            }
        } else if (A.compareTo(C) == 0 && B.compareTo(C) == 1) {
            BigDecimal D = B.abs();
            if (type.equals("tpv_change")) {
                comfa = "跌";
            } else {
                comfa = "跌 " + D;
            }
        } else if (A.compareTo(C) == -1 && B.compareTo(C) == 1) {
            comfa = "平";
        } else if (A.compareTo(C) == 1 && B.compareTo(C) == -1) {
            comfa = "平";
        } else if (A.compareTo(C) == 0 && B.compareTo(C) == 0) {
            comfa = "平";
        }
        return comfa;

    }
}
