package com.itopview.wm.dt.report;


import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QFilter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 库存拣配实提工作台
 *
 * @author WaangShuGuang
 * @description 查询插件
 * @date 2023年9月15日
 */
public class InventoryRptQuery extends AbstractReportListDataPlugin {

    private static final String algoKey = InventoryRptQuery.class.getName();
    private static List<String> fieldKeys_ship = new ArrayList<>();// 销售发货

    @Override
    public DataSet query(ReportQueryParam param, Object obj) throws Throwable {
        String whereFilter = createWhereStr(param);
        DataSet instockDataSet = query(whereFilter);
        return instockDataSet;
    }

    public static DataSet query(String whereFilter) {
        DataSet mainDataSet = mainDataSet(whereFilter);
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
        return createDataSet.where("tpv_qty > 0 and tpv_danjutype = 'C'").orderBy(new String[]{"tpv_shipmentsdate desc"});
    }

    private static DataSet mainDataSet(String whereFilter) {
        initAllFields();
        QFilter[] qFilters = null;
        DataSet tpv_dt_salesshipping = ORM.create().queryDataSet(algoKey, "tpv_dt_salesshipping",
                String.join(",", fieldKeys_ship), qFilters);
        if (YEM.isNotEmpty(whereFilter)) {
            tpv_dt_salesshipping = tpv_dt_salesshipping.where(whereFilter);
        }
        return tpv_dt_salesshipping;
    }

    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        return whereFilter;
    }

    private static void initAllFields() {
        fieldKeys_ship.clear();
        addFields("tpv_seleid", "id");// 来源单据ID
        addFields("tpv_sourceid", "tpv_materialentry.tpv_dt_salepick_in");// 来源分录ID
        addFields("tpv_srcentryid", "tpv_materialentry");// 来源单据父分录ID
        addFields("tpv_source", "'C'");// 来源单据
        addFields("tpv_shipments", "billno");// 发货单号
        addFields("tpv_shipmentsdate", "tpv_bizdate");// 发货日期
        addFields("tpv_market", "tpv_contractnumber");// 销售合同号
        addFields("tpv_customer", "tpv_customer");// 客户
        addFields("tpv_customers", "tpv_customers");// 本家客户（第三方销售原客户）
        addFields("tpv_partysales", "tpv_partysales");// 本家销售合同（第三方销售原合同）
        //addFields("tpv_projectname", "tpv_projectname");// 配供项目简称(不确定)
        addFields("tpv_inventorytype", "tpv_inventorytype");// 库存/直运
        addFields("tpv_nopicking", "tpv_nopicking");// 是否免拣配
        addFields("tpv_material", "tpv_materialentry.tpv_dt_salepick_in.tpv_material_in");// 商品
        addFields("tpv_prodname", "tpv_materialentry.tpv_dt_salepick_in.tpv_prodname_in");// 品名
        addFields("tpv_shader", "tpv_materialentry.tpv_dt_salepick_in.tpv_shader_in");// 材质
        addFields("tpv_origin", "tpv_materialentry.tpv_dt_salepick_in.tpv_origin_in");// 产地
        addFields("tpv_specation", "tpv_materialentry.tpv_dt_salepick_in.tpv_specation_in");// 规格
        addFields("tpv_pickbill_zy", "tpv_materialentry.tpv_dt_salepick_in.tpv_pickbill_in");// 配货单据
        addFields("tpv_billno_zy", "tpv_materialentry.tpv_dt_salepick_in.tpv_billno_in");// 单据编号
        addFields("tpv_supplier_zy", "tpv_materialentry.tpv_dt_salepick_in.tpv_supplier_in");// 供应商
        addFields("tpv_volumenumber_zy", "tpv_materialentry.tpv_dt_salepick_in.tpv_volumenumber_in");// 卷号
        addFields("tpv_qty", "tpv_materialentry.tpv_dt_salepick_in.tpv_pickqty_in");// 数量
        addFields("tpv_ofpackages", "tpv_materialentry.tpv_dt_salepick_in.tpv_packnum_in");// 包装数量
        addFields("tpv_ofmeasure", "tpv_materialentry.tpv_dt_salepick_in.tpv_measure");// 计量单位
        addFields("tpv_realsubmit", "tpv_materialentry.tpv_dt_salepick_in.tpv_practicalok_in");// 实提完毕
        addFields("tpv_danjutype", "billstatus");//单据状态
        //addFields("tpv_ofmeasure", "true");// 计量单位
    }

    static {

    }

    private static void addFields(String fieldKey, String fieldKey_bill) {
        fieldKeys_ship.add(fieldKey_bill + " " + fieldKey);
    }
}
