package com.itopview.wm.dt.report;


import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * 库存免拣配实提工作台
 *
 * @author WaangShuGuang
 * @description 查询插件
 * @date 2023年9月15日
 */
public class NoInventoryRptQuery extends AbstractReportListDataPlugin {

    private static final String algoKey = NoInventoryRptQuery.class.getName();
    private static List<String> fieldKeys_ship = new ArrayList<>();// 销售发货

    @Override
    public DataSet query(ReportQueryParam param, Object obj) throws Throwable {
        String whereFilter = createWhereStr(param);
        DataSet instockDataSet = query(whereFilter);
        return instockDataSet;
    }

    public static DataSet query(String whereFilter) {
        DataSet mainDataSet = mainDataSet(whereFilter);
        List<Long> materialids = new ArrayList<>();
        DataSet copy = mainDataSet.copy();
        for (Row row : copy) {
            if (!materialids.contains(row.getLong("tpv_material"))) {
                materialids.add(row.getLong("tpv_material"));
            }
        }
        Map<Long, HashMap<String, BigDecimal>> matersidData = matersidData(materialids);//查询物料数据
        Map<String, Boolean> configuranewData = configuranewData();//查询底表分录数据
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        for (DynamicObject col : col_s) {
            if (matersidData.containsKey(col.getLong("tpv_material"))) {
                BigDecimal tpv_destroyquantitysum = col.getBigDecimal("tpv_destroyquantitysum");
                BigDecimal tpv_qtyrecord = col.getBigDecimal("tpv_qtyrecord");
                BigDecimal qty = tpv_qtyrecord.compareTo(BigDecimal.ZERO) != 0 ? tpv_qtyrecord : BigDecimal.ONE;
                HashMap<String, BigDecimal> bigDecimalMap = matersidData.get(col.getLong("tpv_material"));
                BigDecimal shkdQuantitystart = bigDecimalMap.get("shkd_quantitystart");
                BigDecimal shkdQuantityend = bigDecimalMap.get("shkd_quantityend");
                BigDecimal shkdFloatpercentage = bigDecimalMap.get("shkd_floatpercentage");
                BigDecimal subtract = tpv_destroyquantitysum.subtract(tpv_qtyrecord);
                BigDecimal absqty = subtract.divide(qty, 2, RoundingMode.HALF_UP).abs();
                BigDecimal startqty = absqty.subtract(shkdQuantitystart);
                BigDecimal endqty = absqty.subtract(shkdQuantityend);
                if (startqty.compareTo(BigDecimal.ZERO) >= 0 && endqty.compareTo(BigDecimal.ZERO) < 0) {
                    if (absqty.compareTo(shkdFloatpercentage) > 0) {
                        col.set("tpv_islogistics","true");
                    }
                }

            }
            BigDecimal tpv_packnum = col.getBigDecimal("tpv_packnum");
            BigDecimal tpv_fpackagerealsum = col.getBigDecimal("tpv_fpackagerealsum");
            if (tpv_packnum.compareTo(tpv_fpackagerealsum) != 0) {
                col.set("tpv_islogistics","true");
            }
            if (configuranewData.containsKey(col.getString("tpv_sourceid"))) {
                col.set("tpv_logisticsok", configuranewData.get(col.getString("tpv_sourceid")));
            }
        }
        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
        return createDataSet.where("tpv_qty > 0 and tpv_danjutype = 'C'").orderBy(new String[]{"tpv_shipmentsdate desc","tpv_srcentryid asc"});
    }

    private static DataSet mainDataSet(String whereFilter) {
        initAllFields();
        QFilter[] qFilters = null;
        DataSet tpv_dt_salesshipping = ORM.create().queryDataSet(algoKey, "tpv_dt_salesshipping",
                String.join(",", fieldKeys_ship), qFilters);
        if (YEM.isNotEmpty(whereFilter)) {
            tpv_dt_salesshipping = tpv_dt_salesshipping.where(whereFilter);
        }
        //查询直运业务实提底表，将分录销提数量和采提数量汇总
        DataSet copy = tpv_dt_salesshipping.copy();
        List<Long> entryids = new ArrayList<>();
        for (Row row : copy) {
            if (!entryids.contains(row.getLong("tpv_sourceid"))) {
                entryids.add(row.getLong("tpv_sourceid"));
            }
        }
        QFilter qFilterBlend = new QFilter("id", QCP.in,entryids);
        qFilterBlend.and(new QFilter("tpv_sonentryentity.id",QCP.large_than,0));
        DataSet tpv_dt_noconfiguration = ORM.create().queryDataSet(algoKey, "tpv_dt_noconfiguration",
                "id blendid,tpv_sonentryentity,tpv_sonentryentity.id blendentryid, tpv_sonentryentity.tpv_destroyquantity destsalesqty," +
                        "tpv_sonentryentity.tpv_fpackagereal tpv_fpackagereal",
                qFilterBlend.toArray());

        DataSet copyBlend = tpv_dt_noconfiguration.copy();
        List<Long> blendids = new ArrayList<>();
        for (Row row : copyBlend) {
            if (!blendids.contains(row.getLong("blendentryid"))) {
                blendids.add(row.getLong("blendentryid"));
            }
        }
        //查询销售出库和其他出库数量，根据直运业务实提底表分录行id，将出库数量汇总
        QFilter qFilterSalout = new QFilter("billentry.tpv_srcbillsubentryid", QCP.in,blendids);
        qFilterSalout.and(new QFilter("billentry.id",QCP.large_than,0));
        DataSet im_saloutbill = ORM.create().queryDataSet(algoKey, "im_saloutbill",
                "id,billentry,billentry.qty outqty,billentry.tpv_srcbillsubentryid tpv_directid", qFilterSalout.toArray());
        im_saloutbill = im_saloutbill.groupBy(new String[]{"tpv_directid"}).sum("outqty").finish();
//        QFilter qFilteOtherout = new QFilter("billentry.tpv_directconsignmentid", QCP.in,blendids);
//        qFilteOtherout.and(new QFilter("billentry.id",QCP.large_than,0));
//        DataSet tpv_im_otheroutbill = ORM.create().queryDataSet(algoKey, "im_otheroutbill",
//                "id,billentry,billentry.qty outqty,billentry.tpv_directconsignmentid  tpv_directid", qFilteOtherout.toArray());
//        tpv_im_otheroutbill = tpv_im_otheroutbill.groupBy(new String[]{"tpv_directid"}).sum("outqty").finish();
//        DataSet outbillData = im_saloutbill.union(tpv_im_otheroutbill).groupBy(new String[]{"tpv_directid"}).sum("outqty").finish();
        DataSet outbillData = im_saloutbill;
        tpv_dt_noconfiguration = tpv_dt_noconfiguration.leftJoin(outbillData).on("blendentryid","tpv_directid")
                .select("blendid","destsalesqty","tpv_fpackagereal","outqty").finish();
        tpv_dt_noconfiguration = tpv_dt_noconfiguration.groupBy(new String[]{"blendid"})
                .sum("destsalesqty").sum("tpv_fpackagereal").sum("outqty").finish();
        DataSet dataSet = tpv_dt_salesshipping.leftJoin(tpv_dt_noconfiguration).on("tpv_sourceid","blendid")
                .select(RptUtil.getDataSetFiledAlias(tpv_dt_salesshipping),new String[]{"destsalesqty tpv_destroyquantitysum",
                        "tpv_fpackagereal tpv_fpackagerealsum","outqty tpv_issuedquantitys"}).finish();
        dataSet = RptUtil.zeroNullNums(dataSet);
        dataSet = dataSet.addField("tpv_qty - tpv_issuedquantitys tpv_unissuedquanti","tpv_unissuedquanti");
        dataSet = dataSet.addField("tpv_qty - tpv_fpackagerealsum tpv_unfpackagerealsum","tpv_unfpackagerealsum");
        dataSet = RptUtil.zeroNullNums(dataSet);
        dataSet = dataSet.addField("false tpv_islogistics","tpv_islogistics");
        dataSet = dataSet.addField("false tpv_logisticsok","tpv_logisticsok");
        return dataSet;
    }

    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        return whereFilter;
    }

    private static Map matersidData(List<Long> materialids) {
        Map<Long, HashMap<String, BigDecimal>> map = new HashMap<>();
        QFilter qFilterMater = new QFilter("shkd_floatingentry.id",QCP.large_than,0);
        if (materialids.size() > 0) {
            qFilterMater.and(new QFilter("id", QCP.in, materialids));
        }
        DataSet bd_material = ORM.create().queryDataSet(algoKey, "bd_material", "id matersid,shkd_floatingentry," +
                        "shkd_floatingentry.shkd_quantitystart shkd_quantitystart,shkd_floatingentry.shkd_quantityend shkd_quantityend," +
                        "(shkd_floatingentry.shkd_floatpercentage * 0.01) shkd_floatpercentage",
                qFilterMater.toArray());
        bd_material = RptUtil.zeroNullNums(bd_material);
        DataSet copy = bd_material.copy();
        for (Row row : copy) {
            HashMap<String, BigDecimal> bigDecimalMap = new HashMap<>();
            bigDecimalMap.put("shkd_quantitystart",row.getBigDecimal("shkd_quantitystart"));
            bigDecimalMap.put("shkd_quantityend",row.getBigDecimal("shkd_quantityend"));
            bigDecimalMap.put("shkd_floatpercentage",row.getBigDecimal("shkd_floatpercentage"));
            map.put(row.getLong("matersid"),bigDecimalMap);
        }
        return map;
    }
    private static Map<String, Boolean> configuranewData() {
        Map<String, Boolean> map = new HashMap<>();
        QFilter qFilterBlend = new QFilter("tpv_sonentryentity.id",QCP.large_than,0);
        DynamicObject[] tpv_dt_noconfiguration = BusinessDataServiceHelper.load("tpv_dt_noconfiguration",
                "id blendid,tpv_sonentryentity, tpv_sonentryentity.tpv_fatherid, tpv_sonentryentity.tpv_destroysure",
                qFilterBlend.toArray());
        if (tpv_dt_noconfiguration.length > 0) {
            for (DynamicObject dynamicObject : tpv_dt_noconfiguration) {
                DynamicObjectCollection dynamicObjects = dynamicObject.getDynamicObjectCollection("tpv_sonentryentity");
                for (DynamicObject object : dynamicObjects) {
                    String tpvFatherid = object.getString("tpv_fatherid");
                    boolean tpvDestroysure = object.getBoolean("tpv_destroysure");
                    if (!tpvDestroysure) {
                        map.put(tpvFatherid, false);
                        break;
                    } else {
                        map.put(tpvFatherid, true);
                    }
                }
            }
        }
        return map;
    }

    private static void initAllFields() {
        fieldKeys_ship.clear();
        addFields("tpv_seleid", "id");// 来源单据ID
        addFields("tpv_sourceid", "tpv_materialentry");// 来源单据分录ID
        addFields("tpv_srcentryid", "tpv_materialentry");// 来源单据分录ID
//        addFields("tpv_repertoryson", "tpv_materialentry.tpv_dt_salepick_in");// 来源单据分录ID
        addFields("tpv_source", "'C'");// 来源单据
        addFields("tpv_shipments", "billno");// 发货单号
        addFields("tpv_shipmentsdate", "tpv_bizdate");// 发货日期
        addFields("tpv_market", "tpv_contractnumber");// 销售合同号
        addFields("tpv_customer", "tpv_customer");// 客户
        addFields("tpv_customers", "tpv_customers");// 本家客户（第三方销售原客户）
        addFields("tpv_partysales", "tpv_partysales");// 本家销售合同（第三方销售原合同）
        //addFields("tpv_projectname", "tpv_projectname");// 配供项目简称(不确定)
        addFields("tpv_inventorytype", "tpv_inventorytype");// 库存/直运
        addFields("tpv_nopicking", "tpv_nopicking");// 是否免拣配
        addFields("tpv_material", "tpv_materialentry.tpv_material");// 商品
        addFields("tpv_prodname", "tpv_materialentry.tpv_prodname");// 品名
        addFields("tpv_shader", "tpv_materialentry.tpv_shader");// 材质
        addFields("tpv_origin", "tpv_materialentry.tpv_origin");// 产地
        addFields("tpv_specation", "tpv_materialentry.tpv_specation");// 规格
        //addFields("tpv_pickbill_zy", "tpv_materialentry.tpv_dt_salepick_in.tpv_pickbill_in");// 配货单据
        //addFields("tpv_billno_zy", "tpv_materialentry.tpv_dt_salepick_in.tpv_billno_in");// 单据编号
        //addFields("tpv_supplier_zy", "tpv_materialentry.tpv_dt_salepick_in.tpv_supplier_in");// 供应商
        //addFields("tpv_volumenumber_zy", "tpv_materialentry.tpv_dt_salepick_in.tpv_volumenumber_in");// 卷号
        addFields("tpv_qty", "tpv_materialentry.tpv_qty");// 数量
        addFields("tpv_packnum", "tpv_materialentry.tpv_packnum");// 包装数量
        addFields("tpv_ofmeasure", "tpv_materialentry.tpv_unit");// 计量单位
        addFields("tpv_ofpackagingunit", "tpv_materialentry.tpv_packagingunit");// 包装单位
        addFields("tpv_realsubmit", "tpv_materialentry.tpv_practicalok");// 实提完毕
        addFields("tpv_danjutype", "billstatus");//单据状态
        addFields("tpv_qtyrecord", "tpv_materialentry.tpv_qtyrecord");//数量(记录)
        addFields("tpv_qtynumrecord", "tpv_materialentry.tpv_qtynumrecord");//包装数量(记录)
        addFields("tpv_rationtype", "tpv_rationtype");//定量方式
        addFields("tpv_isautoinsales", "tpv_isautoinsales");//自动带入销提
        addFields("tpv_isautosalesout", "tpv_isautosalesout");//自动销售出库
        addFields("tpv_department", "tpv_department");//部门
        //addFields("tpv_ofmeasure", "true");// 计量单位
    }

    static {

    }

    private static void addFields(String fieldKey, String fieldKey_bill) {
        fieldKeys_ship.add(fieldKey_bill + " " + fieldKey);
    }
}
