package com.itopview.wm.sc.formplugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.Control;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.TextEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.util.EventObject;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author ljw
 * @version 1.0
 * @description:期货出/入金申请单 期货公司带出对方账户和对方银行
 * @date 2023/8/11 21:32
 */
public class FuturesapplicationAbstractBillPlugIn extends AbstractBillPlugIn implements BeforeF7SelectListener {
    @Override
    //值更新
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        //获取银行信息的单据体
        String name = e.getProperty().getName();
        ChangeData[] changeData = e.getChangeSet();
        int rowIndex = changeData[0].getRowIndex();
        switch (name) {
            case "tpv_futurescompanies":
                DynamicObject futurescompanies = (DynamicObject) this.getModel().getValue("tpv_futurescompanies");//获取期货公司
                if (futurescompanies != null) {
                    DynamicObjectCollection entrys = futurescompanies.getDynamicObjectCollection("entry_bank");// 获取银行信息的单据体
                    for (DynamicObject entry : entrys) {// 循环单据体
                        boolean isdefaultBank = entry.getBoolean("isdefault_bank");
                        if (isdefaultBank) {
                            DynamicObject bank = entry.getDynamicObject("bank");//获取到对方账户的值
                            String isdefault_bank = entry.getString("isdefault_bank");
                            String bankaccount = entry.getString("bankaccount");//对方银行
                            if (isdefault_bank.equals("true")) {
                                if (bank != null) {
                                    this.getModel().setValue("tpv_counterparty", bank);//对对方账户进行赋值
                                } else {
                                    this.getModel().setValue("tpv_counterparty", null);// 对对方账户进行赋值
                                }
                                if (bankaccount != null){
                                    this.getModel().setValue("tpv_theotherbank",bankaccount);
                                }else {
                                    this.getModel().setValue("tpv_theotherbank",null);
                                }
                            }
                        }
                    }
                } else {
                    this.getModel().setValue("tpv_counterparty", null);
                    this.getModel().setValue("tpv_theotherbank", null);
                }
                break;
            default:
                break;
        }
    }

    /**
     * 根据所选部门过滤显示采购合同号和采购销售号
     *对方账户根据期货公司过滤
     * @param e
     */
    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        BasedataEdit supplies = this.getControl("tpv_purchasecontract");//采购合同号
        supplies.addBeforeF7SelectListener(this);
        BasedataEdit alescontract = this.getControl("tpv_salescontract");//采购销售号
        alescontract.addBeforeF7SelectListener(this);
        BasedataEdit counterparty = this.getControl("tpv_counterparty");//对方银行
        counterparty.addBeforeF7SelectListener(this);
        TextEdit tpv_theotherbank = this.getView().getControl("tpv_theotherbank"); //对方账号
        tpv_theotherbank.addButtonClickListener(this);
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String key = e.getProperty().getName();
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        DynamicObject tpv_department = dataEntity.getDynamicObject("tpv_department");
        DynamicObject org = dataEntity.getDynamicObject("org");
        if (tpv_department != null || org != null) {
            QFilter qFilter = new QFilter("billstatus",QCP.equals, "C");
            if (tpv_department != null){
                 qFilter.and(new QFilter("tpv_department", QCP.equals, tpv_department.getLong("id")));
            }
            if (org != null ){
                qFilter.and(new QFilter("org", QCP.equals, org.getLong("id")));
            }
            if (key.equals("tpv_purchasecontract")) {
                ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
                showParameter.getListFilterParameter().getQFilters().add(qFilter);

            }
            if (key.equals("tpv_salescontract")) {
                ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
                showParameter.getListFilterParameter().getQFilters().add(qFilter);
            }
        }
        if (key.equals("tpv_counterparty")) {
            DynamicObject tpvFuturescompanies = dataEntity.getDynamicObject("tpv_futurescompanies");
            DynamicObjectCollection entryBank = tpvFuturescompanies.getDynamicObjectCollection("entry_bank");
            List<Long> tpvPurinentryid = entryBank.stream().map(x -> x.getLong("bank.id")).collect(Collectors.toList());
            QFilter qFilter = new QFilter("id",QCP.in, tpvPurinentryid.size() > 0 ? tpvPurinentryid : 0l);
            ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
            showParameter.getListFilterParameter().getQFilters().add(qFilter);
        }
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        Control c = (Control) evt.getSource();
        String key = c.getKey().toLowerCase();
        if (key.equals("tpv_theotherbank")) { //对方账号
            FormShowParameter showParameter = new FormShowParameter();
            showParameter.setFormId("tpv_bankno");
            showParameter.getOpenStyle().setShowType(ShowType.Modal);
            this.getView().showForm(showParameter);
        }
    }
}
