package com.itopview.wm.sc.formplugin.form;

import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;

import java.util.EventObject;
import java.util.HashMap;
import java.util.Map;

/**
 * @ClassName SendFormPlugin
 * @Description 销售发货 -仓库携带数据
 * @Author yhj
 * @Date 2023/9/26 15:04
 * @Version 1.0
 */
public class SendFormPlugin extends AbstractBillPlugIn {
    /**
     * @description: 单据字段【仓库】更新而默认带出仓库基础资料<发货指令送达方式>分录中【设为默认】=“是”的一行的对应数据。
     * @author yhj
     * @date 2023/9/26 15:07
     * @version 1.0
     */
    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name=e.getProperty().getName();
        ChangeData cd = e.getChangeSet()[0];
        switch(name) {
            case "tpv_entrepot":
                entrepot(cd);
                break;
            default:
                break;
        }
    }

    private void entrepot(ChangeData cd) {
        IDataModel model = this.getModel();
        DynamicObject dataEntity = model.getDataEntity(true);
        DynamicObjectCollection tpvMaterialentry = dataEntity.getDynamicObjectCollection("tpv_materialentry");
        DynamicObject tpvEntrepotObject = (DynamicObject) model.getValue("tpv_entrepot");
        if (YEM.isEmpty(tpvEntrepotObject)){
            model.setValue("tpv_sendstate", null);
            model.setValue("tpv_sendingpath",null);
            model.setValue("tpv_interface",null);
            model.setValue("tpv_fax",null);
            model.setValue("tpv_eaddress",null);
            model.setValue("tpv_specification",null);
            model.setValue("tpv_default",null);
            return;
        }
        long entrepotId = (long) ((DynamicObject) cd.getNewValue()).getPkValue();
        DynamicObject entrepot = BusinessDataServiceHelper.loadSingle(entrepotId, "bd_warehouse");
        DynamicObjectCollection sendmode = entrepot.getDynamicObjectCollection("tpv_sendmode");//发货指令发送方式
        if (sendmode.size() > 0) {
            Boolean flag = false;
            for (DynamicObject obj:sendmode) {
                boolean tpvDefault = obj.getBoolean("tpv_default");
                String sendingpath = obj.getString("tpv_sendingpath");//发送途径
                DynamicObject api = obj.getDynamicObject("tpv_interface");//API
                String fax = obj.getString("tpv_fax");//传真号
                String eaddress = obj.getString("tpv_eaddress");//邮箱地址
                String specification = obj.getString("tpv_specification");//途径说明
                if (tpvDefault) {
                    if ("E".equals(sendingpath)) {
                        model.setValue("tpv_sendstate", "A");
                    }
                    model.setValue("tpv_sendingpath",sendingpath);
                    model.setValue("tpv_interface",api);
                    model.setValue("tpv_fax",fax);
                    model.setValue("tpv_eaddress",eaddress);
                    model.setValue("tpv_specification",specification);
                    model.setValue("tpv_default",tpvDefault);
                    flag = true;
                }
            }
            if (!flag) {
                model.setValue("tpv_sendstate", null);
                model.setValue("tpv_sendingpath",null);
                model.setValue("tpv_interface",null);
                model.setValue("tpv_fax",null);
                model.setValue("tpv_eaddress",null);
                model.setValue("tpv_specification",null);
                model.setValue("tpv_default",null);
            }
        } else {
            model.setValue("tpv_sendstate", null);
            model.setValue("tpv_sendingpath",null);
            model.setValue("tpv_interface",null);
            model.setValue("tpv_fax",null);
            model.setValue("tpv_eaddress",null);
            model.setValue("tpv_specification",null);
            model.setValue("tpv_default",null);
        }
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        // 侦听文本字段按钮点击事件
        this.addItemClickListeners("tbmain");
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String key=evt.getItemKey();
        if(StringUtils.equals("tpv_upwarehouse", key)) {//修改仓库
            // 创建弹出动态表单页面对象
            FormShowParameter showParameter = new FormShowParameter();
            // 设置弹出页面的标识
            showParameter.setFormId("tpv_iscb_advconap");
            Map<String, Object> map = new HashMap<>();
            DynamicObject entrepot = (DynamicObject) this.getModel().getValue("tpv_entrepot");
            if (YEM.isEmpty(entrepot)) {
                this.getView().showTipNotification("请先填写仓库");
                return;
            }
            map.put("tpv_entrepot",entrepot.getLong("id"));
            // 存入获取到的动态表单数据
            showParameter.setCustomParams(map);
            // 状态
            showParameter.setStatus(OperationStatus.ADDNEW);
            // 设置页面关闭回调方法
            // CloseCallBack参数：回调插件，回调标识
            showParameter.setCloseCallBack(new CloseCallBack(this, "tpv_upwarehouse"));
            // 设置弹出页面打开方式
            showParameter.getOpenStyle().setShowType(ShowType.Modal);
            // 打开动态表单
            this.getView().showForm(showParameter);
        }
    }

    /**
     * 动态表单关闭回调事件
     */

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        // super.closedCallBack(closedCallBackEvent);
        Object returnData = closedCallBackEvent.getReturnData();
        // 判断标识是否匹配，并验证返回值不为空，不验证返回值可能会报空指针
        if (org.apache.commons.lang3.StringUtils.equals(closedCallBackEvent.getActionId(), "tpv_upwarehouse")
                && null != closedCallBackEvent.getReturnData()) {
            // 这里返回对象为Object，可强转成相应的其他类型，
            // 单条数据可用String类型传输，返回多条数据可放入map中，也可使用json等方式传输
            HashMap<String, Object> values = (HashMap<String, Object>) returnData;
            this.setInwardInfo(values);
        }
    }

    private void setInwardInfo(HashMap<String, Object> values) {
        IDataModel model = this.getModel();
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        model.setValue("tpv_sendingpath", values.get("tpv_sendingpaths"));
        // 回填动态表单的接口方
        model.setValue("tpv_interface", values.get("tpv_interface") != null ? Long.valueOf((String) values.get("tpv_interface")) : null);
        // 回填动态表单的传真号
        model.setValue("tpv_fax", values.get("tpv_faxs"));
        // 回填动态表单的邮箱地址
        model.setValue("tpv_eaddress", values.get("tpv_eaddresss"));
        // 回填动态表单的用途
        model.setValue("tpv_specification", values.get("tpv_specifications"));
        model.setValue("tpv_default", values.get("tpv_defaults"));
        SaveServiceHelper.update( dataEntity );//写入数据库
        SaveServiceHelper.save(new DynamicObject[] {dataEntity});
    }
}
