package com.itopview.wm.sc.formplugin.form;

import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;

/**
 * ClassName: SupFundDetails
 * Description:
 * date: 2023/10/13 15:18
 *
 * @author: Guo jiaChen
 * @version:
 * @since JDK 1.8
 */
public class SupFundDetails extends AbstractBillPlugIn {
    private final static String ENTITY_ENTRY = "tpv_entryentity";

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        String[] itemListener = {"tpv_toolbarap1", "tpv_baritemap"};
        this.addItemClickListeners(itemListener);
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String key = evt.getItemKey();
        if ("tpv_baritemap1".equals(key)) {
            this.getModel().deleteEntryData(ENTITY_ENTRY);
            this.getView().updateView(ENTITY_ENTRY);
            return;
        }
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        Date year = dataEntity.getDate("tpv_deptfundyear");//部门资金生成年份
        if (year == null) {
            this.getView().showTipNotification("请选择年份!");
            return;
        }
        String month = dataEntity.getString("tpv_deptfundmonth");//部门资金生成月份
        if (YEM.isEmpty(month)) {
            this.getView().showTipNotification("请选择月份!");
            return;
        }
        DynamicObject org = (DynamicObject) dataEntity.getDynamicObject("org");//公司
        if (org == null) {
            this.getView().showTipNotification("请选择组织!");
            return;
        }
        DynamicObject tpvDepartment = (DynamicObject) dataEntity.getDynamicObject("tpv_department");//公司
        if (tpvDepartment == null) {
            this.getView().showTipNotification("请选择部门!");
            return;
        }
        Date queryDate = getQueryDate(year, month);
        switch (key) {
            case "tpv_baritemap"://生成部门资金占用明细
//                createDeptDetails(queryDate, org);
                this.getModel().deleteEntryData(ENTITY_ENTRY);
                createDeptDetail(queryDate, org, tpvDepartment);
                break;
            default:
                break;
        }
    }

    public void createDeptDetail(Date startDate, DynamicObject org, DynamicObject tpvDepartment) {
        Date currentDate = startDate;
        Date endDay = DateUtil.getMonthEndDay(currentDate);
        DynamicObjectCollection entryEntity = this.getModel().getEntryEntity(ENTITY_ENTRY);

        QFilter orgdutyQfilter = new QFilter("enable", QCP.equals, "1");
        orgdutyQfilter.and(new QFilter("orgduty.number", QCP.in, new String[]{"ZHKC002"}));
        DataSet orgduty = ORM.create().queryDataSet("bos_org_dutyrelation", "bos_org_dutyrelation", "org.id tpv_depid",
                orgdutyQfilter.toArray());
        DataSet copyOrgduty = orgduty.copy();
        List<Long> orgdutyIds = new ArrayList<>();
        for (Row row : copyOrgduty) {
            if (!orgdutyIds.contains(row.getLong("tpv_depid"))) {
                orgdutyIds.add(row.getLong("tpv_depid"));
            }
        }

        List<QFilter> qFilter = new ArrayList<>();
        qFilter.add(new QFilter("tpv_type", "=", "J"));//国内证
        qFilter.add(new QFilter("tpv_realpreocp", "=", "A"));//真实/预占用=真实
        qFilter.add(new QFilter("tpv_datetime", QCP.large_equals, currentDate));
        qFilter.add(new QFilter("tpv_datetime", QCP.less_equals, endDay));
        qFilter.add(new QFilter("tpv_org.id", "=", org.getPkValue()));
        qFilter.add(new QFilter("tpv_depertment.id", "=", tpvDepartment.getPkValue()));
        if (orgdutyIds.size() > 0) {
            qFilter.add(new QFilter("tpv_department.id", QCP.in, orgdutyIds));
        } else {
            qFilter.add(new QFilter("tpv_department.id", QCP.less_than, 0));
        }
        DynamicObject[] scCapitallimits = BusinessDataServiceHelper.load("tpv_sc_capitallimit",
                "tpv_orignalamount,tpv_datetime,tpv_org,tpv_depertment,tpv_addorsub",
                qFilter.toArray(new QFilter[qFilter.size()]));//部门资金额度底表
        //业务处理

        while (currentDate.compareTo(endDay) <= 0) {
            BigDecimal tpvTotalAmount = BigDecimal.ZERO;//国内证占用费 全部《资金占用初始化》中【国内证占用费】占用金额
            DynamicObject[] tpvZ = BusinessDataServiceHelper.load("tpv_zjzyll", "tpv_dayrate,tpv_deoccupyrate",
                    new QFilter[]{
                            new QFilter("tpv_begindate", QCP.less_equals, currentDate)
                            , new QFilter("tpv_enddate", QCP.large_equals, currentDate)}
            );//资金占用利率
            DynamicObject[] scFundsinitializeds = BusinessDataServiceHelper.load("tpv_sc_fundsinitializeds",
                    "tpv_entryentity,tpv_occupancyfee",
                    new QFilter[]{
                            new QFilter("tpv_entryentity.tpv_datebusiness", "=", currentDate)
                    }
            );//资金占用初始化
            for (DynamicObject scFundsinitialized : scFundsinitializeds) {
                DynamicObjectCollection collection = scFundsinitialized.getDynamicObjectCollection("tpv_entryentity");
                for (DynamicObject dynamicObject : collection) {
                    BigDecimal tpvOccupancyfee = dynamicObject.getBigDecimal("tpv_occupancyfee");
                    tpvTotalAmount = tpvTotalAmount.add(tpvOccupancyfee);
                }
            }
            DynamicObject currentRow = entryEntity.addNew();
            BigDecimal tpvIncrease = BigDecimal.ZERO;//国内证占用费 增加金额
            BigDecimal tpvDecrease = BigDecimal.ZERO;//国内证占用费 减少金额
            BigDecimal tpvAmount = BigDecimal.ZERO;//国内证占用费 前一天占用金额
            BigDecimal tpvAmountQ = BigDecimal.ZERO;//国内证占用费 期初数据（前一天的占用）
            BigDecimal supAmount = BigDecimal.ZERO;//供应链金融 期初数据（前一天的占用）占用金额
            BigDecimal tpvSupAmount = BigDecimal.ZERO;//供应链金融 前一天占用金额
            for (DynamicObject scCapitallimit : scCapitallimits) {
                Date tpvDatetime = scCapitallimit.getDate("tpv_datetime");
                String tpvAddorsub = scCapitallimit.getString("tpv_addorsub");//增减
                if (DateUtil.isSameDay(tpvDatetime, currentDate)) {
                    if ("A".equals(tpvAddorsub)) {//增加
                        tpvIncrease = tpvIncrease.add(scCapitallimit.getBigDecimal("tpv_orignalamount"));
                    }
                    if ("B".equals(tpvAddorsub)) {//减少
                        tpvDecrease = tpvDecrease.add(scCapitallimit.getBigDecimal("tpv_orignalamount"));
                    }
                }
            }

            DynamicObjectCollection entity = this.getModel().getEntryEntity(ENTITY_ENTRY);
            for (DynamicObject dynamicObject : entity) {
                Date tpvBusinessdates = dynamicObject.getDate("tpv_businessdates");
                if (entity.size() != 1) {
                    if (DateUtil.isSameDay(DateUtil.addDays(currentDate, -1), tpvBusinessdates)) {
                        tpvAmount = tpvAmount.add(dynamicObject.getBigDecimal("tpv_amount"));
                        tpvSupAmount = tpvSupAmount.add(dynamicObject.getBigDecimal("tpv_supamount"));
                    }
                } else if (entity.size() == 1) {
                    Date date = DateUtil.addDays(currentDate, -1);
                    DynamicObject[] tpvScSupfunddetails = BusinessDataServiceHelper.load("tpv_sc_supfunddetails",
                            "tpv_entryentity,tpv_amount,tpv_supamount",
                            new QFilter[]{new QFilter("tpv_entryentity.tpv_businessdates", "=", date)
                                    , new QFilter("tpv_org.id", "=", org.getPkValue())
                                    , new QFilter("tpv_depertment.id", "=", tpvDepartment.getPkValue())
                            });
                    for (DynamicObject tpvScSupfunddetail : tpvScSupfunddetails) {
                        DynamicObjectCollection tpvEntryentity = tpvScSupfunddetail.getDynamicObjectCollection("tpv_entryentity");
                        if (tpvEntryentity.size() != 0) {
                            tpvAmountQ = tpvAmountQ.add(tpvEntryentity.get(tpvEntryentity.size() - 1).getBigDecimal("tpv_amount"));
                            supAmount = supAmount.add(tpvEntryentity.get(tpvEntryentity.size() - 1).getBigDecimal("tpv_supamount"));
                        }

                    }
                }
            }

            //供应链金融
            String formatDateHms = DateUtil.formatDateHms(currentDate);
            BigDecimal tpvAmountF = BigDecimal.ZERO;//计税金额
            BigDecimal tpvPaidcapamt = BigDecimal.ZERO;//已还款金额
            DynamicObject[] tpvDtFeedoaresults = BusinessDataServiceHelper.load("tpv_dt_feedoaresults", "tpv_amount",
                    new QFilter[]{new QFilter("tpv_startdate", "=", formatDateHms)});//农业银行向中航推送融资放款信息
            for (DynamicObject tpvDtFeedoaresult : tpvDtFeedoaresults) {
                tpvAmountF = tpvAmountF.add(tpvDtFeedoaresult.getBigDecimal("tpv_amount"));
            }
            DynamicObject[] tpvDtReparesults = BusinessDataServiceHelper.load("tpv_dt_reparesult", "tpv_paidcapamt,tpv_expectloanamt",
                    new QFilter[]{new QFilter("tpv_repaydate", "=", formatDateHms)});//客户融资还款信息
            for (DynamicObject tpvDtReparesult : tpvDtReparesults) {
                tpvPaidcapamt = tpvPaidcapamt.add(tpvDtReparesult.getBigDecimal("tpv_expectloanamt"));
            }
            //供应链金融
            currentRow.set("tpv_supincrease", tpvAmountF);//增加
            currentRow.set("tpv_supdecrease", tpvPaidcapamt);//减少
            currentRow.set("tpv_supamount", tpvAmountF.subtract(tpvPaidcapamt).add(supAmount).add(tpvSupAmount));//占用金额
            //国内证占用费
            currentRow.set("tpv_companys", org);
            currentRow.set("tpv_dept", tpvDepartment);
            currentRow.set("tpv_businessdates", currentDate);
            currentRow.set("tpv_increase", tpvIncrease);//增加
            currentRow.set("tpv_decrease", tpvDecrease);//减少  //TODO 付款处理中付款金额
            currentRow.set("tpv_amount", tpvIncrease.subtract(tpvDecrease).add(tpvAmount).add(tpvTotalAmount).add(tpvAmountQ));
            //国内证占用费 利息
            if (tpvIncrease.subtract(tpvDecrease).add(tpvAmount).add(tpvTotalAmount).add(tpvAmountQ).compareTo(BigDecimal.ZERO) > 0) {
                if (tpvZ.length != 0) {
                    BigDecimal tpvDayrate = tpvZ[0].getBigDecimal("tpv_dayrate");
                    currentRow.set("tpv_interest", tpvIncrease.subtract(tpvDecrease).add(tpvAmount).add(tpvTotalAmount).add(tpvAmountQ).multiply(tpvDayrate));
                }
            } else {
                if (tpvZ.length != 0) {
                    BigDecimal tpvDeoccupyrate = tpvZ[0].getBigDecimal("tpv_deoccupyrate");
                    currentRow.set("tpv_interest", tpvIncrease.subtract(tpvDecrease).add(tpvAmount).add(tpvTotalAmount).add(tpvAmountQ).multiply(tpvDeoccupyrate));
                }
            }
            //供应链金融
            if (tpvAmountF.subtract(tpvPaidcapamt).add(supAmount).add(tpvSupAmount).compareTo(BigDecimal.ZERO) > 0) {
                if (tpvZ.length != 0) {
                    BigDecimal tpvDayrate = tpvZ[0].getBigDecimal("tpv_dayrate");
                    currentRow.set("tpv_supinterest", tpvAmountF.subtract(tpvPaidcapamt).add(supAmount).add(tpvSupAmount).multiply(tpvDayrate));
                }
            } else {
                if (tpvZ.length != 0) {
                    BigDecimal tpvDeoccupyrate = tpvZ[0].getBigDecimal("tpv_deoccupyrate");
                    currentRow.set("tpv_supinterest", tpvAmountF.subtract(tpvPaidcapamt).add(supAmount).add(tpvSupAmount).multiply(tpvDeoccupyrate));
                }
            }
            currentDate = DateUtil.addDays(currentDate, 1);

        }
        this.getView().updateView("tpv_entryentity");
    }


    /**
     * 生成部门资金占用明细
     *
     * @param startDate
     * @param org
     */
    private void createDeptDetails(Date startDate, DynamicObject org) {
        Date currentDate = startDate;
        Date endDay = DateUtil.getMonthEndDay(startDate);
        ArrayList<QFilter> filters = new ArrayList<>();
        String string = org.getString("name");
        filters.add(new QFilter("org.name", QCP.equals, org.getString("name")));

        DynamicObjectCollection entity = this.getModel().getEntryEntity(ENTITY_ENTRY);
        this.getModel().deleteEntryData(ENTITY_ENTRY);
        //获取中间表的数据
        DataSet mainDataSet = ORM.create().queryDataSet(this.getClass().getName(), "tpv_middle",
                        "tpv_middletables,tpv_middletables.tpv_text,tpv_middletables.tpv_combofield,tpv_middletables.tpv_date,tpv_middletables.tpv_amount,tpv_middletables.tpv_currency,tpv_middletables.tpv_coreno"
                        , filters.toArray(new QFilter[0]))
                .select("tpv_middletables.tpv_combofield source", "tpv_middletables.tpv_date date", "tpv_middletables.tpv_amount amount", "tpv_middletables.tpv_currency currency", "tpv_middletables.tpv_coreno coreno");

//        DataSet mainDataSet1 = ORM.create().queryDataSet(this.getClass().getName(), "tpv_sc_capitallimit",
//                        "tpv_operation,tpv_middletables.tpv_text,tpv_middletables.tpv_combofield,tpv_middletables.tpv_date,tpv_middletables.tpv_amount,tpv_middletables.tpv_currency,tpv_middletables.tpv_coreno"
//                        , filters.toArray(new QFilter[0]))
//                .select("tpv_middletables.tpv_combofield source", "tpv_middletables.tpv_date date", "tpv_middletables.tpv_amount amount", "tpv_middletables.tpv_currency currency", "tpv_middletables.tpv_coreno coreno");

        //按日期排序
        DataSet groupbyDataSet = mainDataSet.orderBy(new String[]{"date", "source", "amount", "currency", "coreno"});
        //数据格式转换
        DynamicObjectCollection curDyList = ORM.create().toPlainDynamicObjectCollection(groupbyDataSet);
        //TODO 将数据源从中间的表的数据 替换成 信用证的数据(修改 queryDate 方法即可)
        curDyList = (DynamicObjectCollection) queryDate(currentDate, endDay);
        if (curDyList.size() == 0) {

            this.getView().showTipNotification("当前月份暂无数据,请切换后重新查看!");
            return;
        }
        //去重开证处理id
        Set<String> curID = getAllCreditID(curDyList);
        //数据重构
        HashMap<String, HashMap<String, BigDecimal>> analyzedData = analyzeCurrentData(curDyList);
        //将当月数据按天分组
        HashMap<String, DynamicObjectCollection> dataMap = groupDataByDay(curDyList);
        //前一天金额占用
        BigDecimal lastDayOcp = BigDecimal.ZERO;
        //开始日期
        Date beginDate = DateUtil.addDays(curDyList.get(0).getDate("date"), -1);
        //业务处理
        while (currentDate.compareTo(endDay) <= 0) {
            DynamicObject currentRow = entity.addNew();
            String d1 = DateUtil.formatDate(currentDate, "YYYY-MM-dd");
            DynamicObjectCollection currentData = dataMap.get(d1);
            if (null != currentData) {
                HashMap<String, BigDecimal> map = getSupIncrease(analyzedData, currentData, lastDayOcp, curID, beginDate);
                lastDayOcp = map.get("tpv_amount");
                currentRow.set("tpv_businessdates", currentDate);
                currentRow.set("tpv_company", null);
                currentRow.set("tpv_dept", null);
                currentRow.set("tpv_supincrease", map.get("tpv_supincrease"));
                currentRow.set("tpv_supdecrease", map.get("tpv_supdecrease"));
                currentRow.set("tpv_supamount", map.get("tpv_supamount"));
                currentRow.set("tpv_supinterest", 0);
                currentRow.set("tpv_increase", map.get("tpv_increase"));
                currentRow.set("tpv_decrease", map.get("tpv_decrease"));
                currentRow.set("tpv_amount", map.get("tpv_amount"));
                currentRow.set("tpv_interest", getRate(currentDate, map.get("tpv_amount")));
            } else {
                currentRow.set("tpv_businessdates", currentDate);
                currentRow.set("tpv_amount", lastDayOcp);
            }
            currentDate = DateUtil.addDays(currentDate, 1);
        }
        this.getView().updateView("tpv_entryentity");

    }

    private BigDecimal getRate(Date currentDate, BigDecimal ocp) {
        ArrayList<QFilter> filters = new ArrayList<>();

        filters.add(new QFilter("tpv_begindate", QCP.less_equals, currentDate));
        filters.add(new QFilter("tpv_enddate", QCP.large_equals, currentDate));
        filters.add(new QFilter("enable", QCP.equals, "1"));
        DataSet mainDataSet = ORM.create().queryDataSet(this.getClass().getName(), "tpv_zjzyll",
                        "tpv_dayrate,tpv_deoccupyrate"
                        , filters.toArray(new QFilter[0]))
                .select("tpv_dayrate dayrate", "tpv_deoccupyrate deoccupyrate");
        DynamicObjectCollection curDyList = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        if (curDyList.size() == 0) {
            return BigDecimal.ZERO;
        }
        BigDecimal dayrate = BigDecimal.ZERO;
        BigDecimal deoccupyrate = BigDecimal.ZERO;
        for (DynamicObject object : curDyList) {
            dayrate = object.getBigDecimal("dayrate");
            deoccupyrate = object.getBigDecimal("deoccupyrate");
            break;
        }
        if (ocp.compareTo(BigDecimal.ZERO) < 0) {
            return ocp.multiply(deoccupyrate);
        } else {
            return ocp.multiply(dayrate);
        }
    }

    /**
     * 将当月数据按天分组
     *
     * @param curDyList
     * @return
     */
    private HashMap<String, DynamicObjectCollection> groupDataByDay(DynamicObjectCollection curDyList) {
        HashMap<String, DynamicObjectCollection> dataMap = new HashMap<>();
        for (DynamicObject col : curDyList) {
            String string = col.toString();
            String tpvDate = DateUtil.formatDate(col.getDate("date"), "YYYY-MM-dd");
            DynamicObjectCollection collection1 = dataMap.get(tpvDate);
            if (null == collection1) {
                DynamicObjectCollection tpvMiddletables = new DynamicObjectCollection();
                tpvMiddletables.add(col);
                dataMap.put(tpvDate, tpvMiddletables);
            } else {
                collection1.add(col);
                dataMap.put(tpvDate, collection1);
            }
        }
        return dataMap;
    }

    /**
     * 信用证ID去重
     *
     * @param curDyList
     * @return
     */
    private Set<String> getAllCreditID(DynamicObjectCollection curDyList) {
        Set<String> curID = new HashSet<>();//所有开证处理id
        for (DynamicObject dynamicObject : curDyList) {
            String combofield = dynamicObject.getString("source");//数据来源
            if (combofield.equals("A")) {
                String string = dynamicObject.getString("coreno");
                curID.add(string);
            }
        }
        return curID;
    }

    /**
     * 计算每一天的数据
     *
     * @param dataMap
     * @param currentData
     * @param lastDayOcp
     * @param coreNumCol
     * @param beginDate
     * @return
     */
    private HashMap<String, BigDecimal> getSupIncrease(HashMap<String, HashMap<String, BigDecimal>> dataMap, List<DynamicObject> currentData, BigDecimal lastDayOcp, Set<String> coreNumCol, Date beginDate) {
        Set<String> coreNums = new HashSet<>();
        String dateStr = "";
        Date monthFirstDay = null;
        HashMap<String, BigDecimal> map = new HashMap<>(3);
        BigDecimal supIncrease = new BigDecimal(0);//增加(供应链金融)
        BigDecimal supDecrease = new BigDecimal(0);//减少(供应链金融)
        BigDecimal supAmounts = new BigDecimal(0);//占用(供应链金融)
        BigDecimal increase = new BigDecimal(0);//增加(国内证占用费)
        BigDecimal decrease = new BigDecimal(0);//减少(国内证占用费)
        BigDecimal amounts = new BigDecimal(0);//占用(国内证占用费)
        for (DynamicObject object : currentData) {
            String coreNum = object.getString("coreno");
            Date date = object.getDate("date");
            monthFirstDay = DateUtil.getMonthFirstDay(date);
            dateStr = DateUtil.formatDate(date);

            if (object.getString("source").equals("F")) {//付款
                decrease = decrease.add(object.getBigDecimal("amount"));
            }

            String string = object.getString("coreno");
            coreNums.add(string);
        }
        BigDecimal before = BigDecimal.ZERO;
        BigDecimal desc = BigDecimal.ZERO;
        for (String core : coreNumCol) {
            HashMap<String, BigDecimal> decimalHashMap = dataMap.get(core);

            HashMap<String, BigDecimal> ocpAmount = getDayOcp(decimalHashMap, dateStr, beginDate, monthFirstDay);//信用证当天的金额占用,减少,期初
            BigDecimal today = ocpAmount.get("today");
            desc = desc.add((ocpAmount.get("desc")) == null ? BigDecimal.ZERO : ocpAmount.get("desc"));
            if (desc.compareTo(BigDecimal.ZERO) != 0) {
                today = BigDecimal.ZERO;
                decrease = decrease.subtract(desc);
                desc = BigDecimal.ZERO;
            }
            before = before.add((ocpAmount.get("before")) == null ? BigDecimal.ZERO : ocpAmount.get("before"));
            if (null == today) {
                today = BigDecimal.ZERO;
            }
            if (null == before) {
                before = BigDecimal.ZERO;
            }
            amounts = amounts.add(today);
        }
        lastDayOcp = lastDayOcp.add(before);
        map.put("tpv_increase", amounts.subtract(lastDayOcp).add(decrease).subtract(before));
        map.put("tpv_decrease", decrease);
        map.put("tpv_amount", amounts);
        //TODO 处理查询到的供应链金融的数据(当天的数据)
        map.put("supIncrease", null);//TODO
        map.put("supDecrease", null);//TODO
        map.put("supAmounts", null);//TODO
        return map;
    }

    private HashMap<String, BigDecimal> getDayOcp(HashMap<String, BigDecimal> decimalHashMap, String dateStr, Date beginDate, Date monthFirstDay) {
        HashMap<String, BigDecimal> map = new HashMap<>();
        if (decimalHashMap == null) {
            return null;
        }
        BigDecimal decimal = decimalHashMap.get(dateStr) == null ? BigDecimal.ZERO : decimalHashMap.get(dateStr);
        if (decimal.compareTo(BigDecimal.ZERO) < 0) {
            map.put("desc", decimal);
        } else {
            map.put("today", decimal);
        }
        Date date = DateUtil.parseDate(dateStr);
        String dateStrs = dateStr;
        while (decimal.equals(BigDecimal.ZERO) && date.after(beginDate)) {
            //计算期初
            if (date.before(monthFirstDay) && dateStrs.substring(8).equals("01")) {
                decimal = decimalHashMap.get(dateStr);
                map.put("before", decimal);
            }
            //计算占用
            decimal = decimalHashMap.get(dateStr) == null ? BigDecimal.ZERO : decimalHashMap.get(dateStr);
            if (decimal.compareTo(BigDecimal.ZERO) < 0) {
                decimal = BigDecimal.ZERO;
                map.put("today", decimal);
                break;
            }
            map.put("today", decimal);
            Date before = DateUtil.addDays(date, -1);
            dateStr = DateUtil.formatDate(before);
            date = before;
        }
        return map;
    }


    /**
     * 获取时间
     *
     * @param year
     * @param month
     */
    private Date getQueryDate(Date year, String month) {
        String mon = "";
        if (month == null) {

        }
        switch (month) {
            case "Jan":
                mon = "01";
                break;
            case "Feb":
                mon = "02";
                break;
            case "Mar":
                mon = "03";
                break;
            case "Apr":
                mon = "04";
                break;
            case "May":
                mon = "05";
                break;
            case "June":
                mon = "06";
                break;
            case "July":
                mon = "07";
                break;
            case "Aug":
                mon = "08";
                break;
            case "Sept":
                mon = "09";
                break;
            case "Oct":
                mon = "10";
                break;
            case "Nov":
                mon = "11";
                break;
            case "Dec":
                mon = "12";
                break;
            default:
                break;

        }
        LocalDateTime dateTime = year.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
        int year1 = DateUtil.getYear(year);
        Date date = DateUtil.parseDate(String.valueOf(year1) + "-" + mon + "-" + "01");
        return date;
    }

    private boolean compareToDate(Date date1, String d2) {
        String d1 = DateUtil.formatDate(date1, "YYYY-MM-dd");
        return d1.equals(d2);

    }

    private Collection<DynamicObject> queryDate(Date startDate, Date endDay) {
        //信用证状态 已登记          已闭卷       已撤证        改卷中       撤证中       闭卷中
        //         done_register  done_close  done_repeal  change_ing  repeal_ing  close_ing
        //开证
        ArrayList<QFilter> kzFilters = new ArrayList<>();
        kzFilters.add(new QFilter("bizdate", QCP.less_equals, endDay));
        kzFilters.add(new QFilter("bizdate", QCP.large_equals, startDate));
        kzFilters.add(new QFilter("creditstatus", QCP.equals, "done_register"));//信用证状态 = 已登记
        kzFilters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
        DataSet kzLoad = ORM.create().queryDataSet(this.getClass().getName(), "lc_lettercredit",
                        "billstatus,bizdate,amount,currency,creditstatus,billno,biztype"
                        , kzFilters.toArray(new QFilter[0]))
                .select("bizdate date", "amount", "currency", "billno coreno", "biztype source");//,"A source"

        //到单
        ArrayList<QFilter> ddFilters = new ArrayList<>();
        ddFilters.add(new QFilter("arrivaldate", QCP.less_equals, endDay));
        ddFilters.add(new QFilter("arrivaldate", QCP.large_equals, startDate));
        ddFilters.add(new QFilter("arrivaltype", QCP.equals, "credit"));//到单类型 = 信用证
        ddFilters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
        DataSet ddLoad = ORM.create().queryDataSet(this.getClass().getName(), "lc_arrival",
                        "billstatus,arrivaldate,arrivalamount,arrivalcurrency,arrivaltype,billno"//缺少核心单据编码
                        , ddFilters.toArray(new QFilter[0]))
                .select("arrivaldate date", "arrivalamount amount", "arrivalcurrency currency", "billno coreno", "biztype source");//,"E source","coreno"

        //付款
        ArrayList<QFilter> fkFilters = new ArrayList<>();
        fkFilters.add(new QFilter("paydate", QCP.less_equals, endDay));
        fkFilters.add(new QFilter("paydate", QCP.large_equals, startDate));
        fkFilters.add(new QFilter("sourcebilltype", QCP.equals, "lc_arrival"));//源单类型 = 到单处理
        fkFilters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
        DataSet fkLoad = ORM.create().queryDataSet(this.getClass().getName(), "cas_paybill",
                        "billstatus,paydate,actpayamt,currency,sourcebilltype,billno"//缺少核心单据编码
                        , fkFilters.toArray(new QFilter[0]))
                .select("paydate date", "actpayamt amount", "currency", "billno coreno", "biztype source");//,"F source","coreno"

        //提款
        ArrayList<QFilter> tkFilters = new ArrayList<>();
        tkFilters.add(new QFilter("bizdate", QCP.less_equals, endDay));
        tkFilters.add(new QFilter("bizdate", QCP.large_equals, startDate));
        tkFilters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
        DataSet tkLoad = ORM.create().queryDataSet(this.getClass().getName(), "cfm_loanbill",
                        "billstatus,bizdate,convertdrawamt,contractcny,billno"
                        , tkFilters.toArray(new QFilter[0]))
                .select("bizdate date", "convertdrawamt amount", "contractcny currency", "billno coreno", "biztype source");

        //还款
        ArrayList<QFilter> hkFilters = new ArrayList<>();
        hkFilters.add(new QFilter("bizdate", QCP.less_equals, endDay));
        hkFilters.add(new QFilter("bizdate", QCP.large_equals, startDate));
        hkFilters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
        DataSet hkLoad = ORM.create().queryDataSet(this.getClass().getName(), "cfm_repaymentbill",
                        "billstatus,bizdate,amount,currency,billno"
                        , tkFilters.toArray(new QFilter[0]))
                .select("bizdate date", "amount", "currency", "billno coreno", "biztype source");

        DataSet union = kzLoad.union(ddLoad, hkLoad, fkLoad, tkLoad);
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(union);
        //TODO 根据开发计划  1.上面是国内证占用费的查询方法,还需供应链金融的查询方法   2.根据日期排序
        return col_s;
    }

    /**
     * 计算上月最后一天的占用
     *
     * @param lastID
     * @return
     */
    private BigDecimal calculateLastOcp(Set<String> lastID, DynamicObjectCollection lastDyList) {
      /*  //开证
        ArrayList<QFilter> kzFilters = new ArrayList<>();
        kzFilters.add(new QFilter("id",QCP.equals,lastID));
        kzFilters.add(new QFilter("creditstatus",QCP.equals,"done_register"));//信用证状态 = 已登记
        kzFilters.add(new QFilter("billstatus",QCP.equals,"C"));//已审核
        DataSet kzLoad = ORM.create().queryDataSet(this.getClass().getName(),"lc_lettercredit",
                        "billstatus,bizdate,amount,currency,creditstatus"
                        ,kzFilters.toArray(new QFilter[0]))
                .select("bizdate date","amount","currency");

        //到单
        ArrayList<QFilter> ddFilters = new ArrayList<>();
        ddFilters.add(new QFilter("核心单据编码",QCP.equals,lastID));
        ddFilters.add(new QFilter("arrivaltype",QCP.equals,"credit"));//到单类型 = 信用证
        ddFilters.add(new QFilter("billstatus",QCP.equals,"C"));//已审核
        DataSet ddLoad = ORM.create().queryDataSet(this.getClass().getName(),"lc_arrival",
                        "billstatus,arrivaldate,arrivalamount,arrivalcurrency,arrivaltype"
                        ,ddFilters.toArray(new QFilter[0]))
                .select("arrivaldate date","arrivalamount amount","arrivalcurrency currency");

        //付款
        ArrayList<QFilter> fkFilters = new ArrayList<>();
        fkFilters.add(new QFilter("核心单据编码",QCP.equals,lastID));
        fkFilters.add(new QFilter("sourcebilltype",QCP.equals,"lc_arrival"));//源单类型 = 到单处理
        fkFilters.add(new QFilter("billstatus",QCP.equals,"C"));//已审核
        DataSet fkLoad = ORM.create().queryDataSet(this.getClass().getName(),"cas_paybill",
                        "billstatus,paydate,actpayamt,currency,sourcebilltype"
                        ,fkFilters.toArray(new QFilter[0]))
                .select("paydate date","actpayamt amount","currency");
        DataSet union = kzLoad.union(ddLoad,fkLoad);
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(union);

*/
        HashMap<String, HashMap<String, BigDecimal>> listHashMap = new HashMap<>();//期初数据
        for (DynamicObject col : lastDyList) {
            String source = col.getString("source");
            Date date = col.getDate("date");
            BigDecimal amount = col.getBigDecimal("amount");
            String coreno = col.getString("coreno");
            HashMap<String, BigDecimal> map = listHashMap.get(coreno);
            if (null == map) {
                map = new HashMap<String, BigDecimal>();
                map.put("开证处理", BigDecimal.ZERO);
                map.put("改证处理", BigDecimal.ZERO);
                map.put("到单处理", BigDecimal.ZERO);
                map.put("付款处理", BigDecimal.ZERO);
            }
            if ("A".equals(source)) {
                map.put("开证处理", amount);
            }
            if ("D".equals(source)) {
                map.put("改证处理", amount);
            }
            if ("E".equals(source)) {
                map.put("到单处理", amount.add(map.get("到单处理")));
            }
            if ("F".equals(source)) {
                map.put("付款处理", amount.add(map.get("付款处理")));
            }
            listHashMap.put(coreno, map);
        }
        //计算占用
        Set<String> keySet = listHashMap.keySet();
        BigDecimal lastOcp = new BigDecimal("0");
        for (String s : keySet) {
            HashMap<String, BigDecimal> map = listHashMap.get(s);
            BigDecimal maxNum = getBigNum(map.get("开证处理"), map.get("改证处理"), map.get("到单处理"));
            lastOcp = lastOcp.add(maxNum.subtract(map.get("付款处理")));
        }
        return lastOcp;
    }

    private HashMap<String, HashMap<String, BigDecimal>> analyzeCurrentData(DynamicObjectCollection mainDataSet) {
        HashMap<String, HashMap<String, BigDecimal>> listHashMap = new HashMap<>();//当月数据
        for (DynamicObject col : mainDataSet) {
            String source = col.getString("source");
            Date date = col.getDate("date");
            String dateStr = DateUtil.formatDate(date);
            BigDecimal amount = col.getBigDecimal("amount");
            String coreno = col.getString("coreno");
            BigDecimal maxNum = BigDecimal.ZERO;
            HashMap<String, BigDecimal> map = listHashMap.get(coreno);
            if (null == map) {
                map = new HashMap<String, BigDecimal>();
                map.put("开证处理", BigDecimal.ZERO);
                map.put("改证处理", BigDecimal.ZERO);
                map.put("到单处理", BigDecimal.ZERO);
                map.put("付款处理", BigDecimal.ZERO);
                map.put("闭卷处理", BigDecimal.ZERO);
                map.put("撤证处理", BigDecimal.ZERO);
            }
            if ("A".equals(source)) {
                map.put("开证处理", amount);
            }
            if ("B".equals(source)) {
                map.put("闭卷处理", null);
            }
            if ("C".equals(source)) {
                map.put("撤证处理", null);
            }
            if ("D".equals(source)) {
                map.put("改证处理", amount);
            }
            if ("E".equals(source)) {
                map.put("到单处理", amount.add(map.get("到单处理")));
            }
            if ("F".equals(source)) {
                map.put("付款处理", amount.add(map.get("付款处理")));
            }
            BigDecimal subtract = BigDecimal.ZERO;
            if (null == (map.get("闭卷处理")) || null == (map.get("撤证处理"))) {

                maxNum = getBigNum(map.get("开证处理"), map.get("改证处理"), map.get("到单处理"));
                subtract = maxNum.subtract(map.get("付款处理"));
                map.put(dateStr, BigDecimal.ZERO.subtract(subtract));

            } else {
                maxNum = getBigNum(map.get("开证处理"), map.get("改证处理"), map.get("到单处理"));
                subtract = maxNum.subtract(map.get("付款处理"));
                map.put(dateStr, subtract);
            }

            listHashMap.put(coreno, map);
        }
        return listHashMap;
    }

    private BigDecimal getBigNum(BigDecimal... nums) {
        BigDecimal max = nums[0];
        for (BigDecimal num : nums) {
            if (num.compareTo(max) > 0) {
                max = num;
            }
        }
        return max;
    }

}
