package com.itopview.wm.sc.opplugin.op;

import kd.bamp.mbis.common.mega.model.OrgViewType;
import kd.bamp.mbis.common.util.DateUtil;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.clr.DataEntityPropertyCollection;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.DeleteServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.servicehelper.org.OrgUnitServiceHelper;
import kd.tmc.fbp.common.helper.BillChangeHistoryHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Predicate;
import java.util.stream.Stream;

/**
 * @ClassName: CapitalLimitFormPlugin
 * @Description: 资金额度数据来源插件(放款结果反馈底表, 还款结果反馈底表, )
 * @date: 2023/11/22 10:04
 * @author: chen
 */

public class CapitalLimitServiceOp extends AbstractOperationServicePlugIn {
    public static final String FORMID_FeedResults = "tpv_dt_feedoaresults";//放款结果反馈底表
    public static final String FORMID_RepareResults = "tpv_dt_reparesult";//还款结果反馈底表
    public static final String FORMID_LetterCredit = "lc_lettercredit";//开征处理
    public static final String FORMID_Arrival = "lc_arrival";//到单处理
    public static final String FORMID_CapitalLimit = "tpv_sc_capitallimit";//资金额度
    public static final String algo_Key = "CapitalLimitServiceOp";

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_purnumbers");
        keys.add("tpv_combinationnum");
        keys.add("tpv_tjtype");
        keys.add("creditstatus");
        keys.add("tpv_rate");
        keys.add("amount");
        keys.add("isnationalcard");
        keys.add("creditno");
        keys.add("tpv_kzsqusdrate");
        keys.add("tpv_overfillrate");
        keys.add("tpv_department");
        keys.add("tpv_kzdate");
        keys.add("applydate");
        keys.add("tpv_operator");
        keys.add("tpv_loanapplyno");
        keys.add("billno");
        keys.add("currency");
        keys.add("tpv_people");
        keys.add("biztype");
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        List<ExtendedDataEntity> selectedRows = e.getSelectedRows();
        DynamicObject entity = selectedRows.get(0).getDataEntity();
        String formID = entity.getDataEntityType().getName();
        String operationKey = e.getOperationKey();
        if ("save".equals(operationKey) || "audit".equals(operationKey)) {//保存 审核增加数据
            divideSrcData(operationKey, selectedRows, formID);
        }
        if ("unaudit".equals(operationKey)) {//反审核删除数据
            ArrayList<Long> deleteDataID = new ArrayList<>();
            selectedRows.stream().forEach(row -> {
                deleteDataID.add(row.getDataEntity().getLong("id"));
            });
            ArrayList<QFilter> filters = new ArrayList<>();
            filters.add(new QFilter("tpv_triggerbillid", QCP.in, deleteDataID));//单据编号
            DeleteServiceHelper.delete(FORMID_CapitalLimit, filters.toArray(new QFilter[0]));

        }
    }

    /**
     * @return void
     * @author: chen
     * @date: 2023/11/23 10:48
     * @description: 数据分类
     */
    private void divideSrcData(String operationKey, List<ExtendedDataEntity> entities, String formID) {
        List<HashMap<String, Object>> dataList = new ArrayList<>(entities.size());
        switch (formID) {
            case FORMID_FeedResults://放款结果反馈底表(保存触发)
                entities.forEach(obj -> {
                    DynamicObject entity = obj.getDataEntity();
                    HashMap<String, Object> map = new HashMap<>();
                    Long pkValue = (Long) entity.getPkValue();//单据ID
                    String startDate = entity.getString("tpv_startdate");
                    map.put("tpv_type", "L");//类别
                    map.put("tpv_addorsub", "A");//增减
                    map.put("tpv_realpreocp", "A");//真实/预占用
                    map.put("tpv_datetime", DateUtil.parseDate(startDate.isEmpty() ? null : startDate));//日期
                    map.put("tpv_org", entity.getLong("org.id"));//公司
                    map.put("tpv_bigdept", getSupDepartment(entity.getLong("tpv_department.id")));//大部门(额度使用部门)
                    map.put("tpv_depertment", entity.getLong("tpv_department.id"));//对应单据记录部门
                    map.put("tpv_amount", entity.getBigDecimal("amount"));//金额(本位币)
                    map.put("tpv_combinations", getCombinationBySaleNum(entity.getString("tpv_orderno")));//组合号
                    map.put("tpv_salenum", entity.getString("tpv_orderno"));//销售合同号
                    map.put("tpv_purnum", "");//采购合同号
                    map.put("tpv_bonustype", "");//计奖类型
                    map.put("tpv_user", entity.getLong("tpv_operator.id"));//业务员
                    map.put("tpv_corenum", entity.getString("tpv_loanapplyno"));//核心号
                    map.put("tpv_triggerbill", pkValue);//触发单据
                    map.put("tpv_triggerbillno", entity.getString("tpv_loanapplyno"));//触发单据编号
                    map.put("tpv_triggerbillid", pkValue);//触发单据ID
                    map.put("tpv_triggerbillentryids", "");//触发单据分录ID
                    map.put("tpv_triggerdate", new Date());//触发日期
                    map.put("tpv_coreid", entity.getLong("id"));//核心id(隐藏)
                    map.put("tpv_operation", "放款");//操作
                    dataList.add(map);
                });
                break;
            case FORMID_RepareResults://还款结果反馈底表(保存触发)
                entities.forEach(obj -> {
                    DynamicObject entity = obj.getDataEntity();
                    HashMap<String, Object> map = new HashMap<>();
                    Long pkValue = (Long) entity.getPkValue();//单据ID
                    map.put("tpv_type", "L");//类别
                    map.put("tpv_addorsub", "B");//增减
                    map.put("tpv_realpreocp", "A");//真实/预占用
                    map.put("tpv_datetime", DateUtil.parseDate(entity.getString("tpv_repaydate")));//日期
                    map.put("tpv_org", entity.getLong("org.id"));//公司
                    map.put("tpv_bigdept", getSupDepartment(entity.getLong("tpv_department.id")));//大部门(额度使用部门)
                    map.put("tpv_depertment", entity.getLong("tpv_department.id"));//对应单据记录部门
                    map.put("tpv_amount", entity.getBigDecimal("tpv_thispaycapamt"));//金额(本位币)
                    map.put("tpv_combinations", getCombinationBySaleNum(entity.getString("tpv_orderno")));//组合号
                    map.put("tpv_salenum", entity.getString("tpv_orderno"));//销售合同号
                    map.put("tpv_purnum", "");//采购合同号
                    map.put("tpv_bonustype", "");//计奖类型
                    map.put("tpv_user", entity.getLong("tpv_operator.id"));//业务员
                    map.put("tpv_corenum", entity.getString("tpv_loanapplyno"));//核心号
                    map.put("tpv_triggerbill", pkValue);//触发单据
                    map.put("tpv_triggerbillno", entity.getString("tpv_vchno"));//触发单据编号
                    map.put("tpv_triggerbillid", pkValue);//触发单据ID
                    map.put("tpv_triggerbillentryids", "");//触发单据分录ID
                    map.put("tpv_triggerdate", new Date());//触发日期
                    map.put("tpv_coreid", entity.getLong("id"));//核心id(隐藏)
                    map.put("tpv_operation", "还款");//操作
                    dataList.add(map);
                });
                break;
            case FORMID_LetterCredit://开征处理 (审核触发)
                entities.forEach(obj -> {
                    DynamicObject entity = obj.getDataEntity();
                    HashMap<String, Object> map = new HashMap<>();
                    HashMap<String, String> entryData = getEntryData(Optional.ofNullable(entity.getDynamicObjectCollection("tpv_contractdetail")));
                    String status = entity.getString("biztype");//业务类型 开证:open_card 改证:edit_card
                    if ("open_card".equals(status)) {
                        Long pkValue = (Long) entity.getPkValue();//单据ID
                        boolean isNationalCard = entity.getBoolean("isnationalcard");//是否国际证
                        BigDecimal amount = entity.getBigDecimal("amount");//开征金额
                        BigDecimal rate = entity.getBigDecimal("tpv_rate");//汇率
                        rate = rate.compareTo(new BigDecimal(0)) == 0 ? new BigDecimal(1) : rate;//汇率
                        BigDecimal overFillRate = entity.getBigDecimal("tpv_overfillrate");//金额溢装率
                        BigDecimal kzToUSDRate = entity.getBigDecimal("tpv_kzsqusdrate");//开证折USD汇率
                        BigDecimal curAmount = calculateAmount(amount, rate, overFillRate);//金额(本位币)
                        HashMap<String, BigDecimal> historyAmount = getAmountMax(pkValue, null, entity.getString("creditno"));
                        BigDecimal maxAmount = historyAmount.get("maxAmount");
                        if (maxAmount.compareTo(curAmount) > 0) {//本单金额小于历史最大值,不进行插入值
                            return;
                        }

                        //资金额度字段
                        BigDecimal currencyAmount = curAmount.subtract(maxAmount);//金额(本位币)
                        BigDecimal originalAmount = currencyAmount.divide(rate, 2, BigDecimal.ROUND_HALF_UP);//原币金额

                        map.put("tpv_type", isNationalCard ? "I" : "J");//类别
                        map.put("tpv_addorsub", "A");//增减
                        map.put("tpv_realpreocp", "A");//真实/预占用
                        map.put("tpv_datetime", DateUtil.parseDate(entity.getDate("applydate")));//日期
                        map.put("tpv_org", entity.getLong("org.id"));//公司
                        map.put("tpv_bigdept", getSupDepartment(entity.getLong("tpv_department.id")));//大部门(额度使用部门)
                        map.put("tpv_depertment", entity.getLong("tpv_department.id"));//对应单据记录部门
                        map.put("tpv_amount", currencyAmount);//金额(本位币)
                        map.put("tpv_combinations", entryData.get("combination"));//组合号
                        map.put("tpv_salenum", "");//销售合同号
                        map.put("tpv_purnum", entryData.get("purnum"));//采购合同号
                        map.put("tpv_bonustype", entryData.get("bonusType"));//计奖类型
                        map.put("tpv_user", entity.getLong("tpv_people.id"));//业务员
                        map.put("tpv_corenum", entity.getString("billno"));//核心号
                        map.put("tpv_triggerbill", pkValue);//触发单据
                        map.put("tpv_triggerbillno", entity.getString("billno"));//触发单据编号
                        map.put("tpv_triggerbillid", pkValue);//触发单据ID
                        map.put("tpv_triggerbillentryids", entryData.get("entryID"));//触发单据分录ID
                        map.put("tpv_triggerdate", new Date());//触发日期
                        map.put("tpv_currency", entity.getLong("currency.id"));//币种
                        map.put("tpv_orignalamount", originalAmount);//原币金额
                        map.put("tpv_usdamount", originalAmount.multiply(kzToUSDRate));//美元金额
                        map.put("tpv_tousdrate", kzToUSDRate);//对美元汇率
                        map.put("tpv_tormbrate", rate);//对人民币汇率
                        map.put("tpv_coreid", pkValue);//核心id(隐藏)
                        map.put("tpv_operation", "开证");//操作
                        dataList.add(map);
                    }
                });
                break;
            case FORMID_Arrival://到单处理(审核触发)
                entities.forEach(obj -> {
                    DynamicObject entity = obj.getDataEntity();
                    HashMap<String, Object> map = new HashMap<>();
                    HashMap<String, String> entryData = getEntryData(Optional.ofNullable(entity.getDynamicObjectCollection("tpv_contractdetail")));
                    Long pkValue = (Long) entity.getPkValue();//到单ID
                    boolean isNationalCard = entity.getBoolean("lettercredit.isnationalcard");//是否国际证
                    BigDecimal amount = entity.getBigDecimal("arrivalamount");//到单金额
                    String letterCredit = entity.getString("lettercredit.number");//信用证号
                    BigDecimal kzToUSDRate = entity.getBigDecimal("tpv_kzsqusdrate");//开证折USD汇率
                    DynamicObject credit = getLetterCreditByCreditNo(letterCredit, "id,tpv_department.id,tpv_people.id,billno");//获取开证ID指定字段
                    //开征处理字段
                    Long kzID = 0L;
                    Long department = 0L;
                    Long people = 0L;
                    String billNo = "";
                    if (credit != null){
                        kzID = credit.getLong("id");
                        people = credit.getLong("tpv_people.id");
                        department = credit.getLong("tpv_department.id");
                        billNo = credit.getString("billno");
                    }
                    HashMap<String, BigDecimal> historyAmount = getAmountMax(kzID, pkValue, letterCredit);
                    BigDecimal modifyAmount = historyAmount.get("modifyAmount");
                    BigDecimal arrivalAmount = historyAmount.get("arrivalAmount");
                    //到单汇总值小于改证
                    if (arrivalAmount.add(amount).compareTo(modifyAmount) <= 0) {
                        return;
                    }
                    //资金额度字段
                    BigDecimal currencyAmount = arrivalAmount.add(amount).subtract(modifyAmount);//金额(本位币)
                    BigDecimal originalAmount = currencyAmount.divide(historyAmount.get("rate"), 2, BigDecimal.ROUND_HALF_UP) ;//原币金额
                    BigDecimal USDAmount = currencyAmount.multiply(kzToUSDRate);//美元金额

                    map.put("tpv_type", isNationalCard ? "I" : "J");//类别
                    map.put("tpv_addorsub", "A");//增减
                    map.put("tpv_realpreocp", "A");//真实/预占用
                    map.put("tpv_datetime", DateUtil.parseDate(entity.getDate("recorddate")));//日期
                    map.put("tpv_org", entity.getLong("org.id"));//公司
                    map.put("tpv_bigdept", getSupDepartment(department));//大部门(额度使用部门)
                    map.put("tpv_depertment", department);//对应单据记录部门
                    map.put("tpv_amount", currencyAmount);//金额(本位币)
                    map.put("tpv_combinations", entryData.get("combination"));//组合号
                    map.put("tpv_salenum", "");//销售合同号
                    map.put("tpv_purnum", entryData.get("purnum"));//采购合同号
                    map.put("tpv_bonustype", entryData.get("bonusType"));//计奖类型
                    map.put("tpv_user", people);//业务员
                    map.put("tpv_corenum", billNo);//核心号
                    map.put("tpv_triggerbill", pkValue);//触发单据
                    map.put("tpv_triggerbillno", entity.getString("billno"));//触发单据编号
                    map.put("tpv_triggerbillid", pkValue);//触发单据ID
                    map.put("tpv_triggerbillentryids", entryData.get("entryID"));//触发单据分录ID
                    map.put("tpv_triggerdate", new Date());//触发日期
                    map.put("tpv_currency", entity.getLong("arrivalcurrency.id"));//币种
                    map.put("tpv_orignalamount", originalAmount);//原币金额
                    map.put("tpv_usdamount", USDAmount);//美元金额
                    map.put("tpv_tousdrate", kzToUSDRate);//对美元汇率
                    map.put("tpv_tormbrate", entity.getBigDecimal("tpv_rate"));//对人民币汇率
                    map.put("tpv_coreid", pkValue);//核心id(隐藏)
                    map.put("tpv_operation", "到单");//操作
                    dataList.add(map);
                });
                break;
            default:
                break;
        }
        //将获取到的数据插入到资金额度表中
        setDataToBill(dataList);
    }

    /**
     * @param pkID:
     * @return void
     * @author: chen
     * @date: 2023/11/29 10:37
     * @description: 闭卷 撤证
     */
    public void closeCredit(Long pkID, String operationKey) {
        List<HashMap<String, Object>> dataList = new ArrayList<>(1);
        if ("repeal".equals(operationKey) || "close".equals(operationKey)) {//撤证 闭卷
            HashMap<String, Object> map = new HashMap<>();
            DynamicObject entity = BusinessDataServiceHelper.loadSingle(pkID, FORMID_LetterCredit);
            HashMap<String, String> entryData = getEntryData(Optional.ofNullable(entity.getDynamicObjectCollection("tpv_contractdetail")));
            String status = entity.getString("creditstatus");//信用证状态 撤证:done_repeal 闭卷:done_close 已登记:done_register
            boolean isNationalCard = entity.getBoolean("isnationalcard");//是否国际证
            BigDecimal kzToUSDRate = entity.getBigDecimal("tpv_kzsqusdrate");//开证折USD汇率

            //资金额度字段
            Date applyDate = entity.getDate("done_repeal".equals(status) ? "tpv_repealdate" : "tpv_closetime");
            HashMap<String, BigDecimal> changedAmount = getChangedAmount(pkID);
            BigDecimal currencyAmount = changedAmount.get("tpv_amount");//金额(本位币)
            BigDecimal originalAmount = changedAmount.get("tpv_orignalamount");//原币金额
            BigDecimal USDAmount = changedAmount.get("tpv_usdamount");//美元金额

            map.put("tpv_type", isNationalCard ? "I" : "J");//类别
            map.put("tpv_addorsub", "B");//增减
            map.put("tpv_realpreocp", "A");//真实/预占用
            map.put("tpv_datetime", DateUtil.parseDate(applyDate));//日期
            map.put("tpv_org", entity.getLong("org.id"));//公司
            map.put("tpv_bigdept", getSupDepartment(entity.getLong("tpv_department.id")));//大部门(额度使用部门)
            map.put("tpv_depertment", entity.getLong("tpv_department.id"));//对应单据记录部门
            map.put("tpv_amount", currencyAmount);//金额(本位币)
            map.put("tpv_combinations", entryData.get("combination"));//组合号
            map.put("tpv_salenum", "");//销售合同号
            map.put("tpv_purnum", entryData.get("purnum"));//采购合同号
            map.put("tpv_bonustype", entryData.get("bonusType"));//计奖类型
            map.put("tpv_user", entity.getLong("tpv_people.id"));//业务员
            map.put("tpv_corenum", entity.getString("billno"));//核心号
            map.put("tpv_triggerbill", pkID);//触发单据
            map.put("tpv_triggerbillno", entity.getString("billno"));//触发单据编号
            map.put("tpv_triggerbillid", pkID);//触发单据ID
            map.put("tpv_triggerbillentryids", entryData.get("entryID"));//触发单据分录ID
            map.put("tpv_triggerdate", new Date());//触发日期
            map.put("tpv_currency", entity.getLong("currency.id"));//币种
            map.put("tpv_orignalamount", originalAmount);//原币金额
            map.put("tpv_usdamount", USDAmount);//美元金额
            map.put("tpv_tousdrate", kzToUSDRate);//对美元汇率
            map.put("tpv_tormbrate", entity.getBigDecimal("tpv_rate"));//对人民币汇率
            map.put("tpv_coreid", pkID);//核心id(隐藏)
            map.put("tpv_operation", "撤证/闭卷");//操作
            dataList.add(map);
        }
        setDataToBill(dataList);
    }

    /**
     * @param pkIDs:
     * @return void
     * @author: chen
     * @date: 2023/11/29 10:37
     * @description: 改证(改证申请审核时触发)
     */
    public static void changeCredit(Map<Long,Long> pkIDs) {
        List<HashMap<String, Object>> dataList = new ArrayList<>(pkIDs.size());
        pkIDs.keySet().stream().forEach(pkID -> {
            HashMap<String, Object> map = new HashMap<>();
            DynamicObject entity = BusinessDataServiceHelper.loadSingle(pkIDs.get(pkID), FORMID_LetterCredit);
            HashMap<String, String> entryData = getEntryData(Optional.ofNullable(entity.getDynamicObjectCollection("tpv_contractdetail")));
            String status = entity.getString("biztype");//业务类型 开证:open_card 改证:edit_card
            if ("edit_card".equals(status)) {
                boolean isNationalCard = entity.getBoolean("isnationalcard");//是否国际证
                BigDecimal amount = entity.getBigDecimal("amount");//开征金额
                BigDecimal rate = entity.getBigDecimal("tpv_rate");//汇率
                rate = rate.compareTo(new BigDecimal(0)) == 0 ? new BigDecimal(1) : rate;//汇率
                BigDecimal overFillRate = entity.getBigDecimal("tpv_overfillrate");//金额溢装率
                BigDecimal kzToUSDRate = entity.getBigDecimal("tpv_kzsqusdrate");//开证折USD汇率
                BigDecimal curAmount = calculateAmount(amount, rate, overFillRate);//金额(本位币)

                HashMap<String, BigDecimal> historyAmount = getAmountMax(pkIDs.get(pkID), null, entity.getString("creditno"));
                BigDecimal maxAmount = historyAmount.get("maxAmount");
                if (curAmount.compareTo(maxAmount) <= 0) {//本单金额小于历史最大值,不进行插入值
                    return;
                }

                //资金额度字段
                Date applyDate = entity.getDate("applydate");//资金额度日期
                BigDecimal currencyAmount = curAmount.subtract(maxAmount);//金额(本位币)
                BigDecimal originalAmount = currencyAmount.divide(rate, 2, BigDecimal.ROUND_HALF_UP);//原币金额

                map.put("tpv_type", isNationalCard ? "I" : "J");//类别
                map.put("tpv_addorsub", "A");//增减
                map.put("tpv_realpreocp", "A");//真实/预占用
                map.put("tpv_datetime", DateUtil.parseDate(applyDate));//日期
                map.put("tpv_org", entity.getLong("org.id"));//公司
                map.put("tpv_bigdept", getSupDepartment(entity.getLong("tpv_department.id")));//大部门(额度使用部门)
                map.put("tpv_depertment", entity.getLong("tpv_department.id"));//对应单据记录部门
                map.put("tpv_amount", currencyAmount);//金额(本位币)
                map.put("tpv_combinations", entryData.get("combination"));//组合号
                map.put("tpv_salenum", "");//销售合同号
                map.put("tpv_purnum", entryData.get("purnum"));//采购合同号
                map.put("tpv_bonustype", entryData.get("bonusType"));//计奖类型
                map.put("tpv_user", entity.getLong("tpv_people.id"));//业务员
                map.put("tpv_corenum", entity.getString("billno"));//核心号
                map.put("tpv_triggerbill", pkID);//触发单据
                map.put("tpv_triggerbillno", entity.getString("billno"));//触发单据编号
                map.put("tpv_triggerbillid", pkID);//触发单据ID
                map.put("tpv_triggerbillentryids", entryData.get("entryID"));//触发单据分录ID
                map.put("tpv_triggerdate", new Date());//触发日期
                map.put("tpv_currency", entity.getLong("currency.id"));//币种
                map.put("tpv_orignalamount", originalAmount);//原币金额
                map.put("tpv_usdamount", originalAmount.multiply(kzToUSDRate));//美元金额
                map.put("tpv_tousdrate", kzToUSDRate);//对美元汇率
                map.put("tpv_tormbrate", rate);//对人民币汇率
                map.put("tpv_coreid", pkID);//核心id(隐藏)
                map.put("tpv_operation", "改证");//操作
                dataList.add(map);
            }
        });
        setDataToBill(dataList);
    }

    /**
     * @param letterCredit:
     * @return Long
     * @author: chen
     * @date: 2023/11/30 16:38
     * @description: 根据信用证号获取开证处理ID
     */
    private DynamicObject getLetterCreditByCreditNo(String letterCredit ,String selectFiled) {
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("creditno", QCP.equals, letterCredit));//单据编号
        DynamicObject object = BusinessDataServiceHelper.loadSingle("lc_lettercredit", selectFiled, filters.toArray(new QFilter[0]));
        return object;
    }


    /**
     * @param pkValue:
     * @return BigDecimal
     * @author: chen
     * @date: 2023/11/28 17:31
     * @description: 根据核心ID获取 金额(本位币)汇总 原币金额 美元金额
     */
    private static HashMap<String, BigDecimal> getChangedAmount(Long pkValue) {
        HashMap<String, BigDecimal> map = new HashMap<>();
        BigDecimal amount = new BigDecimal(0);
        BigDecimal USDAmount = new BigDecimal(0);
        BigDecimal originalAmount = new BigDecimal(0);
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("tpv_coreid", QCP.equals, pkValue));//核心单据ID
        DataSet load = ORM.create().queryDataSet(algo_Key, "tpv_sc_capitallimit", "tpv_amount,tpv_orignalamount,tpv_usdamount,tpv_addorsub", filters.toArray(new QFilter[0]));
        load.print(true);
        if (!load.isEmpty()) {
            for (Row row : load) {
                String addOrSub = row.getString("tpv_addorsub");
                if ("A".equals(addOrSub)) {
                    amount = amount.add(row.getBigDecimal("tpv_amount"));
                    originalAmount = originalAmount.add(row.getBigDecimal("tpv_orignalamount"));
                    USDAmount = USDAmount.add(row.getBigDecimal("tpv_usdamount"));
                } else {
                    amount = amount.subtract(row.getBigDecimal("tpv_amount"));
                    originalAmount = originalAmount.subtract(row.getBigDecimal("tpv_orignalamount"));
                    USDAmount = USDAmount.subtract(row.getBigDecimal("tpv_usdamount"));
                }
            }
        }
        map.put("tpv_amount", amount);
        map.put("tpv_orignalamount", originalAmount);
        map.put("tpv_usdamount", USDAmount);
        return map;
    }

    /**
     * @param dataList:
     * @return void
     * @author: chen
     * @date: 2023/11/23 15:04
     * @description: 将获取到的数据插入到资金额度表中
     */
    private static void setDataToBill(List<HashMap<String, Object>> dataList) {
        ArrayList<DynamicObject> capitalList = new ArrayList(dataList.size());//保存到数据库的数据
        dataList.stream().forEach(dataMap -> {
            //新增资金额度单据
            DynamicObject dynamicObject = BusinessDataServiceHelper.newDynamicObject("tpv_sc_capitallimit");
//            dynamicObject.set("billstatus", "C");
            DataEntityPropertyCollection properties = dynamicObject.getDynamicObjectType().getProperties();
            for (String key : dataMap.keySet()) {
                if (properties.containsKey(key)) {
                    dynamicObject.set(key, dataMap.get(key));
                }
            }
            capitalList.add(dynamicObject);
        });
        SaveServiceHelper.saveOperate("save", FORMID_CapitalLimit, capitalList.toArray(new DynamicObject[0]), OperateOption.create()); // 保存到数据库
    }

    /**
     * @param kzID:
     * @param ldID:
     * @param letterCredit:
     * @return HashMap<String, BigDecimal>
     * @author: chen
     * @date: 2023/11/30 16:39
     * @description: 获取历史版本金额的最大值
     */
    private static HashMap<String, BigDecimal> getAmountMax(Long kzID, Long ldID, String letterCredit) {
        HashMap<String, BigDecimal> historyMaxAmount = new HashMap<>();
        //查找历史版本中改证的最大值,到单处理汇总值
        BigDecimal modifyAmount = new BigDecimal(0);
        BigDecimal arrivalAmount = new BigDecimal(0);
        BigDecimal rate = new BigDecimal(0);
        //获取历史版本(计算到单的最大值)
        DynamicObject[] history = BillChangeHistoryHelper.getChangeHistoryByBizId(kzID, FORMID_LetterCredit);
        for (DynamicObject dynamicObject : history) {
            DynamicObject dynamicObject1 = BillChangeHistoryHelper.deserializeBillInfo(dynamicObject);
            rate = dynamicObject1.getBigDecimal("tpv_rate");
            BigDecimal amount = dynamicObject1.getBigDecimal("amount");//开征金额
            BigDecimal overFillRate = dynamicObject1.getBigDecimal("tpv_overfillrate");//金额溢装率
            BigDecimal curAmount = calculateAmount(amount, rate, overFillRate);
            if (curAmount.compareTo(modifyAmount) > 0) {
                modifyAmount = curAmount;
            }
        }
        //获取到单的汇总值
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("id", QCP.not_equals, ldID));//排除本单
        filters.add(new QFilter("lettercredit.number", QCP.equals, letterCredit));//排除本单
        DataSet load = ORM.create().queryDataSet(algo_Key, FORMID_Arrival, "id,arrivalamount,tpv_rate", filters.toArray(new QFilter[0]));
        for (Row row : load) {
            BigDecimal decimal = row.getBigDecimal("arrivalamount") == null ? new BigDecimal(1) : row.getBigDecimal("arrivalamount");
            rate = row.getBigDecimal("tpv_rate") == null ? new BigDecimal(1) : row.getBigDecimal("tpv_rate");
            arrivalAmount = arrivalAmount.add(decimal.multiply(rate));
        }
        historyMaxAmount.put("rate", rate);
        historyMaxAmount.put("modifyAmount", modifyAmount);
        historyMaxAmount.put("arrivalAmount", arrivalAmount);
        historyMaxAmount.put("maxAmount", modifyAmount.compareTo(arrivalAmount) > 0 ? modifyAmount : arrivalAmount);
        return historyMaxAmount;
    }

    /**
     * @param entryEntityOpt:
     * @return HashMap<String, String>
     * @author: chen
     * @date: 2023/11/28 11:03
     * @description: 获取分录数据(组合号, 合同号, 计奖类型)
     */
    private static HashMap<String, String> getEntryData(Optional<DynamicObjectCollection> entryEntityOpt) {

        HashMap<String, String> entryData = new HashMap();
        Set<String> combination = new HashSet<>();//组合号
        Set<String> purNum = new HashSet<>();//采购合同号
        Set<String> bonusType = new HashSet<>();//计奖类型
        Set<String> entryID = new HashSet<>();//分录ID
        entryEntityOpt.ifPresent(entryEntity -> {
            if (entryEntity.size() == 0) {
                entryData.put("bonusType", "");
                entryData.put("purnum", "");
                entryData.put("combination", "");
                entryData.put("id", "");
            } else {
                entryEntity.stream().forEach(row -> {
                    purNum.add(row.getString("tpv_purnumbers"));
                    bonusType.add(row.getString("tpv_tjtype.name"));
                    combination.add(row.getString("tpv_combinationnum.name"));
                    entryID.add(row.getString("id"));
                });
                entryData.put("purnum", StringUtils.join(purNum.toArray(), ","));
                entryData.put("bonusType", StringUtils.join(bonusType.toArray(), ","));
                entryData.put("combination", StringUtils.join(combination.toArray(), ","));
                entryData.put("entryID", StringUtils.join(entryID.toArray(), ","));
            }
        });
        return entryData;
    }

    /**
     * @param saleNum:
     * @return Long
     * @author: chen
     * @date: 2023/11/23 14:59
     * @description: 根据销售合同号获取组合号
     */
    private String getCombinationBySaleNum(String saleNum) {
        if (saleNum.isEmpty()) {
            return "";
        }
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("tpv_combinationnumber.number", QCP.equals, saleNum));//销售分录的合同号
        DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "tpv_bd_salescontract",
                "id,tpv_combinationnumber.number", filters.toArray(new QFilter[0]));
        if (queryDataSet.isEmpty()) {
            return null;
        } else {
            String name = "";
            for (Row row : queryDataSet) {
                name = row.getString("tpv_combinationnumber.name");
            }
            return name;
        }
    }

    /**
     * @param id:
     * @return Long
     * @author: chen
     * @date: 2023/11/23 10:36
     * @description: 获取上级部门
     */
    private static Long getSupDepartment(Long id) {
        if (id == null || id.equals(0L)) {
            return 0L;
        }
        List<Long> orgIds = new ArrayList<>(1);
        orgIds.add(id);
        // 加载指定ID的动态对象
        DynamicObject object = BusinessDataServiceHelper.loadSingle(id, "bos_adminorg");
        // 获取组织形态的名称
        String orgPatternName = object.getString("orgpattern.name");
        // 如果不是部门，则递归获取上级部门
        if (!"部门".equals(orgPatternName)) {
            // 获取父级部门的ID
            Map<Long, Long> orgParentMap = OrgUnitServiceHelper.getDirectSuperiorOrg(OrgViewType.Admin, orgIds);
            // 递归调用getSupDepartment，传入父级部门的ID
            id = getSupDepartment(orgParentMap.get(id));
        }
        return id;
    }

    /**
     * @param :
     * @return Object
     * @author: chen
     * @date: 2023/11/27 15:59
     * @description: 计算金额
     */
    private static BigDecimal calculateAmount(BigDecimal amount, BigDecimal rate, BigDecimal overFillRate) {
        return amount.multiply(rate).multiply(overFillRate.divide(new BigDecimal(100)).add(new BigDecimal(1)));
    }
}
