package com.itopview.wm.sys.formplugin.dynamic;

import java.util.*;

import com.itopview.wm.sys.utils.BothCardHelper;

import com.itopview.wm.sys.utils.LinkManHeplper;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.ShowFormHelper;
import kd.bos.form.control.Control;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.fi.ap.helper.OrgHelper;

/**
 * 添加公司抬头-动态表单插件
 *
 * @author longh
 * @description:
 * @date 2022年6月20日 下午2:18:34
 */
public class AddCompanyEdit extends AbstractBillPlugIn {

    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners(new String[]{"tpv_companybank", "tpv_companyname", "tpv_companylinkman"});
        this.initF7();
    }

    private void initF7() {
        BasedataEdit orgF7 = (BasedataEdit) this.getControl("tpv_company");
        orgF7.addBeforeF7SelectListener((beforeF7SelectEvent) -> {
            ListShowParameter showParameter = (ListShowParameter) beforeF7SelectEvent.getFormShowParameter();
            QFilter qFilter = new QFilter("orgpattern", "in", OrgHelper.getOrgPattern());
            showParameter.getListFilterParameter().setFilter(qFilter);
        });
    }

    public void click(EventObject evt) {
        super.click(evt);
        Control c = (Control) evt.getSource();
        String key = c.getKey().toLowerCase();
        BasedataEdit control = null;
        switch (key) {
            case "tpv_companyname":
                control = (BasedataEdit) this.getControl("tpv_company");
                control.click();
                break;
            case "tpv_companybank":
                control = (BasedataEdit) this.getControl("tpv_companybankf7");
                control.click();
                break;
            case "tpv_companylinkman":
                LinkManHeplper.orderFilter(this.getView(), this.getClass().getName(), "tpv_companylinkman");
                break;
        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent e) {
        super.closedCallBack(e);
        String id = e.getActionId();
        Object returnData = e.getReturnData();
        switch (id) {
            case "tpv_companylinkman":
                LinkManHeplper.closeBack(e, this.getModel());
                break;
        }
    }

    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String key = e.getProperty().getName();
        ChangeData[] changeData = e.getChangeSet();
        Object newValue = changeData[0].getNewValue();
        switch (key) {
            case "tpv_company":
                this.orgChanged(newValue);
                break;
            case "tpv_companyname":
                if ("".equals(newValue)) {
                    this.getModel().setValue("tpv_company", null);
                }
                break;
            case "tpv_companybankf7":
                this.bankf7Changed(newValue);
                break;
        }
    }

    private void orgChanged(Object newValue) {
        DynamicObject org = (DynamicObject) newValue;
        if (org != null) {
            BothCardHelper.fillCoInfo(this.getModel(), org);
        }
    }

    private void bankf7Changed(Object newValue) {
        DynamicObject bank = (DynamicObject) newValue;
        if (bank != null) {
            this.getModel().setValue("tpv_companybank", bank.getLocaleString("name").getLocaleValue());
        }
    }
}