package com.itopview.wm.sys.formplugin.dynamic;

import com.itopview.wm.sys.utils.BothCardHelper;
import com.itopview.wm.sys.utils.LinkManHeplper;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.control.Control;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.fi.arapcommon.helper.BaseDataHelper;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

/**
 * 添加客户抬头-动态表单插件
 *
 * @author longh
 * @description:
 * @date 2022年6月20日 下午6:29:33
 */
public class AddCustomerEdit extends AbstractBillPlugIn {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners(new String[]{"tpv_customername", "tpv_customerbank", "tpv_customeracct", "tpv_customerlinkman"});
        this.initF7();
    }

    private void initF7() {
        IFormView parentView = this.getView().getParentView();
        IDataModel parentModel = parentView.getModel();
        DynamicObject org = (DynamicObject) parentModel.getValue("org");
        if (org != null) {
            BasedataEdit baseDataCtr = (BasedataEdit) this.getControl("tpv_customer");
            baseDataCtr.addBeforeF7SelectListener((beforeF7SelectEvent) -> {
                ListShowParameter showParameter = (ListShowParameter) beforeF7SelectEvent.getFormShowParameter();
                QFilter baseDataFilter = BaseDataHelper.getBasedataFilter(showParameter.getBillFormId(), org.getLong("id"));
                showParameter.getListFilterParameter().getQFilters().add(baseDataFilter);
            });
        }
    }

    public void click(EventObject evt) {
        super.click(evt);
        Control c = (Control) evt.getSource();
        String key = c.getKey().toLowerCase();
        BasedataEdit control = null;
        switch (key) {
            case "tpv_customername":
                control = (BasedataEdit) this.getControl("tpv_customer");
                control.click();
                break;
            case "tpv_customerbank":
                control = (BasedataEdit) this.getControl("tpv_customerbankf7");
                control.click();
                break;
            case "tpv_customeracct":
                this.customerAcctShowF7();
                break;
            case "tpv_customerlinkman":
                LinkManHeplper.orderFilter(this.getView(), this.getClass().getName(), "tpv_customerlinkman");
                break;
        }
    }

    /**
     * 选择银行账号
     */
    private void customerAcctShowF7() {
        DynamicObject basedata = (DynamicObject) this.getModel().getValue("tpv_customer");
        if (ObjectUtils.isEmpty(basedata)) {
            this.getView().showTipNotification("请先选择客户！");
        } else {
            String asstactType = basedata.getDataEntityType().getName();
            DynamicObjectCollection coll = BothCardHelper.getBankCollInfo(basedata.getPkValue(), asstactType);
            if (coll == null || coll.isEmpty()) {
                this.getView().showTipNotification("请维护对应客户的银行信息");
                return;
            }

            FormShowParameter fsp = BothCardHelper.getCustomerBankInfoShowParameter(basedata.getPkValue());
            CloseCallBack closeCallBack = new CloseCallBack(this.getPluginName(), "customerAcctCallBack");
            fsp.setCloseCallBack(closeCallBack);
            this.getView().showForm(fsp);

        }
    }

    public void closedCallBack(ClosedCallBackEvent e) {
        super.closedCallBack(e);
        String id = e.getActionId();
        Object returnData = e.getReturnData();
        switch (id) {
            case "customerAcctCallBack":
                this.customerAcctCallBack(returnData);
                break;
            case "tpv_customerlinkman":
                LinkManHeplper.closeBack(e, this.getModel());
                break;
        }
    }

    /**
     * 选择银行账号回调带出银行信息
     *
     * @param returnData
     */
    private void customerAcctCallBack(Object returnData) {
        DynamicObject rowInfo = (DynamicObject) returnData;
        if (rowInfo != null) {
            this.getModel().setValue("tpv_customeracct", rowInfo.getString("bankaccount"));
            this.getModel().setValue("tpv_customerbank", rowInfo.getString("bank.name"));
            this.getModel().setValue("tpv_customerbankf7", rowInfo.get("bank"));
        }
    }

    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String key = e.getProperty().getName();
        ChangeData[] changeData = e.getChangeSet();
        Object newValue = changeData[0].getNewValue();
        switch (key) {
            case "tpv_customername":
                this.customerChanged((String) newValue);
                break;
            case "tpv_customer":
                this.customerF7Changed(newValue);
                break;
            case "tpv_customerbankf7":
                this.customerbankf7Changed(newValue);
        }

    }

    /**
     * 填写客户名称后根据名称查询客户基础资料并带出
     *
     * @param newValue
     */
    private void customerChanged(String newValue) {
        IDataModel model = this.getModel();
        if (YEM.isNotEmpty(newValue)) {
            List<QFilter> qFilters = new ArrayList<>();
            qFilters.add(new QFilter("status", "=", "C"));
            qFilters.add(new QFilter("enable", "=", "1"));
            qFilters.add(new QFilter("name", "=", newValue));
            DynamicObject[] cols = BusinessDataServiceHelper.load("bd_customer", "id", qFilters.toArray(new QFilter[0]));
            for (DynamicObject col : cols) {
                model.setValue("tpv_customer", col.getPkValue());
                return;
            }
        }
        model.setValue("tpv_customer", null);
    }

    /**
     * 选择客户带出客户信息
     *
     * @param newValue
     */
    private void customerF7Changed(Object newValue) {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        DynamicObject newDyObj = (DynamicObject) newValue;
        if (newDyObj != null) {
            newDyObj = BusinessDataServiceHelper.loadSingle(newDyObj.getPkValue(), newDyObj.getDataEntityType().getName());
            model.setValue("tpv_customertin", newDyObj.getString("tx_register_no"));
            DynamicObject tpvCustomer = (DynamicObject) this.getModel().getValue("tpv_customer");
            DynamicObject tpvDepartment = (DynamicObject) this.getView().getParentView().getModel().getValue("tpv_department");
            if (tpvCustomer != null && tpvDepartment != null) {

                long currentUser = YEM.getCurrentUserId();
                DynamicObject bosUser = BusinessDataServiceHelper.loadSingleFromCache(currentUser, "bos_user");
                List<Long> dept = new ArrayList<>();
                DynamicObjectCollection entryentity = bosUser.getDynamicObjectCollection("entryentity");
                for (DynamicObject row : entryentity) {
                    dept.add(row.getDynamicObject("dpt").getLong("id"));
                    DynamicObject bosAdminorg = BusinessDataServiceHelper.loadSingle(row.getDynamicObject("dpt").getPkValue(), "bos_adminorg");
                    DynamicObjectCollection structures = bosAdminorg.getDynamicObjectCollection("structure");
                    for (DynamicObject structure : structures) {
                        DynamicObject viewparent = structure.getDynamicObject("viewparent");
                        if (viewparent != null) {
                            dept.add(viewparent.getLong("id"));
                        }
                    }
                }
//                if(dept.size()!=0){
//                    DynamicObject[] adminOrgs = BusinessDataServiceHelper.load("bos_adminorg", "", new QFilter[]{new QFilter("structure.viewparent", "in", dept)});
//                    for (DynamicObject adminOrg : adminOrgs) {
//                        dept.add(adminOrg.getLong("id"));
//                    }
//                }


                DynamicObject[] bdContactin = BusinessDataServiceHelper.load("tpv_bd_contactin", "name,tpv_address,tpv_phone,tpv_mailbox",
                        new QFilter[]{new QFilter("tpv_customer", QCP.equals, tpvCustomer.getPkValue().toString()), new QFilter("tpv_entryentity.tpv_basedata.id", QCP.in, dept)}, "createtime desc");
                if (bdContactin.length != 0) {
                    model.setValue("tpv_customeraddr", bdContactin[0].get("tpv_address"));
                    model.setValue("tpv_customertel", bdContactin[0].get("tpv_phone"));
                    model.setValue("tpv_customerlinkman", bdContactin[0].get("name"));
                    model.setValue("tpv_customeremail", bdContactin[0].get("tpv_mailbox"));
                } else {
                    model.setValue("tpv_customeraddr", null);
                    model.setValue("tpv_customertel", null);
                    model.setValue("tpv_customerlinkman", null);
                    model.setValue("tpv_customeremail", null);
                }
            }
//            model.setValue("tpv_customeraddr", newDyObj.getLocaleString("bizpartner_address").getLocaleValue());
//            model.setValue("tpv_customertel", newDyObj.getString("bizpartner_phone"));
//            model.setValue("tpv_customerlinkman", newDyObj.getLocaleString("linkman").getLocaleValue());
//            model.setValue("tpv_customeremail", newDyObj.getString("postal_code"));
            String acctBank = null;
            Object bank = null;
            String bankName = null;
            DynamicObjectCollection bankColls = newDyObj.getDynamicObjectCollection("entry_bank");
            for (int i = 0; i < bankColls.size(); i++) {
                DynamicObject bankInfo = bankColls.get(i);
                if (bankInfo.getBoolean("isdefault_bank")) {
                    acctBank = bankInfo.getString("bankaccount");
                    bank = bankInfo.get("bank");
                    bankName = bankInfo.getString("bank.name");
                    break;
                }
                if (i == 0) {
                    acctBank = bankInfo.getString("bankaccount");
                    bank = bankInfo.get("bank");
                    bankName = bankInfo.getString("bank.name");
                }
            }
            model.setValue("tpv_customeracct", acctBank);
            model.setValue("tpv_customerbank", bankName);
            model.setValue("tpv_customerbankf7", bank);
            model.getDataEntity(true).set("tpv_customername", newDyObj.getLocaleString("name").getLocaleValue());
            view.updateView("tpv_customername");
        } else {
            model.setValue("tpv_customeraddr", null);
            model.setValue("tpv_customertel", null);
            model.setValue("tpv_customerlinkman", null);
            model.setValue("tpv_customeremail", null);
            model.setValue("tpv_customeracct", null);
            model.setValue("tpv_customerbank", null);
            model.setValue("tpv_customerbankf7", null);
            model.setValue("tpv_customertin", null);
            model.getDataEntity(true).set("tpv_customername", null);
            view.updateView("tpv_customername");
        }
    }

    /**
     * 选择开户行带出开户行名称
     *
     * @param newValue
     */
    private void customerbankf7Changed(Object newValue) {
        DynamicObject bank = (DynamicObject) newValue;
        if (bank != null) {
            this.getModel().setValue("tpv_customerbank", bank.getLocaleString("name").getLocaleValue());
        }
    }

}