package com.itopview.wm.sys.formplugin.dynamic;

import com.itopview.wm.sys.utils.BothCardHelper;
import com.itopview.wm.sys.utils.LinkManHeplper;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.control.Control;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.fi.arapcommon.helper.BaseDataHelper;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

/**
 * 添加供应商抬头-动态表单插件
 *
 * @author longh
 * @description:
 * @date 2022年6月20日 下午4:40:11
 */
public class AddSupplierEdit extends AbstractBillPlugIn {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners(new String[]{"tpv_suppliername", "tpv_supplierbank", "tpv_supplieracct", "tpv_supplierlinkman"});
        this.initF7();
    }

    private void initF7() {
        IFormView parentView = this.getView().getParentView();
        IDataModel parentModel = parentView.getModel();
        DynamicObject org = (DynamicObject) parentModel.getValue("org");
        Long orgId = org != null ? org.getLong("id") : 0L;//公司名称
        BasedataEdit baseDataCtr = (BasedataEdit) this.getControl("tpv_supplier");
        baseDataCtr.addBeforeF7SelectListener((beforeF7SelectEvent) -> {
            ListShowParameter showParameter = (ListShowParameter) beforeF7SelectEvent.getFormShowParameter();
            QFilter baseDataFilter = BaseDataHelper.getBasedataFilter(showParameter.getBillFormId(), orgId);
            showParameter.getListFilterParameter().getQFilters().add(baseDataFilter);
        });
    }

    public void click(EventObject evt) {
        super.click(evt);
        Control c = (Control) evt.getSource();
        String key = c.getKey().toLowerCase();
        BasedataEdit control = null;
        switch (key) {
            case "tpv_suppliername":
                control = (BasedataEdit) this.getControl("tpv_supplier");
                control.click();
                break;
            case "tpv_supplierbank":
                control = (BasedataEdit) this.getControl("tpv_supplierbankf7");
                control.click();
                break;
            case "tpv_supplieracct":
                this.supplierAcctShowF7();
                break;
            case "tpv_supplierlinkman":
                LinkManHeplper.orderFilter(this.getView(), this.getClass().getName(), "tpv_supplierlinkman");
                break;
        }
    }

    /**
     * 选择银行账号
     */
    private void supplierAcctShowF7() {
        DynamicObject basedata = (DynamicObject) this.getModel().getValue("tpv_supplier");
        if (ObjectUtils.isEmpty(basedata)) {
            this.getView().showTipNotification("请先选择供应商！");
        } else {
            String asstactType = basedata.getDataEntityType().getName();
            DynamicObjectCollection coll = BothCardHelper.getBankCollInfo(basedata.getPkValue(), asstactType);
            if (coll == null || coll.isEmpty()) {
                this.getView().showTipNotification("请维护对应供应商的银行信息");
                return;
            }

            FormShowParameter fsp = BothCardHelper.getSupplierBankInfoShowParameter(basedata.getPkValue());
            CloseCallBack closeCallBack = new CloseCallBack(this.getPluginName(), "supplierAcctCallBack");
            fsp.setCloseCallBack(closeCallBack);
            this.getView().showForm(fsp);

        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent e) {
        super.closedCallBack(e);
        String id = e.getActionId();
        Object returnData = e.getReturnData();
        switch (id) {
            case "supplierAcctCallBack":
                this.supplierAcctCallBack(returnData);
                break;
            case "tpv_supplierlinkman":
                LinkManHeplper.closeBack(e, this.getModel());
                break;
        }
    }

    /**
     * 选择银行账号回调带出银行信息
     *
     * @param returnData
     */
    private void supplierAcctCallBack(Object returnData) {
        DynamicObject rowInfo = (DynamicObject) returnData;
        if (rowInfo != null) {
            this.getModel().setValue("tpv_supplieracct", rowInfo.getString("bankaccount"));
            this.getModel().setValue("tpv_supplierbank", rowInfo.getString("bank.name"));
            this.getModel().setValue("tpv_supplierbankf7", rowInfo.get("bank"));
        }
    }

    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String key = e.getProperty().getName();
        ChangeData[] changeData = e.getChangeSet();
        Object newValue = changeData[0].getNewValue();
        switch (key) {
            case "tpv_suppliername":
                this.supplierChanged((String) newValue);
                break;
            case "tpv_supplier":
                this.supplierF7Changed(newValue);
                break;
            case "tpv_supplierbankf7":
                this.supplierBankf7Changed(newValue);
        }

    }

    /**
     * 填写供应商名称后根据名称查询供应商基础资料并带出
     *
     * @param newValue
     */
    private void supplierChanged(String newValue) {
        IDataModel model = this.getModel();
        if (YEM.isNotEmpty(newValue)) {
            List<QFilter> qFilters = new ArrayList<>();
            qFilters.add(new QFilter("status", "=", "C"));
            qFilters.add(new QFilter("enable", "=", "1"));
            qFilters.add(new QFilter("name", "=", newValue));
            DynamicObject[] cols = BusinessDataServiceHelper.load("bd_supplier", "id", qFilters.toArray(new QFilter[0]));
            for (DynamicObject col : cols) {
                model.setValue("tpv_supplier", col.getPkValue());
                return;
            }
        }
        model.setValue("tpv_supplier", null);
    }

    /**
     * 选择供应商带出供应商信息
     *
     * @param newValue
     */
    private void supplierF7Changed(Object newValue) {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        DynamicObject newDyObj = (DynamicObject) newValue;
        if (newDyObj != null) {
            newDyObj = BusinessDataServiceHelper.loadSingle(newDyObj.getPkValue(), newDyObj.getDataEntityType().getName());
            model.setValue("tpv_suppliertin", newDyObj.getString("tx_register_no"));
            DynamicObject tpvSupplier = (DynamicObject) this.getModel().getValue("tpv_supplier");
            DynamicObject tpvDepartment = (DynamicObject) this.getView().getParentView().getModel().getValue("tpv_department");
            if (tpvSupplier != null && tpvDepartment!=null) {

                long currentUser = YEM.getCurrentUserId();
                DynamicObject bosUser = BusinessDataServiceHelper.loadSingleFromCache(currentUser, "bos_user");
                List<Long> dept = new ArrayList<>();
                DynamicObjectCollection entryentity = bosUser.getDynamicObjectCollection("entryentity");
                for (DynamicObject row : entryentity) {
                    dept.add(row.getDynamicObject("dpt").getLong("id"));
                    DynamicObject bosAdminorg = BusinessDataServiceHelper.loadSingle(row.getDynamicObject("dpt").getPkValue(), "bos_adminorg");
                    DynamicObjectCollection structures =bosAdminorg.getDynamicObjectCollection("structure");
                    for (DynamicObject structure : structures) {
                        DynamicObject viewparent = structure.getDynamicObject("viewparent");
                        if (viewparent != null) {
                            dept.add(viewparent.getLong("id"));
                        }
                    }
                }
//                if(dept.size()!=0){
//                    DynamicObject[] adminOrgs = BusinessDataServiceHelper.load("bos_adminorg", "", new QFilter[]{new QFilter("structure.viewparent", "in", dept)});
//                    for (DynamicObject adminOrg : adminOrgs) {
//                        dept.add(adminOrg.getLong("id"));
//                    }
//                }

                DynamicObject[] bdContactin = BusinessDataServiceHelper.load("tpv_bd_contactin", "name,tpv_address,tpv_phone,tpv_mailbox",
                        new QFilter[]{new QFilter("tpv_supplier", QCP.equals, tpvSupplier.getPkValue().toString()),new QFilter("tpv_entryentity.tpv_basedata.id", QCP.in, dept)}, "createtime desc");
                if (bdContactin.length != 0) {
                    model.setValue("tpv_supplieraddr", bdContactin[0].get("tpv_address"));
                    model.setValue("tpv_suppliertel",  bdContactin[0].get("tpv_phone"));
                    model.setValue("tpv_supplierlinkman", bdContactin[0].get("name"));
                    model.setValue("tpv_supplieremail", bdContactin[0].get("tpv_mailbox"));
                } else {
                    model.setValue("tpv_supplieraddr", null);
                    model.setValue("tpv_suppliertel",  null);
                    model.setValue("tpv_supplierlinkman", null);
                    model.setValue("tpv_supplieremail", null);
                }
            }
//            model.setValue("tpv_supplieraddr", newDyObj.getLocaleString("bizpartner_address").getLocaleValue());
//            model.setValue("tpv_suppliertel", newDyObj.getString("bizpartner_phone"));
//            model.setValue("tpv_supplierlinkman", newDyObj.getLocaleString("linkman").getLocaleValue());
//            model.setValue("tpv_supplieremail", newDyObj.getString("postal_code"));
            String acctBank = null;
            Object bank = null;
            String bankName = null;
            DynamicObjectCollection bankColls = newDyObj.getDynamicObjectCollection("entry_bank");
            for (int i = 0; i < bankColls.size(); i++) {
                DynamicObject bankInfo = bankColls.get(i);
                if (bankInfo.getBoolean("isdefault_bank")) {
                    acctBank = bankInfo.getString("bankaccount");
                    bank = bankInfo.get("bank");
                    bankName = bankInfo.getString("bank.name");
                    break;
                }
                if (i == 0) {
                    acctBank = bankInfo.getString("bankaccount");
                    bank = bankInfo.get("bank");
                    bankName = bankInfo.getString("bank.name");
                }
            }
            model.setValue("tpv_supplieracct", acctBank);
            model.setValue("tpv_supplierbank", bankName);
            model.setValue("tpv_supplierbankf7", bank);
            model.getDataEntity(true).set("tpv_suppliername", newDyObj.getLocaleString("name").getLocaleValue());
            view.updateView("tpv_suppliername");
        } else {
            model.setValue("tpv_supplieraddr", null);
            model.setValue("tpv_suppliertel",  null);
            model.setValue("tpv_supplierlinkman", null);
            model.setValue("tpv_supplieremail", null);
            model.setValue("tpv_supplieracct", null);
            model.setValue("tpv_supplierbank", null);
            model.setValue("tpv_supplierbankf7", null);
            model.setValue("tpv_suppliertin", null);
            model.getDataEntity(true).set("tpv_suppliername", null);
            view.updateView("tpv_suppliername");
        }
    }

    /**
     * 选择开户行带出开户行名称
     *
     * @param newValue
     */
    private void supplierBankf7Changed(Object newValue) {
        DynamicObject bank = (DynamicObject) newValue;
        if (bank != null) {
            this.getModel().setValue("tpv_supplierbank", bank.getLocaleString("name").getLocaleValue());
        }
    }

}