package com.itopview.wm.sys.formplugin.dynamic;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;

import com.itopview.wm.sys.utils.UserOrgUtil;

import kd.bamp.mbis.common.util.DynamicObjectUtil;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.filter.FilterBuilder;
import kd.bos.entity.filter.FilterCondition;
import kd.bos.entity.tree.TreeNode;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.Control;
import kd.bos.form.control.FilterGrid;
import kd.bos.form.control.Search;
import kd.bos.form.control.TreeView;
import kd.bos.form.control.events.SearchEnterEvent;
import kd.bos.form.control.events.SearchEnterListener;
import kd.bos.form.control.events.TreeNodeClickListener;
import kd.bos.form.control.events.TreeNodeEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

/**
 * 卷宗查询-动态表单插件
 * @description:
 * @author longh
 * @date 2021年9月23日 下午7:07:07
 */
public class BizFileQueryEdit extends AbstractFormPlugin implements BeforeF7SelectListener,TreeNodeClickListener,SearchEnterListener{

	private final static String KEY_DATAFLEX = "tpv_dataflex";
	private final static String KEY_DATACHILD = "tpv_cm_bizfilecontainer";
	private final static String KEY_TEMPLATE = "tpv_bizfiletemplate";
	
	@Override
	public void beforeBindData(EventObject e) {
		super.beforeBindData(e);
		this.bindFilterGrid();
	}
	
	private void bindFilterGrid() {
		IFormView view = this.getView();
		FilterGrid filterGrid = view.getControl("tpv_filtergridap");
		DynamicObject tpl = (DynamicObject) this.getModel().getValue(KEY_TEMPLATE);
		if(tpl != null) {
			filterGrid.setEntityNumber(tpl.getString("tpv_relationbill.number"));
		}else {
			filterGrid.setEntityNumber("");
		}
		filterGrid.SetValue(new FilterCondition());
		
		TreeView tree = view.getControl("tpv_treeview");
		tree.deleteAllNodes();
	}

	@Override
	public void registerListener(EventObject e) {
		super.registerListener(e);
		this.addClickListeners(new String[] { "tpv_query" });// 按钮面板等click监听
		Search searchap = this.getControl("tpv_searchap");// 搜索监听
		searchap.addEnterListener(this);
		BasedataEdit appliers = this.getControl(KEY_TEMPLATE);// F7过滤监听
		appliers.addBeforeF7SelectListener(this);
		BasedataEdit orgs = this.getControl("tpv_org");// F7过滤监听
		orgs.addBeforeF7SelectListener(this);
		TreeView treeView = this.getControl("tpv_treeview");// 树形节点click监听
		treeView.addTreeNodeClickListener(this);
	}
	
	@Override
	public void propertyChanged(PropertyChangedArgs e) {
		super.propertyChanged(e);
		String key = e.getProperty().getName();
		switch (key) {
		case KEY_TEMPLATE:
			this.getView().updateView();
			break; 	
		default:
			break;
		}
	}

	@Override
	public void click(EventObject evt) {
		final Control c = (Control)evt.getSource();
		String key = c.getKey().toLowerCase();
		switch (key) {
		case "tpv_query":
			this.initTreeNode("");
			break;
		default:
			break;
		}
	}

	private void initTreeNode(String text) {
		IFormView view = this.getView();
		IDataModel model = this.getModel();
		DynamicObject lv1 = (DynamicObject) model.getValue(KEY_TEMPLATE);
        if(lv1 != null) {
        	TreeView tree = this.getView().getControl("tpv_treeview");
    		//构造根节点
    		TreeNode root = new TreeNode();
    		root.setParentid("");
    		root.setText("全部");
    		root.setId("0");
    		root.setIsOpened(true);
    		
    		//构造父节点
        	TreeNode rootNode = new TreeNode("0", "1", lv1.getString("name"), true);
    		rootNode.setIsOpened(true);//是否默认展开
    		root.addChild(rootNode);
    		
    		//构造子节点
    		DynamicObject tpv_relationbill = lv1.getDynamicObject("tpv_relationbill");
    		if(tpv_relationbill != null) {
    			tpv_relationbill = BusinessDataServiceHelper.loadSingle(tpv_relationbill.getPkValue(), "bos_entityobject");
    			String billKey = tpv_relationbill.getString("number");
    			String billName = tpv_relationbill.getString("name");
    			String modeltype = tpv_relationbill.getString("modeltype");
    			MainEntityType dataType  = EntityMetadataCache.getDataEntityType(billKey); 
    			
    			List<QFilter> qFilters = new ArrayList<>();
    			DynamicObjectCollection orgs = (DynamicObjectCollection) model.getValue("tpv_org");
    			if(orgs != null && orgs.size() > 0) {
    				List<Long> ids = new ArrayList<>();
    				orgs.forEach((v) -> {
    					ids.add(v.getDynamicObject("fbasedataid").getLong("id"));
    				});
    				qFilters.add(new QFilter("org", QCP.in, ids));
    			}else {
    				qFilters.add(new QFilter("org", QCP.in, UserOrgUtil.getUserChangeAbleOrgIds()));
    			}
    			String orderbyStr = "";
    			//查询核心单据方案
    			QFilter qFilter = new QFilter("tpv_iscorebill", QCP.equals, true);
    			qFilter.and(new QFilter("parent.parent.id", QCP.equals, lv1.getPkValue()));
    			DynamicObject lv3 = BusinessDataServiceHelper.loadSingle("tpv_cm_bizfiletemplate", 
	    				"id,tpv_filtergird,tpv_orderbycolno,tpv_orderby", new QFilter[] {qFilter});
    			if(lv3 != null) {
    				String tpv_filtergird = lv3.getString("tpv_filtergird");
    				if(!StringUtils.isBlank(tpv_filtergird)) {
    					FilterCondition filterCondition = SerializationUtils.fromJsonString(tpv_filtergird, FilterCondition.class);
    					if (filterCondition != null){
    	       			 	FilterBuilder fBuilder = new FilterBuilder(dataType, filterCondition); 
    	       			 	fBuilder.buildFilter(); 
    	       			 	qFilters.addAll(fBuilder.getQFilters());
    				    }
    				}
    				
    				String tpv_orderbycolno = lv3.getString("tpv_orderbycolno");
    				String tpv_orderby = lv3.getString("tpv_orderby");
    				if(tpv_orderbycolno.length() > 0 && tpv_orderby.length() > 0) {
    					orderbyStr = tpv_orderbycolno + " " + tpv_orderby;
    				}
    			}
    			
    			FilterGrid filterGrid = this.getView().getControl("tpv_filtergridap");
    			FilterCondition filterCondition = filterGrid.getFilterGridState().getFilterCondition();
    			if (filterCondition != null){
       			 	FilterBuilder fBuilder = new FilterBuilder(dataType, filterCondition); 
       			 	fBuilder.buildFilter(); 
       			 	qFilters.addAll(fBuilder.getQFilters());
			    }
    			
    			String noFieldKey = "billno";
    			if("BaseFormModel".equals(modeltype)) {
    				noFieldKey = "number";
    			}
    			String selectFields = DynamicObjectUtil.getSelectfields(billKey, false);
    			DynamicObject[] bills = BusinessDataServiceHelper.load(billKey, selectFields, qFilters.toArray(new QFilter[0]), orderbyStr);
    			for (DynamicObject bill : bills) {
    				String billno = bill.getString(noFieldKey);
    				if(text.trim().length() > 0 && !billno.contains(text)) {
        				continue;
        			}
    				String contractstatus = "";
    				try {
						String tpv_contractstatus = bill.getString("tpv_contractstatus");
						if("B".equals(tpv_contractstatus)) {
	    					contractstatus = "（已关闭）";
	    				}
					} catch (Exception e) {
						e.printStackTrace();
					}
    				
    				TreeNode childNode = new TreeNode("1", billno, billName + "：" + billno + contractstatus, false);
        			childNode.setIsOpened(true);//是否默认展开
        			rootNode.addChild(childNode);
				}
    		}
    		
    		tree.deleteAllNodes();
    		tree.addNode(root);
    		tree.focusNode(root);
    		
    		Search searchap = this.getControl("tpv_searchap");
    		searchap.setSearchKey("");
        }else {
        	view.showMessage("请选择卷宗方案！");
        }
		
	}
	
	@Override
    public void treeNodeClick(TreeNodeEvent e) {
    	String nodeId = e.getNodeId().toString();
    	if(nodeId.length() > 1) {
    		initData(nodeId);
    	}
    }

	private void initData(String nodeId) {
		IFormView view = this.getView();
		IDataModel model = this.getModel();
		DynamicObject lv1 = (DynamicObject) model.getValue(KEY_TEMPLATE);
        if(lv1 != null) {
        	active2Form(view, lv1.getPkValue(), nodeId, ShowType.InContainer, KEY_DATAFLEX);
        }else {
        	view.showMessage("请选择卷宗方案！");
        }
	}

	public static void active2Form(IFormView view, Object fid, String nodeId, ShowType showType, String targetKey) {
		FormShowParameter showParameter = new FormShowParameter();
		showParameter.setStatus(OperationStatus.EDIT);
		showParameter.getOpenStyle().setShowType(showType);
		if(StringUtils.isNotBlank(targetKey)) {
			showParameter.getOpenStyle().setTargetKey(targetKey);
		}
		showParameter.setFormId(KEY_DATACHILD);
		Map<String, Object> customParams = new HashMap<>();
		customParams.put("fid", fid);
		customParams.put("nodeId", nodeId);
		customParams.put("uuid", UUID.randomUUID().toString());
		showParameter.setCustomParams(customParams);
		view.showForm(showParameter);
	}

	@Override
	public void search(SearchEnterEvent arg0) {
		String text = arg0.getText();
		this.initTreeNode(text);
	}

	@Override
	public void beforeF7Select(BeforeF7SelectEvent arg0) {
		String name = arg0.getProperty().getName();
		ListShowParameter showParameter = (ListShowParameter) arg0.getFormShowParameter();
		switch (name) {
		case KEY_TEMPLATE:   
			QFilter qFilter = new QFilter("level", QCP.equals, "1");
			showParameter.getTreeFilterParameter().getQFilters().add(qFilter);
			showParameter.getListFilterParameter().getQFilters().add(qFilter);
			break;
		case "tpv_org":   
			QFilter qFilter1 = new QFilter("id", QCP.in, UserOrgUtil.getUserChangeAbleOrgIds());
			showParameter.getTreeFilterParameter().getQFilters().add(qFilter1);
			showParameter.getListFilterParameter().getQFilters().add(qFilter1);
			break;
		default:
			break;
		}
	}
	
}
