package com.itopview.wm.sys.formplugin.dynamic;

import java.util.EventObject;
import java.util.Map;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

import com.itopview.wm.utils.YEM;

import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.control.CodeEdit;
import kd.bos.form.control.Control;
import kd.bos.form.events.ClientCallBackEvent;
import kd.bos.form.plugin.AbstractFormPlugin;

/**
 * 代码编辑器-表单插件
 * 
 * @description JavaScript脚本编辑器，用于配置表达式计算结果
 * @author longh
 * @date 2023年12月13日
 */
public class CodeFormEdit extends AbstractFormPlugin {
	private static final String KEY_ENGINE = "nashorn";// Nashorn -JavaScript引擎
	private static final String KEY_CTRL = "tpv_codeeditap";
	private static final String KEY_EDIT = "setText";

	@Override
	public void registerListener(EventObject e) {
		super.registerListener(e);
		this.addClickListeners("btnok", "tpv_test");
	}

	@Override
	public void afterCreateNewData(EventObject e) {
		super.afterCreateNewData(e);
		this.getView().addClientCallBack(KEY_EDIT);
	}

	@Override
	public void clientCallBack(ClientCallBackEvent e) {
		super.clientCallBack(e);
		String key = e.getName();
		switch (key) {
		case KEY_EDIT:
			setCodeText();
			break;
		default:
			break;
		}
	}

	private void setCodeText() {
		IFormView view = this.getView();
		FormShowParameter showParameter = view.getFormShowParameter();
		Map<String, Object> customParams = showParameter.getCustomParams();
		String jsCode = (String) customParams.get("jsCode");

		CodeEdit codeEdit = view.getControl(KEY_CTRL);
		if (YEM.isEmpty(jsCode)) {
			jsCode = "//示例\r\nvar rs = 100;\r\nif(rs > 0){\r\n  rs = '正数'\r\n}else if(rs < 0){\r\n  rs = '负数'\r\n}else{\r\n  rs = '零'\r\n}";
		}
		codeEdit.setText(jsCode);
	}

	@Override
	public void click(EventObject evt) {
		super.click(evt);
		String key = ((Control) evt.getSource()).getKey();
		switch (key) {
		case "tpv_test":
			this.showResult();
			break;
		case "btnok":
			this.btnOk();
			break;
		}
	}

	private void showResult() {
		IFormView view = this.getView();
		CodeEdit codeEdit = view.getControl(KEY_CTRL);
		String jsCode = codeEdit.getText();
		jsCode = dynamicInit(jsCode);
		view.showMessage("" + getRunResult(jsCode));
	}

	private Object getRunResult(String jsCode) {
		ScriptEngineManager manager = new ScriptEngineManager();
		ScriptEngine engine = manager.getEngineByName(KEY_ENGINE);
		Object result = null;
		try {
			result = engine.eval(jsCode);
		} catch (ScriptException e) {
			e.printStackTrace();
		}
		return result;
	}

	private String dynamicInit(String jsCode) {
		if (jsCode == null) {
			jsCode = "";
		}
		jsCode = jsCode.replaceAll("dynamic", "2");
		return jsCode;
	}

	private void btnOk() {

	}

}
