package com.itopview.wm.sys.formplugin.dynamic;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.ChartUtil;

import kd.bos.form.FormShowParameter;
import kd.bos.form.chart.Axis;
import kd.bos.form.chart.AxisType;
import kd.bos.form.chart.LineSeries;
import kd.bos.form.chart.PointLineChart;
import kd.bos.form.chart.Position;
import kd.bos.form.plugin.AbstractFormPlugin;

/**
 * 汇率走势-表单插件
 * @description:点线图
 * @author longh
 * @date 2022年6月24日 下午4:18:31
 */
public class ExchangeChartEdit extends AbstractFormPlugin {

	@Override
	public void afterCreateNewData(EventObject e) {
		super.afterCreateNewData(e);
		FormShowParameter fsp = this.getView().getFormShowParameter();
		Map<String, Map<String, BigDecimal>> datas = fsp.getCustomParam("datas");
		List<String> dateList = fsp.getCustomParam("dateList");
		String curName = fsp.getCustomParam("curName");
		
		PointLineChart chart = (PointLineChart) this.getControl("tpv_pointlinechartap");
		chart.clearData();
		chart.setShowTooltip(true);
		chart.setMargin(Position.left, "20px");
		
		Axis xaxis = chart.createXAxis("日期",AxisType.category);
		Axis yaxis = chart.createYAxis("折RMB汇率", AxisType.value);
		
		HashMap<String,Object> axisTick = new HashMap<>();
		axisTick.put("interval", Integer.valueOf(0));
		axisTick.put("show", true);
		xaxis.setPropValue("axisTick", axisTick);
		
		HashMap<String,Object> axisLineMap = new HashMap<>();
		HashMap<String,Object> lineStyleMap = new HashMap<>();
		lineStyleMap.put("color", "#999999");
		axisLineMap.put("lineStyle", lineStyleMap);
		xaxis.setPropValue("axisLine", axisLineMap);
		
		HashMap<String,Object> axisLabel = new HashMap<>();
		axisLabel.put("interval", Integer.valueOf(0));
		axisLabel.put("show", true);
		axisLabel.put("rotate", Integer.valueOf(45));
//		axisLabel.put("color", "#E2E2E2");
		xaxis.setPropValue("axisLabel", axisLabel);
		
		for (Entry<String, Map<String, BigDecimal>> entry : datas.entrySet()) {
			String key = entry.getKey();
			if(key.equals(curName) || "ALL".equals(curName)) {
				List<Double> list = new ArrayList<>();
				for (String dateStr : dateList) {
					BigDecimal excval = entry.getValue().get(dateStr);
					list.add(excval == null ? null : excval.doubleValue());
				}
				LineSeries lineSeries = chart.createSeries(key);
				lineSeries.setItemColor(ChartUtil.getRandColor());
				lineSeries.setPropValue("symbol", ChartUtil.getSymbol());
				lineSeries.setPropValue("symbolSize", 10);
				lineSeries.setData(list.toArray(new Number[0]));
				if(!"ALL".equals(curName)) {
					list.stream().filter(v -> v != null).min(Comparator.comparing(Double::doubleValue)).ifPresent(v -> {
						yaxis.setMin(BigDecimalUtils.multiply(v, 0.99, 3));
					});
				}
			}
		}
		xaxis.setCategorys(dateList.toArray(new String[0]));
		
		chart.bindData(null);
	}
	
}
