package com.itopview.wm.sys.formplugin.dynamic;

import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.sdk.plugin.Plugin;

import java.math.BigDecimal;
import java.util.EventObject;

/**
 * 物料报价单-表单插件
 *
 * @author longh
 * @description:
 * @date 2024年2月20日
 */
public class SrcPurlistEdit extends AbstractBillPlugIn implements Plugin, BeforeF7SelectListener {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        ((BasedataEdit) this.getControl("kdtest_cmcof_entry")).addBeforeF7SelectListener(this);
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String name = e.getProperty().getName();
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        if ("kdtest_cmcof_entry".equals(name)) {
            IDataModel model = this.getModel();
            int index = model.getEntryCurrentRowIndex("kdtest_subentryentity");
            DynamicObject kdtest_cmcoefficient = (DynamicObject) model.getValue("kdtest_cmcoefficient", index);
            if (kdtest_cmcoefficient != null) {
                QFilter qFilter = new QFilter("kdtest_cmcoefficient", QCP.equals, kdtest_cmcoefficient.getPkValue());
                showParameter.getListFilterParameter().setFilter(qFilter);
            }
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String key = e.getProperty().getName();
        //综合量化计算方案
        if ("kdtest_cmcoefsettings".equals(key)) {
            ChangeData[] changeData = e.getChangeSet();
            int rowIndex = changeData[0].getRowIndex();
            IDataModel model = this.getModel();
            model.deleteEntryData("kdtest_subentryentity");
            DynamicObject cmcoefsettings = (DynamicObject) model.getValue(key, rowIndex);
            if (cmcoefsettings != null) {
                DynamicObjectCollection entry = cmcoefsettings.getDynamicObjectCollection("kdtest_entryentity");
                for (DynamicObject row : entry) {
                    int subRowIndex = model.createNewEntryRow("kdtest_subentryentity");
                    DynamicObject kdtest_cmcoefficient = row.getDynamicObject("kdtest_cmcoefficient");
                    model.setValue("kdtest_cmcoefficient", kdtest_cmcoefficient, subRowIndex, rowIndex);
                    if (kdtest_cmcoefficient != null) {
                        kdtest_cmcoefficient = BusinessDataServiceHelper.loadSingleFromCache(kdtest_cmcoefficient.getPkValue(), kdtest_cmcoefficient.getDynamicObjectType().getName());
                        DynamicObjectCollection kdtest_element = kdtest_cmcoefficient.getDynamicObjectCollection("kdtest_entryentity");
                        for (DynamicObject row1 : kdtest_element) {
                            model.setValue("kdtest_cmcof_entry", row1.getPkValue(), subRowIndex, rowIndex);
                        }
                    }
                }
            }
        }
        //系数、金额
        String amtNum = (String) this.getModel().getValue("kdtest_amtnum");
        if (YEM.isEmpty(amtNum)) {
            amtNum = "loctaxamount";
        }
        if ("kdtest_coefficient".equals(key) || amtNum.equals(key)) {
            setAmt(amtNum);
        }
        //元素
        if ("kdtest_cmcof_entry".equals(key)) {
            IDataModel model = this.getModel();
            ChangeData[] changeData = e.getChangeSet();
            int rowIndex = changeData[0].getRowIndex();
            DynamicObject cmcof_entry = (DynamicObject) model.getValue(key, rowIndex);
            if (cmcof_entry != null) {
                int index = model.getEntryCurrentRowIndex("entryentity");
                model.setValue("kdtest_coefficient", cmcof_entry.getBigDecimal("kdtest_coefficient"), rowIndex, index);
            }
        }
    }

    private void setAmt(String amtNum) {
        IDataModel model = this.getModel();
        int index = model.getEntryCurrentRowIndex("entryentity");
        DynamicObject rowEntity = model.getEntryRowEntity("entryentity", index);
        if (rowEntity != null) {
            BigDecimal amt = rowEntity.getBigDecimal(amtNum);
            BigDecimal cmAmt = amt;
            DynamicObjectCollection subEntry = rowEntity.getDynamicObjectCollection("kdtest_subentryentity");
            for (DynamicObject row : subEntry) {
                BigDecimal kdtest_coefficient = row.getBigDecimal("kdtest_coefficient");
                if (kdtest_coefficient.compareTo(BigDecimal.ZERO) > 0) {
                    kdtest_coefficient = kdtest_coefficient.subtract(BigDecimal.ONE);
                }
                cmAmt = cmAmt.add(amt.multiply(kdtest_coefficient));
            }
            model.setValue("kdtest_amount", cmAmt, index);
        }
    }

}