package com.itopview.wm.sys.formplugin.dynamic;

import java.util.EventObject;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.filter.FilterCondition;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.control.FilterGrid;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.plugin.AbstractFormPlugin;

/**
 * 校验规则-表单插件
 * @description:回填变更方案操作校验规则
 * @author longh
 * @date 2021年11月19日 上午10:01:27
 */
public class VersionCheckRuleEdit extends AbstractFormPlugin {

	@Override
	public void beforeBindData(EventObject e) {
		super.beforeBindData(e);
		FormShowParameter showParameter = this.getView().getFormShowParameter();
		String entityNumber = showParameter.getCustomParam("entityNumber");
		FilterGrid filterGrid = this.getView().getControl("tpv_filtergridap");
		//设置过滤控件关联单据实体
		filterGrid.setEntityNumber(entityNumber);
	}
	
	@Override
	public void afterBindData(EventObject e) {
		super.afterBindData(e);
		IFormView view = this.getView();
		FormShowParameter showParameter = view.getFormShowParameter();
		JSONObject data = showParameter.getCustomParam("data");
		if(data != null) {
			String tpv_checkjson = data.getString("tpv_checkjson");//序列化校验条件
			if(!tpv_checkjson.isEmpty()) {
				JSONObject parseObject = JSON.parseObject(tpv_checkjson);
				//加载过滤控件数据
				String tpv_filtergridap = parseObject.getString("tpv_filtergridap");
				if(!tpv_filtergridap.isEmpty()) {
					FilterCondition filterCondition = SerializationUtils.fromJsonString(tpv_filtergridap, FilterCondition.class);
					if (filterCondition != null){
				    	FilterGrid filterGrid = view.getControl("tpv_filtergridap");
				    	filterGrid.SetValue(filterCondition);
				    }
				}
			}
		}
	}
	
	@Override
	public void afterCreateNewData(EventObject evt) {
		super.afterCreateNewData(evt);
		IDataModel model = this.getModel();
		IFormView view = this.getView();
		FormShowParameter showParameter = view.getFormShowParameter();
		JSONObject data = showParameter.getCustomParam("data");
		if(data != null) {
			String tpv_checkjson = data.getString("tpv_checkjson");//序列化校验条件
			if(!tpv_checkjson.isEmpty()) {
				JSONObject parseObject = JSON.parseObject(tpv_checkjson);
				//加载描述
				model.setValue("tpv_desc", parseObject.getString("tpv_desc"));
				//加载条件为真提示
				model.setValue("tpv_truethrow", parseObject.getBoolean("tpv_truethrow"));
				//加载出错提示
				model.setValue("tpv_message", parseObject.getString("tpv_message"));
			}
		}
		
	}
	
	@Override
	public void afterDoOperation(AfterDoOperationEventArgs e) {
		super.afterDoOperation(e);
		String operateKey = e.getOperateKey();
		switch (operateKey) {
		case "ok" :
			this.afterDoOK();
			break;
		}
	}
	
	/**
	 * 点击确定回传数据
	 * @param evt
	 * @param key
	 */
	private void afterDoOK() {
		IDataModel model = this.getModel();
		IFormView view = this.getView();
		FilterGrid filterGrid = view.getControl("tpv_filtergridap");
	    FilterCondition filterCondition = filterGrid.getFilterGridState().getFilterCondition();
	    String tpv_filtergridap = "";
	    if (filterCondition != null){
	    	tpv_filtergridap = SerializationUtils.toJsonString(filterCondition);
	    }
	    JSONObject jsonObj = new JSONObject();
	    jsonObj.put("tpv_desc", model.getValue("tpv_desc"));
	    jsonObj.put("tpv_truethrow", model.getValue("tpv_truethrow"));
	    jsonObj.put("tpv_message", model.getValue("tpv_message"));
	    jsonObj.put("tpv_filtergridap", tpv_filtergridap);
		view.returnDataToParent(jsonObj);
        view.close();
	}
	
}
