package com.itopview.wm.sys.formplugin.form;

import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.sys.utils.NewVersionHelper;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.form.IClientViewProxy;
import kd.bos.form.IFormView;
import kd.bos.form.control.AttachmentPanel;
import kd.bos.form.control.events.BeforeAttachmentRemoveEvent;
import kd.bos.form.control.events.BeforeAttachmentRemoveListener;
import kd.bos.form.control.events.UploadEvent;
import kd.bos.form.control.events.UploadListener;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.events.ClientCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 附件表格-通用插件
 *
 * @author longh
 * @description 附件类型分录<->附件面板-同步
 * @date 2023年8月25日
 */
public class AttachmentEntryFormPlugin extends AbstractBillPlugIn
        implements UploadListener, BeforeF7SelectListener, BeforeAttachmentRemoveListener {
    private static final String KEY_ATT_ENTRY = "tpv_attachmentdetail";
    private static final String KEY_ATT_TAG = "tpv_attachmentcon_tag";
    private static final String KEY_ATT_PANEL = "attachmentpanel";

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        BasedataEdit attachmenttype = this.getView().getControl("tpv_attachmenttype");
        attachmenttype.addBeforeF7SelectListener(this);
        AttachmentPanel attachmentPanel = this.getView().getControl("attachmentpanel");
        attachmentPanel.addUploadListener(this);
        attachmentPanel.addBeforeRemoveListener(this);
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        setAttachmentEntry();
        this.getModel().setDataChanged(false);
        setAttFileVisble();
    }

    private void setAttFileVisble() {
        AttachmentPanel panel = getControl(KEY_ATT_PANEL);
        List<Map<String, Object>> attachmentData = panel.getAttachmentData();
        if (attachmentData == null && attachmentData.isEmpty()) {
            return;
        }
        for (Map<String, Object> attachmentMap : attachmentData) {
            String visiablePreview = "1";//是否显示预览按钮
            String visiableDownload = "1";//是否显示下载按钮
            String visiableDelete = "0";//是否显示删除按钮
            String visiableRename = "1";//是否显示重命名按钮
            //visible参数规则为01组成,0代表不可见,1代表可见
            attachmentMap.put("visible", visiablePreview + visiableDownload + visiableDelete + visiableRename);
        }
        IClientViewProxy clientViewProxy = getView().getService(IClientViewProxy.class);
        clientViewProxy.setEntryProperty(KEY_ATT_PANEL, "data", attachmentData);
    }

    @Override
    public void afterUpload(UploadEvent evt) {
        setAttachmentEntry();
//		DynamicObjectCollection entryRows = this.getModel().getEntryEntity("tpv_attachmentdetail");
//		int entryCurrentRowIndex = this.getModel().getEntryCurrentRowIndex("tpv_attachmentdetail");
//		if (entryRows.size() > 0) {
//			afterData(entryRows, entryCurrentRowIndex);
//		}
    }

    @Override
    public void afterRemove(UploadEvent evt) {
        setAttachmentEntry();
    }

    protected void setAttachmentEntry() {
        IFormView view = this.getView();
        IDataModel model = this.getModel();
        List<JSONObject> attList = new ArrayList<>();
        String jsonStr = (String) model.getValue(KEY_ATT_TAG);
        JSONObject json = YEM.isEmpty(jsonStr) ? new JSONObject() : JSONObject.parseObject(jsonStr);
        model.deleteEntryData(KEY_ATT_ENTRY);
        AttachmentPanel attachmentpanel = view.getControl(KEY_ATT_PANEL);
        List<Map<String, Object>> attachmentDatas = attachmentpanel.getAttachmentData();
        for (Map<String, Object> attachmentData : attachmentDatas) {
            String name = (String) attachmentData.get("name");// 附件名称
            Map<?, ?> maps = (Map<?, ?>) attachmentData.get("creator");

//            String uid = (String) attachmentData.get("uid");// 附件名称
//            JSONObject attJson = new JSONObject();
//            Set<String> strings = json.keySet();
//            for (String string : strings) {
//                JSONObject jsonObject = json.getJSONObject(string);
//                String uidJSON = jsonObject.getString("uid");
//                if (YEM.isNotEmpty(uidJSON) && uidJSON.equals(uid)) {
//                    attJson = jsonObject;
//                }
//            }

			JSONObject attJson = json.containsKey(name) ? json.getJSONObject(name) : new JSONObject();
            attJson.put("name", name);
            long times = (long) attachmentData.get("createdate");
            Date date = new Date(times);
            attJson.put("createdate", date);
            String lang = YEM.requestContext().getLang().toString();
            attJson.put("uid", attachmentData.get("uid"));
            if (maps != null && maps.containsKey(lang)) {
                attJson.put("creator", maps.get(lang));
            }
            json.put(name, attJson);
            attList.add(attJson);
        }

        List<JSONObject> collect = attList.stream().sorted(new Comparator<JSONObject>() {
            @Override
            public int compare(JSONObject o1, JSONObject o2) {
                Date date1 = o1.getDate("createdate");
                Date date2 = o2.getDate("createdate");
                if (date1 == null || date2 == null) {
                    return 0;
                }
                return date1.compareTo(date2);
            }

        }).collect(Collectors.toList());
        for (JSONObject attJson : collect) {
            int index = model.createNewEntryRow(KEY_ATT_ENTRY);
            model.beginInit();
            model.setValue("tpv_attachmentname", attJson.get("name"), index);
            model.setValue("tpv_attachmenttype", attJson.get("tpv_attachmenttype"), index);
            model.setValue("tpv_uplink", attJson.get("creator"), index);
            model.setValue("tpv_uploadtime", attJson.get("createdate"), index);
            model.setValue("tpv_sealstatus", attJson.get("tpv_sealstatus"), index);
            model.setValue("tpv_printingmethod", attJson.get("tpv_printingmethod"), index);
            model.setValue("tpv_inquiry", attJson.get("tpv_inquiry"), index);
            model.setValue("tpv_signaturestatus", attJson.get("tpv_signaturestatus"), index);
            model.setValue("tpv_lock", attJson.get("tpv_lock"), index);
            model.setValue("tpv_charactercode", attJson.get("tpv_charactercode"), index);
            model.setValue("tpv_pretrialcode", attJson.get("tpv_pretrialcode"), index);
            model.setValue("tpv_uid", attJson.get("uid"), index);
            model.endInit();
            view.updateView("tpv_attachmentname", index);
            view.updateView("tpv_attachmenttype", index);
            view.updateView("tpv_uplink", index);
            view.updateView("tpv_uploadtime", index);
            view.updateView("tpv_sealstatus", index);
            view.updateView("tpv_printingmethod", index);
            view.updateView("tpv_inquiry", index);
            view.updateView("tpv_signaturestatus", index);
            view.updateView("tpv_lock", index);
            view.updateView("tpv_charactercode", index);
            view.updateView("tpv_pretrialcode", index);
            view.updateView("tpv_uid", index);
        }
        model.setValue(KEY_ATT_TAG, json.toJSONString());
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        ChangeData[] changeSet = e.getChangeSet();
        int rowIndex = changeSet[0].getRowIndex();
        String name = e.getProperty().getName();
        switch (name) {
            case "tpv_attachmenttype":
                setAttachmentType(rowIndex);
                break;
            case "tpv_attachmentcount":
                uploadAttachment(changeSet);
                break;
        }

    }

    private void uploadAttachment(ChangeData[] changeSet) {
        Integer oldCount = (Integer) changeSet[0].getOldValue();
        Integer newCount = (Integer) changeSet[0].getNewValue();
        if (oldCount != newCount) {
            this.getView().addClientCallBack("auto_ save", 0);
        }
    }

    @Override
    public void clientCallBack(ClientCallBackEvent e) {
        super.clientCallBack(e);
        if (e.getName().equals("auto_ save")) {
            AttachmentPanel attachmentPanel = getControl(KEY_ATT_PANEL);
            List<Map<String, Object>> data = attachmentPanel.getAttachmentData();
            if (data != null & !data.isEmpty()) {
//				String billstatus = (String) this.getView().getModel().getValue("billstatus");
//				if (!"A".equals(billstatus)) {
                OperationResult saveResult = this.getView().invokeOperation("save", YEM.getOptionNoMsg());
                //}
            }
        }
    }

    private void setAttachmentType(int rowIndex) {
        IDataModel model = this.getModel();
        String jsonStr = (String) model.getValue(KEY_ATT_TAG);
        JSONObject json = YEM.isEmpty(jsonStr) ? new JSONObject() : JSONObject.parseObject(jsonStr);
        String tpv_attachmentname = (String) model.getValue("tpv_attachmentname", rowIndex);
        DynamicObject tpv_attachmenttype = (DynamicObject) model.getValue("tpv_attachmenttype", rowIndex);
        if (json.containsKey(tpv_attachmentname)) {
            JSONObject attJson = json.getJSONObject(tpv_attachmentname);
            if (attJson == null) {
                attJson = new JSONObject();
            }
            attJson.put("tpv_attachmenttype", tpv_attachmenttype == null ? null : tpv_attachmenttype.getPkValue());

            json.put(tpv_attachmentname, attJson);
        }
        model.setValue(KEY_ATT_TAG, json.toJSONString());
    }

    @Override
    public void afterDoOperation(AfterDoOperationEventArgs e) {
        super.afterDoOperation(e);
        OperationResult operationResult = e.getOperationResult();
        if (operationResult != null && operationResult.isSuccess()) {
            setAttachmentEntry();
            this.getModel().setDataChanged(false);
        }
    }

    @Override
    public void beforeAttachmentRemove(BeforeAttachmentRemoveEvent e) {

    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String name = e.getProperty().getName();
        switch (name) {
            case "tpv_attachmenttype":
                attachmentTypeF7(e);
                break;
            default:
                break;
        }
    }

    /**
     * 附件类型过滤
     *
     * @param e
     */
    private void attachmentTypeF7(BeforeF7SelectEvent e) {
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        String billKey = this.getView().getEntityId();
        NewVersionHelper vHelper = new NewVersionHelper(billKey);
        if (vHelper.isChanged()) {
            billKey = vHelper.getRequireKey();
        }
        QFilter qFilter = new QFilter("tpv_attachbillentry.tpv_requiredbill.number", QCP.equals, billKey);
        showParameter.getListFilterParameter().getQFilters().add(qFilter);
    }

}
