package com.itopview.wm.sys.formplugin.form;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.clr.DataEntityPropertyCollection;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.EntityType;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.events.ItemClickEvent;

/**
 * 业务卷宗跳转插件
 * @description:1、点击业务卷宗快速跳转到符合条件的卷宗明细
 * @author longh
 * @date 2022年1月10日 下午5:03:09
 */
public class BizFileActiveFormPlugIn extends AbstractBillPlugIn {

	@Override
	public void itemClick(ItemClickEvent evt) {
		super.itemClick(evt);
		String itemKey = evt.getItemKey();
		switch (itemKey) {
		case "tpv_bizfile":
			this.clickActive2BizFile();
			break;
		default:
			break;
		}
	}
	
	/**
	 * 点击业务卷宗快速跳转到符合条件的卷宗明细
	 */
	private void clickActive2BizFile() {
		IFormView view = this.getView();
		IDataModel model = this.getModel();
		Map<String, Object> customParams = new HashMap<>();
		customParams.put("entityNumber", model.getDataEntityType().getName());
		String billno = (String) model.getValue("billno");
		customParams.put("billno", billno);
		List<String> billIds = new ArrayList<String>();
		addBillno(billIds, billno);
		
		String entityNumber = model.getDataEntityType().getName();
		MainEntityType mainType = EntityMetadataCache.getDataEntityType(entityNumber);
		DataEntityPropertyCollection properties = mainType.getProperties();
		if(properties.containsKey("tpv_corebillno")) {
			String no = (String) model.getValue("tpv_corebillno");
			addBillno(billIds, no);
		}
		if(properties.containsKey("tpv_sourcebillno")) {
			String no = (String) model.getValue("tpv_sourcebillno");
			addBillno(billIds, no);
		}
		Map<String, EntityType> entities = mainType.getAllEntities();
		if(entities.containsKey("billentry")) {
			DynamicObjectCollection entryEntities = model.getEntryEntity("billentry");
			DataEntityPropertyCollection properties2 = entryEntities.getDynamicObjectType().getProperties();
			for (DynamicObject entryEntity : entryEntities) {
				if(properties2.containsKey("mainbillnumber")) {
					String no = entryEntity.getString("mainbillnumber");
					addBillno(billIds, no);
				}
				if(properties2.containsKey("srcbillnumber")) {
					String no = entryEntity.getString("srcbillnumber");
					addBillno(billIds, no);
				}
			}
		}
		customParams.put("nodeId", billIds);
		
		FormShowParameter showParameter = new FormShowParameter();
		showParameter.setStatus(OperationStatus.EDIT);
		showParameter.getOpenStyle().setShowType(ShowType.MainNewTabPage);
//		showParameter.setFormId("tpv_cm_bizfilequick");
		showParameter.setFormId("tpv_tpl_filecollectquick");
		showParameter.setCustomParams(customParams);
		view.showForm(showParameter);
	}

	private void addBillno(List<String> billIds, String no) {
		if(!no.isEmpty() && !billIds.contains(no)) {
			billIds.add(no);
		}
	}
	
}
