package com.itopview.wm.sys.formplugin.form;

import java.util.EventObject;
import java.util.Set;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.sys.utils.MetaPropertyHelper;

import kd.bos.base.AbstractBasePlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.filter.FilterCondition;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.form.control.Control;
import kd.bos.form.control.FilterGrid;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.events.BeforeDoOperationEventArgs;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.form.operate.AbstractOperate;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QFilter;
import kd.bos.util.StringUtils;

/**
 * 关联单据设置-表单插件
 * @description:
 * @author longh
 * @date 2022年8月8日 下午3:45:17
 */
public class DevProcessFormPlugin extends AbstractBasePlugIn implements BeforeF7SelectListener {
	private static final String[] REL_COLS = new String[]{"tpv_left", "tpv_thiscol", "tpv_thiscolno", "tpv_qcp", "tpv_relcol", "tpv_relcolno", "tpv_right", "tpv_andor"};
	
	@Override
	public void beforeBindData(EventObject e) {
		super.beforeBindData(e);
		this.bindFilters();
		this.bindFilterGrid();
		this.getModel().setDataChanged(false);
	}

	@Override
	public void afterBindData(EventObject e) {
		super.afterBindData(e);
		this.bindFilterGridData();
	}
	
	private void bindFilters() {
		String tpv_colmap = this.getModel().getDataEntity().getString("tpv_relation");
		JSONArray entry = null;
		if (StringUtils.isNotEmpty(tpv_colmap)) {
			entry = JSON.parseArray(tpv_colmap);
		}
		this.batchFilterEntry("tpv_filters", entry);
	}

	private void batchFilterEntry(String entryName, JSONArray entry) {
		IDataModel model = this.getModel();
		model.deleteEntryData(entryName);
		if (entry != null) {
			for (int i = 0; i < entry.size(); i++) {
				JSONObject rowData = entry.getJSONObject(i);
				int index = model.createNewEntryRow(entryName);
				Set<String> keySet = rowData.keySet();
				for (String key : keySet) {
					String bill = "";
					String billKey = String.valueOf(rowData.get(key));
					switch (key) {
					case "tpv_thiscol":
					case "tpv_relcol":
						break;
					case "tpv_thiscolno":
						bill = ((DynamicObject) model.getValue("tpv_requirebill")).getString("number");
						model.setValue("tpv_thiscol", MetaPropertyHelper.getEntityLangName(bill, billKey), index);
						model.setValue(key, rowData.get(key), index);
						break;
					case "tpv_relcolno":
						bill = ((DynamicObject) model.getValue("group")).getString("tpv_entitynumber");
						model.setValue("tpv_relcol", MetaPropertyHelper.getEntityLangName(bill, billKey), index);
						model.setValue(key, rowData.get(key), index);
						break;
					default:
						model.setValue(key, rowData.get(key), index);
						break;
					}
				}
			}
		}
	}

	private void bindFilterGrid() {
		FilterGrid filterGrid = this.getView().getControl("tpv_filtergridap");
		DynamicObject tpv_relationbill = (DynamicObject) this.getModel().getValue("tpv_requirebill");
		if(tpv_relationbill != null) {
			filterGrid.setEntityNumber(tpv_relationbill.getString("number"));
		}else {
			filterGrid.setEntityNumber("1122");
		}
	}
	
	private void bindFilterGridData() {
		String tpv_filtergird = this.getModel().getDataEntity().getString("tpv_filtergird");
		if(StringUtils.isNotEmpty(tpv_filtergird)) {
			FilterCondition filterCondition = SerializationUtils.fromJsonString(tpv_filtergird, FilterCondition.class);
			if (filterCondition != null){
		    	FilterGrid filterGrid = this.getView().getControl("tpv_filtergridap");
		    	filterGrid.SetValue(filterCondition);
		    }
		}
	}
	
	@Override
	public void registerListener(EventObject e) {
		super.registerListener(e);
		this.addClickListeners(new String[]{"tpv_thiscol", "tpv_relcol"});
		((BasedataEdit) this.getControl("tpv_requirebill")).addBeforeF7SelectListener(this);
	}
	
	@Override
	public void click(EventObject evt) {
		super.click(evt);
		String key = ((Control) evt.getSource()).getKey();
		switch (key) {
		case "tpv_thiscol" :
			this.clickBillCol(evt, key, "tpv_requirebill", "number");
			break;
		case "tpv_relcol" :
			this.clickBillCol(evt, key, "group", "tpv_entitynumber");
			break;
		}
	}
	
	private void clickBillCol(EventObject evt, String key, String baseCol, String childCol) {
		DynamicObject requireBill = MetaPropertyHelper.getAndCheckF7Value(this.getView(), baseCol);
		if(requireBill != null) {
			String formKey = requireBill.getString(childCol);
			MetaPropertyHelper.showPropertySelect(this, key, formKey, false);
		}
	}
	
	@Override
	public void propertyChanged(PropertyChangedArgs e) {
		super.propertyChanged(e);
		String name = e.getProperty().getName();
		switch (name) {
		case "tpv_iscorebill" :
			this.isCoreBillChanged();
			break;
		case "tpv_requirebill" :
			this.requireBillChanged();
			break;
		case "group" :
			this.isCoreBillChanged();
			break;
		}
	}
	
	private void isCoreBillChanged() {
		DynamicObject dataEntity = this.getModel().getDataEntity(true);
		if(dataEntity.getBoolean("tpv_iscorebill")) {
			DynamicObject group = dataEntity.getDynamicObject("group");
			if(group != null) {
				this.getModel().setValue("tpv_requirebill", group.get("tpv_relationbill"));
				return;
			}
		}
		this.getModel().setValue("tpv_requirebill", null);
	}

	private void requireBillChanged() {
		DynamicObject dataEntity = this.getModel().getDataEntity(true);
    	String rel = dataEntity.getString("group.tpv_entitynumber");
    	String req = dataEntity.getString("tpv_requirebill.number");
    	if(req != null && rel.equals(req)) {
    		dataEntity.set("tpv_iscorebill", true);
    	}
		this.getView().updateView();
		this.getModel().deleteEntryData("tpv_filters");
		FilterGrid filterGrid = this.getView().getControl("tpv_filtergridap");
    	filterGrid.SetValue(new FilterCondition());
	}
	
	@Override
	public void closedCallBack(ClosedCallBackEvent e) {
		super.closedCallBack(e);
		String key = e.getActionId();
		switch (key) {
		case "tpv_thiscol" :
			MetaPropertyHelper.callBackEntryData(this.getModel(), e, "tpv_filters", "tpv_thiscolno", "tpv_thiscol", true);
			break;
		case "tpv_relcol" :
			MetaPropertyHelper.callBackEntryData(this.getModel(), e, "tpv_filters", "tpv_relcolno", "tpv_relcol", true);
			break;
		}
	}
	
	@Override
	public void beforeF7Select(BeforeF7SelectEvent e) {
		IDataEntityProperty p = e.getProperty();
		String key = p.getName();
		switch (key) {
		case "tpv_requirebill" :
			this.beforeRequireBill(e);
			break;
		}
	}
	
	private void beforeRequireBill(BeforeF7SelectEvent e) {
		QFilter qFilter = new QFilter("modeltype", "in", new String[]{"BillFormModel", "BaseFormModel"});
		((ListShowParameter) e.getFormShowParameter()).getListFilterParameter().getQFilters().add(qFilter);
	}
	
	public void beforeDoOperation(BeforeDoOperationEventArgs e) {
		super.beforeDoOperation(e);
		String key = ((AbstractOperate) e.getSource()).getOperateKey();
		switch (key) {
		case "save" :
		case "submit" :
			this.beforeSave(e);
			break;
		}
	}
	
	@Override
	public void afterDoOperation(AfterDoOperationEventArgs e) {
		super.afterDoOperation(e);
		OperationResult operationResult = e.getOperationResult();
		if(operationResult != null && operationResult.isSuccess()) {
			this.bindFilters();
			this.bindFilterGridData();
			this.getModel().setDataChanged(false);
		}
	}

	private void beforeSave(BeforeDoOperationEventArgs e) {
		IDataModel model = this.getModel();
		String tpv_filters = this.parseRelCol2Json(REL_COLS, "tpv_filters");
		model.setValue("tpv_relation", tpv_filters);
		String tpv_filtergird = this.parseFilterGrid2Json("tpv_filtergridap");
		model.setValue("tpv_filtergird", tpv_filtergird);
	}
	
	private String parseRelCol2Json(String[] relCols, String entryKey) {
		DynamicObjectCollection rows = this.getModel().getEntryEntity(entryKey);
		if (rows.isEmpty()) {
			return null;
		} else {
			JSONArray json = new JSONArray(rows.size());
			for (DynamicObject row : rows) {
				JSONObject values = new JSONObject(relCols.length);
				for (String col : relCols) {
					String value = row.getString(col);
					values.put(col, value);
				}
				json.add(values);
			}
			
			return json.toJSONString();
		}
	}
	
	private String parseFilterGrid2Json(String filterGridKey) {
		String filter = "";
		FilterGrid filterGrid = this.getView().getControl(filterGridKey);
	    FilterCondition filterCondition = filterGrid.getFilterGridState().getFilterCondition();
	    if (filterCondition != null){
	        filter = SerializationUtils.toJsonString(filterCondition);
	    }
	    
		return filter;
	}
	
}
