package com.itopview.wm.sys.formplugin.form;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.itopview.wm.sys.task.TxExchangeRateTask;
import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.DynamicObjectUtil;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.container.Tab;
import kd.bos.form.control.Control;
import kd.bos.form.events.PreOpenFormEventArgs;
import kd.bos.metadata.form.container.TabPageAp;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

/**
 * 实时汇率对接-表单插件
 * @description:
 * @author longh
 * @date 2022年6月21日 下午5:21:04
 */
public class ExchangeEditFormPlugIn extends AbstractBillPlugIn {

	@Override
	public void preOpenForm(PreOpenFormEventArgs e) {
		super.preOpenForm(e);
		DynamicObject bill = BusinessDataServiceHelper.loadSingle("tpv_iscb_exchangeedit", "id", new QFilter[] {});
		if(bill != null) {
			BillShowParameter bsp = (BillShowParameter) e.getFormShowParameter();
			bsp.setPkId(bill.getPkValue());
			bsp.setStatus(OperationStatus.VIEW);
		}
	}
	
	@Override
	public void afterLoadData(EventObject e) {
		super.afterLoadData(e);
		this.showChart();
	}
	
	@Override
	public void registerListener(EventObject e) {
		super.registerListener(e);
		this.addClickListeners(new String[]{"tpv_labelap"});
	}
	
	@Override
	public void click(EventObject evt) {
		super.click(evt);
		String key = ((Control) evt.getSource()).getKey();
		switch (key) {
		case "tpv_labelap" :
			BillShowParameter showParameter = new BillShowParameter();
			DynamicObject sch_schedule = BusinessDataServiceHelper.loadSingle("sch_schedule", "id", 
					new QFilter[] {new QFilter("number", "=", "TxExchangeRateTask")});
			if(sch_schedule != null) {
				showParameter.setPkId(sch_schedule.getPkValue());
				showParameter.setFormId("sch_schedule");
				showParameter.setStatus(OperationStatus.EDIT);
				showParameter.getOpenStyle().setShowType(ShowType.MainNewTabPage);
				this.getView().showForm(showParameter);
			}else {
				this.getView().showTipNotification("未设置初始化调度计划方案。");
			}
			break;
		default:
			break;
		}
	}
	
	/**
     * 内嵌折线图
     */
	private void showChart() {
		IFormView view = this.getView();
		OperationStatus status = view.getFormShowParameter().getStatus();
		if(!OperationStatus.VIEW.equals(status)) {
			return;
		}
		List<QFilter> qFilters = new ArrayList<>();
		qFilters.add(new QFilter("status", QCP.equals, "C"));
		qFilters.add(new QFilter("enable", QCP.equals, "1"));
		qFilters.add(new QFilter("exctable.number", QCP.equals, TxExchangeRateTask.ERT_NO));
		qFilters.add(new QFilter("effectdate", ">", DateUtil.getLastMonthDay(new Date())));
		String selectFields = DynamicObjectUtil.getSelectfields("bd_exrate_tree", false);
		DynamicObject[] bd_exrate_tree = BusinessDataServiceHelper.load("bd_exrate_tree", selectFields, qFilters.toArray(new QFilter[0]), "effectdate");
		Map<String, Map<String, BigDecimal>> datas = new HashMap<>();
		List<String> dateList = new ArrayList<>();
		for (DynamicObject exrate : bd_exrate_tree) {
			String effectdate = DateUtil.formatDate(exrate.getDate("effectdate"));
			String curName = exrate.getLocaleString("orgcur.name").getLocaleValue();
			BigDecimal excval = exrate.getBigDecimal("excval");
			if(datas.containsKey(curName)) {
				datas.get(curName).put(effectdate, excval);
			}else {
				Map<String, BigDecimal> map = new HashMap<>();
				map.put(effectdate, excval);
				datas.put(curName, map);
			}
			if(!dateList.contains(effectdate)) {
				dateList.add(effectdate);
			}
		}
		String tpv_tabpageap = "tpv_tabpageap";
		this.showFormInContainer(datas, dateList, view, "ALL", tpv_tabpageap);
		int i = 1;
		List<Map<String, Object>> items = new ArrayList<>();
		for (String curName : datas.keySet()) {
			String targetKey = tpv_tabpageap + i++;
			TabPageAp tabPageAp = new TabPageAp();
			tabPageAp.setKey(targetKey);
			tabPageAp.setName(new LocaleString(curName));
			items.add(tabPageAp.createControl());
			
			this.showFormInContainer(datas, dateList, view, curName, targetKey);
		}
		Tab tab = view.getControl("tpv_tabap");
		tab.addControls(items);
	}

	private void showFormInContainer(Map<String, Map<String, BigDecimal>> datas, List<String> dateList, IFormView view,
			String curName, String targetKey) {
		FormShowParameter fsp = new FormShowParameter();
		fsp.setStatus(OperationStatus.EDIT);
		fsp.getOpenStyle().setShowType(ShowType.InContainer);
		fsp.getOpenStyle().setTargetKey(targetKey);
		fsp.setFormId("tpv_iscb_exchangechart");
		fsp.setCustomParam("curName", curName);
		fsp.setCustomParam("datas", datas);
		fsp.setCustomParam("dateList", dateList);
		view.showForm(fsp);
	}

}
