package com.itopview.wm.sys.formplugin.form;

import java.util.EventObject;
import java.util.HashMap;
import java.util.Map;

import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.tree.TreeNode;
import kd.bos.form.container.Tab;
import kd.bos.form.control.Image;
import kd.bos.form.control.TreeView;
import kd.bos.form.control.events.TabSelectEvent;
import kd.bos.form.control.events.TabSelectListener;
import kd.bos.form.control.events.TreeNodeClickListener;
import kd.bos.form.control.events.TreeNodeEvent;
import kd.bos.form.plugin.AbstractFormPlugin;

/**
 * 创建模板页面pc-表单插件扩展
 * @description:修改外贸模板描述
 * @author longh
 * @date 2022年6月13日 下午7:43:31
 */
public class ExtBizPageNewTemplatePcPlugin extends AbstractFormPlugin implements TreeNodeClickListener, TabSelectListener {
	private static final String BIZTPLTREE = "biztpltree";
	private static final String BIZTPLTAB = "biztpltab";
	private static final String TREETAB = "treetab";
	private static final String PAGETYPE = "pagetype";
	private static final String PAGETYPE_PC = "pc";
	private static final String TEXTAREAFIELD = "textareafield";
	private static final String IMAGEAP = "imageap3";
	private static Map<String, String> templateDesMap = new HashMap<>();
	private static Map<String, String> templateImageMap = new HashMap<>();
	
	@Override
	public void registerListener(EventObject e) {
		super.registerListener(e);
		TreeView biztpltree = (TreeView) this.getView().getControl(BIZTPLTREE);
		biztpltree.addTreeNodeClickListener(this);
		Tab tab = (Tab) this.getControl(TREETAB);
		tab.addTabSelectListener(this);
		this.loadTemplateDescription();
		this.loadTemplateImage();
	}
	
	@Override
	public void treeNodeClick(TreeNodeEvent evt) {
		Object nodeId = evt.getNodeId();
		this.nodeClick(nodeId);
	}
	
	@Override
	public void tabSelected(TabSelectEvent e) {
		if (BIZTPLTAB.equals(e.getTabKey())) {
			this.renderTplTree();
		}
	}
	
	private void nodeClick(Object nodeId) {
		Map<String, Object> configInfo = this.getView().getFormShowParameter().getCustomParams();
		String pageType = configInfo.get(PAGETYPE).toString();

		String templateDescription = templateDesMap.get(nodeId);
		if (StringUtils.isNotBlank(templateDescription)) {
			if (PAGETYPE_PC.equals(pageType)) {
				this.getModel().setValue(TEXTAREAFIELD, templateDescription);
			}
		}

		String imageUrl = (String) templateImageMap.get(nodeId);
		Image imageap3 = (Image) this.getControl(IMAGEAP);
		if (imageap3 != null && StringUtils.isNotBlank(imageUrl)) {
			imageap3.setUrl(imageUrl);
		}

	}
	
	private void renderTplTree() {
		TreeNode root = (TreeNode) SerializationUtils.fromJsonString(this.getPageCache().get("root"), TreeNode.class);
		if(root != null && root.getChildren().size() > 0) {
			this.nodeClick(root.getChildren().get(0).getId());
		}
	}

	private void loadTemplateDescription() {
		templateDesMap.put("tpv_tpl_bill1", "外贸通用模板1（基本信息）\n继承单据带组织模板并做了优化，加入了外贸特性字段及功能，包含基本信息页签、附件面板、制单信息等基本内容");
		templateDesMap.put("tpv_tpl_bill2", "外贸通用模板2（基本信息 + 财务信息）\n包含外贸通用模板1全部内容，增加了财务信息页签");
		templateDesMap.put("tpv_tpl_bill3", "外贸通用模板3（基本信息 + 财务信息 + 明细信息）\n包含外贸通用模板2全部内容，增加了明细信息页签");
		templateDesMap.put("tpv_tpl_bill4", "外贸通用模板4（基本信息 + 财务信息 + 明细信息 + 公司抬头）\n包含外贸通用模板3全部内容，增加了抬头信息页签");
		templateDesMap.put("tpv_tpl_base", "简易基础资料模板\n简化字段并缩小界面改为模态显示，取消提交审核操作\n包含分组基础资料带组织模板全部功能，保留基础资料模板功能，其他功能禁用可恢复");
		templateDesMap.put("tpv_tpl_basedataorg", "简易基础资料带组织模板\n简化字段并缩小界面改为模态显示，取消提交审核操作\n包含分组基础资料带组织模板全部功能，保留基础资料带组织模板功能，其他功能禁用可恢复");
		templateDesMap.put("tpv_tpl_basegroup", "简易分组基础资料模板\n简化字段并缩小界面改为模态显示，取消提交审核操作\n包含分组基础资料带组织模板全部功能，保留分组基础资料模板功能，其他功能禁用可恢复");
		templateDesMap.put("tpv_tpl_basegrouporg", "简易分组基础资料带组织模板\n简化字段并缩小界面改为模态显示，取消提交审核操作\n包含分组基础资料带组织模板全部功能，取消功能禁用可恢复");
		templateDesMap.put("tpv_tpl_basetree", "简易树形基础资料模板\n简化字段并缩小界面改为模态显示，取消提交审核操作\n包含树形基础资料带组织模板全部功能，保留树形基础资料模板功能，其他功能禁用可恢复");
		templateDesMap.put("tpv_tpl_basetreeorg", "简易树形基础资料带组织模板\n简化字段并缩小界面改为模态显示，取消提交审核操作\n包含树形基础资料带组织模板全部功能，保留树形基础资料带组织模板功能，其他功能禁用可恢复");
		templateDesMap.put("tpv_tpl_basetreeorg", "动态回填模板\n通过动态回填方案配置实现一般回填需求");
	}

	private void loadTemplateImage() {
		templateImageMap.put("tpv_tpl_bill1", "/images/pc/other/bos_zzl_480_266.png");
		templateImageMap.put("tpv_tpl_bill2", "/images/pc/other/bos_zzl_480_266.png");
		templateImageMap.put("tpv_tpl_bill3", "/images/pc/other/bos_zzl_480_266.png");
		templateImageMap.put("tpv_tpl_bill4", "/images/pc/other/bos_zzl_480_266.png");
	}
	
}
