package com.itopview.wm.sys.formplugin.form;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.sys.utils.MetaPropertyHelper;

import kd.bos.base.AbstractBasePlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.EntityType;
import kd.bos.entity.EntryType;
import kd.bos.entity.LinkEntryType;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.SubEntryType;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.filter.FilterCondition;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.form.control.Control;
import kd.bos.form.control.FilterGrid;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.events.BeforeDoOperationEventArgs;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.ComboEdit;
import kd.bos.form.field.ComboItem;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.form.operate.AbstractOperate;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QFilter;
import kd.bos.util.StringUtils;

/**
 * 卷宗方案-表单插件
 * @description:
 * @author longh
 * @date 2021年9月17日 上午10:48:35
 */
public class RelationBillFormPlugin extends AbstractBasePlugIn implements BeforeF7SelectListener {
	private static final String[] BILL_COLS = new String[]{"tpv_billcol", "tpv_billcolno"};
	private static final String[] REL_COLS = new String[]{"tpv_left", "tpv_thiscol", "tpv_thiscolno", "tpv_qcp", "tpv_relcol", "tpv_relcolno", "tpv_right", "tpv_andor"};
	
	@Override
	public void beforeBindData(EventObject e) {
		super.beforeBindData(e);
		this.isableAndBindRelbill();
		this.bindFilterGrid();
		this.initSettings();
	}
	
	private void initSettings() {
		this.bindColsMap();
		this.bindOrderbyCol();
		this.bindFilters();
		this.getModel().setDataChanged(false);
	}

	@Override
	public void afterBindData(EventObject e) {
		super.afterBindData(e);
		this.bindFilterGridData();
		DynamicObject entityObj = (DynamicObject) this.getModel().getValue("tpv_requirebill");
		String entityNumber = "";
		if(entityObj != null) {
			entityNumber = entityObj.getString("number");
		}
		this.fillEntityCombo(entityNumber);
	}
	
	private void isableAndBindRelbill() {
		DynamicObject dataEntity = this.getModel().getDataEntity(true);
		int level = dataEntity.getInt("level");
		if(level == 2 || level == 3) {
			DynamicObject parent = dataEntity.getDynamicObject("parent");
			if(parent != null) {
				this.getModel().setValue("tpv_relationbill", parent.getDynamicObject("tpv_relationbill"));
			}
		}else if(level == 4) {
			this.getView().showMessage("当前级次为3，不能继续新增子级方案！");
			this.getView().close();
		}
	}

	private void bindColsMap() {
		String tpv_colmap = this.getModel().getDataEntity().getString("tpv_colmap");
		JSONObject entry = null;
		if (StringUtils.isNotEmpty(tpv_colmap)) {
			entry = JSON.parseObject(tpv_colmap);
		}
		this.batchSetEntry("tpv_colsmap", entry);
	}
	
	private void batchSetEntry(String entryName, JSONObject entry) {
		IDataModel model = this.getModel();
		model.deleteEntryData(entryName);
		
		DynamicObject tpv_requirebill = (DynamicObject) model.getValue("tpv_requirebill");
		if (entry != null && tpv_requirebill != null) {
			Set<String> cols = entry.keySet();
			JSONArray colValues = null;
			int[] indexs = null;
			Iterator<String> iterator = cols.iterator();
			
			while (true) {
				String col;
				do {
					if (!iterator.hasNext()) {
						return;
					}
					col = (String) iterator.next();
				} while ("tpv_billcol".equals(col));

				colValues = entry.getJSONArray(col);
				int index = colValues.size();
				if (indexs == null) {
					indexs = model.batchCreateNewEntryRow(entryName, index);
				}

				for (int i = 0; i < index; ++i) {
					String key = String.valueOf(colValues.get(i));
					model.setValue(col, key, indexs[i]);
					String bill = tpv_requirebill.getString("number");
					model.setValue("tpv_billcol", MetaPropertyHelper.getEntityLangName(bill, key), indexs[i]);
				}
			}
		}
	}
	
	private void bindOrderbyCol() {
		IDataModel model = this.getModel();
		String tpv_orderbycolno = (String) model.getValue("tpv_orderbycolno");
		DynamicObject tpv_relationbill = (DynamicObject) model.getValue("tpv_relationbill");
		if(tpv_relationbill != null && !tpv_orderbycolno.isEmpty()) {
			String bill = tpv_relationbill.getString("number");
			model.setValue("tpv_orderbycol", MetaPropertyHelper.getEntityLangName(bill, tpv_orderbycolno));
		}
	}
	
	private void bindFilters() {
		String tpv_colmap = this.getModel().getDataEntity().getString("tpv_relation");
		JSONArray entry = null;
		if (StringUtils.isNotEmpty(tpv_colmap)) {
			entry = JSON.parseArray(tpv_colmap);
		}
		this.batchFilterEntry("tpv_filters", entry);
	}

	private void batchFilterEntry(String entryName, JSONArray entry) {
		IDataModel model = this.getModel();
		model.deleteEntryData(entryName);
		if (entry != null) {
			for (int i = 0; i < entry.size(); i++) {
				JSONObject rowData = entry.getJSONObject(i);
				int index = model.createNewEntryRow(entryName);
				Set<String> keySet = rowData.keySet();
				for (String key : keySet) {
					String bill = "";
					String billKey = rowData.getString(key);
					switch (key) {
					case "tpv_thiscol":
					case "tpv_relcol":
						break;
					case "tpv_thiscolno":
						bill = ((DynamicObject) model.getValue("tpv_requirebill")).getString("number");
						model.setValue("tpv_thiscol", MetaPropertyHelper.getEntityLangName(bill, billKey), index);
						model.setValue(key, rowData.get(key), index);
						break;
					case "tpv_relcolno":
						bill = ((DynamicObject) model.getValue("tpv_relationbill")).getString("number");
						model.setValue("tpv_relcol", MetaPropertyHelper.getEntityLangName(bill, billKey), index);
						model.setValue(key, rowData.get(key), index);
						break;
					default:
						model.setValue(key, rowData.get(key), index);
						break;
					}
				}
			}
		}
	}

	private void bindFilterGrid() {
		FilterGrid filterGrid = this.getView().getControl("tpv_filtergridap");
		DynamicObject tpv_relationbill = (DynamicObject) this.getModel().getValue("tpv_requirebill");
		if(tpv_relationbill != null) {
			filterGrid.setEntityNumber(tpv_relationbill.getString("number"));
		}else {
			filterGrid.setEntityNumber("1122");
		}
	}
	
	private void bindFilterGridData() {
		String tpv_filtergird = this.getModel().getDataEntity().getString("tpv_filtergird");
		if(!StringUtils.isBlank(tpv_filtergird)) {
			FilterCondition filterCondition = SerializationUtils.fromJsonString(tpv_filtergird, FilterCondition.class);
			if (filterCondition != null){
		    	FilterGrid filterGrid = this.getView().getControl("tpv_filtergridap");
		    	filterGrid.SetValue(filterCondition);
		    }
		}
	}
	
	private void fillEntityCombo(String entityNumber) {
		ComboEdit entryEditor = (ComboEdit) this.getView().getControl("tpv_selectentity");
		if(!entityNumber.isEmpty()) {
			MainEntityType mainType = EntityMetadataCache.getDataEntityType(entityNumber);
			List<ComboItem> entryItems = this.buildEntryComboItems(mainType);
			entryEditor.setComboItems(entryItems);
			String tpv_selectentity = (String) this.getModel().getValue("tpv_selectentity");
			String selectentity = "";
			for (ComboItem comboItem : entryItems) {
				if(comboItem.getValue().equals(tpv_selectentity)) {
					selectentity = tpv_selectentity;
				}
			}
			this.getModel().setValue("tpv_selectentity", selectentity);
		}else {
			entryEditor.setComboItems(null);
		}
	}
	
	private List<ComboItem> buildEntryComboItems(MainEntityType mainType) {
		ArrayList<ComboItem> entryItems = new ArrayList<>();
		ComboItem blankItem = new ComboItem();
		blankItem.setValue("");
		blankItem.setCaption(new LocaleString());
		entryItems.add(blankItem);
		if (mainType != null) {
			Map<String, EntityType> entitys = mainType.getAllEntities();
			for (EntityType entity : entitys.values()) {
				if (!(entity instanceof LinkEntryType) && entity instanceof EntryType && !(entity instanceof SubEntryType)) {
					ComboItem entryItem = new ComboItem();
					entryItem.setValue(entity.getName());
					entryItem.setCaption(entity.getDisplayName());
					entryItems.add(entryItem);
				}
			}
		}

		return entryItems;
	}
	
	@Override
	public void registerListener(EventObject e) {
		super.registerListener(e);
		this.addClickListeners(new String[]{"tpv_billcol", "tpv_thiscol", "tpv_relcol", "tpv_orderbycol"});
		((BasedataEdit) this.getControl("tpv_requirebill")).addBeforeF7SelectListener(this);
		((BasedataEdit) this.getControl("tpv_relationbill")).addBeforeF7SelectListener(this);
	}
	
	@Override
	public void click(EventObject evt) {
		super.click(evt);
		String key = ((Control) evt.getSource()).getKey();
		switch (key) {
			case "tpv_billcol" :
				this.clickBillCol(evt, key, "tpv_requirebill", true, true);
				break;
			case "tpv_thiscol" :
				this.clickBillCol(evt, key, "tpv_requirebill", false, true);
				break;
			case "tpv_relcol" :
				this.clickBillCol(evt, key, "tpv_relationbill", false, false);
				break;
			case "tpv_orderbycol" :
				this.clickBillColAny(evt, key, "tpv_relationbill", false);
				break;
		}
	}
	
	private void clickBillCol(EventObject evt, String key, String baseCol, boolean isMulti, boolean isAddEntry) {
		DynamicObject requireBill = MetaPropertyHelper.getAndCheckF7Value(this.getView(), baseCol);
		if(requireBill != null) {
			String formKey = requireBill.getString("number");
			String tpv_selectentity = (String) this.getModel().getValue("tpv_selectentity");
			List<String> entityKeys = new ArrayList<String>();
			if(isAddEntry) {
				entityKeys.add(tpv_selectentity);
			}
			entityKeys.add(formKey);
			MetaPropertyHelper.showPropertySelect(this, key, formKey, isMulti, entityKeys);
		}
	}
	
	private void clickBillColAny(EventObject evt, String key, String baseCol, boolean isMulti) {
		DynamicObject requireBill = MetaPropertyHelper.getAndCheckF7Value(this.getView(), baseCol);
		if(requireBill != null) {
			String formKey = requireBill.getString("number");
			MetaPropertyHelper.showPropertySelect(this, key, formKey, isMulti);
		}
	}
	
	@Override
	public void propertyChanged(PropertyChangedArgs e) {
		super.propertyChanged(e);
		String name = e.getProperty().getName();
		switch (name) {
			case "tpv_requirebill" :
				this.requireBillChanged(e);
				break;
		}
	}
	
	private void requireBillChanged(PropertyChangedArgs e) {
		this.bindFilterGrid();
		this.getView().updateView("tpv_filtergridap");
		
		this.getModel().deleteEntryData("tpv_colsmap");
		this.getModel().deleteEntryData("tpv_filters");
		
		FilterGrid filterGrid = this.getView().getControl("tpv_filtergridap");
    	filterGrid.SetValue(new FilterCondition());
	}
	
	@Override
	public void closedCallBack(ClosedCallBackEvent e) {
		super.closedCallBack(e);
		String key = e.getActionId();
		IDataModel model = this.getModel();
		switch (key) {
			case "tpv_billcol" :
				MetaPropertyHelper.callBackEntryData(model, e, "tpv_colsmap", "tpv_billcolno", "tpv_billcol", false);
				break;
			case "tpv_thiscol" :
				MetaPropertyHelper.callBackEntryData(model, e, "tpv_filters", "tpv_thiscolno", "tpv_thiscol", true);
				break;
			case "tpv_relcol" :
				MetaPropertyHelper.callBackEntryData(model, e, "tpv_filters", "tpv_relcolno", "tpv_relcol", true);
				break;
			case "tpv_orderbycol" :
				MetaPropertyHelper.callBackBillData(model, e, "tpv_orderbycolno", "tpv_orderbycol");
				break;
		}
	}
	
	@Override
	public void beforeF7Select(BeforeF7SelectEvent e) {
		IDataEntityProperty p = e.getProperty();
		String key = p.getName();
		switch (key) {
			case "tpv_requirebill" :
			case "tpv_relationbill" :
				this.beforeRequireBill(e);
				break;
		}
	}
	
	private void beforeRequireBill(BeforeF7SelectEvent e) {
		QFilter qFilter = new QFilter("modeltype", "in", new String[]{"BillFormModel", "BaseFormModel"});
		((ListShowParameter) e.getFormShowParameter()).getListFilterParameter().getQFilters().add(qFilter);
	}
	
	public void beforeDoOperation(BeforeDoOperationEventArgs e) {
		super.beforeDoOperation(e);
		String key = ((AbstractOperate) e.getSource()).getOperateKey();
		switch (key) {
			case "save" :
			case "submit" :
				this.beforeSave(e);
				break;
		}
	}
	
	@Override
	public void afterDoOperation(AfterDoOperationEventArgs e) {
		super.afterDoOperation(e);
		OperationResult operationResult = e.getOperationResult();
		if(operationResult != null && operationResult.isSuccess()) {
			this.bindFilterGridData();
			this.initSettings();
		}
	}

	private void beforeSave(BeforeDoOperationEventArgs e) {
		IDataModel model = this.getModel();
		String tpv_colmap = this.parseBillCol2Json(BILL_COLS, "tpv_colsmap");
		model.setValue("tpv_colmap", tpv_colmap);
		String tpv_filters = this.parseRelCol2Json(REL_COLS, "tpv_filters");
		model.setValue("tpv_relation", tpv_filters);
		String tpv_filtergird = this.parseFilterGrid2Json("tpv_filtergridap");
		model.setValue("tpv_filtergird", tpv_filtergird);
	}
	
	private String parseBillCol2Json(String[] colsName, String entryKey) {
		DynamicObjectCollection rows = this.getModel().getEntryEntity(entryKey);
		if (rows.isEmpty()) {
			return null;
		} else {
			JSONObject json = new JSONObject(colsName.length);
			JSONArray values = null;
			for (int i = 0; i < colsName.length; ++i) {
				String col = colsName[i];
				values = new JSONArray(rows.size());
				Iterator<DynamicObject> iterator = rows.iterator();
				while (iterator.hasNext()) {
					DynamicObject row = (DynamicObject) iterator.next();
					values.add(row.getString(col));
				}
				json.put(col, values);
			}

			return json.toJSONString();
		}
	}
	
	private String parseRelCol2Json(String[] relCols, String entryKey) {
		DynamicObjectCollection rows = this.getModel().getEntryEntity(entryKey);
		if (rows.isEmpty()) {
			return null;
		} else {
			JSONArray json = new JSONArray(rows.size());
			for (DynamicObject row : rows) {
				JSONObject values = new JSONObject(relCols.length);
				for (String col : relCols) {
					String value = row.getString(col);
					values.put(col, value);
				}
				json.add(values);
			}
			
			return json.toJSONString();
		}
	}
	
	private String parseFilterGrid2Json(String filterGridKey) {
		String filter = "";
		FilterGrid filterGrid = this.getView().getControl(filterGridKey);
	    FilterCondition filterCondition = filterGrid.getFilterGridState().getFilterCondition();
	    if (filterCondition != null){
	        filter = SerializationUtils.toJsonString(filterCondition);
	    }
	    
		return filter;
	}
	
}
