package com.itopview.wm.sys.formplugin.form;

import com.itopview.wm.sys.utils.PrintHelper;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.datamodel.events.AfterDeleteRowEventArgs;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.CloseCallBack;
import kd.bos.form.IFormView;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.mvc.bill.BillView;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 外贸通用模板1基类
 *
 * @author longh
 * @description:1、继承了版本变更功能 2、业务员部门携带、过滤
 * @date 2022年6月16日 下午2:30:41
 */
public class Tpl1BillPlugIn extends AbstractVersionBillPlugIn implements BeforeF7SelectListener {

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        OperationStatus status = ((BillView) e.getSource()).getFormShowParameter().getStatus();
        boolean addnew = OperationStatus.ADDNEW.equals(status);
        Object sourceBillId = this.getModel().getValue("tpv_sourcebillid");
        Object tpv_operator = this.getModel().getValue("tpv_operator");
//        initDept(addnew);
        //单据新增 部门为空.
        if (addnew && YEM.isEmpty(sourceBillId) && YEM.isEmpty(tpv_operator)) {
            this.getModel().setValue("tpv_department", null);
        }
        this.getModel().setDataChanged(false);
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        ((BasedataEdit) this.getControl("tpv_department")).addBeforeF7SelectListener(this);
        ((BasedataEdit) this.getControl("tpv_operator")).addBeforeF7SelectListener(this);
        ((BasedataEdit) this.getControl("tpv_approvalprocess")).addBeforeF7SelectListener(this);
        ((BasedataEdit) this.getControl("tpv_company")).addBeforeF7SelectListener(this);
        ((BasedataEdit) this.getControl("tpv_department_aps")).addBeforeF7SelectListener(this);
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String itemKey = evt.getItemKey();
        switch (itemKey) {
            case "tpv_print":
                try {
                    PrintHelper.print(this.getView());
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
                break;
            default:
                break;
        }
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String name = e.getProperty().getName();
        String entityId = this.getModel().getDataEntityType().getName();
        switch (name) {
            case "tpv_department":
//			deftF7(e);
                break;
            case "tpv_operator":
                operatorF7(e, name);
                break;
            case "tpv_approvalprocess":
                whethertopublish(e,entityId);
                break;
            case "tpv_company":
                company(e);
                break;
            case "tpv_department_aps":
                departmentAps(e);
                break;
            default:
                break;
        }
    }

    private void departmentAps(BeforeF7SelectEvent e) {
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        List<QFilter> QFilters = new ArrayList<QFilter>();
        DynamicObject[] bosAdminorgs = BusinessDataServiceHelper.load("bos_adminorg", "structure,viewparent", null);
        List<Long> list = Arrays.stream(bosAdminorgs)
                .flatMap(bosAdminorg -> bosAdminorg.getDynamicObjectCollection("structure").stream())
                .map(structure -> structure.getDynamicObject("viewparent"))
                .filter(Objects::nonNull)
                .map(viewparent -> viewparent.getLong("id"))
                .collect(Collectors.toList());
        QFilters.add(new QFilter("id", "not in", list));
        showParameter.getListFilterParameter().setQFilters(QFilters);
    }

    private void company(BeforeF7SelectEvent e) {
        DynamicObject tpvDepartment = (DynamicObject) this.getModel().getValue("tpv_department");
        List<QFilter> QFilters = new ArrayList();
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        if (tpvDepartment != null) {
            DynamicObject[] bosOrg = BusinessDataServiceHelper.load("bos_org_orgrelation_dept", "fromorg", new QFilter[]{new QFilter("toorg", "=", tpvDepartment.getPkValue())});
            List<Long> listOrg = Arrays.stream(bosOrg)
                    .map(org -> org.getDynamicObject("fromorg"))
                    .filter(Objects::nonNull)
                    .map(toorg -> toorg.getLong("id"))
                    .collect(Collectors.toList());
            QFilters.add(new QFilter("id", "in", listOrg));
        }
        QFilters.add(new QFilter("fissale", "=", true));
        showParameter.getListFilterParameter().setQFilters(QFilters);
    }

    //2024.04.26   zyf--【审批流程】过滤
    public static void whethertopublish(BeforeF7SelectEvent e, String entityId) {
        if (!entityId.isEmpty()) {
            QFilter qFilter = new QFilter("publish", QCP.equals, true);
            qFilter.and(new QFilter("discard", QCP.equals, false));
            qFilter.and(new QFilter("entrabillid.number", QCP.equals, entityId));
            ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
            showParameter.getListFilterParameter().getQFilters().add(qFilter);
        }
    }


    private void operatorF7(BeforeF7SelectEvent e, String key) {
        DynamicObject tpvCompany = (DynamicObject) this.getModel().getValue("tpv_company");
        if (tpvCompany != null) {
            List<QFilter> QFilters = new ArrayList();
            ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
            DynamicObject[] bosOrg = BusinessDataServiceHelper.load("bos_org_orgrelation_dept", "toorg", new QFilter[]{new QFilter("fromorg", "=", tpvCompany.getPkValue())});
            List<Long> listOrg = Arrays.stream(bosOrg)
                    .map(org -> org.getDynamicObject("toorg"))
                    .filter(Objects::nonNull)
                    .map(toorg -> toorg.getLong("id"))
                    .collect(Collectors.toList());
            CloseCallBack closeCallBack = new CloseCallBack(this, key);
            showParameter.setCloseCallBack(closeCallBack);
            QFilters.add(new QFilter("entryentity.dpt", "in", listOrg));
            showParameter.getListFilterParameter().setQFilters(QFilters);
            showParameter.getListFilterParameter().setOrderBy("entryentity.seq asc");
        } else {
            e.setCancel(true);
            this.getView().showTipNotification("请先选择公司！");
        }

    }

    /**
     * 部门过滤
     *
     * @param e
     */
    @SuppressWarnings("unused")
    private void deftF7(BeforeF7SelectEvent e) {
        IDataModel model = this.getModel();
        DynamicObject tpv_operator = (DynamicObject) model.getValue("tpv_operator");
        if (tpv_operator != null) {
            long userId = tpv_operator.getLong("id");
            List<Long> deptIds = getDeptIds(userId);
            QFilter qFilter = new QFilter("id", QCP.in, deptIds);
            ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
            showParameter.getTreeFilterParameter().getQFilters().add(qFilter);
            showParameter.getListFilterParameter().getQFilters().add(qFilter);
        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent e) {
        super.closedCallBack(e);
        String actionId = e.getActionId();
        switch (actionId) {
            case "tpv_operator":
                operatorCallBack(e);
                break;
            default:
                break;
        }
    }

    private void operatorCallBack(ClosedCallBackEvent e) {
        IDataModel model = this.getModel();
        ListSelectedRowCollection selectedRows = (ListSelectedRowCollection) e.getReturnData();
        if (selectedRows != null && selectedRows.size() > 0) {
            ListSelectedRow selectedRow = selectedRows.get(0);
            long userId = (long) selectedRow.getPrimaryKeyValue();//人员id
            long userEntryId = (long) selectedRow.getEntryPrimaryKeyValue();//部门分录id
            model.beginInit();
            model.setValue("tpv_operator", userId);
            DynamicObjectCollection entry = getUserDepts(userId);
            for (DynamicObject row : entry) {
                if (userEntryId == row.getLong("id")) {
                    model.setValue("tpv_department", row.get("org"));
                }
            }
            model.endInit();
            this.getView().updateView("tpv_operator");
            this.getView().updateView("tpv_department");
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String key = e.getProperty().getName();
        switch (key) {
            case "tpv_operator":
//                initDept(true);
                break;
            case "tpv_collaborative":
            case "tpv_department_aps":
                setValues();
                break;
            case "tpv_company":
                setOrg();
                break;
            case "tpv_automaticoff"://自动签就绪
                Boolean automaticoff = (Boolean) this.getModel().getValue("tpv_automaticoff");
                if (automaticoff) {
                    try {
                        PrintHelper.print(this.getView());
                    } catch (IOException ex) {
                        throw new RuntimeException(ex);
                    }
                }
            default:
                break;
        }
    }

    /**
     * @param :
     * @return void
     * @author: chen
     * @date: 2023/12/11 15:01
     * @description: 选择公司时给主业务组织赋值
     */
    private void setOrg() {
        IDataModel model = this.getModel();
        DynamicObject tpv_company = (DynamicObject) model.getValue("tpv_company");
        model.beginInit();
        if (tpv_company != null) {
            long companyID = tpv_company.getLong("id");
            model.setValue("org", companyID);
        } else {
            model.setValue("org", null);
            model.setValue("tpv_operator", null);
            model.setValue("tpv_department", null);
        }

        model.endInit();
        this.getView().updateView("org");
        this.getView().updateView("tpv_operator");
        this.getView().updateView("tpv_department");
//        initDept(true);
    }

    /**
     * 业务员带出部门,并设置部门锁定性
     *
     * @param setDeptValue
     */
    private void initDept(boolean setDeptValue) {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        boolean enableDept = false;
        DynamicObject tpv_operator = (DynamicObject) model.getValue("tpv_operator");
        Long deptId = null;
        List<Long> deptIds = new ArrayList<>();
        if (tpv_operator != null) {
            long userId = tpv_operator.getLong("id");
            deptIds = getDeptIds(userId);
            if (deptIds.size() > 0) {
                deptId = deptIds.get(0);
            }
            if (deptIds.size() > 1) {
                enableDept = true;
            }
            if (deptIds.size() == 0) {
                //部门为空
            }
        } else {
            model.setValue("tpv_department", null);
        }
        if (setDeptValue && deptIds.size() == 1) {
            model.setValue("tpv_department", deptId);
        }
        view.setEnable(enableDept, "tpv_department");
    }

    /**
     * 获取可选部门ID
     *
     * @param userId
     * @return
     */
    public static List<Long> getDeptIds(long userId) {
        List<Long> deptIds = new ArrayList<>();
        // List<Long> allDepartmentByUserId =
        // UserServiceHelper.getAllDepartmentByUserId(userId);
        // List<Long> allDepartmentByUserId = YEM.getUserChangeAbleOrgIds(userId);
        // QFilter qFilter = new QFilter("id", QCP.in, allDepartmentByUserId);
        // qFilter.and(new QFilter("orgpattern.number", QCP.equals, "Orgform06"));
        // DynamicObjectCollection query = QueryServiceHelper.query("bos_org", "id",
        // qFilter.toArray());
        // for (DynamicObject org : query) {
        // deptIds.add(org.getLong("id"));
        // }
        DynamicObjectCollection query = getUserDepts(userId);
        for (DynamicObject org : query) {
            deptIds.add(org.getLong("org"));
        }
        return deptIds;
    }

    private static DynamicObjectCollection getUserDepts(long userId) {
        QFilter qFilter = new QFilter("user", QCP.equals, userId);
//        qFilter.and(new QFilter("org.orgpattern.number", QCP.equals, "Orgform06"));
        return QueryServiceHelper.query("bos_userposition", "id,org", qFilter.toArray());
    }


    @Override
    public void afterDeleteRow(AfterDeleteRowEventArgs e) {
        String name = e.getEntryProp().getName();
        switch (name) {
            case "tpv_approvalsetting"://审批选项分录删行时重新赋值
                setValues();
                break;
        }
    }

    /**
     * 协同审批赋值
     */

    private void setValues() {
        ArrayList<String> approvaldepts = new ArrayList<>();
        DynamicObjectCollection entity = this.getModel().getEntryEntity("tpv_approvalsetting");
        for (DynamicObject object : entity) {
            boolean collaborative = object.getBoolean("tpv_collaborative");
            if (collaborative && object.getDynamicObject("tpv_department_aps") != null) {
                approvaldepts.add(object.getString("tpv_department_aps.name"));
            }
        }
        this.getModel().setValue("tpv_approvaldepts", approvaldepts.size() > 0 ? String.join(",", approvaldepts) : null);
    }
}
