package com.itopview.wm.sys.formplugin.form;

import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.Date;
import java.util.EventObject;

/**
 * 外贸通用模板2基类
 * @description:1、带出默认本位币、汇率表、汇率
 * @author longh
 * @date 2022年6月16日 下午2:30:41
 */
public class Tpl2BillPlugIn extends AbstractBillPlugIn {

	@Override
	public void afterCreateNewData(EventObject e) {
		super.afterCreateNewData(e);
//		this.setCurrencyAndExchangeTable();
		this.setExchangeRate();
	}
	
	/**
	 * 默认本位币、汇率表加载，根据（核算组织本位币设置）
	 */
	private void setCurrencyAndExchangeTable() {
		DynamicObject org = (DynamicObject) this.getModel().getValue("org");
		if(org != null) {
			DynamicObject bd_accountingsys_base = BusinessDataServiceHelper.loadSingle("bd_accountingsys_base", 
					"id,basecurrrency,exratetable", new QFilter[] {new QFilter("baseacctorg", QCP.equals, org.getPkValue())});
			if(bd_accountingsys_base != null) {
				DynamicObject basecurrrency = bd_accountingsys_base.getDynamicObject("basecurrrency");
				DynamicObject exratetable = bd_accountingsys_base.getDynamicObject("exratetable");
				this.getModel().setValue("tpv_currencybase", basecurrrency);
				this.getModel().setValue("tpv_exratetable", exratetable);
			}
		}
	}

	/**
	 * 汇率获取
	 */
	private void setExchangeRate() {
		Object tpv_exratetable = this.getModel().getValue("tpv_exratetable");
		Object tpv_currency = this.getModel().getValue("tpv_currency");
		Object tpv_currencybase = this.getModel().getValue("tpv_currencybase");
		Date tpv_exratedate = (Date) this.getModel().getValue("tpv_exratedate");
		BigDecimal exchangeRateCny = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 1L, tpv_exratedate);
		BigDecimal exchangeRateUsd = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 6L, tpv_exratedate);
		BigDecimal exchangeRate = YEM.getExchangeRate(tpv_exratetable, tpv_currency, tpv_currencybase, tpv_exratedate);
		this.getModel().setValue("tpv_exchangeratermb", exchangeRateCny);
		this.getModel().setValue("tpv_exchangerateusd", exchangeRateUsd);
		this.getModel().setValue("tpv_exchangerate", exchangeRate);
	}
	
	public void propertyChanged(PropertyChangedArgs e) {
		super.propertyChanged(e);
		String key = e.getProperty().getName();
		switch (key) {
		case "tpv_exratetable" :
		case "tpv_exratedate" :
		case "tpv_currency" :
		case "tpv_currencybase" :
			this.setExchangeRate();
			break;
		}
	}

}