package com.itopview.wm.sys.formplugin.list;

import com.itopview.wm.sys.formplugin.form.Tpl1BillPlugIn;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.form.events.BeforeDoOperationEventArgs;
import kd.bos.form.events.SetFilterEvent;
import kd.bos.form.operate.FormOperate;
import kd.bos.list.IListView;
import kd.bos.list.events.BeforeShowBillFormEvent;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.util.List;

/**
 * @ClassName: Tpl1ListFilterPlugIn
 * @Description:
 * @date: 2023/12/20 11:49
 * @author: chen
 */
public class ApprovalSettingListFilterPlugIn extends AbstractVersionListPlugin{
    @Override
    public void setFilter(SetFilterEvent e) {
        super.setFilter(e);

        long userId = YEM.getCurrentUserId();
        List<Long> deptIds = Tpl1BillPlugIn.getDeptIds(userId);
        List<QFilter> qFilterList = e.getQFilters();// 获取过滤规则列表
        qFilterList.clear();// 清空过滤规则
        //启用审批选项
        //可查看
        qFilterList.add(new QFilter("tpv_isuse", QCP.equals, false)
                .or(new QFilter("tpv_isuse", QCP.equals, true)
                        .or("tpv_approvalsetting.tpv_department_aps", QCP.in, deptIds.toArray())
                        .and("tpv_approvalsetting.tpv_queryable", QCP.equals, true)
                        .and("tpv_isuse", QCP.equals, true)
                        .or("tpv_department", QCP.in, deptIds)));
    }
    /**
     * 设置不可编辑用户的操作权限
     *
     * @param args
     */
    @Override
    public void beforeDoOperation(BeforeDoOperationEventArgs args) {
        super.beforeDoOperation(args);
        FormOperate source = (FormOperate) args.getSource();
        String key = source.getOperateKey();
        String name = "";
        switch (key) {
            case "delete"://删除
                name = "删除";
                judgeOperate(name, args);
                break;
            case "submit"://提交
                name = "提交";
                judgeOperate(name, args);
                break;
            case "unsubmit"://撤销
                name = "撤销";
                judgeOperate(name, args);
                break;
            case "audit"://审核
                name = "审核";
                judgeOperate(name, args);
                break;
            case "unaudit"://反审核
                name = "反审核";
                judgeOperate(name, args);
                break;
            case "push"://关联生成
                name = "关联生成";
                judgeOperate(name, args);
                break;
        }
    }

    /**
     * 操作权限判断
     *
     * @param key
     */
    private void judgeOperate(String key, BeforeDoOperationEventArgs args) {
        ListSelectedRowCollection selectedRows = this.getSelectedRows();
        String formId = ((IListView) this.getView()).getBillFormId();// 获取单据标识
        selectedRows.stream().forEach(rows -> {
            Long pkValue = (Long) rows.getPrimaryKeyValue();
            if (!isEdit(pkValue, formId)) {
                //没有操作权限,
                args.setCancel(true);
                String billNo = rows.getBillNo();
                this.getView().showTipNotification("单据编号: " + billNo + " , " + key + "失败," + "您没有该权限!");
            }
        });

    }

    @Override
    public void beforeShowBill(BeforeShowBillFormEvent e) {
        super.beforeShowBill(e);
        BillShowParameter parameter = e.getParameter();
        //单据新增时,不做操作
        Object pkId = parameter.getPkId();
        if (pkId == null) {
            return;
        }
        //获取单据ID
        Long pkValue = (Long) pkId;

        String formId = ((IListView) this.getView()).getBillFormId();// 获取单据标识
        DynamicObject loadSingle = BusinessDataServiceHelper.loadSingle(pkValue, formId);
        if (!isEdit(pkValue, formId)) {
            parameter.setStatus(OperationStatus.VIEW);//设置单据打开状态为查看
            loadSingle.set("tpv_isedit", false);//工具栏锁定
            SaveServiceHelper.update(loadSingle);
        }
    }

    private boolean isEdit(Long pkValue, String formId) {
        //当前登陆人ID
        long userId = YEM.getCurrentUserId();
        //登陆人所在部门
        List<Long> deptIds = Tpl1BillPlugIn.getDeptIds(userId);
        DynamicObject loadSingle = BusinessDataServiceHelper.loadSingle(pkValue, formId);


        boolean isUse = loadSingle.getBoolean("tpv_isuse");//是否启用
        if (!isUse) {
            return true;
        }
        DynamicObjectCollection collection = loadSingle.getDynamicObjectCollection("tpv_approvalsetting");
        for (DynamicObject dynamicObject : collection) {

            long deptId = dynamicObject.getLong("tpv_department_aps.id");
            boolean editable = dynamicObject.getBoolean("tpv_editable");
            // 当前登陆人所在的部门被允许编辑,或者单据头的部门存在在于当前登陆人所在的部门
            if ((editable && deptIds.contains(deptId)) || deptIds.contains(loadSingle.getLong("tpv_department.id"))) {
                return true;
            }
            //当前登陆人所在部门不被允许编辑,设置单据状态为查看,锁定工具栏
            if (!editable && deptIds.contains(deptId)) {
                return false;
            }
        }
        return true;
    }
}
