package com.itopview.wm.sys.formplugin.list;

import com.itopview.wm.utils.DynamicObjectUtil;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.ITreeModel;
import kd.bos.entity.tree.TreeNode;
import kd.bos.form.IFormView;
import kd.bos.form.control.TreeView;
import kd.bos.form.control.events.TreeNodeEvent;
import kd.bos.list.BillList;
import kd.bos.list.plugin.StandardTreeListPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.sdk.plugin.Plugin;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 文档查看-树形基础资料列表插件
 *
 * @author longh
 * @description:
 * @date 2024年2月19日
 */
public class FileViewTreeListPlugin extends StandardTreeListPlugin implements Plugin {

    @Override
    public void initialize() {
        super.initialize();
        initTreeNode();
    }

    private void initTreeNode() {
        IFormView view = this.getView();
        view.setVisible(false, "btnnew", "btnedit", "btndel");
        ITreeModel treeModel = this.getTreeModel();
        TreeNode rootNode = treeModel.createRootNode();
        treeModel.setRoot(rootNode);
        TreeView treeView = view.getControl("treeview");
        treeView.addNode(rootNode);
        String rootNodeId = rootNode.getId();
        Map<String, TreeNode> nodeMap = new HashMap<>();

        QFilter qFilter = new QFilter("enable", QCP.equals, "1");
        qFilter.and("status", QCP.equals, "C");
        String selectfields = DynamicObjectUtil.getSelectfields("tpv_filedirectory");
        DynamicObject[] nodeBills = BusinessDataServiceHelper.load("tpv_filedirectory", selectfields, qFilter.toArray(), "level");
        for (DynamicObject nodeBill : nodeBills) {
            TreeNode node = new TreeNode();
            int level = nodeBill.getInt("level");
            String nodeId = nodeBill.getString("id");
            node.setId(nodeId);
            node.setText(nodeBill.getString("name"));
            if (level == 1) {
                node.setParentid(rootNodeId);
                rootNode.addChild(node);
            } else {
                DynamicObject parent = nodeBill.getDynamicObject("parent");
                if (parent != null) {
                    String parentid = parent.getString("id");
                    node.setParentid(parentid);
                    TreeNode treeNode = nodeMap.get(parentid);
                    if (treeNode != null) {
                        treeNode.addChild(node);
                    }
                } else {
                    continue;
                }
            }
            treeView.addNode(node);
            nodeMap.put(nodeId, node);
        }
    }

    @Override
    public void treeNodeClick(TreeNodeEvent e) {
        super.treeNodeClick(e);
        String nodeId = (String) e.getNodeId();
        if (nodeId != null && nodeId.length() == 19) {
            List<Long> ids = new ArrayList<>();
            List<String> fileBillIds = new ArrayList<>();
            Map<String, String> nodeMap = new HashMap<>();
            QFilter qFilter = new QFilter("enable", QCP.equals, "1");
            qFilter.and("status", QCP.equals, "C");
            String selectfields = DynamicObjectUtil.getSelectfields("tpv_filedirectory");
            DynamicObject[] nodeBills = BusinessDataServiceHelper.load("tpv_filedirectory", selectfields, qFilter.toArray());
            for (DynamicObject nodeBill : nodeBills) {
                String id = nodeBill.getString("id");
                String parentid = "";
                DynamicObject parent = nodeBill.getDynamicObject("parent");
                if (parent != null) {
                    parentid = parent.getString("id");
                }
                nodeMap.put(id, parentid);
            }
            ids.add(Long.valueOf(nodeId));
            getTreeIds(nodeId, ids, nodeMap);

            DynamicObject[] bills = BusinessDataServiceHelper.load("tpv_fileupload", "id", new QFilter("group", QCP.in, ids).toArray());
            for (DynamicObject bill : bills) {
                fileBillIds.add(bill.getString("id"));
            }
            BillList billlistap = this.getControl("billlistap");
            billlistap.setFilter(new QFilter("finterid", QCP.in, fileBillIds));
        }
    }

    private void getTreeIds(String nodeId, List<Long> ids, Map<String, String> nodeMap) {
        for (Map.Entry<String, String> entry : nodeMap.entrySet()) {
            String parentid = entry.getValue();
            if (parentid.equals(nodeId)) {
                String newNodeId = entry.getKey();
                ids.add(Long.valueOf(newNodeId));
                getTreeIds(newNodeId, ids, nodeMap);
            }
        }
    }

}