package com.itopview.wm.sys.formplugin.list;

import java.util.List;

import com.itopview.wm.sys.formplugin.form.Tpl1BillPlugIn;
import com.itopview.wm.utils.YEM;

import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.filter.CommonFilterColumn;
import kd.bos.filter.FilterColumn;
import kd.bos.form.FormShowParameter;
import kd.bos.form.events.BeforeDoOperationEventArgs;
import kd.bos.form.events.FilterContainerInitArgs;
import kd.bos.form.events.SetFilterEvent;
import kd.bos.form.field.ComboItem;
import kd.bos.form.operate.FormOperate;
import kd.bos.list.IListView;
import kd.bos.list.events.BeforeShowBillFormEvent;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.permission.api.HasPermOrgResult;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;

/**
 * 外贸通用模板1-列表插件
 *
 * @author longh
 * @description:1、业务组织常用条件过滤（按当前登录用户组织权限） 
 * 2、继承了创建新版本功能
 * 3、贸易性质常用条件默认值加载（根据菜单入口参数Key: tradeProFilterDefaultValue ,Value: 贸易性质名称）
 * @date 2021年10月25日 下午5:49:18
 */
public class Tpl1ListPlugIn extends AbstractVersionListPlugin {

    @Override
    public void filterContainerInit(FilterContainerInitArgs args) {
        super.filterContainerInit(args);
        this.delEnableOrgFilterColumn(args);
        this.setTradeProFilterDefValue(args);
    }

    /**
     * 业务组织常用条件过滤（按当前登录用户组织权限）
     *
     * @param args
     */
    private void delEnableOrgFilterColumn(FilterContainerInitArgs args) {
        CommonFilterColumn filterColumn = (CommonFilterColumn) args.getFilterColumn("org.id");
        if (filterColumn != null) {
            long lUserId = RequestContext.get().getCurrUserId();
            HasPermOrgResult hasPermOrgResult = UserServiceHelper.getUserChangeAbleOrg(lUserId, true);
            List<Long> hasPermOrgs = hasPermOrgResult.getHasPermOrgs();

            List<ComboItem> comboItems = filterColumn.getComboItems();
            for (int i = comboItems.size() - 1; i >= 0; i--) {
                ComboItem comboItem = comboItems.get(i);
                String value = comboItem.getValue();
                if (value != null && !hasPermOrgs.contains(Long.valueOf(value))) {
                    comboItems.remove(i);
                }
            }
            filterColumn.setDefaultValue(null);
        }
    }

    /**
     * 贸易性质常用条件默认值加载（根据菜单入口参数Key: tradeProFilterDefaultValue ,Value: 贸易性质名称）
     *
     * @param args
     */
    private void setTradeProFilterDefValue(FilterContainerInitArgs args) {
        FilterColumn filterColumn = args.getFilterColumn("tpv_tradeproperty.name");
        if (filterColumn != null) {
            FormShowParameter showParameter = this.getView().getFormShowParameter();
            Object customParam = showParameter.getCustomParam("tradeProFilterDefaultValue");
            if (customParam != null) {
                DynamicObject bill = BusinessDataServiceHelper.loadSingle("tpv_bd_tradeproperty", "id",
                        new QFilter[]{new QFilter("name", QCP.equals, customParam.toString())});
                if (bill != null) {
                    filterColumn.setDefaultValue(bill.getPkValue().toString());
                }
            }
        }
    }

}
