package com.itopview.wm.sys.report;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;

import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.DataType;
import kd.bos.algo.Field;
import kd.bos.algo.JoinType;
import kd.bos.algo.RowMeta;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.IDataEntityType;
import kd.bos.entity.BillEntityType;
import kd.bos.entity.EntryType;
import kd.bos.entity.SubEntryType;
import kd.bos.entity.property.EntryProp;
import kd.bos.entity.property.LinkEntryProp;
import kd.bos.entity.report.AbstractReportColumn;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportColumn;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;

/**
 * 数据字典
 * 
 * @description 报表查询插件
 * @author longh
 * @date 2023年6月26日
 */
public class DataDictionaryRptQuery extends AbstractReportListDataPlugin {
	private static final String algoKey = DataDictionaryRptQuery.class.getName();
	private static List<String> fieldKeys1 = new ArrayList<>();// 表单元数据
	private static List<String> fieldKeys2 = new ArrayList<>();// 表单字段
	private static List<String> noPropNames = new ArrayList<>();
	static {
		noPropNames.add("LongProp");
		noPropNames.add("LinkEntryProp");
		noPropNames.add("DynamicLocaleProperty");
	}

	@Override
	public DataSet query(ReportQueryParam param, Object obj) throws Throwable {
		return query(param);
	}

	@Override
	public List<AbstractReportColumn> getColumns(List<AbstractReportColumn> columns) throws Throwable {
		FilterInfo filterInfo = this.getQueryParam().getFilter();
		String rptType = filterInfo.getString("tpv_type");
		List<String> fieldKeys = "2".equals(rptType) ? fieldKeys2 : fieldKeys1;
		for (int i = 0; i < columns.size(); i++) {
			ReportColumn column = (ReportColumn) columns.get(i);
			String fieldKey = column.getFieldKey();
			if (!fieldKeys.contains(fieldKey)) {
				columns.remove(i--);
			}
		}
		return super.getColumns(columns);
	}

	private DataSet query(ReportQueryParam param) {
		FilterInfo filterInfo = param.getFilter();
		String rptType = filterInfo.getString("tpv_type");
		if ("2".equals(rptType)) {// 表单字段
			DynamicObject tpv_fentityobj = filterInfo.getDynamicObject("tpv_fentityobj");
			String entityKey = tpv_fentityobj == null ? "" : tpv_fentityobj.getString("number");
			Field[] formFields = getFormFields();
			Collection<Object[]> collection = getFormFieldsData(entityKey);
			return Algo.create(algoKey).createDataSet(collection, new RowMeta(formFields));
		} else {// 表单元数据
			DataSet formmeta = queryFormMeta();
			DataSet bizapp = queryBizApp();
			DataSet dataSet = formmeta.join(bizapp, JoinType.INNER).on("bizappid", "tpv_appid")
					.select(RptUtil.getDataSetFiledAlias(formmeta), RptUtil.getDataSetFiledAlias(bizapp)).finish();
			fieldKeys1 = RptUtil.getDataSetAliasList(dataSet);
			return dataSet.where(createWhereStr(param))
					.orderBy(new String[] { "tpv_bizcloud", "tpv_appid", "tpv_modeltype" });
		}
	}

	private Collection<Object[]> getFormFieldsData(String entityKey) {
		Collection<Object[]> collection = new ArrayList<>();
		if (YEM.isNotEmpty(entityKey)) {
			initMetaData(collection, YEM.getDataEntityType(entityKey));
		}
		return collection;
	}

	private void initMetaData(Collection<Object[]> collection, IDataEntityType entityType) {
		String localeName = getLocaleName(entityType.getDisplayName());
		String parentLocaleName = "";
		if (entityType instanceof BillEntityType) {
			parentLocaleName = localeName;
			localeName = "单据头";
		} else if (entityType instanceof EntryType) {
			parentLocaleName = getLocaleName(entityType.getParent().getDisplayName());
		}
		List<IDataEntityType> entryTypes = new ArrayList<>();
		for (IDataEntityProperty property : entityType.getProperties()) {
			String number = property.getName();
			String alias = property.getAlias();
			String classSimpleName = property.getClass().getSimpleName();
			if (property instanceof EntryProp && !(property instanceof LinkEntryProp)) {
				entryTypes.add(((EntryProp) property).getItemType());
			} else {
				boolean isId = "id".equals(number);
				if (!noPropNames.contains(classSimpleName) || isId) {
					String displayName = isId ? "ID" : getLocaleName(property.getDisplayName());
					String fullnumber = number;
					String fullname = displayName;
					IDataEntityType parent = property.getParent();
					if (parent instanceof SubEntryType) {
						IDataEntityType grandParent = parent.getParent();
						fullnumber = String.format("%s.%s.%s", grandParent.getName(), parent.getName(), fullnumber);
						fullname = String.format("%s.%s.%s", getLocaleName(grandParent.getDisplayName()),
								getLocaleName(parent.getDisplayName()), fullname);
					} else if (parent instanceof EntryType) {
						fullnumber = String.format("%s.%s", parent.getName(), fullnumber);
						fullname = String.format("%s.%s", getLocaleName(parent.getDisplayName()), fullname);
					}
					int i = 0;
					Object[] row = new Object[fieldKeys2.size()];
					row[i++] = parentLocaleName;
					row[i++] = localeName;
					row[i++] = number;
					row[i++] = fullnumber;
					row[i++] = alias;
					row[i++] = displayName;
					row[i++] = fullname;
					row[i++] = classSimpleName;
					row[i++] = classSimpleName;
					collection.add(row);
				}
			}
		}
		for (IDataEntityType entryType : entryTypes) {
			initMetaData(collection, entryType);
		}
	}

	private String getLocaleName(LocaleString displayName) {
		return displayName == null ? "" : displayName.getLocaleValue();
	}

	private Field[] getFormFields() {
		fieldKeys2.clear();
		fieldKeys2.add("tpv_parent2");
		fieldKeys2.add("tpv_entity2");
		fieldKeys2.add("tpv_number2");
		fieldKeys2.add("tpv_fullnumber2");
		fieldKeys2.add("tpv_alias2");
		fieldKeys2.add("tpv_name2");
		fieldKeys2.add("tpv_fullname2");
		fieldKeys2.add("tpv_datatype2");
		fieldKeys2.add("tpv_aliastype2");

		List<Field> fields = new ArrayList<>();
		for (String key : fieldKeys2) {
			fields.add(new Field(key, DataType.StringType));
		}
		return fields.toArray(new Field[0]);
	}

	private DataSet queryFormMeta() {
		List<String> fields = new ArrayList<>();
		fields.add("id tpv_entityid");
		fields.add("number tpv_entitykey");
		fields.add("name tpv_entityname");
		fields.add("bizappid");
		fields.add("modeltype tpv_modeltype");
		fields.add("isv tpv_isv");
		return ORM.create().queryDataSet(algoKey, "bos_formmeta", String.join(",", fields), null);
	}

	private DataSet queryBizApp() {
		List<String> fields = new ArrayList<>();
		fields.add("id tpv_appid");
		fields.add("number tpv_appkey");
		fields.add("name tpv_appname");
		fields.add("bizcloud tpv_bizcloud");
		fields.add("bizcloud.name tpv_bizcloudname");
		fields.add("bizcloud.number tpv_bizcloudkey");
		fields.add("dbroute tpv_dbroute");
		return ORM.create().queryDataSet(algoKey, "bos_devportal_bizapp", String.join(",", fields), null);
	}

	private String createWhereStr(ReportQueryParam param) {
		FilterInfo filterInfo = param.getFilter();
		String whereFilter = "1 = 1";
		// 业务云
		String tpv_fbizcloud = filterInfo.getString("tpv_fbizcloud");
		if (YEM.isNotEmpty(tpv_fbizcloud)) {
			whereFilter = String.format("%s and (tpv_bizcloudkey like '%%%s%%' or tpv_bizcloudname like '%%%s%%')",
					whereFilter, tpv_fbizcloud, tpv_fbizcloud);
		}
		// 应用
		String tpv_fapp = filterInfo.getString("tpv_fapp");
		if (YEM.isNotEmpty(tpv_fapp)) {
			whereFilter = String.format("%s and (tpv_appkey like '%%%s%%' or tpv_appname like '%%%s%%')", whereFilter,
					tpv_fapp, tpv_fapp);
		}
		// 表单
		String tpv_fentity = filterInfo.getString("tpv_fentity");
		if (YEM.isNotEmpty(tpv_fentity)) {
			whereFilter = String.format("%s and (tpv_entitykey like '%%%s%%' or tpv_entityname like '%%%s%%')",
					whereFilter, tpv_fentity, tpv_fentity);
		}
		// 模型类型
		String tpv_fmodeltype = filterInfo.getString("tpv_fmodeltype");
		if (YEM.isNotEmpty(tpv_fmodeltype)) {
			whereFilter = String.format("%s and tpv_modeltype = '%s'", whereFilter, tpv_fmodeltype);
		}
		// 开发商标识
		String tpv_fisv = filterInfo.getString("tpv_fisv");
		if (YEM.isNotEmpty(tpv_fisv)) {
			whereFilter = String.format("%s and tpv_isv = '%s'", whereFilter, tpv_fisv);
		}
		// 二开类型
		String tpv_fext = filterInfo.getString("tpv_fext");
		if ("B".equals(tpv_fext)) {// 非扩展
			whereFilter += " and tpv_entitykey not like '%_ext'";
		} else if ("C".equals(tpv_fext)) {// 仅扩展
			whereFilter += " and tpv_entitykey like '%_ext'";
		}

		return whereFilter;
	}

}
