package com.itopview.wm.sys.task;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.itopview.wm.utils.AttachmentFieldUtil;
import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.DynamicObjectUtil;
import com.itopview.wm.utils.FileUtil;

import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.exception.KDException;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.DeleteServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

/**
 * 日志归档-定时任务
 * @description:
 * @author longh
 * @date 2022年5月24日 下午3:35:09
 */
public class LogFileTask extends AbstractTask {
	private static final String KEY_FORM = "tpv_imagebasedatatest";
	private static final String LOG_FORM = "tpv_bs_logfile";
	private static final int SIZE_MAX = 2;
	
	@Override
	public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
		
		List<QFilter> qFilters = new ArrayList<>();
//		qFilters.add(new QFilter("createtime", QCP.equals, ""));
		String selectFields = DynamicObjectUtil.getSelectfields(KEY_FORM, false);
		DynamicObject[] bills = BusinessDataServiceHelper.load(KEY_FORM, selectFields, 
				qFilters.toArray(new QFilter[0]), "createtime", SIZE_MAX);
		
		if(bills.length == SIZE_MAX) {
			Date now = new Date();
			MainEntityType type = EntityMetadataCache.getDataEntityType(KEY_FORM);
			String monthStr = new SimpleDateFormat("yyyyMMdd").format(now);
			FileUtil fileUtil = new FileUtil(LOG_FORM);
			Object[] delIds = new Object[bills.length];
			try {
				File newFile = fileUtil.createNewFile(String.format("LOG_%s.txt", monthStr));
				BufferedWriter writer = new BufferedWriter(new FileWriter(newFile));
				for (int i = 0; i < bills.length; i++) {
					DynamicObject bill = bills[i];
					delIds[i] = bill.getPkValue();
					
					String time = DateUtil.formatDateHms(bill.getDate("createtime"));
					String successNums = bill.getString("tpv_textfield2");
					String failNums = bill.getString("tpv_textfield3");
					String logType = bill.getString("tpv_textfield");
					String number = bill.getString("number");
					String name = bill.getString("name");
					String description = bill.getString("tpv_textfield1");
					
					writer.write(String.format("Time:%s	Suucess:%s	fail:%s\r\n%s	%s	%s\r\n%s\r\n\r\n",
							time, successNums, failNums, logType, number, name, description));
				}
				writer.close();
				
				DynamicObject attObj = AttachmentFieldUtil.uploadAttachment(newFile.getPath());
				
				DynamicObject logObj = BusinessDataServiceHelper.newDynamicObject(LOG_FORM);
				logObj.set("name", "苍穹<->星空接口日志");
				DynamicObjectCollection tpv_attachmentfield = logObj.getDynamicObjectCollection("tpv_attachmentfield");
				DynamicObjectType attType = tpv_attachmentfield.getDynamicObjectType();
				DynamicObject attField = new DynamicObject(attType);
				attField.set("fbasedataid", attObj);
				attField.set("fbasedataid_id", attObj.getPkValue());
				tpv_attachmentfield.add(attField);
				OperationResult saveOperate = SaveServiceHelper.saveOperate(LOG_FORM, new DynamicObject[] {logObj}, OperateOption.create());
				if(saveOperate.isSuccess()) {
					DeleteServiceHelper.delete(type, delIds);
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
	}

}
