package com.itopview.wm.sys.task;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.ExchangeRateUtil;

import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.exception.KDBizException;
import kd.bos.exception.KDException;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

/**
 * 实时汇率获取
 * @description:调用接口获取实时汇率并写入实时汇率表
 * @author longh
 * @date 2022年6月21日 下午9:05:30
 */
public class TxExchangeRateTask extends AbstractTask {
	public static final String ERT_NO = "TXY_01";
	
	@Override
	public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
		DynamicObject bd_exratetable = BusinessDataServiceHelper.loadSingle("bd_exratetable", "id", 
				new QFilter[] {new QFilter("number", "=", ERT_NO)});
		DynamicObject[] currencys = BusinessDataServiceHelper.load("bd_currency", "id,number", 
				new QFilter[] {new QFilter("enable", "=", "1")});
		DynamicObject bd_currency_cny = null;
		Map<String, DynamicObject> currencyMap = new HashMap<>();
		for (DynamicObject currency : currencys) {
			String currISO = currency.getString("number");
			if("CNY".equals(currISO)) {
				bd_currency_cny = currency;
			}else {
				currencyMap.put(currISO, currency);
			}
		}
		if(bd_exratetable == null) {
			bd_exratetable = BusinessDataServiceHelper.newDynamicObject("bd_exratetable");
			bd_exratetable.set("number", ERT_NO);
			bd_exratetable.set("name", "实时汇率表");
			bd_exratetable.set("status", "C");
			bd_exratetable.set("enable", "1");
			bd_exratetable.set("cur", bd_currency_cny);
			SaveServiceHelper.save(new DynamicObject[] {bd_exratetable});
		}
		Date startOfDay = DateUtil.getStartOfDay(new Date());
		DynamicObject[] bd_exrate_tree = BusinessDataServiceHelper.load("bd_exrate_tree", "id,orgcur", 
				new QFilter[] {new QFilter("effectdate", "=", startOfDay)});
		List<String> existList = new ArrayList<>();
		for (DynamicObject basedata : bd_exrate_tree) {
			existList.add(basedata.getString("orgcur.number"));
		}
		for (String fromCurrISO : currencyMap.keySet()) {
			if(existList.contains(fromCurrISO)) {
				continue;
			}
			BigDecimal excval = ExchangeRateUtil.getExchangeRateVal(fromCurrISO, "CNY");
			if(excval.compareTo(BigDecimal.ZERO) == 0) {
				throw new KDBizException("汇率获取为零，云市场汇率获取次数可能已经用完。");
			}
			BigDecimal indirectexrate = BigDecimalUtils.div(BigDecimal.ONE, excval, 10);
			DynamicObject new_bd_exrate_tree = BusinessDataServiceHelper.newDynamicObject("bd_exrate_tree");
			new_bd_exrate_tree.set("orgcur", currencyMap.get(fromCurrISO));
			new_bd_exrate_tree.set("cur", bd_currency_cny);
			new_bd_exrate_tree.set("excval", excval);
			new_bd_exrate_tree.set("indirectexrate", indirectexrate);
			new_bd_exrate_tree.set("status", "C");
			new_bd_exrate_tree.set("enable", "1");
			new_bd_exrate_tree.set("exctype", "2");
			new_bd_exrate_tree.set("exctable", bd_exratetable);
			new_bd_exrate_tree.set("effectdate", startOfDay);
			SaveServiceHelper.save(new DynamicObject[] {new_bd_exrate_tree});
		}
	}

}
