package com.itopview.wm.sys.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.Stack;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.YEM;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.entity.BillEntityType;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.EntityType;
import kd.bos.entity.EntryType;
import kd.bos.entity.LinkEntryType;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.ValueMapItem;
import kd.bos.entity.filter.FilterBuilder;
import kd.bos.entity.filter.FilterCondition;
import kd.bos.entity.property.AmountProp;
import kd.bos.entity.property.BasedataProp;
import kd.bos.entity.property.BooleanProp;
import kd.bos.entity.property.ComboProp;
import kd.bos.entity.property.DateProp;
import kd.bos.entity.property.DateTimeProp;
import kd.bos.entity.property.DecimalProp;
import kd.bos.entity.property.EntryProp;
import kd.bos.entity.property.IntegerProp;
import kd.bos.entity.property.MuliLangTextProp;
import kd.bos.entity.property.PictureProp;
import kd.bos.entity.property.PriceProp;
import kd.bos.entity.property.TextProp;
import kd.bos.metadata.entity.businessfield.BasedataField;
import kd.bos.metadata.entity.commonfield.CheckBoxField;
import kd.bos.metadata.entity.commonfield.ComboField;
import kd.bos.metadata.entity.commonfield.ComboItem;
import kd.bos.metadata.entity.commonfield.DateField;
import kd.bos.metadata.entity.commonfield.DateTimeField;
import kd.bos.metadata.entity.commonfield.DecimalField;
import kd.bos.metadata.entity.commonfield.Field;
import kd.bos.metadata.entity.commonfield.IntegerField;
import kd.bos.metadata.entity.commonfield.MuliLangTextField;
import kd.bos.metadata.entity.commonfield.PictureField;
import kd.bos.metadata.entity.commonfield.TextField;
import kd.bos.metadata.form.control.EntryAp;
import kd.bos.metadata.form.control.EntryFieldAp;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.scmc.sbs.common.util.MultiLangUtils;

/**
 * 业务卷宗工具类
 * @description:动态构建字段辅助
 * @author longh
 * @date 2021年9月28日 下午3:28:22
 */
public class BizFileBuildHelper {
	private MainEntityType mainType;
	private Set<String> selectedEntity = new HashSet<>();
	private List<String> selectedField = new ArrayList<>();
	private Map<String, String> selectedFieldName = new HashMap<>();
	private List<IDataEntityProperty> selectedProp = new ArrayList<>();
	private Map<String, List<IDataEntityProperty>> entityProp = new HashMap<>();
	private EntryAp entryAp;
	private String title;
	private String nodeId;
	private String uuid;
	private String requireBillKey;
	private String relationBillKey;
	private DynamicObject[] requireBills;
	
	public MainEntityType getMainType() {
		return mainType;
	}

	public Set<String> getSelectedEntity() {
		return selectedEntity;
	}

	public List<String> getSelectedField() {
		return selectedField;
	}

	public Map<String, String> getSelectedFieldName() {
		return selectedFieldName;
	}
	
	public List<IDataEntityProperty> getSelectedProp() {
		return selectedProp;
	}
	
	public Map<String, List<IDataEntityProperty>> getEntityProp() {
		return entityProp;
	}

	public EntryAp getEntryAp() {
		return entryAp;
	}
	
	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}
	
	public String getNodeId() {
		return nodeId;
	}
	
	public String getUuid() {
		return uuid;
	}
	
	public String getRequireBillKey() {
		return requireBillKey;
	}

	public String getRelationBillKey() {
		return relationBillKey;
	}

	public DynamicObject[] getRequireBills() {
		return requireBills;
	}
	
	public BizFileBuildHelper(String nodeId, String colMap, String relation, String filtergird, String reqBillKey, String relBillKey) {
		this.mainType = EntityMetadataCache.getDataEntityType(reqBillKey);
    	JSONObject colsEntry = JSON.parseObject(colMap);
		if (colsEntry != null) {
			JSONArray colValues = colsEntry.getJSONArray("tpv_billcolno");
			for (Object col : colValues) {
				String key = String.valueOf(col);
				String entityKeyWithLang = MultiLangUtils.getEntityKeyWithLang(reqBillKey, key);
				if(!key.equals(entityKeyWithLang)) {
					this.selectedField.add(key);
					this.selectedFieldName.put(key, entityKeyWithLang);
				}
			}
		}
		List<QFilter> reqFilters = this.createFilters(nodeId, relation, filtergird, reqBillKey, relBillKey);
    	if(reqFilters.size() > 0) {
    		String selectFields = String.join(",", this.selectedField);
    		this.requireBills = BusinessDataServiceHelper.load(reqBillKey, selectFields, reqFilters.toArray(new QFilter[0]));
    	}
	}
	
	public BizFileBuildHelper(String title, String nodeId, String uuid, String tpv_selectentity, String colMap, String relation, String filtergird, String reqBillKey, String relBillKey) {
		this.setTitle(title);
		this.nodeId = nodeId;
		this.uuid = uuid;
		this.requireBillKey = reqBillKey;
		this.relationBillKey = relBillKey;
		this.mainType = EntityMetadataCache.getDataEntityType(reqBillKey);
    	Iterator<Entry<String, EntityType>> iterator = this.mainType.getAllEntities().entrySet().iterator();
    	while (iterator.hasNext()) {
    		Entry<String, EntityType> entity = (Entry<String, EntityType>) iterator.next();
    		String name = entity.getValue().getName();
    		if (!(entity.getValue() instanceof LinkEntryType) && name.equals(tpv_selectentity)) {
    			this.selectedEntity.add(name);
    		}
    	}
    	JSONObject colsEntry = JSON.parseObject(colMap);
		if (colsEntry != null) {
			JSONArray colValues = colsEntry.getJSONArray("tpv_billcolno");
			for (Object col : colValues) {
				String key = String.valueOf(col);
				String entityKeyWithLang = MultiLangUtils.getEntityKeyWithLang(reqBillKey, key);
				if(!key.equals(entityKeyWithLang)) {
					this.selectedField.add(key);
					this.selectedFieldName.put(key, entityKeyWithLang);
				}
			}
		}
		List<QFilter> reqFilters = this.createFilters(nodeId, relation, filtergird, reqBillKey, relBillKey);
    	if(reqFilters.size() > 0) {
    		String selectFields = String.join(",", this.selectedField);
    		this.requireBills = BusinessDataServiceHelper.load(reqBillKey, selectFields, reqFilters.toArray(new QFilter[0]));
    	}
    	
    	this.createDynamicEntryAp();
	}

	private List<QFilter> createFilters(String nodeId, String relation, String filtergird, String reqBillKey, String relBillKey) {
		String noKey = "billno";
		DynamicObject bos_entity = BusinessDataServiceHelper.loadSingle("bos_entityobject", "id,number,modeltype", 
				new QFilter[] {new QFilter("number", QCP.equals, reqBillKey)});
		if(bos_entity != null && "BaseFormModel".equals(bos_entity.getString("modeltype"))) {
			noKey = "number";
		}
		QFilter billFilter = new QFilter(noKey, QCP.equals, nodeId);
		
		List<QFilter> filters = new ArrayList<>();
		if(reqBillKey.equals(relBillKey)) {
			filters.add(billFilter);
		}else {
			JSONArray relationData = JSON.parseArray(relation);
			if(relationData != null) {
				List<String> selectedField = new ArrayList<>();
				selectedField.add("id");
				Map<Integer, Map<String, String>> filterMaps = new HashMap<>();
				StringBuilder relationStr = new StringBuilder();
	    		for (int i = 0; i < relationData.size(); i++) {
					JSONObject rowData = relationData.getJSONObject(i);
					String tpv_left = this.getBrackets(rowData.getString("tpv_left"), '(');
					String tpv_right = this.getBrackets(rowData.getString("tpv_right"), ')');
					String tpv_andor = rowData.getString("tpv_andor");
					relationStr.append(tpv_left).append(i).append(tpv_right).append(tpv_andor);
					
					String tpv_relcolno = rowData.getString("tpv_relcolno");
					Map<String, String> filterMap = new HashMap<>();
					filterMap.put("key", rowData.getString("tpv_thiscolno"));
					filterMap.put("qcp", rowData.getString("tpv_qcp"));
					filterMap.put("value", tpv_relcolno);
					filterMaps.put(i, filterMap);
					
					if(tpv_relcolno != null && tpv_relcolno.length() > 0) {
						selectedField.add(tpv_relcolno);
					}
				}
	    		String selectFields = String.join(",", selectedField);
	    		DynamicObject relBill = BusinessDataServiceHelper.loadSingle(relBillKey, selectFields, new QFilter[] {billFilter});
	    		if(relBill != null) {
	    			QFilter filter = this.createFilter(relationStr.toString(), relBill, filterMaps);
		    		if(filter != null) {
		    			filters.add(filter);
		    		}
		    		if(!filtergird.isEmpty()) {
		    			FilterCondition filterCondition = SerializationUtils.fromJsonString(filtergird, FilterCondition.class);
		    			if (filterCondition != null){
		       			 	FilterBuilder fBuilder = new FilterBuilder(this.mainType, filterCondition); 
		       			 	fBuilder.buildFilter(); 
		       			 filters.addAll(fBuilder.getQFilters());
		    		    }
		    		}
				}
			}
		}
		return filters;
	}

	private QFilter createFilter(String relationStr, DynamicObject relBill, Map<Integer, Map<String, String>> filterMaps) {
		System.out.println(relationStr);
		Stack<Integer> stack = new Stack<>();
		stack.add(0);
		QFilter qf = null;
		String andor = "";
		for(int i = 0; i < relationStr.length(); i++) {
			char c = relationStr.charAt(i);
			if(c == '('){
				stack.add(i+1);
			}else if(c == ')'){
				int index = (int) stack.pop();
				if(stack.size() == 1){
					String relStr = relationStr;
					relStr = relStr.substring(index, i);
					QFilter qFilter = this.createFilter(relStr, relBill, filterMaps);
					if(qf == null) {
						qf = qFilter;
					}else if("A".equals(andor)) {
						qf = qf.and(qFilter);
					}else if("B".equals(andor)) {
						qf = qf.or(qFilter);
					}
				}else if(index == 0){
					stack.add(0);
				}
			}else if(stack.size() <= 1) {
				if(c == 'A' || c == 'B'){
					andor = String.valueOf(c);
				}else {
					Map<String, String> map = filterMaps.get(Character.getNumericValue(c));
					Object value = relBill.get(map.get("value"));
					if(value instanceof DynamicObject) {
						value = ((DynamicObject) value).getPkValue();
					}
					QFilter qFilter = new QFilter(map.get("key"), map.get("qcp"), value);
					if(qf == null) {
						qf = qFilter;
					}else if("A".equals(andor)) {
						qf = qf.and(qFilter);
					}else if("B".equals(andor)) {
						qf = qf.or(qFilter);
					}
				}
			}
		}
		return qf;
	}

	private String getBrackets(String str, char c) {
		String code = "";
		if(str != null && str.length() > 0) {
			for (int i = 0; i < Integer.valueOf(str); i++) {
				code += c;
			}
		}
		return code;
	}

	private EntryAp createDynamicEntryAp() {
		this.entryAp = new EntryAp();
		this.entryAp.setKey("entryap");
		this.entryAp.setShowSeq(true);
		
		List<EntryFieldAp> fieldAps = createDynamicEntryFieldAps(this.mainType);
		Map<String, EntryFieldAp> fieldApMaps = new HashMap<>();
		for (EntryFieldAp fieldAp : fieldAps) {
			fieldApMaps.put(fieldAp.getId(), fieldAp);
		}
		for (String selectedFieldKey : this.selectedField) {
			String key = selectedFieldKey.contains(".") ? YEM.substringAfterLast(selectedFieldKey, ".") : selectedFieldKey;
			if(fieldApMaps.containsKey(key)) {
				this.entryAp.getItems().add(fieldApMaps.get(key));
			}
		}
		
		return this.entryAp;
	}

	@SuppressWarnings("rawtypes")
	private List<EntryFieldAp> createDynamicEntryFieldAps(EntityType entityType) {
		List<EntryFieldAp> buildFields = new ArrayList<>(entityType.getProperties().size());
		ArrayList<Object> subEntityTypes = new ArrayList<>();
		Iterator iterator = entityType.getProperties().iterator();
		while (iterator.hasNext()) {
			IDataEntityProperty subType = (IDataEntityProperty) iterator.next();
			if (subType instanceof EntryProp) {
				DynamicObjectType subType1 = ((EntryProp) subType).getDynamicCollectionItemPropertyType();
				if (subType1 instanceof EntityType && this.selectedEntity.contains(subType1.getName())) {
					subEntityTypes.add(subType1);
				}
			} else {
				if(entityType instanceof BillEntityType) {
					String entityName = entityType.getName();
					String key = subType.getName();
					if(this.selectedField.contains(key)) {
						buildFields.add(this.createDynamicEntryFieldAp(entityName, subType, this.selectedFieldName.get(key)));
					}
				}else if(entityType instanceof EntryType){
					String entityName = entityType.getName();
					String key = entityName + "." + subType.getName();
					if(this.selectedField.contains(key)) {
						buildFields.add(this.createDynamicEntryFieldAp(entityName, subType, this.selectedFieldName.get(key)));
					}
				}
			}
		}
		iterator = subEntityTypes.iterator();
		while (iterator.hasNext()) {
			EntityType subType2 = (EntityType) iterator.next();
			buildFields.addAll(this.createDynamicEntryFieldAps(subType2));
		}
		return buildFields;
	}
	
	private EntryFieldAp createDynamicEntryFieldAp(String entityName, IDataEntityProperty prop, String propName) {
		this.selectedProp.add(prop);
		if(this.entityProp.containsKey(entityName)) {
			this.entityProp.get(entityName).add(prop);
		}else {
			List<IDataEntityProperty> props = new ArrayList<>();
			props.add(prop);
			this.entityProp.put(entityName, props);
		}
		
		String propKey = prop.getName();
		EntryFieldAp fieldAp = new EntryFieldAp();
		fieldAp.setId(propKey);
		fieldAp.setKey(propKey);
		fieldAp.setName(new LocaleString(propName));
		
		if(prop instanceof TextProp) {
			if(prop instanceof MuliLangTextProp) {
				fieldAp.setField(this.buildMuliLangTextField(prop));//多语言文本
			}else {
				fieldAp.setField(this.buildTextField(prop));//文本
				if("billno".equals(propKey) || "number".equals(propKey)) {
					fieldAp.setHyperlink(true);//单据编号
				}
			}
		}else if(prop instanceof DecimalProp) {
			if(prop instanceof PriceProp) {
				fieldAp.setField(this.buildDecimalField(prop));//单价
			}else if(prop instanceof AmountProp) {
				fieldAp.setField(this.buildDecimalField(prop));//金额
				fieldAp.setSummary(1);
			}else if(prop instanceof IntegerProp) {
				fieldAp.setField(this.buildIntegerField(prop));//整数
			}else {
				fieldAp.setField(this.buildDecimalField(prop));//小数
			}
		}else if(prop instanceof DateTimeProp) {
			if(prop instanceof DateProp) {
				fieldAp.setField(this.buildDateField(prop));//日期
			}else {
				fieldAp.setField(this.buildDateTimeField(prop));//长日期
			}
		}else if(prop instanceof BooleanProp) {
			fieldAp.setField(this.buildCheckBoxField(prop));//复选框
		}else if(prop instanceof ComboProp) {
			fieldAp.setField(this.buildComboField(prop));//下拉列表
			fieldAp.setComboShowStyle(2);
		}else if(prop instanceof PictureProp) {
			fieldAp.setField(this.buildPictureField(prop));//图片
		}else if(prop instanceof BasedataProp) {
			fieldAp.setField(this.buildBasedataField(prop));//基础资料
		}else {
			fieldAp.setField(this.buildTextField(prop));//文本
		}
		
		return fieldAp;
	}

	private Field<?> buildTextField(IDataEntityProperty prop) {
		TextField field = new TextField();
		field.setId(prop.getName());
		field.setKey(prop.getName());
		
		return field;
	}
	
	private Field<?> buildMuliLangTextField(IDataEntityProperty prop) {
		MuliLangTextField field = new MuliLangTextField();
		field.setId(prop.getName());
		field.setKey(prop.getName());
		
		return field;
	}
	
	private Field<?> buildIntegerField(IDataEntityProperty prop) {
		IntegerField field = new IntegerField();
		field.setId(prop.getName());
		field.setKey(prop.getName());
		
		return field;
	}
	
	private Field<?> buildDecimalField(IDataEntityProperty prop) {
		DecimalField field = new DecimalField();
		field.setId(prop.getName());
		field.setKey(prop.getName());
		field.setScale(4);
		
		return field;
	}
	
	private Field<?> buildDateTimeField(IDataEntityProperty prop) {
		DateTimeProp dateTimeProp = (DateTimeProp) prop;
		DateTimeField field = new DateTimeField();
		field.setId(prop.getName());
		field.setKey(prop.getName());
		field.setRegionType(dateTimeProp.getRegionType());
		
		return field;
	}
	
	private Field<?> buildDateField(IDataEntityProperty prop) {
		DateField field = new DateField();
		field.setId(prop.getName());
		field.setKey(prop.getName());
		
		return field;
	}
	
	private Field<?> buildCheckBoxField(IDataEntityProperty prop) {
		CheckBoxField field = new CheckBoxField();
		field.setId(prop.getName());
		field.setKey(prop.getName());
		
		return field;
	}
	
	private Field<?> buildComboField(IDataEntityProperty prop) {
		ComboField field = new ComboField();
		field.setId(prop.getName());
		field.setKey(prop.getName());
		List<ComboItem> items = new ArrayList<>();
		for (ValueMapItem valueMapItem : ((ComboProp) prop).getComboItems()) {
			ComboItem comboItem = new ComboItem();
			comboItem.setCaption(valueMapItem.getName());
			comboItem.setValue(valueMapItem.getValue());
			comboItem.setImageKey(valueMapItem.getImageKey());
			items.add(comboItem);
		}
		field.setItems(items);
		
		return field;
	}
	
	private Field<?> buildPictureField(IDataEntityProperty prop) {
		PictureField field = new PictureField();
		field.setId(prop.getName());
		field.setKey(prop.getName());
		
		return field;
	}
	
	private Field<?> buildBasedataField(IDataEntityProperty prop) {
		BasedataField field = new BasedataField();
		field.setId(prop.getName());
		field.setKey(prop.getName());
		field.setViewDetail(false);
		
		return field;
	}
	
}
