package com.itopview.wm.sys.utils;

import com.itopview.wm.utils.YEM;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.Label;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.util.StringUtils;
import kd.fi.ap.enums.BillStatusEnum;

/**
 * 抬头信息加载
 * @description:
 * @author longh
 * @date 2022年6月20日 上午9:58:58
 */
public class BothCardHelper {
	private static Log log = LogFactory.getLog(BothCardHelper.class);
	private static final String ADD_CO = "tpv_flex_addco";
	private static final String INFO_CO = "tpv_flex_coinfo";
	private static final String ADD_GYS = "tpv_flex_addgys";
	private static final String INFO_GYS = "tpv_flex_gysinfo1";
	private static final String ADD_KH = "tpv_flex_addbuyer";
	private static final String INFO_KH = "tpv_flex_buyerinfo1";

	public static void showChangePage(AbstractBillPlugIn form, String formId) {
		IFormView view = form.getView();
		IDataModel model = view.getModel();
		if (model.getValue("billstatus").equals(BillStatusEnum.SAVE.getValue())) {
			FormShowParameter parameter = new FormShowParameter();
			parameter.getOpenStyle().setShowType(ShowType.Modal);
			parameter.setFormId(formId);
			parameter.setStatus(OperationStatus.ADDNEW);
			parameter.setCloseCallBack(new CloseCallBack(form, formId));
			view.showForm(parameter);
		}
	}
	
	/**
	 * 公司抬头信息实体赋值
	 * @param model
	 * @param org
	 */
	public static void fillCoInfo(IDataModel model, DynamicObject org) {
		try {
			model.setValue("tpv_company", org.getPkValue());
			if (StringUtils.isEmpty(org.getLocaleString("ffirmname").getLocaleValue())) {
				model.setValue("tpv_companyname", org.getLocaleString("name").getLocaleValue());//公司名称--名称
			} else {
				model.setValue("tpv_companyname", org.getLocaleString("ffirmname").getLocaleValue());//公司名称--公司名称
			}
			model.setValue("tpv_companytin", org.get("ftaxregnum"));//纳税人识别号(公司)--纳税识别号
			model.setValue("tpv_companybank", org.getLocaleString("depositbank").getLocaleValue());//开户行(公司)--开户行
			model.setValue("tpv_companyacct", org.get("bankaccount"));//账号(公司)--银行账户
			model.setValue("tpv_companylinkman", org.getLocaleString("frepresentative").getLocaleValue());//法人(公司)--法定代表人
			model.setValue("tpv_companytel", org.get("phone"));//电话(公司)--联系电话
			model.setValue("tpv_companyaddr", org.getLocaleString("faddress").getLocaleValue());//地址(公司)--住所
		} catch (NullPointerException e) {
			e.printStackTrace();
			log.error(e);
		}
	}
	
	/**
	 * 公司抬头信息标签赋值
	 * @param form
	 */
	public static void setCoLabel(AbstractBillPlugIn form) {
		try {
			IDataModel model = form.getView().getModel();
			if (YEM.isEmpty(model.getValue("tpv_companyname"))) {
				form.getView().setVisible(Boolean.TRUE, new String[]{ADD_CO});
				form.getView().setVisible(Boolean.FALSE, new String[]{INFO_CO});
			} else {
				((Label) form.getControl("tpv_conamelabel")).setText((String) model.getValue("tpv_companyname"));//公司名称
				((Label) form.getControl("tpv_cotin")).setText((String) model.getValue("tpv_companytin"));//纳税人识别号(公司)
				((Label) form.getControl("tpv_cobank")).setText((String) model.getValue("tpv_companybank"));//开户行(公司)
				((Label) form.getControl("tpv_coacct")).setText((String) model.getValue("tpv_companyacct"));//账号(公司)
				((Label) form.getControl("tpv_colinkman")).setText((String) model.getValue("tpv_companylinkman"));//联系人(公司)
				((Label) form.getControl("tpv_cotel")).setText((String) model.getValue("tpv_companytel"));//电话(公司)
				((Label) form.getControl("tpv_coaddr")).setText((String) model.getValue("tpv_companyaddr"));//地址(公司)
				((Label) form.getControl("tpv_coemail")).setText((String) model.getValue("tpv_companyemail"));//E-Mail(公司)
				form.getView().setVisible(Boolean.FALSE, new String[]{ADD_CO});
				form.getView().setVisible(Boolean.TRUE, new String[]{INFO_CO});
			}
		} catch (NullPointerException e) {
			e.printStackTrace();
			log.error(e);
		}
	}
	
	/**
	 * 供应商抬头信息标签赋值
	 * @param form
	 */
	public static void setSupplierLabel(AbstractBillPlugIn form) {
		try {
			IDataModel model = form.getView().getModel();
			if (YEM.isEmpty(model.getValue("tpv_suppliername"))) {
				form.getView().setVisible(Boolean.TRUE, new String[]{ADD_GYS});
				form.getView().setVisible(Boolean.FALSE, new String[]{INFO_GYS});
			} else {
				((Label) form.getControl("tpv_gysname")).setText((String) model.getValue("tpv_suppliername"));//公司名称
				((Label) form.getControl("tpv_gystin")).setText((String) model.getValue("tpv_suppliertin"));//纳税人识别号(供应商)
				((Label) form.getControl("tpv_gysbank")).setText((String) model.getValue("tpv_supplierbank"));//开户行(供应商)
				((Label) form.getControl("tpv_gysacct")).setText((String) model.getValue("tpv_supplieracct"));//账号(供应商)
				((Label) form.getControl("tpv_gyslinkman")).setText((String) model.getValue("tpv_supplierlinkman"));//联系人(供应商)
				((Label) form.getControl("tpv_gystel")).setText((String) model.getValue("tpv_suppliertel"));//电话(供应商)
				((Label) form.getControl("tpv_gysaddr")).setText((String) model.getValue("tpv_supplieraddr"));//地址(供应商)
				((Label) form.getControl("tpv_gysemail")).setText((String) model.getValue("tpv_supplieremail"));//E-Mail(供应商)
				form.getView().setVisible(Boolean.FALSE, new String[]{ADD_GYS});
				form.getView().setVisible(Boolean.TRUE, new String[]{INFO_GYS});
			}
		} catch (NullPointerException e) {
			e.printStackTrace();
			log.error(e);
		}
	}
	
	/**
	 * 客户抬头信息标签赋值
	 * @param form
	 */
	public static void setCustomerLabel(AbstractBillPlugIn form) {
		try {
			IDataModel model = form.getView().getModel();
			if (YEM.isEmpty(model.getValue("tpv_customername"))) {
				form.getView().setVisible(Boolean.TRUE, new String[]{ADD_KH});
				form.getView().setVisible(Boolean.FALSE, new String[]{INFO_KH});
			} else {
				((Label) form.getControl("tpv_payername")).setText((String) model.getValue("tpv_customername"));//公司名称
				((Label) form.getControl("tpv_payertin")).setText((String) model.getValue("tpv_customertin"));//纳税人识别号(客户)
				((Label) form.getControl("tpv_payerbank")).setText((String) model.getValue("tpv_customerbank"));//开户行(客户)
				((Label) form.getControl("tpv_payeracct")).setText((String) model.getValue("tpv_customeracct"));//账号(客户)
				((Label) form.getControl("tpv_payerlinkman")).setText((String) model.getValue("tpv_customerlinkman"));//联系人(客户)
				((Label) form.getControl("tpv_payertel")).setText((String) model.getValue("tpv_customertel"));//电话(客户)
				((Label) form.getControl("tpv_payeraddr")).setText((String) model.getValue("tpv_customeraddr"));//地址(客户)
				((Label) form.getControl("tpv_payeremail")).setText((String) model.getValue("tpv_customeremail"));//E-Mail(客户)
				form.getView().setVisible(Boolean.FALSE, new String[]{ADD_KH});
				form.getView().setVisible(Boolean.TRUE, new String[]{INFO_KH});
			}
		} catch (NullPointerException e) {
			e.printStackTrace();
			log.error(e);
		}
	}
	
	public static DynamicObjectCollection getBankCollInfo(Object pk, String asstactType) {
		DynamicObject supp = BusinessDataServiceHelper.loadSingle(pk, asstactType);
		DynamicObjectCollection bankColls = supp.getDynamicObjectCollection("entry_bank");
		return bankColls != null && bankColls.size() > 0 ? bankColls : null;
	}
	
	public static FormShowParameter getSupplierBankInfoShowParameter(Object pk) {
		FormShowParameter fsp = new FormShowParameter();
		fsp.getOpenStyle().setShowType(ShowType.Modal);
		fsp.setStatus(OperationStatus.ADDNEW);
		fsp.setCustomParam("bill", "bd_supplier");
		fsp.setCustomParam("pk", pk);
		fsp.setFormId("tpv_tpl_addbankinfo");
		fsp.setCaption("供应商-银行信息");
		return fsp;
	}
	
	public static FormShowParameter getCustomerBankInfoShowParameter(Object pk) {
		FormShowParameter fsp = new FormShowParameter();
		fsp.getOpenStyle().setShowType(ShowType.Modal);
		fsp.setStatus(OperationStatus.ADDNEW);
		fsp.setCustomParam("bill", "bd_customer");
		fsp.setCustomParam("pk", pk);
		fsp.setFormId("tpv_tpl_addbankinfo");
		fsp.setCaption("客户-银行信息");
		return fsp;
	}
	
}