package com.itopview.wm.sys.utils;

import com.itopview.wm.utils.DynamicObjectUtil;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.IDataEntityType;
import kd.bos.dataentity.metadata.ISimpleProperty;
import kd.bos.dataentity.metadata.clr.DataEntityPropertyCollection;
import kd.bos.dataentity.metadata.dynamicobject.DynamicLocaleProperty;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.entity.operate.result.IOperateInfo;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.property.BillTypeProp;
import kd.bos.entity.property.EntryProp;
import kd.bos.entity.property.MulBasedataProp;
import kd.bos.exception.KDBizException;
import kd.bos.mq.MQFactory;
import kd.bos.mq.MessagePublisher;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;

import java.util.*;

/**
 * 版本变更帮助类
 * @description:
 * @author longh
 * @date 2022年12月1日 下午4:50:18
 */
public class NewVersionHelper {
	public static final String KEY_REL = "tpv_tpl_versionsrel";//【版本变更关联追踪表】单据标识-扩展开发库
	public static final String KEY_TPL = "tpv_tpl_versionsetting";//【创建新版本方案维护】单据标识
	public static final String KEY_LIST = "tpv_tpl_versionresume";//【变更履历】单据标识
	public static final Map<String, String> KEY_TPL_MAP = new HashMap<>();
	static {
		KEY_TPL_MAP.put(null, "tpv_tpl_versionsrel");
		KEY_TPL_MAP.put("secd", "tpv_tpl_versionsrel");
		KEY_TPL_MAP.put("scm", "tpv_tpl_versionsrel_scm");
		KEY_TPL_MAP.put("tmc", "tpv_tpl_versionsrel_tmc");
	}
	private boolean isChanged = false;
	private DynamicObject tplObj;
	private String requireKey;
	private String changeKey;
	
	public boolean isChanged() {
		return isChanged;
	}

	public DynamicObject getTplObj() {
		return tplObj;
	}
	
	public String getRequireKey() {
		return requireKey;
	}

	public String getChangeKey() {
		return changeKey;
	}

	public NewVersionHelper(String entityNumber){
		tplObj = queryTemplate(entityNumber);
		if(tplObj != null) {
			requireKey = tplObj.getString("tpv_requirebill.number");
			changeKey = tplObj.getString("tpv_changebill.number");
			if(entityNumber.equals(changeKey)) {
				isChanged = true;
			}
		}
	}
	
	/**
	 * 查询版本变更方案
	 * @param entityNumber
	 * @param changeBill
	 * @return
	 */
	public static DynamicObject queryTemplate(String entityNumber) {
		QFilter qFilter = new QFilter("enable", QCP.equals, "1");
//		qFilter.and(new QFilter("status", QCP.equals, "C"));
		qFilter.and(new QFilter("tpv_requirebill.number", QCP.equals, entityNumber).or(new QFilter("tpv_changebill.number", QCP.equals, entityNumber)));
		String selectFields = DynamicObjectUtil.getSelectfields(KEY_TPL, false);
		selectFields = DynamicObjectUtil.getEntrySelectfields(selectFields, KEY_TPL, "tpv_colsmap", false);
		selectFields = DynamicObjectUtil.getEntrySelectfields(selectFields, KEY_TPL, "tpv_checkmap", false);
		selectFields = DynamicObjectUtil.getEntrySelectfields(selectFields, KEY_TPL, "tpv_checkdelrow", false);
		selectFields = DynamicObjectUtil.getEntrySelectfields(selectFields, KEY_TPL, "tpv_billtypes", false);
		return BusinessDataServiceHelper.loadSingleFromCache(KEY_TPL, selectFields, new QFilter[] { qFilter });
	}
	
	/**
	 * 构建返回持久化的变更单实体
	 * @param entityNumber
	 * @param fromPkId
	 * @param save
	 * @return
	 */
	public static DynamicObject newVersion(String entityNumber, Object fromPkId) {
		return newVersion(queryTemplate(entityNumber), fromPkId);
	}
	
	/**
	 * 构建返回持久化的变更单实体
	 * @param tplObj
	 * @param fromPkId
	 * @param save
	 * @return
	 */
	public static DynamicObject newVersion(DynamicObject tplObj, Object fromPkId) {
		noSettings(tplObj);
		String fromKey = tplObj.getString("tpv_requirebill.number");
		String toKey = tplObj.getString("tpv_changebill.number");
		DynamicObject oldDataEntity = BusinessDataServiceHelper.loadSingle(fromPkId, fromKey);
		DynamicObject newDataEntity = BusinessDataServiceHelper.newDynamicObject(toKey);
		DynamicObjectUtil.copyDynamicObject(oldDataEntity, newDataEntity, 2, false, true, true);
		newDataEntity.set("billstatus", "A");
		newDataEntity.set("tpv_masterid", fromPkId);
		int vno = oldDataEntity.getInt("tpv_version") + 1;
		newDataEntity.set("tpv_version", vno);
		newDataEntity.set("tpv_effective", false);
		newDataEntity.set("billno", oldDataEntity.getString("billno").concat("_V").concat(String.valueOf(vno)));
		newDataEntity.set("tpv_changeman", UserServiceHelper.getCurrentUser("name"));
		newDataEntity.set("tpv_changedate", new Date());
		newDataEntity.set("tpv_effector", null);
		newDataEntity.set("tpv_effectdate", null);
		setBillTypes(newDataEntity, tplObj.getDynamicObjectCollection("tpv_billtypes"), true);
//		SaveServiceHelper.save(new DynamicObject[] {newDataEntity});
		if ("tpv_dt_salescontract_ext".equals(toKey) || "tpv_dt_purchase_inh".equals(toKey)) {
			newDataEntity.set("tpv_hbsptype_apt", null);
			newDataEntity.set("tpv_mainbillkey_apt", null);
			newDataEntity.set("tpv_mainbillno_apt", null);
			newDataEntity.set("tpv_additional", null);
			newDataEntity.set("tpv_join", null);
			newDataEntity.set("tpv_approvalprocess", null);
			newDataEntity.set("tpv_approvaldepts", null);
			newDataEntity.set("tpv_approvalandshare", null);
			newDataEntity.set("tpv_isuse", null);
			DynamicObjectCollection tpvApprovaltogether = newDataEntity.getDynamicObjectCollection("tpv_approvaltogether");
			tpvApprovaltogether.clear();
			DynamicObjectCollection tpvApprovalsetting = newDataEntity.getDynamicObjectCollection("tpv_approvalsetting");
			tpvApprovalsetting.clear();
		}
		OperationResult result = SaveServiceHelper.saveOperate(toKey, new DynamicObject[] {newDataEntity}, OperateOption.create());
		checkResult(result, "变更单保存失败。", newDataEntity);
		matchVersions(oldDataEntity, newDataEntity, tplObj.getPkValue());
		oldDataEntity.set("tpv_changing", true);
		SaveServiceHelper.update(oldDataEntity);
		attachmentSend(oldDataEntity, newDataEntity);
		return newDataEntity;
	}
	
	/**
	 * 变更单删除
	 * @param tplObj
	 * @param dataEntity
	 */
	public static void deleteChange(DynamicObject tplObj, DynamicObject dataEntity) {
		String entityNumber = dataEntity.getDynamicObjectType().getName();
		String tpv_requirebill = tplObj.getString("tpv_requirebill.number");
		String tpv_changebill = tplObj.getString("tpv_changebill.number");
		long masterid = dataEntity.getLong("tpv_masterid");
		if(masterid > 0 && entityNumber.equals(tpv_changebill)) {
			DynamicObject oldDataEntity = BusinessDataServiceHelper.loadSingle(masterid, tpv_requirebill);
			oldDataEntity.set("tpv_changing", false);
			SaveServiceHelper.update(oldDataEntity);
		}
	}

	/**
	 * 变更单生效
	 * @param tplObj
	 * @param dataEntity
	 */
	public static void effectChange(String entityNumber, DynamicObject dataEntity) {
		effectChange(queryTemplate(entityNumber), dataEntity);
	}
	
	/**
	 * 变更单生效
	 * @param tplObj
	 * @param dataEntity
	 */
	public static void effectChange(DynamicObject tplObj, DynamicObject dataEntity) {
		noSettings(tplObj);
		String entityNumber = dataEntity.getDynamicObjectType().getName();
		String tpv_requirebill = tplObj.getString("tpv_requirebill.number");
		String tpv_changebill = tplObj.getString("tpv_changebill.number");
		dataEntity = BusinessDataServiceHelper.loadSingle(dataEntity.getPkValue(), tpv_changebill);
		int vno = dataEntity.getInt("tpv_version");
		long masterid = dataEntity.getLong("tpv_masterid");
		if(masterid > 0 && entityNumber.equals(tpv_changebill)) {
			DynamicObject oldDataEntity = BusinessDataServiceHelper.loadSingle(masterid, tpv_requirebill);
			if(vno == 2) {//第一次变更要生成版本为一的变更单记录初版单据内容
				DynamicObject newDataEntity = BusinessDataServiceHelper.newDynamicObject(tpv_changebill);
				DynamicObjectUtil.copyDynamicObject(oldDataEntity, newDataEntity, 2, false, true, true);
				newDataEntity.set("tpv_version", 1);
				newDataEntity.set("tpv_masterid", masterid);
				newDataEntity.set("billno", oldDataEntity.getString("billno").concat("_V1"));
				newDataEntity.set("tpv_effective", true);
				setBillTypes(newDataEntity, tplObj.getDynamicObjectCollection("tpv_billtypes"), true);
				if ("tpv_dt_salescontract_ext".equals(entityNumber) || "tpv_dt_purchase_inh".equals(entityNumber)) {
					newDataEntity.set("tpv_hbsptype_apt", dataEntity.get("tpv_hbsptype_apt"));
					newDataEntity.set("tpv_mainbillkey_apt", dataEntity.get("tpv_mainbillkey_apt"));
					newDataEntity.set("tpv_mainbillno_apt", dataEntity.get("tpv_mainbillno_apt"));
					newDataEntity.set("tpv_additional", dataEntity.get("tpv_additional"));
					newDataEntity.set("tpv_join", dataEntity.get("tpv_join"));
					newDataEntity.set("tpv_approvalprocess", dataEntity.get("tpv_approvalprocess"));
					newDataEntity.set("tpv_approvaldepts", dataEntity.get("tpv_approvaldepts"));
					newDataEntity.set("tpv_approvalandshare", dataEntity.get("tpv_approvalandshare"));
					newDataEntity.set("tpv_isuse", dataEntity.get("tpv_isuse"));
					DynamicObjectCollection tpvApprovaltogether = newDataEntity.getDynamicObjectCollection("tpv_approvaltogether");
					tpvApprovaltogether.clear();
					DynamicObjectCollection approvaltogether = dataEntity.getDynamicObjectCollection("tpv_approvaltogether");
					for (DynamicObject dynamicObject : approvaltogether) {
						DynamicObjectUtil.copyDynamicObject(dynamicObject,tpvApprovaltogether.addNew());
					}
					DynamicObjectCollection tpvApprovalsetting = newDataEntity.getDynamicObjectCollection("tpv_approvalsetting");
					tpvApprovalsetting.clear();
					DynamicObjectCollection approvalsetting = dataEntity.getDynamicObjectCollection("tpv_approvalsetting");
					for (DynamicObject dynamicObject : approvalsetting) {
						DynamicObjectUtil.copyDynamicObject(dynamicObject,tpvApprovalsetting.addNew());
					}
				}
//				SaveServiceHelper.save(new DynamicObject[] {newDataEntity});
				OperationResult result1 = SaveServiceHelper.saveOperate(tpv_changebill, new DynamicObject[] {newDataEntity}, OperateOption.create());
				checkResult(result1, "一版本变更单保存失败。", newDataEntity);
				matchVersions(oldDataEntity, newDataEntity, tplObj.getPkValue());
			}
			//查询关联表数据
			String dbRouteKey = oldDataEntity.getDynamicObjectType().getDBRouteKey();
			QFilter qFilter = new QFilter("tpv_tplid", QCP.equals, tplObj.getPkValue());
			qFilter.and(new QFilter("tpv_masterid", QCP.equals, oldDataEntity.getPkValue()));
			qFilter.and(new QFilter("tpv_xmasterid", QCP.equals, dataEntity.getPkValue()));
			String selectFields = DynamicObjectUtil.getSelectfields(KEY_TPL_MAP.get(dbRouteKey), false);
			DynamicObject[] rels = BusinessDataServiceHelper.load(KEY_TPL_MAP.get(dbRouteKey), selectFields, new QFilter[] { qFilter });
			//根据关联表反写变更数据到源单
			dataEntity.set("tpv_effector", UserServiceHelper.getCurrentUser("name"));
			dataEntity.set("tpv_effectdate", new Date());
			dataEntity.set("tpv_effective", true);
			
			List<DynamicObject> relObjList = new ArrayList<>();
			copyChange2SrcByVersionRelData(oldDataEntity, dataEntity, dataEntity, oldDataEntity, rels, relObjList, tplObj.getPkValue());
			if(relObjList.size() > 0) {
				SaveServiceHelper.save(relObjList.toArray(new DynamicObject[0]));
			}
			setBillTypes(oldDataEntity, tplObj.getDynamicObjectCollection("tpv_billtypes"), false);
			attachmentSend(dataEntity, oldDataEntity);
			
//			SaveServiceHelper.save(new DynamicObject[] {dataEntity});
//			SaveServiceHelper.save(new DynamicObject[] {oldDataEntity});
			OperationResult result2 = SaveServiceHelper.saveOperate(tpv_changebill, new DynamicObject[] {dataEntity}, OperateOption.create());
			checkResult(result2, "变更单生效保存失败。", dataEntity);
			if ("tpv_dt_salescontract_ext".equals(entityNumber) || "tpv_dt_purchase_inh".equals(entityNumber)) {
				oldDataEntity.set("tpv_approvalprocess", oldDataEntity.get("tpv_approvalprocess"));
			}
			OperationResult result3 = SaveServiceHelper.saveOperate(tpv_requirebill, new DynamicObject[] {oldDataEntity}, OperateOption.create());
			checkResult(result3, "源单生效保存失败。", oldDataEntity);
		}
	}
	
	/**
	 * 同步附件面板
	 * @param fromObj
	 * @param toObj
	 */
	public static void attachmentSend(DynamicObject fromObj, DynamicObject toObj) {
		if (fromObj != null && toObj != null && fromObj.getDataEntityType().getProperties().containsKey("tpv_attachmentcon")) {
			MessagePublisher mp = MQFactory.get().createSimplePublisher("tpv_wm", "tpv_attachmentsend");
			try {
				Map<String,DynamicObject> map = new HashMap<>();
				map.put("from", fromObj);
				map.put("to", toObj);
				mp.publishInDbTranscation("secd", map);
		    } finally {
		        mp.close();
		    }
		}
	}
	
	private static void noSettings(DynamicObject tplObj) {
		if(tplObj == null) {
			throw new KDBizException("未设置版本变更方案。");
		}
	}
	
	/**
	 * 检验单据生成结果，失败则提示并抛出异常
	 * @param result
	 * @param dataEntity 
	 */
	public static void checkResult(OperationResult result, String errMsgTitle, DynamicObject dataEntity) {
		if(!result.isSuccess()) {
			StringBuffer buffer = new StringBuffer(errMsgTitle);
			buffer.append("\r\n").append("Message:").append(result.getMessage()).append("\r\n").append("Details:");
			List<IOperateInfo> allErrorOrValidateInfo = result.getAllErrorOrValidateInfo();
			for (IOperateInfo iOperateInfo : allErrorOrValidateInfo) {
				buffer.append("\r\n");
				String message = iOperateInfo.getMessage();
				if(message.contains("正在PC端编辑该记录") || message.contains("当前单据已在其他页签中打开")) {
					SaveServiceHelper.save(new DynamicObject[] {dataEntity});
					return;
				}
				buffer.append(message);
			}
			throw new KDBizException(buffer.toString());
		}
	}
	
	/**
	 * 变更单单据类型映射
	 * @param newDataEntity
	 * @param tpv_billtypes
	 */
	public static void setBillTypes(DynamicObject newDataEntity, DynamicObjectCollection tpv_billtypes, boolean change) {
		String fromBilltype = change ? "tpv_srcbilltype" : "tpv_changebilltype";
		String toBilltype = change ? "tpv_changebilltype" : "tpv_srcbilltype";	
		DataEntityPropertyCollection properties = newDataEntity.getDynamicObjectType().getProperties();
		for(IDataEntityProperty property : properties) {
			String name = property.getName();
			//分录处理
			if(property instanceof EntryProp) {
				DynamicObjectCollection dyObjCollection = newDataEntity.getDynamicObjectCollection(name);
				for(DynamicObject row : dyObjCollection) {
					setBillTypes(row, tpv_billtypes, change);
				}
			}//字段处理
			else if(property instanceof BillTypeProp){
				DynamicObject newBillType = null;
				String billTypeId = newDataEntity.getDynamicObject(name) == null ? "" : newDataEntity.getDynamicObject(name).getPkValue().toString();
				for (DynamicObject row : tpv_billtypes) {
					DynamicObject tpv_srcbilltype = row.getDynamicObject(fromBilltype);
					if(tpv_srcbilltype != null && tpv_srcbilltype.getString("id").equals(billTypeId)) {
						newBillType = row.getDynamicObject(toBilltype);
					}
				}
				newDataEntity.set(name, newBillType);
			}
		}
	}
	
	/**
	 * 构建原始单据和变更单据的关联关系
	 * @param from
	 * @param to
	 * @param tplPkId
	 */
	private static void matchVersions(DynamicObject from, DynamicObject to, Object tplPkId) {
		List<DynamicObject> relObjList = new ArrayList<>();
		DynamicObjectType toType = to.getDynamicObjectType();
		DataEntityPropertyCollection toProperties = toType.getProperties();
		DynamicObjectType fromType = from.getDynamicObjectType();
		DataEntityPropertyCollection fromProperties = fromType.getProperties();
		//表头处理
		DynamicObject relObj = newVersionRelData(from, to, tplPkId, fromType.getName(), from.getPkValue(), to.getPkValue());
		relObjList.add(relObj);
		for(IDataEntityProperty toProperty : toProperties) {
			//分录处理
			if(toProperty instanceof EntryProp) {
				String name = toProperty.getName();
				if(!fromProperties.containsKey(name)) {
					continue;
				}
				DynamicObjectCollection toDynamicObjectCollection = to.getDynamicObjectCollection(name);
				DynamicObjectCollection fromDynamicObjectCollection = from.getDynamicObjectCollection(name);
				for(DynamicObject fromObj : fromDynamicObjectCollection) {
					if(fromObj.getPkValue() == null) {
						continue;
					}
					Object fromEntryPk = fromObj.getPkValue();//原始主键
					String fromEntrySeq = fromObj.getString("seq");//原始序号
					for(DynamicObject toObj : toDynamicObjectCollection) {
						if(toObj.getPkValue() == null) {
							continue;
						}
						Object toEntryPk = toObj.getPkValue();//变更主键
						String toEntrySeq = toObj.getString("seq");//原始序号
						if(fromEntrySeq.equals(toEntrySeq)) {
							DynamicObject relEntryObj = newVersionRelData(from, to, tplPkId, fromDynamicObjectCollection.getDynamicObjectType().getName(),
									fromEntryPk, toEntryPk);
							relObjList.add(relEntryObj);
							break;
						}
					}
				}
			}
		}
		SaveServiceHelper.save(relObjList.toArray(new DynamicObject[0]));
	}
	
	/**
	 * 构建【版本变更关联追踪表】数据
	 * @param from
	 * @param to
	 * @param tplPkId
	 * @param entitykey
	 * @param fromEntryPk
	 * @param toEntryPk
	 * @return
	 */
	private static DynamicObject newVersionRelData(DynamicObject from, DynamicObject to, Object tplPkId, String entitykey, Object fromEntryPk, Object toEntryPk) {
		DynamicObject relObj = null;
		String dbRouteKey = from.getDynamicObjectType().getDBRouteKey();
		if(KEY_TPL_MAP.containsKey(dbRouteKey)) {
			relObj = BusinessDataServiceHelper.newDynamicObject(KEY_TPL_MAP.get(dbRouteKey));
		}else {
			throw new KDBizException("当前模块分库未配置版本关联追踪表，请联系管理员。");
		}
		relObj.set("tpv_tplid", tplPkId);//变更方案ID
		relObj.set("tpv_masterid", from.getPkValue());//主数据内码
		relObj.set("tpv_xmasterid", to.getPkValue());//变更单内码
		relObj.set("tpv_vno", to.get("tpv_version"));//版本号
		relObj.set("tpv_entitykey", entitykey);//单据(体)标识
		relObj.set("tpv_pkid", fromEntryPk);//原始主键
		relObj.set("tpv_xpkid", toEntryPk);//变更单主键
		
		return relObj;
	}
	
	/**
	 * 根据关联表反写变更数据到源单
	 * @param srcEntity
	 * @param changeEntity
	 * @param from
	 * @param to
	 * @param rels
	 * @param relObjList
	 * @param tplPkId
	 */
	private static void copyChange2SrcByVersionRelData(DynamicObject srcEntity, DynamicObject changeEntity, DynamicObject from, DynamicObject to, 
			 DynamicObject[] rels, List<DynamicObject> relObjList, Object tplPkId) {
		DynamicObjectType toType = to.getDynamicObjectType();
		DataEntityPropertyCollection toProperties = toType.getProperties();
		DynamicObjectType fromType = from.getDynamicObjectType();
		DataEntityPropertyCollection fromProperties = fromType.getProperties();
		String entityNumber = changeEntity.getDynamicObjectType().getName();
		for(IDataEntityProperty toProperty : toProperties) {
			String name = toProperty.getName();
			if(!fromProperties.containsKey(name)) {
				continue;
			}
			if ("tpv_dt_salescontract_ext".equals(entityNumber) || "tpv_dt_purchase_inh".equals(entityNumber)) {
				boolean flag = false;
				switch (name) {
					case "tpv_hbsptype_apt": case "tpv_mainbillkey_apt": case "tpv_mainbillno_apt": case "tpv_additional":
					case "tpv_join": case "tpv_approvalprocess": case "tpv_approvaldepts": case "tpv_approvalandshare":
					case "tpv_isuse": case "tpv_approvaltogether": case "tpv_billkey_apt": case "tpv_billno_apt": case "tpv_billtype_apt":
					case "tpv_supplier_apt": case "tpv_customer_apt": case "tpv_materials_apt": case "tpv_qty_apt": case "tpv_price_apt":
					case "tpv_amt_apt": case "tpv_srcbillkey_apt": case "tpv_billid_apt":
					case "tpv_approvalsetting": case "tpv_department_aps": case "tpv_queryable": case "tpv_editable": case "tpv_collaborative":
						flag = true;
						break;
					default:
						break;
				}
				if (flag) {
					continue;
				}
			}

			//分录处理
			if(toProperty instanceof EntryProp) {
				List<String> fromEntryPks = new ArrayList<>();
				List<String> toEntryPks = new ArrayList<>();
				DynamicObjectCollection toDynamicObjectCollection = to.getDynamicObjectCollection(name);
				DynamicObjectCollection fromDynamicObjectCollection = from.getDynamicObjectCollection(name);
				for (DynamicObject fromObj : fromDynamicObjectCollection) {
					String fromEntryPk = fromObj.getPkValue() == null ? "" : fromObj.getPkValue().toString();//变更分录ID
					DynamicObject dynamicObject = null;
					boolean isadd = false;//是追加的行
					if(fromEntryPk.length() > 0) {
						fromEntryPks.add(fromEntryPk);
						A:for (DynamicObject toObj : toDynamicObjectCollection) {
							String toEntryPk = toObj.getPkValue() == null ? "" : toObj.getPkValue().toString();//原始分录ID
							if(toEntryPk.length() > 0) {
								//关联处理
								for (DynamicObject rel : rels) {
									String tpv_entitykey = rel.getString("tpv_entitykey");//单据体标识
									String tpv_pkid = rel.getString("tpv_pkid");//原始主键
									String tpv_xpkid = rel.getString("tpv_xpkid");//变更单主键
									if(name.equals(tpv_entitykey) && toEntryPk.equals(tpv_pkid) && fromEntryPk.equals(tpv_xpkid)) {
										dynamicObject = toObj;
										break A;
									}
								}
							}
						}
					}
					DynamicObjectType toObjType = toDynamicObjectCollection.getDynamicObjectType();
					if(dynamicObject == null) {
						dynamicObject = new DynamicObject(toObjType);
						isadd = true;
					}
					
					copyChange2SrcByVersionRelData(srcEntity, changeEntity, fromObj, dynamicObject, rels, relObjList, tplPkId);
					if(isadd) {
						long genLongId = ORM.create().genLongId(toObjType);
						toEntryPks.add(String.valueOf(genLongId));
						dynamicObject.set("id", genLongId);
						toDynamicObjectCollection.add(dynamicObject);
						DynamicObject relObj = newVersionRelData(srcEntity, changeEntity, tplPkId, toObjType.getName(), genLongId, fromObj.getPkValue());
						relObjList.add(relObj);
					}
				}
				//去掉删除的行
				for (DynamicObject rel : rels) {
					String tpv_entitykey = rel.getString("tpv_entitykey");//单据体标识
					String tpv_pkid = rel.getString("tpv_pkid");//原始主键
					String tpv_xpkid = rel.getString("tpv_xpkid");//变更单主键
					if(name.equals(tpv_entitykey) && fromEntryPks.contains(tpv_xpkid)) {
						toEntryPks.add(tpv_pkid);
					}
				}
				for (int i = 0; i < toDynamicObjectCollection.size(); i++) {
					DynamicObject toObj = toDynamicObjectCollection.get(i);
					String toEntryPk = toObj.getPkValue() == null ? "0" : toObj.getPkValue().toString();
					if(!"0".equals(toEntryPk) && !toEntryPks.contains(toEntryPk)) {
						toDynamicObjectCollection.remove(i--);
					}
				}
			}//多选基础资料类处理
			else if(toProperty instanceof MulBasedataProp) {
				MulBasedataProp mulBasedataProp = ((MulBasedataProp) toProperty);
				
				IDataEntityType itemType = mulBasedataProp.getItemType();
				ISimpleProperty primaryKey = itemType.getPrimaryKey();
				DynamicObjectCollection toColls = (DynamicObjectCollection)to.get(name);
				toColls.clear();
				DynamicObjectType toDynamicObjectType = toColls.getDynamicObjectType();
				
				DynamicObjectCollection fromColls = (DynamicObjectCollection)from.get(name);
				for (DynamicObject coll : fromColls) {
					DynamicObject mulBaseData = new DynamicObject(toDynamicObjectType);
					copyChange2SrcByVersionRelData(srcEntity, changeEntity, coll, mulBaseData, rels, relObjList, tplPkId);
					primaryKey.resetValue(mulBaseData);
					toColls.add(mulBaseData);
				}
			}//多语言表处理
			else if(toProperty instanceof DynamicLocaleProperty) {
				DynamicLocaleProperty localeProperty = ((DynamicLocaleProperty) toProperty);
				
				IDataEntityType itemType = localeProperty.getItemType();
				ISimpleProperty primaryKey = itemType.getPrimaryKey();	
				DynamicObjectCollection toColls = (DynamicObjectCollection)to.get(name);
				toColls.clear();
				DynamicObjectType toDynamicObjectType = toColls.getDynamicObjectType();
				
				DynamicObjectCollection fromColls = (DynamicObjectCollection)from.get(name);
				for (DynamicObject coll : fromColls) {
					DynamicObject mulBaseData = new DynamicObject(toDynamicObjectType);
					copyChange2SrcByVersionRelData(srcEntity, changeEntity, coll, mulBaseData, rels, relObjList, tplPkId);
					primaryKey.resetValue(mulBaseData);
					toColls.add(mulBaseData);
				}
			}//字段处理
			else {
				if(from.get(name) == null) {
					continue;
				}
				if("id".equals(name) || "billno".equals(name)) {
					continue;
				}
				to.set(name, from.get(name));
			}
		}
	}

}
