package com.itopview.wm.sys.utils;

import com.alibaba.excel.util.ObjectUtils;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.sys.api.yargonpdfapi.model.ImagePdfModel;
import com.itopview.wm.sys.api.yargonpdfapi.model.PdfFileModel;
import com.itopview.wm.utils.DynamicObjectUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.OperationStatus;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.serialization.SerializationUtils;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.filter.FilterBuilder;
import kd.bos.entity.filter.FilterCondition;
import kd.bos.fileservice.FileItem;
import kd.bos.fileservice.FileService;
import kd.bos.fileservice.FileServiceFactory;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.Label;
import kd.bos.form.control.ProgressBar;
import kd.bos.form.plugin.print.ComplexSettingItem;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.print.api.PrintTask;
import kd.bos.print.api.PrintWork;
import kd.bos.print.core.service.PrtAttach;
import kd.bos.print.core.service.PrtAttach.AttachDetail;
import kd.bos.print.service.BosPrintServiceHelper;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.servicehelper.user.UserConfigServiceHelper;
import kd.bos.url.UrlService;
import tpv.tpv_wm.tpv_iscb.java.api.fadada.ContractSignUtil;
import tpv.tpv_wm.tpv_iscb.java.api.fadada.entitys.ContractSignTask;
import tpv.tpv_wm.tpv_iscb.java.api.fadada.entitys.locate;
import tpv.tpv_wm.tpv_sys.java.formplugin.dynamic.DetailsTestEdit;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static com.itopview.wm.sys.api.yargonpdfapi.PdfFileController.imagepdf;
import static com.itopview.wm.sys.api.yargonpdfapi.PdfFileController.pdfFile;
import static com.itopview.wm.utils.AttachmentUtil.uploadAttachmentPanel;
import static com.itopview.wm.utils.FileUtil.base64FileSize;

/**
 * 二开打印
 *
 * @author longh
 * @description
 * @date 2023年10月16日
 */
public class PrintHelper {

    public static void print(IFormView view) throws IOException {
        IDataModel model = view.getModel();
        DynamicObject dataEntity = model.getDataEntity();
        Object pkId = dataEntity.getPkValue();//获取当前单据id
        String billon = dataEntity.getString("billno");//单据编号
        String creator = dataEntity.getString("creator.name");//创建人
        String department = dataEntity.getString("tpv_department.name");//部门
        String org = dataEntity.getString("org.name");//公司
        int fileseq = dataEntity.getInt("tpv_signaturesnum");//签署次数
        if (YEM.isEmpty(pkId)) {
            view.showTipNotification("请先保存单据。");
            return;
        }
        String entityId = view.getEntityId();//获取当前单据标识
        Long id = ORM.create().genLongId("tpv_iscb_asynchronous");//底表id
        FormShowParameter formShowParameter = view.getFormShowParameter();
        String formId = formShowParameter.getFormId();//获取到扩展单标识
        // {"printlang":"zh_CN","templateid":"36CTUSI7KXWD","hSavePage":false,"vSavePage":false,"pageNumType":"1","pageSeparation":false}
        String tplId = "";
        String printLang = "";
        Object complexSettingJson = null;
        Boolean filter = false;
        String setting = UserConfigServiceHelper.getSetting(YEM.getCurrentUserId(), formId + "_printsetting");//采购合同打印设置配置信息
        if (setting != null) {
            JSONObject fromJsonString = SerializationUtils.fromJsonString(setting, JSONObject.class);
            if (fromJsonString != null) {
                tplId = fromJsonString.getString("templateid");//打印模板
                printLang = fromJsonString.getString("printlang");//语言
                complexSettingJson = fromJsonString.get("complexsetting");//过滤条件

            }
        }

        if (YEM.isEmpty(tplId)) {
            view.showTipNotification("没有设置打印模板，请在“打印”->“设置”中进行设置。");
            return;
        }

        if (YEM.isNotEmpty(complexSettingJson)) {
            List<ComplexSettingItem> objs = new ArrayList();
            if (!ObjectUtils.isEmpty(complexSettingJson)) {
                if (complexSettingJson instanceof String) {
                    objs = (List<ComplexSettingItem>) SerializationUtils.fromJsonStringToList(complexSettingJson.toString(), ComplexSettingItem.class);
                } else {
                    objs = (List<ComplexSettingItem>) SerializationUtils.fromJsonStringToList(SerializationUtils.toJsonString(complexSettingJson), ComplexSettingItem.class);
                }
            }
            List<Object> pkids = new ArrayList();
            pkids.add(view.getModel().getDataEntity(true).getPkValue());
            filter = getFilter(objs, entityId, pkids, tplId);
        }

        if (filter) {
            view.showTipNotification("打印条件未匹配成功!");
        } else {
            Integer tpvPrintcount = (Integer) view.getModel().getValue("tpv_printcount");
            ArrayList<Object> pkIds = new ArrayList<>();
            pkIds.add(pkId);
            // 打印任务
            ArrayList<PrintTask> taskList = new ArrayList<>();
            PrintTask printTask = new PrintTask();
            printTask.setTplId(tplId);// 打印模板的id
            printTask.setPkIds(pkIds);
            printTask.setPrintType("billForm");// 打印对象的类型，是单据（billForm),报表（report)，还是动态表单(dynamic)
            printTask.setFormId(entityId);
            taskList.add(printTask);
            // 打印作业
            PrintWork printWork = new PrintWork();
            printWork.setPrintLang(YEM.isEmpty(printLang) ? "zh_CN" : printLang);//设置打印语言
            printWork.setExpType("pdf");//设置打印格式
            printWork.setTaskList(taskList);
            // 生成打印模板生成的pdf文件
            PrtAttach doPrint = BosPrintServiceHelper.doPrint(printWork);
            String filePath = "";
            String pdfFileName = "";
            List<AttachDetail> attachDetails = doPrint.getAttachDetail();
            for (AttachDetail attachDetail : attachDetails) {
                filePath = attachDetail.getFilePath();//txtpdfurl
                String beforeLast = YEM.substringBeforeLast(attachDetail.getFileName(), ".");
                String afterLast = YEM.substringAfterLast(attachDetail.getFileName(), ".");
                pdfFileName = beforeLast + (tpvPrintcount == 0 ? "" : "_" + tpvPrintcount) + "." + afterLast;//txtpdf的文件名
                String selectFieid2 = DynamicObjectUtil.getSelectfields("tpv_iscb_strategy", false);// 获取单据头所有字段标识
                selectFieid2 = DynamicObjectUtil.getEntrySelectfields(selectFieid2, "tpv_iscb_strategy", "tpv_iscb_strategy_in", false);
                DynamicObject[] bills = BusinessDataServiceHelper.load("tpv_iscb_strategy", selectFieid2, new QFilter[]{
                        new QFilter("tpv_printbillxs.number", QCP.equals, entityId).and("tpv_iscb_strategy_py.tpv_printemplates.printtplid.masterid", QCP.equals, tplId)
                                .and("enable", QCP.equals, "1")
                });
                if (bills.length == 0) {
                    DynamicObject creatorsname = (DynamicObject) view.getModel().getValue("tpv_operator");//操作人
                    Long creatorsid = creatorsname.getLong("id");
                    // 无处理策略
                    InputStream inputStream = FileServiceFactory.getAttachmentFileService().getInputStream(filePath);
                    ByteArrayOutputStream bytestream = new ByteArrayOutputStream();
                    byte[] buffer = new byte[1024];
                    int ch;
                    while ((ch = inputStream.read(buffer)) != -1) {
                        bytestream.write(buffer, 0, ch);
                    }
                    byte[] input = bytestream.toByteArray();
                    bytestream.close();
                    String attachmentfile = Base64.getEncoder().encodeToString(input);
                    double size = base64FileSize(attachmentfile);
                    int sizes = (int) size;
                    String attachmentKey = "attachmentpanel";
                    if (fileseq != 0) {
                        uploadAttachmentPanel(entityId, pkId, attachmentKey, pdfFileName + "(" + fileseq + ")", filePath, creatorsid, sizes);
                    } else {
                        uploadAttachmentPanel(entityId, pkId, attachmentKey, pdfFileName, filePath, creatorsid, sizes);
                    }
                    view.invokeOperation("refresh");
                    String previewUrl = UrlService.getAttachmentPreviewUrl(filePath);
                    previewUrl(view, previewUrl);
                    view.getModel().setValue("tpv_printcount",1 + tpvPrintcount);
                    view.invokeOperation("save");
                    return;
                } else {
                    FormShowParameter showParameter = new FormShowParameter();
                    showParameter.setStatus(OperationStatus.ADDNEW);
                    showParameter.getOpenStyle().setShowType(ShowType.Modal);
                    showParameter.setFormId("tpv_iscb_progress");//进度动态表单
                    Map<String, Object> map = new HashMap<>();
                    map.put("pkId", pkId);
                    map.put("ID", id);
                    map.put("filePath", filePath);
                    map.put("pdfFileName", pdfFileName);
                    map.put("entityId", entityId);
                    map.put("dataEntity", dataEntity);
                    map.put("tplId", tplId);
                    map.put("billon", billon);
                    map.put("creator", creator);
                    map.put("department", department);
                    map.put("org", org);
                    showParameter.setCustomParams(map);
                    view.showForm(showParameter);
                }
            }
        }
    }

    public static Boolean getFilter(List<ComplexSettingItem> objs, String formKey, List<Object> pkids, String tplId) {
        List<ComplexSettingItem> availableFilterSetting = objs.stream().filter((p) -> {
            return p.isEnable() && kd.bos.dataentity.utils.StringUtils.isNotBlank(p.getFiltertemplateid());
        }).collect(Collectors.toList());
        if (!availableFilterSetting.isEmpty()) {
            MainEntityType dt = EntityMetadataCache.getDataEntityType(formKey);
            for (ComplexSettingItem item : availableFilterSetting) {
                String filtertemplateid = item.getFiltertemplateid();
                if (!tplId.equals(filtertemplateid)) {
                    continue;
                }
                if (kd.bos.dataentity.utils.StringUtils.isNotBlank(item.getFiltercondition()) && item.getFiltertype() == 1) {
                    FilterCondition filterCondition = null;
                    if (item.getFiltercondition() instanceof String) {
                        filterCondition = (FilterCondition) SerializationUtils.fromJsonString(item.getFiltercondition().toString(), FilterCondition.class);
                    } else {
                        filterCondition = (FilterCondition) SerializationUtils.fromJsonString(SerializationUtils.toJsonString(item.getFiltercondition()), FilterCondition.class);
                    }
                    FilterBuilder filterBuilder = new FilterBuilder(dt, filterCondition, true);
                    filterBuilder.buildFilter(true);
                    String pkName = dt.getPrimaryKey().getName();
                    QFilter[] filters = new QFilter[]{filterBuilder.getQFilter(), new QFilter(pkName, "in", pkids)};
                    Map<Object, DynamicObject> filterObjs = BusinessDataServiceHelper.loadFromCache(formKey, filters);
                    if (filterObjs.size() == 0) {
                        return true;
                    }
                    return false;
                }
            }
        }
        return false;
    }


    /**
     * 二开打印
     *
     * @description
     * @author GX
     * 参数：提示页面的视图,单据id,底表id,附件地址，附件名称，单据标识，打印模板
     */
    public static void getAttachDateil(IFormView view, Long pkId, Long ID, String filePath, String pdfFileName, String entityId, String tplId, String billon, String creator, String department, String org) throws Exception {
        String url = "";//地址
        Boolean falg = false;
        Long creatorsid = 0L;
        int fileseq = 0;
        if (view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
            DynamicObject creatorsname = (DynamicObject) view.getParentView().getModel().getValue("tpv_operator");//操作人
            if (null != creatorsname) {
                creatorsid = creatorsname.getLong("id");
                fileseq = (int) view.getParentView().getModel().getValue("tpv_signaturesnum");//签署次数
            } else {
                view.showTipNotification("请先填写业务员！在进行操作！");
            }
        } else {
            fileseq = (int) view.getModel().getValue("tpv_signaturesnum");//签署次数
        }
        String selectfield = com.itopview.wm.utils.DynamicObjectUtil.getSelectfields("tpv_iscb_asynchronous", false);
        selectfield = DynamicObjectUtil.getEntrySelectfields(selectfield, "tpv_iscb_asynchronous", "tpv_iscb_asynchr_cl", false);
        DynamicObject loadSingle = BusinessDataServiceHelper.loadSingle(pkId, entityId);
        String appid = ((MainEntityType) loadSingle.getDataEntityType()).getAppId();//应用标识
        String base64 = "";
        int row = 0;
        MainEntityType dataType = EntityMetadataCache.getDataEntityType(entityId);
        Label richTextEditor = view.getControl("tpv_labelap");
        Date date = new Date();
        int endtime = (int) view.getModel().getValue("tpv_timeout");//倒计时
        InputStream inputStream = FileServiceFactory.getAttachmentFileService().getInputStream(filePath);
        ByteArrayOutputStream bytestream = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int ch;
        while ((ch = inputStream.read(buffer)) != -1) {
            bytestream.write(buffer, 0, ch);
        }
        byte[] input = bytestream.toByteArray();
        bytestream.close();
        String attachmentfile = Base64.getEncoder().encodeToString(input);
        // 此处应判断是否关联套打文件处理策略
        String selectFieid2 = DynamicObjectUtil.getSelectfields("tpv_iscb_strategy", false);// 获取单据头所有字段标识
        selectFieid2 = DynamicObjectUtil.getEntrySelectfields(selectFieid2, "tpv_iscb_strategy", "tpv_iscb_strategy_in", false);
        DynamicObject[] bills = BusinessDataServiceHelper.load("tpv_iscb_strategy", selectFieid2, new QFilter[]{
                new QFilter("tpv_printbillxs.number", QCP.equals, entityId).and("tpv_iscb_strategy_py.tpv_printemplates.printtplid.masterid", QCP.equals, tplId)
                        .and("enable", QCP.equals, "1")
        });//查询套打处理策略
        DynamicObject tpv_typeattachment = bills[0].getDynamicObject("tpv_typeattachment");//附件类型
        Long typeattachment = 0L;
        if (tpv_typeattachment != null) {
            typeattachment = tpv_typeattachment.getLong("id");//附件类型id
        }
        Boolean attachmentlock = bills[0].getBoolean("tpv_attachmentlock");//附件锁定
        List<QFilter> qFilters = new ArrayList<>();
        DynamicObject data = BusinessDataServiceHelper.newDynamicObject("tpv_iscb_asynchronous");
        DynamicObjectCollection entryentity = data.getDynamicObjectCollection("tpv_iscb_asynchr_cl");//获取单据体数据集
        DynamicObject[] asynchronous = BusinessDataServiceHelper.load("tpv_iscb_asynchronous", selectfield, new QFilter[]{
                new QFilter("id", QCP.equals, ID)});//查询底表
        QFilter[] qFilter = new QFilter[]{new QFilter("id", QCP.equals, pkId)};
        String selectfieldsFast = DynamicObjectUtil.getSelectfieldsFast(entityId);
        DynamicObject[] queryDj = BusinessDataServiceHelper.load(entityId, selectfieldsFast, qFilter);
        boolean b = entityId.equals("tpv_dt_purchase") || entityId.equals("tpv_dt_salescontract") || entityId.equals("tpv_cm_tsht")
                || entityId.equals("tpv_dt_domsettc") || entityId.equals("tpv_dt_domsettls") || entityId.equals("tpv_tm_kyoping")
                || entityId.equals("tpv_dt_reletrade") || entityId.equals("tpv_dt_salesshipping") && falg ||
                entityId.equals("tpv_dt_confirmation") && falg || entityId.equals("tpv_dt_freigroambill") && falg ||
                entityId.equals("tpv_wms_wlzlhe") && falg || entityId.equals("tpv_dt_transdirbill") && falg || entityId.equals("tpv_dt_transoutbill") && falg;
        if (bills.length > 0 && asynchronous.length == 0) {//底表中查询不到，调用规则同时添加底表
            data.set("id", ID);//ID
            data.set("tpv_bills", entityId);//单据标识
            data.set("tpv_billsid", pkId);//单据id
            data.set("tpv_tactics", bills[0]);//处理策略
            data.set("billstatus", "C");//单据状态
            data.set("tpv_time", date);//时间
            data.set("tpv_filepath", filePath);//附件地址
            data.set("tpv_pdffilename", pdfFileName);//附件名称
            data.set("tpv_tplid", tplId);//模板
            data.set("tpv_creators", creator);//业务员
            data.set("tpv_departments", department);//部门
            data.set("tpv_atcontent", attachmentfile);//当前内容
            data.set("tpv_lybillon", billon);//来源单据编码
            data.set("tpv_lyorg", org);//来源单据公司
            data.set("tpv_timeout", endtime);
            data.set("tpv_viewtest", "d");
            data.set("tpv_signaturesnum", fileseq);
            SaveServiceHelper.save(new DynamicObject[]{data});
            for (DynamicObject bill : bills) {
                InvocationRules(bill, dataType, qFilters, pkId, entityId, data, entryentity, row, pdfFileName, richTextEditor, view, billon, creator, department, org, ID, selectfield);
            }//循环处理规则
            QFilter qFilterx = new QFilter("id", QCP.equals, ID);
            DynamicObject[] iscb_asynchronous = BusinessDataServiceHelper.load("tpv_iscb_asynchronous", selectfield, qFilterx.toArray());
            DynamicObjectCollection entryentitys = iscb_asynchronous[0].getDynamicObjectCollection("tpv_iscb_asynchr_cl");
            attachmentfile = iscb_asynchronous[0].getString("tpv_atcontent");//当前内容
            int a = 0;
            int j = 0;
            for (int i = 0; i < entryentitys.size(); i++) {
                String tpv_state = entryentitys.get(i).getString("tpv_state");//状态
                Boolean satisfycondition = entryentitys.get(i).getBoolean("tpv_satisfycondition");//是否满足
                if ("A".equals(tpv_state)) {
                    a++;
                }
                if (satisfycondition) {
                    j++;
                }
            }
            for (DynamicObject entry : entryentitys) {
                if ("tpv_iscb_fdddispose".equals(entry.getString("tpv_rulestype"))) {
                    falg = true;
                }
            }
            if (j == a) {
                byte[] bytes = Base64.getDecoder().decode(attachmentfile);
                ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(bytes);
                try {
                    url = url(appid, entityId, pkId.toString(), pdfFileName, byteArrayInputStream);
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
                double size = base64FileSize(attachmentfile);
                int sizes = (int) size;
                //全部成功
                //上传附件模板
                String attachmentKey = "attachmentpanel";
                if (fileseq != 0) {
                    uploadAttachmentPanel(entityId, pkId, attachmentKey, pdfFileName + "(" + fileseq + ")", url, creatorsid, sizes);
                    autoExtracted(queryDj, pdfFileName + "(" + fileseq + ")", typeattachment, attachmentlock);
                } else {
                    uploadAttachmentPanel(entityId, pkId, attachmentKey, pdfFileName, url, creatorsid, sizes);
                    autoExtracted(queryDj, pdfFileName, typeattachment, attachmentlock);
                }
                String previewUrl = UrlService.getAttachmentPreviewUrl(url);
                data.set("tpv_cometoanend", true);//结束
                richTextEditor.setText("请求成功！");
                SaveServiceHelper.update(data);//保存
                SaveServiceHelper.update(loadSingle);
                if (view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
                    previewUrl(view.getParentView(), previewUrl);
                    ProgressBar progressBar = view.getControl("tpv_progressbarap");
                    if (progressBar != null) {
                        progressBar.stop();
                    }
                    if (b) {
                        int automaticnum = (int) view.getParentView().getModel().getValue("tpv_automaticnum");
                        loadSingle.set("tpv_automaticnum", automaticnum + 1);
                    }
                    loadSingle.set("tpv_signaturesnum", fileseq + 1);
                    SaveServiceHelper.update(loadSingle);
                    view.getParentView().invokeOperation("refresh");
                    view.sendFormAction(view.getParentView());
                    view.getPageCache().put("startprogress", "false");
                    view.close();
                }
                return;
            }
        }
        if (bills.length > 0 && asynchronous.length > 0) {//查到底表
            for (DynamicObject asynchr : asynchronous) {//循环底表数据
                Date tpv_time = asynchr.getDate("tpv_time");
                Long diff = date.getTime() - tpv_time.getTime();
                int ss = (int) (diff / 1000);
                if (ss > endtime && view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
                    richTextEditor.setText("请求超时，请联系管理员！");
                    ProgressBar progressBar = view.getControl("tpv_progressbarap");
                    if (progressBar != null) {
                        progressBar.stop();
                        return;
                    }
                }
                Boolean tpv_cometoanend = asynchr.getBoolean("tpv_cometoanend");//结束
                if (tpv_cometoanend && view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
                    richTextEditor.setText("请求失败，请联系管理员！");
                    view.getPageCache().put("startprogress", "false");
                    ProgressBar progressBar = view.getControl("tpv_progressbarap");
                    if (progressBar != null) {
                        progressBar.stop();
                        return;
                    }
                }
                DynamicObjectCollection iscb_asynchr_cl = asynchr.getDynamicObjectCollection("tpv_iscb_asynchr_cl");//处理进度分录
                for (DynamicObject asynchr_cl : iscb_asynchr_cl) {
                    String tpv_state = asynchr_cl.getString("tpv_state");//状态
                    Boolean tpv_satisfycondition = asynchr_cl.getBoolean("tpv_satisfycondition");//是否满足条件
                    base64 = asynchr.getString("tpv_atcontent");//当前内容
                    if (tpv_satisfycondition) {
                        if ("C".equals(tpv_state)) {//未开始
                            int seq = asynchr_cl.getInt("Seq");//行号
                            if (seq != 0) {
                                int a = 0;
                                int j = 0;
                                for (int i = 0; i < seq; i++) {
                                    String state = iscb_asynchr_cl.get(i).getString("tpv_state");//状态
                                    Boolean satisfycondition = iscb_asynchr_cl.get(i).getBoolean("tpv_satisfycondition");//是否满足
                                    if ("A".equals(state)) {
                                        a++;
                                    }
                                    if (satisfycondition) {
                                        j++;
                                    }
                                }
                                if (j - 1 == a) {//处理底表
                                    String tpv_rules = asynchr_cl.getString("tpv_rulestype");//规则
                                    Boolean tpv_imaging = asynchr_cl.getBoolean("tpv_imaging");//是否图形化
                                    if ("tpv_iscb_yydispose".equals(tpv_rules)) {//易氩
                                        //调用接口处理
                                        Long rulexid = asynchr_cl.getLong("tpv_rulex.id");//易氩
                                        String selectyyfield = com.itopview.wm.utils.DynamicObjectUtil.getSelectfields("tpv_iscb_yydispose", false);
                                        QFilter qFilterxs = new QFilter("id", QCP.equals, rulexid);
                                        DynamicObject[] tpv_yyprocrulexs = BusinessDataServiceHelper.load("tpv_iscb_yydispose", selectyyfield, qFilterxs.toArray());
                                        if (tpv_imaging) {//是否图形化
                                            CustomApiResult<Map> imagepdfs = invokeYYimg(base64, pdfFileName);//调用图形化接口
                                            if (!imagepdfs.isStatus()) {
                                                asynchr_cl.set("tpv_state", "D");//状态失败
                                                asynchr.set("tpv_cometoanend", true);//结束
                                                SaveServiceHelper.update(asynchr);//保存
                                                if (view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
                                                    richTextEditor.setText("请求失败，请联系管理员！");
                                                    view.getPageCache().put("startprogress", "false");
                                                    ProgressBar progressBar = view.getControl("tpv_progressbarap");
                                                    if (progressBar != null) {
                                                        progressBar.stop();
                                                        return;
                                                    }
                                                }
                                                break;
                                            } else {
                                                String dataobj = imagepdfs.getData().get("data").toString();
                                                Map<String, String> map = new HashMap<>();
                                                for (String pair : dataobj.split(",")) {
                                                    String[] keyValue = pair.split("pdfBase64=");
                                                    if (keyValue.length == 2) {
                                                        map.put("pdfBase64", keyValue[1].trim());
                                                    }
                                                }
                                                base64 = map.get("pdfBase64").replace("}", "");
                                                asynchr.set("tpv_atcontent", base64);//当前内容
                                                asynchr_cl.set("tpv_state", "A");//状态完成
                                                SaveServiceHelper.update(asynchr);//保存
                                            }
                                        } else {
                                            CustomApiResult<Map> yypdf = invokeYY(tpv_yyprocrulexs[0], pkId, entityId, billon, creator, department, org, base64, ID, seq, pdfFileName);//非图形化接口
                                            asynchr_cl.set("tpv_state", "B");//状态处理中
                                            SaveServiceHelper.update(asynchr);//保存
                                            if (!yypdf.isStatus()) {
                                                asynchr_cl.set("tpv_state", "D");//状态失败
                                                asynchr.set("tpv_cometoanend", true);//结束
                                                SaveServiceHelper.update(asynchr);//保存
                                                if (view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
                                                    richTextEditor.setText("请求失败，请联系管理员！");
                                                    view.getPageCache().put("startprogress", "false");
                                                    ProgressBar progressBar = view.getControl("tpv_progressbarap");
                                                    if (progressBar != null) {
                                                        progressBar.stop();
                                                        return;
                                                    }
                                                }
                                                break;
                                            }
                                        }
                                    }
                                    if ("tpv_iscb_fdddispose".equals(tpv_rules)) {//FDD
                                        Long rulefddid = asynchr_cl.getLong("tpv_rulex.id");//法大大
                                        String fddselectfield = com.itopview.wm.utils.DynamicObjectUtil.getSelectfields("tpv_iscb_fdddispose", false);
                                        QFilter qFilterfdd = new QFilter("id", QCP.equals, rulefddid);
                                        DynamicObject[] tpv_fddprocrulexs = BusinessDataServiceHelper.load("tpv_iscb_fdddispose", fddselectfield, qFilterfdd.toArray());
                                        DynamicObject tpv_seal = tpv_fddprocrulexs[0].getDynamicObject("tpv_seal");//印章
                                        // 调用接口处理
                                        byte[] bytes = Base64.getDecoder().decode(base64);
                                        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(bytes);
                                        try {
                                            url = url(appid, entityId, pkId.toString(), pdfFileName, byteArrayInputStream);
                                        } catch (IOException e) {
                                            throw new RuntimeException(e);
                                        }
                                        InputStream inputStreams = FileServiceFactory.getAttachmentFileService().getInputStream(url);
                                        CustomApiResult<Map<String, String>> fddpdf = invokeFDD(tpv_fddprocrulexs[0], tpv_seal, pkId, entityId, org, inputStreams, ID, row, pdfFileName);
                                        //FDD接口调用
                                        if (!fddpdf.isStatus()) {
                                            asynchr_cl.set("tpv_state", "D");//状态失败
                                            asynchr_cl.set("tpv_errorinfo", fddpdf.getMessage());//错误信息
                                            asynchr.set("tpv_cometoanend", true);//结束
                                            SaveServiceHelper.update(asynchr);//保存
                                            if (view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
                                                richTextEditor.setText("请求失败，请联系管理员！");
                                                view.getPageCache().put("startprogress", "false");
                                                ProgressBar progressBar = view.getControl("tpv_progressbarap");
                                                if (progressBar != null) {
                                                    progressBar.stop();
                                                }
                                            }
                                            break;
                                        } else {
                                            asynchr_cl.set("tpv_state", "B");//状态处理中
                                            asynchr_cl.set("tpv_signtaskid", fddpdf.getData());//签署任务id
                                            SaveServiceHelper.update(asynchr);//保存
                                        }
                                    }
                                }
                            } else {
                                String tpv_rules = asynchr_cl.getString("tpv_rulestype");//规则
                                Boolean tpv_imaging = asynchr_cl.getBoolean("tpv_imaging");//是否图形化
                                if ("tpv_iscb_yydispose".equals(tpv_rules)) {//易氩
                                    //调用接口处理
                                    Long rulexid = asynchr_cl.getLong("tpv_rulex.id");//易氩
                                    String selectyyfield = com.itopview.wm.utils.DynamicObjectUtil.getSelectfields("tpv_iscb_yydispose", false);
                                    QFilter qFilterxs = new QFilter("id", QCP.equals, rulexid);
                                    DynamicObject[] tpv_yyprocrulexs = BusinessDataServiceHelper.load("tpv_iscb_yydispose", selectyyfield, qFilterxs.toArray());
                                    if (tpv_imaging) {//是否图形化
                                        CustomApiResult<Map> imagepdfs = invokeYYimg(base64, pdfFileName);//调用图形化接口
                                        if (!imagepdfs.isStatus()) {
                                            asynchr_cl.set("tpv_state", "D");//状态失败
                                            asynchr.set("tpv_cometoanend", true);//结束
                                            SaveServiceHelper.update(asynchr);//保存
                                            if (view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
                                                richTextEditor.setText("请求失败，请联系管理员！");
                                                view.getPageCache().put("startprogress", "false");
                                                ProgressBar progressBar = view.getControl("tpv_progressbarap");
                                                if (progressBar != null) {
                                                    progressBar.stop();
                                                    return;
                                                }
                                            }
                                            break;
                                        } else {
                                            String dataobj = imagepdfs.getData().get("data").toString();
                                            Map<String, String> map = new HashMap<>();
                                            for (String pair : dataobj.split(",")) {
                                                String[] keyValue = pair.split("pdfBase64=");
                                                if (keyValue.length == 2) {
                                                    map.put("pdfBase64", keyValue[1].trim());
                                                }
                                            }
                                            base64 = map.get("pdfBase64").replace("}", "");
                                            asynchr.set("tpv_atcontent", base64);//当前内容
                                            asynchr_cl.set("tpv_state", "A");//状态完成
                                            SaveServiceHelper.update(asynchr);//保存
                                        }
                                    } else {
                                        CustomApiResult<Map> yypdf = invokeYY(tpv_yyprocrulexs[0], pkId, entityId, billon, creator, department, org, base64, ID, seq, pdfFileName);//非图形化接口
                                        asynchr_cl.set("tpv_state", "B");//状态处理中
                                        SaveServiceHelper.update(asynchr);//保存
                                        if (!yypdf.isStatus()) {
                                            asynchr_cl.set("tpv_state", "D");//状态失败
                                            asynchr.set("tpv_cometoanend", true);//结束
                                            SaveServiceHelper.update(asynchr);//保存
                                            if (view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
                                                richTextEditor.setText("请求失败，请联系管理员！");
                                                view.getPageCache().put("startprogress", "false");
                                                ProgressBar progressBar = view.getControl("tpv_progressbarap");
                                                if (progressBar != null) {
                                                    progressBar.stop();
                                                    return;
                                                }
                                            }
                                            break;
                                        }
                                    }
                                }
                                if ("tpv_iscb_fdddispose".equals(tpv_rules)) {//FDD
                                    Long rulefddid = asynchr_cl.getLong("tpv_rulex.id");//法大大
                                    String fddselectfield = com.itopview.wm.utils.DynamicObjectUtil.getSelectfields("tpv_iscb_fdddispose", false);
                                    QFilter qFilterfdd = new QFilter("id", QCP.equals, rulefddid);
                                    DynamicObject[] tpv_fddprocrulexs = BusinessDataServiceHelper.load("tpv_iscb_fdddispose", fddselectfield, qFilterfdd.toArray());
                                    DynamicObject tpv_seal = tpv_fddprocrulexs[0].getDynamicObject("tpv_seal");//印章
                                    // 调用接口处理
                                    byte[] bytes = Base64.getDecoder().decode(base64);
                                    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(bytes);
                                    try {
                                        url = url(appid, entityId, pkId.toString(), pdfFileName, byteArrayInputStream);
                                    } catch (IOException e) {
                                        throw new RuntimeException(e);
                                    }
                                    InputStream inputStreams = FileServiceFactory.getAttachmentFileService().getInputStream(url);
                                    CustomApiResult<Map<String, String>> fddpdf = invokeFDD(tpv_fddprocrulexs[0], tpv_seal, pkId, entityId, org, inputStreams, ID, row, pdfFileName);
                                    //FDD接口调用
                                    if (!fddpdf.isStatus()) {
                                        asynchr_cl.set("tpv_state", "D");//状态失败
                                        asynchr_cl.set("tpv_errorinfo", fddpdf.getMessage());//错误信息
                                        asynchr.set("tpv_cometoanend", true);//结束
                                        SaveServiceHelper.update(asynchr);//保存
                                        if (view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
                                            richTextEditor.setText("请求失败，请联系管理员！");
                                            view.getPageCache().put("startprogress", "false");
                                            ProgressBar progressBar = view.getControl("tpv_progressbarap");
                                            if (progressBar != null) {
                                                progressBar.stop();
                                            }
                                        }
                                        break;
                                    } else {
                                        asynchr_cl.set("tpv_state", "B");//状态处理中
                                        asynchr_cl.set("tpv_signtaskid", fddpdf.getData());//签署任务id
                                        SaveServiceHelper.update(asynchr);//保存
                                    }
                                }
                            }
                        }
                    }
                }
                QFilter qFilterx = new QFilter("id", QCP.equals, ID);
                DynamicObject[] iscb_asynchronous = BusinessDataServiceHelper.load("tpv_iscb_asynchronous", selectfield, qFilterx.toArray());
                DynamicObjectCollection entryentitys = iscb_asynchronous[0].getDynamicObjectCollection("tpv_iscb_asynchr_cl");
                int a = 0;
                int j = 0;
                for (int i = 0; i < entryentitys.size(); i++) {
                    String tpv_state = entryentitys.get(i).getString("tpv_state");//状态
                    Boolean satisfycondition = entryentitys.get(i).getBoolean("tpv_satisfycondition");//是否满足
                    if ("A".equals(tpv_state)) {
                        a++;
                    }
                    if (satisfycondition) {
                        j++;
                    }
                }
                for (DynamicObject entry : entryentitys) {
                    if ("tpv_iscb_fdddispose".equals(entry.getString("tpv_rulestype"))) {
                        falg = true;
                    }
                }
                if (j == a) {
                    byte[] bytes = Base64.getDecoder().decode(base64);
                    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(bytes);
                    try {
                        url = url(appid, entityId, pkId.toString(), pdfFileName, byteArrayInputStream);
                    } catch (IOException e) {
                        throw new RuntimeException(e);
                    }
                    double size = base64FileSize(base64);
                    int sizes = (int) size;
                    //全部成功
                    //上传附件模板
                    String attachmentKey = "attachmentpanel";
                    if (fileseq != 0) {
                        uploadAttachmentPanel(entityId, pkId, attachmentKey, pdfFileName + "(" + fileseq + ")", url, creatorsid, sizes);
                        autoExtracted(queryDj, pdfFileName + "(" + fileseq + ")", typeattachment, attachmentlock);
                    } else {
                        uploadAttachmentPanel(entityId, pkId, attachmentKey, pdfFileName, url, creatorsid, sizes);
                        autoExtracted(queryDj, pdfFileName, typeattachment, attachmentlock);
                    }
                    autoExtracted(queryDj, pdfFileName, typeattachment, attachmentlock);
                    String previewUrl = UrlService.getAttachmentPreviewUrl(url);
                    data.set("tpv_cometoanend", true);//结束
                    SaveServiceHelper.update(data);//保存
                    if (view != null && "j".equals(view.getModel().getValue("tpv_viewtest"))) {
                        richTextEditor.setText("请求成功！");
                        previewUrl(view.getParentView(), previewUrl);
                        ProgressBar progressBar = view.getControl("tpv_progressbarap");
                        if (progressBar != null) {
                            progressBar.stop();
                        }
                        if (b) {
                            int automaticnum = (int) view.getParentView().getModel().getValue("tpv_automaticnum");
                            loadSingle.set("tpv_automaticnum", automaticnum + 1);
                        }
                        loadSingle.set("tpv_signaturesnum", fileseq + 1);
                        SaveServiceHelper.update(loadSingle);
                        view.getParentView().invokeOperation("refresh");
                        view.sendFormAction(view.getParentView());
                        view.getPageCache().put("startprogress", "false");
                        view.close();
                    }
                    return;
                }
            }
        }
    }

    /**
     * 底表中查询不到，调用规则同时添加底表
     * bill:套打处理策略
     *
     * @throws Exception
     */
    private static void InvocationRules(DynamicObject bill, MainEntityType dataType, List<QFilter> qFilters, Long pkId, String entityId, DynamicObject data, DynamicObjectCollection entryentity, int row, String pdfFileName, Label richTextEditor, IFormView view, String billon, String creator, String department, String org, Long ID, String selectfield) {
        String base64 = "";
        String url = "";
        DynamicObject[] tpv_fddprocrulexs = null;
        DynamicObject loadSingle = BusinessDataServiceHelper.loadSingle(pkId, entityId);
        String appid = ((MainEntityType) loadSingle.getDataEntityType()).getAppId();//应用标识
        DynamicObjectCollection tpv_iscb_strategy_in = bill.getDynamicObjectCollection("tpv_iscb_strategy_in");//调用规则
        for (DynamicObject iscb_strategy : tpv_iscb_strategy_in) {
            Boolean query = true;
            FilterCondition filterCondition = new FilterCondition();
            String tpv_checkjson = iscb_strategy.getString("tpv_checkjson");//序列化校验条件
            if (!tpv_checkjson.isEmpty()) {
                filterCondition = SerializationUtils.fromJsonString(tpv_checkjson, FilterCondition.class);
                if (filterCondition != null) {
                    FilterBuilder fBuilder = new FilterBuilder(dataType, filterCondition);
                    fBuilder.buildFilter();
                    qFilters.addAll(fBuilder.getQFilters());
                    qFilters.add(new QFilter("id", "=", pkId));//判断当前单据是否满足条件
                }
                query = QueryServiceHelper.exists(entityId, qFilters.toArray(new QFilter[0]));
            }
            DynamicObject tpv_yyprocrulexs = iscb_strategy.getDynamicObject("tpv_yyprocrulexs");//易氩处理规则
            DynamicObject fddprocrulexs = iscb_strategy.getDynamicObject("tpv_fddprocrulexs");//FDD处理规则
            if (fddprocrulexs != null) {
                Long fddid = fddprocrulexs.getLong("id");
                String selectFieid2 = kd.scm.common.util.DynamicObjectUtil.getSelectfields("tpv_iscb_fdddispose", false);// 获取单据头所有字段标识
                selectFieid2 = DynamicObjectUtil.getEntrySelectfields(selectFieid2, "tpv_iscb_fdddispose", "tpv_iscb_fdddispose_xy", false);
                tpv_fddprocrulexs = BusinessDataServiceHelper.load("tpv_iscb_fdddispose", selectFieid2, new QFilter[]{
                        new QFilter("id", QCP.equals, fddid)});
            }
            DynamicObject tpv_seal = iscb_strategy.getDynamicObject("tpv_seal");//印章
            DynamicObject entry = new DynamicObject(entryentity.getDynamicObjectType());//获取单据体数据类型
            if (tpv_yyprocrulexs != null) {//易氩
                Boolean tpv_imaging = tpv_yyprocrulexs.getBoolean("tpv_imaging");//是否图形化
                entry.set("seq", ++row);
                entry.set("tpv_satisfycondition", query);//满足条件
                entry.set("tpv_rulestype", "tpv_iscb_yydispose");//规则
                entry.set("tpv_rulex", tpv_yyprocrulexs.getLong("id"));
                entry.set("tpv_imaging", tpv_imaging);
                entry.set("tpv_state", "C");//未开始
                entryentity.add(entry);//单据体数据
                SaveServiceHelper.save(new DynamicObject[]{data});//保存
                if (tpv_imaging) {//是否图形化
                    QFilter qFilterx = new QFilter("id", QCP.equals, ID);
                    DynamicObject[] iscb_asynchronous = BusinessDataServiceHelper.load("tpv_iscb_asynchronous", selectfield, qFilterx.toArray());
                    DynamicObjectCollection tpv_iscb_asynchr_cl = iscb_asynchronous[0].getDynamicObjectCollection("tpv_iscb_asynchr_cl");
                    int a = 0;
                    for (int i = 0; i < tpv_iscb_asynchr_cl.size(); i++) {
                        String tpv_state = tpv_iscb_asynchr_cl.get(i).getString("tpv_state");//状态
                        if ("A".equals(tpv_state)) {
                            a++;
                        }
                    }
                    if (tpv_iscb_asynchr_cl.size() - 1 == a) {
                        base64 = iscb_asynchronous[0].getString("tpv_atcontent");//当前内容
                        CustomApiResult<Map> imagepdf = new CustomApiResult<>();
                        if (query) {
                            imagepdf = invokeYYimg(base64, pdfFileName);//调用图形化接口
                        }
                        if (!imagepdf.isStatus()) {
                            entry.set("tpv_state", "D");//状态失败
                            entry.set("tpv_errorinfo", imagepdf.getMessage());//错误信息
                            data.set("tpv_cometoanend", true);//结束
                            entryentity.add(entry);//单据体数据
                            SaveServiceHelper.update(data);//保存
                            if (view != null) {
                                richTextEditor.setText("请求失败，请联系管理员！");
                                view.getPageCache().put("startprogress", "false");
                                ProgressBar progressBar = view.getControl("tpv_progressbarap");
                                if (progressBar != null) {
                                    progressBar.stop();
                                }
                            }
                            break;
                        } else {
                            String dataobj = imagepdf.getData().get("data").toString();
                            Map<String, String> map = new HashMap<>();
                            for (String pair : dataobj.split(",")) {
                                String[] keyValue = pair.split("pdfBase64=");
                                if (keyValue.length == 2) {
                                    map.put("pdfBase64", keyValue[1].trim());
                                }
                            }
                            base64 = map.get("pdfBase64").replace("}", "");
                            data.set("tpv_atcontent", base64);//当前内容
                            entry.set("tpv_state", "A");//状态完成
                            entryentity.add(entry);//单据体数据
                            SaveServiceHelper.update(data);//保存
                        }
                    }
                } else {
                    QFilter qFilterx = new QFilter("id", QCP.equals, ID);
                    DynamicObject[] iscb_asynchronous = BusinessDataServiceHelper.load("tpv_iscb_asynchronous", selectfield, qFilterx.toArray());
                    DynamicObjectCollection tpv_iscb_asynchr_cl = iscb_asynchronous[0].getDynamicObjectCollection("tpv_iscb_asynchr_cl");
                    int a = 0;
                    for (int i = 0; i < tpv_iscb_asynchr_cl.size(); i++) {
                        String tpv_state = tpv_iscb_asynchr_cl.get(i).getString("tpv_state");//状态
                        if ("A".equals(tpv_state)) {
                            a++;
                        }
                    }
                    if (tpv_iscb_asynchr_cl.size() - 1 == a) {
                        base64 = iscb_asynchronous[0].getString("tpv_atcontent");//当前内容
                        CustomApiResult<Map> yypdf = new CustomApiResult<>();
                        if (query) {
                            yypdf = invokeYY(tpv_yyprocrulexs, pkId, entityId, billon, creator, department, org, base64, ID, row, pdfFileName);//非图形化接口
                        }
                        entry.set("tpv_state", "B");//状态处理中
                        entryentity.add(entry);//单据体数据
                        SaveServiceHelper.update(data);//保存
                        if (!yypdf.isStatus()) {
                            entry.set("tpv_state", "D");//状态失败
                            entry.set("tpv_errorinfo", yypdf.getMessage());//错误信息
                            data.set("tpv_cometoanend", true);//结束
                            entryentity.add(entry);//单据体数据
                            SaveServiceHelper.update(data);//保存
                            if (view != null) {
                                richTextEditor.setText("请求失败，请联系管理员！");
                                view.getPageCache().put("startprogress", "false");
                                ProgressBar progressBar = view.getControl("tpv_progressbarap");
                                if (progressBar != null) {
                                    progressBar.stop();
                                }
                            }
                            break;
                        }
                    }
                }
            }
            if (tpv_fddprocrulexs != null) {//FDD
                entry.set("seq", ++row);
                entry.set("tpv_satisfycondition", query);//满足条件
                entry.set("tpv_rulestype", "tpv_iscb_fdddispose");//规则
                entry.set("tpv_rulex", fddprocrulexs.getLong("id"));
                entry.set("tpv_imaging", false);
                entry.set("tpv_state", "C");//未开始
                entryentity.add(entry);//单据体数据
                SaveServiceHelper.save(new DynamicObject[]{data});//保存
                QFilter qFilterx = new QFilter("id", QCP.equals, ID);
                DynamicObject[] iscb_asynchronous = BusinessDataServiceHelper.load("tpv_iscb_asynchronous", selectfield, qFilterx.toArray());
                DynamicObjectCollection tpv_iscb_asynchr_cl = iscb_asynchronous[0].getDynamicObjectCollection("tpv_iscb_asynchr_cl");
                int a = 0;
                for (int i = 0; i < tpv_iscb_asynchr_cl.size(); i++) {
                    String tpv_state = tpv_iscb_asynchr_cl.get(i).getString("tpv_state");//状态
                    if ("A".equals(tpv_state)) {
                        a++;
                    }
                }
                if (tpv_iscb_asynchr_cl.size() - 1 == a) {
                    base64 = iscb_asynchronous[0].getString("tpv_atcontent");//当前内容
                    byte[] bytes = Base64.getDecoder().decode(base64);
                    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(bytes);
                    try {
                        url = url(appid, entityId, pkId.toString(), pdfFileName, byteArrayInputStream);
                    } catch (IOException e) {
                        throw new RuntimeException(e);
                    }
                    InputStream inputStream = FileServiceFactory.getAttachmentFileService().getInputStream(url);
                    CustomApiResult<Map<String, String>> fddpdf = new CustomApiResult<>();
                    if (query) {
                        fddpdf = invokeFDD(tpv_fddprocrulexs[0], tpv_seal, pkId, entityId, org, inputStream, ID, row, pdfFileName);
                    }
                    entry.set("tpv_state", "B");//状态处理中
                    entryentity.add(entry);//单据体数据
                    SaveServiceHelper.update(data);//保存
                    //FDD接口调用
                    if (!fddpdf.isStatus()) {
                        entry.set("tpv_state", "D");//状态失败
                        entry.set("tpv_errorinfo", fddpdf.getMessage());//错误信息
                        data.set("tpv_cometoanend", true);//结束
                        entryentity.add(entry);//单据体数据
                        SaveServiceHelper.update(data);//保存
                        if (view != null) {
                            richTextEditor.setText("请求失败，请联系管理员！");
                            view.getPageCache().put("startprogress", "false");
                            ProgressBar progressBar = view.getControl("tpv_progressbarap");
                            if (progressBar != null) {
                                progressBar.stop();
                            }
                        }
                        break;
                    }
                }
            }
        }//循环处理规则分录
    }

    /**
     * 调用易氩接口（图形化）
     *
     * @return
     * @throws Exception
     */
    private static CustomApiResult<Map> invokeYYimg(String attachmentfile, String pdfFileName) {
        ImagePdfModel imagePdfModel = new ImagePdfModel();
        imagePdfModel.setPdfBase64(attachmentfile);
        imagePdfModel.setPdfFileName(pdfFileName);
        CustomApiResult<Map> imagepdf = imagepdf(imagePdfModel);
        return imagepdf;
    }


    /**
     * 调用易氩接口（非图形化）
     *
     * @return
     * @throws Exception
     */
    private static CustomApiResult<Map> invokeYY(DynamicObject tpv_yyprocrulexs, Long pkId, String entityId, String billon, String creator, String department, String org, String attachmentfile, Long ID, int row, String pdfFileName) {
        PdfFileModel inputVo = new PdfFileModel();
        String qr_page = tpv_yyprocrulexs.getString("tpv_qrscanning");//二维码页数
        String qr_location = tpv_yyprocrulexs.getString("tpv_qrcoordinatez");//二维码坐标（纵）
        String qr_locationx = tpv_yyprocrulexs.getString("tpv_qrcoordinate");//二维码坐标（横）
        String qr_size = tpv_yyprocrulexs.getString("tpv_paramssize");//二维码尺寸
        Long qrcontent = tpv_yyprocrulexs.getLong("tpv_qrcontent.id");//二维码内容
        String Resize_percent = tpv_yyprocrulexs.getString("tpv_pdfzoom");//缩放百分比
        Boolean textinversion = tpv_yyprocrulexs.getBoolean("tpv_textinversion");//是否开启横纵画布自动选择
        String Rotate = textinversion ? "1" : "0";
        String qr_text = DetailsTestEdit.bulidRichText(qrcontent, pkId, entityId);
        String Scode_page = tpv_yyprocrulexs.getString("tpv_charscanning");//字符码页面
        String Scode_size = tpv_yyprocrulexs.getString("tpv_charssize");//字符码尺寸
        String Scode_location = tpv_yyprocrulexs.getString("tpv_charcoordinatez");//字符码坐标(纵)
        String Scode_locationx = tpv_yyprocrulexs.getString("tpv_charcoordinate");//字符码坐标(横)
        inputVo.setQr_page(qr_page);// 二维码页面:0：无 1：首页 2：每页 免码用印传0，默认传2
        inputVo.setQr_location(qr_location);// 二维码坐标(像素): x1,y1
        inputVo.setQr_size(qr_size);// 二维码尺寸:75
        inputVo.setQr_text(qr_text);// 二维码内容
        inputVo.setContract_no(billon);// 附件所在单据号的外码
        inputVo.setVprepare_uid(creator);// 匹配小易用户编码显示：单据制单人ucode
        inputVo.setDepartment_id(department);// 匹配小易部门编码显示：大部门内码
        inputVo.setGrantor_uid(creator);// 匹配小易用户编码显示：点击[批准用印]的人员ucode
        inputVo.setCorpbcode(department);// 传corpbcode对应名称
        inputVo.setCode(org);// 单据ccode对应的langname
        inputVo.setRemark(department + "的附件，由" + creator + "操作。");// 备注
        inputVo.setFile_b64(attachmentfile);// 源文件doc（x）、文字pdf、xls（x）的base64
        inputVo.setFile_name(pdfFileName);// 源文件名:doc（x）、文字pdf、xls（x）
        inputVo.setResize_percent(Resize_percent);//缩放百分比
        inputVo.setRotate(Rotate);// 0：不开启自动翻转,1：开启自动翻转,默认1
        inputVo.setQr_locationx(qr_locationx);// 二维码坐标(像素): x1,y1
        inputVo.setScode_text(qr_text);// 字符码
        inputVo.setScode_page(Scode_page);// 字符码页面: 1：首页 2：每页 3：末页 默认传2
        inputVo.setScode_size(Scode_size);// 字符码尺寸:75像素
        inputVo.setScode_location(Scode_location);// 字符码坐标
        inputVo.setScode_locationx(Scode_locationx);// 第二套字符码坐标
        DynamicObject loadSingle = BusinessDataServiceHelper.loadSingle(pkId, entityId);
        inputVo.setSrcEntity(loadSingle);//来源单据实体
        inputVo.setTpv_operate("A");//打印
        inputVo.setTpv_sourcebillscode(billon);//来源单据编码
        inputVo.setAsynchronousid(ID);//打印id
        inputVo.setAsynchronousrow(row);//打印分录行号
        CustomApiResult<Map> pdffile = pdfFile(inputVo);
        return pdffile;
    }


    /**
     * 调用FDD接口
     *
     * @return
     * @throws Exception
     */
    private static CustomApiResult<Map<String, String>> invokeFDD(DynamicObject tpv_fddprocrulexs, DynamicObject tpv_seal, Long pkId, String entityId, String org, InputStream inputStream, Long ID, int row, String pdfFileName) {
        ContractSignTask inputVo = new ContractSignTask();
        int pagenum = 0;
        int xindex = 0;
        int yindex = 0;
        inputVo.setIsAcrossSign(false);//是否骑缝章
        String fddId = tpv_seal.getString("tpv_fddid");//印章
        String sealkeyword = tpv_fddprocrulexs.getString("tpv_sealkeyword");//关键字
        String sealocatemode = tpv_fddprocrulexs.getString("tpv_sealocatemode");//印章定位方式
        Boolean tpv_perforation = tpv_fddprocrulexs.getBoolean("tpv_perforation");//骑缝章
        if ("B".equals(sealocatemode)) {//指标列表
            inputVo.setIsKeyWord(false);//是否关键字定位
            inputVo.setIsCompanySign(true);//是否企业章
            List<locate> LocateCoordinates = new ArrayList<>();//签章坐标
            DynamicObjectCollection iscb_fdddispose_xy = tpv_fddprocrulexs.getDynamicObjectCollection("tpv_iscb_fdddispose_xy");//定位盖章
            for (DynamicObject dynamicObject : iscb_fdddispose_xy) {
                pagenum = dynamicObject.getInt("tpv_pagenum");//页码
                xindex = dynamicObject.getInt("tpv_xindex");//x轴
                yindex = dynamicObject.getInt("tpv_yindex");//y轴
                locate locates = new locate();
                locates.setX(xindex);//x轴
                locates.setY(yindex);//y轴
                locates.setPageNum(pagenum);//页码
                LocateCoordinates.add(locates);
            }
            inputVo.setLocateCoordinates(LocateCoordinates);
        } else {//关键字
            inputVo.setIsKeyWord(true);//是否关键字定位
            inputVo.setIsCompanySign(true);//是否企业章
            inputVo.setSealId(fddId);
            inputVo.setKeyWord(sealkeyword);//关键字
        }
        if (tpv_perforation) {
            int perforationy = tpv_fddprocrulexs.getInt("tpv_perforationy");//骑缝Y坐标
            int pagetype = Integer.parseInt(tpv_fddprocrulexs.getString("tpv_pagetype"));//骑缝页面类型
            String slotpage = tpv_fddprocrulexs.getString("tpv_slotpage");//指定骑缝页面
            inputVo.setIsAcrossSign(true);//是否骑缝章
            inputVo.setAcrossSealY(perforationy);// 骑缝章位置 纵向
            inputVo.setSealId(fddId);// 骑缝章ID
            inputVo.setPageType(pagetype);// 骑缝章类型
            inputVo.setCustomPage(slotpage);//指定骑缝页面
        }
        inputVo.setSignFileName(pdfFileName);
        DynamicObject loadSingle = BusinessDataServiceHelper.loadSingle(pkId, entityId);
        inputVo.setSrcEntity(loadSingle);
        inputVo.setCompanyName(org);
        inputVo.setIsAutoSign(true);
        inputVo.setSealstatus("E");
        inputVo.setSceneInfo("F");
        inputVo.setInputStream(inputStream);
        inputVo.setPrintid(ID);//打印id
        inputVo.setAsynchronousrow(row);//打印分录行号
        ContractSignUtil contractSignUtil = new ContractSignUtil();
        CustomApiResult<Map<String, String>> mapCustomApiResult = contractSignUtil.creatSignTask(inputVo);
        return mapCustomApiResult;
    }


    private static void previewUrl(IFormView view, String previewUrl) {
        FormShowParameter showParameter = new FormShowParameter();
        showParameter.setStatus(OperationStatus.EDIT);
        showParameter.getOpenStyle().setShowType(ShowType.MainNewTabPage);
        showParameter.setFormId("bos_printpreview");//打印预览页面
        showParameter.setCustomParam("src", previewUrl);
        view.showForm(showParameter);
    }

    /**
     * 获取url
     *
     * @param appId    应用标识
     * @param formId   单据标识
     * @param pkId     单据内码
     * @param fileName 文件名
     * @param stream   接口返回的base64转的文件流
     * @return url
     * @throws IOException
     */
    private static String url(String appId, String formId, String pkId, String fileName, InputStream stream) throws IOException {
        Date today = new Date();
        DateFormat df = new SimpleDateFormat("yyyyMM");
        String NewDateStr = df.format(today);
        String attachmentPath = "/" + RequestContext.get().getTenantId() + "/" + RequestContext.get().getAccountId()
                + "/" + NewDateStr + "/" + appId + "/" + formId + "/" + pkId + "/attachments/";
        FileService fs = FileServiceFactory.getAttachmentFileService();
        String path = attachmentPath + fileName;
        FileItem fi = new FileItem(fileName, path, stream);//"接口返回的base64转的文件流")
        fi.setCreateNewFileWhenExists(true);
        String url = fs.upload(fi);
        return url;
    }

    private static void autoExtracted(DynamicObject[] queryHW, String tpvSignfilename, Long attachmenttype, Boolean bill) {
        if (queryHW.length > 0) {
            DynamicObject query = queryHW[0];
            String tpvAttachmentconTag = query.getString("tpv_attachmentcon_tag");
            JSONObject json = JSONObject.parseObject(tpvAttachmentconTag);
            if (json != null) {
                JSONObject attJson = new JSONObject();
                attJson.put("tpv_lock", bill);
                attJson.put("tpv_sealstatus", "K");//自动签完成
                attJson.put("tpv_signaturestatus", "A");
                attJson.put("tpv_attachmenttype", attachmenttype);
                json.put(tpvSignfilename, attJson);
                query.set("tpv_attachmentcon_tag", json);
                SaveServiceHelper.update(query);
            }
        }
    }
}