package com.itopview.wm.sys.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.itopview.wm.utils.RptUtil;

import kd.bos.algo.DataSet;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.permission.api.HasPermOrgResult;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;

/**
 * 业务单元数据获取
 * @description:
 * @author longh
 * @date 2021年12月21日 下午3:26:46
 */
public class UserOrgUtil {
	
	public static DynamicObjectCollection queryOrgData() {
		List<QFilter> qFilters = new ArrayList<>();
//		qFilters.add(new QFilter("id", QCP.not_equals, "100000"));
		qFilters.add(new QFilter("enable", QCP.equals, "1"));
		qFilters.add(new QFilter("structure.isfreeze", QCP.equals, false));
		qFilters.add(new QFilter("structure.view", QCP.equals, "15"));
		
		String selectFields = "id,structure.viewparent parent, orgpattern.name tpv_orgpattern";
		String orderBy = "orgpattern.number";
		
		return QueryServiceHelper.query("bos_adminorg", selectFields, qFilters.toArray(new QFilter[0]), orderBy);
	}
	
	public static List<Long> getUserChangeAbleOrgIds() {
		long lUserId = RequestContext.get().getCurrUserId();
//		long lUserId = Long.valueOf(RequestContext.get().getUserId());
		HasPermOrgResult hasPermOrgResult = UserServiceHelper.getUserChangeAbleOrg(lUserId, true);
		return hasPermOrgResult.getHasPermOrgs();
	}
	
	public static List<Long> getAbleOrgIds() {
		return getAbleOrgIds(queryOrgData(), getUserChangeAbleOrgIds());
	}
	
	public static List<Long> getAbleOrgIds(DynamicObjectCollection queryOrgData, List<Long> userChangeAbleOrgIds) {
		List<Long> userAbleOrgIds = new ArrayList<>();
		//Map<id, parent>
		Map<Long, Long> map = new HashMap<>();
		for (DynamicObject row : queryOrgData) {
			long id = row.getLong("id");
			long parent = row.getLong("parent");
			map.put(id, parent);
		}
		for (Long userChangeAbleOrgId : userChangeAbleOrgIds) {
			if(map.containsKey(userChangeAbleOrgId)) {
				userAbleOrgIds.add(userChangeAbleOrgId);
				userAbleOrgIds.addAll(getOrgParentIds(map, map.get(userChangeAbleOrgId)));
			}
		}
		
		return userAbleOrgIds.stream().distinct().collect(Collectors.toList());
	}
	
	public static List<Long> getSortAbleOrgIds() {
		return getSortAbleOrgIds(queryOrgData(), getUserChangeAbleOrgIds());
	}
	
	public static List<Long> getSortAbleOrgIds(DynamicObjectCollection queryOrgData, List<Long> userChangeAbleOrgIds) {
		return getSortAbleOrgIds(queryOrgData, 0L, getAbleOrgIds(queryOrgData, userChangeAbleOrgIds));
	}
	
	private static List<Long> getOrgParentIds(Map<Long, Long> orgMap, Long parent) {
		List<Long> list = new ArrayList<>();
		if(orgMap.containsKey(parent) && !list.contains(parent)) {
			list.add(parent);
			list.addAll(getOrgParentIds(orgMap, orgMap.get(parent)));
		}
		
		return list;
	}
	
	private static List<Long> getSortAbleOrgIds(DynamicObjectCollection queryData, Long orgId, List<Long> userAbleOrgIds) {
		List<Long> list = new ArrayList<>();
		for (DynamicObject row : queryData) {
			long id = row.getLong("id");
			long parent = row.getLong("parent");
			if(parent == orgId && !list.contains(id) && userAbleOrgIds.contains(id)) {
				list.add(id);
				list.addAll(getSortAbleOrgIds(queryData, id, userAbleOrgIds));
			}
		}
		return list;
	}
	
	public static List<Long> getSortOrgIds() {
		return getSortOrgIds(queryOrgData(), 0L);
	}
	
	public static List<Long> getSortOrgIds(DynamicObjectCollection queryData, Long orgId) {
		List<Long> list = new ArrayList<>();
		for (DynamicObject row : queryData) {
			long id = row.getLong("id");
			long parent = row.getLong("parent");
			if(parent == orgId && !list.contains(id)) {
				list.add(id);
				list.addAll(getSortOrgIds(queryData, id));
			}
		}
		return list;
	}
	
	public static DataSet addOrgLevelName(DataSet dateSet, String leftKey, String rightKey) {
		List<QFilter> qFilters = new ArrayList<>();
//		qFilters.add(new QFilter("id", QCP.not_equals, "100000"));
		qFilters.add(new QFilter("enable", QCP.equals, "1"));
		qFilters.add(new QFilter("structure.isfreeze", QCP.equals, false));
		qFilters.add(new QFilter("structure.view", QCP.equals, "15"));
		String selectFields = "id,structure.vieworg org, structure.vieworg.name orgname, structure.level level";
		DataSet queryDataSet = ORM.create().queryDataSet(UserOrgUtil.class.getName(), "bos_adminorg", selectFields, qFilters.toArray(new QFilter[0]));
		
		List<String> dataSetAliasList = RptUtil.getDataSetAliasList(queryDataSet);
		dataSetAliasList.add("(case when level = 2 then '——' + orgname else "
				+ "(case when level = 3 then '————' + orgname else orgname end) end) " + rightKey);
		queryDataSet = queryDataSet.select(String.join(",", dataSetAliasList));
		dateSet = dateSet.leftJoin(queryDataSet)
				.on(leftKey, "org")
				.select(RptUtil.getDataSetFiledAlias(dateSet), new String[] {rightKey})
				.finish();
		
		return dateSet;
	}
	
}
