package com.itopview.wm.tm.opplugin.op;

import com.itopview.wm.sys.utils.NewVersionHelper;
import kd.bos.data.BusinessDataReader;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.metadata.IDataEntityType;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.botp.runtime.ConvertOperationResult;
import kd.bos.entity.botp.runtime.PushArgs;
import kd.bos.entity.datamodel.IRefrencedataProvider;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.operate.result.IOperateInfo;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.validate.ValidateResultCollection;
import kd.bos.exception.KDBizException;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.botp.ConvertServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Description 协平单审核后自动下推生成高低计单
 * @Author LinRui
 * Date 2023/8/23 19:53
 **/
public class KyopingToDataAutoOp extends AbstractOperationServicePlugIn {
    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        String key = e.getOperationKey();
        switch (key) {
            case "audit":
                if (this.getOperationResult().isSuccess()) {
                    DynamicObject[] dataEntity = e.getDataEntities();
                    for (DynamicObject data : dataEntity) {
                        data = BusinessDataServiceHelper.loadSingle(data.getPkValue(), data.getDataEntityType().getName());
                        this.createHighLowBillData(data);
                    }
                }
                break;
            default:
                break;
        }

    }

    /**
     * 生成高低计单
     * @param data
     */
    private void createHighLowBillData(DynamicObject data) {
        //NewVersionHelper newVersionHelper = new NewVersionHelper("tpv_es_exportpresent");

        String tpv_compendirection = data.getString("tpv_compendirection");  //补偿方向
        if ("A".equals(tpv_compendirection)) {   //A:客户补我司
            this.pushHighlowBillNewData(data,tpv_compendirection);
        } else if ("B".equals(tpv_compendirection)) {  //B:我司补客户
            this.pushHighlowBillNewData(data,tpv_compendirection);
        } else {
            this.pushHighlowBillNewData(data,null);
        }
    }

    /**
     * @Description: //调用协平单下推高低计单的转换规则
     * @param: data - [kd.bos.dataentity.entity.DynamicObject]
     * @param: tpv_compendirection - [java.lang.String]
     * @return: void
     **/
    private void pushHighlowBillNewData(DynamicObject data,String tpv_compendirection) {
        PushArgs pushArgs = new PushArgs();
        pushArgs.setSourceEntityNumber(data.getDataEntityType().getName());
        pushArgs.setTargetEntityNumber("tpv_dt_highlowj");
        pushArgs.setHasRight(true);
        pushArgs.setBuildConvReport(true);
        List<ListSelectedRow> selectedRows = new ArrayList<>();
        ListSelectedRow srcBill1 = new ListSelectedRow(data.getPkValue());
        selectedRows.add(srcBill1);
        pushArgs.setSelectedRows(selectedRows);
        ConvertOperationResult pushResult = ConvertServiceHelper.push(pushArgs);
        MainEntityType targetMainType = EntityMetadataCache.getDataEntityType("tpv_dt_highlowj");
        List<DynamicObject> targetBillObjs = pushResult.loadTargetDataObjects(new IRefrencedataProvider() {
            @Override
            public void fillReferenceData(Object[] objs, IDataEntityType dType) {
                BusinessDataReader.loadRefence(objs, dType);
            }
        }, targetMainType);
        OperationResult saveOperate = OperationServiceHelper.executeOperate("save", "tpv_dt_highlowj",
                targetBillObjs.toArray(new DynamicObject[0]), OperateOption.create());
        if (!saveOperate.isSuccess()) {
            this.getOperationResult().setSuccess(false);
            this.getOperationResult().setMessage("协平单自动生成高低计单失败!!!");
        }
    }
}
