package com.itopview.wm.utils;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.TimeZone;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.json.JSONObject;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.fi.bcm.business.integrationnew.util.Base64Encoder;


/**
 * 汇率获取工具类
 * @description:
 * @author longh
 * @date 2021年10月20日 下午5:15:17
 */
public class ExchangeRateUtil {
	private static Log logger = LogFactory.getLog(ExchangeRateUtil.class);

	/**
	 * 获取实时汇率
	 * @param fromCurrISO
	 * @param toCurrISO
	 * @return
	 */
	public static BigDecimal getExchangeRateVal(String fromCurrISO, String toCurrISO) {
		fromCurrISO = "RMB".equals(fromCurrISO) ? "CNY" : fromCurrISO;
		toCurrISO = "RMB".equals(toCurrISO) ? "CNY" : toCurrISO;
		if(fromCurrISO.equals(toCurrISO)) {
			return BigDecimal.ONE;
		}
		BigDecimal rate = BigDecimal.ZERO;
		try {
			rate = getExchangeRate(fromCurrISO, toCurrISO);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("获取实时汇率失败。", e);
		}
		
		return rate;
	}
	
	/**
	 * 调用接口换算实时汇率 
	 * @param fromCurrISO 原币ISO
	 * @param toCurrISO 换算币ISO
	 * @return
	 * @throws Exception 
	 */
	public static BigDecimal getExchangeRate(String fromCurrISO, String toCurrISO) throws Exception {
		BigDecimal rate = BigDecimal.ZERO;
		DynamicObject tpv_iscb_exchangeedit = BusinessDataServiceHelper.loadSingle("tpv_iscb_exchangeedit", "id,tpv_secretid,tpv_secretkey", new QFilter[] {});

		//云市场分配的密钥Id
		String secretId = "AKIDo9lcMATSL2lPUg6180OsT5bYwRf6242RZZTg";
		//云市场分配的密钥Key
		String secretKey = "2Gikg9kb3pah7ri4Mxoy816c7z9l3x7kIr4vIe2i";
		if (tpv_iscb_exchangeedit != null) {
			secretId = tpv_iscb_exchangeedit.getString("tpv_secretid");
			secretKey = tpv_iscb_exchangeedit.getString("tpv_secretkey");
		}else {
			throw new Exception("没有维护云市场汇率转换密钥。");
		}
		
		String source = "market";

		Calendar cd = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss 'GMT'", Locale.US);
		sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
		String datetime = sdf.format(cd.getTime());
		// 签名
		String auth = calcAuthorization(source, secretId, secretKey, datetime);
		// 请求方法
		String method = "GET";
		// 请求头
		Map<String, String> headers = new HashMap<String, String>();
		headers.put("X-Source", source);
		headers.put("X-Date", datetime);
		headers.put("Authorization", auth);

		// 查询参数
		Map<String, String> queryParams = new HashMap<String, String>();
		queryParams.put("amount", "100");
		if("RMB".equals(fromCurrISO)){
			queryParams.put("from", "CNY");
		}else{
			queryParams.put("from", fromCurrISO);
		}
		if("RMB".equals(toCurrISO)){
			queryParams.put("to", "CNY");
		}else{
			queryParams.put("to", toCurrISO);
		}
		// body参数
		Map<String, String> bodyParams = new HashMap<String, String>();

		// url参数拼接
		String url = "https://service-p90qcght-1257101137.ap-shanghai.apigateway.myqcloud.com/release/exchange/convert";
		if (!queryParams.isEmpty()) {
			url += "?" + urlencode(queryParams);
		}

		BufferedReader in = null;
		URL realUrl = new URL(url);
		HttpURLConnection conn = (HttpURLConnection) realUrl.openConnection();
		conn.setConnectTimeout(5000);
		conn.setReadTimeout(5000);
		conn.setRequestMethod(method);

		// request headers
		for (Map.Entry<String, String> entry : headers.entrySet()) {
			conn.setRequestProperty(entry.getKey(), entry.getValue());
		}

		// request body
		Map<String, Boolean> methods = new HashMap<String, Boolean>();
		methods.put("POST", true);
		methods.put("PUT", true);
		methods.put("PATCH", true);
		Boolean hasBody = methods.get(method);
		if (hasBody != null) {
			conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");

			conn.setDoOutput(true);
			DataOutputStream out = new DataOutputStream(conn.getOutputStream());
			out.writeBytes(urlencode(bodyParams));
			out.flush();
			out.close();
		}

		// 定义 BufferedReader输入流来读取URL的响应
		in = new BufferedReader(new InputStreamReader(conn.getInputStream(),"UTF-8"));
		String line;
		String result = "";
		while ((line = in.readLine()) != null) {
			result += line;
		}
		JSONObject json = new JSONObject(result);
		rate = BigDecimalUtils.toBigDecimal(json.getJSONObject("result").get("rate"));
		if (in != null) {
			in.close();
		}
		return rate;
	}
	
	public static String calcAuthorization(String source, String secretId, String secretKey, String datetime)
    		throws NoSuchAlgorithmException, UnsupportedEncodingException, InvalidKeyException {
		String signStr = "x-date: " + datetime + "\n" + "x-source: " + source;
		Mac mac = Mac.getInstance("HmacSHA1");
		Key sKey = new SecretKeySpec(secretKey.getBytes("UTF-8"), mac.getAlgorithm());
		mac.init(sKey);
		byte[] hash = mac.doFinal(signStr.getBytes("UTF-8"));
		String sig = Base64Encoder.byteArrayToBase64(hash);
		
		String auth = "hmac id=\"" + secretId + "\", algorithm=\"hmac-sha1\", headers=\"x-date x-source\", signature=\"" + sig + "\"";
		return auth;
	}
	
	public static String urlencode(Map<?, ?> map) throws UnsupportedEncodingException {
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<?, ?> entry : map.entrySet()) {
            if (sb.length() > 0) {
                sb.append("&");
            }
            sb.append(String.format("%s=%s",
                    URLEncoder.encode(entry.getKey().toString(), "UTF-8"),
                    URLEncoder.encode(entry.getValue().toString(), "UTF-8")
            ));
        }
        return sb.toString();
    }
	
}
