package com.itopview.wm.utils;

import java.util.Map;

import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.lang.Nullable;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

/**
 * http请求工具类，依赖于spring-web
 * @description:
 * 
 * @example	"http://localhost:8080/user/getByInfo?name=zhangsan&age=30"
 * 			
 * 			doGet("http://localhost:8080/user/getByName?name={1}&age={2}", "zhangsan", "30")
 * 
 * 			Map<String, String> params = new HashMap();
 *			params.put("name", "zhangsan");
 *			params.put("age", "30");
 * 			doGet("http://localhost:8080/user/getByName?name={name}&age={age}", params)
 * 
 * @author longh
 * @date 2022年8月11日 下午2:23:20
 */
public class HttpUtils {

	private static HttpUtils instance;

	public static HttpUtils getInstance() {
		if (instance == null) {
			synchronized (HttpUtils.class) {
				if (instance == null) {
					instance = new HttpUtils();
				}
			}
		}
		return instance;
	}
	
	private RestTemplate getRestTemplate() {
		SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
		requestFactory.setConnectTimeout(10000);
		requestFactory.setReadTimeout(20000);
		
		return new RestTemplate(requestFactory);
	}
	
	public String doGet(String url) {
        return doGetForObject(url, String.class);
	}
	
	public String doGet(String url, Object... uriVariables) {
        return doGetForObject(url, String.class, uriVariables);
	}
	
	public String doGet(String url, Map<String, ?> uriVariables) {
        return doGetForObject(url, String.class, uriVariables);
	}
	
	public <T> T doGetForObject(String url, Class<T> responseType, Object... uriVariables) {
		T forObject = null;
        try {
        	forObject = getRestTemplate().getForObject(url, responseType);
		} catch (RestClientException e) {
			e.printStackTrace();
		}
        return forObject;
	}
	
	public <T> T doGetForObject(String url, Class<T> responseType, Map<String, ?> uriVariables) {
		T forObject = null;
        try {
        	forObject = getRestTemplate().getForObject(url, responseType, uriVariables);
		} catch (RestClientException e) {
			e.printStackTrace();
		}
        return forObject;
	}
	
	public String doPost(String url, @Nullable Object request) {
        return doPostForObject(url, request, String.class);
	}
	
	public String doPost(String url, @Nullable Object request, Object... uriVariables) {
        return doPostForObject(url, request, String.class, uriVariables);
	}
	
	public <T> T doPostForObject(String url, @Nullable Object request, Class<T> responseType, Object... uriVariables) {
        T forObject = null;
        try {
        	forObject = getRestTemplate().postForObject(url, request, responseType, uriVariables);
		} catch (RestClientException e) {
			e.printStackTrace();
		}
        return forObject;
	}

}
