package com.itopview.wm.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.UUID;

import kd.bos.context.RequestContext;
import kd.bos.exception.KDBizException;
import kd.bos.fileservice.FileItem;
import kd.bos.fileservice.FileService;
import kd.bos.fileservice.FileServiceFactory;
import kd.bos.fileservice.enums.PreviewParams;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.util.StringUtils;

public class WordToPdfUtils {
	private static final Log log = LogFactory.getLog(WordToPdfUtils.class);

	public static String getPdfUrl(String docUrl, String fileName) {
		String docFileName = fileName + ".doc";
		log.info("wordToPdf docUrl : " + docUrl);
		if (StringUtils.isEmpty(docUrl)) {
			return docUrl;
		} else {
			FileService service = FileServiceFactory.getAttachmentFileService();

			try {
				ByteArrayOutputStream e = readInputStream(service.getInputStream(docUrl));
				Throwable arg4 = null;

				String arg11;
				try {
					ByteArrayInputStream inputStream = new ByteArrayInputStream(e.toByteArray());
					Throwable arg6 = null;

					try {
						String middleUrl = uploadFile(inputStream, docFileName, getDefaultPath(docFileName));
						log.info("wordToPdf middleUrl : " + middleUrl);
						String pdfName = fileName + ".pdf";
						InputStream pdfInputStream = wordToPdf(middleUrl, docFileName);
						String path = uploadFile(pdfInputStream, pdfName, getDefaultPath(pdfName));
						log.info("wordToPdf path : " + middleUrl);
						arg11 = path;
					} catch (Throwable arg36) {
						arg6 = arg36;
						throw arg36;
					} finally {
						if (inputStream != null) {
							if (arg6 != null) {
								try {
									inputStream.close();
								} catch (Throwable arg35) {
									arg6.addSuppressed(arg35);
								}
							} else {
								inputStream.close();
							}
						}

					}
				} catch (Throwable arg38) {
					arg4 = arg38;
					throw arg38;
				} finally {
					if (e != null) {
						if (arg4 != null) {
							try {
								e.close();
							} catch (Throwable arg34) {
								arg4.addSuppressed(arg34);
							}
						} else {
							e.close();
						}
					}

				}

				return arg11;
			} catch (IOException arg40) {
				throw new KDBizException("文件生成失败，请联系管理员。");
			}
		}
	}

	private static InputStream wordToPdf(String path, String filename) {
		log.info("path:" + path + ",filename:" + filename);
		InputStream in = null;
		if (!StringUtils.isEmpty(filename) && !StringUtils.isEmpty(path)) {
			String subfix = filename.substring(filename.lastIndexOf(46) + 1);
			FileService service = FileServiceFactory.getAttachmentFileService();
			InputStream doc = service.getInputStream(path);
			if (!"doc".equalsIgnoreCase(subfix) && !"docx".equalsIgnoreCase(subfix)) {
				in = doc;
			} else {
				try {
					Map<String, Object> var8 = service.preview(filename, path, RequestContext.get().getUserAgent());
					String status = (String) var8.get(PreviewParams.STATUS.getEnumName());
					if (!PreviewParams.ERROR.getEnumName().equals(status)
							&& !PreviewParams.XLSX_SUCCESS.getEnumName().equals(status)
							&& (PreviewParams.PDF_SUCCESS.getEnumName().equals(status)
									|| PreviewParams.NOT_NEED_CHANGE.getEnumName().equals(status))) {
						in = (InputStream) var8.get(PreviewParams.RESULT.getEnumName());
					}
				} catch (Exception arg7) {
					throw new KDBizException("生成pdf文件失败，请检查预览服务是否可用。");
				}
			}
		}

		if (in == null) {
			throw new KDBizException("生成pdf文件失败，请检查预览服务是否可用。");
		} else {
			return in;
		}
	}

	private static ByteArrayOutputStream readInputStream(InputStream inputStream) throws IOException {
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		byte[] buffer = new byte[1024];

		int len;
		while ((len = inputStream.read(buffer)) > -1) {
			outputStream.write(buffer, 0, len);
		}

		outputStream.flush();
		return outputStream;
	}

	private static String uploadFile(InputStream in, String fileName, String path) {
		FileItem fileItem = new FileItem(fileName, path, in);
		fileItem.setCreateNewFileWhenExists(false);
		FileService service = FileServiceFactory.getAttachmentFileService();
		if (StringUtils.isNotEmpty(path)) {
			service.delete(path);
		}

		return service.upload(fileItem);
	}

	private static String getDefaultPath(String name) {
		try {
			name = URLEncoder.encode(name, "utf-8");
		} catch (UnsupportedEncodingException arg1) {
			log.info(arg1.getMessage());
		}
		String tenantId = RequestContext.get().getTenantId();
		String accountId = RequestContext.get().getAccountId();
		String uuid = UUID.randomUUID().toString().replace("-", "");
		String dateStr = new SimpleDateFormat("yyyyMM").format(new Date());
		return String.format("/%s/%s/%s/%s/%s/%s", new Object[]{tenantId, accountId, dateStr, uuid, "word2pdf", name});
	}
}