package tpv.fi.tpv_ap_ext.formplugin.form;

import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.dataentity.resource.ResManager;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.entity.datamodel.IBillModel;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.datamodel.events.*;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.property.EntryProp;
import kd.bos.exception.KDBizException;
import kd.bos.form.*;
import kd.bos.form.control.Control;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.*;
import kd.bos.form.field.*;
import kd.bos.form.operate.AbstractOperate;
import kd.bos.list.ListFilterParameter;
import kd.bos.list.ListShowParameter;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.mvc.bill.BillView;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.fi.ap.business.helper.InvoiceCardHelper;
import kd.fi.ap.enums.BillStatusEnum;
import kd.fi.ap.enums.DiscountModeEnum;
import kd.fi.ap.errorcode.SystemErrorCode;
import kd.fi.ap.formplugin.formservice.invoice.ApInvoiceBillImportHelper;
import kd.fi.ap.helper.ApHelper;
import kd.fi.ap.helper.OrgHelper;
import kd.fi.ap.piaozone.ShowInvoiceCloudPageUtil;
import kd.fi.arapcommon.business.price.IPriceCalculate;
import kd.fi.arapcommon.business.price.PriceLocalCalculator;
import kd.fi.arapcommon.business.price.PriceTaxTotalCalculator;
import kd.fi.arapcommon.business.price.UnitPriceCalculator;
import kd.fi.arapcommon.enums.InvoiceType;
import kd.fi.arapcommon.form.FormServiceHelper;
import kd.fi.arapcommon.helper.*;
import kd.fi.arapcommon.util.EmptyUtils;
import kd.fi.arapcommon.util.OperationUtils;
import kd.fi.arapcommon.util.StdConfig;
import kd.fi.arapcommon.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * @ClassName: InvoiceEdit
 * @Description:
 * @date: 2024/1/22 17:36
 * @author: chen
 */
public class InvoiceEdit extends kd.fi.ap.formplugin.InvoiceEdit {
    private boolean changed = true;
    private boolean changeType = true;
    private static final String INVOICETYPE = "invoiceType";
    private InitHelper init;
    private static final String[] s = new String[]{"receivablesacct", "asstact", "sellertin", "selleraddr", "sellertel", "sellerbank", "selleracct", "selleracct", "asstactname"};
    private static final String[] ENTRY = new String[]{"price", "e_amount", "e_amountbase", "e_tax", "discountamt", "discountlocalamt", "e_pricetaxtotal", "e_pricetaxtotalbase", "e_relatedamt", "e_unrelatedamt", "e_unmatchamt", "e_unmatchqty"};
    private static final String[] HEAD = new String[]{"pricetaxtotal", "pricetaxtotalbase", "amount", "amountbase", "tax", "unrelatedamt", "unmatchamt"};

    private boolean isOtrType() {
        BillShowParameter billShowParameter = (BillShowParameter) this.getView().getFormShowParameter();
        Object operate = billShowParameter.getCustomParam("invoiceType");
        return "OTHER".equals(operate);
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        String key = e.getProperty().getName();
        ChangeData[] changeData = e.getChangeSet();
        Object newValue = changeData[0].getNewValue();
        Object oldValue = changeData[0].getOldValue();
        int rowIndex = changeData[0].getRowIndex();
        switch (key) {
            case "asstacttype":
                this.asstactTypeChanged(newValue);
                break;
            case "asstact":
                this.asstactChanged(newValue);
                break;
            case "receivablessupp":
                this.receivablessuppF7Changed(newValue, this.getModel());
                break;
            case "payeerf7":
                this.erPayeeF7Changed((DynamicObject) newValue);
                break;
            case "exratedate":
                if (Objects.isNull(newValue)) {
                    FormServiceHelper.recoverOldValue(this.getView(), "exratedate", oldValue);
                    return;
                }
                break;
            case "exratetable":
                if (ObjectUtils.isEmpty(newValue)) {
                    FormServiceHelper.recoverOldValue(this.getView(), "exratetable", oldValue);
                }
                break;
            case "currency":
                if (newValue == null) {
                    FormServiceHelper.recoverOldValue(this.getView(), "currency", oldValue);
                    return;
                }

                if (ObjectUtils.isEmpty(oldValue)) {
                    return;
                }

                int o = ((DynamicObject) oldValue).getInt("priceprecision");
                int n = ((DynamicObject) newValue).getInt("priceprecision");
                if (o != n) {
                    for (int i = 0; i < this.getModel().getEntryRowCount("entry"); ++i) {
                        this.initPrice(this.getModel(), o, n, i);
                    }
                }
                break;
            case "invoicetypeview":
                this.getModel().setValue("invoicetype", this.getModel().getValue("invoicetypeview"));
                this.setTaxDeduction(newValue);
                break;
            case "invoicetype":
                this.setTaxDeduction(newValue);
                String type = (String) newValue;
                String typeNo = InvoiceType.getTypeNo(type);
                Long invoiceTypePK = InvoiceTypeHelper.getInvoiceTypePK(typeNo);
                this.getModel().setValue("invoicetypef7", invoiceTypePK);
                if (invoiceTypePK == 0L) {
                    this.getModel().beginInit();
                    this.getModel().setValue("invoicetypef7", (Object) null);
                    this.getModel().endInit();
                    this.getView().updateView("invoicetypef7");
                }
                break;
            case "invoicetypef7":
                DynamicObject invoicetype = (DynamicObject) newValue;
                if (invoicetype != null) {
                    String number = invoicetype.getString("number");
                    String value = InvoiceType.getValue(number);
                    if (value == null) {
                        value = number;
                    }

                    this.getModel().setValue("invoicetype", value);
                } else {
                    this.getModel().setValue("invoicetype", (Object) null);
                }
                break;
            case "biztype":
                this.bizTypeChanged(changeData[0]);
                break;
            case "issuedate":
                this.dateChanged(newValue);
                this.setReceivedate(newValue);
                break;
            case "material":
                this.getModel().setValue("e_materialversion", (Object) null);
                this.materialChanged(changeData[0]);
                break;
            case "measureunit":
                this.calculateQty(rowIndex);
                break;
            case "quantity":
                if (BigDecimal.ZERO.compareTo((BigDecimal)newValue) == 0) {
                    FormServiceHelper.recoverOldValue(this.getView(), "quantity", newValue, rowIndex);
                }
            case "taxrate":
            case "discountrate":
                this.calculateAmt(this.getModel(), rowIndex);
                break;
            case "price":
                boolean isPresent = (Boolean) this.getModel().getValue("ispresent", rowIndex);
                if (isPresent) {
                    BigDecimal price = (BigDecimal) newValue;
                    if (BigDecimal.ZERO.compareTo(price) == 0) {
                        this.getModel().setValue("discountmode", "NULL", rowIndex);
                    } else {
                        this.getModel().setValue("discountmode", "PERCENT", rowIndex);
                        this.getModel().setValue("discountrate", new BigDecimal(100), rowIndex);
                    }
                }

                this.calculateAmt(this.getModel(), rowIndex);
                break;
            case "discountamt":
                if (!"TOTAL".equals(this.getModel().getValue("discountmode"))) {
                    break;
                }
            case "discountmode":
                this.getModel().setValue("discountrate", BigDecimal.ZERO, rowIndex);
                if ("TOTAL".equals(newValue)) {
                    this.getModel().setValue("discountamt", BigDecimal.ZERO, rowIndex);
                }

                this.calculateAmt(this.getModel(), rowIndex);
                break;
            case "ispresent":
                boolean curIsPresent = (Boolean) newValue;
                BigDecimal rice = (BigDecimal) this.getModel().getValue("price", rowIndex);
                if (curIsPresent && BigDecimal.ZERO.compareTo(rice) != 0) {
                    this.getModel().setValue("discountmode", "PERCENT", rowIndex);
                    this.getModel().setValue("discountrate", new BigDecimal(100), rowIndex);
                } else {
                    this.getModel().setValue("discountmode", "NULL", rowIndex);
                }
                break;
            case "e_pricetaxtotal":
                Object oldVal = changeData[0].getOldValue();
                BigDecimal newVal = (BigDecimal) newValue;
                BigDecimal val = (BigDecimal) this.getModel().getValue("pricetaxtotal");
                if (ObjectUtils.isEmpty(oldVal)) {
                    new BigDecimal(0);
                } else {
                    this.getModel().setValue("pricetaxtotal", val.subtract((BigDecimal) oldVal).add(newVal));
                }

                if (this.changed) {
                    this.calculateAmt(this.getModel(), rowIndex);
                }
                break;
            case "pricetaxtotal":
                this.pricetaxtotalChanged(newValue);
                break;
            case "exchangerate":
                if (EmptyUtils.isEmpty(newValue)) {
                    this.getView().showErrorNotification(ResManager.loadKDString("该结算币别汇率为空！", "InvoiceEdit_8", "fi-ar-formplugin", new Object[0]));
                    return;
                }

                this.calculateAmt(this.getModel(), -1);
                break;
            case "expense":
                if (this.isOtrType() || "OTHER".equals(this.getModel().getValue("invoicetype"))) {
                    this.getModel().setValue("expenseitem", newValue, 0);
                }
                break;
            case "e_tax":
                this.changeTax(newValue, changeData[0].getRowIndex());
                break;
            case "corebilltype":
                this.getModel().setValue("corebillno", (Object) null, rowIndex);
                this.getModel().setValue("corebillentryseq", (Object) null, rowIndex);
        }

    }

    private void setTaxDeduction(Object invoiceType) {
        if (!"SP".equals(invoiceType) && !"SE".equals(invoiceType)) {
            this.getModel().setValue("istaxdeduction", Boolean.FALSE);
        } else {
            this.getModel().setValue("istaxdeduction", Boolean.TRUE);
        }

    }

    private void changeTax(Object newValue, int rowIndex) {
        if (!ObjectUtils.isEmpty(newValue)) {
            IDataModel m = this.getModel();
            BigDecimal e_pricetaxtotal = (BigDecimal) m.getValue("e_pricetaxtotal", rowIndex);
            BigDecimal exchangerate = (BigDecimal) m.getValue("exchangerate");
            String quotation = (String) m.getValue("quotation");
            if ("1".equals(quotation) && exchangerate.compareTo(BigDecimal.ZERO) == 0) {
                quotation = "0";
            }

            if (ObjectUtils.isEmpty(exchangerate)) {
                this.getView().showMessage(ResManager.loadKDString("该结算币别汇率为空！", "InvoiceEdit_8", "fi-ap-formplugin", new Object[0]));
            } else {
                BigDecimal e_amount = e_pricetaxtotal.subtract((BigDecimal) newValue);
                m.setValue("e_amount", e_amount, rowIndex);
                DynamicObject basecurrency = (DynamicObject) m.getValue("basecurrency");
                if (basecurrency == null) {
                    this.getView().showMessage(ResManager.loadKDString("结算币别值不能为空", "InvoiceEdit_9", "fi-ap-formplugin", new Object[0]));
                } else {
                    int localprecision = basecurrency.getInt("amtprecision");
                    BigDecimal amountbase = BigDecimal.ZERO;
                    if ("1".equals(quotation)) {
                        amountbase = e_amount.divide(exchangerate, localprecision, RoundingMode.HALF_UP);
                    } else {
                        amountbase = e_amount.multiply(exchangerate).setScale(localprecision, RoundingMode.HALF_UP);
                    }

                    m.setValue("e_amountbase", amountbase, rowIndex);
                    this.setHeadAmount();
                }
            }
        }
    }

    private void initPrice(IDataModel m, int oldPriceprecision, int newPriceprecision, int currentRow) {
        boolean includeTax = (Boolean) m.getValue("isincludetax");
        BigDecimal oldpricetaxtotal;
        if (includeTax) {
            oldpricetaxtotal = (BigDecimal) m.getValue("e_pricetaxtotal", currentRow);
            m.setValue("e_pricetaxtotal", oldpricetaxtotal.setScale(newPriceprecision, RoundingMode.DOWN), currentRow);
        } else {
            oldpricetaxtotal = (BigDecimal) m.getValue("price", currentRow);
            m.setValue("price", oldpricetaxtotal.setScale(newPriceprecision, RoundingMode.DOWN), currentRow);
        }

    }

    private void calculateAmt(IDataModel m, int index) {
        m.beginInit();
        if (index != -1) {
            this.calculatorEntryAmt(m, index, false);
        } else {
            for (int i = 0; i < m.getEntryRowCount("entry"); ++i) {
                this.calculatorEntryAmt(m, i, false);
            }
        }

        this.setHeadAmount();
        m.endInit();
        IFormView v = this.getView();
        String[] var4;
        int var5;
        int var6;
        String h;
        if (index != -1) {
            var4 = ENTRY;
            var5 = var4.length;

            for (var6 = 0; var6 < var5; ++var6) {
                h = var4[var6];
                v.updateView(h, index);
            }
        } else {
            for (int i = 0; i < m.getEntryRowCount("entry"); ++i) {
                String[] var11 = ENTRY;
                var6 = var11.length;

                for (int var12 = 0; var12 < var6; ++var12) {
                    String e = var11[var12];
                    v.updateView(e, i);
                }
            }
        }

        var4 = HEAD;
        var5 = var4.length;

        for (var6 = 0; var6 < var5; ++var6) {
            h = var4[var6];
            v.updateView(h);
        }

    }

    private void calculatorEntryAmt(IDataModel m, int index, boolean isbotpPush) {
        boolean includeTax = (Boolean) m.getValue("isincludetax");
        DynamicObject currency = (DynamicObject) m.getValue("currency");
        int precision = ObjectUtils.isEmpty(currency) ? 2 : currency.getInt("amtprecision");
        DynamicObject basecurrency = (DynamicObject) m.getValue("basecurrency");
        int localprecision = ObjectUtils.isEmpty(basecurrency) ? 2 : basecurrency.getInt("amtprecision");
        BigDecimal rate = (BigDecimal) m.getValue("exchangerate");
        String quotation = (String) m.getValue("quotation");
        if ("1".equals(quotation) && rate.compareTo(BigDecimal.ZERO) == 0) {
            quotation = "0";
        }

        BigDecimal qty = (BigDecimal) m.getValue("quantity", index);
        BigDecimal taxRate = (BigDecimal) m.getValue("taxrate", index);
        String discountMode = (String) m.getValue("discountmode", index);
        BigDecimal discountRate = (BigDecimal) m.getValue("discountrate", index);
        BigDecimal discountAmt = (BigDecimal) m.getValue("discountamt", index);
        BigDecimal unitPrice = (BigDecimal) m.getValue("price", index);
        IPriceCalculate calculator = null;
        if (includeTax) {
            BigDecimal pricetaxtotal = (BigDecimal) m.getValue("e_pricetaxtotal", index);
            calculator = new PriceTaxTotalCalculator(isbotpPush, unitPrice, BigDecimal.ZERO, qty, pricetaxtotal, taxRate, discountMode, discountRate, precision, BigDecimal.ZERO, discountAmt);
        } else {
            calculator = new UnitPriceCalculator(qty, unitPrice, taxRate, discountMode, discountRate, precision, BigDecimal.ZERO, discountAmt);
        }

        PriceLocalCalculator localCalculator = new PriceLocalCalculator((IPriceCalculate) calculator, rate, localprecision, quotation);
        localCalculator.calculate();
        this.resetEntryProp(m, localCalculator, index);
    }

    private void resetEntryProp(IDataModel m, PriceLocalCalculator localCalculator, int index) {
        m.setValue("price", localCalculator.getUnitprice(), index);
        m.setValue("actprice", localCalculator.getActunitprice(), index);
        m.setValue("actpricetax", localCalculator.getActtaxunitprice(), index);
        m.setValue("e_tax", localCalculator.getTax(), index);
        m.setValue("discountamt", localCalculator.getDiscountamount(), index);
        m.setValue("discountlocalamt", localCalculator.getDiscountamountlocal(), index);
        m.setValue("e_amount", localCalculator.getAmount(), index);
        m.setValue("e_amountbase", localCalculator.getAmountlocal(), index);
        BigDecimal priceTaxTotal = localCalculator.getPricetaxtotal();
        m.setValue("e_pricetaxtotal", priceTaxTotal, index);
        m.setValue("e_unrelatedamt", priceTaxTotal, index);
        m.setValue("e_unmatchamt", priceTaxTotal, index);
        m.setValue("e_unmatchqty", localCalculator.getQuantity(), index);
        m.setValue("e_pricetaxtotalbase", localCalculator.getPricetaxtotallocal(), index);
    }

    private void setHeadAmount() {
        IDataModel model = this.getModel();
        int rowCount = model.getEntryRowCount("entry");
        BigDecimal amount = BigDecimal.ZERO;
        BigDecimal amountbase = BigDecimal.ZERO;
        BigDecimal pricetaxtotal = BigDecimal.ZERO;
        BigDecimal unrelatedamt = BigDecimal.ZERO;
        BigDecimal pricetaxtotalbase = BigDecimal.ZERO;
        BigDecimal tax = BigDecimal.ZERO;

        for (int rowIndex = 0; rowIndex < rowCount; ++rowIndex) {
            amount = amount.add((BigDecimal) model.getValue("e_amount", rowIndex));
            amountbase = amountbase.add((BigDecimal) model.getValue("e_amountbase", rowIndex));
            pricetaxtotal = pricetaxtotal.add((BigDecimal) model.getValue("e_pricetaxtotal", rowIndex));
            pricetaxtotalbase = pricetaxtotalbase.add((BigDecimal) model.getValue("e_pricetaxtotalbase", rowIndex));
            tax = tax.add((BigDecimal) model.getValue("e_tax", rowIndex));
            unrelatedamt = unrelatedamt.add((BigDecimal) model.getValue("e_unrelatedamt", rowIndex));
        }

        model.setValue("amount", amount);
        model.setValue("amountbase", amountbase);
        model.setValue("pricetaxtotal", pricetaxtotal);
        model.setValue("unrelatedamt", unrelatedamt);
        model.setValue("unmatchamt", unrelatedamt);
        model.setValue("pricetaxtotalbase", pricetaxtotalbase);
        model.setValue("tax", tax);
    }

    private void pricetaxtotalChanged(Object newValue) {
        if (this.isOtrType() || "OTHER".equals(this.getModel().getValue("invoicetype"))) {
            IDataModel m = this.getModel();
            m.beginInit();
            m.setValue("isincludetax", Boolean.TRUE);
            m.setValue("e_amount", newValue, 0);
            m.setValue("e_pricetaxtotal", newValue, 0);
            this.calculatorEntryAmt(this.getModel(), 0, false);
            this.setHeadAmount();
            m.endInit();
            this.getView().updateView();
        }

    }

    private void erPayeeF7Changed(DynamicObject erPayee) {
        if (erPayee != null) {
            DynamicObject user = (DynamicObject) this.getModel().getValue("receivablessupp");
            if (user == null) {
                QFilter filter = new QFilter("name", "=", erPayee.getString("name"));
                Map<Object, DynamicObject> userMap = BusinessDataServiceHelper.loadFromCache("bos_user", "id, name, number", new QFilter[]{filter});
                if (userMap != null && userMap.size() > 0) {
                    this.getModel().setValue("receivablessupp", ((DynamicObject[]) userMap.values().toArray(new DynamicObject[0]))[0]);
                }
            }

            this.getModel().setValue("receivablesacct", erPayee.getString("payeraccount"));
        } else {
            this.getModel().setValue("receivablesacct", (Object) null);
        }

    }

    private void asstactTypeChanged(Object newValue) {
        if (newValue != null) {
            this.getView().setVisible(Boolean.FALSE, new String[]{"flex_sellerinfo"});
            this.getView().setVisible(Boolean.TRUE, new String[]{"flex_addseller"});
            String[] var2 = s;
            int var3 = var2.length;

            for (int var4 = 0; var4 < var3; ++var4) {
                String string = var2[var4];
                this.getModel().setValue(string, (Object) null);
            }
        }

    }

    private void asstactChanged(Object newValue) {
        if (newValue == null) {
            this.getModel().setValue("asstactname", this.getModel().getValue("asstactname"));
        } else {
            this.getModel().setValue("asstactname", ((DynamicObject) newValue).getLocaleString("name").getLocaleValue());
        }

    }

    private void receivablessuppF7Changed(Object newValue, IDataModel model) {
        DynamicObject supp = (DynamicObject) newValue;
        if (ObjectUtils.isEmpty(model.getValue("serialno"))) {
            if (supp != null) {
                if ("bos_user".equals(this.getModel().getValue("asstacttype"))) {
                    DynamicObject erPayeeInfo = this.getErPayeeInfo(supp.getLong("id"), supp.getLocaleString("name").getLocaleValue());
                    if (erPayeeInfo != null) {
                        this.getModel().setValue("payeerf7", erPayeeInfo.getPkValue());
                    } else if (this.getModel().getValue("payeerf7") != null) {
                        this.getModel().setValue("payeerf7", (Object) null);
                    } else {
                        this.getModel().setValue("receivablesacct", (Object) null);
                    }
                } else {
                    supp = BusinessDataServiceHelper.loadSingleFromCache(supp.getPkValue(), this.getModel().getValue("asstacttype").toString());
                    String acctBank = null;
                    DynamicObjectCollection bankColls = supp.getDynamicObjectCollection("entry_bank");
                    if (bankColls != null && bankColls.size() > 0) {
                        if (bankColls.size() == 1) {
                            DynamicObject bankInfo = (DynamicObject) bankColls.get(0);
                            acctBank = bankInfo.getString("bankaccount");
                        } else {
                            for (int i = 0; i < bankColls.size(); ++i) {
                                DynamicObject bankInfo = (DynamicObject) bankColls.get(i);
                                if (bankInfo.getBoolean("isdefault_bank")) {
                                    acctBank = bankInfo.getString("bankaccount");
                                    break;
                                }

                                if (i == 0) {
                                    acctBank = bankInfo.getString("bankaccount");
                                }
                            }
                        }
                    }

                    this.getModel().setValue("receivablesacct", acctBank);
                }
            }

        }
    }

    private DynamicObject getErPayeeInfo(long userId, String name) {
        DynamicObject defaultAccount = null;
        QFilter uFilter = new QFilter("name", "=", name);
        uFilter = uFilter.and(new QFilter("enable", "=", Boolean.TRUE));
        uFilter.and(new QFilter("payer", "=", userId));
        QFilter[] qFilters = new QFilter[]{uFilter};
        String selectFields = "id,name,payer,payeraccount,payeraccount01,payeraccount02,payerbank_id,payerbank.name,payerbank.number,isdefault";
        Map<Object, DynamicObject> defaultAccountMap = BusinessDataServiceHelper.loadFromCache("er_payeer", selectFields, qFilters);
        if (defaultAccountMap != null && defaultAccountMap.size() > 0) {
            List<DynamicObject> accountList = new ArrayList(defaultAccountMap.values());
            defaultAccount = (DynamicObject) accountList.get(0);
            Iterator var10 = accountList.iterator();

            while (var10.hasNext()) {
                DynamicObject account = (DynamicObject) var10.next();
                if (account.getBoolean("isdefault")) {
                    defaultAccount = account;
                    break;
                }
            }
        }

        return defaultAccount;
    }

    private void dateChanged(Object newValue) {
        Date issueDate = (Date) newValue;
        if (issueDate != null) {
            DateEdit dateEdit = (DateEdit) this.getControl("lastpaydate");
            dateEdit.setMinDate(issueDate);
            Date lastpayDate = (Date) this.getModel().getValue("lastpaydate");
            if (lastpayDate != null && lastpayDate.before(issueDate)) {
                this.getModel().setValue("lastpaydate", issueDate);
            }

        }
    }

    private void setReceivedate(Object newValue) {
        Date issuedate = (Date) newValue;
        if (issuedate != null) {
            Date receivedate = (Date) this.getModel().getValue("receivedate");
            if (EmptyUtils.isEmpty(receivedate) || receivedate.before(issuedate)) {
                this.getModel().setValue("receivedate", issuedate);
            }

            DateEdit dateEdit = (DateEdit) this.getControl("receivedate");
            dateEdit.setMinDate(issuedate);
        }
    }

    private void calculateQty(int rowIndex) {
        IDataModel model = this.getModel();
        DynamicObject measureUnit = (DynamicObject) model.getValue("measureunit", rowIndex);
        if (measureUnit != null) {
            BigDecimal quantity = (BigDecimal) model.getValue("quantity", rowIndex);
            if (quantity != null) {
                model.setValue("quantity", quantity.setScale(measureUnit.getInt("precision"), 4), rowIndex);
            }

        }
    }


    private void bizTypeChanged(ChangeData bizType) {
        String newValue = bizType.getNewValue().toString();
        if (newValue != null) {
            if ("PUR".equals(newValue)) {
                this.getModel().setValue("asstacttype", "bd_supplier");
            }

            this.getPageCache().put("bizType", bizType.getOldValue().toString());
            if (this.changeType && this.getModel().getEntryRowCount("entry") != 0 && !this.isOtrType()) {
                this.getView().showConfirm(ResManager.loadKDString("切换业务类型将删除所有分录，是否继续？", "InvoiceEdit_7", "fi-ap-formplugin", new Object[0]), MessageBoxOptions.OKCancel, new ConfirmCallBackListener("clearEntry"));
            }
        }

    }

    private void materialChanged(ChangeData changeData) {
        int currentIndex = changeData.getRowIndex();
        Object newValue = changeData.getNewValue();
        DynamicObject material = (DynamicObject) newValue;
        if (material != null) {
            this.getModel().setValue("spectype", material.getLocaleString("modelnum").getLocaleValue(), currentIndex);
            if (material.getDynamicObject("baseunit") != null) {
                this.getModel().setValue("measureunit", material.getDynamicObject("baseunit").getPkValue(), currentIndex);
            }

            String serialNo = this.getModel().getValue("serialno").toString();
            if (material.getDynamicObject("taxrate") != null && StringUtils.isEmpty(serialNo)) {
                Object taxrateId = material.getDynamicObject("taxrate").getPkValue();
                IDataModel model = this.getModel();
                model.setValue("taxrateid", taxrateId, currentIndex);
            }
        } else {
            this.getModel().setValue("measureunit", "", currentIndex);
            this.getModel().setValue("spectype", "", currentIndex);
        }

    }

}
