package tpv.fi.tpv_ap_ext.opplugin.validator;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.resource.ResManager;
import kd.bos.dataentity.utils.ObjectUtils;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.validate.AbstractValidator;

import java.math.BigDecimal;

/**
 * @ClassName: BillPriceValidator
 * @Description:
 * @date: 2024/1/16 14:54
 * @author: chen
 */
public class BillPriceValidator extends AbstractValidator {

    @Override
    public void validate() {
        String operateKey = this.getOperateKey();
        ExtendedDataEntity[] datas = this.getDataEntities();
        String billName = datas[0].getDataEntity().getDataEntityType().getName();
        ExtendedDataEntity[] var4 = datas;
        int var5 = datas.length;

        for(int var6 = 0; var6 < var5; ++var6) {
            ExtendedDataEntity data = var4[var6];
            DynamicObject bill = data.getDataEntity();
            boolean includetax = false;
            if ("ap_busbill".equals(billName) || "ar_finarbill".equals(billName) || "ar_busbill".equals(billName) || "ap_finapbill".equals(billName)) {
                includetax = bill.get("isincludetax") != null && bill.getBoolean("isincludetax");
            }

            DynamicObjectCollection entries = bill.getDynamicObjectCollection(this.getBillEntryName(billName));

            for(int i = 0; i < entries.size(); ++i) {
                DynamicObject row = (DynamicObject)entries.get(i);
                boolean ispresent = row.get(this.getBillPresentName(billName)) != null && row.getBoolean(this.getBillPresentName(billName));
                if (!ispresent) {
                    if (includetax) {
                        this.checkEntryPrice(row.get(this.getBillPriceTaxName(billName)), ResManager.loadKDString("含税单价", "BillPriceValidator_0", "fi-arapcommon", new Object[0]), i + 1, data);
                    } else {
                        this.checkEntryPrice(row.get(this.getBillPriceName(billName)), ResManager.loadKDString("单价", "BillPriceValidator_1", "fi-arapcommon", new Object[0]), i + 1, data);
                    }
                }

                if ("submit".equals(operateKey) && !"ar_revcfmbill".equals(billName)) {
                    BigDecimal quantity = row.getBigDecimal(this.getBillQuantityName(billName));
                    BigDecimal tax = row.getBigDecimal(this.getBillTaxName(billName));
                    BigDecimal amount = row.getBigDecimal(this.getBillAmountName(billName));
                    BigDecimal discountAmount = row.getBigDecimal(this.getBillDiscountAmountName(billName));
                    BigDecimal priceTaxTotal = row.getBigDecimal(this.getBillPriceTaxTotalName(billName));
                    if (quantity.multiply(tax).compareTo(BigDecimal.ZERO) < 0 || quantity.multiply(amount).compareTo(BigDecimal.ZERO) < 0 || quantity.multiply(discountAmount).compareTo(BigDecimal.ZERO) < 0 || quantity.multiply(priceTaxTotal).compareTo(BigDecimal.ZERO) < 0) {
//                        this.addErrorMessage(data, String.format(ResManager.loadKDString("“单据体”第%1$s行，数量、税额、金额、折扣额、%2$s的方向需一致，请修改。", "BillPriceValidator_3", "fi-arapcommon", new Object[0]), i + 1, this.getPriceTaxTotalLocalString(billName)));
                    }
                }
            }
        }

    }
    private void checkEntryPrice(Object price, String name, int seq, ExtendedDataEntity data) {
        if (!ObjectUtils.isEmpty(price) && ((BigDecimal)price).compareTo(BigDecimal.ZERO) < 0) {
//            this.addErrorMessage(data, String.format(ResManager.loadKDString("“明细”第%1$s行，%2$s要大于等于0！", "BillPriceValidator_2", "fi-arapcommon", new Object[0]), seq, name));
        }

    }

    private String getBillPriceName(String billName) {
        String billPriceName = "";
        if (!"ap_busbill".equals(billName) && !"ar_finarbill".equals(billName) && !"ar_busbill".equals(billName) && !"ar_revcfmbill".equals(billName) && !"ar_invoice".equals(billName)) {
            if ("ap_finapbill".equals(billName) || "ap_invoice".equals(billName)) {
                billPriceName = "price";
            }
        } else {
            billPriceName = "e_unitprice";
        }

        return billPriceName;
    }

    private String getBillPriceTaxName(String billName) {
        String billPriceTaxName = "";
        if (!"ap_busbill".equals(billName) && !"ar_finarbill".equals(billName) && !"ar_busbill".equals(billName)) {
            if ("ap_finapbill".equals(billName)) {
                billPriceTaxName = "pricetax";
            }
        } else {
            billPriceTaxName = "e_taxunitprice";
        }

        return billPriceTaxName;
    }

    private String getBillEntryName(String billName) {
        String billEntryName = "";
        if (!"ap_busbill".equals(billName) && !"ar_finarbill".equals(billName) && !"ar_busbill".equals(billName) && !"ar_revcfmbill".equals(billName) && !"ar_invoice".equals(billName) && !"ap_invoice".equals(billName)) {
            if ("ap_finapbill".equals(billName)) {
                billEntryName = "detailentry";
            }
        } else {
            billEntryName = "entry";
        }

        return billEntryName;
    }

    private String getBillPresentName(String billName) {
        String billPresentName = "";
        if (!"ap_busbill".equals(billName) && !"ar_finarbill".equals(billName) && !"ar_busbill".equals(billName) && !"ar_revcfmbill".equals(billName) && !"ar_invoice".equals(billName)) {
            if ("ap_finapbill".equals(billName) || "ap_invoice".equals(billName)) {
                billPresentName = "ispresent";
            }
        } else {
            billPresentName = "e_ispresent";
        }

        return billPresentName;
    }

    private String getBillPriceTaxTotalName(String billName) {
        String priceTaxTotal = "";
        if (!"ap_finapbill".equals(billName) && !"ap_busbill".equals(billName) && !"ap_invoice".equals(billName)) {
            if ("ar_finarbill".equals(billName) || "ar_invoice".equals(billName) || "ar_busbill".equals(billName)) {
                priceTaxTotal = "e_recamount";
            }
        } else {
            priceTaxTotal = "e_pricetaxtotal";
        }

        return priceTaxTotal;
    }

    private String getBillAmountName(String billName) {
        String amount = "";
        if ("ap_finapbill".equals(billName) || "ap_busbill".equals(billName) || "ap_invoice".equals(billName) || "ar_finarbill".equals(billName) || "ar_invoice".equals(billName) || "ar_busbill".equals(billName)) {
            amount = "e_amount";
        }

        return amount;
    }

    private String getBillDiscountAmountName(String billName) {
        String discountAmount = "";
        if ("ap_finapbill".equals(billName)) {
            discountAmount = "discountamount";
        } else if (!"ap_busbill".equals(billName) && !"ar_finarbill".equals(billName) && !"ar_invoice".equals(billName) && !"ar_busbill".equals(billName)) {
            if ("ap_invoice".equals(billName)) {
                discountAmount = "discountamt";
            }
        } else {
            discountAmount = "e_discountamount";
        }

        return discountAmount;
    }

    private String getBillTaxName(String billName) {
        String tax = "";
        if ("ap_finapbill".equals(billName) || "ap_busbill".equals(billName) || "ap_invoice".equals(billName) || "ar_finarbill".equals(billName) || "ar_invoice".equals(billName) || "ar_busbill".equals(billName)) {
            tax = "e_tax";
        }

        return tax;
    }

    private String getBillQuantityName(String billName) {
        String quantity = "";
        if (!"ap_finapbill".equals(billName) && !"ap_invoice".equals(billName)) {
            if ("ap_busbill".equals(billName) || "ar_finarbill".equals(billName) || "ar_invoice".equals(billName) || "ar_busbill".equals(billName)) {
                quantity = "e_quantity";
            }
        } else {
            quantity = "quantity";
        }

        return quantity;
    }

    private String getPriceTaxTotalLocalString(String billName) {
        String priceTaxTotalKey = "";
        if (!"ap_finapbill".equals(billName) && !"ap_busbill".equals(billName)) {
            if (!"ap_invoice".equals(billName) && !"ar_invoice".equals(billName)) {
                if ("ar_finarbill".equals(billName) || "ar_busbill".equals(billName)) {
                    priceTaxTotalKey = ResManager.loadKDString("应收金额", "BillPriceValidator_7", "fi-arapcommon", new Object[0]);
                }
            } else {
                priceTaxTotalKey = ResManager.loadKDString("价税合计", "BillPriceValidator_6", "fi-arapcommon", new Object[0]);
            }
        } else {
            priceTaxTotalKey = ResManager.loadKDString("应付金额", "BillPriceValidator_5", "fi-arapcommon", new Object[0]);
        }

        return priceTaxTotalKey;
    }
}
