package tpv.sys.basedata.java.formplugin.list;

import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.botp.runtime.ConvertOperationResult;
import kd.bos.entity.botp.runtime.PushArgs;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.operate.botp.Push;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.servicehelper.botp.ConvertServiceHelper;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;

/**
 * 客户列表录入供应商按钮
 * WangShuGuang
 */
public class CustomerAddListPlugin extends AbstractListPlugin {
    private static final String TPV_BARITEMAP = "tpv_baritemap";

    private static String BD_CUSTOMER = "bd_customer";//客户

    private static String BD_SUPPLIER = "bd_supplier";//供应商

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        addItemClickListeners(TPV_BARITEMAP);
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        //监听录入供应上按钮
        if (evt.getItemKey().equals(TPV_BARITEMAP)) {
            showInfoForm(this.getView());
        }
    }

    private void showInfoForm(IFormView view) {
        // 获取单据体
        FormShowParameter showParameter = new FormShowParameter();
        showParameter.setStatus(OperationStatus.ADDNEW);
        showParameter.getOpenStyle().setShowType(ShowType.Modal);
        // 设置弹出页面的标识
        showParameter.setFormId("tpv_bd_customerpopup");
        showParameter.setCloseCallBack(new CloseCallBack(this, "tpv_rawmessages"));
        view.showForm(showParameter);
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
        String key = closedCallBackEvent.getActionId();
        Object returnData = closedCallBackEvent.getReturnData();
        if ("tpv_rawmessages".equals(key) && null != closedCallBackEvent.getReturnData()) {
            BillShowParameter showParameter = new BillShowParameter();
            //设置弹出页面的编码
            showParameter.setFormId("bd_customer");
            //设置弹出页面打开方式，支持模态，新标签等
            showParameter.getOpenStyle().setShowType(ShowType.MainNewTabPage);
            HashMap<String, Object> values = (HashMap<String, Object>) returnData;
            if (!values.isEmpty()){
                String sign = values.get("sign").toString();
                if (StringUtils.equals("1",sign)){
                    //直接新建和新增选中客户按钮
                    showParameter.setStatus(OperationStatus.ADDNEW);
                    showParameter.setCustomParam("name", values.get("name"));
                    showParameter.setCustomParam("societycreditcode", values.get("societycreditcode"));//统一社会信用代码
                    showParameter.setCustomParam("artificialperson", values.get("artificialperson"));//法人代表
                    showParameter.setCustomParam("tpv_industrialyphon", values.get("tpv_industrialyphon"));//工商登记电话
                    showParameter.setCustomParam("curegcapital", values.get("curegcapital"));//注册资本币种
                    showParameter.setCustomParam("regcapital", values.get("regcapital"));//注册资本
                    showParameter.setCustomParam("businessterm", values.get("businessterm"));//营业期限
                    showParameter.setCustomParam("establishdate", values.get("establishdate"));//成立日期
                    showParameter.setCustomParam("bizpartner_address", values.get("bizpartner_address"));//注册地址
                    showParameter.setCustomParam("tpv_registrationstatus", values.get("tpv_registrationstatus"));//登记状态
                    showParameter.setCustomParam("tpv_paidupcapital", values.get("tpv_paidupcapital"));//实缴资本
                    showParameter.setCustomParam("tpv_businesstype", values.get("tpv_businesstype"));//企业类型
                    showParameter.setCustomParam("tpv_insured", values.get("tpv_insured"));//参保人数
                    showParameter.setCustomParam("tpv_listed", values.get("tpv_listed"));//是否上市
                    showParameter.setCustomParam("tpv_typeoflisting", values.get("tpv_typeoflisting"));//上市类型
                    showParameter.setCustomParam("tpv_stocksymbol1", values.get("tpv_stocksymbol1"));//股票代码
                    showParameter.setCustomParam("tpv_businessscope", values.get("tpv_businessscope"));//经营范围
                    showParameter.setCustomParam("tpv_enterprisenature", values.get("tpv_enterprisenature"));//企业性质
                    showParameter.setCustomParam("tpv_aptitude", values.get("tpv_aptitude"));//A级纳税人
                    this.getView().showForm(showParameter);
                } else if (StringUtils.equals("2",sign)){
                    //已有客户，直接打开按钮
                    showParameter.setStatus(OperationStatus.VIEW);
                    showParameter.setFormId(values.get("FormId").toString());
                    showParameter.setPkId(values.get("PkId"));
                    this.getView().showForm(showParameter);
                } else if (StringUtils.equals("3",sign)){
                    //将本供应商复制为客户按钮
                    //客户下推供应商
                    List<ListSelectedRow> selectedRowsTwo = new ArrayList<>();
                    ListSelectedRow listSelectedRowTwo = new ListSelectedRow(values.get("id"));//来源单据ID
                    selectedRowsTwo.add(listSelectedRowTwo);
                    PushArgs pushArgs = new PushArgs();
                    pushArgs.setSourceEntityNumber(BD_SUPPLIER); // 必选，源单标识
                    pushArgs.setTargetEntityNumber(BD_CUSTOMER); // 必选，目标单标识
                    pushArgs.setAutoSave(true);//是否保存
                    pushArgs.setRuleId("1768184748389218304"); // 可选，传入本次下推使用的转换规则id；传入空值，由系统自动寻找合适的转换规则
                    pushArgs.setBuildConvReport(true); // 是否输出详细错误报告
                    pushArgs.setSelectedRows(selectedRowsTwo); // 必选，设置需要下推的源单及分录内码
                    ConvertOperationResult pushResult = ConvertServiceHelper.push(pushArgs);
                    if (pushResult.isSuccess()) {
                        Push.showTargetBill(this.getView(), pushArgs, pushResult, false);
                    } else {
                        Push.showReport(this.getView(), pushArgs, pushResult);
                    }
                }
            } /*else {
                showParameter.setCustomParam("name", values.get("name"));
            }*/
            //this.getView().showForm(showParameter);
        }
    }
}
