package tpv.sys.basedata.java.opplugin.op;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.itopview.wm.utils.YEM;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import tpv.tpv_wm.tpv_iscb.java.api.ant.GetPublicSentimentList;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * @ClassName: BdCustomerBusinesDataOp
 * @Description: 更新工商数据（客户）
 * @author: zbh
 * @date: 2024-07-03
 */
public class BdCustomerBusinesDataOp extends AbstractOperationServicePlugIn {

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] entities = e.getDataEntities();
        String itemKey = e.getOperationKey();
        for (DynamicObject entity : entities) {
            BusinesData(entity);
        }
    }

    public static void BusinesData(DynamicObject enty) {

        DynamicObject dataEntity = BusinessDataServiceHelper.loadSingle(enty.getPkValue(), "bd_customer");
        //更新工商数据
        String number = dataEntity.getString("number");//客户编码
        String tpv_sourcebillno = dataEntity.getString("tpv_sourcebillno");//来源单据编码
        DynamicObject bdSupplier = null;
        if (StringUtils.isEmpty(tpv_sourcebillno)) {
            //查询所关联的客户
            bdSupplier = BusinessDataServiceHelper.loadSingle("bd_supplier", "tpv_sourcebillno,id,name,societycreditcode,tpv_qichachaman,tpv_qichachadate",
                    new QFilter("tpv_sourcebillno", QCP.equals, number).toArray());
        } else {
            bdSupplier = BusinessDataServiceHelper.loadSingle("bd_supplier", "number,id,name,societycreditcode,tpv_qichachaman,tpv_qichachadate",
                    new QFilter("number", QCP.equals, tpv_sourcebillno).toArray());
        }
        String name = dataEntity.getLocaleString("name").getLocaleValue();
        //统一社会信用代码 NEW: 如果统一社会信用代码有值则为入参，否则以名称为入参
        String societycreditcode = dataEntity.getString("societycreditcode");
        if (YEM.isNotEmpty(societycreditcode)) {
            name = societycreditcode;
        }
        //供应商名称
        String bdSupplierName = null;
        if (null != bdSupplier) {
            bdSupplier = BusinessDataServiceHelper.loadSingle(bdSupplier.getPkValue(),
                    bdSupplier.getDynamicObjectType().getName());
            String bdSupplierCreditcode = bdSupplier.getString("societycreditcode");
            if (YEM.isNotEmpty(bdSupplierCreditcode)) {
                bdSupplierName = bdSupplierCreditcode;
            } else {
                bdSupplierName = bdSupplier.getLocaleString("name").getLocaleValue();
            }
        }
        String responseSupplier = null;//供应商
        JSONObject jsonObjectSupplier = new JSONObject();//供应商
        try {
            if (StringUtils.isNotEmpty(bdSupplierName)) {
                //有对应的客户
                responseSupplier = GetPublicSentimentList.enterPrise(bdSupplierName);
                jsonObjectSupplier = JSONObject.parseObject(responseSupplier);
            }
            if (null != jsonObjectSupplier) {
                //客户赋值
                JSONArray constituentTags = jsonObjectSupplier.getJSONArray("constituent_tags");//企业性质
                JSONArray listed_types = jsonObjectSupplier.getJSONArray("listed_types");//上市类型
                if (!constituentTags.isEmpty()) {
                    String tpv_enterprisenature = constituentTags.toJSONString();
                    String stringWithoutBrackets = tpv_enterprisenature.replace("[", "").replace("]", "");
                    String finalString = stringWithoutBrackets.replace("\"", "");
                    bdSupplier.set("tpv_enterprisenature", finalString);//企业性质（蚂蚁获取）
                    if (StringUtils.equals(finalString, "民营控股")) {
                        bdSupplier.set("tpv_natureoftheenterpr2", "MY");//企业性质
                    } else {
                        bdSupplier.set("tpv_natureoftheenterpr2", "GY");//企业性质
                    }
                }
                bdSupplier.set("artificialperson", jsonObjectSupplier.getString("legal_representative"));//法人代表
                bdSupplier.set("tpv_industrialyphon", jsonObjectSupplier.getString("org_tel"));//工商登记电话
                bdSupplier.set("tpv_billingnum", jsonObjectSupplier.getString("org_tel"));//开票电话号码
                String curegcapital = jsonObjectSupplier.getString("register_capital_currency");
                if (StringUtils.equals("人民币", curegcapital)) {
                    //人民币
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, "人民币").toArray());
                    bdSupplier.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                } else if (StringUtils.equals("港元", curegcapital)) {
                    //港元
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, curegcapital).toArray());
                    bdSupplier.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                } else {
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, curegcapital).toArray());
                    if (null != bd_currency) {
                        bdSupplier.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                    }
                }
                bdSupplier.set("societycreditcode", jsonObjectSupplier.getString("uc_code"));//统一社会信用代码
                bdSupplier.set("tx_register_no", jsonObjectSupplier.getString("uc_code"));//纳税人识别号
                bdSupplier.set("regcapital", jsonObjectSupplier.getString("register_capital_value"));//注册资本
                bdSupplier.set("businessterm", jsonObjectSupplier.getString("oprt_end_date"));//营业期限
                bdSupplier.set("establishdate", jsonObjectSupplier.getDate("register_date"));//成立日期
                bdSupplier.set("bizpartner_address", jsonObjectSupplier.getString("register_place"));//注册地址
                bdSupplier.set("tpv_billingaddress", jsonObjectSupplier.getString("register_place"));//开票地址
                bdSupplier.set("tpv_registrationstatus", jsonObjectSupplier.getString("org_state"));//登记状态
                bdSupplier.set("tpv_paidupcapital", jsonObjectSupplier.getString("org_rec_cptl_rmb"));//实缴资本
                bdSupplier.set("tpv_businesstype", jsonObjectSupplier.getString("org_type"));//企业类型
                bdSupplier.set("tpv_insured", jsonObjectSupplier.getString("employee_num"));//参保人数
                bdSupplier.set("tpv_province", jsonObjectSupplier.get("register_province"));//省
                bdSupplier.set("tpv_city", jsonObjectSupplier.get("register_city"));//市
                bdSupplier.set("tpv_county", jsonObjectSupplier.get("register_county"));//县
                if (StringUtils.isNotEmpty(jsonObjectSupplier.getString("latetype"))) {
                    dataEntity.set("tpv_listed", true);//是否上市
                    bdSupplier.set("tpv_listed", true);//是否上市
                } else {
                    bdSupplier.set("tpv_listed", false);//是否上市
                }
                if (listed_types.isEmpty()) {
                    bdSupplier.set("tpv_typeoflisting", null);//上市类型
                } else {
                    String jsonString = listed_types.toJSONString();
                    String stringWithoutBrackets = jsonString.replace("[", "").replace("]", "");
                    String finalString = stringWithoutBrackets.replace("\"", "");
                    bdSupplier.set("tpv_typeoflisting", finalString);//上市类型
                }
                bdSupplier.set("tpv_stocksymbol1", jsonObjectSupplier.getString("stock_code"));//股票代码
                bdSupplier.set("businessscope", jsonObjectSupplier.getString("oprt_scope"));//经营范围
                SaveServiceHelper.update(bdSupplier);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            String response = GetPublicSentimentList.enterPrise(name);
            JSONObject jsonObject = JSONObject.parseObject(response);
            if (null != jsonObject) {
                JSONArray constituentTags = jsonObject.getJSONArray("constituent_tags");//企业性质
                JSONArray listed_types = jsonObject.getJSONArray("listed_types");//上市类型
                if (!constituentTags.isEmpty()) {
                    String tpv_enterprisenature = constituentTags.toJSONString();
                    String stringWithoutBrackets = tpv_enterprisenature.replace("[", "").replace("]", "");
                    String finalString = stringWithoutBrackets.replace("\"", "");
                    dataEntity.set("tpv_enterprisenature", finalString);//企业性质
                    if (StringUtils.equals(finalString, "民营控股")) {
                        dataEntity.set("tpv_natureoftheenterpr2", "MY");//企业性质
                    } else {
                        dataEntity.set("tpv_natureoftheenterpr2", "GY");//企业性质
                    }
                }
                dataEntity.set("artificialperson", jsonObject.getString("legal_representative"));//法人代表
                dataEntity.set("tpv_industrialyphon", jsonObject.getString("org_tel"));//工商登记电话
                dataEntity.set("tpv_billingnum", jsonObject.getString("org_tel"));//开票电话号码
                String curegcapital = jsonObject.getString("register_capital_currency");
                if (StringUtils.equals("人民币", curegcapital)) {
                    //人民币
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, "人民币").toArray());
                    dataEntity.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                } else if (StringUtils.equals("港元", curegcapital)) {
                    //港元
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, curegcapital).toArray());
                    dataEntity.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                } else {
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, curegcapital).toArray());
                    if (null != bd_currency) {
                        dataEntity.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                    }
                }
                dataEntity.set("societycreditcode", jsonObject.getString("uc_code"));//统一社会信用代码
                dataEntity.set("tx_register_no", jsonObject.getString("uc_code"));//纳税人识别号
                dataEntity.set("regcapital", jsonObject.getString("register_capital_value"));//注册资本
                dataEntity.set("businessterm", jsonObject.getString("oprt_end_date"));//营业期限
                dataEntity.set("establishdate", jsonObject.getDate("register_date"));//成立日期
                dataEntity.set("bizpartner_address", jsonObject.getString("register_place"));//注册地址
                dataEntity.set("tpv_billingaddress", jsonObjectSupplier.getString("register_place"));//开票地址
                dataEntity.set("tpv_registrationstatus", jsonObject.getString("org_state"));//登记状态
                dataEntity.set("tpv_paidupcapital", jsonObject.getString("org_rec_cptl_rmb"));//实缴资本
                dataEntity.set("tpv_businesstype", jsonObject.getString("org_type"));//企业类型
                dataEntity.set("tpv_province", jsonObject.get("register_province"));//省
                dataEntity.set("tpv_city", jsonObject.get("register_city"));//市
                dataEntity.set("tpv_county", jsonObject.get("register_county"));//县
                if (StringUtils.isNotEmpty(jsonObject.getString("latetype"))) {
                    dataEntity.set("tpv_listed", true);//是否上市
                } else {
                    dataEntity.set("tpv_listed", false);//是否上市
                }
                if (listed_types.isEmpty()) {
                    dataEntity.set("tpv_typeoflisting", null);//上市类型
                } else {
                    String jsonString = listed_types.toJSONString();
                    String stringWithoutBrackets = jsonString.replace("[", "").replace("]", "");
                    String finalString = stringWithoutBrackets.replace("\"", "");
                    dataEntity.set("tpv_typeoflisting", finalString);//上市类型
                }
                dataEntity.set("tpv_stocksymbol1", jsonObject.getString("stock_code"));//股票代码
                dataEntity.set("businessscope", jsonObject.getString("oprt_scope"));//经营范围
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            String employeeNumResponse = GetPublicSentimentList.employeeNum(name);
            JSONObject employeeNumJson = JSONObject.parseObject(employeeNumResponse);
            Map<String, JSONObject> sourceMap = new HashMap<>();
            if (null != employeeNumJson) {
                // 获取"hits"数组
                JSONArray hitsArray = employeeNumJson.getJSONObject("hits").getJSONArray("hits");
                if (!hitsArray.isEmpty()) {
                    // 遍历数组，提取每个元素中的"_source"字段
                    for (int i = 0; i < hitsArray.size(); i++) {
                        JSONObject hit = hitsArray.getJSONObject(i);
                        JSONObject source = hit.getJSONObject("_source");
                        sourceMap.put(source.getString("report_year"), source);
                    }
                }
                if (!sourceMap.isEmpty()) {
                    // 使用Stream找到最大的年份键
                    Optional<String> maxYearKey = sourceMap.keySet().stream()
                            .max(String::compareTo);
                    if (maxYearKey.isPresent()) {
                        // 获取对应的JSONObject
                        JSONObject valueForMaxYear = sourceMap.get(maxYearKey.get());
                        JSONObject socialInsurance = valueForMaxYear.getJSONObject("social_insurance");
                        Integer maximum = maxCount(socialInsurance);
                        if (null != maximum) {
                            dataEntity.set("tpv_insured", maximum.toString());//参保人数
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            if (StringUtils.isNotEmpty(bdSupplierName)) {
                responseSupplier = GetPublicSentimentList.enterPriseYear(name);
                JsonObject jsonObject = JsonParser.parseString(responseSupplier).getAsJsonObject();
                if (jsonObject != null) {
                    String tpvAptitude = jsonObject.getAsJsonObject("hits").getAsJsonArray("hits").get(0).getAsJsonObject().getAsJsonObject("_source").get("YEAR").getAsString();
                    if (null != tpvAptitude && !StringUtils.equals(tpvAptitude, "0")) {
                        bdSupplier.set("tpv_aptitude", "A级纳税人");//纳税人资质
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            if (StringUtils.isNotEmpty(bdSupplierName)) {
                Map<String, JSONObject> sourceMap = new HashMap<>();
                String employeeNumResponse = GetPublicSentimentList.employeeNum(name);
                JSONObject employeeNumJson = JSONObject.parseObject(employeeNumResponse);
                if (null != employeeNumJson) {
                    // 获取"hits"数组
                    JSONArray hitsArray = employeeNumJson.getJSONObject("hits").getJSONArray("hits");
                    if (!hitsArray.isEmpty()) {
                        // 遍历数组，提取每个元素中的"_source"字段
                        for (int i = 0; i < hitsArray.size(); i++) {
                            JSONObject hit = hitsArray.getJSONObject(i);
                            JSONObject source = hit.getJSONObject("_source");
                            sourceMap.put(source.getString("report_year"), source);
                        }
                    }
                    if (!sourceMap.isEmpty()) {
                        // 使用Stream找到最大的年份键
                        Optional<String> maxYearKey = sourceMap.keySet().stream()
                                .max(String::compareTo);
                        if (maxYearKey.isPresent()) {
                            // 获取对应的JSONObject
                            JSONObject valueForMaxYear = sourceMap.get(maxYearKey.get());
                            JSONObject socialInsurance = valueForMaxYear.getJSONObject("social_insurance");
                            Integer maximum = maxCount(socialInsurance);
                            if (null != maximum) {
                                bdSupplier.set("tpv_insured", maximum.toString());//参保人数
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            String responseYear = GetPublicSentimentList.enterPriseYear(name);
            JsonObject jsonObject = JsonParser.parseString(responseYear).getAsJsonObject();
            if (jsonObject != null) {
                String tpvAptitude = jsonObject.getAsJsonObject("hits").getAsJsonArray("hits").get(0).getAsJsonObject().getAsJsonObject("_source").get("YEAR").getAsString();
                if (null != tpvAptitude && !StringUtils.equals(tpvAptitude, "0")) {
                    dataEntity.set("tpv_aptitude", "A级纳税人");//纳税人资质
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        //对应供应商
        String userNameSupplier = RequestContext.get().getUserName();
        if (null != bdSupplier) {
            bdSupplier.set("tpv_qichachaman", userNameSupplier);//工商数据最新更新人
            Date currentDateSupplier = new Date();
            bdSupplier.set("tpv_qichachadate", currentDateSupplier);//工商数据最新更新日
            SaveServiceHelper.update(bdSupplier);
        }
        //客户
        String userName = RequestContext.get().getUserName();
        dataEntity.set("tpv_qichachaman", userName);//工商数据最新更新人
        Date currentDate = new Date();
        dataEntity.set("tpv_qichachadate", currentDate);//工商数据最新更新日
//        OperateOption option = YEM.getOption();
//        SaveServiceHelper.update(dataEntity);
//        OperationResult saveT =
//        OperationServiceHelper.executeOperate("save", "bd_customer",
//                new DynamicObject[]{dataEntity}, option);
        SaveServiceHelper.save(new DynamicObject[]{dataEntity});
    }

    public static Integer maxCount(JSONObject jsonObject) {
        String endowmentInsurance = jsonObject.getString("endowmentInsurance");//城镇职⼯基本养⽼保险⼈数
        endowmentInsurance = endowmentInsurance.replace("人", "");
        String medicalInsurance = jsonObject.getString("medicalInsurance");//职⼯基本医疗保险⼈数
        medicalInsurance = medicalInsurance.replace("人", "");
        String maternityInsurance = jsonObject.getString("maternityInsurance");//⽣育保险⼈数
        maternityInsurance = maternityInsurance.replace("人", "");
        String unemploymentInsurance = jsonObject.getString("unemploymentInsurance");//失业保险⼈数
        unemploymentInsurance = unemploymentInsurance.replace("人", "");
        String employmentInjuryInsurance = jsonObject.getString("employmentInjuryInsurance");//⼯商保险
        employmentInjuryInsurance = employmentInjuryInsurance.replace("人", "");
        try {
            // 假设字符串只包含数字
            int endowmentInsuranceCount = Integer.parseInt(endowmentInsurance);
            int medicalInsuranceCount = Integer.parseInt(medicalInsurance);
            int maternityInsuranceCount = Integer.parseInt(maternityInsurance);
            int unemploymentInsuranceCount = Integer.parseInt(unemploymentInsurance);
            int employmentInjuryInsuranceCount = Integer.parseInt(employmentInjuryInsurance);

            int maxCount = Math.max(endowmentInsuranceCount,
                    Math.max(medicalInsuranceCount,
                            Math.max(maternityInsuranceCount,
                                    Math.max(unemploymentInsuranceCount, employmentInjuryInsuranceCount))));
            return maxCount;
        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
        return null;
    }
}
