package tpv.sys.basedata.java.opplugin.op;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.itopview.wm.utils.YEM;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import tpv.tpv_wm.tpv_iscb.java.api.ant.GetPublicSentimentList;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * @ClassName: BdSupplierBusinesDataOp
 * @Description: 更新工商数据（供应商）
 * @author: zbh
 * @date: 2024-07-03
 */
public class BdSupplierBusinesDataOp extends AbstractOperationServicePlugIn {
    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] entities = e.getDataEntities();
        String itemKey = e.getOperationKey();
        for (DynamicObject entity : entities) {
            BusinesData(entity);
        }
    }

    public void BusinesData(DynamicObject enty) {

        DynamicObject dataEntity = BusinessDataServiceHelper.loadSingle(enty.getPkValue(), "bd_supplier");
        //更新工商数据
        String number = dataEntity.getString("number");//供应商编码
        String tpv_sourcebillno = dataEntity.getString("tpv_sourcebillno");//来源单据编码
        DynamicObject bdCustomer = null;
        if (StringUtils.isEmpty(tpv_sourcebillno)) {
            //查询所关联的客户
            bdCustomer = BusinessDataServiceHelper.loadSingle("bd_customer", "tpv_sourcebillno,id,name,societycreditcode,tpv_qichachaman,tpv_qichachadate",
                    new QFilter("tpv_sourcebillno", QCP.equals, number).toArray());
        } else {
            bdCustomer = BusinessDataServiceHelper.loadSingle("bd_customer", "number,id,name,societycreditcode,tpv_qichachaman,tpv_qichachadate",
                    new QFilter("number", QCP.equals, tpv_sourcebillno).toArray());
        }
        //供应商名称
        String name = dataEntity.getLocaleString("name").getLocaleValue();
        //统一社会信用代码 NEW: 如果统一社会信用代码有值则为入参，否则以名称为入参
        String societycreditcode = dataEntity.getString("societycreditcode");
        if (YEM.isNotEmpty(societycreditcode)) {
            name = societycreditcode;
        }
        //客户名称/统一社会信用代码  NEW: 如果统一社会信用代码有值则为入参，否则以名称为入参
        String bdCustomerName = null;
        if (YEM.isNotEmpty(bdCustomer)) {
            bdCustomer = BusinessDataServiceHelper.loadSingle(bdCustomer.getPkValue(),
                    bdCustomer.getDynamicObjectType().getName());
            String bdCustomerCreditcode = bdCustomer.getString("societycreditcode");
            if (YEM.isNotEmpty(bdCustomerCreditcode)) {
                bdCustomerName = bdCustomerCreditcode;
            } else {
                bdCustomerName = bdCustomer.getLocaleString("name").getLocaleValue();
            }

        }
        String responseCustomer = null;//客户
        JSONObject jsonObjectCustomer = new JSONObject();//客户
        try {
            if (StringUtils.isNotEmpty(bdCustomerName)) {
                //有对应的客户
                responseCustomer = GetPublicSentimentList.enterPrise(bdCustomerName);
                jsonObjectCustomer = JSONObject.parseObject(responseCustomer);
            }
            if (!jsonObjectCustomer.isEmpty()) {
                //客户赋值
                JSONArray constituentTags = jsonObjectCustomer.getJSONArray("constituent_tags");//企业性质
                JSONArray listed_types = jsonObjectCustomer.getJSONArray("listed_types");//上市类型
                if (!constituentTags.isEmpty()) {
//                        String tpv_enterprisenature = constituentTags.toJSONString();
//                        String stringWithoutBrackets = tpv_enterprisenature.replace("[", "").replace("]", "");
//                        String finalString = stringWithoutBrackets.replace("\"", "");

                    String finalString = findShortestString(constituentTags);
                    bdCustomer.set("tpv_enterprisenature", finalString);//企业性质
                    if (StringUtils.equals(finalString, "民营控股")) {
                        bdCustomer.set("tpv_natureoftheenterpr2", "MY");//企业性质
                    } else if (StringUtils.equals(finalString, "")) {
                        bdCustomer.set("tpv_natureoftheenterpr2", null);//企业性质
                    } else {
                        bdCustomer.set("tpv_natureoftheenterpr2", "GY");//企业性质
                    }
                }
                bdCustomer.set("artificialperson", jsonObjectCustomer.getString("legal_representative"));//法人代表
                bdCustomer.set("tpv_industrialyphon", jsonObjectCustomer.getString("org_tel"));//工商登记电话
                String curegcapital = jsonObjectCustomer.getString("register_capital_currency");
                if (StringUtils.equals("人民币", curegcapital)) {
                    //人民币
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, "人民币").toArray());
                    bdCustomer.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                } else if (StringUtils.equals("港元", curegcapital)) {
                    //港元
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, curegcapital).toArray());
                    bdCustomer.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                } else {
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, curegcapital).toArray());
                    if (null != bd_currency) {
                        bdCustomer.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                    }
                }
                bdCustomer.set("societycreditcode", jsonObjectCustomer.getString("uc_code"));//统一社会信用代码
                bdCustomer.set("tx_register_no", jsonObjectCustomer.getString("uc_code"));//纳税人识别号
                bdCustomer.set("regcapital", jsonObjectCustomer.getString("register_capital_value"));//注册资本
                bdCustomer.set("businessterm", jsonObjectCustomer.getString("oprt_end_date"));//营业期限
                bdCustomer.set("establishdate", jsonObjectCustomer.getDate("register_date"));//成立日期
                bdCustomer.set("bizpartner_address", jsonObjectCustomer.getString("register_place"));//注册地址
                bdCustomer.set("tpv_registrationstatus", jsonObjectCustomer.getString("org_state"));//登记状态
                bdCustomer.set("tpv_paidupcapital", jsonObjectCustomer.getString("org_rec_cptl_rmb"));//实缴资本
                bdCustomer.set("tpv_businesstype", jsonObjectCustomer.getString("org_type"));//企业类型
                bdCustomer.set("tpv_insured", jsonObjectCustomer.getString("employee_num"));//参保人数
                dataEntity.set("tpv_province", jsonObjectCustomer.get("register_province"));//省
                dataEntity.set("tpv_city", jsonObjectCustomer.get("register_city"));//市
                dataEntity.set("tpv_county", jsonObjectCustomer.get("register_county"));//县
                if (StringUtils.isNotEmpty(jsonObjectCustomer.getString("latetype"))) {
                    dataEntity.set("tpv_listed", true);//是否上市
                    bdCustomer.set("tpv_listed", true);//是否上市
                } else {
                    bdCustomer.set("tpv_listed", false);//是否上市
                }
                if (listed_types.isEmpty()) {
                    bdCustomer.set("tpv_typeoflisting", null);//上市类型
                } else {
                    String jsonString = listed_types.toJSONString();
                    String stringWithoutBrackets = jsonString.replace("[", "").replace("]", "");
                    String finalString = stringWithoutBrackets.replace("\"", "");
                    bdCustomer.set("tpv_typeoflisting", finalString);//上市类型
                }
                bdCustomer.set("tpv_stocksymbol1", jsonObjectCustomer.getString("stock_code"));//股票代码
                bdCustomer.set("businessscope", jsonObjectCustomer.getString("oprt_scope"));//经营范围
                SaveServiceHelper.update(bdCustomer);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            String response = GetPublicSentimentList.enterPrise(name);
            JSONObject jsonObject = JSONObject.parseObject(response);
            if (null != jsonObject) {
                JSONArray constituentTags = jsonObject.getJSONArray("constituent_tags");//企业性质
                JSONArray listed_types = jsonObject.getJSONArray("listed_types");//上市类型
                if (!constituentTags.isEmpty()) {
//                        String tpv_enterprisenature = constituentTags.toJSONString();
//                        String stringWithoutBrackets = tpv_enterprisenature.replace("[", "").replace("]", "");
//                        String finalString = stringWithoutBrackets.replace("\"", "");

                    String finalString = findShortestString(constituentTags);
                    dataEntity.set("tpv_enterprisenature", finalString);//企业性质
                    if (StringUtils.equals(finalString, "民营控股")) {
                        dataEntity.set("tpv_natureoftheenterpr2", "MY");//企业性质
                    } else if (StringUtils.equals(finalString, "")) {
                        dataEntity.set("tpv_natureoftheenterpr2", null);//企业性质
                    } else {
                        dataEntity.set("tpv_natureoftheenterpr2", "GY");//企业性质
                    }
                }
                dataEntity.set("artificialperson", jsonObject.getString("legal_representative"));//法人代表
                dataEntity.set("tpv_industrialyphon", jsonObject.getString("org_tel"));//工商登记电话
                String curegcapital = jsonObject.getString("register_capital_currency");
                if (StringUtils.equals("人民币", curegcapital)) {
                    //人民币
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, "人民币").toArray());
                    dataEntity.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                } else if (StringUtils.equals("港元", curegcapital)) {
                    //港元
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, curegcapital).toArray());
                    dataEntity.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                } else {
                    DynamicObject bd_currency = BusinessDataServiceHelper.loadSingle("bd_currency", "id",
                            new QFilter("name", QCP.equals, curegcapital).toArray());
                    if (null != bd_currency) {
                        dataEntity.set("curegcapital", bd_currency.getLong("id"));//注册资本币种
                    }
                }
                dataEntity.set("societycreditcode", jsonObject.getString("uc_code"));
                dataEntity.set("tx_register_no", jsonObject.getString("uc_code"));
                dataEntity.set("regcapital", jsonObject.getString("register_capital_value"));
                dataEntity.set("businessterm", jsonObject.getString("oprt_end_date"));
                dataEntity.set("establishdate", jsonObject.getDate("register_date"));
                dataEntity.set("bizpartner_address", jsonObject.getString("register_place"));
                dataEntity.set("tpv_registrationstatus", jsonObject.getString("org_state"));
                dataEntity.set("tpv_paidupcapital", jsonObject.getString("org_rec_cptl_rmb"));
                dataEntity.set("tpv_businesstype", jsonObject.getString("org_type"));//企业类型
                dataEntity.set("tpv_insured", jsonObject.getString("employee_num"));//参保人数
                dataEntity.set("tpv_province", jsonObject.get("register_province"));//省
                dataEntity.set("tpv_city", jsonObject.get("register_city"));//市
                dataEntity.set("tpv_county", jsonObject.get("register_county"));//县
                if (StringUtils.isNotEmpty(jsonObject.getString("latetype"))) {
                    dataEntity.set("tpv_listed", true);//是否上市
                } else {
                    dataEntity.set("tpv_listed", false);//是否上市
                }
                if (listed_types.isEmpty()) {
                    dataEntity.set("tpv_typeoflisting", null);//上市类型
                } else {
                    String jsonString = listed_types.toJSONString();
                    String stringWithoutBrackets = jsonString.replace("[", "").replace("]", "");
                    String finalString = stringWithoutBrackets.replace("\"", "");
                    dataEntity.set("tpv_typeoflisting", finalString);
                }
                dataEntity.set("tpv_stocksymbol1", jsonObject.getString("stock_code"));
                dataEntity.set("businessscope", jsonObject.getString("oprt_scope"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            String employeeNumResponse = GetPublicSentimentList.employeeNum(name);
            JSONObject employeeNumJson = JSONObject.parseObject(employeeNumResponse);
            Map<String, JSONObject> sourceMap = new HashMap<>();
            if (null != employeeNumJson) {
                // 获取"hits"数组
                JSONArray hitsArray = employeeNumJson.getJSONObject("hits").getJSONArray("hits");
                if (!hitsArray.isEmpty()) {
                    // 遍历数组，提取每个元素中的"_source"字段
                    for (int i = 0; i < hitsArray.size(); i++) {
                        JSONObject hit = hitsArray.getJSONObject(i);
                        JSONObject source = hit.getJSONObject("_source");
                        sourceMap.put(source.getString("report_year"), source);
                    }
                }
                if (!sourceMap.isEmpty()) {
                    // 使用Stream找到最大的年份键
                    Optional<String> maxYearKey = sourceMap.keySet().stream()
                            .max(String::compareTo);
                    if (maxYearKey.isPresent()) {
                        // 获取对应的JSONObject
                        JSONObject valueForMaxYear = sourceMap.get(maxYearKey.get());
                        JSONObject socialInsurance = valueForMaxYear.getJSONObject("social_insurance");
                        Integer maximum = maxCount(socialInsurance);
                        if (null != maximum) {
                            dataEntity.set("tpv_insured", maximum.toString());//参保人数
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            if (StringUtils.isNotEmpty(bdCustomerName)) {
                responseCustomer = GetPublicSentimentList.enterPriseYear(name);
                JsonObject jsonObject = JsonParser.parseString(responseCustomer).getAsJsonObject();
                if (jsonObject != null) {
                    String tpvAptitude = jsonObject.getAsJsonObject("hits").getAsJsonArray("hits").get(0).getAsJsonObject().getAsJsonObject("_source").get("YEAR").getAsString();
                    if (null != tpvAptitude && !StringUtils.equals(tpvAptitude, "0")) {
                        bdCustomer.set("tpv_aptitude", "A级纳税人");//纳税人资质
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            if (StringUtils.isNotEmpty(bdCustomerName)) {
                String employeeNumResponse = GetPublicSentimentList.employeeNum(name);
                JSONObject employeeNumJson = JSONObject.parseObject(employeeNumResponse);
                Map<String, JSONObject> sourceMap = new HashMap<>();
                if (null != employeeNumJson) {
                    // 获取"hits"数组
                    JSONArray hitsArray = employeeNumJson.getJSONObject("hits").getJSONArray("hits");
                    if (!hitsArray.isEmpty()) {
                        // 遍历数组，提取每个元素中的"_source"字段
                        for (int i = 0; i < hitsArray.size(); i++) {
                            JSONObject hit = hitsArray.getJSONObject(i);
                            JSONObject source = hit.getJSONObject("_source");
                            sourceMap.put(source.getString("report_year"), source);
                        }
                    }
                    if (!sourceMap.isEmpty()) {
                        // 使用Stream找到最大的年份键
                        Optional<String> maxYearKey = sourceMap.keySet().stream()
                                .max(String::compareTo);
                        if (maxYearKey.isPresent()) {
                            // 获取对应的JSONObject
                            JSONObject valueForMaxYear = sourceMap.get(maxYearKey.get());
                            JSONObject socialInsurance = valueForMaxYear.getJSONObject("social_insurance");
                            Integer maximum = maxCount(socialInsurance);
                            if (null != maximum) {
                                bdCustomer.set("tpv_insured", maximum.toString());//参保人数
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            String responseYear = GetPublicSentimentList.enterPriseYear(name);
            JsonObject jsonObject = JsonParser.parseString(responseYear).getAsJsonObject();
            if (jsonObject != null) {
                String tpvAptitude = jsonObject.getAsJsonObject("hits").getAsJsonArray("hits").get(0).getAsJsonObject().getAsJsonObject("_source").get("YEAR").getAsString();
                if (null != tpvAptitude && !StringUtils.equals(tpvAptitude, "0")) {
                    dataEntity.set("tpv_aptitude", "A级纳税人");//纳税人资质
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        //对应客户
        String userNameCustomer = RequestContext.get().getUserName();
        if (null != bdCustomer) {
            bdCustomer.set("tpv_qichachaman", userNameCustomer);//工商数据最新更新人
            Date currentDateCustomer = new Date();
            bdCustomer.set("tpv_qichachadate", currentDateCustomer);//工商数据最新更新日
            SaveServiceHelper.update(bdCustomer);
        }
        //供应商
        String userName = RequestContext.get().getUserName();
        dataEntity.set("tpv_qichachaman", userName);
        Date currentDate = new Date();
        dataEntity.set("tpv_qichachadate", currentDate);
//        OperateOption option = YEM.getOption();
        //写入数据库
//        SaveServiceHelper.update(dataEntity);
        //调用整单刷新功能
//        OperationResult saveT =
//        OperationServiceHelper.executeOperate("save", "bd_supplier",
//                new DynamicObject[]{dataEntity}, option);
        SaveServiceHelper.save(new DynamicObject[]{dataEntity});
    }

    public Integer maxCount(JSONObject jsonObject) {
        String endowmentInsurance = jsonObject.getString("endowmentInsurance");//城镇职⼯基本养⽼保险⼈数
        endowmentInsurance = endowmentInsurance.replace("人", "");
        String medicalInsurance = jsonObject.getString("medicalInsurance");//职⼯基本医疗保险⼈数
        medicalInsurance = medicalInsurance.replace("人", "");
        String maternityInsurance = jsonObject.getString("maternityInsurance");//⽣育保险⼈数
        maternityInsurance = maternityInsurance.replace("人", "");
        String unemploymentInsurance = jsonObject.getString("unemploymentInsurance");//失业保险⼈数
        unemploymentInsurance = unemploymentInsurance.replace("人", "");
        String employmentInjuryInsurance = jsonObject.getString("employmentInjuryInsurance");//⼯商保险
        employmentInjuryInsurance = employmentInjuryInsurance.replace("人", "");
        try {
            // 假设字符串只包含数字
            int endowmentInsuranceCount = Integer.parseInt(endowmentInsurance);
            int medicalInsuranceCount = Integer.parseInt(medicalInsurance);
            int maternityInsuranceCount = Integer.parseInt(maternityInsurance);
            int unemploymentInsuranceCount = Integer.parseInt(unemploymentInsurance);
            int employmentInjuryInsuranceCount = Integer.parseInt(employmentInjuryInsurance);

            int maxCount = Math.max(endowmentInsuranceCount,
                    Math.max(medicalInsuranceCount,
                            Math.max(maternityInsuranceCount,
                                    Math.max(unemploymentInsuranceCount, employmentInjuryInsuranceCount))));
            return maxCount;
        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static String findShortestString(JSONArray jsonArray) {
        String shortest = null;
        int shortestLength = Integer.MAX_VALUE;
        for (int i = 0; i < jsonArray.size(); i++) {
            String str = jsonArray.getString(i);
            if (str.length() <= shortestLength) {
                shortest = str;
                shortestLength = str.length();
            }
        }

        return shortest;
    }
}
