package tpv.sys.basedata.java.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.db.DB;
import kd.bos.db.DBRoute;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author l jw
 * @date 2023/9/6 14:51
 * @className ClientServicePlugIn
 * @description 客户保存效验是否重名
 */
public class ClientSaveOp extends AbstractOperationServicePlugIn {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("name");
        fieldKeys.add("societycreditcode");
        fieldKeys.add("tpv_sourcebillid");
        fieldKeys.add("tpv_specname");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] bills = this.getDataEntities();
                if ("save".equals(this.getOperateKey()) || "submit".equals(this.getOperateKey())) {
                    for (int i = 0; i < bills.length; ++i) {
                        ExtendedDataEntity bill = bills[i];
                        //监控按钮保存
                        this.clickSave(bill);
                    }
                }
            }

            private void clickSave(ExtendedDataEntity dataEntities) {
                DynamicObject entity = dataEntities.getDataEntity();
                String name = entity.getString("name");//名称
                String societycreditcode = entity.getString("societycreditcode");//统一社会信用代码
                long tpv_sourcebillid = entity.getLong("tpv_sourcebillid");
                if (YEM.isNotEmpty(name)) {
                    String removedName = removeSpecialCharacters(name);
                    DynamicObject[] vatinvoiceSup = BusinessDataServiceHelper.load("bd_supplier", "id,billno,name"
                            , new QFilter[]{new QFilter("tpv_specname", "=", removedName)});
                    DynamicObject[] bdCustomer = BusinessDataServiceHelper.load("bd_customer", "id,billno,name",
                            new QFilter[]{new QFilter("tpv_specname", "=", removedName), new QFilter("id", QCP.not_equals, entity.getLong("id"))});

                    if (vatinvoiceSup.length > 0 && tpv_sourcebillid <= 0l) {
                        this.addMessage(dataEntities, "供应商中有相同的名称，请修改后保存！");
                    }
                    if (bdCustomer.length > 0) {
                        this.addMessage(dataEntities, "客户中有相同的名称，请修改后保存！");
                    }
                }
                if (YEM.isNotEmpty(societycreditcode)) {
                    QFilter qFilterTwo = new QFilter("societycreditcode", QCP.equals, societycreditcode);//查询相同的
                    DynamicObject[] vatinvoiceCodeSup = BusinessDataServiceHelper.load("bd_supplier", "id,billno,name,societycreditcode"
                            , new QFilter[]{qFilterTwo}, null);
                    DynamicObject[] vatinvoiceCodeCus = BusinessDataServiceHelper.load("bd_customer", "id,billno,name,societycreditcode"
                            , new QFilter[]{qFilterTwo, new QFilter("id", QCP.not_equals, entity.getLong("id"))}, null);
                    if (vatinvoiceCodeSup.length > 0 && tpv_sourcebillid <= 0l) {
                        this.addMessage(dataEntities, "供应商中有相同的统一社会信用代码，请修改后保存！");
                    }
                    if (vatinvoiceCodeCus.length > 0) {
                        this.addMessage(dataEntities, "客户中有相同的统一社会信用代码，请修改后保存！");
                    }
                }
            }

        });
    }

    public String removeSpecialCharacters(String text) {
        // 匹配除了英文字母（不分大小写）、数字和汉字以外的所有字符，并忽略大小写
        String regex = "[^a-zA-Z0-9\\u4e00-\\u9fa5]";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(text);
        return matcher.replaceAll("");
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        DynamicObject[] entities = e.getDataEntities();
        for (DynamicObject entity : entities) {
            String name = entity.getString("name");//名称
            if (YEM.isNotEmpty(name)) {
                String removedName = removeSpecialCharacters(name);
                entity.set("tpv_specname", removedName);
            }
        }
    }
}
