package tpv.sys.basedata.java.opplugin.op;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 *@Description: 同步单据转基础资料中 famasterid  = fid
 *@Project: zhck
 *@Package: tpv.sys.basedata.java.opplugin.op.SynchronizefidSaveOp
 *@Date: 2024/05/08 14:44
 *@Author: Heisenberg Wenbei He
 *@Version: 1.0
 */
public class SynchronizefidSaveOp extends AbstractOperationServicePlugIn {
    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        String operationKey = e.getOperationKey();
        if (!("save".equals(operationKey) || "submit".equals(operationKey))) {
            return;
        }
        List<DynamicObject> collect = Arrays.stream(e.getDataEntities())
                .filter(single -> this.getSign(single.getDataEntityType().getName()).isPresent())
                .map(single ->
                        BusinessDataServiceHelper.loadSingle
                                (single.getPkValue(), this.getSign(single.getDataEntityType().getName()).get()))
                .filter(Objects::nonNull)
                .peek(single -> single.set("masterid", single.getPkValue()))
                .collect(Collectors.toList());
        SaveServiceHelper.save(collect.toArray(new DynamicObject[0]));
    }

    private Optional<String> getSign(String billSign) {
        // 获取 单据转基础资料标识映射 表.
        // Retrieve Documnet to Basic Information Identification Mapping Table.
        DynamicObject loadSingle = BusinessDataServiceHelper.loadSingle("tpv_synchronizefid"
                , new QFilter[]{new QFilter("billno", "=", "SynchronizeID-001")});
        DynamicObjectCollection synchronizentry = loadSingle.getDynamicObjectCollection("tpv_synchronizentry");
        return synchronizentry.stream()
                .filter(row -> billSign.equals(row.getString("tpv_tablename")))
                .map(row -> row.getString("tpv_bdataname"))
                .findFirst();
    }
}
