package tpv.tmc.lc.java.formplugin.dynamic;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.form.control.Control;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import tpv.tmc.lc.java.opplugin.op.CapitalLimitServiceOp;

import java.util.EventObject;
import java.util.Optional;

/**
 * @Description: 【闭卷人】、【闭卷时间】、【操作时间】字段锁定，闭卷确认后回填
 * @Project: zhck
 * @Package: com.itopview.wm.sc.formplugin.dynamic.BackFillChangeCreditEdit
 * @Date: 2023/11/23 15:44
 * @Author: Heisenberg Wenbei He
 * @Version: 1.0
 */
public class BackFillChangeCreditEdit extends AbstractFormPlugin {

    private static final String OPERATE = "operate";
    private static final String ENTITY = "entity"; // 表单标识

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        addClickListeners("btnok", "btncancel");
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        String operate = this.getView().getFormShowParameter().getCustomParam(OPERATE);
        if ("close".equals(operate)) {
            this.getModel().setValue("tpv_businesstype", "A");
        } else if ("repeal".equals(operate)) {
            this.getModel().setValue("tpv_businesstype", "B");
        }
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        Control source = (Control) evt.getSource();
        Long billId = (Long) this.getModel().getValue("id");
        String entity = this.getView().getFormShowParameter().getCustomParam(ENTITY);
        if ("btnok".equals(source.getKey())) {
            switch (entity) {
                case "lc_lettercredit":
                    synchronize(billId);
                    String operate = (String) this.getView().getFormShowParameter().getCustomParam(OPERATE);
                    CapitalLimitServiceOp capitalLimitServiceOp = new CapitalLimitServiceOp();
                    capitalLimitServiceOp.closeCredit(billId, operate);
                    break;
                case "lc_receipt":
                    updateInfo(billId);
                    break;
                default:
                    break;
            }
//            确定后向资金额度底表插入数据

        }
    }

    private void synchronize(Long billId) {
        if (billId != null) {
            String operate = this.getView().getFormShowParameter().getCustomParam(OPERATE);
            DynamicObject lcReceipt = BusinessDataServiceHelper.loadSingle(billId, "lc_lettercredit");
            Optional.ofNullable(lcReceipt)
                    .ifPresent(single -> {
                        if ("close".equals(operate)) {
                            single.set("tpv_closer", this.getModel().getValue("tpv_closer")); // 闭卷人
                            single.set("tpv_closetime", this.getModel().getValue("tpv_closetime")); //
                        } else if ("repeal".equals(operate)) {
                            single.set("tpv_repealer", this.getModel().getValue("tpv_withdrawer")); // 撤证人
                            single.set("tpv_repealdate", this.getModel().getValue("tpv_withdrawtime")); // 撤证时间
                        }
                        single.set("tpv_operatedate", this.getModel().getValue("tpv_operatedate")); // 操作时间
                        SaveServiceHelper.save(new DynamicObject[]{single});
                    });
        }
    }

    private void updateInfo(Long billId) {
        if (billId != null) {
        DynamicObject lcReceipt = BusinessDataServiceHelper.loadSingle(billId, "lc_receipt");
        Optional.ofNullable(lcReceipt)
                .ifPresent(single -> {
                    single.set("tpv_closer", this.getModel().getValue("tpv_closer")); // 闭卷人
                    single.set("tpv_closetime", this.getModel().getValue("tpv_closetime")); //
                    single.set("tpv_operatedate", this.getModel().getValue("tpv_operatedate")); // 操作时间
                    SaveServiceHelper.save(new DynamicObject[]{single});
                });
    }}
}
