package tpv.tmc.lc.java.formplugin.dynamic;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.control.Control;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.EventObject;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicLong;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhck
 * @Package: com.itopview.wm.sc.formplugin.dynamic.PopupContractFormPlugin
 * @Date: 18:39 2023/11/06
 * @Description: 开证处理合同明细分录 绑定《开证登记》
 * 可选择同公司项下的已母证信用证号认领的收证 开证类型为“母子证-前背”，该字段非必选，
 * * 选择“母证信用证号”，自动带出对应收证的【开证人】
 * @Version: 1.0
 */
public class PopupContractEdit extends AbstractFormPlugin {

    private static final String DATA_ENTITY = "tpv_entryentity1";

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners("btncancel", "btnok"); // 监听确认和取消按钮
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        this.init();
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        Control source = (Control) evt.getSource();
        // 点击确认按钮
        if (StringUtils.equals(source.getKey(), "btnok")) {
            EntryGrid entryGrid = this.getControl(DATA_ENTITY);
            int[] selectRows = entryGrid.getSelectRows();
            DynamicObjectCollection dataEntity = this.getModel().getEntryEntity(DATA_ENTITY);
            Optional.ofNullable(selectRows)
                    .filter(selectitems -> selectitems.length > 0)
                    .ifPresent(selectitems -> this.getView().returnDataToParent(dataEntity.get(selectitems[0])));
        } else if (StringUtils.equals(source.getKey(), "btncancel")) {
            // 被点击控件为取消则设置返回值为空并关闭页面（在页面关闭回调方法中必须验证返回值不为空，否则会报空指针）
            this.getView().returnDataToParent(null);
        }
        this.getView().close();
    }

    private void init() {
        Map<String, Object> customParams = this.getView().getFormShowParameter().getCustomParams();
        Object department = customParams.get("department");
        Object org = customParams.get("org");
        DynamicObject[] lettercredit = BusinessDataServiceHelper.load("lc_lettercredit",
                "tpv_contractdetail,tpv_purnumbers,tpv_kzappmon,tpv_tjtype,tpv_combinationnum," +
                        "tpv_naturetrade,tpv_businesstype,tpv_purmoney,tpv_purcurrency,tpv_purdepartment,tpv_pruper," +
                        "tpv_appusdmon,tpv_appusdrate,tpv_apprmbmon,tpv_department,creditno",
                new QFilter[]{new QFilter("billstatus", QCP.equals, "C")
                        , new QFilter("tpv_department", QCP.equals, department)
                        , new QFilter("org", QCP.equals, org)});
        DynamicObjectCollection currentForm = this.getModel().getDataEntity(true)
                .getDynamicObjectCollection(DATA_ENTITY);
        AtomicLong pkValue = new AtomicLong();
        Optional.ofNullable(lettercredit)
                .ifPresent(credit -> Arrays.stream(credit)
                        .map(single -> {
                            pkValue.set((long) single.getPkValue());
                            return single.getDynamicObjectCollection("tpv_contractdetail");
                        })
                        .filter(collection -> !collection.isEmpty())
                        .filter(group -> group.get(0).getDynamicObject("tpv_purcurrency") != null)
                        .forEach(entry -> {
                            for (DynamicObject currentRow : entry) {
                                DynamicObject newRow = currentForm.addNew();
                                // 源单id
                                newRow.set("tpv_sourcebill", pkValue.get());
                                DynamicObject lcLettercreditF7 = BusinessDataServiceHelper.loadSingle(pkValue.get(), "lc_lettercredit_f7");
                                newRow.set("tpv_lcnum", lcLettercreditF7);
                                // 币别
                                DynamicObject purcurrency = currentRow.getDynamicObject("tpv_purcurrency");
                                Optional.ofNullable(purcurrency)
                                        .ifPresent(currency -> newRow.set("tpv_purcurrency", currency));
                                // 组合号
                                DynamicObject combinationnum = currentRow.getDynamicObject("tpv_combinationnum");
                                Optional.ofNullable(combinationnum)
                                        .ifPresent(conbinnum -> newRow.set("tpv_combinationnum", conbinnum));
                                // 业务类型
                                DynamicObject businesstype = currentRow.getDynamicObject("tpv_businesstype");
                                Optional.ofNullable(businesstype)
                                        .ifPresent(type -> newRow.set("tpv_businesstype", type));
                                // 贸易性质
                                DynamicObject naturetrade = currentRow.getDynamicObject("tpv_naturetrade");
                                Optional.ofNullable(naturetrade)
                                        .ifPresent(qualiity -> newRow.set("tpv_naturetrade", qualiity));
                                // 合同部门
                                DynamicObject purdepartment = currentRow.getDynamicObject("tpv_purdepartment");
                                Optional.ofNullable(purdepartment)
                                        .ifPresent(purdepart -> newRow.set("tpv_purdepartment", purdepart));
                                // 合同业务员
                                DynamicObject pruper = currentRow.getDynamicObject("tpv_pruper");
                                Optional.ofNullable(pruper)
                                        .ifPresent(person -> newRow.set("tpv_pruper", person));
                                newRow.set("tpv_purmoney", currentRow.getBigDecimal("tpv_purmoney"));

                                newRow.set("tpv_appusdmon", currentRow.getBigDecimal("tpv_purmoney")); // 开证申请折USD金额
                                newRow.set("tpv_appusdrate", currentRow.getBigDecimal("tpv_purmoney")); // 开证申请折USD汇率
                                newRow.set("tpv_apprmbmon", currentRow.getBigDecimal("tpv_purmoney")); // 开证申请折RMB金额
                                newRow.set("tpv_contractid", currentRow.getString("tpv_purnumbers"));
                                newRow.set("tpv_kzappmon", currentRow.getBigDecimal("tpv_kzappmon"));
                            }
                        }));
        this.getView().updateView(DATA_ENTITY);
    }
}
