package tpv.tmc.lc.java.formplugin.dynamic;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.control.Control;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.EventObject;
import java.util.Map;
import java.util.Optional;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhck
 * @Package: com.itopview.wm.sc.formplugin.dynamic.ShowCreditTypeEdit
 * @Date: 2023/11/13 10:46
 * @Description: 手工选择该信用证向下的{分录-即期远期列表}，单选
 * @Version: 1.0
 */
public class ShowCreditTypeEdit extends AbstractFormPlugin {
    private static final String SPOT_FORWARD = "tpv_spotforward"; // 即期远期列表

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners("btncancel", "btnok"); // 监听确认和取消按钮
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        this.init();
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        Control source = (Control) evt.getSource();
        // 点击确认按钮
        if (StringUtils.equals(source.getKey(), "btnok")) {
            EntryGrid entryGrid = this.getControl(SPOT_FORWARD);
            int[] selectRows = entryGrid.getSelectRows();
            DynamicObjectCollection dataEntity = this.getModel().getEntryEntity(SPOT_FORWARD);
            Optional.ofNullable(selectRows)
                    .filter(selectItems -> selectItems.length > 0)
                    .ifPresent(selectItems -> this.getView().returnDataToParent(dataEntity.get(selectItems[0])));
            this.getView().close();
        } else if (StringUtils.equals(source.getKey(), "btncancel")) {
            // 被点击控件为取消则设置返回值为空并关闭页面（在页面关闭回调方法中必须验证返回值不为空，否则会报空指针）
            this.getView().returnDataToParent(null);
            this.getView().close();
        }
    }

    private void init() {
        Map<String, Object> customParams = this.getView().getFormShowParameter().getCustomParams();
        String billId = (String) customParams.get("bill");
        DynamicObject bill = BusinessDataServiceHelper.loadSingle("lc_lettercredit",new QFilter[]{new QFilter("creditno","=",billId)});
        if (bill != null) {
            DynamicObjectCollection collection = bill.getDynamicObjectCollection(SPOT_FORWARD);
            DynamicObjectCollection currentForm = this.getModel().getDataEntity(true).getDynamicObjectCollection(SPOT_FORWARD);
            if (collection != null) {
                for (DynamicObject currentRow : collection) {
                    DynamicObject newRow = currentForm.addNew();
                    // 信用证类型
                    String creditType = currentRow.getString("tpv_letcre");
                    Optional.ofNullable(creditType)
                            .ifPresent(type -> newRow.set("tpv_letcre", type));
                    // 远期天数
                    BigDecimal furlook = currentRow.getBigDecimal("tpv_furlook");
                    Optional.ofNullable(furlook)
                            .ifPresent(days -> newRow.set("tpv_furlook", days));
                    // 远期天数起算方式
                    String furlookhk = currentRow.getString("tpv_furlookhk");
                    Optional.ofNullable(furlookhk)
                            .ifPresent(calculatorWays -> newRow.set("tpv_furlookhk", calculatorWays));

                    String wedoor = currentRow.getString("tpv_wedoor");
                    Optional.ofNullable(wedoor)
                            .ifPresent(door -> newRow.set("tpv_wedoor", door));
                }
            }
        }
    }
}
