package tpv.tmc.lc.java.formplugin.form;

import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.property.TextProp;
import kd.bos.form.CloseCallBack;
import kd.bos.form.IFormView;
import kd.bos.form.ShowFormHelper;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.events.BeforeItemClickEvent;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.control.events.RowClickEvent;
import kd.bos.form.control.events.RowClickEventListener;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.RefBillEdit;
import kd.bos.form.field.TextEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;
import tpv.tpv_wm.tpv_dt.java.api.client.DHLClient;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static javax.swing.UIManager.getString;

/**
 * @ClassName: bizpresFormPlugin
 * @Description: 交单申请，处理功能
 * @author: zbh
 * @date: 2023-11-17
 */
public class BizPresFormPlugin extends AbstractBillPlugIn implements RowClickEventListener, BeforeF7SelectListener {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addItemClickListeners("tpv_advcontoolbarap");
        EntryGrid entryGrid = this.getView().getControl("tpv_kdmentinfor");
        if (entryGrid != null) {
            entryGrid.addRowClickListener(this);
        }
        ((BasedataEdit) this.getControl("tpv_people")).addBeforeF7SelectListener(this);
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String name = e.getProperty().getName();
        String entityId = this.getModel().getDataEntityType().getName();
        switch (name) {
            case "tpv_people":
                operatorF7(e, name);
                break;
        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent e) {
        super.closedCallBack(e);
        String actionId = e.getActionId();
        switch (actionId) {
            case "tpv_people":
                operatorCallBack(e);
                break;
            default:
                break;
        }
    }

    private void operatorCallBack(ClosedCallBackEvent e) {
        IDataModel model = this.getModel();
        ListSelectedRowCollection selectedRows = (ListSelectedRowCollection) e.getReturnData();
        if (selectedRows != null && selectedRows.size() > 0) {
            ListSelectedRow selectedRow = selectedRows.get(0);
            long userId = (long) selectedRow.getPrimaryKeyValue();//人员id
            long userEntryId = (long) selectedRow.getEntryPrimaryKeyValue();//部门分录id
            model.beginInit();
            model.setValue("tpv_people", userId);
            DynamicObjectCollection entry = getUserDepts(userId);
            for (DynamicObject row : entry) {
                if (userEntryId == row.getLong("id")) {
                    model.setValue("tpv_department", row.get("org"));
                }
            }
            model.endInit();
            this.getView().updateView("tpv_people");
            this.getView().updateView("tpv_department");
        }
    }

    private static DynamicObjectCollection getUserDepts(long userId) {
        QFilter qFilter = new QFilter("user", QCP.equals, userId);
//        qFilter.and(new QFilter("org.orgpattern.number", QCP.equals, "Orgform06"));
        return QueryServiceHelper.query("bos_userposition", "id,org", qFilter.toArray());
    }

    private void operatorF7(BeforeF7SelectEvent e, String key) {
        DynamicObject tpvCompany = (DynamicObject) this.getModel().getValue("org");
        if (tpvCompany != null) {
            List<QFilter> QFilters = new ArrayList();
            ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
            DynamicObject[] bosOrg = BusinessDataServiceHelper.load("bos_org_orgrelation_dept", "toorg", new QFilter[]{new QFilter("fromorg", "=", tpvCompany.getPkValue())});
            List<Long> listOrg = Arrays.stream(bosOrg)
                    .map(org -> org.getDynamicObject("toorg"))
                    .filter(Objects::nonNull)
                    .map(toorg -> toorg.getLong("id"))
                    .collect(Collectors.toList());
            CloseCallBack closeCallBack = new CloseCallBack(this, key);
            showParameter.setCloseCallBack(closeCallBack);
            QFilters.add(new QFilter("entryentity.dpt", "in", listOrg));
            showParameter.getListFilterParameter().setQFilters(QFilters);
        } else {
            e.setCancel(true);
            this.getView().showTipNotification("请先选择公司！");
        }

    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
//        DynamicObject tpvApplicant = (DynamicObject) this.getModel().getValue("tpv_people");
//
//        if (tpvApplicant != null) {
//            long applierId = (long) tpvApplicant.getPkValue();
//            this.getModel().setValue("tpv_department", UserServiceHelper.getUserMainOrgId(applierId), 0);
//        }
        this.getView().setEnable(false, "tpv_advconbaritemap");
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        String name = e.getProperty().getName();
        switch (name) {
            case "tpv_billladingdate":
            case "tpv_vailddates":
            case "tpv_pertimes":
                dateAddDay();
                break;
            case "tpv_ratetable":
            case "tpv_amountpre":
                rateTable();
                break;
            case "tpv_couriercomp":
                advpaymentInfor();
                break;
            case "tpv_docopresentat":
                String falag = (String) this.getModel().getValue("tpv_docopresentat");
                if (YEM.isEmpty(falag)) {
                    this.setMustInput("tpv_other", false);
                } else {
                    if (falag.contains("G")) {
                        this.setMustInput("tpv_other", true);
                    } else {
                        this.setMustInput("tpv_other", false);
                    }
                }
                break;
        }
    }

    private void setMustInput(String sign, boolean flag) {
        TextEdit tpvQuotaforms = this.getView().getControl(sign);
        tpvQuotaforms.setMustInput(flag);
        TextProp property = (TextProp) tpvQuotaforms.getProperty();
        property.setMustInput(flag);
    }

    @Override
    public void entryRowClick(RowClickEvent evt) {
        int row = evt.getRow();
        if (row < 0) {
            this.getView().setEnable(false, "tpv_advconbaritemap");
            return;
        }
        DynamicObjectCollection tpvAdvPaymentInfor = this.getModel().getEntryEntity("tpv_kdmentinfor");
        String tpvCouriercomp = tpvAdvPaymentInfor.get(row).getString("tpv_couriercomp");
        EntryGrid entryGrid = this.getControl("tpv_kdmentinfor");
        int[] selectRows = entryGrid.getSelectRows();
        if (selectRows.length == 1) {
            if ("A".equals(tpvCouriercomp)) {
                this.getView().setEnable(true, "tpv_advconbaritemap");
            } else {
                this.getView().setEnable(false, "tpv_advconbaritemap");
            }
        } else if (selectRows.length > 1) {
            for (int i = 0; i < tpvAdvPaymentInfor.size(); i++) {
                String couriercomp = tpvAdvPaymentInfor.get(i).getString("tpv_couriercomp");
                if ("A".equals(couriercomp)) {
                    this.getView().setEnable(true, "tpv_advconbaritemap");
                } else {
                    this.getView().setEnable(false, "tpv_advconbaritemap");
                    return;
                }
            }
        }

    }

    /**
     * @Title: advpaymentInfor
     * @Descriptin: 【快递公司】=“DHL”时通过接口获取，其他选项手工填
     * @retrn: a
     */
    public void advpaymentInfor() {
        DynamicObjectCollection tpvAdvPaymentInfor = this.getModel().getEntryEntity("tpv_kdmentinfor");
        EntryGrid entryGrid = this.getControl("tpv_kdmentinfor");
        int[] selectRows = entryGrid.getSelectRows();
        for (int selectRow : selectRows) {
            DynamicObject payInfo = tpvAdvPaymentInfor.get(selectRow);
            String tpvCouriercomp = payInfo.getString("tpv_couriercomp");
            if ("A".equals(tpvCouriercomp)) {
                this.getView().setEnable(true, "tpv_advconbaritemap");
                this.getView().setEnable(false, selectRow, "tpv_datemailing");
                this.getView().setEnable(false, selectRow, "tpv_datelodgment");
                this.getView().setEnable(false, selectRow, "tpv_statuesk");
                this.getModel().setValue("tpv_datemailing", null);
                this.getModel().setValue("tpv_datelodgment", null);
                this.getModel().setValue("tpv_statuesk", null);
            } else {
                this.getView().setEnable(false, "tpv_advconbaritemap");
                this.getView().setEnable(true, selectRow, "tpv_datemailing");
                this.getView().setEnable(true, selectRow, "tpv_datelodgment");
                this.getView().setEnable(true, selectRow, "tpv_statuesk");
                this.getModel().setValue("tpv_datemailing", null);
                this.getModel().setValue("tpv_datelodgment", null);
                this.getModel().setValue("tpv_statuesk", null);
            }
        }
    }

    /**
     * @Title: rateTable
     * @Descriptin: 根据汇率表带出汇率，计算金额
     * @retrn: a
     */
    private void rateTable() {
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        DynamicObject tpvRateTable = dataEntity.getDynamicObject("tpv_ratetable");
        BigDecimal amount = dataEntity.getBigDecimal("tpv_amountpre");
        Date date = new Date();
        Object tpvCurrency = this.getModel().getValue("currency");
        BigDecimal exchangeRateCny = YEM.getExchangeRate(tpvRateTable, tpvCurrency, 1L, date);
        BigDecimal exchangeRateUsd = YEM.getExchangeRate(tpvRateTable, tpvCurrency, 6L, date);
        if (tpvRateTable == null) {
            this.getModel().setValue("tpv_rate", null);//汇率
            this.getModel().setValue("tpv_kzsqusdrate", null);//折usd金额
            this.getModel().setValue("tpv_amountfields", null);//折rmb金额
            this.getModel().setValue("tpv_kzsqusdmon", null);//折usd金额
        } else {
            this.getModel().setValue("tpv_rate", exchangeRateCny);//汇率
            this.getModel().setValue("tpv_kzsqusdrate", exchangeRateUsd);//折usd金额
            this.getModel().setValue("tpv_amountfields", exchangeRateCny.multiply(amount));//折rmb金额
            this.getModel().setValue("tpv_kzsqusdmon", exchangeRateUsd.multiply(amount));//折usd金额
        }

    }

    /**
     * @Title: dateAddDay
     * @Descriptin:（提单日+交单天数）小于等于信用证有效期，则交单截止日=（提单日+交单天数）；（提单日+交单天数）大于信用证有效期，则交单截止=信用证有效期。
     * @retrn: a
     */
    public void dateAddDay() {
        Integer tpvPertimes = (Integer) this.getModel().getValue("tpv_pertimes");
        Date tpvBillladingdate = (Date) this.getModel().getValue("tpv_billladingdate");
        Date tpvVailddates = (Date) this.getModel().getValue("tpv_vailddates");
        Date billladingdate = DateUtil.addDays(tpvBillladingdate, tpvPertimes);
        if (tpvVailddates != null && billladingdate != null) {
            if (tpvVailddates.before(billladingdate)) {
                this.getModel().setValue("tpv_presentations", tpvVailddates);
            } else {
                this.getModel().setValue("tpv_presentations", billladingdate);
            }
        } else if (tpvVailddates == null && tpvBillladingdate != null) {
            this.getModel().setValue("tpv_presentations", tpvBillladingdate);
        }
    }


    /**
     * @param evt:
     * @return void
     * @author: chen
     * @date: 2023/12/1 14:07
     * @description: 获取快递信息
     */
    @Override
    public void itemClick(ItemClickEvent evt) {
        String key = evt.getItemKey();
        if (key.equals("tpv_advconbaritemap")) {
            DynamicObjectCollection tpvAdvPaymentInfor = this.getModel().getEntryEntity("tpv_kdmentinfor");
            EntryGrid entryGrid = this.getControl("tpv_kdmentinfor");
            int[] selectRows = entryGrid.getSelectRows();
            for (int selectRow : selectRows) {
                DynamicObject dynamicObject = tpvAdvPaymentInfor.get(selectRow);
                String courierNum = dynamicObject.getString("tpv_couriernum");
                HashMap<String, Object> shipmentDesc = DHLClient.setShipmentDesc(courierNum, selectRow);
                if (shipmentDesc.get("errorMsg").toString().isEmpty()) {
                    this.getModel().setValue("tpv_datemailing", shipmentDesc.get("tpv_sendoutdate"), selectRow);//寄出日期
                    this.getModel().setValue("tpv_datelodgment", shipmentDesc.get("tpv_receipteddate"), selectRow);
                    this.getModel().setValue("tpv_statuesk", shipmentDesc.get("tpv_courierstatus"), selectRow);
                } else {
                    this.getView().showTipNotification(String.format("快递单号,%s", shipmentDesc.get("errorMsg")));
                }
            }
        }
    }
}
