package tpv.tmc.lc.java.formplugin.form;

import kd.bos.bill.BillOperationStatus;
import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.resource.ResManager;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.form.CloseCallBack;
import kd.bos.form.ShowType;
import kd.bos.form.control.events.BeforeItemClickEvent;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.list.IListView;
import kd.bos.list.events.BeforeShowBillFormEvent;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.tmc.fbp.common.enums.BillStatusEnum;
import kd.tmc.fbp.common.helper.TmcDataServiceHelper;
import kd.tmc.fbp.common.util.EmptyUtil;
import kd.tmc.lc.common.enums.LetterBizTypeEnum;
import kd.tmc.lc.common.enums.LetterCreditStatusEnum;
import kd.tmc.lc.formplugin.lettercredit.LetterCreditList;

import java.util.Arrays;
import java.util.EventObject;
import java.util.List;
import java.util.stream.Collectors;

public class ExtLetterCreditList extends LetterCreditList {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners("tblrepealapply");
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        ListSelectedRowCollection selectedRows = ((IListView) this.getView()).getSelectedRows();
        if (selectedRows.size() == 0) {
            return;
        }
        switch (evt.getItemKey()) {
            case "tblrepealapply":
                if (this.checkCanRepeal(this.getSelectedId())) {
                    this.showRepealCreditView();
                }
                break;
            default:
                break;
        }
    }

    @Override
    public void beforeItemClick(BeforeItemClickEvent evt) {
        super.beforeItemClick(evt);
        switch (evt.getItemKey()) {
            case "tblhistory":
                this.checkReasonable(evt);
                break;
            case "maintenance":
                this.getPageCache().put("maintenance", "true");
                break;
            case "tblrepeal":
                this.checkRepeal(evt);
                break;
            default:
                break;
        }
    }

    private void checkRepeal(BeforeItemClickEvent evt) {
        if (this.getListBillId().isEmpty()) {
            return;
        }
        DynamicObject[] load = BusinessDataServiceHelper.load("lc_bizapply", "tpv_oriisspro,biztype,tpv_poweroff,lettercredit,billstatus"
                , new QFilter[]{new QFilter("tpv_bustype", "=", "C")
                        , new QFilter("billstatus", "=", "C")});
        List<Object> list = Arrays.stream(load)
                .filter(single -> single.getDynamicObject("tpv_oriisspro") != null)
                .map(single -> single.getDynamicObject("tpv_oriisspro").getPkValue())
                .collect(Collectors.toList());
        Long id = this.getListBillId().get(0);
        DynamicObject lcLettercredit = BusinessDataServiceHelper.loadSingle(id, "lc_lettercredit");
        if (!list.contains(id)) {
            this.getView().showTipNotification("存在撤证申请才可撤证！");
            evt.setCancel(true);
        }

    }

    private void checkReasonable(BeforeItemClickEvent evt) {
        if (this.getListBillId().isEmpty()) {
            return;
        }
        Long id = this.getListBillId().get(0);
        DynamicObject lcLettercredit = BusinessDataServiceHelper.loadSingle(id, "lc_lettercredit");
        String creditNumber = lcLettercredit.getString("creditno");

        DynamicObject[] load = BusinessDataServiceHelper.load("lc_bizapply"
                , "tpv_modifybill,biztype,tpv_poweroff,lettercredit,billstatus,tpv_isnot"
                , new QFilter[]{new QFilter("biztype", "=", "edit_card")
                        , new QFilter("tpv_poweroff", "=", false)
                        , new QFilter("lettercredit.number", "=", creditNumber)
                        , new QFilter("tpv_isnot", "=", false)
                        , new QFilter("billstatus", "=", "C")});

        DynamicObject[] used = BusinessDataServiceHelper.load("lc_bizapply"
                , "tpv_modifybill,biztype,tpv_poweroff,lettercredit,billstatus,tpv_isnot"
                , new QFilter[]{new QFilter("biztype", "=", "edit_card")
                        , new QFilter("tpv_poweroff", "=", false)
                        , new QFilter("lettercredit.number", "=", creditNumber)
                        , new QFilter("tpv_isnot", "=", true)
                        , new QFilter("billstatus", "=", "C")});

        List<Object> list = Arrays.stream(load)
                .map(single -> single.getPkValue())
                .distinct()
                .collect(Collectors.toList());

        List<Object> usedList = Arrays.stream(used)
                .filter(single -> single.getDynamicObject("tpv_modifybill") != null)
                .map(single -> single.getDynamicObject("tpv_modifybill").getPkValue())
                .distinct()
                .collect(Collectors.toList());

        list.removeAll(usedList);

        list = list.stream()
                .map(single -> BusinessDataServiceHelper.loadSingle(single, "lc_bizapply"))
                .filter(single -> single.getDynamicObject("lettercredit") != null)
                .map(single -> single.getDynamicObject("lettercredit").getString("number"))
                .distinct()
                .collect(Collectors.toList());


        if (!list.contains(creditNumber)) {
            this.getView().showTipNotification("存在未关联的改证申请且未关闭的信用证才可改证！");
            evt.setCancel(true);
        }
    }

    private boolean checkCanRepeal(Long id) {
        if (id == null) {
            return false;
        }
        DynamicObject letterCredit = this.loadLetterCredit(id);
        QFilter qFilter = new QFilter("lettercredit", "=", letterCredit.getPkValue());
        boolean exists = TmcDataServiceHelper.exists("lc_arrival", qFilter.toArray());
        if (!exists && BillStatusEnum.AUDIT.getValue().equals(letterCredit.getString("billstatus")) && LetterCreditStatusEnum.DONE_REGISTER.getValue().equals(letterCredit.getString("creditstatus"))) {
            return true;
        } else {
            this.getView().showTipNotification(ResManager.loadKDString("只有单据状态为已审核&信用证状态为已登记，且信用证项下无到单，才可发起撤证。", "LetterCreditList_7", "tmc-lc-formplugin", new Object[0]));
            return false;
        }
    }

    private DynamicObject loadLetterCredit(Long id) {
        DynamicObject letterCredit = TmcDataServiceHelper.loadSingle(id, "lc_lettercredit", "creditstatus,billstatus");
        return letterCredit;
    }

    /**
     * @author: Heisenberg
     * @date: 2023/12/6 20:12
     * @description: 弹出撤证申请动态表单
     */
    private void showRepealCreditView() {
        BillShowParameter parameter = new BillShowParameter();
        parameter.setFormId("tpv_bizapply_certificate");
        parameter.setStatus(OperationStatus.ADDNEW);
        parameter.setParentFormId("lc_bizapply");
        parameter.setCustomParam("id", this.getListBillId().get(0));
        DynamicObject lcLettercredit = BusinessDataServiceHelper.loadSingle(this.getListBillId().get(0), "lc_lettercredit");
        String billno = lcLettercredit.getString("billno");
        DynamicObject lcBizapply = BusinessDataServiceHelper.loadSingle("lc_bizapply", "",
                new QFilter[]{new QFilter("tpv_oriisspro.billno", "=", billno)
                        , new QFilter("billstatus", "=", "C")
                        , new QFilter("tpv_bustype", "=", "C")});
        if (lcBizapply != null) {
            this.getView().showTipNotification("只有单据状态为已审核&信用证状态为已登记，且信用证项下无已审核的撤证申请，才可发起撤证申请。");
        } else {
            parameter.setCloseCallBack(new CloseCallBack(this, "repeal"));
//            parameter.getOpenStyle().setShowType(ShowType.Modal);
//            StyleCss inlineStyleCss = new StyleCss();
//            parameter.getOpenStyle().setInlineStyleCss(inlineStyleCss);
            parameter.getOpenStyle().setShowType(ShowType.MainNewTabPage);
            this.getView().showForm(parameter);
        }
    }

    @Override
    public void beforeShowBill(BeforeShowBillFormEvent e) {
        if (this.getPageCache().getAll().containsKey("history")) {
            e.getParameter().setCustomParam("history", "true");
            Long id = super.getSelectedId();
            if (EmptyUtil.isEmpty(id)) {
                return;
            }
            String bill = "tpv_lc_bizapply_changinh";
            boolean flag = false;
            if (this.getPageCache().getAll().containsKey("maintenance")) {
                bill = "tpv_lc_bizapply_changmo";
                flag = true;
            }
            e.getParameter().setPkId((Object) null);
            e.getParameter().setCaption("改证");
            e.getParameter().setFormId(bill);
            e.getParameter().setStatus(OperationStatus.ADDNEW);
            e.getParameter().setBillStatus(BillOperationStatus.EDIT);
            e.getParameter().setCustomParam("lettercredit", id);
            e.getParameter().setCustomParam("biztype", LetterBizTypeEnum.EDIT_CARD.getValue());
            e.getParameter().setCustomParam("tpv_isnot", true);
            e.getParameter().setCustomParam("tpv_parentchange", flag);
        }
        this.getPageCache().remove("maintenance");
        this.getPageCache().remove("history");
    }

    /**
     * @return List<Long>
     * @author: Heisenberg
     * @date: 2023/11/23 19:08
     * @description: 已选中的单据列表
     */
    private List<Long> getListBillId() {
        IListView view = (IListView) this.getView();
        ListSelectedRowCollection selectedRows = view.getSelectedRows();// 查询列表所勾选的数据
        return selectedRows.stream()
                .map(listSelectedRow -> (Long) listSelectedRow.getPrimaryKeyValue())
                .distinct()
                .collect(Collectors.toList());
    }
}
