package tpv.tmc.lc.java.formplugin.form;

import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.Date;
import java.util.EventObject;

/**
 * @Description: 收证处理 -表单插件
 * @Author LinRui
 * @Date 2024/2/28 10:28
 */
public class LcLreceiptTpvFormPlugin extends AbstractBillPlugIn {
    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        this.setCurrencyAndExchangeTable();
        this.setExchangeRate();
    }

    /**
     * 默认本位币、汇率表加载，根据（核算组织本位币设置）
     */
    private void setCurrencyAndExchangeTable() {
        DynamicObject org = (DynamicObject) this.getModel().getValue("org");
        if (org != null) {
            DynamicObject bd_accountingsys_base = BusinessDataServiceHelper.loadSingle("bd_accountingsys_base",
                    "id,basecurrrency,exratetable", new QFilter[]{new QFilter("baseacctorg", QCP.equals, org.getPkValue())});
            if (bd_accountingsys_base != null) {
                DynamicObject exratetable = BusinessDataServiceHelper.loadSingle("bd_exratetable"
                        , new QFilter[]{new QFilter("number", "=", "ERT-23")});
                DynamicObject basecurrrency = bd_accountingsys_base.getDynamicObject("basecurrrency");
                if (exratetable != null) {
                    this.getModel().setValue("tpv_exchangetable", exratetable); //汇率表
                }
//                DynamicObject exratetable = bd_accountingsys_base.getDynamicObject("exratetable");
                this.getModel().setValue("tpv_currencybase", basecurrrency); //本位币
            }
        }
    }

    /**
     * 汇率获取
     */
    private void setExchangeRate() {
        Object tpv_exratetable = this.getModel().getValue("tpv_exchangetable"); //汇率表
        Object tpv_currency = this.getModel().getValue("currency"); //结算币别
        Object tpv_currencybase = this.getModel().getValue("tpv_currencybase"); //本位币
        Date tpv_exratedate = (Date) this.getModel().getValue("tpv_exratedate"); //汇率日期
        BigDecimal exchangeRateCny = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 1L, tpv_exratedate);
        BigDecimal exchangeRateUsd = YEM.getExchangeRate(tpv_exratetable, tpv_currency, 6L, tpv_exratedate);
        BigDecimal exchangeRate = YEM.getExchangeRate(tpv_exratetable, tpv_currency, tpv_currencybase, tpv_exratedate);
        this.getModel().setValue("tpv_rmbrate", exchangeRateCny); //折RMB汇率
        this.getModel().setValue("tpv_tousdrate", exchangeRateUsd); //折USD汇率
        this.getModel().setValue("tpv_exchangerate", exchangeRate); //汇率
    }

    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String key = e.getProperty().getName();
        switch (key) {
            case "tpv_exchangetable": //汇率表
            case "tpv_exratedate": //汇率日期
            case "currency": //结算币别
            case "tpv_currencybase": //本位币
                this.setExchangeRate();
                break;
            case "creditno":
                this.calcuVersion();
                break;
            case "tpv_beneficiary":
                DynamicObject tpvBeneficiary = (DynamicObject) this.getModel().getValue("tpv_beneficiary");
                if (tpvBeneficiary != null) {
                    this.getModel().setValue("org", tpvBeneficiary);
                }
                break;
            default:
                break;
        }
    }

    /**
     * @author: Heisenberg
     * @date: 2024/4/24 下午6:42
     * @description: 改证次数计算
     */
    private void calcuVersion() {
        String creditno = (String) this.getModel().getValue("creditno");
        if (creditno.isEmpty()) {
            return;
        }
        DynamicObject[] load = BusinessDataServiceHelper.load("lc_receipt", "creditno"
                , new QFilter[]{new QFilter("creditno", QCP.equals, creditno)
                        , new QFilter("billstatus", QCP.equals, "C")});
        int version = 0;
        if (load.length > 0) {
            version = load.length + 1;
        }
        this.getModel().setValue("tpv_version", version);
    }
}
