package tpv.tmc.lc.java.formplugin.form;

import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.property.RefBillProp;
import kd.bos.form.CloseCallBack;
import kd.bos.form.ShowType;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.field.RefBillEdit;
import kd.bos.form.field.TextEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhck
 * @Package: com.itopview.wm.sc.formplugin.form.LetterCreditFormPlugin
 * @Date: 16:07 2023/11/13
 * @Description: ，点击 撤证 记录此刻时间与撤证人，改变信用证状态为“已撤证”
 * @Version: 1.0
 */
public class LetterCreditFormPlugin extends AbstractBillPlugIn implements BeforeF7SelectListener {

    private static final String PARENT_CREDIT = "tpv_parentcredit"; // 母信用证号
    private static final String REPEALER = "tpv_repealer"; // 撤证人
    private static final String REPEAL_DATE = "tpv_repealdate"; // 撤证日期
    private static final String REPEAL_CREDIT = "tpv_repealcredit"; // 撤证按钮
    private static final String REPEAL_APPLY = "tpv_repealapply"; // 撤证申请
    private static final String CREDIT_STATUS = "tpv_creditstatus"; // 信用证状态
    private static final String ISSUED_TYPE = "tpv_issuedtype"; // 开证
    private static final String CHANGE_TIMES = "tpv_version"; // 改证次数
    private static final String RATE_TABLE = "tpv_ratetable"; // 改证次数
    private static final String CHANGE_VERSION = "tpv_changeversion"; // 改证及历史版本明细
    private static final String TO_RMBRATE = "tpv_kzembrate"; // 开证申请折RMB汇率
    private static final String TO_USDRATE = "tpv_kzsqusdrate"; // 开证申请折USD汇率
    private static final String USD = "tpv_usdrate"; // 美元原币
    private static final String RMB = "tpv_rmbrate"; // 人民币原币
    private static final String CURRENCY = "currency"; // 开证币别
    private static final String AMOUNT = "amount"; // 开证金额
    private static final String CURRENT_DATE = "tpv_kzdate"; // 开证日期
    private static final String RMB_AMOUNT = "tpv_amountfields"; // 开证申请折RMB金额
    private static final String USD_AMOUNT = "tpv_kzsqusdmon"; // 开证申请折USD金额

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        RefBillEdit edit = this.getControl(PARENT_CREDIT);
        edit.addBeforeF7SelectListener(this);
        TextEdit confirmingBank = this.getControl("confirmingbank");
        confirmingBank.addButtonClickListener(this);
        RefBillEdit tpv_quotaforms = this.getView().getControl("tpv_quotaforms");
        tpv_quotaforms.addBeforeF7SelectListener(this);
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        // 改证及历史版本明细单据体行数
        DynamicObjectCollection changeHistory = this.getModel().getDataEntity(true).getDynamicObjectCollection(CHANGE_VERSION);
        this.getModel().setValue(CHANGE_TIMES, changeHistory.getRowCount()); // 改证次数为单据体改证历史版本的行数
        this.filterHistory();
        this.getView().setEnable(true,"benefiter");
        this.getView().updateView("benefiter");
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        this.isMustInput();
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        switch (evt.getItemKey()) {
            case REPEAL_CREDIT:
                this.getModel().setValue(REPEALER, YEM.getCurrentUserId());
                this.getModel().setValue(REPEAL_DATE, new Date());
                this.getModel().setValue(CREDIT_STATUS, "done_repeal");
                break;
            case REPEAL_APPLY:
                this.popupContractForm();
                break;
            default:
                break;
        }

    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        switch (e.getProperty().getName()) {
            case "tpv_purnumbers":
                this.queryMargin();
                break;
            case "credittype":
                this.isMustInput();
                break;
            case RATE_TABLE:
                this.calcuAmount();
                break;
            case "tpv_isnotyy":
                boolean flag = (boolean) this.getModel().getValue("tpv_isnotyy");
                this.setMustInput("tpv_quotaforms", flag);
                break;
            default:
                break;
        }
    }

    /**
     * @param beforeF7SelectEvent: F7选择事件
     * @Author: Heisenberg
     * @Date: 2023/11/14 9:31
     * @Description: 母证信用证号, 可选择同公司项下的已认领的收证开证类型为“母子证-前背 添加f7过滤
     */
    @Override
    public void beforeF7Select(BeforeF7SelectEvent beforeF7SelectEvent) {
        DynamicObject org = (DynamicObject) this.getModel().getValue("org");
        DynamicObject department = (DynamicObject) this.getModel().getValue("tpv_department");

        if (department == null) {
            this.getView().showTipNotification("请先选择部门！");
            beforeF7SelectEvent.setCancel(true);
            return;
        }

        List<Object> ids = new ArrayList<>();
        ids.add(department.getPkValue());
        department = BusinessDataServiceHelper.loadSingle(department.getPkValue(), "bos_adminorg");
        String orgpattern = department.getDynamicObject("orgpattern").getString("number");
        if ("OF-01".equals(orgpattern)) {

            DynamicObjectCollection collection = department.getDynamicObjectCollection("structure");

            collection.stream()
                    .filter(row -> row.getDynamicObject("viewparent") != null)
                    .map(row -> BusinessDataServiceHelper.loadSingle(row.getDynamicObject("viewparent").getPkValue(), "bos_adminorg"))
                    .filter(row -> "Orgform06".equals(row.getDynamicObject("orgpattern").getString("number")))
                    .parallel()
                    .forEach(row -> ids.add(row.getPkValue()));

        }

        ListShowParameter showParameter = (ListShowParameter) beforeF7SelectEvent.getFormShowParameter();
        List<QFilter> qFilters = new ArrayList<>();
        qFilters.add(new QFilter("billstatus", "=", "C"));

        switch (beforeF7SelectEvent.getProperty().getName()) {
            case PARENT_CREDIT:
                qFilters.add(new QFilter("tpv_claimnoticebillno", QCP.not_equals, ""));
                break;
            case "tpv_quotaforms":
                qFilters.add(new QFilter("shkd_dept", "in", ids));
                break;
            default:
                break;
        }

        showParameter.getListFilterParameter().setQFilters(qFilters);
    }


    private void filterHistory() {
        this.getModel().beginInit();
        DynamicObjectCollection collection = this.getModel().getDataEntity(true).getDynamicObjectCollection("modifyinfo");
        List<Integer> seq = collection.stream()
                .filter(single -> !BusinessDataServiceHelper.loadSingle(single.getLong("h_applyid"), "lc_bizapply")
                        .getBoolean("tpv_isnot"))
                .map(single -> single.getInt("seq"))
                .collect(Collectors.toList());
        Iterator<DynamicObject> iterator = collection.iterator();
        while (iterator.hasNext()) {
            DynamicObject single = iterator.next();
            if (seq.contains(single.getInt("seq"))) {
                iterator.remove();
            }
        }
        this.getModel().endInit();
        this.getView().updateView("modifyinfo");
        this.getModel().setDataChanged(false);
    }

    /**
     * @param sign:
     * @param flag:
     * @return void
     * @author: Heisenberg
     * @date: 2024/4/18 下午3:13
     * @description: 设置必录
     */
    private void setMustInput(String sign, boolean flag) {
        RefBillEdit tpvQuotaforms = this.getView().getControl(sign);
        tpvQuotaforms.setMustInput(flag);
        RefBillProp property = (RefBillProp) tpvQuotaforms.getProperty();
        property.setMustInput(flag);
    }

    /**
     * @author: Heisenberg
     * @date: 2023/12/6 19:11
     * @description:
     */
    private void popupContractForm() {
        BillShowParameter parameter = new BillShowParameter();
        parameter.setFormId("tpv_bizapply_certificate");
        parameter.setStatus(OperationStatus.ADDNEW);
        parameter.setParentFormId("lc_bizapply");
        parameter.setCustomParam("id", this.getModel().getValue("id"));
        parameter.setCloseCallBack(new CloseCallBack(this, "repeal"));
        parameter.getOpenStyle().setShowType(ShowType.Modal);
        this.getView().showForm(parameter);
    }

    /**
     * @author: Heisenberg
     * @date: 2023/12/4 14:39
     * @description: 代理开保证金动态查询
     */
    private void queryMargin() {
        String contract = (String) this.getModel().getValue("tpv_purnumbers");
        DynamicObject[] margin = BusinessDataServiceHelper.load("cas_recbill",
                "billno,tpv_purchasecontracts,currency,settleamountbase,settleamount",
                new QFilter[]{new QFilter("tpv_purchasecontractxs.billno", QCP.equals, contract)});
        Optional.ofNullable(margin)
                .ifPresent(collection -> Arrays.stream(collection)
                        .distinct()
                        .forEach(single -> {
                            int entryRow = this.getModel().createNewEntryRow("tpv_entryentity3");
                            this.getModel().setValue("tpv_gatnumber", single.getString("billno"), entryRow);
                            this.getModel().setValue("tpv_salenumber",
                                    single.getDynamicObject("tpv_purchasecontracts").getString("billno"), entryRow);
                            this.getModel().setValue("tpv_patcurreny",
                                    single.getDynamicObject(CURRENCY).getString("id"), entryRow);
                            this.getModel().setValue("tpv_promoney", single.getBigDecimal("settleamount"), entryRow);
                            this.getModel().setValue("tpv_brmonrmb", single.getBigDecimal("settleamountbase"), entryRow);
                        })
                );
    }

    private void calcuAmount() {
        DynamicObject rateTable = (DynamicObject) this.getModel().getValue(RATE_TABLE); //  汇率表
        DynamicObject usd = (DynamicObject) this.getModel().getValue(USD); // 美元
        DynamicObject rmb = (DynamicObject) this.getModel().getValue(RMB); // 人民币
        DynamicObject currency = (DynamicObject) this.getModel().getValue(CURRENCY); // 人民币

        Date date = (Date) this.getModel().getValue(CURRENT_DATE); // 开证日期

        BigDecimal formerRate = (BigDecimal) this.getModel().getValue(TO_RMBRATE); // 开证申请折RMB汇率
        BigDecimal amount = (BigDecimal) this.getModel().getValue(AMOUNT); // 开证金额

        // 开证折RMB汇率
        BigDecimal exchangeRateCny = YEM.getExchangeRate(rateTable, currency, rmb, date);

        // 开证日期美元转人民币汇率
        BigDecimal rate = YEM.getExchangeRate(rateTable, usd, rmb, date);

        // 开证申请折RMB汇率】/美元对人民币汇率 ，保留四位小数
        BigDecimal usdrate = formerRate.divide(rate, 4);

        // 赋值转USD汇率
        this.getModel().setValue(TO_USDRATE, usdrate);

        // 赋值转RMB汇率
        this.getModel().setValue(TO_RMBRATE, exchangeRateCny);

        // 开证折RMB金额
        this.getModel().setValue(RMB_AMOUNT, amount.multiply(exchangeRateCny));

        // 开证折USD金额
        this.getModel().setValue(USD_AMOUNT, amount.multiply(usdrate));
    }

    private void isMustInput() {
        DynamicObject credittype = (DynamicObject) this.getModel().getValue("credittype");
        RefBillEdit basedataEdit = this.getControl(PARENT_CREDIT);
        RefBillProp basedataProp = (RefBillProp) basedataEdit.getProperty();
        if (credittype!=null){
            String name = credittype.getString("number");
            if ("002".equals(name)){//前对背信用证
                basedataEdit.setMustInput(false);
                this.getView().setEnable(true,"tpv_parentcredit");
            } else if ("003".equals(name)) {//低风险开证
                basedataEdit.setMustInput(true);
                this.getView().setEnable(true,"tpv_parentcredit");
            }else if ("001".equals(name)){
                basedataEdit.setMustInput(false);
             this.getView().setEnable(false,"tpv_parentcredit");
            }
        }
    }
}