package tpv.tmc.lc.java.formplugin.form;

import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.property.RefBillProp;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.ShowType;
import kd.bos.form.control.Control;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.ComboEdit;
import kd.bos.form.field.RefBillEdit;
import kd.bos.form.field.TextEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhck
 * @Package: com.itopview.wm.sc.formplugin.form.PopupcontractFormPlugin
 * @Date: 14:09 2023/11/06
 * @Description: 《来单登记》【信用证号】点击弹出动态表单
 * @Version: 1.0
 */
public class PopupContractFormPlugin extends AbstractBillPlugIn implements BeforeF7SelectListener {
    private static final String LETTER_CREDIT = "tpv_lettercredit"; // 信用证号
    private static final String LETTER_CREDIT_TYPE = "tpv_xcredittype"; // 信用证类型
    private static final String CREDIT_CONTRACT = "tpv_contractdetail"; // 《开证处理》[合同明细]
    private static final String CREDIT_NOWORLATER = "tpv_advconap"; // 《即期远期》动态表单
    private static final String DATA = "data"; // 《开证处理》[合同明细]动态表单对接码
    private static final String TYPE = "type"; // 《开证处理》[即期远期]动态表单对接码
    private static final String SOURCE_BILL = "tpv_sourcebill"; // 《开证处理》[即期远期]动态表单对接码
    private static final String SUPPLIER = "tpv_supplier"; // 供应商
    private static final String CURRENCY = "currency"; // 币别
    private static final String USDRATE = "tpv_kzsqusdrate"; // 折USD汇率
    private static final String RMBRATE = "tpv_rate"; // 汇率
    private static final String USDAMOUNT = "tpv_kzsqusdmon"; // 折USD金额
    private static final String AMOUNT = "amount"; // 开证金额
    private static final String RATE_TABLE = "tpv_ratetable"; // 汇率表
    private static final String FORWARDDAYS = "forwarddays"; // 远期天数
    private static final String CALA_FORWARDWAY = "tpv_forwardway"; // 远期天数起算方式

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        String lettercredit = (String) this.getModel().getValue("tpv_lettercredit");
        if (lettercredit.isEmpty()) {
            this.getModel().setValue("credittype", null);
        }
        DynamicObject bdExratetable = BusinessDataServiceHelper.loadSingle("bd_exratetable", new QFilter[]{new QFilter("number", "=", "ERT-12")});
        this.getModel().setValue("tpv_ratetable",bdExratetable);
        this.getModel().setValue("org",null);
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        TextEdit credit = this.getView().getControl(LETTER_CREDIT);
        TextEdit letterCredit = this.getView().getControl(LETTER_CREDIT_TYPE);
        credit.addClickListener(this);
        letterCredit.addClickListener(this);
        BasedataEdit tpv_purnumfs = this.getView().getControl("tpv_purnumfs");//采购合同号
        tpv_purnumfs.addBeforeF7SelectListener(this);
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String name = e.getProperty().getName();
        switch (name) {
            case "tpv_purnumfs"://采购合同号
                qFilterThisOrgCombin(e);
                break;
        }
    }

    private void qFilterThisOrgCombin(BeforeF7SelectEvent e) {
        String tpvSettlement = (String) this.getModel().getValue("tpv_settlement");
        DynamicObject org = (DynamicObject) this.getModel().getValue("org");//公司
        DynamicObject tpvDepartment = (DynamicObject) this.getModel().getValue("tpv_department");//部门
        String bulid="";
        if ("B".equals(tpvSettlement)){
            bulid="DA";
        } else if ("C".equals(tpvSettlement)) {
            bulid="DP";
        }else if ("D".equals(tpvSettlement)){
            bulid="保函";
        }

        QFilter qFilter = new QFilter("billstatus", "=", "C");
        if (org != null) {
            qFilter.and("tpv_company", "=", org.getPkValue());
        }
        if (tpvDepartment != null) {
            qFilter.and("tpv_department", "=", tpvDepartment.getPkValue());
        }
        if (!"E".equals(tpvSettlement)) {
            qFilter.and("tpv_dt_purchase_u.tpv_payment.name", "=", bulid);
        }
        DynamicObject[] tpvDtPurchases = BusinessDataServiceHelper.load("tpv_dt_purchase", "", new QFilter[]{qFilter});
        List<Long> idList = Arrays.stream(tpvDtPurchases).map(s -> s.getLong("id")).collect(Collectors.toList());
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        List<QFilter> QFilters = new ArrayList<>();
        QFilters.add(new QFilter("id", "in", idList));
        showParameter.getListFilterParameter().setQFilters(QFilters);
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        Control source = (Control) evt.getSource();
        switch (source.getKey()) {
            case LETTER_CREDIT:
                DynamicObject tpvDepartment = (DynamicObject) this.getModel().getValue("tpv_department");
                DynamicObject org = (DynamicObject) this.getModel().getValue("org");
                if (tpvDepartment != null && org != null) {
                    Map<String, Object> map = new HashMap<>();
                    map.put("department", tpvDepartment.getPkValue());
                    map.put("org", org.getPkValue());
                    this.popupContractForm(map, CREDIT_CONTRACT, DATA);
                }
                break;
            case LETTER_CREDIT_TYPE:
                String sourceBill = (String) this.getModel().getValue("tpv_lettercredit");
                if (sourceBill != null) {
                    HashMap<String, Object> map = new HashMap<>();
                    map.put("bill", sourceBill);
                    this.popupContractForm(map, CREDIT_NOWORLATER, TYPE);
                } else {
                    this.getView().showTipNotification("请先选择信用证号！");
                }
                break;
            default:
                break;
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        switch (e.getProperty().getName()) {
            case "tpv_totalamount":
            case RATE_TABLE:
            case "arrivaldate":
            case "arrivalamount":
                this.calcuAmount();
                this.settlement();
                this.forwardwayDate();
                break;
            case "tpv_settlement":
            case "tpv_credittype":
            case "tpv_xcredittype":
            case "tpv_forwardway":
            case "tpv_ladbilldate":
            case "forwarddays":
                this.reset(e);
                this.settlement();
                this.forwardwayDate();
                break;
            case "tpv_purchasenum":
                this.renderEntry();
                break;
            case "tpv_purnumbers":
                this.calcuTimes();
                break;
            case "tpv_isnotyy":
                boolean flag = (boolean) this.getModel().getValue("tpv_isnotyy");
                this.setMustInput("tpv_quotaforms", flag);
                break;
            case "isdiscrepancy":
                Boolean isdiscrepancy = (Boolean) this.getModel().getValue("isdiscrepancy");
                if (!isdiscrepancy) {
                    this.getModel().setValue("discrepancy", null);
                }
                break;
            case "tpv_lettercredit":
                String tpvLettercredit = (String) this.getModel().getValue("tpv_lettercredit");
                if ("".equals(tpvLettercredit) || tpvLettercredit == null) {
                    this.getModel().setValue("currency",null);
                    this.getModel().setValue("tpv_lzdate",null);
                    this.getModel().setValue("amount",null);
                    this.getModel().setValue("tpv_camtoverrate",null);
                    this.getModel().setValue("tpv_camtlessrate",null);
                    this.getModel().setValue("tpv_benefiter",null);
                    this.getModel().setValue("tpv_credittype",null);
                    this.getModel().setValue("tpv_xcredittype",null);
                    this.getModel().setValue("credittype",null);
                    this.getModel().setValue("forwarddays",null);
                    this.getModel().setValue("tpv_forwardway",null);
                    this.getModel().deleteEntryData("tpv_contractdetail");
                }
                break;
            case"tpv_company":
                DynamicObject tpvCompany = (DynamicObject) this.getModel().getValue("tpv_company");
                if (tpvCompany!=null){
                    this.getModel().setValue("org",tpvCompany);
                }
                break;
            default:
                break;
        }
    }

    public void forwardwayDate() {
        String xcredittype = (String) this.getModel().getValue("tpv_xcredittype");//即期远期
        String tpvXcredittype = (String) this.getModel().getValue("tpv_credittype");//即期远期
        String tpvSettlement = (String) this.getModel().getValue("tpv_settlement");//结算方式
        String tpvForwardway = (String) this.getModel().getValue("tpv_forwardway");//远期天数起算方式
        Integer forwarddays = (Integer) this.getModel().getValue("forwarddays");//远期天数
        Date arrivaldate = (Date) this.getModel().getValue("arrivaldate");//来单日期
        Date tpvLadbilldate = (Date) this.getModel().getValue("tpv_ladbilldate");//提单日
        Date tpvLzdate = (Date) this.getModel().getValue("tpv_lzdate");//开证日期
        if ("B".equals(tpvForwardway)||"C".equals(tpvForwardway)) {
            this.getModel().setValue("tpv_billlading","A");
        }
        if ("B".equals(tpvSettlement) || ("B".equals(tpvXcredittype) || "C".equals(tpvXcredittype))||("假远期".equals(xcredittype)||"远期".equals(xcredittype))) {//结算方式为Da
            if ("A".equals(tpvForwardway)) {//见单后 预计最迟付款日期=来单日期+远期天数
                this.getModel().setValue("endpaydate", DateUtil.addDays(arrivaldate, forwarddays));
            } else if ("B".equals(tpvForwardway) || "C".equals(tpvForwardway)) {//提单日后/起 预计最迟付款日期=提单日+远期天数
                this.getModel().setValue("endpaydate", DateUtil.addDays(tpvLadbilldate, forwarddays));
            } else if ("D".equals(tpvForwardway)) {//开证日起 预计最迟付款日期=开证日期+远期天数
                this.getModel().setValue("endpaydate", DateUtil.addDays(tpvLzdate, forwarddays));
            }
        } else if ("A".equals(tpvSettlement) || "A".equals(tpvXcredittype)) {
            this.getModel().setValue("endpaydate", DateUtil.addWorkDays(arrivaldate, 5));
        } else if ("C".equals(tpvSettlement)) {
            this.getModel().setValue("endpaydate", DateUtil.addWorkDays(arrivaldate, 5));
        }

    }

    public void settlement() {
        String tpv_xcredittype = (String) this.getModel().getValue("tpv_xcredittype");
        String tpvXcredittype = (String) this.getModel().getValue("tpv_credittype");//即期远期
        String tpvSettlement = (String) this.getModel().getValue("tpv_settlement");//结算方式
        if ("B".equals(tpvSettlement) || ("B".equals(tpvXcredittype) || "C".equals(tpvXcredittype))||"假远期".equals(tpv_xcredittype)||"远期".equals(tpv_xcredittype)) {
            Date arrivaldate = (Date) this.getModel().getValue("arrivaldate");//来单日期
            Date date = DateUtil.addWorkDays(arrivaldate, 5);
            this.getModel().setValue("endacceptdate", date);
        }
    }


    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
        switch (closedCallBackEvent.getActionId()) {
            case "data":
                this.getModel().deleteEntryData(CREDIT_CONTRACT);
                this.returnDataCarryInfo(closedCallBackEvent);
                this.getView().invokeOperation("save", YEM.getOptionNoMsg());
                break;
            case "type":
                this.getView().invokeOperation("save", YEM.getOptionNoMsg());
                this.returnTypeCarryInfo(closedCallBackEvent);
                break;
            default:
                break;
        }
        this.settlement();
        this.forwardwayDate();
    }


    /**
     * @param closedCallBackEvent: 闭合回调事件
     * @Author: Heisenberg
     * @Date: 2023/11/14 9:36
     * @Description: 手工选择该信用证向下的{分录-即期远期列表}，单选
     */
    private void returnTypeCarryInfo(ClosedCallBackEvent closedCallBackEvent) {
        DynamicObject type = (DynamicObject) closedCallBackEvent.getReturnData();
        if (type != null) {
            // 信用证类型
            String letcre = type.getString("tpv_letcre");
            switch (letcre) {
                case "A":
                    letcre = "即期";
                    break;
                case "B":
                    letcre = "远期";
                    break;
                case "C":
                    letcre = "假远期";
                    break;
                default:
                    break;
            }
            Optional.ofNullable(letcre)
                    .ifPresent(credit -> this.getModel().setValue("tpv_xcredittype", credit));
            // 远期天数
            Optional.ofNullable(type.getBigDecimal("tpv_furlook"))
                    .ifPresent(furlook -> this.getModel().setValue(FORWARDDAYS, furlook.setScale(4)));
            // 远期天数计算方式
            Optional.ofNullable(type.getString("tpv_furlookhk"))
                    .ifPresent(lookhk -> this.getModel().setValue(CALA_FORWARDWAY, lookhk));
        }
    }

    /**
     * @param closedCallBackEvent: 闭合回调事件
     * @Author: Heisenberg
     * @Date: 2023/11/14 9:38
     * @Description: 弹出《开证处理》{合同明细}分录，单选，带出 【采购合同号】、【合同币别】、【合同金额】、【金额溢装率】、【金额溢短率】
     * 【供应商】、【业务部门】、【业务员】
     */
    private void returnDataCarryInfo(ClosedCallBackEvent closedCallBackEvent) {
        DynamicObject data = (DynamicObject) closedCallBackEvent.getReturnData();
        Optional.ofNullable(data)
                .ifPresent(target -> {
                    //  获取《开证处理》单据对象
                    DynamicObject lettercredit = BusinessDataServiceHelper.loadSingle(
                            target.getString(SOURCE_BILL), "lc_lettercredit");
                    // 信用证币别
                    this.getModel().setValue(CURRENCY, lettercredit.getDynamicObject(CURRENCY));
                    // 来单币别
                    this.getModel().setValue("arrivalcurrency", lettercredit.getDynamicObject(CURRENCY));
                    // 远期天数
                    this.getModel().setValue(FORWARDDAYS, lettercredit.getInt(FORWARDDAYS));
                    // 远期天数起算方式
                    this.getModel().setValue(CALA_FORWARDWAY, lettercredit.getString("tpv_forwardsway"));
                    this.getModel().setValue("tpv_credittype", lettercredit.getString("tpv_lettercty"));
                    // 信用证号
                    this.getModel().setValue(LETTER_CREDIT, lettercredit.getString("creditno"));
                    // 标准信用证号
                    DynamicObject lettercreditF7 = BusinessDataServiceHelper.loadSingle("lc_lettercredit_f7",
                            new QFilter[]{new QFilter("number", QCP.equals, lettercredit.getString("creditno"))});
                    Optional.ofNullable(lettercreditF7)
                            .ifPresent(baseData -> {
                                this.getModel().beginInit(); // 取消值更新
                                this.getModel().setValue("lettercredit", baseData.getString("id"));
                                this.getModel().endInit(); // 取消值更新
                                this.getView().updateView("lettercredit");
                            });
                    // 即期远期
                    this.getModel().setValue("tpv_credittype", lettercredit.getString("tpv_lettercty"));
                    // 信用证类型
                    this.getModel().setValue("credittype", lettercredit.getDynamicObject("credittype"));
                    // 信用证金额
                    this.getModel().setValue(AMOUNT, lettercredit.getBigDecimal(AMOUNT));
                    // 金额溢装率
                    this.getModel().setValue("tpv_camtoverrate", lettercredit.getBigDecimal("tpv_overfillrate"));
                    // 金额溢短率
                    this.getModel().setValue("tpv_camtlessrate", lettercredit.getBigDecimal("tpv_shorate"));
                    // 汇率
                    this.getModel().setValue(RMBRATE, lettercredit.getBigDecimal(RMBRATE));
                    // 折RMB金额
                    this.getModel().setValue("tpv_kzsqrmbrates", target.getBigDecimal("tpv_apprmbmon"));
                    // 折USD汇率
                    this.getModel().setValue(USDRATE, target.getBigDecimal("tpv_appusdrate"));
                    // 折USD金额
                    this.getModel().setValue(USDAMOUNT, target.getBigDecimal("tpv_appusdmon"));
                    // 汇率表
//                    this.getModel().setValue(RATE_TABLE, lettercredit.getDynamicObject(RATE_TABLE));
                    // 受益人
                    this.getModel().setValue("tpv_benefiter", lettercredit.getDynamicObject(SUPPLIER));
                    // 受益人
                    this.getModel().setValue("benefiter", lettercredit.getDynamicObject(SUPPLIER));
                    BigDecimal amount = target.getBigDecimal("tpv_kzappmon");

                    // 获取《采购合同》单据对象
                    DynamicObject contractBill = BusinessDataServiceHelper.loadSingle(
                            "tpv_dt_purchase", new QFilter[]{
                                    new QFilter("tpv_contractnumber", QCP.equals, target.getString("tpv_contractid"))});
                    int row = this.getModel().createNewEntryRow(CREDIT_CONTRACT);
                    // 源单id
                    this.getModel().setValue(SOURCE_BILL, target.getString(SOURCE_BILL), row);
                    //采购合同号
                    this.getModel().setValue("tpv_purnumfs", BusinessDataServiceHelper.loadSingle("tpv_dt_purchase",new QFilter[]{new QFilter("tpv_contractnumber","=", data.getString("tpv_contractid"))}), row);
                    // 来单金额
                    this.getModel().setValue("tpv_totalamount", amount, row);
                    // 采购合同号
                    this.getModel().setValue("tpv_purnumbers", data.getString("tpv_contractid"), row);
                    // 币别
                    this.getModel().setValue("tpv_purcurrency", contractBill.getDynamicObject("tpv_currency"), row);
                    // 开证金额
                    this.getModel().setValue("tpv_purmoney", lettercredit.getBigDecimal(AMOUNT), row);
                    // 供应商
                    this.getModel().setValue(SUPPLIER, contractBill.getDynamicObject(SUPPLIER), row);
                    // 提奖类型
                    this.getModel().setValue("tpv_tjtype", contractBill.getDynamicObject("tpv_typeofreward"), row);
                    // 业务部门
                    this.getModel().setValue("tpv_purdepartment", contractBill.getDynamicObject("tpv_department"), row);
                    // 组合号
                    this.getModel().setValue("tpv_combinationnum", contractBill.getDynamicObject("tpv_combinationnumber"), row);
                    // 贸易性质
                    this.getModel().setValue("tpv_naturetrade", contractBill.getDynamicObject("tpv_tradeproperty"), row);
                    // 策略号
                    this.getModel().setValue("tpv_strategynum", contractBill.getDynamicObject("tpv_policynumber"), row);
                    // 合同业务员
                    this.getModel().setValue("tpv_pruper", contractBill.getDynamicObject("tpv_operator"), row);
                });
    }

    /**
     * @author: Heisenberg
     * @date: 2023/11/14 10:08
     * @description: 弹出上游《开证处理》【合同明细】分录
     */
    private void popupContractForm(Map<String, Object> map, String formId, String sign) {
        FormShowParameter parameter = new FormShowParameter();
        parameter.setFormId(formId);
        parameter.setStatus(OperationStatus.ADDNEW);
        parameter.setCustomParams(map);
        parameter.setCloseCallBack(new CloseCallBack(this, sign));
        parameter.getOpenStyle().setShowType(ShowType.Modal);
        this.getView().showForm(parameter);
    }

    private void setMustInput(String sign, boolean flag) {
        RefBillEdit tpvQuotaforms = this.getView().getControl(sign);
        tpvQuotaforms.setMustInput(flag);
        RefBillProp property = (RefBillProp) tpvQuotaforms.getProperty();
        property.setMustInput(flag);
    }

    /**
     * @author: Heisenberg
     * @date: 2023/12/7 21:59
     * @description: 来单金额改变动态修改其他金额信息
     */
    private void calcuAmount() {
        BigDecimal amount = (BigDecimal) this.getModel().getValue("arrivalamount"); // 来单金额
        DynamicObject tpvRatetable = (DynamicObject) this.getModel().getValue(RATE_TABLE); // 汇率表
        Date arrivaldate = (Date) this.getModel().getValue("arrivaldate"); // 汇率日期
        DynamicObject arrivalcurrency = (DynamicObject) this.getModel().getValue("arrivalcurrency"); // 来单币别
        // 开证折RMB汇率
        BigDecimal exchangeRateCny = YEM.getExchangeRate(tpvRatetable, arrivalcurrency, 1L, arrivaldate);
        // 开证折USD汇率
        BigDecimal exchangeRateUsd = YEM.getExchangeRate(tpvRatetable, arrivalcurrency, 6L, arrivaldate);
        // 开证转RMB汇率
        this.getModel().setValue(RMBRATE, exchangeRateCny);
        // 赋值转USD汇率
        this.getModel().setValue(USDRATE, exchangeRateUsd);
        // 开证折RMB金额
        this.getModel().setValue("tpv_kzsqrmbrates", amount.multiply(exchangeRateCny));
        // 开证折USD金额
        this.getModel().setValue(USDAMOUNT, amount.multiply(exchangeRateUsd));
    }

    private void reset(PropertyChangedArgs e) {
        if ("E".equals(e.getChangeSet()[0].getNewValue())) {
            this.getModel().getDataEntity(true).getDynamicObjectCollection(CREDIT_CONTRACT).clear();
            this.getView().updateView(CREDIT_CONTRACT);
            this.getModel().setValue("tpv_settlement", "E");
        }
        if ("A".equals(e.getChangeSet()[0].getNewValue())) {
            this.getModel().getDataEntity(true).getDynamicObjectCollection(CREDIT_CONTRACT).clear();
            this.getView().updateView(CREDIT_CONTRACT);
//            this.getModel().setValue("tpv_settlement", "A");
        }
    }

    /**
     * @author: Heisenberg
     * @date: 2024/1/18 17:26
     * @description: 结算方式为其他时，合同分录由采购合同号带出
     */
    private void renderEntry() {
        DynamicObject target = (DynamicObject) this.getModel().getValue("tpv_purchasenum");
        if (null != target) {
            // 获取《采购合同》单据对象
            DynamicObject contractBill = BusinessDataServiceHelper.loadSingle(
                    "tpv_dt_purchase", new QFilter[]{
                            new QFilter("billno", QCP.equals, target.getString("number"))});
            int row = this.getModel().createNewEntryRow(CREDIT_CONTRACT);
            // 采购合同号
            this.getModel().setValue("tpv_purnumbers", contractBill.getString("tpv_contractnumber"), row);
            // 币别
            this.getModel().setValue("tpv_purcurrency", contractBill.getDynamicObject("tpv_currency"), row);
            // 合同金额
            this.getModel().setValue("tpv_purmoney", target.getBigDecimal("tpv_contractamount"), row);
            // 供应商
            this.getModel().setValue(SUPPLIER, contractBill.getDynamicObject(SUPPLIER), row);
            // 提奖类型
            this.getModel().setValue("tpv_tjtype", contractBill.getDynamicObject("tpv_typeofreward"), row);
            // 业务部门
            this.getModel().setValue("tpv_purdepartment", contractBill.getDynamicObject("tpv_department"), row);
            // 组合号
            this.getModel().setValue("tpv_combinationnum", contractBill.getDynamicObject("tpv_combinationnumber"), row);
            // 贸易性质
            this.getModel().setValue("tpv_naturetrade", contractBill.getDynamicObject("tpv_tradeproperty"), row);
            // 策略号
            this.getModel().setValue("tpv_strategynum", contractBill.getDynamicObject("tpv_policynumber"), row);
            // 合同业务员
            this.getModel().setValue("tpv_pruper", contractBill.getDynamicObject("tpv_operator"), row);
        } else {
            this.getModel().deleteEntryData("tpv_contractdetail");
        }
    }

    /**
     * @author: Heisenberg
     * @date: 2024/1/18 17:28
     * @description: 计算来单次数
     */
    private void calcuTimes() {
        String method = (String) this.getModel().getValue("tpv_settlement");
        int times = 1;
        if ("E".equals(method)) {
            DynamicObject[] load = BusinessDataServiceHelper.load("lc_arrival", "",
                    new QFilter[]{new QFilter("tpv_purchasenum.id", QCP.equals,
                            ((DynamicObject) this.getModel().getValue("tpv_purchasenum")).getPkValue()),
                            new QFilter("billstatus", QCP.equals, "C")});
            times = load.length + 1;
        } else {
            DynamicObject[] load = BusinessDataServiceHelper.load("lc_arrival", "",
                    new QFilter[]{new QFilter(LETTER_CREDIT, QCP.equals, this.getModel().getValue(LETTER_CREDIT)),
                            new QFilter("billstatus", QCP.equals, "C")});
            times = load.length + 1;
        }
        this.getModel().setValue("tpv_incomtimes", times);
    }
}
