package tpv.tmc.lc.java.opplugin.validator;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.validate.AbstractValidator;

import java.math.BigDecimal;
import java.util.List;

/**
 * @ClassName: BizapplyOpapy
 * @Description: 校检协同占比
 * @author: zbh
 * @date: 2023-11-09
 */
public class BizapplyOpapyAuditValidator extends AbstractOperationServicePlugIn {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_customer");
        keys.add("tpv_approvaltogether");
        keys.add("tpv_supplier_apt");
        keys.add("tpv_collaborasetting");
        keys.add("tpv_isnotsx");
        keys.add("tpv_synergy");
        keys.add("tpv_bustype");
    }


    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        DynamicObject[] entities = e.getDataEntities();
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] bills = this.getDataEntities();
                String key = this.getOperateKey();
                for (int i = 0; i < bills.length; i++) {
                    switch (key) {
                        case "submit":
                            synergySup(bills[i]);
                            break;
                        default:
                            break;
                    }
                }
            }

            /**
             * @Title: synergySup
             * @Description: 协同占比不为1
             * @return: void
             */
            private void synergySup(ExtendedDataEntity extendedDataEntity) {
                DynamicObject dataEntity = extendedDataEntity.getDataEntity();
                BigDecimal synergySum = BigDecimal.ZERO;
                boolean tpvIsnotsx = dataEntity.getBoolean("tpv_isnotsx");
                DynamicObjectCollection tpvCollaborasetting = dataEntity.getDynamicObjectCollection("tpv_collaborasetting");
                for (DynamicObject dynamicObject : tpvCollaborasetting) {
                    BigDecimal tpvSynergy = dynamicObject.getBigDecimal("tpv_synergy");
                    synergySum = synergySum.add(tpvSynergy);
                }
                if (tpvIsnotsx) {
                    if (synergySum.compareTo(new BigDecimal(100)) == 0) {
                    } else {
                        this.addMessage(extendedDataEntity, "协同占比之和不为100！");
                    }
                }
            }
        });
    }
}
