package tpv.tpv_wm.tpv_cm.java.formplugin.form;

import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.IEntryOperate;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.CloseCallBack;
import kd.bos.form.IFormView;
import kd.bos.form.ShowFormHelper;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.HashMap;
import java.util.Map;

/**
 * @ClassName: AltimeterVerificationFormPlugin
 * @Description: 高低计核销单-表单插件
 * @date: 2023/11/15 10:30
 * @author: Guo jiaChen
 */
public class AltimeterVerificationFormPlugin extends AbstractFormPlugin {
    private static final String VerificationEntryDetail = "tpv_verificatedetail";//核销明细
    private static final String SettleEntryDetail = "tpv_settledetail";//结算明细


    @Override
    public void registerListener(EventObject e) {
        this.addItemClickListeners("tpv_toolbarap");//工具栏
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String key = evt.getItemKey();
        if ("tpv_chooseverificate".equals(key)) {
            openDyForm();//打开高低计单列表
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        ChangeData[] changeData = e.getChangeSet();
        int index = changeData[0].getParentRowIndex();
        switch (name) {
            case "tpv_amounttax1":
                calculateTaxAmount(index);
                break;
            default:
                break;
        }
    }

    /**
     * @author: chen
     * @date: 2023/11/18 23:13
     * @description: 打开高低计单列表
     */
    private void openDyForm() {
        if (getSettleCurIndex() != -1) {
            DynamicObject value = (DynamicObject) this.getModel().getValue("tpv_customer", 0);
            Long id = value == null ? 0L : value.getLong("id");
            String highOrLow = (String) this.getModel().getValue("tpv_highorlow", 0);
            //列表parameter
            ListShowParameter parameter = ShowFormHelper.createShowListForm("tpv_cm_uneven", true);
            //添加过滤条件
            parameter.getListFilterParameter().getQFilters()
                    .add(new QFilter("billstatus", QCP.equals, "C")
                            .and(new QFilter("tpv_unevened", QCP.not_equals, highOrLow))
                            .and(new QFilter("tpv_cleariparty.id", QCP.equals, id)));
            parameter.setCloseCallBack(new CloseCallBack(this, "verification"));
            this.getView().showForm(parameter);
        } else {
            this.getView().showTipNotification("请先选择结算明细分录中的一行数据");
        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
        String key = closedCallBackEvent.getActionId();
        IDataModel model = this.getModel();
        Object returnData = closedCallBackEvent.getReturnData();
        if ("verification".equals(key) && null != returnData) {
            ListSelectedRowCollection rows = ((ListSelectedRowCollection) returnData);
            ArrayList<Long> pkIdList = new ArrayList<>();
            if (rows.size() > 0) {
                for (ListSelectedRow row : rows) {
                    Long primaryKeyValue = (Long) row.getPrimaryKeyValue();
                    pkIdList.add(primaryKeyValue);
                }
                setInvoiceData(pkIdList);//分录赋值
                calculateTaxAmount(0);//更新含税金额
            }
        }
    }

    /**
     * @param pkIdList:
     * @return void
     * @author: chen
     * @date: 2023/11/18 23:16
     * @description: 分录填充数据
     */
    private void setInvoiceData(ArrayList<Long> pkIdList) {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        IDataModel parentModel = view.getParentView().getModel();
        DynamicObjectCollection entity = model.getEntryEntity(VerificationEntryDetail);
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("id", QCP.in, pkIdList));//已审核
        BigDecimal amountTaxSum = BigDecimal.ZERO;

        DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "tpv_cm_uneven",
                "id,billno,billstatus,tpv_cleariparty,tpv_unevened,tpv_advconap" +
                        ",tpv_advconap.tpv_saleconum,tpv_advconap.tpv_purconum,tpv_advconap.tpv_businesstype" +
                        ",tpv_advconap.tpv_itemcodes,tpv_advconap.tpv_conprodname,tpv_advconap.tpv_taxamounts,tpv_advconap.tpv_amountincrea,tpv_advconap.tpv_notaxamounts" +
                        ",tpv_advconap.tpv_departmet,tpv_advconap.tpv_crew,tpv_advconap.tpv_firm,tpv_advconap.tpv_vatrates"
                , filters.toArray(new QFilter[0]));
        DataSet selected = queryDataSet.select("id", "billno", "tpv_cleariparty", "tpv_unevened", "tpv_advconap.tpv_saleconum saleconum"
                , "tpv_advconap.tpv_purconum purconum", "tpv_advconap.tpv_businesstype businesstype", "tpv_advconap.tpv_itemcodes itemcodes"
                , "tpv_advconap.tpv_conprodname conprodname", "tpv_advconap.tpv_taxamounts taxamounts", "tpv_advconap.tpv_amountincrea amountincrea"
                , "tpv_advconap.tpv_notaxamounts notaxamounts", "tpv_advconap.tpv_departmet departmet", "tpv_advconap.tpv_vatrates vatrates"
                , "tpv_advconap.tpv_crew crew", "tpv_advconap.tpv_firm firm", "tpv_advconap");

        int index = getSettleCurIndex();
        if (index == -1) {
            return;
        }
        for (Row row : selected) {
            int i = this.getModel().createNewEntryRow(VerificationEntryDetail);
            this.getModel().setEntryCurrentRowIndex(SettleEntryDetail, index);
            BigDecimal taxAmounts = row.getBigDecimal("taxamounts");
            amountTaxSum = amountTaxSum.add(taxAmounts);
            model.setValue("tpv_altimeternum1", row.getString("billno"), i, index);//高低计单号
            model.setValue("tpv_salenums1", row.getString("saleconum"), i, index);//销售合同号
            model.setValue("tpv_purconnum1", row.getString("purconum"), i, index);//采购合同号
            model.setValue("tpv_highorlow1", row.getString("tpv_unevened"), i, index);//高计/低计
            model.setValue("tpv_customer1", row.getLong("tpv_cleariparty"), i, index);//结算方
            model.setValue("tpv_material1", row.getLong("itemcodes"), i, index);//商品编码
            model.setValue("tpv_businesstype1", row.getString("businesstype"), i, index);//业务类型
            model.setValue("tpv_prodname1", row.getLong("conprodname"), i, index);//品名
            model.setValue("tpv_amounttax1", taxAmounts, i, index);//含税金额
            model.setValue("tpv_intaxrate1", row.getBigDecimal("vatrates"), i, index);//增值税率
            model.setValue("tpv_amountintax1", row.getLong("amountincrea"), i, index);//增值税额
            model.setValue("tpv_amountnotax1", row.getLong("notaxamounts"), i, index);//不含税金额
            model.setValue("tpv_dept1", row.getLong("departmet"), i, index);//部门
            model.setValue("tpv_user1", row.getLong("crew"), i, index);//人员
            model.setValue("tpv_firm1", row.getLong("firm"), i, index);//公司
            model.setValue("tpv_srcbillentryid1", row.getLong("tpv_advconap"), i, index);//来源单据分录id
            model.setValue("tpv_srcbillno1", row.getLong("id"), i, index);//来源单据id
        }
        this.getView().updateView(VerificationEntryDetail);

    }



    /**
     * @param index:
     * @return void
     * @author: chen
     * @date: 2023/11/18 23:19
     * @description: 计算含税金额
     */
    private void calculateTaxAmount(int index) {
        IEntryOperate entryOperate = this.getModel();
        entryOperate.setEntryCurrentRowIndex(SettleEntryDetail, index);    // 必须先指定父行号
        DynamicObjectCollection entity = entryOperate.getEntryEntity(VerificationEntryDetail);// 取分录数据包
        BigDecimal amountTaxSub = BigDecimal.ZERO;
        BigDecimal amountTax = BigDecimal.ZERO;
        amountTax = (BigDecimal) this.getModel().getValue("tpv_amounttax", index);
        for (DynamicObject object : entity) {
            amountTaxSub = amountTaxSub.add(object.getBigDecimal("tpv_amounttax1"));
        }
        //子分录含税金额之和大于父分录含税金额,以小值为准
        this.getModel().beginInit();
        if (amountTaxSub.compareTo(amountTax) > 0) {
            BigDecimal subtract = amountTaxSub.subtract(amountTax);
            for (int i = entity.size() - 1; i >= 0; i--) {
                BigDecimal lastAmount = entity.get(i).getBigDecimal("tpv_amounttax1");
                if (lastAmount.compareTo(subtract) < 0) {
                    this.getModel().setValue("tpv_amounttax1", new BigDecimal(0), i, index);
                    subtract = subtract.subtract(lastAmount);
                } else {
                    this.getModel().setValue("tpv_amounttax1", lastAmount.subtract(subtract), i, index);
                    break;
                }
            }
        } else {
            this.getModel().setValue("tpv_amounttax", amountTaxSub, index);
        }
        this.getModel().endInit();
//        this.getView().updateView(SettleEntryDetail);
//        this.getView().updateView(VerificationEntryDetail);
    }

    /**
     * @param :
     * @return int
     * @author: chen
     * @date: 2023/11/18 23:20
     * @description: 获取父分录的选中行索引
     */
    private int getSettleCurIndex() {
        EntryGrid control = this.getControl(SettleEntryDetail);
        int[] selectRows = control.getSelectRows();
        int index = -1;
        if (selectRows.length > 0) {
            index = selectRows[0];
        }
        return index;
    }

}
