package tpv.tpv_wm.tpv_cm.java.formplugin.list;

import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.ShowType;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.list.IListView;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ClassName: RemarkWriteListPlugin
 * Description: 特殊合同备注回填
 * date: 2023/11/4 15:22
 *
 * @author: Guo jiaChen
 */
public class RemarkWriteListPlugin extends AbstractListPlugin {

    @Override
    public void itemClick(ItemClickEvent evt) {
        String key = evt.getItemKey();
        List<Long> billId = this.getListBillId();
        if ("tpv_notes".equals(key)) {
            if (!billId.isEmpty()) {
                if (billId.size() == 1) {
                    // 创建弹出动态表单页面对象
                    FormShowParameter showParameter = new FormShowParameter();
                    // 设置弹出页面的标识
                    showParameter.setFormId("tpv_cm_special");
                    Map<String, Object> map = this.showInfoForm();
                    showParameter.setCustomParams(map);
                    // 状态
                    showParameter.setStatus(OperationStatus.ADDNEW);
                    // 设置页面关闭回调方法
                    showParameter.setCloseCallBack(new CloseCallBack(this, "tpv_notes"));
                    // 设置弹出页面打开方式
                    showParameter.getOpenStyle().setShowType(ShowType.Modal);
                    // 打开动态表单
                    this.getView().showForm(showParameter);
                } else if (billId.size() > 1) {
                    this.getView().showErrorNotification("请只选择一条数据进行回填！");
                }
                DynamicObject dataEntity = BusinessDataServiceHelper.loadSingle(billId.get(0), "tpv_cm_tsht");
            } else {
                this.getView().showErrorNotification("请选择一条数据进行回填！");
            }
        }
    }

    /**
     * @param values:
     * @return void
     * @author: chen
     * @date: 2023/11/18 23:24
     * @description: 保存数据
     */
    private void setInwardInfo(String values) {
        List<Long> billId = this.getListBillId();
        DynamicObject dataEntity = BusinessDataServiceHelper.loadSingle(billId.get(0), "tpv_cm_tsht");
        dataEntity.set("tpv_remark", values);
        SaveServiceHelper.update(dataEntity);
        this.getView().invokeOperation("save");
        ((IListView) this.getView()).refresh();
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        Object returnData = closedCallBackEvent.getReturnData();
        // 判断标识是否匹配，并验证返回值不为空，不验证返回值可能会报空指针
        if (StringUtils.equals(closedCallBackEvent.getActionId(), "tpv_notes")
                && null != closedCallBackEvent.getReturnData()) {
            // 单条数据可用String类型传输，返回多条数据可放入map中，也可使用json等方式传输
            String values = (String) returnData;
            this.setInwardInfo(values);
        }
    }

    /**
     * @param :
     * @return List<Long>
     * @author: chen
     * @date: 2023/11/18 23:24
     * @description: 货取列表选中行的单据id
     */
    private List<Long> getListBillId() {
        IListView view = (IListView) this.getView();
        ListSelectedRowCollection selectedRows = view.getSelectedRows();// 查询列表所勾选的数据
        List<Long> billId = new ArrayList<>();
        for (ListSelectedRow listSelectedRow : selectedRows) {
            Object pkId = listSelectedRow.getPrimaryKeyValue();// 遍历获得每条数据的内码
            if (!billId.contains((long) pkId)) {
                billId.add((long) pkId);
            }
        }
        return billId;
    }

    /**
     * 动表单获取原单据字段
     */
    private Map<String, Object> showInfoForm() {
        // 创建集合保存需要获取的字段值
        Map<String, Object> map = new HashMap<>();
        List<Long> billId = this.getListBillId();
        DynamicObject dataEntity = BusinessDataServiceHelper.loadSingle(billId.get(0), "tpv_cm_tsht");
        String tpvRemark = dataEntity.getString("tpv_remark"); // 判断获得的各项数据
        if (null != tpvRemark && "" != tpvRemark) {
            map.put("tpv_renote", tpvRemark);
        }
        return map;
    }

}