package tpv.tpv_wm.tpv_cm.java.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

/**
 * ClassName: AltimeterVerificationOp
 * Description:
 * date: 2023/11/15 17:15
 *
 * @author: Guo jiaChen
 */
public class AltimeterVerificationOp extends AbstractOperationServicePlugIn {

    private static final String algoKey = "AltimeterVerificationOp";
    private static final String VerificationEntryDetail = "tpv_verificatedetail";
    private static final String SettleEntryDetail = "tpv_settledetail";


    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                StringBuilder errorMsg = new StringBuilder("");
                ExtendedDataEntity[] bills = this.getDataEntities();
                if ("save".equals(this.getOperateKey()) || "submit".equals(this.getOperateKey())) {
                    for (int i = 0; i < bills.length; ++i) {
                        ExtendedDataEntity bill = bills[i];
                        HashMap<String, String> result = validateAmount(bill);
                        if (result.containsKey("error")) {
                            this.addMessage(bill, result.get("error"));
                        }
                    }
                }
                //审核
                if ("audit".equals(this.getOperateKey())) {
                    for (int i = 0; i < bills.length; ++i) {
                        ExtendedDataEntity bill = bills[i];
                        String result = changeData(bill);
                        errorMsg.append(result);
                        if (!errorMsg.toString().isEmpty()) {
                            this.addMessage(bill, errorMsg.toString());
                        }
                    }
                }
            }

            /**
             * @Title: validateAmount
             * @Description: 金额校验
             */
            private HashMap<String, String> validateAmount(ExtendedDataEntity dataEntities) {
                DynamicObject dataEntity = dataEntities.getDataEntity();
                HashMap<String, String> validateStatus = new HashMap<>();
                Long pkValue = (Long) dataEntity.getPkValue();
                BigDecimal settleAmount = BigDecimal.ZERO;
                BigDecimal verificationAmount = BigDecimal.ZERO;
                DynamicObjectCollection settleDetail = dataEntity.getDynamicObjectCollection(SettleEntryDetail);

                Set<Long> srcBillID = new HashSet<>();
                Set<String> srcSubBillID = new HashSet<>();
                HashMap<String, BigDecimal> verificationMap = new HashMap<>();//核销单 <分录id,核销金额>
                HashMap<String, BigDecimal> curVerificationMap = new HashMap<>();//核销单-核销明细 <高低计id,核销金额> 计算本单金额
                //结算明细
                for (DynamicObject setObject : settleDetail) {
                    Integer seq = (Integer) setObject.get("seq");
                    settleAmount = settleAmount.add(setObject.getBigDecimal("tpv_amounttax"));
                    String srcBillNo = setObject.getString("tpv_srcbillno");
                    String srcSubBillNo = setObject.getString("tpv_srcbillentryid");
                    srcBillID.add(toLong(srcBillNo));
                    srcSubBillID.add(srcSubBillNo);

                    DynamicObjectCollection verificationDetail = setObject.getDynamicObjectCollection(VerificationEntryDetail);
                    //核销明细
                    for (DynamicObject verObject : verificationDetail) {
                        String srcBillNo1 = verObject.getString("tpv_srcbillno1");//高低计单id
                        BigDecimal amountTax = verObject.getBigDecimal("tpv_amounttax1");//含税金额
                        if (curVerificationMap.containsKey(srcBillNo1)) {
                            curVerificationMap.put(srcBillNo1, curVerificationMap.get(srcBillNo1).add(amountTax));
                        } else {
                            curVerificationMap.put(srcBillNo1, amountTax);
                        }
                        verificationAmount = verificationAmount.add(amountTax);
                        srcBillID.add(toLong(srcBillNo1));
                    }
                }
                //保存校验
                for (Long id : srcBillID) {
                    //核销单
                    ArrayList<QFilter> filters = new ArrayList<>();
                    filters.add(new QFilter("id", QCP.not_equals, pkValue));//排除本单
                    filters.add(new QFilter("tpv_settledetail.tpv_srcbillno", QCP.equals, id.toString()));//高低计单id = 父分录来源单据id or 子分录来源单据id
                    filters.add(new QFilter("tpv_settledetail.tpv_verificatedetail.tpv_srcbillno1", QCP.equals, id.toString()));//高低计单id = 父分录来源单据id or 子分录来源单据id
                    //高低计单
                    ArrayList<QFilter> filter = new ArrayList<>();
                    filter.add(new QFilter("id", QCP.equals, id));
                    filter.add(new QFilter("billstatus", QCP.equals, "C"));
                    BigDecimal amountTaxSum = BigDecimal.ZERO;
                    BigDecimal taxAmount = BigDecimal.ZERO;
                    String billNo = "";
                    //核销单
                    DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "tpv_cm_altverification",
                            "tpv_settledetail.tpv_verificatedetail,tpv_settledetail.tpv_srcbillno,tpv_settledetail.tpv_verificatedetail.tpv_srcbillno1,tpv_settledetail.tpv_verificatedetail.tpv_amounttax1"
                            , filters.toArray(new QFilter[0]));
                    for (Row row : queryDataSet) {
                        amountTaxSum = amountTaxSum.add(row.getBigDecimal("tpv_settledetail.tpv_verificatedetail.tpv_amounttax1"));
                    }
                    //加上本单核销金额
                    if (curVerificationMap.containsKey(id.toString())) {
                        amountTaxSum = amountTaxSum.add(curVerificationMap.get(id.toString()));
                    }
                    //高低计单
                    DataSet querysDataSet = ORM.create().queryDataSet(this.getClass().getName(), "tpv_cm_uneven",
                            "billno,tpv_taxamount", filter.toArray(new QFilter[0]));
                    querysDataSet.print(true);
                    for (Row row : querysDataSet) {
                        taxAmount = row.getBigDecimal("tpv_taxamount");
                        billNo = row.getString("billno");
                    }

                    if (taxAmount.compareTo(amountTaxSum) < 0) {//校验失败,返回失败信息
                        validateStatus.put("error", "保存失败,核销金额超出高低计单(" + billNo + ")的含税金额,请检查!!!");
                        return validateStatus;
                    }
                }
                validateStatus.put("OK", "保存成功!");
                //校验通过,重新计算未核销金额
                ArrayList<QFilter> filters = new ArrayList<>();
                filters.add(new QFilter("id", QCP.not_equals, pkValue));//排除本单
                filters.add(new QFilter("tpv_settledetail.tpv_srcbillentryid", QCP.in, srcSubBillID));//来源分录id
                DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "tpv_cm_altverification",
                        "tpv_settledetail.tpv_srcbillentryid,tpv_settledetail.tpv_amountverificatehid", filters.toArray(new QFilter[0]));
                if (queryDataSet.isEmpty()) {
                    return validateStatus;
                }
                DataSet select = queryDataSet.select("tpv_settledetail.tpv_amountverificatehid sumamount", "tpv_settledetail.tpv_srcbillentryid tpv_srcbillentryid");
                DataSet executeSql = select.executeSql("select sum(sumamount) ,tpv_srcbillentryid GROUP BY tpv_srcbillentryid");
                for (Row row : executeSql) {
                    verificationMap.put(row.getString("tpv_srcbillentryid"), row.getBigDecimal("sumamount"));
                }
                for (DynamicObject object : settleDetail) {
                    String srcbillentryid = object.getString("tpv_srcbillentryid");
                    BigDecimal decimal = object.getBigDecimal("tpv_amountverificatehid");
                    BigDecimal sumDecimal = decimal.add(verificationMap.get(srcbillentryid));
                    object.set("tpv_amountverificatehid", sumDecimal);//修改核销金额
                    object.set("tpv_nuverificatamount", object.getBigDecimal("tpv_amounttaxhid").subtract(sumDecimal));//修改未核销金额
                }
                return validateStatus;
            }

            /**
             * @Title: changeData
             * @Description: 反写数据
             */
            private String changeData(ExtendedDataEntity dataEntities) {
                DynamicObject dataEntity = dataEntities.getDataEntity();
                HashSet<Long> billIDs = new HashSet<>();
                HashSet<String> subIDs = new HashSet<>();
                //反写数据
                DynamicObjectCollection collection = dataEntity.getDynamicObjectCollection(SettleEntryDetail);
                collection.forEach(col -> {
                    String subID = col.getString("tpv_srcbillentryid");//分录id
                    String billID = col.getString("tpv_srcbillno");//单据id
                    BigDecimal amountTax = col.getBigDecimal("tpv_amounttax");
                    billIDs.add(toLong(billID));
//                    subIDs.add(toLong(subID));
//                    billIDs.add((billID));
                    subIDs.add((subID));
                });
                return updateAltimeterVerificationData(billIDs, subIDs) + updatePlanBillStatus(billIDs, null, true);
            }

            public Long toLong(Object longObj) {
                if (longObj == null) {
                    return null;
                } else if (longObj instanceof Long) {
                    return (Long) longObj;
                } else {
                    return Long.valueOf(String.valueOf(longObj));
                }
            }
        });
    }

    /**
     * @Title: updateAltimeterVerificationData
     * @Description: 反写高低计单核销金额
     */
    public static String updateAltimeterVerificationData(HashSet<Long> billIDs, HashSet<String> subIDs) {
        HashMap<Long, BigDecimal> verificationAmount = new HashMap<>();
        //高低计单
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("id", QCP.in, billIDs));
        filters.add(new QFilter("billstatus", QCP.equals, "C"));
        DynamicObject[] load = BusinessDataServiceHelper.load("tpv_cm_uneven", "id,tpv_advconap,tpv_cancelqtys,tpv_taxamounts,tpv_unwrittenqtys,tpv_planqty,tpv_unplanqty"
                , filters.toArray(new QFilter[0]));
        //核销单
        ArrayList<QFilter> filter = new ArrayList<>();
        filter.add(new QFilter("tpv_settledetail.tpv_srcbillentryid", QCP.in, subIDs));//
        filter.add(new QFilter("billstatus", QCP.equals, "C"));
        DataSet queryDataSet = ORM.create().queryDataSet(algoKey, "tpv_cm_altverification", "tpv_settledetail.tpv_srcbillentryid,tpv_settledetail.tpv_amounttax"
                , filter.toArray(new QFilter[0]));
        DataSet executeSql = queryDataSet.executeSql("select tpv_settledetail.tpv_srcbillentryid  ,sum(tpv_settledetail.tpv_amounttax)  GROUP BY tpv_settledetail.tpv_srcbillentryid");
        for (Row row : executeSql) {
            BigDecimal amountTax = row.getBigDecimal("tpv_settledetail.tpv_amounttax");
            Long entryID = row.getLong("tpv_settledetail.tpv_srcbillentryid");
            verificationAmount.put(entryID, amountTax);
        }

        for (DynamicObject object : load) {
            DynamicObjectCollection collection = object.getDynamicObjectCollection("tpv_advconap");
            for (DynamicObject dynamicObject : collection) {
                Long entryID = dynamicObject.getLong("id");
                if (verificationAmount.containsKey(entryID)) {
                    BigDecimal amount = verificationAmount.get(entryID);//获得本次核销金额
                    BigDecimal cancelQty = dynamicObject.getBigDecimal("tpv_cancelqtys");//已核销金额
                    BigDecimal taxAmount = dynamicObject.getBigDecimal("tpv_taxamounts");//含税金额
                    dynamicObject.set("tpv_cancelqtys", cancelQty.add(amount));//核销金额
                    dynamicObject.set("tpv_unwrittenqtys", taxAmount.subtract(amount.add(cancelQty)));//未核销金额
                }

            }
        }
        return saveData(load, "tpv_cm_uneven");
    }

    /**
     * @Title: updatePlanBillStatus
     * @Description: 反写高低计单规划金额, 控制规划单是否禁止
     */
    public static String updatePlanBillStatus(HashSet<Long> billIDs, HashMap<Long, BigDecimal> subIDs, boolean forbidPlan) {
        //规划单
        ArrayList<QFilter> planFilters = new ArrayList<>();
        planFilters.add(new QFilter("tpv_sourcebillid", QCP.in, billIDs));
        planFilters.add(new QFilter("billstatus", QCP.equals, "C"));
        planFilters.add(new QFilter("tpv_invalid", QCP.equals, false));
        DynamicObject[] plan = BusinessDataServiceHelper.load("tpv_cm_plannedbills", "id,billno,tpv_invalid,tpv_sourcebillid"
                , planFilters.toArray(new QFilter[0]));
        //高低计
        ArrayList<QFilter> unevenFilters = new ArrayList<>();
        unevenFilters.add(new QFilter("id", QCP.in, billIDs));

        unevenFilters.add(new QFilter("billstatus", QCP.equals, "C"));
        DynamicObject[] load = BusinessDataServiceHelper.load("tpv_cm_uneven", "id,tpv_advconap,tpv_cancelqtys,tpv_unwrittenqtys,tpv_planqty,tpv_unplanqty,tpv_advconap.tpv_entryid,tpv_taxamounts"
                , unevenFilters.toArray(new QFilter[0]));
        if (forbidPlan) { //作废规划单
            //规划单禁用
            for (DynamicObject object : plan) {
                object.set("tpv_invalid", true);
            }
            //反写规划金额
            for (DynamicObject object : load) {
                DynamicObjectCollection collection = object.getDynamicObjectCollection("tpv_advconap");
                for (DynamicObject dynamicObject : collection) {
                    Long entryID = dynamicObject.getLong("id");
                    dynamicObject.set("tpv_planqty", BigDecimal.ZERO);//规划金额
                    dynamicObject.set("tpv_unplanqty", dynamicObject.getBigDecimal("tpv_unwrittenqtys"));//未规划金额
                }
            }
            SaveServiceHelper.save(plan);
            return saveData(load, "tpv_cm_uneven");
        } else {
            for (DynamicObject dynamicObject : load) {
                for (DynamicObject tpvAdvconap : dynamicObject.getDynamicObjectCollection("tpv_advconap")) {
                    long entryid = tpvAdvconap.getLong("tpv_entryid");
                    BigDecimal tpvUnplanqty = tpvAdvconap.getBigDecimal("tpv_unplanqty");
                    BigDecimal tpvTaxamounts = tpvAdvconap.getBigDecimal("tpv_taxamounts");
                    BigDecimal tpvCancelqtys = tpvAdvconap.getBigDecimal("tpv_cancelqtys");

                    if (subIDs.containsKey(entryid)) {
                        BigDecimal tpvPlanqty = tpvAdvconap.getBigDecimal("tpv_planqty");
                        BigDecimal addtpvPlanqty = tpvPlanqty.add(subIDs.get(entryid));
                        tpvUnplanqty = tpvTaxamounts.subtract(tpvCancelqtys).subtract(addtpvPlanqty);
                        tpvAdvconap.set("tpv_planqty", addtpvPlanqty);
                        tpvAdvconap.set("tpv_unplanqty", tpvUnplanqty);
                    }
                }
            }
            return saveData(load, "tpv_cm_uneven");
        }
    }

    /**
     * @Title: saveData
     * @Description: 保存数据
     */
    private static String saveData(DynamicObject[] plan, String entityID) {
        OperationResult saveOperate = OperationServiceHelper.executeOperate("save", entityID,
                plan, YEM.getOptionNoMsg());  //调用界面的保存按钮校验【回填单据的界面】
        if (!saveOperate.isSuccess()) {
            return "自动保存失败。" + YEM.getOpMsg(saveOperate);
        }
        return "";
    }
}
