package tpv.tpv_wm.tpv_crm.java.formplugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.LocaleString;
import kd.bos.entity.ValueMapItem;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.property.ComboProp;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.ComboEdit;
import kd.bos.form.field.ComboItem;
import kd.bos.form.field.RefBillEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.util.*;

/**
 * ClassName: SupplierLimitsFromPlugin
 * Description: 供应商额度维护单据插件
 * date: 2023/8/30 17:06
 *
 * @author: Guo jiaChen
 * @version:
 * @since JDK 1.8
 */
public class SupplierLimitsBillFromPlugin extends AbstractBillPlugIn implements BeforeF7SelectListener {

    /**
     * 监听事件
     *
     * @param e
     */
    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        BasedataEdit org = (BasedataEdit) this.getView().getControl("tpv_org");
        org.addBeforeF7SelectListener((BeforeF7SelectListener) this);
        RefBillEdit guarantyNums = (RefBillEdit) this.getView().getControl("tpv_guarantynums");
        guarantyNums.addBeforeF7SelectListener((BeforeF7SelectListener) this);
        this.addItemClickListeners("bar_save");
    }


    /**
     * 数据绑定事件
     *
     * @param e
     */
    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        setTpvEffictivetypeTitle(); //设置有效期类型
    }

    /**
     * 值更新事件
     *
     * @param e
     */
    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        if ("tpv_credit".equals(name)) {
            clearOrg();
        }
        if ("tpv_billtype".equals(name)) {
            setTpvEffictivetypeTitle();
        }

    }

    /**
     * 设置显示下拉标题
     */
    private void setTpvEffictivetypeTitle() {
        String tpv_billtype = (String) this.getModel().getValue("tpv_billtype");
        if ("A".equals(tpv_billtype)) {
            //设置下拉列表显示标题
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("长期有效", "A");
            fillEntityCombo(map);
        } else if ("B".equals(tpv_billtype)) {
            //设置下拉列表显示标题
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("定期有效", "B");
            map.put("依存担保", "C");
            fillEntityCombo(map);
        } else {
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("长期有效", "A");
            map.put("定期有效", "B");
            map.put("依存担保", "C");
            fillEntityCombo(map);
        }
    }


    /**
     * 设置是否失效
     */
    private void clearOrg() {
        Map<String, Object> text = new HashMap<String, Object>();

        Object value = this.getModel().getValue("tpv_credit");
        if (value.equals("A")) {
            text.put("zh_CN", "公司");
        }
        if (value.equals("B")) {
            text.put("zh_CN", "部门");
        }

        if (null != value) {
            this.getModel().setValue("tpv_org", null);
        }
        this.getView().updateControlMetadata("tpv_org", text);//参数1：单据体标识
        this.getView().updateView("tpv_org");

    }


    /**
     * 基础资料过滤
     *
     * @param e
     */
    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String name = e.getProperty().getName();
        switch (name) {
            //组织过滤
            case "tpv_org":
                qFilterOrg(e);
                break;
            //担保单号过滤
            case "tpv_guarantynums":
                qFilterGuarantyNums(e);
                break;
        }

    }

    /**
     * 担保单号过滤
     *
     * @param e
     */
    private void qFilterGuarantyNums(BeforeF7SelectEvent e) {
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        List<QFilter> QFilters = new ArrayList<QFilter>();
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        Date tpv_bizdate = dataEntity.getDate("tpv_bizdate");//申请日期
        DynamicObject org = (DynamicObject) this.getModel().getValue("tpv_org");

        if (null != org) {
            String orgName = org.getString("name");
            QFilters.add(new QFilter("org.name", QCP.equals, orgName));//当前组织
        }
        //构造过滤条件
        QFilters.add(new QFilter("tpv_startdate", QCP.less_equals, tpv_bizdate));//大于起始日期
        QFilters.add(new QFilter("tpv_deadline", QCP.large_equals, tpv_bizdate));//小于结束日期
        QFilters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
        showParameter.getListFilterParameter().setQFilters(QFilters);

        Object value = this.getModel().getValue("tpv_guarantynums");
        if (value == null) {
            this.getModel().setValue("tpv_guarantynums", null);
        }

    }

    /**
     * 组织过滤
     *
     * @param e
     */
    private void qFilterOrg(BeforeF7SelectEvent e) {
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        List<QFilter> QFilters = new ArrayList<QFilter>();
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        Object Credit = dataEntity.get("tpv_credit");//公司/部门授信
        //获取下拉框的值
        ComboProp comboProp = (ComboProp) dataEntity.getDynamicObjectType().getProperties().get("tpv_credit");
        List<ValueMapItem> comboData = comboProp.getComboItems();

        String credit = null;//创建变量,存储选中的下拉标题
        if (Credit == null) {
            return;
        }
        //遍历下拉框的数据,获取选中值的下拉标题
        for (ValueMapItem item : comboData) {
            if (Credit.equals(item.getValue())) {
                credit = item.getName().toString();
                break;
            }
        }
        //构造过滤条件(组织形态.名称=授信)
        QFilters.add(new QFilter("orgpattern.name", QCP.equals, credit)); //设置过滤条件：形态
        showParameter.getListFilterParameter().setQFilters(QFilters);

    }


    /**
     * 设置下拉项
     *
     * @param map
     */
    private void fillEntityCombo(Map<String, Object> map) {
        ComboEdit mainEntry = this.getView().getControl("tpv_effictivetype");
        if (map.size() > 0) {
            List<ComboItem> entryItems = this.buildEntryComboItems(map);
            mainEntry.setComboItems(entryItems);
        }
    }

    /**
     * @param map
     * @return
     */
    private List<ComboItem> buildEntryComboItems(Map<String, Object> map) {
        ArrayList<ComboItem> entryItems = new ArrayList<>();
        for (String s : map.keySet()) {
            ComboItem comboItem = new ComboItem();
            comboItem.setValue((String) map.get(s));
            comboItem.setCaption(new LocaleString(s));
            entryItems.add(comboItem);
        }
        return entryItems;
    }


}
