package tpv.tpv_wm.tpv_crm.java.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author l jw
 * @date 2023/10/24 15:50
 * @className CustmianSubmitOp
 * @description 客户额度维护提交效验
 */
public class CustmianSubmitOp extends AbstractOperationServicePlugIn {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_validtype");
        keys.add("tpv_orgcredit");
        keys.add("billno");
        keys.add("tpv_cusgroup");
        keys.add("tpv_groupconamt");
        keys.add("tpv_groupdelim");
        keys.add("tpv_crm_cuslimit");
        keys.add("tpv_quotatype");
        keys.add("tpv_limitamt");
        e.setFieldKeys(keys);
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                StringBuilder errorMsg = new StringBuilder("");
                ExtendedDataEntity[] bills = this.getDataEntities();
                if ("submit".equals(this.getOperateKey()) && null != e.getDataEntities()) {
                    for (int i = 0; i < bills.length; ++i) {
                        ExtendedDataEntity bill = bills[i];
                        Updatecombofield(bill);
                        validateIsOneEntrySuppliers(bill);
                        errorMsg.append(validateIsOneEntrySuppliers(bill));
                        if (!errorMsg.toString().isEmpty()){
                            this.addMessage(bill,errorMsg.toString());
                        }
                    }
                }
            }

            /**
             * @Title: validateIsSuppliers
             * @Description: 校验{分录-供应商额度 }中供应商，只允许存在一个【供应商集团】下
             * @param:
             * @return: String
             */
            private String validateIsOneEntrySuppliers(ExtendedDataEntity bills) {
                DynamicObject dataEntity = bills.getDataEntity();
                String errorMsg = "";
                ArrayList<QFilter> ddFilters = new ArrayList<>();
                ddFilters.add(new QFilter("id", QCP.not_equals, dataEntity.getLong("id")));
                ddFilters.add(new QFilter("tpv_invalids", QCP.equals, false));
                DynamicObject[] load = BusinessDataServiceHelper.load("tpv_crm_custmain",
                        "id,tpv_cusgroup,tpv_crm_cuslimit,tpv_crm_cuslimit.tpv_custom", ddFilters.toArray(new QFilter[0]));
                Map<String,List<String>> mapSupBySupGroup = new HashMap<>();
                List<String> supGroupList = new ArrayList<>();
                if (load.length > 0) {
                    //查询历史数据
                    for (DynamicObject dynamicObject : load) {
                        String tpvSuppliergroup = dynamicObject.getString("tpv_cusgroup");
                        DynamicObjectCollection dynamicObjects = dynamicObject.getDynamicObjectCollection("tpv_crm_cuslimit");
                        if (dynamicObjects.size() > 0) {
                            for (DynamicObject object : dynamicObjects) {
                                DynamicObject tpvSuppliers = object.getDynamicObject("tpv_custom");
                                if (YEM.isNotEmpty(tpvSuppliers)) {
                                    String supId = tpvSuppliers.getString("id");
                                    if (!mapSupBySupGroup.containsKey(supId)) {
                                        supGroupList = new ArrayList<>();
                                        supGroupList.add(tpvSuppliergroup);
                                        mapSupBySupGroup.put(supId,supGroupList);
                                    } else {
                                        List<String> listSupGroup = mapSupBySupGroup.get(supId);
                                        if (!listSupGroup.contains(tpvSuppliergroup)) {
                                            listSupGroup.add(tpvSuppliergroup);
                                            mapSupBySupGroup.put(supId,listSupGroup);
                                        }
                                    }
                                }
                            }
                        }
                    }
                    //校验本单供应商
                    DynamicObjectCollection tpvSupplierlimitSup = dataEntity.getDynamicObjectCollection("tpv_crm_cuslimit");
                    if (tpvSupplierlimitSup.size() > 0) {
                        String tpvSuppliergroup = dataEntity.getString("tpv_cusgroup");
                        Map<String,String> mapSup = new HashMap<>();
                        Map<String,String> mapSupName = new HashMap<>();
                        List<String> rows = new ArrayList<>();
                        for (int i = 0; i < tpvSupplierlimitSup.size(); i++) {
                            DynamicObject dynamicObject = tpvSupplierlimitSup.get(i);
                            DynamicObject tpvSuppliers = dynamicObject.getDynamicObject("tpv_custom");
                            if (YEM.isNotEmpty(tpvSuppliers)) {
                                String supId = tpvSuppliers.getString("id");
                                String supName = tpvSuppliers.getString("name");
                                if (!mapSup.containsKey(supId)) {
                                    mapSup.put(supId,tpvSuppliergroup);
                                    mapSupName.put(supId,supName);
                                }
                            }
                        }
                        if (mapSup.size() > 0) {
                            for (Map.Entry<String, String> supEntry : mapSup.entrySet()) {
                                if (mapSupBySupGroup.containsKey(supEntry.getKey())) {
                                    List<String> strings = mapSupBySupGroup.get(supEntry.getKey());
                                    if (strings.size() > 0) {
                                        if (strings.size() == 1) {
                                            if (!strings.contains(supEntry.getValue())) {
                                                if (mapSupName.containsKey(supEntry.getKey())) {
                                                    rows.add(mapSupName.get(supEntry.getKey()));
                                                } else {
                                                    rows.add(supEntry.getKey());
                                                }

                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (rows.size() > 0) {
                            errorMsg = "查询所有历史数据，本单分录中: 客户 " + String.join(",", rows)
                                    + ",已存在请检查!\n";
                        }
                    }
                }
                return errorMsg;
            }

            private void Updatecombofield(ExtendedDataEntity dataEntities) {
                DynamicObject dataEntity = dataEntities.getDataEntity();
                String tpvValidtype = dataEntity.getString("tpv_validtype");//单据类型为长期的
                if ("A".equals(tpvValidtype)) {
                    String tpvOrgcredit = dataEntity.getString("tpv_orgcredit");//公司/部门
                    String billno = dataEntity.getString("billno");
                    String tpv_cusgroup = dataEntity.getString("tpv_cusgroup");//客户集团
                    BigDecimal tpv_groupconamts = dataEntity.getBigDecimal("tpv_groupconamt");
                    BigDecimal tpv_groupdelims = dataEntity.getBigDecimal("tpv_groupdelim");
                    DynamicObjectCollection entrys = dataEntity.getDynamicObjectCollection("tpv_crm_cuslimit");
                    for (DynamicObject entry : entrys) {
                        int row = entry.getInt("Seq");//可以加减
                        String tpv_quotatype = entry.getString("tpv_quotatype");
                        if (tpv_quotatype != null && "A".equals(tpv_quotatype)) {
                            BigDecimal amount = entry.getBigDecimal("tpv_limitamt");//签约额度
                            if (amount.compareTo(tpv_groupconamts) == 1) {
                                this.addMessage(dataEntities, "分录第" + row + "行签约金额已超出表头签约金额，请检查！！！");
                            }
                        } else if (tpv_quotatype != null && "B".equals(tpv_quotatype)) {
                            BigDecimal limitamt = entry.getBigDecimal("tpv_limitamt");//送货额度
                            if (limitamt.compareTo(tpv_groupdelims) == 1) {
                                this.addMessage(dataEntities, "分录第" + row + "行送货金额已超出表头送货金额，请检查！！！");
                            }
                        }
                    }
//                    QFilter qFilter = new QFilter("tpv_cusgroup", QCP.equals, tpv_cusgroup);
//                    qFilter.and("tpv_orgcredit", QCP.equals, tpvOrgcredit);
//                    qFilter.and("billno", QCP.not_equals, billno);
//                    DynamicObject[] tpv_crm_custmain = BusinessDataServiceHelper.load("tpv_crm_custmain", "id,billno,tpv_orgcredit,tpv_cusgroup"
//                            , new QFilter[]{qFilter}, null);
//                    if (tpv_crm_custmain.length > 0) {
//                        for (int i = 0; i < tpv_crm_custmain.length; i++) {
//                            DynamicObject custmain = tpv_crm_custmain[i];
//                            String orgcredit = custmain.getString("tpv_orgcredit");
//                            if ("A".equals(orgcredit)) {
//                                this.addMessage(dataEntities, "同一公司，同一【客户集团】下只能有一个长期有效的《客户额度维护》！！！");
//                                return;
//
//                            } else if ("B".equals(orgcredit)) {
//                                this.addMessage(dataEntities, "同一部门，同一【客户集团】下只能有一个长期有效的《客户额度维护》！！！");
//                                return;
//                            }
//                        }
//                    }
                    if ("B".equals(tpvOrgcredit)) {
                        BigDecimal a = BigDecimal.ZERO;
                        BigDecimal b = BigDecimal.ZERO;
                        BigDecimal c = BigDecimal.ZERO;
                        BigDecimal d = BigDecimal.ZERO;
                        QFilter qFilters = new QFilter("tpv_cusgroup", QCP.equals, tpv_cusgroup);
                        qFilters.and("tpv_orgcredit", QCP.equals, "A");
                        DynamicObject[] tpv_custmain = BusinessDataServiceHelper.load("tpv_crm_custmain", "id,billno,tpv_orgcredit,tpv_cusgroup"
                                , new QFilter[]{qFilters}, null);
                        if (tpv_custmain.length == 0) {
                            this.addMessage(dataEntities, "【客户集团】下必须先存在【公司】授信，才能存在【部门】授信");
                            return;
                        }

                        QFilter qFiltere = new QFilter("tpv_cusgroup", QCP.equals, tpv_cusgroup);
                        qFiltere.and("tpv_orgcredit", QCP.equals, "A");
                        DynamicObject[] tpvcustmain = BusinessDataServiceHelper.load("tpv_crm_custmain", "id,billno,tpv_orgcredit,tpv_cusgroup,tpv_groupconamt,tpv_groupdelim"
                                , new QFilter[]{qFiltere}, null);
                        if (tpvcustmain.length > 0) {
                            for (int j = 0; j < tpvcustmain.length; j++) {
                                DynamicObject custmain = tpvcustmain[j];
                                BigDecimal tpv_groupconamt = custmain.getBigDecimal("tpv_groupconamt");//签约额度
                                BigDecimal tpv_groupdelim = custmain.getBigDecimal("tpv_groupdelim");//送货额度
                                a = a.add(tpv_groupconamt);
                                b = b.add(tpv_groupdelim);
                            }
                        }
                        BigDecimal tpv_groupconamt = dataEntity.getBigDecimal("tpv_groupconamt");//签约额度
                        BigDecimal tpv_groupdelim = dataEntity.getBigDecimal("tpv_groupdelim");//送货额度
                        String tpv_documentype = dataEntity.getString("tpv_validtype");//单据类型
                        if ("A".equals(tpv_documentype)) {
                            c = c.add(tpv_groupconamt);
                            d = d.add(tpv_groupdelim);
                        }
                        if (a.compareTo(c) == -1) {
                            this.addMessage(dataEntities, "【部门】总签约额度不能大于【公司】总签约额度！！！");
                        }
                        if (b.compareTo(d) == -1) {
                            this.addMessage(dataEntities, "【部门】总送货额度不能大于【公司】总送货额度！！！");
                        }
                    }
                }
            }
        });
    }
}
