package tpv.tpv_wm.tpv_crm.java.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ClassName: SupplierLimitOp
 * Description: 供应商额度维护服务插件
 * date: 2023/10/26 15:25
 *
 * @author: Guo jiaChen
 */
public class SupplierLimitOp extends AbstractOperationServicePlugIn {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_credit");
        keys.add("tpv_signquota");
        keys.add("tpv_forwordquota");
        keys.add("tpv_spotpayquota");
        keys.add("tpv_suppliergroup");
        keys.add("billstatus");
        keys.add("tpv_billtype");
        keys.add("tpv_ifeffective");
        keys.add("tpv_supplierlimit_sup");
        keys.add("tpv_quotatype");
        keys.add("tpv_suppliers");
        keys.add("tpv_amountquota");
        keys.add("tpv_billtype");
        e.setFieldKeys(keys);
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                StringBuilder errorMsg = new StringBuilder("");
                ExtendedDataEntity[] bills = this.getDataEntities();
                if ("submit".equals(this.getOperateKey())) {
                    for (int i = 0; i < bills.length; ++i) {
                        ExtendedDataEntity bill = bills[i];
                        errorMsg.append(validateIsFreeze(bill));
                        errorMsg.append(validateIsOneEntrySuppliers(bill));
//                        errorMsg.append(validateIsAmount(bill));
//                        errorMsg.append(validateIsSuppliers(bill));
//                        errorMsg.append(validateCreditLimit(bill));
//                        errorMsg.append(validateIsGroupOnly(bill));
                        if (!errorMsg.toString().isEmpty()){
                            this.addMessage(bill,errorMsg.toString());
                        }
                    }
                }
            }

            /**
             * @Title: validateIsSuppliers
             * @Description: 校验{分录-供应商额度 }中供应商，只允许存在一个【供应商集团】下
             * @param:
             * @return: String
             */
            private String validateIsOneEntrySuppliers(ExtendedDataEntity bills) {
                DynamicObject dataEntity = bills.getDataEntity();
                String errorMsg = "";
                ArrayList<QFilter> ddFilters = new ArrayList<>();
                ddFilters.add(new QFilter("id", QCP.not_equals, dataEntity.getLong("id")));
                ddFilters.add(new QFilter("tpv_ifeffective", QCP.equals, false));
                DynamicObject[] load = BusinessDataServiceHelper.load("tpv_bd_supplierlimits",
                        "id,tpv_suppliergroup,tpv_supplierlimit_sup,tpv_supplierlimit_sup.tpv_suppliers", ddFilters.toArray(new QFilter[0]));
                Map<String,List<String>> mapSupBySupGroup = new HashMap<>();
                List<String> supGroupList = new ArrayList<>();
                if (load.length > 0) {
                    //查询历史数据
                    for (DynamicObject dynamicObject : load) {
                        String tpvSuppliergroup = dynamicObject.getString("tpv_suppliergroup");
                        DynamicObjectCollection dynamicObjects = dynamicObject.getDynamicObjectCollection("tpv_supplierlimit_sup");
                        if (dynamicObjects.size() > 0) {
                            for (DynamicObject object : dynamicObjects) {
                                DynamicObject tpvSuppliers = object.getDynamicObject("tpv_suppliers");
                                if (YEM.isNotEmpty(tpvSuppliers)) {
                                    String supId = tpvSuppliers.getString("id");
                                    if (!mapSupBySupGroup.containsKey(supId)) {
                                        supGroupList = new ArrayList<>();
                                        supGroupList.add(tpvSuppliergroup);
                                        mapSupBySupGroup.put(supId,supGroupList);
                                    } else {
                                        List<String> listSupGroup = mapSupBySupGroup.get(supId);
                                        if (!listSupGroup.contains(tpvSuppliergroup)) {
                                            listSupGroup.add(tpvSuppliergroup);
                                            mapSupBySupGroup.put(supId,listSupGroup);
                                        }
                                    }
                                }
                            }
                        }
                    }
                    //校验本单供应商
                    DynamicObjectCollection tpvSupplierlimitSup = dataEntity.getDynamicObjectCollection("tpv_supplierlimit_sup");
                    if (tpvSupplierlimitSup.size() > 0) {
                        String tpvSuppliergroup = dataEntity.getString("tpv_suppliergroup");
                        Map<String,String> mapSup = new HashMap<>();
                        Map<String,String> mapSupName = new HashMap<>();
                        List<String> rows = new ArrayList<>();
                        for (int i = 0; i < tpvSupplierlimitSup.size(); i++) {
                            DynamicObject dynamicObject = tpvSupplierlimitSup.get(i);
                            DynamicObject tpvSuppliers = dynamicObject.getDynamicObject("tpv_suppliers");
                            if (YEM.isNotEmpty(tpvSuppliers)) {
                                String supId = tpvSuppliers.getString("id");
                                String supName = tpvSuppliers.getString("name");
                                if (!mapSup.containsKey(supId)) {
                                    mapSup.put(supId,tpvSuppliergroup);
                                    mapSupName.put(supId,supName);
                                }
                            }
                        }
                        if (mapSup.size() > 0) {
                            for (Map.Entry<String, String> supEntry : mapSup.entrySet()) {
                                if (mapSupBySupGroup.containsKey(supEntry.getKey())) {
                                        List<String> strings = mapSupBySupGroup.get(supEntry.getKey());
                                        if (strings.size() > 0) {
                                            if (strings.size() == 1) {
                                                if (!strings.contains(supEntry.getValue())) {
                                                    if (mapSupName.containsKey(supEntry.getKey())) { //判断是否根据供应商id，有对应名称
                                                        rows.add(mapSupName.get(supEntry.getKey()));
                                                    } else {
                                                        rows.add(supEntry.getKey());
                                                    }

                                                }
                                            }
                                        }
                                }
                            }
                        }
                        if (rows.size() > 0) {
                            errorMsg = "查询所有历史数据，本单分录中: 供应商 " + String.join(",", rows)
                                    + ",已存在请检查!\n";
                        }
                    }
                }
                return errorMsg;
            }

            /**
             * @Title: validateIsSuppliers
             * @Description: 校验（根据【全司/部门授信】查询历史数据，过滤出所有分录供应商）本单不允许存在历史存在过的供应商
             * @param:
             * @return: String
             */
            private String validateIsSuppliers(ExtendedDataEntity bills) {
                DynamicObject dataEntity = bills.getDataEntity();
                String type = dataEntity.getString("tpv_billtype");
                if ("B".equals(type)){//单据类型 = 临时,不用校验
                    return "";
                }
                String errorMsg = "";
                String tpvCredit = dataEntity.getString("tpv_credit"); //公司/部门授信
                DynamicObject tpvOrg = dataEntity.getDynamicObject("tpv_org"); //公司/部门授信
                if (YEM.isNotEmpty(tpvCredit)) {
                    ArrayList<QFilter> ddFilters = new ArrayList<>();
                    if (YEM.isNotEmpty(tpvOrg)) {
                        ddFilters.add(new QFilter("tpv_org.id", QCP.equals, tpvOrg.getLong("id")));
                    }
                    ddFilters.add(new QFilter("tpv_credit", QCP.equals, tpvCredit));
                    ddFilters.add(new QFilter("id", QCP.not_equals, dataEntity.getLong("id")));
                    DynamicObject[] load = BusinessDataServiceHelper.load("tpv_bd_supplierlimits",
                            "id,tpv_supplierlimit_sup,tpv_supplierlimit_sup.tpv_suppliers", ddFilters.toArray(new QFilter[0]));
                    List<String> supList = new ArrayList<>();
                    if (load.length > 0) {
                        for (DynamicObject dynamicObject : load) {
                            DynamicObjectCollection dynamicObjects = dynamicObject.getDynamicObjectCollection("tpv_supplierlimit_sup");
                            if (dynamicObjects.size() > 0) {
                                for (DynamicObject object : dynamicObjects) {
                                    DynamicObject tpvSuppliers = object.getDynamicObject("tpv_suppliers");
                                    if (YEM.isNotEmpty(tpvSuppliers)) {
                                        String supId = tpvSuppliers.getString("id");
                                        supList.add(supId);
                                    }
                                }
                            }
                        }
                    } //去相同【全司/部门授信】下所有供应商历史数据
                    DynamicObjectCollection tpvSupplierlimitSup = dataEntity.getDynamicObjectCollection("tpv_supplierlimit_sup");
                    if (tpvSupplierlimitSup.size() > 0) {
                        List<String> rows = new ArrayList<>();
                        for (int i = 0; i < tpvSupplierlimitSup.size(); i++) {
                            DynamicObject dynamicObject = tpvSupplierlimitSup.get(i);
                            DynamicObject tpvSuppliers = dynamicObject.getDynamicObject("tpv_suppliers");
                            if (YEM.isNotEmpty(tpvSuppliers)) {
                                String supId = tpvSuppliers.getString("id");
                                boolean supOnlyFlag = supList.contains(supId);
                                if (supOnlyFlag) {
                                    int row = i + 1;
                                    rows.add(String.valueOf(row));
                                }
                            }
                        }
                        if (rows.size() > 0) {
                            errorMsg =   "根据【全司/单独授信】查询所有历史数据，本单分录中: " + String.join(",",rows)
                                    + ",供应商已存在请检查!\n";
                        }
                    }
                }
                return errorMsg;
            }

            /**
             * @Title: validateIsGroupOnly
             * @Description: 校验【供应商集团】 + 【全司/部门授信】唯一
             * @param:
             * @return: String
             */
            private String validateIsGroupOnly(ExtendedDataEntity bills) {
                DynamicObject dataEntity = bills.getDataEntity();
                String type = dataEntity.getString("tpv_billtype");
                if ("B".equals(type)){//单据类型 = 临时,不用校验
                    return "";
                }
                String errorMsg = "";
                String supplierGroup = dataEntity.getString("tpv_suppliergroup");
                Object pkValue = dataEntity.getPkValue();
                ArrayList<QFilter> ddFilters = new ArrayList<>();
                String tpvBilltype = dataEntity.getString("tpv_billtype");
                String tpvCredit = dataEntity.getString("tpv_credit"); //公司/部门授信
                if (YEM.isNotEmpty(tpvBilltype) && tpvBilltype.equals("A")) {//单据类型 = 长期
                    if ("A".equals(tpvCredit)) {
                        ddFilters.add(new QFilter("tpv_credit", QCP.equals, tpvCredit));
                    }
                    if ("B".equals(tpvCredit)) {
                        ddFilters.add(new QFilter("tpv_credit", QCP.equals, tpvCredit));
                    }
                    ddFilters.add(new QFilter("tpv_suppliergroup", QCP.equals, supplierGroup));
                    ddFilters.add(new QFilter("id", QCP.not_equals, pkValue));
                    DynamicObject[] load = BusinessDataServiceHelper.load("tpv_bd_supplierlimits", "", ddFilters.toArray(new QFilter[0]));
                    if (load.length > 0) {
                        errorMsg = "单据编号: " + dataEntity.getString("billno")
                                + ",供应商集团已存在请检查!\n";
                    }
                }
                return errorMsg;
            }

            /**
             * @Title: onAddValidators
             * @Description: 校验额度授信
             * @param:
             * @return: void
             */
            private String validateCreditLimit(ExtendedDataEntity dataEntities) {
                DynamicObject dataEntity = dataEntities.getDataEntity();
                String credit = dataEntity.getString("tpv_credit").equals("A") ? "公司" : "部门";
                BigDecimal signQuota = dataEntity.getBigDecimal("tpv_signquota");
                BigDecimal forwordQuota = dataEntity.getBigDecimal("tpv_forwordquota");
                BigDecimal spotpayQuota = dataEntity.getBigDecimal("tpv_spotpayquota");
                BigDecimal comSignQuota = BigDecimal.ZERO;
                BigDecimal comForwordQuota = BigDecimal.ZERO;
                BigDecimal comSpotpayQuota = BigDecimal.ZERO;
                BigDecimal deptSignQuota = BigDecimal.ZERO;
                BigDecimal deptForwordQuota = BigDecimal.ZERO;
                BigDecimal deptSpotpayQuota = BigDecimal.ZERO;
                String supplierGroup = dataEntity.getString("tpv_suppliergroup");
                ArrayList<QFilter> ddFilters = new ArrayList<>();
                ddFilters.add(new QFilter("tpv_billtype", QCP.equals, "A"));//单据类型 = 长期
                ddFilters.add(new QFilter("tpv_suppliergroup", QCP.equals, supplierGroup));//供应商集团
                ddFilters.add(new QFilter("tpv_ifeffective", QCP.equals, false));//未失效
                ddFilters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
                DataSet load = ORM.create().queryDataSet(this.getClass().getName(), "tpv_bd_supplierlimits",
                                "billstatus,tpv_signquota,tpv_credit,tpv_forwordquota,tpv_spotpayquota,tpv_suppliergroup"
                                , ddFilters.toArray(new QFilter[0]))
                        .select("tpv_signquota signquota","tpv_credit credit", "tpv_forwordquota forwordquota", "tpv_spotpayquota spotpayquota","tpv_suppliergroup suppliergroup");
                DataSet groupByCredit = load.executeSql("SELECT credit, sum(signquota) AS totalSignquota, sum(forwordquota) AS totalForwordquota, sum(spotpayquota) AS totalSpotpayquota\n" +
                        "GROUP BY credit");
                for (Row row : groupByCredit) {
                    String credit1 = row.getString("credit");
                    if ("A".equals(credit1)){
                        comSignQuota = row.getBigDecimal("totalSignquota");
                        comForwordQuota = row.getBigDecimal("totalForwordquota");
                        comSpotpayQuota = row.getBigDecimal("totalSpotpayquota");
                    }
                    if ("B".equals(credit1)){
                        deptSignQuota = row.getBigDecimal("totalSignquota");
                        deptForwordQuota = row.getBigDecimal("totalForwordquota");
                        deptSpotpayQuota = row.getBigDecimal("totalSpotpayquota");
                    }
                }

                String condition = "";
                if ("公司".equals(credit)){
                    String typeStr = "";
                    if (deptSignQuota.compareTo(comSignQuota) > 0){
                        typeStr += "集团签约额度,";
                    }
                    if (deptForwordQuota.compareTo(comForwordQuota) > 0){
                        typeStr += "集团远期实付额度,";
                    }
                    if (deptSpotpayQuota.compareTo(comSpotpayQuota) > 0){
                        typeStr += "集团现货额度,";
                    }

                    if (!typeStr.isEmpty()){
                        condition = "单据编号: " + dataEntity.getString("billno")
                                + ",表头字段:"+ typeStr + ",不得小于该供应商集团下各个部门对应额度类型的总和,请检查!\n";
                    }
                }
                if ("部门".equals(credit)){
                    String typeStr = "";
                    deptSignQuota = deptSignQuota.add(signQuota);
                    deptForwordQuota = deptForwordQuota.add(forwordQuota);
                    deptSpotpayQuota = deptSpotpayQuota.add(spotpayQuota);
                    if (deptSignQuota.compareTo(comSignQuota) > 0){
                        typeStr += "集团签约额度,";
                    }
                    if (deptForwordQuota.compareTo(comForwordQuota) > 0){
                        typeStr += "集团远期实付额度,";
                    }
                    if (deptSpotpayQuota.compareTo(comSpotpayQuota) > 0){
                        typeStr += "集团现货额度,";
                    }

                    if (!typeStr.isEmpty()){
                        condition = "单据编号: " + dataEntity.getString("billno")
                                + "该集团供应商下各个部门的:"+ typeStr + "的总和不得大于该供应商集团下公司对应类型的额度,请检查!\n";
                    }
                }
                return condition;
            }

            /**
             * @Title: onAddValidators
             * @Description: 额度类型 + 额度金额校验
             * @param:
             * @return: void
             */
            private String validateIsAmount(ExtendedDataEntity dataEntities) {
                DynamicObject dataEntity = dataEntities.getDataEntity();
                BigDecimal signQuota = dataEntity.getBigDecimal("tpv_signquota");
                BigDecimal forwordQuota = dataEntity.getBigDecimal("tpv_forwordquota");
                BigDecimal spotpayQuota = dataEntity.getBigDecimal("tpv_spotpayquota");
                DynamicObjectCollection supEntity = dataEntity.getDynamicObjectCollection("tpv_supplierlimit_sup");
                //String quotaType = supEntity.get(rowIndex).getString("tpv_quotatype");

                String errorMsg = "";
                String typeStr = "";
                int i = 1;
                for (DynamicObject object : supEntity) {
                    String curQuotaType = object.getString("tpv_quotatype");
                    BigDecimal amount = object.getBigDecimal("tpv_amountquota");
                    BigDecimal curAmount = BigDecimal.ZERO;
                    //签约额度
                    if ("A".equals(curQuotaType)){
                        curAmount = signQuota;
                        typeStr = "签约额度";
                    }
                    //远期实付额度
                    if ("B".equals(curQuotaType)){
                        curAmount = forwordQuota;
                        typeStr = "远期实付额度";
                    }
                    //现货实付额度
                    if ("C".equals(curQuotaType)) {
                        curAmount = spotpayQuota;
                        typeStr = "现货实付额度";
                    }
                    if (amount.compareTo(curAmount) > 0){
                        errorMsg += "单据编号: " + dataEntity.getString("billno")
                                + ",供应商额度分录第"+ i + "行,[额度金额]的数值已超出表头" + typeStr + "数值,请检查!" + "\n";
                    }
                    i++;
                }
                return errorMsg;
            }

            /**
             * @Title: onAddValidators
             * @Description: 供应商 + 额度类型 唯一校验
             * @param:
             * @return: void
             */
            private String validateIsFreeze(ExtendedDataEntity dataEntities) {
                DynamicObject dataEntity = dataEntities.getDataEntity();
                String type = dataEntity.getString("tpv_billtype");
                if ("B".equals(type)){//单据类型 = 临时,不用校验
                    return "";
                }
                HashMap<Long, String> map = new HashMap<>();
                DynamicObjectCollection supEntity = dataEntity.getDynamicObjectCollection("tpv_supplierlimit_sup");
                String errorMsg = "";
                int i = 1;
                for (DynamicObject object : supEntity) {
                    DynamicObject suppliers = object.getDynamicObject("tpv_suppliers");
                    if (null == suppliers){
                        continue;
                    }
                    long curSupID = suppliers.getLong("id");
                    String curQuotaType = object.getString("tpv_quotatype");
                    BigDecimal decimal = object.getBigDecimal("tpv_amountquota");//额度金额

                    if ("0E-10".equals(decimal.toString())){
                        errorMsg += "请填写供应商额度分录第" + i +"额度金额(万元)";
                        continue;
                    }
                    String quotaType = map.get(curSupID);
                    if (curQuotaType.equals(quotaType)){
                       errorMsg += "单据编号: " + dataEntity.getString("billno")
                                        + ",供应商额度分录第"+ i + "行,[" + suppliers.getString("name") + "]的额度类型重复,请检查!" + "\n";
                    }
                    map.put(curSupID,curQuotaType);
                    i++;
                }
                return errorMsg;
            }
        });
    }
}
