package tpv.tpv_wm.tpv_crm.java.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ClassName: SupplierLimitSaveOp
 * Description: 供应商额度维护服务插件
 * date: 2023/10/26 15:25
 *
 * @author: Guo jiaChen
 */
public class SupplierLimitSaveOp extends AbstractOperationServicePlugIn {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_credit");
        keys.add("tpv_signquota");
        keys.add("tpv_forwordquota");
        keys.add("tpv_spotpayquota");
        keys.add("tpv_suppliergroup");
        keys.add("billstatus");
        keys.add("tpv_billtype");
        keys.add("tpv_ifeffective");
        keys.add("tpv_supplierlimit_sup");
        keys.add("tpv_quotatype");
        keys.add("tpv_suppliers");
        keys.add("tpv_amountquota");
        keys.add("tpv_billtype");
        e.setFieldKeys(keys);
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                StringBuilder errorMsg = new StringBuilder("");
                ExtendedDataEntity[] bills = this.getDataEntities();
                if ("save".equals(this.getOperateKey())) {
                    for (int i = 0; i < bills.length; ++i) {
                        ExtendedDataEntity bill = bills[i];
                        errorMsg.append(validateIsOneEntrySuppliers(bill));
                        if (!errorMsg.toString().isEmpty()){
                            this.addMessage(bill,errorMsg.toString());
                        }
                    }
                }
            }

            /**
             * @Title: validateIsSuppliers
             * @Description: 校验{分录-供应商额度 }中供应商，只允许存在一个【供应商集团】下
             * @param:
             * @return: String
             */
            private String validateIsOneEntrySuppliers(ExtendedDataEntity bills) {
                DynamicObject dataEntity = bills.getDataEntity();
                String errorMsg = "";
                ArrayList<QFilter> ddFilters = new ArrayList<>();
                ddFilters.add(new QFilter("id", QCP.not_equals, dataEntity.getLong("id")));
                ddFilters.add(new QFilter("tpv_ifeffective", QCP.equals, false));
                DynamicObject[] load = BusinessDataServiceHelper.load("tpv_bd_supplierlimits",
                        "id,tpv_suppliergroup,tpv_supplierlimit_sup,tpv_supplierlimit_sup.tpv_suppliers", ddFilters.toArray(new QFilter[0]));
                Map<String,List<String>> mapSupBySupGroup = new HashMap<>();
                List<String> supGroupList = new ArrayList<>();
                if (load.length > 0) {
                    //查询历史数据
                    for (DynamicObject dynamicObject : load) {
                        String tpvSuppliergroup = dynamicObject.getString("tpv_suppliergroup");
                        DynamicObjectCollection dynamicObjects = dynamicObject.getDynamicObjectCollection("tpv_supplierlimit_sup");
                        if (dynamicObjects.size() > 0) {
                            for (DynamicObject object : dynamicObjects) {
                                DynamicObject tpvSuppliers = object.getDynamicObject("tpv_suppliers");
                                if (YEM.isNotEmpty(tpvSuppliers)) {
                                    String supId = tpvSuppliers.getString("id");
                                    if (!mapSupBySupGroup.containsKey(supId)) {
                                        supGroupList = new ArrayList<>();
                                        supGroupList.add(tpvSuppliergroup);
                                        mapSupBySupGroup.put(supId,supGroupList);
                                    } else {
                                        List<String> listSupGroup = mapSupBySupGroup.get(supId);
                                        if (!listSupGroup.contains(tpvSuppliergroup)) {
                                            listSupGroup.add(tpvSuppliergroup);
                                            mapSupBySupGroup.put(supId,listSupGroup);
                                        }
                                    }
                                }
                            }
                        }
                    }
                    //校验本单供应商
                    DynamicObjectCollection tpvSupplierlimitSup = dataEntity.getDynamicObjectCollection("tpv_supplierlimit_sup");
                    if (tpvSupplierlimitSup.size() > 0) {
                        String tpvSuppliergroup = dataEntity.getString("tpv_suppliergroup");
                        Map<String,String> mapSup = new HashMap<>();
                        Map<String,String> mapSupName = new HashMap<>();
                        List<String> rows = new ArrayList<>();
                        for (int i = 0; i < tpvSupplierlimitSup.size(); i++) {
                            DynamicObject dynamicObject = tpvSupplierlimitSup.get(i);
                            DynamicObject tpvSuppliers = dynamicObject.getDynamicObject("tpv_suppliers");
                            if (YEM.isNotEmpty(tpvSuppliers)) {
                                String supId = tpvSuppliers.getString("id");
                                String supName = tpvSuppliers.getString("name");
                                if (!mapSup.containsKey(supId)) {
                                    mapSup.put(supId,tpvSuppliergroup);
                                    mapSupName.put(supId,supName);
                                }
                            }
                        }
                        if (mapSup.size() > 0) {
                            for (Map.Entry<String, String> supEntry : mapSup.entrySet()) {
                                if (mapSupBySupGroup.containsKey(supEntry.getKey())) {
                                        List<String> strings = mapSupBySupGroup.get(supEntry.getKey());
                                        if (strings.size() > 0) {
                                            if (strings.size() == 1) {
                                                if (!strings.contains(supEntry.getValue())) {
                                                    if (mapSupName.containsKey(supEntry.getKey())) { //判断是否根据供应商id，有对应名称
                                                        rows.add(mapSupName.get(supEntry.getKey()));
                                                    } else {
                                                        rows.add(supEntry.getKey());
                                                    }

                                                }
                                            }
                                        }
                                }
                            }
                        }
                        if (rows.size() > 0) {
                            errorMsg = "查询所有历史数据，本单分录中: 供应商 " + String.join(",", rows)
                                    + ",已存在请检查!\n";
                        }
                    }
                }
                return errorMsg;
            }
        });
    }
}
