package tpv.tpv_wm.tpv_crm.java.report;

import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.sdk.plugin.Plugin;
import tpv.tpv_wm.tpv_dt.java.utils.PersonnelDepUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @Description: 供应商预付款占用-合同明细  -报表查询
 * @Author LinRui
 * @Date 2023/10/12 9:54
 */
public class SupCreditConRptQuery extends AbstractReportListDataPlugin implements Plugin {
    private static final String algoKey = SupCreditConRptQuery.class.getName();

    @Override
    public DataSet query(ReportQueryParam reportQueryParam, Object obj) throws Throwable {
        FilterInfo filter = reportQueryParam.getFilter();
        String whereFilter = createWhereStr(reportQueryParam);
        //占用余额全部为0不显示
        boolean tpv_zero_f = filter.getBoolean("tpv_zero_f");
        DataSet instockDataSet = query(whereFilter);
        if (tpv_zero_f) { // 占用余额全部为0不显示
            instockDataSet = instockDataSet.where("tpv_residuesigning <> 0 or tpv_residueafter <> 0 or tpv_residuespots <> 0 or tpv_residueforward <> 0");
        }
        String whereFi="2=2";
        String whereStr = PersonnelDepUtils.createWhereStr(filter, "tpv_dep", "tpv_muldepartment");
        if (!"".equals(whereStr)){
            whereFi=String.format("%s%s", whereFi, whereStr);
        }
        return instockDataSet.where(whereFi);
    }

    public static DataSet query(String whereFilter) {
        DataSet mainDataSet = mainDataSet(whereFilter);
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        for (DynamicObject col : col_s) {
            String tpvSource = col.getString("tpv_source");
            if ("A".equals(tpvSource)) {
                BigDecimal tpvAddafter = col.getBigDecimal("tpv_addafter"); //货后(占用增项)
                BigDecimal tpvAddspots = col.getBigDecimal("tpv_addspots"); //现货(占用增项)
                BigDecimal tpvAddforward = col.getBigDecimal("tpv_addforward"); //远期(占用增项)
                BigDecimal tpvDeduction = col.getBigDecimal("tpv_deduction"); //占用减项
                BigDecimal tpv_residueafter = tpvAddafter.subtract(tpvDeduction); // 货后(占用余额)
                BigDecimal subtractAfter = tpvDeduction.subtract(tpvAddafter); //占用减项剩余
                if (tpv_residueafter.compareTo(BigDecimal.ZERO) < 0) {
                    col.set("tpv_residueafter", 0.0);
                } else {
                    col.set("tpv_residueafter", tpv_residueafter);
                }
                if (subtractAfter.compareTo(BigDecimal.ZERO) <= 0) {
                    col.set("tpv_residuespots", tpvAddspots);
                    col.set("tpv_residueforward", tpvAddforward);
                } else {
                    BigDecimal tpv_residuespots = tpvAddspots.subtract(subtractAfter); // 现货(占用余额)
                    BigDecimal subtractSpots = subtractAfter.subtract(tpvAddspots); //占用减项剩余
                    if (tpv_residuespots.compareTo(BigDecimal.ZERO) < 0) {
                        col.set("tpv_residuespots", 0.0);
                    } else {
                        col.set("tpv_residuespots", tpv_residuespots);
                    }
                    if (subtractSpots.compareTo(BigDecimal.ZERO) <= 0) {
                        col.set("tpv_residueforward", tpvAddforward);
                    } else {
                        BigDecimal tpv_residueforward = tpvAddforward.subtract(subtractSpots); // 远期(占用余额)
                        col.set("tpv_residueforward", tpv_residueforward);
                    }
                }
            }
        }

        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(SupCreditConRptQuery.class.getName()).createDataSet(inputs);
        return createDataSet.orderBy(new String[]{"tpv_bizdate desc"});
    }

    /**
     * 将部门形态为小组的部门，换成上级部门，此方法用来为供应商额度使用情况表联查汇总占用余额数据
     * */
    public static DataSet queryLeftJoinUse(String whereFilter) {
        DataSet mainDataSet = mainDataSet(whereFilter);
        Map<Long, Long> parentDeptIds = mapParentDeptIds();
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        for (DynamicObject col : col_s) {
            Long tpvDep = col.getLong("tpv_dep");
            if (tpvDep.compareTo(0l) > 0) {
                if (parentDeptIds.containsKey(tpvDep)) {
                    Long aLong = parentDeptIds.get(tpvDep);
                    col.set("tpv_dep",aLong);
                }
            }
            String tpvSource = col.getString("tpv_source");
            if ("A".equals(tpvSource)) {
                BigDecimal tpvAddafter = col.getBigDecimal("tpv_addafter"); //货后(占用增项)
                BigDecimal tpvAddspots = col.getBigDecimal("tpv_addspots"); //现货(占用增项)
                BigDecimal tpvAddforward = col.getBigDecimal("tpv_addforward"); //远期(占用增项)
                BigDecimal tpvDeduction = col.getBigDecimal("tpv_deduction"); //占用减项
                BigDecimal tpv_residueafter = tpvAddafter.subtract(tpvDeduction); // 货后(占用余额)
                BigDecimal subtractAfter = tpvDeduction.subtract(tpvAddafter); //占用减项剩余
                if (tpv_residueafter.compareTo(BigDecimal.ZERO) < 0) {
                    col.set("tpv_residueafter", 0.0);
                } else {
                    col.set("tpv_residueafter", tpv_residueafter);
                }
                if (subtractAfter.compareTo(BigDecimal.ZERO) <= 0) {
                    col.set("tpv_residuespots", tpvAddspots);
                    col.set("tpv_residueforward", tpvAddforward);
                } else {
                    BigDecimal tpv_residuespots = tpvAddspots.subtract(subtractAfter); // 现货(占用余额)
                    BigDecimal subtractSpots = subtractAfter.subtract(tpvAddspots); //占用减项剩余
                    if (tpv_residuespots.compareTo(BigDecimal.ZERO) < 0) {
                        col.set("tpv_residuespots", 0.0);
                    } else {
                        col.set("tpv_residuespots", tpv_residuespots);
                    }
                    if (subtractSpots.compareTo(BigDecimal.ZERO) <= 0) {
                        col.set("tpv_residueforward", tpvAddforward);
                    } else {
                        BigDecimal tpv_residueforward = tpvAddforward.subtract(subtractSpots); // 远期(占用余额)
                        col.set("tpv_residueforward", tpv_residueforward);
                    }
                }
            }
        }

        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(SupCreditConRptQuery.class.getName()).createDataSet(inputs);
        return createDataSet.orderBy(new String[]{"tpv_bizdate desc"});
    }

    private static Map<Long,Long> mapParentDeptIds(){
        Map<Long,Long> mapParentId = new HashMap<>();
        QFilter qFilter = new QFilter("enable", QCP.equals, "1");
        qFilter.and(new QFilter("orgpattern.number",QCP.equals,"OF-01"));
        DynamicObject[] bosAdminorgs = BusinessDataServiceHelper.load("bos_adminorg","id,structure,structure.viewparent", qFilter.toArray());
        if (bosAdminorgs.length > 0) {
            for (DynamicObject bosAdminorg : bosAdminorgs) {
                DynamicObjectCollection dynamicObjects = bosAdminorg.getDynamicObjectCollection("structure");
                if (dynamicObjects.size() > 0) {
                    for (DynamicObject dynamicObject : dynamicObjects) {
                        DynamicObject viewparent = dynamicObject.getDynamicObject("viewparent");
                        if (YEM.isNotEmpty(viewparent)) {
                            mapParentId.put(bosAdminorg.getLong("id"),viewparent.getLong("id"));
                        }
                    }
                }
            }
        }
        return mapParentId;
    }

    private static DataSet mainDataSet(String whereFilter) {
        QFilter[] qFilters = new QFilter[]{new QFilter("billstatus", QCP.equals, "C")};
        DataSet tpv_dt_purchase = purchase(qFilters);

        //以来源分录id为主维度，再以汇款关系类型，求和，算出货后(占用增项)
        DataSet addAmt = addAmt(tpv_dt_purchase);
        //以来源分录id为主维度，关联查入库前（货物流转单）、入库后（成本明细单）(占用减项)
        DataSet deduction = deduction(tpv_dt_purchase);

        DataSet dataSet = addAmt.leftJoin(deduction).on("id", "id")
                .select(new String[]{"id", "tpv_dep", "tpv_sup", "tpv_source", "tpv_purorsupno", "tpv_bizdate",
                        "tpv_addsigning", "tpv_addafter", "tpv_addspots", "tpv_addforward", "tpv_deduction",
                        "(case when tpv_cargostatus == 'B' or tpv_cargostatus == 'C' then 0.0 else tpv_addsigning - tpv_deduction end) tpv_residuesigning",
                        "0.0 tpv_residueafter", "0.0 tpv_residuespots", "0.0 tpv_residueforward", "tpv_isusesingle"}).finish();
        //DataSet dataSet = tpv_dt_purchase.union(tpv_dt_agreement);
        dataSet = RptUtil.zeroNullNums(dataSet);
        DataSet tpv_dt_agreement = supAgreement(qFilters); //供应商长协
        dataSet = dataSet.union(tpv_dt_agreement);
        if (YEM.isNotEmpty(whereFilter)) {
            dataSet = dataSet.where(whereFilter);
        }
        return dataSet;
    }

    private static DataSet supAgreement(QFilter[] qFilters) {
        ArrayList<QFilter> list = new ArrayList<>();
        list.add(new QFilter("billstatus", QCP.equals, "C"));
        list.add(new QFilter("tpv_protocoltype",QCP.in,new String[]{"A","C"}));
        DataSet tpv_dt_agreement = ORM.create().queryDataSet(algoKey, "tpv_dt_agreement",
                "billno,id,tpv_department tpv_dep,tpv_supplier tpv_sup,'B' tpv_source,tpv_protocol tpv_purorsupno,tpv_bizdate", list.toArray(new QFilter[list.size()])).distinct();
        //付款申请
        QFilter qFilterPay = new QFilter("billstatus",QCP.equals, "C");
        DataSet tpv_ap_payapply_ext = ORM.create().queryDataSet(algoKey, "ap_payapply",
                "entry.shkd_dt_agreement.tpv_protocol agreementNo,entry.e_payamount amount,entry.e_expenseitem.number e_expenseitem",
                qFilterPay.toArray());
        //收款处理
        QFilter qFilterRec = new QFilter("billstatus",QCP.equals,"D");
        DataSet shkd_cas_recbill_ext = ORM.create().queryDataSet(algoKey, "cas_recbill",
                "entry.shkd_dt_agreement.tpv_protocol agreementNo,entry.e_receivableamt amount,entry.e_expenseitem.number e_expenseitem",
                qFilterRec.toArray());
        DataSet finish = tpv_ap_payapply_ext.union(shkd_cas_recbill_ext);
        List<String> aliasList = new ArrayList<>();
        aliasList.add("agreementNo");
        aliasList.add("0.0 tpv_addafter");
        aliasList.add("0.0 tpv_addspots");
        aliasList.add("(case when e_expenseitem == 'FYXM150.1020' then amount else 0.0 end) tpv_addforward");
        finish = finish.select(aliasList.toArray(aliasList.toArray(new String[0])))
                .groupBy(new String[]{"agreementNo"})
                .sum("tpv_addafter").sum("tpv_addspots").sum("tpv_addforward").finish();

        DataSet dataSet = tpv_dt_agreement.leftJoin(finish).on("tpv_purorsupno", "agreementNo")
                .select(new String[]{"id", "tpv_dep", "tpv_sup", "tpv_source", "tpv_purorsupno", "tpv_bizdate",
                        "0.0 tpv_addsigning", "tpv_addafter", "tpv_addspots", "tpv_addforward","0.0 tpv_deduction", "0.0 tpv_residuesigning",
                        "0.0 tpv_residueafter", "0.0 tpv_residuespots", "tpv_addforward tpv_residueforward", "false tpv_isusesingle"}).finish();

        dataSet = RptUtil.zeroNullNums(dataSet);
        return dataSet;
    }

    private static DataSet purchase(QFilter[] qFilters) {
        DataSet dataSet = ORM.create().queryDataSet(algoKey, "tpv_dt_purchase",
                        "billno,id,tpv_cargostatus,tpv_department tpv_dep,tpv_supplier tpv_sup,'A' tpv_source,tpv_contractnumber tpv_purorsupno,tpv_bizdate," +
                                "tpv_materialentry,tpv_materialentry.id entryidM,(tpv_materialentry.tpv_effectiveqty * tpv_materialentry.tpv_taxprice) tpv_addsigning," +
                                "0 - tpv_prepoccupancy tpv_prepoccupancy,tpv_useasingleticket tpv_isusesingle", qFilters)
                .groupBy(new String[]{"id", "billno", "tpv_dep", "tpv_sup", "tpv_source", "tpv_cargostatus", "tpv_bizdate", "tpv_purorsupno", "tpv_prepoccupancy", "tpv_isusesingle"})
                .sum("tpv_addsigning").finish();
        return dataSet;
    }

    private static DataSet deduction(DataSet tpv_dt_purchase) {
        QFilter[] qFiltersHW = new QFilter[]{new QFilter("billstatus", QCP.equals, "C")};
        //查询货物流转单
        DataSet tpvDtFreigroambill = ORM.create().queryDataSet(algoKey, "tpv_dt_freigroambill",
                        "id,billno,tpv_delivery,tpv_sourcebillid,tpv_materialentry,tpv_materialentry.id entryid,tpv_materialentry.tpv_cobillid tpv_srcbillid," +
                                "tpv_materialentry.tpv_srcentryid tpv_srcentryid," +
                                "(tpv_materialentry.tpv_qty - tpv_materialentry.tpv_alreroamnum - tpv_materialentry.tpv_tinnum) * tpv_materialentry.tpv_taxprice tpv_hwcbamt", qFiltersHW)
                .select("tpv_srcbillid","(case when tpv_delivery in('A','B') then tpv_hwcbamt else 0.0 end) tpv_hwcbamt")
                .groupBy(new String[]{"tpv_srcbillid"})
                .sum("tpv_hwcbamt").finish();
        QFilter[] qFiltersCB = new QFilter[]{new QFilter("billstatus", QCP.equals, "C")
                .and(new QFilter("tpv_tiggdoutype.number", QCP.not_in, new String[]{"im_otherinbill","im_otheroutbill"}))
                .and(new QFilter("tpv_costdirec", QCP.equals, "A"))
                .and(new QFilter("tpv_costype.number", QCP.in, new String[]{"FYXM150.1101"}))};
        //查询成品明细单
        DataSet tpvCmCostdispose = ORM.create().queryDataSet(algoKey, "tpv_cm_costdispose", "id,billno,tpv_purconnum.id tpv_srcbillid,tpv_sourcebillid,tpv_origampintax tpv_hwcbamt", qFiltersCB)
                .groupBy(new String[]{"tpv_srcbillid"})
                .sum("tpv_hwcbamt").finish();
        DataSet union = tpvDtFreigroambill.union(tpvCmCostdispose).groupBy(new String[]{"tpv_srcbillid"}).sum("tpv_hwcbamt").finish();
        DataSet hwOrCbAmt = RptUtil.zeroNullNums(union);
        DataSet dataSet = tpv_dt_purchase.leftJoin(hwOrCbAmt).on("id", "tpv_srcbillid")
                .select(new String[]{"id", "tpv_hwcbamt - tpv_prepoccupancy tpv_deduction"}).finish()
                .select(new String[]{"id", "(case when tpv_deduction != null then tpv_deduction else 0.0 end) tpv_deduction"});
        dataSet = RptUtil.zeroNullNums(dataSet);
        return dataSet;
    }

    private static DataSet addAmt(DataSet tpv_dt_purchase) {
//        QFilter qFilters = new QFilter("billstatus", QCP.equals, "C");
/*DataSet shkd_payment_application = ORM.create().queryDataSet(algoKey, "shkd_payment_application",
                        "id,shkd_payment_entry,shkd_payment_entry.id entryid,shkd_payment_entry.shkd_purcontract.tpv_name contraNo," +
                                "(case when shkd_payment_entry.shkd_expense_item.number == 'FYXM150.1070' then shkd_payment_entry.shkd_applyamount else 0.0 end) tpv_addafterA," +
                                "(case when shkd_payment_entry.shkd_expense_item.number == 'FYXM150.1090' then shkd_payment_entry.shkd_applyamount else 0.0 end) tpv_addspotsA," +
                                "(case when shkd_payment_entry.shkd_expense_item.number == 'FYXM150.1010' then shkd_payment_entry.shkd_applyamount else 0.0 end) tpv_addforwardA",
                        new QFilter[]{qFilters}) //,0.0 tpv_addafter,0.0 tpv_addspots,0.0 tpv_addforward
                //.groupBy(new String[]{"e_corebillentryid","tpv_payoodsrela","e_corebillid"}).finish()
                .groupBy(new String[]{"contraNo"})
                .sum("tpv_addafterA").sum("tpv_addspotsA").sum("tpv_addforwardA").finish();*/
        //付款申请
        QFilter qFilterApPay = new QFilter("billstatus",QCP.equals, "C");
        DataSet tpv_ap_payapply_ext = ORM.create().queryDataSet(algoKey, "ap_payapply",
                "entry.shkd_purcontract.tpv_name contraNo,entry.e_payamount e_payamount,entry.lockedamt lockedamt,entry.e_expenseitem.number e_expenseitem",
                qFilterApPay.toArray());
        tpv_ap_payapply_ext = RptUtil.zeroNullNums(tpv_ap_payapply_ext);
        tpv_ap_payapply_ext = tpv_ap_payapply_ext.select("contraNo","e_payamount - lockedamt amount","e_expenseitem");
        QFilter qFilterCasPay = new QFilter("billstatus",QCP.not_equals, "A");
        DataSet shkd_cas_paybill_ext = ORM.create().queryDataSet(algoKey, "cas_paybill",
                "entry.shkd_purcontract.tpv_name contraNo,entry.e_payableamt amount,entry.e_expenseitem.number e_expenseitem",
                qFilterCasPay.toArray());
        //收款处理
        QFilter qFilterRec = new QFilter("billstatus",QCP.equals,"D");
        DataSet shkd_cas_recbill_ext = ORM.create().queryDataSet(algoKey, "cas_recbill",
                "entry.shkd_purcontract.tpv_name contraNo,entry.e_receivableamt amount,entry.e_expenseitem.number e_expenseitem",
                qFilterRec.toArray());
        DataSet finish = tpv_ap_payapply_ext.union(shkd_cas_recbill_ext).union(shkd_cas_paybill_ext);
        List<String> aliasList = new ArrayList<>();
        aliasList.add("contraNo");
        aliasList.add("(case when e_expenseitem == 'FYXM150.1070' then amount else 0.0 end) tpv_addafter");
        aliasList.add("(case when e_expenseitem in ('FYXM150.1090','FYXM150.1100') then amount else 0.0 end) tpv_addspots");
        aliasList.add("(case when e_expenseitem in ('FYXM150.1010','FYXM150.1080') then amount else 0.0 end) tpv_addforward");
        finish = finish.select(aliasList.toArray(aliasList.toArray(new String[0])))
                .groupBy(new String[]{"contraNo"})
                .sum("tpv_addafter").sum("tpv_addspots").sum("tpv_addforward").finish();

        DataSet dataSet = tpv_dt_purchase.leftJoin(finish).on("tpv_purorsupno", "contraNo")
                .select(new String[]{"id", "billno", "tpv_dep", "tpv_sup", "tpv_source", "tpv_cargostatus", "tpv_bizdate", "tpv_purorsupno", "tpv_prepoccupancy",
                        "tpv_addsigning", "tpv_addafter", "tpv_addspots", "tpv_addforward", "tpv_isusesingle"}).finish();
        dataSet = RptUtil.zeroNullNums(dataSet);
        return dataSet;
    }

    static {
    }

    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        // 部门
        DynamicObject tpv_dep_f = filterInfo.getDynamicObject("tpv_dep_f");
        if (YEM.isNotEmpty(tpv_dep_f)) {
            whereFilter = String.format("%s and tpv_dep = %s", whereFilter, tpv_dep_f.getPkValue());
        }
        // 供应商
        DynamicObject tpv_supplier_f = filterInfo.getDynamicObject("tpv_supplier_f");
        if (YEM.isNotEmpty(tpv_supplier_f)) {
            whereFilter = String.format("%s and tpv_sup = %s", whereFilter, tpv_supplier_f.getPkValue());
        }
        // 来源类型
        String tpv_source_f = filterInfo.getString("tpv_source_f");
        if (YEM.isNotEmpty(tpv_source_f)) {
            whereFilter = String.format("%s and tpv_source = '%s'", whereFilter, tpv_source_f);
        }
        // 采购合同号
        String tpv_purcontrnum_f = filterInfo.getString("tpv_purcontrnum_f");
        if (YEM.isNotEmpty(tpv_purcontrnum_f)) {
            whereFilter = String.format("%s and LOWER(tpv_purorsupno) LIKE LOWER('%%%s%%')", whereFilter, tpv_purcontrnum_f);
        }
        // 是否显示使用单票额度
        Boolean tpv_isusesingle_f = filterInfo.getBoolean("tpv_isusesingle_f");
        if (tpv_isusesingle_f) {
            whereFilter = String.format("%s and tpv_isusesingle = true", whereFilter);
        }
        return whereFilter;
    }

}
