package tpv.tpv_wm.tpv_crm.java.report;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.sdk.plugin.Plugin;
import tpv.tpv_wm.tpv_dt.java.utils.PersonnelDepUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @Description: 供应商预付款单票占用-合同明细 -报表查询
 * @Author LinRui
 * @Date 2023/10/12 9:54
 */
public class SupCreditOneConRptQuery extends AbstractReportListDataPlugin implements Plugin {
    private static final String algoKey = SupCreditOneConRptQuery.class.getName();

    @Override
    public DataSet query(ReportQueryParam reportQueryParam, Object obj) throws Throwable {
        FilterInfo filter = reportQueryParam.getFilter();
        String whereFilter = createWhereStr(reportQueryParam);
        boolean flag = filter.getBoolean("tpv_zero_f");
        DataSet instockDataSet = query(whereFilter);
        if (flag) { // 占用余额全部为0不显示
            instockDataSet = instockDataSet.where("tpv_residuesigning <> 0 or tpv_residueafter <> 0 or tpv_residuespots <> 0 or tpv_residueforward <> 0");
        }
        String whereFi="2=2";
        String whereStr = PersonnelDepUtils.createWhereStr(filter, "tpv_dep", "tpv_muldepartment");
        if (!"".equals(whereStr)){
            whereFi=String.format("%s%s", whereFi, whereStr);
        }
        return instockDataSet.where(whereFi);
    }

    public static DataSet query(String whereFilter) {
        DataSet mainDataSet = mainDataSet(whereFilter);
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        for (DynamicObject col : col_s) {
            BigDecimal tpvLeaveafter = col.getBigDecimal("tpv_leaveafter"); // 现货(理论剩余)
            BigDecimal tpvLeaveforward = col.getBigDecimal("tpv_leaveforward"); // 远期(理论剩余)
            if (tpvLeaveafter.compareTo(BigDecimal.ZERO) < 0 && tpvLeaveforward.compareTo(BigDecimal.ZERO) > 0) {
                BigDecimal add = tpvLeaveafter.add(tpvLeaveforward);
                if (add.compareTo(BigDecimal.ZERO) > 0) {
                    col.set("tpv_fastleaveafter", 0.0); // 现货(实际剩余额度)
                    col.set("tpv_fastleaveforward", add); // 远期(实际剩余额度)
                } else {
                    col.set("tpv_fastleaveafter", add);
                    col.set("tpv_fastleaveforward", 0.0);
                }
            } else if (tpvLeaveforward.compareTo(BigDecimal.ZERO) <= 0 || tpvLeaveafter.compareTo(BigDecimal.ZERO) >= 0
                    || (tpvLeaveafter.compareTo(BigDecimal.ZERO) > 0 && tpvLeaveforward.compareTo(BigDecimal.ZERO) > 0)) {
                col.set("tpv_fastleaveafter", tpvLeaveafter);
                col.set("tpv_fastleaveforward", tpvLeaveforward);
            }
        }
        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(SupCreditOneConRptQuery.class.getName()).createDataSet(inputs);
        return createDataSet.orderBy(new String[]{"tpv_bizdate desc"});
    }

    private static DataSet mainDataSet(String whereFilter) {
        //供应商单票额度维护
        DataSet supticket = ORM.create().queryDataSet(algoKey, "tpv_bd_supticket",
                "billno tpv_onevotebillno,id,tpv_ticketinfo,tpv_bizdate tpv_onevobizdate,tpv_department tpv_dep,tpv_operator,tpv_forwordquota,tpv_spotpayquota," +
                        "tpv_apforwordquota,tpv_apspotpayquota,tpv_proconnum tpv_purno,tpv_suppliernew tpv_supnew", new QFilter[]{new QFilter("billstatus",QCP.equals,"C")});
        DataSet supConDataset = SupCreditConRptQuery.query("tpv_isusesingle = true");
        DataSet dataSet = supticket.leftJoin(supConDataset).on("tpv_purno", "tpv_purorsupno")
                .select(new String[]{"id","tpv_onevotebillno","tpv_ticketinfo","tpv_onevobizdate","tpv_operator", "tpv_dep",
                        "tpv_bizdate", "tpv_apforwordquota * 10000 tpv_apforwordquota","tpv_apspotpayquota * 10000 tpv_apspotpayquota",
                        "tpv_purno", "tpv_supnew", "tpv_addsigning", "tpv_addafter", "tpv_addspots", "tpv_addforward", "tpv_deduction", "tpv_residuesigning",
                        "tpv_residueafter", "tpv_residuespots", "tpv_residueforward"}).finish();
        dataSet = RptUtil.zeroNullNums(dataSet);
        dataSet = dataSet.addField("tpv_apforwordquota - tpv_residueforward tpv_leaveforward","tpv_leaveforward");
        dataSet = dataSet.addField("tpv_apspotpayquota - tpv_residuespots tpv_leaveafter","tpv_leaveafter");
        dataSet = dataSet.addField("0.0 tpv_fastleaveafter","tpv_fastleaveafter");
        dataSet = dataSet.addField("0.0 tpv_fastleaveforward","tpv_fastleaveforward");
        dataSet = RptUtil.zeroNullNums(dataSet);
        if (YEM.isNotEmpty(whereFilter)) {
            dataSet = dataSet.where(whereFilter);
        }
        return dataSet;
    }

    static {
    }

    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        // 部门
        DynamicObject tpv_dep_f = filterInfo.getDynamicObject("tpv_dep_f");
        if (YEM.isNotEmpty(tpv_dep_f)) {
            whereFilter = String.format("%s and tpv_dep = %s", whereFilter, tpv_dep_f.getPkValue());
        }
        // 采购合同号
        String tpv_purcontrnum_f = filterInfo.getString("tpv_purcontrnum_f");
        if (YEM.isNotEmpty(tpv_purcontrnum_f)) {
            whereFilter = String.format("%s and LOWER(tpv_purno) LIKE LOWER('%%%s%%')", whereFilter, tpv_purcontrnum_f);
        }
        return whereFilter;
    }

    public static JSONArray resultData(String tpv_dep_f, String tpv_purcontrnum_f) {
        JSONArray result = new JSONArray();
        DynamicObject[] bosAdminorgs = BusinessDataServiceHelper.load("bos_adminorg", "id,name,number", new QFilter[]{new QFilter("enable", QCP.equals,"1")});
        Map<Long, String> deptMap = new HashMap<>();
        Map<String, Long> deptNameMap = new HashMap<>();
        if (bosAdminorgs.length > 0) {
            for (DynamicObject bosAdminorg : bosAdminorgs) {
                Long id = bosAdminorg.getLong("id");
                if (!deptMap.containsKey(id)) {
                    String name = bosAdminorg.getString("name");
                    deptMap.put(id, name);
                    deptNameMap.put(name, id);
                }
            }
        }
        DynamicObject[] bosUsers = BusinessDataServiceHelper.load("bos_user", "id,name,number", new QFilter[]{new QFilter("enable", QCP.equals,"1")});
        Map<Long, String> usersMap = new HashMap<>();
        Map<String, Long> usersNameMap = new HashMap<>();
        if (bosUsers.length > 0) {
            for (DynamicObject bosUser : bosUsers) {
                Long id = bosUser.getLong("id");
                if (!usersMap.containsKey(id)) {
                    String name = bosUser.getString("name");
                    usersMap.put(id, name);
                    usersNameMap.put(name, id);
                }
            }
        }
        String whereFilter = "1 = 1";
        // 部门
        if (YEM.isNotEmpty(tpv_dep_f)) {
            Long tpv_dep_fs = deptNameMap.get(tpv_dep_f) != null ? deptNameMap.get(tpv_dep_f) : 0L;
            if (YEM.isEmpty(tpv_dep_fs)) {
                return result;
            }
            whereFilter = String.format("%s and tpv_dep = %s", whereFilter, tpv_dep_fs);
        }
        // 采购合同号
        if (YEM.isNotEmpty(tpv_purcontrnum_f)) {
            whereFilter = String.format("%s and LOWER(tpv_purno) LIKE LOWER('%%%s%%')", whereFilter, tpv_purcontrnum_f);
        }

        for (Row row : query(whereFilter)) {
            JSONObject jsonObject = new JSONObject(new LinkedHashMap<>());
            String deptName = row.getLong("tpv_dep") != null ? deptMap.get(row.getLong("tpv_dep")) : null;
            String usersName = row.getLong("tpv_operator") != null ? usersMap.get(row.getLong("tpv_operator")) : null;
            jsonObject.put("onevotebillno", row.getString("tpv_onevotebillno"));  //单票额度编号
            jsonObject.put("ticketinfo", row.getString("tpv_ticketinfo"));  //名称
            jsonObject.put("onevobizdate", row.getDate("tpv_onevobizdate"));  //申请日期
            jsonObject.put("operator", usersName != null ? usersName : "");  //申请人
            jsonObject.put("operatorid", row.getLong("tpv_operator") != null ? String.valueOf(row.getLong("tpv_operator")):"");  //申请人
            jsonObject.put("dept", deptName != null ? deptName : "");  //部门
            jsonObject.put("deptid", row.getLong("tpv_dep") != null ? String.valueOf(row.getLong("tpv_dep")):"");  //部门id
            jsonObject.put("purno", row.getString("tpv_purno"));  //采购合同号
            jsonObject.put("bizdate", row.getDate("tpv_bizdate"));  //采购合同号的业务日期
            jsonObject.put("apspotpayquota", BigDecimalUtils.ceil(row.getBigDecimal("tpv_apspotpayquota").doubleValue(), 2)); //现货(额度)
            jsonObject.put("apforwordquota", BigDecimalUtils.ceil(row.getBigDecimal("tpv_apforwordquota").doubleValue(), 2)); //远期(额度)
            jsonObject.put("addsigning", BigDecimalUtils.ceil(row.getBigDecimal("tpv_addsigning").doubleValue(), 2)); //签约(占用增项)
            jsonObject.put("addafter", BigDecimalUtils.ceil(row.getBigDecimal("tpv_addafter").doubleValue(), 2)); //货后(占用增项)
            jsonObject.put("addspots", BigDecimalUtils.ceil(row.getBigDecimal("tpv_addspots").doubleValue(), 2)); //现货(占用增项)
            jsonObject.put("addforward", BigDecimalUtils.ceil(row.getBigDecimal("tpv_addforward").doubleValue(), 2)); //远期(占用增项)
            jsonObject.put("deduction", BigDecimalUtils.ceil(row.getBigDecimal("tpv_deduction").doubleValue(), 2)); //占用减项
            jsonObject.put("residuesigning", BigDecimalUtils.ceil(row.getBigDecimal("tpv_residuesigning").doubleValue(), 2)); //签约(占用余额)
            jsonObject.put("residueafter", BigDecimalUtils.ceil(row.getBigDecimal("tpv_residueafter").doubleValue(), 2)); //货后(占用余额)
            jsonObject.put("residuespots", BigDecimalUtils.ceil(row.getBigDecimal("tpv_residuespots").doubleValue(), 2)); //现货(占用余额)
            jsonObject.put("residueforward", BigDecimalUtils.ceil(row.getBigDecimal("tpv_residueforward").doubleValue(), 2)); //远期(占用余额)
            jsonObject.put("leaveafter", BigDecimalUtils.ceil(row.getBigDecimal("tpv_leaveafter").doubleValue(), 2)); //现货(理论剩余额度)
            jsonObject.put("leaveforward", BigDecimalUtils.ceil(row.getBigDecimal("tpv_leaveforward").doubleValue(), 2)); //远期(理论剩余额度)
            jsonObject.put("fastleaveafter", BigDecimalUtils.ceil(row.getBigDecimal("tpv_fastleaveafter").doubleValue(), 2)); //现货(实际剩余额度)
            jsonObject.put("fastleaveforward", BigDecimalUtils.ceil(row.getBigDecimal("tpv_fastleaveforward").doubleValue(), 2)); //远期(实际剩余额度)

            result.add(jsonObject);
        }
        return result;
    }

}
