package tpv.tpv_wm.tpv_crm.java.report;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.*;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.sdk.plugin.Plugin;

import java.math.BigDecimal;
import java.util.*;

/**
 * @Description: 供应商额度使用情况表
 * @Author LinRui
 * @Date 2023/10/13 20:26
 */
public class SupCreditUseRptQuery extends AbstractReportListDataPlugin implements Plugin {
    private static final String algoKey = SupCreditUseRptQuery.class.getName();

    private static final String supplierlimitsField = "id tpv_billid,billno tpv_billno,tpv_bizdate,tpv_billtype,tpv_ifeffective,tpv_credit tpv_orgcredit,tpv_org.id tpv_dep,org,org.id orgId,org.name orgName,"
            + "tpv_suppliergroup tpv_supgroup,tpv_signquota,tpv_forwordquota,tpv_spotpayquota,tpv_supplierlimit_sup,tpv_supplierlimit_sup.id tpv_entryid,"
            + "tpv_supplierlimit_sup.tpv_suppliers tpv_sup,tpv_supplierlimit_sup.tpv_suppliers.id tpv_suppliersId,"
            + "tpv_supplierlimit_sup.tpv_suppliers.name tpv_suppliersName,tpv_supplierlimit_sup.tpv_suppliers.tpv_gysratings tpv_supplierlev,"
            + "tpv_supplierlimit_sup.tpv_suppliers.tpv_businesstype tpv_enterprisetypes,tpv_supplierlimit_sup.tpv_isfreeze tpv_isfreeze,"
            + "tpv_supplierlimit_sup.tpv_quotatype tpv_quotatype,tpv_supplierlimit_sup.tpv_amountquota tpv_amountquota"; // 供应商额度维护

    @Override
    public DataSet query(ReportQueryParam param, Object obj) throws Throwable {
        FilterInfo filterInfo = param.getFilter();
        String tpv_orgcredit_f = filterInfo.getString("tpv_orgcredit_f");// 查询维度
        String tpv_credittype_f = filterInfo.getString("tpv_credittype_f"); // 授信类型
        String tpv_supgroup_f = filterInfo.getString("tpv_supgroup_f");// 集团
        DynamicObject tpv_dep_f = filterInfo.getDynamicObject("tpv_dep_f");// 部门
        DynamicObject tpv_supplier_f = filterInfo.getDynamicObject("tpv_supplier_f"); // 供应商

        DataSet orderBy = query(tpv_orgcredit_f, tpv_credittype_f, tpv_supgroup_f, tpv_dep_f, tpv_supplier_f);
        return orderBy;
    }

    private static DataSet query(String tpv_orgcredit_f, String tpv_credittype_f, String tpv_supgroup_f,
                                 DynamicObject tpv_dep_f, DynamicObject tpv_supplier_f) {
        QFilter qFilter = new QFilter("billstatus", QCP.equals, "C");
        qFilter.and(new QFilter("tpv_supplierlimit_sup.tpv_suppliers", QCP.is_notnull, ""));
        if (YEM.isNotEmpty(tpv_orgcredit_f)) {
            qFilter.and(new QFilter("tpv_credit", QCP.equals, tpv_orgcredit_f));// 查询维度
        }
//        if (YEM.isNotEmpty(tpv_dep_f) & YEM.isNotEmpty(tpv_orgcredit_f) & "B".equals(tpv_orgcredit_f)) {
//            qFilter.and(new QFilter("tpv_org", QCP.equals, tpv_dep_f.getPkValue()));// 部门
//        }
        if (YEM.isNotEmpty(tpv_supgroup_f)) {
            qFilter.and(new QFilter("tpv_suppliergroup", QCP.like, "%" +tpv_supgroup_f + "%"));// 集团
        }

        // 单独授信
        DataSet queryNoTotal;
        if ("A".equals(tpv_orgcredit_f)) {// 全司
            queryNoTotal = selectsupplierlimitsEntry(qFilter.toArray(), new String[]{"tpv_sup"});
        } else if ("B".equals(tpv_orgcredit_f)) {// 部门
            queryNoTotal = selectsupplierlimitsEntry(qFilter.toArray(), new String[]{"tpv_sup", "tpv_dep"});
        } else if ((YEM.isNotEmpty(tpv_dep_f) & YEM.isNotEmpty(tpv_orgcredit_f) & "B".equals(tpv_orgcredit_f))
                | (YEM.isNotEmpty(tpv_dep_f) & YEM.isEmpty(tpv_orgcredit_f)))  {// 部门
            queryNoTotal = selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "A").and(qFilter).toArray(),
                    new String[]{"tpv_sup"}).union(
                    selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "B").and(qFilter)
                                    .and(qFilter.and(new QFilter("tpv_org", QCP.equals, tpv_dep_f.getPkValue()))).toArray(),
                            new String[]{"tpv_sup", "tpv_dep"}));
        } else {// 全司+部门
            queryNoTotal = selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "A").and(qFilter).toArray(),
                    new String[]{"tpv_sup"}).union(
                    selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "B").and(qFilter).toArray(),
                            new String[]{"tpv_sup", "tpv_dep"}));

        }

        DataSet copy = queryNoTotal.copy();
        List<String> supGroup = new ArrayList<>(); //过滤集团
        Boolean flagDep = true;
        Boolean flagSup = true;
        if (null != copy) {
            for (Row row : copy) {
                if (YEM.isNotEmpty(tpv_supplier_f)) {
                    if (YEM.isNotEmpty(row.getLong("tpv_dep")) & "B".equals(row.getString("tpv_orgcredit")) &
                            tpv_supplier_f.getLong("id") == row.getLong("tpv_sup")) {
                        if (!supGroup.contains(row.getString("tpv_supgroup"))) {
                            supGroup.add(row.getString("tpv_supgroup"));
                        }
                    }
                    if (YEM.isEmpty(row.getLong("tpv_dep")) & "A".equals(row.getString("tpv_orgcredit")) &
                            tpv_supplier_f.getLong("id") == row.getLong("tpv_sup")) {
                        if (!supGroup.contains(row.getString("tpv_supgroup"))) {
                            supGroup.add(row.getString("tpv_supgroup"));
                        }
                    }
                } else {
                    if (YEM.isNotEmpty(row.getLong("tpv_dep")) & "B".equals(row.getString("tpv_orgcredit")) & YEM.isNotEmpty(row.getLong("tpv_sup"))) {
                        if (!supGroup.contains(row.getString("tpv_supgroup"))) {
                            supGroup.add(row.getString("tpv_supgroup"));
                        }
                    }
                }

                if (YEM.isNotEmpty(row.getLong("tpv_dep")) & YEM.isNotEmpty(tpv_dep_f)) {
                    if (tpv_dep_f.getLong("id") == row.getLong("tpv_dep")) {
                        flagDep = false;
                    }
                }
                if (YEM.isNotEmpty(row.getLong("tpv_sup")) & YEM.isNotEmpty(tpv_supplier_f)) {
                    if (tpv_supplier_f.getLong("id") == row.getLong("tpv_sup")) {
                        flagSup = false;
                    }
                }
            }
        }
        if ((supGroup.size() > 0 & YEM.isNotEmpty(tpv_dep_f)) | (supGroup.size() > 0 & YEM.isNotEmpty(tpv_supplier_f))) {
            List<String> strings = new ArrayList<>();
            Arrays.stream(supGroup.toArray()).forEach(x -> {
                String s = "'" + x + "'";
                strings.add("" + s + "");
            });
            String whereFilter = "1=1";
            whereFilter = String.format("%s and tpv_supgroup in(%s)", whereFilter, String.join(",", strings));
            queryNoTotal = queryNoTotal.where(whereFilter);
        }
        if (flagDep & YEM.isEmpty(tpv_orgcredit_f) & YEM.isEmpty(tpv_supgroup_f) & YEM.isEmpty(tpv_credittype_f) & supGroup.size() == 0 & YEM.isEmpty(tpv_supplier_f)
                & YEM.isNotEmpty(tpv_dep_f)) {
            queryNoTotal = selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "A").and(qFilter)
                            .and(qFilter.and(new QFilter("tpv_org", QCP.equals, tpv_dep_f.getPkValue()))).toArray(),
                    new String[]{"tpv_sup"}).union(
                    selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "B").and(qFilter)
                                    .and(qFilter.and(new QFilter("tpv_org", QCP.equals, tpv_dep_f.getPkValue()))).toArray(),
                            new String[]{"tpv_sup", "tpv_dep"}));
        }
        if (flagSup & YEM.isEmpty(tpv_orgcredit_f) & YEM.isEmpty(tpv_supgroup_f) & YEM.isEmpty(tpv_credittype_f) & supGroup.size() == 0 & YEM.isNotEmpty(tpv_supplier_f)) {
            queryNoTotal = selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "A").and(qFilter)
                            .and(qFilter.and(new QFilter("tpv_supplierlimit_sup.tpv_suppliers", QCP.equals, tpv_supplier_f.getPkValue()))).toArray(),
                    new String[]{"tpv_sup"}).union(
                    selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "B").and(qFilter)
                                    .and(qFilter.and(new QFilter("tpv_supplierlimit_sup.tpv_suppliers", QCP.equals, tpv_supplier_f.getPkValue()))).toArray(),
                            new String[]{"tpv_sup", "tpv_dep"}));
        }
        // 总额度
        DataSet queryIsTotal = selectSupplierlimits(queryNoTotal);
        if (YEM.isNotEmpty(tpv_supplier_f)) {
            String whereFilter = "1=1";
            whereFilter = String.format("%s and tpv_sup = %s", whereFilter, tpv_supplier_f.getPkValue());
            queryNoTotal = queryNoTotal.where(whereFilter);
        }
        DataSet union = queryIsTotal.union(queryNoTotal);
        DataSet orderBy = finishDataSet(union)
                .orderBy(new String[]{"tpv_supgroup","tpv_dep asc","tpv_sup asc"}); // "tpv_credittype"
        if (YEM.isNotEmpty(tpv_credittype_f)) {
            orderBy = orderBy.where(String.format("tpv_credittype = '%s'", tpv_credittype_f));// 授信类型
        }
        orderBy = orderBy.select("tpv_orgcredit", "tpv_dep", "tpv_supgroup", "tpv_sup", "tpv_credittype",
                "tpv_signingamt","tpv_forwardamt","tpv_afteramt","tpv_addsigning","tpv_addafter","tpv_addforward","tpv_leavesigning"
                ,"tpv_leaveafter","tpv_leaveforward","tpv_fastleavesigning","tpv_fastleaveafter","tpv_fastleaveforward",
                "Round(tpv_signingamt,2) tpv_signingamts", "Round(tpv_afteramt,2) tpv_afteramts", "Round(tpv_forwardamt,2) tpv_forwardamts",
                "Round(tpv_addsigning,2) tpv_addsignings", "Round(tpv_addafter,2) tpv_addafters", "Round(tpv_addforward,2) tpv_addforwards",
                "Round(tpv_leavesigning,2) tpv_leavesignings", "Round(tpv_leaveafter,2) tpv_leaveafters", "Round(tpv_leaveforward,2) tpv_leaveforwards",
                "Round(tpv_fastleavesigning,2) tpv_fastleavesignings", "Round(tpv_fastleaveafter,2) tpv_fastleaveafters", "Round(tpv_fastleaveforward,2) tpv_fastleaveforwards");
        return orderBy;
    }

    private static DataSet selectSupplierlimits(DataSet queryNoTotal) {
        DataSet supplierlimits = ORM.create()
                .queryDataSet(algoKey, "tpv_bd_supplierlimits", "tpv_billtype,tpv_credit tpv_orgcredit,tpv_org.id tpv_dep,tpv_suppliergroup tpv_supgroup, tpv_ifeffective," +
                        "tpv_signquota,tpv_spotpayquota,tpv_forwordquota", new QFilter[]{new QFilter("billstatus",QCP.equals,"C")})
                .select( "tpv_billtype", "tpv_orgcredit", "tpv_dep", "tpv_supgroup", "tpv_ifeffective",
                        "(case when tpv_ifeffective != true then tpv_signquota * 10000 else 0.0 end) tpv_signquota",
                        "(case when tpv_ifeffective != true then tpv_spotpayquota * 10000 else 0.0 end) tpv_spotpayquota",
                        "(case when tpv_ifeffective != true then tpv_forwordquota * 10000 else 0.0 end) tpv_forwordquota");
        supplierlimits = supplierlimits.groupBy(new String[]{"tpv_orgcredit", "tpv_dep", "tpv_supgroup"})
                .sum("tpv_signquota").sum("tpv_spotpayquota").sum("tpv_forwordquota").finish();
        List<String> aliasList = RptUtil.getDataSetAliasList(queryNoTotal);
        aliasList.add("tpv_signquota");
        aliasList.add("tpv_spotpayquota");
        aliasList.add("tpv_forwordquota");
        DataSet dataSet = queryNoTotal.leftJoin(supplierlimits).on("tpv_orgcredit","tpv_orgcredit").on("tpv_dep","tpv_dep")
                .on("tpv_supgroup","tpv_supgroup")
                .select(aliasList.toArray(new String[aliasList.size()])).finish();
        dataSet = dataSet.groupBy(new String[]{"tpv_orgcredit", "tpv_dep", "tpv_supgroup", "'A' tpv_credittype",
                        "tpv_signquota", "tpv_spotpayquota", "tpv_forwordquota"})
                .sum("tpv_addsigning").sum("tpv_addafter").sum("tpv_addforward") // 理论占用
                .finish()
                .select(new String[]{"tpv_orgcredit", "tpv_dep", "tpv_supgroup", "tpv_credittype",
                        "tpv_signquota tpv_signingamt", "tpv_forwordquota tpv_forwardamt", "tpv_spotpayquota tpv_afteramt",
                        "tpv_addsigning", "tpv_addafter", "tpv_addforward"})
                .select(  "tpv_orgcredit", "tpv_dep", "tpv_supgroup", "null tpv_sup", "tpv_credittype",
                        "tpv_signingamt", "tpv_forwardamt", "tpv_afteramt", "tpv_addsigning", "tpv_addafter", "tpv_addforward",
                        "tpv_signingamt - tpv_addsigning tpv_leavesigning", "tpv_afteramt - tpv_addafter tpv_leaveafter", "tpv_forwardamt - tpv_addforward tpv_leaveforward",
                        "0.0 tpv_fastleavesigning", "0.0 tpv_fastleaveafter", "0.0 tpv_fastleaveforward",
                        "null tpv_signingamts", "null tpv_afteramts", "null tpv_forwardamts", "null tpv_addsignings", "null tpv_addafters", "null tpv_addforwards",
                        "null tpv_leavesignings", "null tpv_leaveafters", "null tpv_leaveforwards", "null tpv_fastleavesignings", "null tpv_fastleaveafters", "null tpv_fastleaveforwards");
        return dataSet;
    }

    /**
     * 查询供应商额度维护分录数据 部门 关联查供应商预付款占用-合同明细 以dept+供应商分组【single】
     *
     * @param keys
     */
    private static DataSet selectsupplierlimitsEntry(QFilter[] qFilters, String[] keys) { // 供应商额度维护分录数据
        DataSet supplierlimitsEntry = ORM.create()
                .queryDataSet(algoKey, "tpv_bd_supplierlimits", supplierlimitsField, qFilters)
                .select( "tpv_billtype", "tpv_orgcredit", "tpv_dep", "tpv_supgroup", "tpv_sup","tpv_isfreeze", "tpv_ifeffective",
                        "(case when tpv_ifeffective != true and tpv_isfreeze in('B','') and tpv_quotatype == 'A' then tpv_amountquota * 10000 else 0.0 end) tpv_signingamt",
                        "(case when tpv_ifeffective != true and tpv_isfreeze in('B','') and tpv_quotatype == 'B' then tpv_amountquota * 10000 else 0.0 end) tpv_forwardamt",
                        "(case when tpv_ifeffective != true and tpv_isfreeze in('B','') and tpv_quotatype == 'C' then tpv_amountquota * 10000 else 0.0 end) tpv_afteramt",
                        "'B' tpv_credittype")
                .groupBy(new String[]{"tpv_orgcredit", "tpv_dep", "tpv_supgroup", "tpv_sup", "tpv_credittype"})
                .sum("tpv_signingamt").sum("tpv_forwardamt").sum("tpv_afteramt").finish();
        DataSet query = SupCreditConRptQuery.queryLeftJoinUse("tpv_isusesingle = false");
        //查询南北数据
        DataSet tpv_crm_supadvanceupns = ORM.create().queryDataSet(algoKey, "tpv_crm_supadvanceupns",
                "id,tpv_sup,tpv_department tpv_dep,tpv_residuesigning signingSupNS,tpv_residuespots spotsSupNS,tpv_residueforward forwardSupNS",
                new QFilter[]{new QFilter("billstatus", QCP.equals, "C")});
        GroupbyDataSet groupbyDataSet;
        GroupbyDataSet groupbySupNS;
        if (keys == null) {
            groupbyDataSet = query.groupBy();
            //分组统计南北数据
            groupbySupNS = tpv_crm_supadvanceupns.groupBy();
        } else {
            groupbyDataSet = query.groupBy(keys);
            //分组统计南北数据
            groupbySupNS = tpv_crm_supadvanceupns.groupBy(keys);
        }
        DataSet finish = groupbyDataSet.sum("tpv_residuesigning").sum("tpv_residuespots").sum("tpv_residueforward")
                .finish(); // 查询供应商预付款占用-合同明细表
        DataSet finishSupNS = groupbySupNS.sum("signingSupNS").sum("spotsSupNS").sum("forwardSupNS")
                .finish(); // 查询南北数据

        JoinDataSet leftJoin = supplierlimitsEntry.leftJoin(finish);
        for (String key : keys) {
            leftJoin.on(key, key);
        }
        DataSet select = leftJoin
                .select(new String[]{"tpv_orgcredit", "tpv_dep",
                        "tpv_supgroup", "tpv_sup", "tpv_credittype",
                        "(case when tpv_signingamt != null then tpv_signingamt else 0.0 end) tpv_signingamt",
                        "(case when tpv_forwardamt != null then tpv_forwardamt else 0.0 end) tpv_forwardamt",
                        "(case when tpv_afteramt != null then tpv_afteramt else 0.0 end) tpv_afteramt",
                        "(case when tpv_residuesigning != null then tpv_residuesigning else 0.0 end) tpv_addsigning",
                        "(case when tpv_residuespots != null then tpv_residuespots else 0.0 end) tpv_addafter",
                        "(case when tpv_residueforward != null then tpv_residueforward else 0.0 end) tpv_addforward"})
                .finish()
                .select("tpv_orgcredit", "tpv_dep",
                        "tpv_supgroup", "tpv_sup", "tpv_credittype",
                        "tpv_signingamt", "tpv_forwardamt", "tpv_afteramt",
                        "tpv_addsigning", "tpv_addafter", "tpv_addforward",
                        "tpv_signingamt - tpv_addsigning tpv_leavesigning",
                        "tpv_afteramt - tpv_addafter tpv_leaveafter",
                        "tpv_forwardamt - tpv_addforward tpv_leaveforward",
                        "0.0 tpv_fastleavesigning",
                        "0.0 tpv_fastleaveafter", "0.0 tpv_fastleaveforward",
                        "null tpv_signingamts", "null tpv_afteramts", "null tpv_forwardamts", "null tpv_addsignings", "null tpv_addafters", "null tpv_addforwards",
                        "null tpv_leavesignings", "null tpv_leaveafters", "null tpv_leaveforwards", "null tpv_fastleavesignings", "null tpv_fastleaveafters", "null tpv_fastleaveforwards");
        JoinDataSet leftJoinSupNS = select.leftJoin(finishSupNS);
        for (String key : keys) {
            leftJoinSupNS.on(key, key);
        }
        List<String> aliasList = RptUtil.getDataSetAliasList(select);
        aliasList.add("(case when signingSupNS != null then signingSupNS else 0.0 end) signingSupNS");
        aliasList.add("(case when spotsSupNS != null then spotsSupNS else 0.0 end) spotsSupNS");
        aliasList.add("(case when forwardSupNS != null then forwardSupNS else 0.0 end) forwardSupNS");
        select = leftJoinSupNS.select(aliasList.toArray(new String[0])).finish();

        select = select.select("tpv_orgcredit", "tpv_dep",
                "tpv_supgroup", "tpv_sup", "tpv_credittype",
                "tpv_signingamt", "tpv_forwardamt", "tpv_afteramt",
                "tpv_addsigning + signingSupNS tpv_addsigning", "tpv_addafter + spotsSupNS tpv_addafter", "tpv_addforward + forwardSupNS tpv_addforward",
                "tpv_leavesigning - signingSupNS tpv_leavesigning", "tpv_leaveafter - spotsSupNS tpv_leaveafter", "tpv_leaveforward - forwardSupNS tpv_leaveforward",
                "tpv_fastleavesigning", "tpv_fastleaveafter", "tpv_fastleaveforward",
                "tpv_signingamts", "tpv_afteramts", "tpv_forwardamts", "tpv_addsignings", "tpv_addafters", "tpv_addforwards",
                "tpv_leavesignings", "tpv_leaveafters", "tpv_leaveforwards", "tpv_fastleavesignings", "tpv_fastleaveafters", "tpv_fastleaveforwards");
        return select;
    }

    private static DataSet finishDataSet(DataSet dataSet) {
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(dataSet);
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            BigDecimal tpvLeavesigning = col.getBigDecimal("tpv_leavesigning"); // 签约(理论剩余)
            col.set("tpv_fastleavesigning", tpvLeavesigning);
            BigDecimal tpvLeaveafter = col.getBigDecimal("tpv_leaveafter"); // 现货(理论剩余)
            BigDecimal tpvLeaveforward = col.getBigDecimal("tpv_leaveforward"); // 远期(理论剩余)
            if (tpvLeaveafter.compareTo(BigDecimal.ZERO) < 0 && tpvLeaveforward.compareTo(BigDecimal.ZERO) > 0) {
                BigDecimal add = tpvLeaveafter.add(tpvLeaveforward);
                if (add.compareTo(BigDecimal.ZERO) > 0) {
                    col.set("tpv_fastleaveafter", 0.0); // 现货(实际剩余额度)
                    col.set("tpv_fastleaveforward", add); // 远期(实际剩余额度)
                } else {
                    col.set("tpv_fastleaveafter", add);
                    col.set("tpv_fastleaveforward", 0.0);
                }
            } else if (tpvLeaveforward.compareTo(BigDecimal.ZERO) <= 0 || tpvLeaveafter.compareTo(BigDecimal.ZERO) >= 0
                    || (tpvLeaveafter.compareTo(BigDecimal.ZERO) > 0 && tpvLeaveforward.compareTo(BigDecimal.ZERO) > 0)) {
                col.set("tpv_fastleaveafter", tpvLeaveafter);
                col.set("tpv_fastleaveforward", tpvLeaveforward);
            }
        }
        RowMeta rowMeta = dataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(SupCreditUseRptQuery.class.getName()).createDataSet(inputs);
        return createDataSet;
    }

    public static JSONObject supadvanceupJsonObject(String tpv_orgcredit_f, String tpv_credittype_f,
                                                    String tpv_supgroup_f, DynamicObject tpv_dep_f, DynamicObject tpv_supplier_f) {
        JSONObject result = new JSONObject();
        for (Row row : query(tpv_orgcredit_f, tpv_credittype_f, tpv_supgroup_f, tpv_dep_f, tpv_supplier_f)) {
            String tpvCredittype = row.getString("tpv_credittype"); //授信类型 【A总额度/B单独授信】
            String tpvOrgcredit = row.getString("tpv_orgcredit"); //查询维度 【A全司/B部门】
            String tpvCredittypeName = null, tpvOrgcreditName = null;
            // 授信类型tpv_credittype : 'A' 总额度 'B' 单独授信 查询维度tpv_orgcredit : 'A' 全司 'B' 部门
            boolean isTotal = "A".equals(tpvCredittype);
            if (isTotal) {
                tpvCredittypeName = "total";
            }
            if ("B".equals(tpvCredittype)) {
                tpvCredittypeName = "single";
            }
            if ("A".equals(tpvOrgcredit)) {
                tpvOrgcreditName = "_org";
            }
            if ("B".equals(tpvOrgcredit)) {
                tpvOrgcreditName = "_dept";
            }

            String concatName = tpvCredittypeName + tpvOrgcreditName; // allorg、alldept、singleorg、singledept

            JSONObject map = new JSONObject();
            BigDecimal tpvSigningamt = row.getBigDecimal("tpv_fastleavesigning");
            BigDecimal tpvAfteramt = row.getBigDecimal("tpv_fastleaveafter");
            BigDecimal tpvForwardamt = row.getBigDecimal("tpv_fastleaveforward");
            if (isTotal || tpvSigningamt.compareTo(BigDecimal.ZERO) > 0) {
                map.put("signingamt", tpvSigningamt); // 签约(实际剩余额度)fast_leave_signing
            }
            if (isTotal || tpvAfteramt.compareTo(BigDecimal.ZERO) > 0) {
                map.put("afteramt", tpvAfteramt); // 现货(实际剩余额度)fast_leave_after
            }
            if (isTotal || tpvForwardamt.compareTo(BigDecimal.ZERO) > 0) {
                map.put("forwardamt", tpvForwardamt); // 远期(实际剩余额度)fast_leave_forward
            }
            result.put(concatName, map);
        }
        if (!result.containsKey("total_org")) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("signingamt", BigDecimal.ZERO);
            jsonObject.put("afteramt", BigDecimal.ZERO);
            jsonObject.put("forwardamt", BigDecimal.ZERO);
            result.put("total_org", jsonObject);
        }
        if (!result.containsKey("total_dept")) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("signingamt", BigDecimal.ZERO);
            jsonObject.put("afteramt", BigDecimal.ZERO);
            jsonObject.put("forwardamt", BigDecimal.ZERO);
            result.put("total_dept", jsonObject);
        }
        if (!result.containsKey("single_org")) {
            result.put("single_org", new JSONObject());
        }
        if (!result.containsKey("single_dept")) {
            result.put("single_dept", new JSONObject());
        }
        return result;
    }

    private static DataSet queryResult(String tpv_orgcredit_f, String tpv_credittype_f, String tpv_supgroup_f,
                                       Long tpv_dep_f, Long tpv_supplier_f) {
        QFilter qFilter = new QFilter("billstatus", QCP.equals, "C");
        qFilter.and(new QFilter("tpv_supplierlimit_sup.tpv_suppliers", QCP.is_notnull, ""));
        if (YEM.isNotEmpty(tpv_orgcredit_f)) {
            qFilter.and(new QFilter("tpv_credit", QCP.equals, tpv_orgcredit_f));// 查询维度
        }
//        if (tpv_dep_f > 0 & YEM.isNotEmpty(tpv_orgcredit_f) & "B".equals(tpv_orgcredit_f)) {
//            qFilter.and(new QFilter("tpv_org", QCP.equals, tpv_dep_f));// 部门
//        }
        if (YEM.isNotEmpty(tpv_supgroup_f)) {
            qFilter.and(new QFilter("tpv_suppliergroup", QCP.like, "%" +tpv_supgroup_f + "%"));// 集团
        }

        // 单独授信
        DataSet queryNoTotal;
        if ("A".equals(tpv_orgcredit_f)) {// 全司
            queryNoTotal = selectsupplierlimitsEntry(qFilter.toArray(), new String[]{"tpv_sup"});
        } else if ("B".equals(tpv_orgcredit_f)) {// 部门
            queryNoTotal = selectsupplierlimitsEntry(qFilter.toArray(), new String[]{"tpv_sup", "tpv_dep"});
        } else if ((tpv_dep_f > 0 & YEM.isNotEmpty(tpv_orgcredit_f) & "B".equals(tpv_orgcredit_f))
                | (tpv_dep_f > 0 & YEM.isEmpty(tpv_orgcredit_f)))  {
            queryNoTotal = selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "A").and(qFilter).toArray(),
                    new String[]{"tpv_sup"}).union(
                    selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "B").and(qFilter)
                                    .and(qFilter.and(new QFilter("tpv_org", QCP.equals, tpv_dep_f))).toArray(),
                            new String[]{"tpv_sup", "tpv_dep"}));
        } else { // 全司+部门
            queryNoTotal = selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "A").and(qFilter).toArray(),
                    new String[]{"tpv_sup"}).union(
                    selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "B").and(qFilter).toArray(),
                            new String[]{"tpv_sup", "tpv_dep"}));

        }

        DataSet copy = queryNoTotal.copy();
        List<String> supGroup = new ArrayList<>(); //过滤集团
        Boolean flagDep = true;
        Boolean flagSup = true;
        if (null != copy) {
            for (Row row : copy) {
                if (tpv_supplier_f > 0l) {
                    if (YEM.isNotEmpty(row.getLong("tpv_dep")) & "B".equals(row.getString("tpv_orgcredit")) &
                            tpv_supplier_f.compareTo(row.getLong("tpv_sup")) == 0 ) {
                        if (!supGroup.contains(row.getString("tpv_supgroup"))) {
                            supGroup.add(row.getString("tpv_supgroup"));
                        }
                    }
                    if (YEM.isEmpty(row.getLong("tpv_dep")) & "A".equals(row.getString("tpv_orgcredit")) &
                            tpv_supplier_f.compareTo(row.getLong("tpv_sup")) == 0) {
                        if (!supGroup.contains(row.getString("tpv_supgroup"))) {
                            supGroup.add(row.getString("tpv_supgroup"));
                        }
                    }
                } else {
                    if (YEM.isNotEmpty(row.getLong("tpv_dep")) & "B".equals(row.getString("tpv_orgcredit")) & YEM.isNotEmpty(row.getLong("tpv_sup"))) {
                        if (!supGroup.contains(row.getString("tpv_supgroup"))) {
                            supGroup.add(row.getString("tpv_supgroup"));
                        }
                    }
                }

                if (YEM.isNotEmpty(row.getLong("tpv_dep")) & tpv_dep_f > 0l) {
                    if (tpv_dep_f.compareTo(row.getLong("tpv_dep")) == 0) {
                        flagDep = false;
                    }
                }
                if (YEM.isNotEmpty(row.getLong("tpv_sup")) & tpv_supplier_f > 0l) {
                    if (tpv_supplier_f.compareTo(row.getLong("tpv_sup")) == 0) {
                        flagSup = false;
                    }
                }
            }
        }
        if ((supGroup.size() > 0 & YEM.isNotEmpty(tpv_dep_f)) | (supGroup.size() > 0 & YEM.isNotEmpty(tpv_supplier_f))) {
            List<String> strings = new ArrayList<>();
            Arrays.stream(supGroup.toArray()).forEach(x -> {
                String s = "'" + x + "'";
                strings.add("" + s + "");
            });
            String whereFilter = "1=1";
            whereFilter = String.format("%s and tpv_supgroup in(%s)", whereFilter, String.join(",", strings));
            queryNoTotal = queryNoTotal.where(whereFilter);
        }
        if (flagDep & YEM.isEmpty(tpv_orgcredit_f) & YEM.isEmpty(tpv_supgroup_f) & YEM.isEmpty(tpv_credittype_f) & supGroup.size() == 0 & tpv_supplier_f == 0l) {
            queryNoTotal = selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "A").and(qFilter)
                            .and(qFilter.and(new QFilter("tpv_org", QCP.equals, tpv_dep_f))).toArray(),
                    new String[]{"tpv_sup"}).union(
                    selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "B").and(qFilter)
                                    .and(qFilter.and(new QFilter("tpv_org", QCP.equals, tpv_dep_f))).toArray(),
                            new String[]{"tpv_sup", "tpv_dep"}));
        }
        if (flagSup & YEM.isEmpty(tpv_orgcredit_f) & YEM.isEmpty(tpv_supgroup_f) & YEM.isEmpty(tpv_credittype_f) & supGroup.size() == 0 & YEM.isNotEmpty(tpv_supplier_f)) {
            queryNoTotal = selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "A").and(qFilter)
                            .and(qFilter.and(new QFilter("tpv_supplierlimit_sup.tpv_suppliers", QCP.equals, tpv_supplier_f))).toArray(),
                    new String[]{"tpv_sup"}).union(
                    selectsupplierlimitsEntry(new QFilter("tpv_credit", QCP.equals, "B").and(qFilter)
                                    .and(qFilter.and(new QFilter("tpv_supplierlimit_sup.tpv_suppliers", QCP.equals, tpv_supplier_f))).toArray(),
                            new String[]{"tpv_sup", "tpv_dep"}));
        }
        // 总额度
        DataSet queryIsTotal = selectSupplierlimits(queryNoTotal);
        if (YEM.isNotEmpty(tpv_supplier_f)) {
            String whereFilter = "1=1";
            whereFilter = String.format("%s and tpv_sup = %s", whereFilter, tpv_supplier_f);
            queryNoTotal = queryNoTotal.where(whereFilter);
        }
        DataSet union = queryIsTotal.union(queryNoTotal);
        DataSet orderBy = finishDataSet(union)
                .orderBy(new String[]{"tpv_supgroup","tpv_dep asc","tpv_sup asc"}); //tpv_billid
        if (YEM.isNotEmpty(tpv_credittype_f)) {
            orderBy = orderBy.where(String.format("tpv_credittype = '%s'", tpv_credittype_f));// 授信类型
        }
        orderBy = orderBy.select("tpv_orgcredit", "tpv_dep", "tpv_supgroup", "tpv_sup", "tpv_credittype",
                "tpv_signingamt","tpv_forwardamt","tpv_afteramt","tpv_addsigning","tpv_addafter","tpv_addforward","tpv_leavesigning"
                ,"tpv_leaveafter","tpv_leaveforward","tpv_fastleavesigning","tpv_fastleaveafter","tpv_fastleaveforward",
                "Round(tpv_signingamt,2) tpv_signingamts", "Round(tpv_afteramt,2) tpv_afteramts", "Round(tpv_forwardamt,2) tpv_forwardamts",
                "Round(tpv_addsigning,2) tpv_addsignings", "Round(tpv_addafter,2) tpv_addafters", "Round(tpv_addforward,2) tpv_addforwards",
                "Round(tpv_leavesigning,2) tpv_leavesignings", "Round(tpv_leaveafter,2) tpv_leaveafters", "Round(tpv_leaveforward,2) tpv_leaveforwards",
                "Round(tpv_fastleavesigning,2) tpv_fastleavesignings", "Round(tpv_fastleaveafter,2) tpv_fastleaveafters", "Round(tpv_fastleaveforward,2) tpv_fastleaveforwards");
        return orderBy;
    }

    public static JSONArray resultData(String tpv_orgcredit_f, String tpv_credittype_f,
                                       String tpv_supgroup_f, String tpv_dep_f, String tpv_supplier_f) {
        String tpv_orgcredit_fs = "";
        String tpv_credittype_fs = "";
        if ("集团额度".equals(tpv_credittype_f)) {
            tpv_credittype_fs = "A";
        } else if ("单抬头额度".equals(tpv_credittype_f)) {
            tpv_credittype_fs = "B";
        }
        if ("全司".equals(tpv_orgcredit_f)) {
            tpv_orgcredit_fs = "A";
        } else if ("部门".equals(tpv_orgcredit_f)) {
            tpv_orgcredit_fs = "B";
        }

        JSONArray result = new JSONArray();
        QFilter filters = new QFilter("status", QCP.equals, "C");
        filters.and("enable", QCP.equals, "1");
        DynamicObject[] bdSuppliers = BusinessDataServiceHelper.load("bd_supplier", "id,name,number", new QFilter[]{filters});
        Map<Long, String> supMap = new HashMap<>();
        Map<String, Long> supNameMap = new HashMap<>();
        if (bdSuppliers.length > 0) {
            for (DynamicObject bdSupplier : bdSuppliers) {
                Long id = bdSupplier.getLong("id");
                String string = bdSupplier.getString("name");
                if ("连云港邦合贸易有限公司".equals(string)) {
                    System.out.println();
                }
                if (!supMap.containsKey(id)) {
                    String name = bdSupplier.getString("name");
                    supMap.put(id, name);
                    supNameMap.put(name, id);
                }
            }
        }
//        Arrays.stream(bdSuppliers).filter(x -> bdSuppliers != null).forEach(x -> {
//            Long id = x.getLong("id");
//            if (!supMap.containsKey(id)) {
//                String name = x.getString("name");
//                supMap.put(id, name);
//                supNameMap.put(name, id);
//            }
//        });
        DynamicObject[] bosAdminorgs = BusinessDataServiceHelper.load("bos_adminorg", "id,name,number", new QFilter[]{new QFilter("enable",QCP.equals,"1")});
        Map<Long, String> deptMap = new HashMap<>();
        Map<String, Long> deptNameMap = new HashMap<>();
        if (bosAdminorgs.length > 0) {
            for (DynamicObject bosAdminorg : bosAdminorgs) {
                Long id = bosAdminorg.getLong("id");
                if (!deptMap.containsKey(id)) {
                    String name = bosAdminorg.getString("name");
                    deptMap.put(id, name);
                    deptNameMap.put(name, id);
                }
            }
        }
//        Arrays.stream(bosAdminorgs).filter(x -> bosAdminorgs != null).forEach(x -> {
//            Long id = x.getLong("id");
//            if (!deptMap.containsKey(id)) {
//                String name = x.getString("name");
//                deptMap.put(id, name);
//                deptNameMap.put(name, id);
//            }
//        });
        Long tpv_dep_fs = deptNameMap.get(tpv_dep_f) != null ? deptNameMap.get(tpv_dep_f) : 0L;
        Long tpv_supplier_fs = supNameMap.get(tpv_supplier_f) != null ? supNameMap.get(tpv_supplier_f) : 0L;
        if (YEM.isEmpty(tpv_supplier_fs) | YEM.isEmpty(tpv_dep_fs)) {
            return result;
        }
        for (Row row : queryResult(tpv_orgcredit_fs, tpv_credittype_fs, tpv_supgroup_f, tpv_dep_fs, tpv_supplier_fs)) {
            JSONObject jsonObject = new JSONObject(new LinkedHashMap<>());
            String supName = row.getLong("tpv_sup") != null ? supMap.get(row.getLong("tpv_sup")) : null;
            String deptName = row.getLong("tpv_dep") != null ? deptMap.get(row.getLong("tpv_dep")) : null;
            jsonObject.put("credittype", row.getString("tpv_credittype"));  //授信维度
            jsonObject.put("orgcredit", row.getString("tpv_orgcredit"));  //查询维度
            jsonObject.put("credittypeText", "A".equals(row.getString("tpv_credittype")) ? "集团额度" : "单抬头额度");  //授信维度
            jsonObject.put("orgcreditText", "A".equals(row.getString("tpv_orgcredit")) ? "全司" : "部门");  //查询维度
            jsonObject.put("dept", deptName != null ? deptName : "");  //部门
            jsonObject.put("supgroup", row.getString("tpv_supgroup"));  //供应商集团
//            jsonObject.put("supplier", supName != null ? supName : "");  //供应商
            jsonObject.put("supplier", supName != null ? supName : "");  //供应商
            jsonObject.put("signingamt", BigDecimalUtils.ceil(row.getBigDecimal("tpv_signingamt").doubleValue(), 2));  //签约(额度)
            jsonObject.put("signingamtText", row.getString("tpv_signingamts"));  //签约(额度)
            jsonObject.put("afteramt", BigDecimalUtils.ceil(row.getBigDecimal("tpv_afteramt").doubleValue(), 2));  //现货(额度)
            jsonObject.put("afteramtText", row.getString("tpv_afteramts"));  //签约(额度)
            jsonObject.put("forwardamt", BigDecimalUtils.ceil(row.getBigDecimal("tpv_forwardamt").doubleValue(), 2));  //远期(额度)
            jsonObject.put("forwardamtText", row.getString("tpv_forwardamts"));  //签约(额度)
            jsonObject.put("addsigning", BigDecimalUtils.ceil(row.getBigDecimal("tpv_addsigning").doubleValue(), 2));  //签约(理论占用)
            jsonObject.put("addsigningText", row.getString("tpv_addsignings"));  //签约(额度)
            jsonObject.put("addafter", BigDecimalUtils.ceil(row.getBigDecimal("tpv_addafter").doubleValue(), 2));  //现货(理论占用)
            jsonObject.put("addafterText", row.getString("tpv_afteramts"));  //签约(额度)
            jsonObject.put("addforward", BigDecimalUtils.ceil(row.getBigDecimal("tpv_addforward").doubleValue(), 2));  //远期(理论占用)
            jsonObject.put("addforwardText", row.getString("tpv_addforwards"));  //签约(额度)
            jsonObject.put("leavesigning", BigDecimalUtils.ceil(row.getBigDecimal("tpv_leavesigning").doubleValue(), 2));  //签约(理论剩余)
            jsonObject.put("leavesigningText", row.getString("tpv_leavesignings"));  //签约(额度)
            jsonObject.put("leaveafter", BigDecimalUtils.ceil(row.getBigDecimal("tpv_leaveafter").doubleValue(), 2));  //现货(理论剩余)
            jsonObject.put("leaveafterText", row.getString("tpv_leaveafters"));  //签约(额度)
            jsonObject.put("leaveforward", BigDecimalUtils.ceil(row.getBigDecimal("tpv_leaveforward").doubleValue(), 2));  //远期(理论剩余)
            jsonObject.put("leaveforwardText", row.getString("tpv_leaveforwards"));  //签约(额度)
            jsonObject.put("fastleavesigning", BigDecimalUtils.ceil(row.getBigDecimal("tpv_fastleavesigning").doubleValue(), 2));  //签约(实际剩余额度)
            jsonObject.put("fastleavesigningText", row.getString("tpv_fastleavesignings"));  //签约(额度)
            jsonObject.put("fastleaveafter", BigDecimalUtils.ceil(row.getBigDecimal("tpv_fastleaveafter").doubleValue(), 2));  //现货(实际剩余额度)
            jsonObject.put("fastleaveafterText", row.getString("tpv_fastleaveafters"));  //签约(额度)
            jsonObject.put("fastleaveforward", BigDecimalUtils.ceil(row.getBigDecimal("tpv_fastleaveforward").doubleValue(), 2));  //远期(实际剩余额度)
            jsonObject.put("fastleaveforwardText", row.getString("tpv_fastleaveforwards"));  //签约(额度)
            result.add(jsonObject);
        }
        return result;
    }

}
