package tpv.tpv_wm.tpv_crm.java.task;

import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.exception.KDException;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * ClassName: SupplierLimitsTask
 * Description: 供应商额度维护分录冻结状态-定时任务
 * date: 2023/10/25 18:44
 *
 * @author: Guo jiaChen
 */
public class SupplierLimitsTask extends AbstractTask {
    private static final LocalDateTime previousDate = LocalDateTime.now().minusDays(180);
    private static HashMap<Long, Boolean> suppliersFreeze = new HashMap<>();

    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
        QFilter[] Filters = new QFilter[1];
        Filters[0] = new QFilter("billstatus", QCP.equals, "C");//单据状态=已审核
        //获取供应商额度维护
        DynamicObject[] bills = BusinessDataServiceHelper.load("tpv_bd_supplierlimits", "tpv_supplierlimit_sup,tpv_suppliers", Filters);
        for (DynamicObject bill : bills) {
            DynamicObjectCollection collection = bill.getDynamicObjectCollection("tpv_supplierlimit_sup");
            DynamicObjectCollection updateCol = changeFreeze(collection);
            bill.set("tpv_supplierlimit_sup", updateCol);
            SaveServiceHelper.update(bill);
        }
    }


    /**
     * @Title: changeFreeze
     * @Description: 修改冻结状态
     * @param:
     * @return: a
     */
    private DynamicObjectCollection changeFreeze(DynamicObjectCollection collection) {
        boolean supFreezeFlag;
        for (DynamicObject object : collection) {
            DynamicObject suppliers = object.getDynamicObject("tpv_suppliers");
            long id = suppliers.getLong("id");
            Boolean flag = suppliersFreeze.get(id);
            if (null == flag) {
                supFreezeFlag = getSupFreeze(id);
                suppliersFreeze.put(id, supFreezeFlag);
            }
            object.set("tpv_isfreeze", getSupFreeze(id) == true ? "A" : "B");
        }
        return collection;
    }

    /**
     * @Title: getSupFreeze
     * @Description: 获取冻结状态
     * @param: id
     * @return:
     */
    private boolean getSupFreeze(long id) {
        Date previousDateTime = Date.from(previousDate.atZone(ZoneId.systemDefault()).toInstant());
        //构造过滤条件
        QFilter[] Filters = new QFilter[4];
        Filters[0] = new QFilter("supplier.id", QCP.equals, id);//供应商名称相等
        Filters[1] = new QFilter("billstatus", QCP.equals, "C");//单据状态=已审核
        Filters[2] = new QFilter("auditdate", QCP.large_equals, previousDateTime);//审核时间大于等于180天之前的时间
        Filters[3] = new QFilter("tpv_ispayment", QCP.equals, true);//审核时间大于等于180天之前的时间
        DynamicObject[] bills = BusinessDataServiceHelper.load("im_purinbill", "biztime,supplier.name,billstatus,auditdate", Filters);

        //180天是否有已审核单据
        return !(bills.length > 0);
    }
}
