package tpv.tpv_wm.tpv_dt.java.api.client;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.sys.utils.Api;
import com.itopview.wm.sys.utils.ApiHelper;
import com.itopview.wm.sys.utils.ApiLog;
import com.itopview.wm.utils.HttpHelper;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * ClassName: DHLClient
 * Description:
 * date: 2023/9/7 9:30
 *
 * @author: Guo jiaChen
 * @version:
 * @since JDK 1.8
 */
public class DHLClient extends AbstractBillPlugIn {
    public final static String TRACKING_NUMBER = "shipmentTrackingNumber";//发货编号
    public final static String REFERENCE_TYPE = "shipmentReferenceType";//运输类型
    public final static String TRACKING_VIEW = "trackingView";//轨迹视图
    public final static String LEVEL_DETAIL = "levelOfDetail";//详细级别
    public final static String SHIPPING_SYSTEM_PLATFORM_NAME = "Shipping-System-Platform-Name";//船只系统平台名称
    public final static String WEBSTORE_PLATFORM_VERSION = "Webstore-Platform-Version";//网店平台版本
    public final static String FORMAT_DATE = "yyyy-MM-dd\'T\'HH:mm:ss";//时间转换格式
    public static ApiHelper apiHelper = new ApiHelper("api.dhl");//apiHelper

    /**
     * @param :
     * @return void
     * @author: chen
     * @date: 2023/11/19 0:32
     * @description: 获取船只信息
     */
    public static HashMap<String, Object> setShipmentDesc(String courier, int rowIndex) {
        HashMap<String, Object> shipmentMsg = new HashMap<>();
        String errorMsg = "";//异常信息

        if (!courier.isEmpty()) {
            //获取返回数据
            String result = runContract(courier);
            if (result.isEmpty()) {
                errorMsg = "接口未配置!";
            } else {
                JSONObject body = JSON.parseObject(result);//将结果转换为JSONObject
                String status = body.getString("status");//获取状态
                if ("400".equals(status)) {
                    //请求异常,订单追踪码格式错误
                    errorMsg += "分录第" + rowIndex + "行:请确认快递单号格式!" + "\n";
                } else if ("404".equals(status)) {
                    //请求失败,订单追踪码输入有误
                    errorMsg += "分录第" + rowIndex + "行:请输入正确的快递单号!" + "\n";
                } else {
                    JSONArray shipments = body.getJSONArray("shipments");
                    JSONObject shipment = (JSONObject) shipments.get(0);
                    String shipmentStatus = shipment.getString("status");//获取状态
                    String timestamp = shipment.getString("shipmentTimestamp");//货物装运时间戳
                    String deliveryDate = shipment.getString("estimatedDeliveryDate");//货物送达时间
                    if ("Success".equals(shipmentStatus) || apiHelper.getParam("successCode").equals(shipmentStatus)) {//请求成功!
                        Map<String, String> shipmentDesc = getShipmentStatus(shipment);//获取当前快递状态
                        String date = "";
                        String time = "";
                        if (shipmentDesc != null) {
                            String typeCode = shipmentDesc.get("typeCode");
                            if ("OK".equals(typeCode)) {//已签收
                                String nowStatus = shipmentDesc.get("description");//当前快递状态
                                date = shipmentDesc.get("date");//年-月-日
                                time = shipmentDesc.get("time");//时-分-秒
                                shipmentMsg.put("tpv_courierstatus", nowStatus);
                            }
                        }
                        Date signDate = null;
                        Date sendDate = null;
                        if (deliveryDate != null) {
                            signDate = parseDate(deliveryDate, FORMAT_DATE);//签收日期
                        }
                        if (!date.isEmpty() && !time.isEmpty()) {
                            signDate = parseDate(date + "T"+ time , FORMAT_DATE);
                        }
                        if (timestamp != null) {
                            sendDate = parseDate(timestamp, FORMAT_DATE);//寄出日期
                        }
                        shipmentMsg.put("tpv_receipteddate", signDate);
                        shipmentMsg.put("tpv_sendoutdate", sendDate);
                    }
                }
            }
        } else {
            errorMsg = "请输入快递单号!";
        }
        shipmentMsg.put("errorMsg", errorMsg);
        return shipmentMsg;
    }


    /**
     * @param number:
     * @return String
     * @author: chen
     * @date: 2023/11/19 0:32
     * @description: 调用接口
     */
    private static String runContract(String number) {
        //  number = "5980623180";

        //请求参数
        String property = TRACKING_NUMBER + "=" + number +
                "&" + REFERENCE_TYPE + "=" + apiHelper.getParam(REFERENCE_TYPE) + //运输类型
                "&" + TRACKING_VIEW + "=" + apiHelper.getParam(TRACKING_VIEW) +  //轨迹视图
                "&" + LEVEL_DETAIL + "=" + apiHelper.getParam(LEVEL_DETAIL) +  //详细级别
                "&" + SHIPPING_SYSTEM_PLATFORM_NAME + "=" + apiHelper.getParam(SHIPPING_SYSTEM_PLATFORM_NAME) + //船只系统平台名称
                "&" + WEBSTORE_PLATFORM_VERSION + "=" + apiHelper.getParam(WEBSTORE_PLATFORM_VERSION);  //网店平台版本

        if (apiHelper.isExist()) {//
            Api api = apiHelper.getApi("dhl");//获取api

            if (api.isExist()) {//api
                ApiLog log = apiHelper.log(api);//获取日志
                //请求地址+请求参数
                String url = api.getUrl() + "?" + property;

                //Get请求(请求头信息)
                HashMap<String, String> headers = new HashMap<>();
                headers.put("Message-Reference", "13713c7bba4c16fab1ac826923640645");
                headers.put("Message-Reference-Date", "Sat,02 Apr 2022 10:13:22 GM");
                headers.put("Accept-Language", "chi");
                headers.put("Authorization", apiHelper.getParam("Authorization"));//验证信息

                String result = "";//返回结果
                try {
                    result = HttpHelper.httpGet(url, headers);//获取请求返回的结果
                    JSONObject body = JSON.parseObject(result);//将结果转换为JSONObject
                    String status = body.getString("status");//获取状态
                    log.setRequestData(url);
                    log.setResponseData(result);
                    if ("400".equals(status) || "404".equals(status)) {
                        log.setSuccess(false);
                    } else {
                        log.setSuccess(true);
                    }
                    log.write();
                } catch (Exception e) {
                    e.printStackTrace();
                    log.setErrorMsg(YEM.exceptionToStr(e));
                    log.setSuccess(false);
                    log.write();
                }
                return result;
            }
        }
        return "";
    }

    /**
     * 获取当前快递状态
     *
     * @param shipment
     */
    private static Map<String, String> getShipmentStatus(JSONObject shipment) {
        Map<String, String> map = new HashMap<>();
        JSONArray pieces = shipment.getJSONArray("pieces");
        JSONObject piece = (JSONObject) pieces.get(0);
        JSONArray events = piece.getJSONArray("events");
        if (events.size() > 0) {
            JSONObject event = (JSONObject) events.get(events.size() - 1);//获取快递最后的状态
            map.put("trackingNumber", piece.getString("trackingNumber"));//获取快递单号
            map.put("date", event.getString("date"));//获取当前节点时间(年-月-日)
            map.put("time", event.getString("time"));//获取当前节点时间(时-分-秒)
            map.put("typeCode", event.getString("typeCode"));//获取当前节点状态码
            map.put("description", event.getString("description"));//获取当前节点状态描述
            return map;
        }
        return null;
    }

    /**
     * 日期格式转换
     *
     * @param date
     * @param formatDate
     * @return
     */
    private static Date parseDate(String date, String formatDate) {
        SimpleDateFormat format = new SimpleDateFormat(formatDate);
        try {
            return format.parse(date);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

}
