package tpv.tpv_wm.tpv_dt.java.formplugin.dynamic;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.control.Control;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @Description: 【回填报关数据】按钮，默认锁定，当【贸易性质】=“出口业务”时解锁，点击弹出当前单据商品明细，上述四个字段必填
 * @Project: zhck
 * @Package: com.itopview.wm.dt.formplugin.dynamic.BackFillSaleEdit
 * @Date: 2023/11/23 16:59
 * @Author: Heisenberg Wenbei He
 * @Version: 1.0
 */
public class BackFillSaleEdit extends AbstractFormPlugin {
    private static final String DATA_ENTITY = "tpv_materialentry"; // 商品明细分录

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners("btncancel", "btnok"); // 监听确认和取消按钮
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        this.init();
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        Control source = (Control) evt.getSource();
        // 点击确认按钮
        if (StringUtils.equals(source.getKey(), "btnok")) {
            DynamicObjectCollection dataEntity = this.getModel().getEntryEntity(DATA_ENTITY);
            HashMap<String, Object> map = new HashMap<>();
            map.put("tpv_customnum", this.getModel().getValue("tpv_customnum"));
            map.put("dataEntity", dataEntity);
            this.getView().returnDataToParent(map);
        } else if (StringUtils.equals(source.getKey(), "btncancel")) {
            // 被点击控件为取消则设置返回值为空并关闭页面（在页面关闭回调方法中必须验证返回值不为空，否则会报空指针）
            this.getView().returnDataToParent(null);
        }
        this.getView().close();
    }

    private void init() {
        Map<String, Object> customParams = this.getView().getFormShowParameter().getCustomParams();
        Object pkValue = customParams.get("pkValue");
        DynamicObject bill = BusinessDataServiceHelper.loadSingle(pkValue, "tpv_dt_salesshipping");
        if (bill == null) {
            return;
        }
        DynamicObjectCollection collection = bill.getDynamicObjectCollection(DATA_ENTITY);
        DynamicObjectCollection currentForm = this.getModel().getDataEntity(true)
                .getDynamicObjectCollection(DATA_ENTITY);
        // 报关单号
        String customnum = bill.getString("tpv_customnum");
        Optional.ofNullable(customnum)
                .ifPresent(number -> this.getModel().setValue("tpv_customnum", number));
        for (DynamicObject currentRow : collection) {
            DynamicObject newRow = currentForm.addNew();
            // 品名
            DynamicObject prodname = currentRow.getDynamicObject("tpv_prodname");
            Optional.ofNullable(prodname)
                    .ifPresent(name -> newRow.set("tpv_prodname", name));
            // 商品编号
            DynamicObject material = currentRow.getDynamicObject("tpv_material");
            Optional.ofNullable(material)
                    .ifPresent(goods -> newRow.set("tpv_material", goods));
            // 规格
            String specation = currentRow.getString("tpv_specation");
            Optional.ofNullable(specation)
                    .ifPresent(norm -> newRow.set("tpv_specation", norm));
            // 材质
            String shader = currentRow.getString("tpv_shader");
            Optional.ofNullable(shader)
                    .ifPresent(texture -> newRow.set("tpv_shader", texture));
            // 产地
            DynamicObject origin = currentRow.getDynamicObject("tpv_origin");
            Optional.ofNullable(origin)
                    .ifPresent(source -> newRow.set("tpv_origin", source));
            // 计量单位
            DynamicObject unit = currentRow.getDynamicObject("tpv_unit");
            Optional.ofNullable(unit)
                    .ifPresent(measure -> newRow.set("tpv_unit", measure));
            // 数量
            BigDecimal qty = currentRow.getBigDecimal("tpv_qty");
            Optional.ofNullable(qty)
                    .ifPresent(quanlity -> newRow.set("tpv_qty", quanlity));
            // 系数
            BigDecimal coefficients = currentRow.getBigDecimal("tpv_coefficients");
            Optional.ofNullable(coefficients)
                    .ifPresent(ratio -> newRow.set("tpv_coefficients", ratio));
            // 包装数量
            BigDecimal packnum = currentRow.getBigDecimal("tpv_packnum");
            Optional.ofNullable(packnum)
                    .ifPresent(packedquanlity -> newRow.set("tpv_packnum", packedquanlity));
            // 包装单位
            DynamicObject packagingunit = currentRow.getDynamicObject("tpv_packagingunit");
            Optional.ofNullable(packagingunit)
                    .ifPresent(packedinit -> newRow.set("tpv_packagingunit", packedinit));
            // 干吨数量
            BigDecimal drytonsnum = currentRow.getBigDecimal("tpv_drytonsnum");
            Optional.ofNullable(drytonsnum)
                    .ifPresent(dryquanlity -> newRow.set("tpv_drytonsnum", dryquanlity));
            // 不含税金额
            BigDecimal nottaxamount = currentRow.getBigDecimal("tpv_nottaxamount");
            Optional.ofNullable(nottaxamount)
                    .ifPresent(excludtaxamount -> newRow.set("tpv_nottaxamount", excludtaxamount));
            // 含税单价
            BigDecimal taxprice = currentRow.getBigDecimal("tpv_taxprice");
            Optional.ofNullable(taxprice)
                    .ifPresent(includtaxprice -> newRow.set("tpv_taxprice", includtaxprice));
            // 不含税单价
            BigDecimal price = currentRow.getBigDecimal("tpv_price");
            Optional.ofNullable(price)
                    .ifPresent(excludtaxprice -> newRow.set("tpv_price", excludtaxprice));
            // 含税金额
            BigDecimal taxpramount = currentRow.getBigDecimal("tpv_taxpramount");
            Optional.ofNullable(taxpramount)
                    .ifPresent(includtaxamount -> newRow.set("tpv_taxpramount", includtaxamount));
            // 增值税率%
            BigDecimal vatrates = currentRow.getBigDecimal("tpv_vatrates");
            Optional.ofNullable(vatrates)
                    .ifPresent(vatrate -> newRow.set("tpv_vatrates", vatrate));
            // 增值税额
            BigDecimal vatmoney = currentRow.getBigDecimal("tpv_vatmoney");
            Optional.ofNullable(vatmoney)
                    .ifPresent(vatamount -> newRow.set("tpv_vatmoney", vatamount));
            // 销售指导价
            BigDecimal thirdpartyprice = currentRow.getBigDecimal("tpv_thirdpartyprice");
            Optional.ofNullable(thirdpartyprice)
                    .ifPresent(guideprice -> newRow.set("tpv_thirdpartyprice", guideprice));
            // 已出库数量
            BigDecimal issuedquantitys = currentRow.getBigDecimal("tpv_issuedquantitys");
            Optional.ofNullable(issuedquantitys)
                    .ifPresent(outnum -> newRow.set("tpv_issuedquantitys", outnum));
            // 未出库数量
            BigDecimal unissuedquanti = currentRow.getBigDecimal("tpv_unissuedquanti");
            Optional.ofNullable(unissuedquanti)
                    .ifPresent(innum -> newRow.set("tpv_unissuedquanti", innum));
            // 已出库包装数量
            BigDecimal packagess = currentRow.getBigDecimal("tpv_packagess");
            Optional.ofNullable(packagess)
                    .ifPresent(shippedQuality -> newRow.set("tpv_packagess", shippedQuality));
            // 未出库包装数量
            BigDecimal unshippeds = currentRow.getBigDecimal("tpv_unshippeds");
            Optional.ofNullable(unshippeds)
                    .ifPresent(unshippedQuality -> newRow.set("tpv_unshippeds", unshippedQuality));
            // 是否实提完毕
            Boolean actuallymentioned = currentRow.getBoolean("tpv_actuallymentioned");
            Optional.ofNullable(actuallymentioned)
                    .ifPresent(isornor -> newRow.set("tpv_actuallymentioned", isornor));
            // 备注
            String remarks = currentRow.getString("tpv_remarks");
            Optional.ofNullable(remarks)
                    .ifPresent(note -> newRow.set("tpv_remarks", note));
            // 结算单价
            BigDecimal settlementprice = currentRow.getBigDecimal("tpv_settlementprice");
            Optional.ofNullable(settlementprice)
                    .ifPresent(settlementPrice -> newRow.set("tpv_settlementprice", settlementPrice));
            // 货物批次选择单据
            String document = currentRow.getString("tpv_document");
            Optional.ofNullable(document)
                    .ifPresent(file -> newRow.set("tpv_document", file));
            // 货物批次选择合同号
            String selectiondocu = currentRow.getString("tpv_selectiondocu");
            Optional.ofNullable(selectiondocu)
                    .ifPresent(file -> newRow.set("tpv_selectiondocu", file));
            // 结算价格确认人
            DynamicObject settlementpriceman = currentRow.getDynamicObject("tpv_settlementpriceman");
            Optional.ofNullable(settlementpriceman)
                    .ifPresent(businessman -> newRow.set("tpv_settlementpriceman", businessman));
            // 结算价格确认时间
            Date date = currentRow.getDate("tpv_date");
            Optional.ofNullable(date)
                    .ifPresent(confirmdate -> newRow.set("tpv_date", confirmdate));
            // 已价格确认数量
            BigDecimal confirmed = currentRow.getBigDecimal("tpv_confirmed");
            Optional.ofNullable(confirmed)
                    .ifPresent(confirmnum -> newRow.set("tpv_confirmed", confirmnum));
            // 价格确认货物批次数量
            BigDecimal cargobatchqty = currentRow.getBigDecimal("tpv_cargobatchqty");
            Optional.ofNullable(cargobatchqty)
                    .ifPresent(batchnum -> newRow.set("tpv_cargobatchqty", batchnum));
            // 数量(记录)
            BigDecimal qtyrecord = currentRow.getBigDecimal("tpv_qtyrecord");
            Optional.ofNullable(qtyrecord)
                    .ifPresent(recordnum -> newRow.set("tpv_qtyrecord", recordnum));
            // FOB单价
            BigDecimal fobprice = currentRow.getBigDecimal("tpv_fobprice");
            Optional.ofNullable(fobprice)
                    .ifPresent(unitprice -> newRow.set("tpv_fobprice", unitprice));
            // FOB金额
            BigDecimal fobamount = currentRow.getBigDecimal("tpv_fobamount");
            Optional.ofNullable(fobamount)
                    .ifPresent(amount -> newRow.set("tpv_fobamount", amount));
            // 报关商品名称
            String customname = currentRow.getString("tpv_customname");
            Optional.ofNullable(customname)
                    .ifPresent(name -> newRow.set("tpv_customname", name));
        }
    }
}
