package tpv.tpv_wm.tpv_dt.java.formplugin.dynamic;

import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.DateUtil;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.IFormView;
import kd.bos.form.control.Control;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.events.BeforeClickEvent;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * ClassName: ContractMatchFormPlugin
 * Description: 匹配合同信息
 * date: 2023/10/31 15:47
 *
 * @author: Guo jiaChen
 */
public class ContractMatchEdit extends AbstractFormPlugin {
    private static final String contractEntry = "tpv_continformation";
    private static final String contractEntryDetail = "tpv_contractinformation";


    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners(new String[]{"btnok", "tpv_query", "tpv_deleteentry"});
        this.addItemClickListeners(new String[]{"tpv_advcontoolbarap"});
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        IFormView view = this.getView();
        Map<String, Object> params = view.getFormShowParameter().getCustomParams();
        Object tpvDepartment = this.getView().getParentView().getModel().getValue("tpv_department");
        this.getModel().setValue("tpv_org",tpvDepartment);
        JSONObject supplier = (JSONObject) params.get("supplier");
        if (null != supplier) {
            this.getModel().setValue("tpv_invoicedept", supplier.get("id"));
        }
        initData(true);
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        ChangeData[] changeSet = e.getChangeSet();
        int index = changeSet[0].getRowIndex();
        switch (name) {
            case "tpv_taxrate1":
            case "tpv_curmatchamo1":
                calculate(index);
                break;
//            case "tpv_rkcurmatchqty1":
//                showMessage(index);
//                break;
//            default:
//                break;
        }
    }

    /**
     * @param : index
     * @return void
     * @author: chen
     * @date: 2023/12/6 9:33
     * @description: 本次匹配数量大于未匹配数量时提示
     */
    private void showMessage(int index) {
        IDataModel model = this.getModel();
        BigDecimal qty1 = (BigDecimal) model.getValue("tpv_qty1", index);
        BigDecimal rkMatchQty1 = (BigDecimal) model.getValue("tpv_rkmatchqty1", index);
        BigDecimal rkCurMatchQty1 = (BigDecimal) model.getValue("tpv_curmatchqty1", index);
        if (qty1.compareTo(rkMatchQty1.add(rkCurMatchQty1)) < 0) {
            this.getView().showTipNotification(String.format("合同信息分录第%s行,本次匹配数量不能大于未匹配数量!", index + 1));
        }

    }

    /**
     * @Title: calculate
     * @Description: 计算本次匹配金额(增值税额) + (不含税)
     */
    private void calculate(int index) {
        BigDecimal taxRate = (BigDecimal) this.getModel().getValue("tpv_taxrate1");
        BigDecimal curmatchamo = (BigDecimal) this.getModel().getValue("tpv_curmatchamo1");
        BigDecimal divisor = new BigDecimal(100);
        BigDecimal decimal = taxRate.divide(divisor);
        if (taxRate != null && curmatchamo != null) {
            BigDecimal curmatchamointax = curmatchamo.divide(decimal.add(new BigDecimal(1)), 10, RoundingMode.HALF_UP).multiply(decimal);
            this.getModel().setValue("tpv_curamountintax1", curmatchamointax, index);
            this.getModel().setValue("tpv_curamountnotax1", curmatchamo.subtract(curmatchamointax), index);
        }
    }

    @Override
    public void beforeClick(BeforeClickEvent evt) {
        super.click(evt);
        String key = ((Control) evt.getSource()).getKey();
        switch (key) {
            case "btnok":
                pickOk();
                break;
            case "tpv_query":
                initData(false);
                break;
            default:
                break;
        }

    }


    /**
     * @Title: pickOk
     * @Description: 校验 + 回填
     */
    private void pickOk() {
        IFormView view = this.getView();
        IFormView parentView = this.getView().getParentView();
        IDataModel parentModel = parentView.getModel();

        EntryGrid control = this.getControl(contractEntryDetail);
        int[] selectRows = control.getSelectRows();
        Boolean flag = false;
        DynamicObjectCollection collections = this.getModel().getDataEntity(true).getDynamicObjectCollection(contractEntryDetail);
        DynamicObjectCollection dynamicObjectCollection = new DynamicObjectCollection();

        //复选框选中
        for (DynamicObject object : collections) {
            if (object.getBoolean("tpv_checkbox")) {
                BigDecimal qty = object.getBigDecimal("tpv_curmatchqty1");
                if (null == qty || qty.compareTo(new BigDecimal("0")) == 0) {
                    view.showErrorNotification("本次匹配数量为空,请检查!");
                    return;
                }
                dynamicObjectCollection.add(object);
                flag = true;
            }
        }
        if (!flag) {
            view.showErrorNotification("未勾选合同信息！");
            return;
        }
        //选中行选中
//        for (int selectRow : selectRows) {
//            DynamicObject object = collections.get(selectRow);
//            BigDecimal qty = object.getBigDecimal("tpv_curmatchqty1");
//            if (null == qty || qty.compareTo(new BigDecimal("0")) == 0) {
//                view.showErrorNotification("本次匹配数量为空,请检查!");
//                return;
//            }
//            dynamicObjectCollection.add(object);
//        }

        HashMap<String, DynamicObjectCollection> collectionHashMap = new HashMap<>();
        collectionHashMap.put("cols", dynamicObjectCollection);
        view.returnDataToParent(collectionHashMap);
        view.sendFormAction(parentView);
        view.close();
    }


    /**
     * @Title: initData
     * @Description: 初始化数据
     */
    private void initData(boolean b) {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        ArrayList<QFilter> filters = new ArrayList<>();
        DynamicObject tpvCompany = (DynamicObject) this.getView().getParentView().getModel().getValue("tpv_company");
        filters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
        DynamicObject supplier = (DynamicObject) model.getValue("tpv_invoicedept");
        DynamicObject org = (DynamicObject) model.getValue("tpv_org");
        String contractnum = (String) model.getValue("tpv_purcontractnum");//采购合同号
        Date startDate = (Date) model.getValue("tpv_invocestartdate");//其实日期
        Date endDate = (Date) model.getValue("tpv_invoceenddate");//截至日期

        if (tpvCompany!=null){
            filters.add(new QFilter("tpv_company", QCP.equals, tpvCompany.getPkValue()));//公司
        }
        if (null != supplier) {
            long supID = supplier.getLong("id");//供应商id
            filters.add(new QFilter("tpv_supplier", QCP.equals, supID));//供应商
        }
        if (null != org) {
            long supID = org.getLong("id");//部门id
            filters.add(new QFilter("tpv_department", QCP.equals, supID));//部门
        }
        if (null != contractnum && contractnum.length() > 0) {
            filters.add(new QFilter("tpv_contractnumber", QCP.ILIKE, "%" + contractnum + "%"));//采购合同号
        }
        if (null != startDate) {
            filters.add(new QFilter("tpv_bizdate", QCP.large_equals, DateUtil.getStartOfDay(startDate)));//开票日期从
        }
        if (null != endDate) {
            filters.add(new QFilter("tpv_bizdate", QCP.less_equals, DateUtil.getEndOfDay(endDate)));//开票日期至
        }

        filters.add(new QFilter("tpv_materialentry", QCP.not_equals, null));//开票日期至
        DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_purchase",
                "id,billno,billstatus,tpv_contractnumber,tpv_department,tpv_supplier" + ",tpv_materialentry" +
                        ",tpv_materialentry.tpv_material,tpv_materialentry.tpv_prodname,org,tpv_operator" +
                        ",tpv_materialentry.tpv_shader,tpv_materialentry.tpv_origin,tpv_materialentry.tpv_qty,tpv_materialentry.tpv_specation," +
                        "tpv_materialentry.tpv_taxpramount,tpv_materialentry.tpv_unit tpv_conunit1"
                , filters.toArray(new QFilter[0]));
        DataSet selected = queryDataSet.select("id", "billno", "tpv_contractnumber contractnumber", "tpv_supplier supplier", "tpv_department department"
                , "tpv_materialentry.tpv_material material", "tpv_materialentry entryID", "tpv_materialentry.tpv_prodname prodname", "tpv_operator operator", "org"
                , "tpv_materialentry.tpv_shader shader", "tpv_materialentry.tpv_origin origin", "tpv_materialentry.tpv_qty qty", "tpv_materialentry.tpv_specation specation",
                "tpv_materialentry.tpv_taxpramount taxpramount","tpv_conunit1");

        selected.print(true);

        //分录填充数据
        if (selected.isEmpty()) {
            this.getModel().deleteEntryData(contractEntryDetail);
            return;
        }
        this.getModel().deleteEntryData(contractEntryDetail);
        ArrayList<String> entryIDs = new ArrayList<>();
        DataSet copy = selected.copy();
        for (Row row : copy) {
            entryIDs.add(row.getString("entryID"));
        }
        setInvoiceData(selected, entryIDs);
    }

    /**
     * @Title: setInvoiceData
     * @Description: 分录填充数据
     */
    private void setInvoiceData(DataSet queryDataSet, ArrayList<String> entryIDS) {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        HashMap<String, BigDecimal> matchedQty = getMatchedQty(entryIDS);//获取已匹配数量等
        IDataModel parentModel = view.getParentView().getModel();
        Long id = (Long) parentModel.getDataEntity().getPkValue();
        DynamicObjectCollection entity = model.getEntryEntity(contractEntryDetail);
        for (Row row : queryDataSet) {
            String entryID = row.getString("entryID");//合同分录ID
            BigDecimal matchedNum = null == matchedQty.get(entryID) ? BigDecimal.ZERO : matchedQty.get(entryID);//已匹配数量
            BigDecimal num =  null == row.getBigDecimal("qty") ? BigDecimal.ZERO : row.getBigDecimal("qty");//数量
            //未匹配数量≤0不显示
            if ((num.subtract(matchedNum)).compareTo(BigDecimal.ZERO) <= 0) {
                continue;
            }
            int rowIndex = this.getModel().createNewEntryRow(contractEntryDetail);
            model.setValue("tpv_condept1", row.getLong("department"), rowIndex);//部门
            model.setValue("tpv_connums1", row.getString("contractnumber"), rowIndex);//采购合同号
            model.setValue("tpv_consupplier1", row.getLong("supplier"), rowIndex);//供应商
            model.setValue("tpv_material1", row.getLong("material"), rowIndex);//物料编码
            model.setValue("tpv_conprodname1", row.getString("prodname"), rowIndex);//品名
            model.setValue("tpv_conshader1", row.getString("shader"), rowIndex);//材质
            model.setValue("tpv_conorigin1", row.getLong("origin"), rowIndex);//产地
            model.setValue("tpv_conspecation1", row.getString("specation"), rowIndex);//规格
            model.setValue("tpv_conqty1", row.getBigDecimal("qty"), rowIndex);//数量
            model.setValue("tpv_contractamo1", row.getBigDecimal("taxpramount"), rowIndex);//金额
            model.setValue("tpv_taxrate1", new BigDecimal("13"), rowIndex);//税率%
            model.setValue("tpv_conentryid1", row.getString("entryID"), rowIndex);//合同分录id
            model.setValue("tpv_conusers1", row.getLong("operator"), rowIndex);//业务员
            model.setValue("tpv_conorgs1", row.getLong("org"), rowIndex);//公司
            model.setValue("tpv_conunit1", row.get("tpv_conunit1"), rowIndex);//公司
        }
    }

    /**
     * @param entryIDS:
     * @return HashMap<String, BigDecimal>
     * @author: chen
     * @date: 2023/11/19 16:57
     * @description: 获取已匹配数量
     */
    private HashMap<String, BigDecimal> getMatchedQty(ArrayList<String> entryIDS) {
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("tpv_continformation.tpv_conentryid", QCP.in, entryIDS));//单据编号
        HashMap<String, BigDecimal> map = new HashMap<>();
        DataSet load = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_purinvomater",
                "id,tpv_continformation,tpv_continformation.tpv_conentryid,tpv_continformation.tpv_curmatchqty"
                , filters.toArray(new QFilter[0]));
        DataSet executeSql = load.executeSql("select tpv_continformation.tpv_conentryid ,sum(tpv_continformation.tpv_curmatchqty) curmatchqty GROUP BY tpv_continformation.tpv_conentryid");

        executeSql.print(true);
        for (Row row : executeSql) {
            BigDecimal decimal = row.getBigDecimal("curmatchqty");
            String purinentryid = row.getString("tpv_continformation.tpv_conentryid");
            map.put(purinentryid, decimal);
        }
        return map;
    }
}

