package tpv.tpv_wm.tpv_dt.java.formplugin.dynamic;

import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.RptUtil;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.clr.DataEntityPropertyCollection;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.IFormView;
import kd.bos.form.control.Control;
import kd.bos.form.control.events.BeforeClickEvent;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 匹配发票信息（费用发票匹配单）
 * @author LiuHB
 * @version 1.0
 */
public class MatchbillEdit extends AbstractFormPlugin {
    private static final String invoiceEntry = "tpv_invoiceinformation";
    private static final String invoiceEntryDetail = "tpv_invoiceinfoentry";
    private static final String selectInvoiceEntry = "tpv_selectinvoiceinfo";

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners(new String[]{"btnok", "tpv_query", "tpv_deleteentry"});
        this.addItemClickListeners(new String[]{"tpv_advcontoolbarap"});
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        IFormView view = this.getView();
        Map<String, Object> params = view.getFormShowParameter().getCustomParams();
//        setEntries();
        JSONObject supplier = (JSONObject) params.get("supplier");
        if (null != supplier){
            this.getModel().setValue("tpv_invoicedept",supplier.get("id"));
        }
        initData();
    }

    /**
     * 分录赋值
     * @deprecated 暂时弃用
     */
    private void setEntries() {
        IFormView view = this.getView();
        IFormView parentView = this.getView().getParentView();
        IDataModel parentModel = parentView.getModel();
        DynamicObjectCollection collection = parentModel.getDataEntity(true).getDynamicObjectCollection(invoiceEntryDetail);
        DataEntityPropertyCollection properties = collection.getDynamicObjectType().getProperties();
        DynamicObjectCollection collections = this.getModel().getDataEntity(true).getDynamicObjectCollection(selectInvoiceEntry);
        DataEntityPropertyCollection selectedProperties = collections.getDynamicObjectType().getProperties();

        for (int i = 0; i < collection.size(); i++) {
            DynamicObject addNew = collections.addNew();
            DynamicObject row = collection.get(i);
            for (IDataEntityProperty property : properties) {
                String proKey = property.getName();
                if (proKey.equals("tpv_taxrate")){
                    addNew.set(proKey,row.get(proKey));
                }
                if (selectedProperties.containsKey(proKey + 1)){
                    addNew.set(proKey + 1,row.get(proKey));
                }
            }
            addNew.set("seq",i + 1);
        }
        this.getView().updateView(selectInvoiceEntry);
    }


    @Override
    public void click(EventObject evt) {
        super.click(evt);
        String key = ((Control) evt.getSource()).getKey();
        switch (key) {
            case "btnok":
                pickOk();
                break;
            case "tpv_query":
                initData();
                break;
            case "btncancel":
                this.getView().close();
            default:
                break;
        }
    }

    /**
     * @Title: pickOk
     * @Description: 校验 + 回填
     */
    private void pickOk() {
        IFormView view = this.getView();
        IFormView parentView = this.getView().getParentView();
        IDataModel parentModel = parentView.getModel();
        DynamicObject dataEntity = parentModel.getDataEntity(true);
        parentModel.deleteEntryData(invoiceEntryDetail);
        DynamicObjectCollection collections = this.getModel().getDataEntity(true).getDynamicObjectCollection(selectInvoiceEntry);
        DataEntityPropertyCollection selectedProperties = collections.getDynamicObjectType().getProperties();
        HashMap<String, DynamicObjectCollection> collectionHashMap = new HashMap<>();
        collectionHashMap.put("cols",collections);
        if (collections.size() > 0) {
            List<String> noByRows = new ArrayList<>();
            for (int i = 0; i < collections.size(); i++) {
                DynamicObject dynamicObject = collections.get(i);
                BigDecimal tpvUnmatchamotax1 = dynamicObject.getBigDecimal("tpv_unmatchamotax1"); //未匹配金额(含税)
                BigDecimal tpvCurmatchamotax1 = dynamicObject.getBigDecimal("tpv_curmatchamotax1"); //本次匹配金额(含税)
                if (tpvCurmatchamotax1.compareTo(tpvUnmatchamotax1) > 0) {
                    int seqRow = i + 1;
                    noByRows.add(String.valueOf(seqRow));
                }
            }
            if (noByRows.size() > 0) {
                view.showTipNotification("发票选择明细第 " + String.join("、", noByRows) + " 行，本次匹配金额(含税)大于未匹配金额(含税)！");
            } else {
                view.returnDataToParent(collectionHashMap);
                view.sendFormAction(parentView);
                view.close();
            }
        } else {
            view.showTipNotification("未选择行数据！");
        }

    }

    /**
     * @Title: subKey
     * @Description:
     * @param: 截取字符串
     * @return: a
     */
    private String subKey(String proKey) {
        if ( null != proKey && proKey.length() > 0 ){
            return proKey.substring(0,proKey.length() - 1);
        }else {
            return "";
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = "";

        ChangeData[] changeData = e.getChangeSet();
        int rowIndex = changeData[0].getRowIndex();
        String key = e.getProperty().getName();
        switch (key) {
            case "tpv_focus":
                pickFocus(rowIndex);
                break;
            case "tpv_invocestartdate":
            case "tpv_invoceenddate":
                checkDate(key);
            default:
                break;
        }
    }

    private void checkDate(String name) {
        Date startDate = (Date) this.getModel().getValue("tpv_invocestartdate");
        Date endDate = (Date) this.getModel().getValue("tpv_invoceenddate");
        String msg = "开票起始日期必须小于截止日期,请重新选择!";
        if (null != startDate && null != endDate){
            if (startDate.after(endDate)){
                this.getView().showTipNotification(msg);
                this.getModel().setValue(name,null);
            }
        }
    }


    /**
     * 选中调整配货信息
     */
    private void pickFocus(int rowIndex) {
        IDataModel model = this.getModel();
//        model.deleteEntryData(selectInvoiceEntry);
        DynamicObject row = model.getEntryRowEntity(invoiceEntry, rowIndex);
        DataEntityPropertyCollection properties = row.getDynamicObjectType().getProperties();
        DynamicObjectCollection entry2 = model.getEntryEntity(selectInvoiceEntry);
        DataEntityPropertyCollection properties2 = entry2.getDynamicObjectType().getProperties();
        int seq = row.getInt("seq");
        if (row.getBoolean("tpv_focus")) {
            List<Integer> seqs = entry2.stream().map(x -> x.getInt("tpv_seq")).collect(Collectors.toList());
            if (!seqs.contains(seq)) {
                int index = model.createNewEntryRow(selectInvoiceEntry);
                for (IDataEntityProperty property : properties) {
                    String key = property.getName();
                    if ("tpv_taxrates".equals(key)){
                        model.setValue(subKey(key),row.get(key), index);
                    }
                    if ("tpv_unmatchamotax".equals(key)) {
                        model.setValue("tpv_curmatchamotax1", row.get(key), index);
                        model.setValue("tpv_unmatchamotax1", row.get(key), index);
                    }
                    if (properties2.containsKey(key + "1")) {
                        model.setValue(key + "1", row.get(key), index);
                    }
                }
                model.setValue("tpv_seq", seq, index);

                BigDecimal tpv_curmatchamotax1 = BigDecimalUtils.toBigDecimal(model.getValue("tpv_curmatchamotax1", index)); //本次金额含税
                BigDecimal tpv_taxrate = BigDecimalUtils.toBigDecimal(model.getValue("tpv_taxrate", index)); //税率

                BigDecimal tpv_unmatchamointax1 = BigDecimalUtils.toBigDecimal(model.getValue("tpv_unmatchamointax1", index)); //未匹配金额(增值税额)
                BigDecimal tpv_matchamotax1 = BigDecimalUtils.toBigDecimal(model.getValue("tpv_matchamotax1", index)); //已匹配金额(含税)
                BigDecimal tpv_unmatchamotax1 = BigDecimalUtils.toBigDecimal(model.getValue("tpv_unmatchamotax1", index)); //未匹配金额(含税)
                if (tpv_curmatchamotax1.compareTo(tpv_unmatchamotax1) != 0) {
                    BigDecimal divide = tpv_curmatchamotax1.divide(BigDecimal.ONE.add(tpv_taxrate.divide(new BigDecimal("100"))));
                    BigDecimal multiply = divide.multiply(tpv_taxrate.divide(new BigDecimal("100")));
                    this.getModel().setValue("tpv_curmatchamointax1",multiply,index);
                }
                if (tpv_curmatchamotax1.compareTo(tpv_unmatchamotax1) == 0) {
                    this.getModel().setValue("tpv_curmatchamointax1",tpv_unmatchamointax1,index);
                }
                BigDecimal tpv_curmatchamointax1 = BigDecimalUtils.toBigDecimal(model.getValue("tpv_curmatchamointax1", index)); //本次匹配金额(增值税额)
                BigDecimal tpv_curmatchamonotax1 = tpv_curmatchamotax1.subtract(tpv_curmatchamointax1);
                this.getModel().setValue("tpv_curmatchamonotax1",tpv_curmatchamonotax1,index);
                this.getView().updateView(selectInvoiceEntry);
            }
        } else {
            for (int i = 0; i < entry2.size(); i++) {
                DynamicObject row2 = entry2.get(i);
                if (seq == row2.getInt("tpv_seq")) {
                    model.deleteEntryRow(selectInvoiceEntry, i);
                }
            }
        }
    }

    /**
     * @Title: initData
     * @Description: 初始化数据
     */
    private void initData() {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        IDataModel parentModel = view.getParentView().getModel();
        ArrayList<QFilter> filters = new ArrayList<>();
        DynamicObject tpvCompany = (DynamicObject) parentModel.getValue("tpv_company");
        DynamicObject tpvInvoicetype =  (DynamicObject)parentModel.getValue("tpv_invoicetype");
        filters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
//        filters.add(new QFilter("biztype", QCP.equals, "FEE"));//业务类型
        DynamicObject supplier = (DynamicObject) model.getValue("tpv_invoicedept");
        String invoiceNums = (String) model.getValue("tpv_invocenums");//发票号
        Date startDate = (Date) model.getValue("tpv_invocestartdate");//其实日期
        Date endDate = (Date) model.getValue("tpv_invoceenddate");//截至日期
        if (null != supplier){
            long supID = supplier.getLong("id");//供应商id
            filters.add(new QFilter("receivablessupp.id", QCP.equals, supID));//供应商
        }
        if (null != invoiceNums && invoiceNums.length() > 0 ){
            filters.add(new QFilter("invoiceno", QCP.ftlike, "%" + invoiceNums + "%"));//采购发票号
        }
        if (null != startDate){
            filters.add(new QFilter("issuedate", QCP.large_equals, DateUtil.getStartOfDay(startDate)));//开票日期从
        }
        if (null != endDate){
            filters.add(new QFilter("issuedate", QCP.less_equals, DateUtil.getEndOfDay(endDate)));//开票日期至
        }
        if(tpvCompany!=null){
            filters.add(new QFilter("org", QCP.equals,tpvCompany.getPkValue()));//组织
        }
        if (tpvInvoicetype!=null){
            filters.add(new QFilter("invoicetypef7", QCP.equals,tpvInvoicetype.getPkValue()));//发票类型
        }
//        DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "ap_invoice",
//                "id tpv_sourceid,billno,billstatus,invoiceno tpv_invocenum,receivablessupp.id tpv_invoicsupplier,issuedate tpv_issuedate," +
//                        "pricetaxtotal tpv_taxamount,amount tpv_notaxamount,tax tpv_amountincrease," +
//                        "shkd_mat_matchamt + shkd_nstaxamount tpv_matchamotax," +
//                        "shkd_mat_matchnotaxamt + shkd_nsnotpictax tpv_matchamonotax," +
//                        "shkd_mat_matchtaxamt + shkd_nsdistax tpv_matchamointax," +
//                        "pricetaxtotal - shkd_mat_matchamt - shkd_nstaxamount tpv_unmatchamotax," +
//                        "amount - shkd_mat_matchnotaxamt - shkd_nsnotpictax tpv_numatchamonotax," +
//                        "tax - shkd_mat_matchtaxamt - shkd_nsdistax tpv_unmatchamointax"
//                , filters.toArray(new QFilter[0]));
        DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "ap_invoice",
                "id,billno,billstatus,invoiceno tpv_invocenum,receivablessupp.id tpv_invoicsupplier,issuedate tpv_issuedate," +
                        "pricetaxtotal tpv_taxamount,amount tpv_notaxamount,tax tpv_amountincrease," +
                        "shkd_mat_matchamt,shkd_mat_matchtaxamt,shkd_mat_matchnotaxamt,shkd_nstaxamount,shkd_nsdistax,shkd_nsnotpictax"
                , filters.toArray(new QFilter[0]));
        queryDataSet = queryDataSet.addField("Cast(id as String) tpv_sourceid","tpv_sourceid");
        Long parenId = this.getView().getParentView().getModel().getDataEntity(true).getLong("id");
        QFilter qFilter = new QFilter("id",QCP.not_equals,parenId);
        if (null != supplier) {
            long supID = supplier.getLong("id");//开票单位id
            qFilter.and(new QFilter("tpv_invoicedept.id", QCP.equals, supID));//供应商
        }
        DataSet tpvDtCostbillmat = ORM.create().queryDataSet(this.getClass().getName(),"tpv_dt_costbillmat",
                "id,billno,tpv_invoiceinfoentry,tpv_invoiceinfoentry.tpv_sourceid tpv_srcid," +
                        "tpv_invoiceinfoentry.tpv_curmatchamotax tpv_curmatchamotax," +
                        "tpv_invoiceinfoentry.tpv_curmatchamonotax tpv_curmatchamonotax," +
                        "tpv_invoiceinfoentry.tpv_curmatchamointax tpv_curmatchamointax", qFilter.toArray());
        tpvDtCostbillmat = tpvDtCostbillmat.groupBy(new String[]{"tpv_srcid"})
                .sum("tpv_curmatchamotax").sum("tpv_curmatchamonotax").sum("tpv_curmatchamointax").finish();
        queryDataSet = queryDataSet.leftJoin(tpvDtCostbillmat).on("tpv_sourceid","tpv_srcid")
                .select(RptUtil.getDataSetFiledAlias(queryDataSet),RptUtil.getDataSetFiledAlias(tpvDtCostbillmat)).finish();
        DataSet dataSet = RptUtil.zeroNullNums(queryDataSet);
        dataSet = dataSet.select("tpv_sourceid","billno tpv_invoicebillno","billstatus","tpv_invocenum","tpv_invoicsupplier","tpv_issuedate",
                "tpv_taxamount","tpv_notaxamount","tpv_amountincrease",
                "shkd_mat_matchamt","shkd_mat_matchtaxamt",
                "shkd_mat_matchnotaxamt","shkd_nstaxamount","shkd_nsdistax","shkd_nsnotpictax",
                "tpv_curmatchamotax","tpv_curmatchamonotax","tpv_curmatchamointax");
        List<String> aliasList = RptUtil.getDataSetAliasList(dataSet);
        aliasList.add("tpv_curmatchamotax + shkd_nstaxamount tpv_matchamotax");
        aliasList.add("tpv_curmatchamonotax + shkd_nsnotpictax tpv_matchamonotax");
        aliasList.add("tpv_curmatchamointax + shkd_nsdistax tpv_matchamointax");
        aliasList.add("tpv_taxamount - tpv_curmatchamotax - shkd_nstaxamount tpv_unmatchamotax");
        aliasList.add("tpv_notaxamount - tpv_curmatchamonotax - shkd_nsnotpictax tpv_numatchamonotax");
        aliasList.add("tpv_amountincrease - tpv_curmatchamointax - shkd_nsdistax tpv_unmatchamointax");
        dataSet = dataSet.select(aliasList.toArray(new String[aliasList.size()]))
                .where("tpv_unmatchamotax > 0")
                .orderBy(new String[]{"tpv_issuedate desc"});
        setInvoiceData(dataSet);
    }

    /**
     * @Title: setInvoiceData
     * @Description: 分录填充数据
     */
    private void setInvoiceData(DataSet queryDataSet) {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        model.deleteEntryData(invoiceEntry);   //清除分录
        DynamicObjectType enrtyType = model.getEntryEntity(invoiceEntry).getDynamicObjectType();
        DataEntityPropertyCollection properties = enrtyType.getProperties();
        List<String> dataSetAliasList = RptUtil.getDataSetAliasList(queryDataSet);

        for (Row row : queryDataSet) {
            int index = model.createNewEntryRow(invoiceEntry);
            for (IDataEntityProperty property : properties) {
                String key = property.getName();
                if (dataSetAliasList.contains(key)) {
                    if ("tpv_invoicebillno".equals(key)) {
                        Object keys=row.get(key);
                        model.setValue(key, keys, index);
                        BigDecimal taxRate = getTaxRate(String.valueOf(keys));
                        taxRate = taxRate != null ? taxRate : BigDecimal.ZERO;
                        model.setValue("tpv_taxrates", taxRate, index);
                        continue;
                    }
                    model.setValue(key, row.get(key), index);
                }
            }
        }
        setFocusByParentModel(); //设置复选框勾选，根据父页面分录数据
    }

    //设置复选框勾选，根据父页面分录数据
    private void setFocusByParentModel() {
        IFormView view = this.getView();
        IFormView parentView = view.getParentView();
        IDataModel parentModel = parentView.getModel();
        DynamicObjectCollection collection = parentModel.getDataEntity(true).getDynamicObjectCollection(invoiceEntryDetail);
        Set<String> set = new HashSet<>();
        if (collection.size() > 0) {
            for (DynamicObject dynamicObject : collection) {
                set.add(dynamicObject.getString("tpv_sourceid"));
            }
        }
        DynamicObjectCollection dynamicObjects = view.getModel().getDataEntity(true).getDynamicObjectCollection(invoiceEntry);
        if (dynamicObjects.size() > 0) {
            for (int i = 0; i < dynamicObjects.size(); i++) {
                DynamicObject dynamicObject = dynamicObjects.get(i);
                if (set != null) {
                    if (set.size() > 0) {
                        if (set.contains(dynamicObject.getString("tpv_sourceid"))) {
                            this.getModel().setValue("tpv_focus",true,i);
                            pickFocus(i);
                        }
                    }
                }
            }
        }
        this.getView().updateView(selectInvoiceEntry);
    }

    private ArrayList<String> getSourceID(String sourceID) {
        if (sourceID.length() == 0){
            return new ArrayList<>();
        }
        String substring = sourceID.substring(0, sourceID.length() - 1);
        String[] split = substring.split(",");
        List<String> collect = Arrays.stream(split).collect(Collectors.toList());
        return (ArrayList<String>) collect;
    }

    /**
     * @Title: getTaxRate
     * @Description: 获取税率
     */
    private BigDecimal getTaxRate(String invoiceno) {
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("billno", QCP.equals, invoiceno));//合同号相等
        DynamicObject query = BusinessDataServiceHelper.loadSingle( "ap_invoice",
                "entry.taxrateid.taxrate"
                , filters.toArray(new QFilter[0]));
        DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "ap_invoice",
                "entry.taxrateid.taxrate,invoiceno"
                , filters.toArray(new QFilter[0]));
        for (Row row : queryDataSet) {
            BigDecimal taxrate = row.getBigDecimal("entry.taxrateid.taxrate");
            return taxrate;
        }

        return BigDecimal.ZERO;
    }

    public Long toLong(Object longObj) {
        if (longObj == null) {
            return null;
        } else if (longObj instanceof Long) {
            return (Long) longObj;
        } else {
            return Long.valueOf(String.valueOf(longObj));
        }
    }

    @Override
    public void beforeClick(BeforeClickEvent evt) {
        super.beforeClick(evt);
        String key = ((Control) evt.getSource()).getKey();
        switch (key) {
            case "btnok":
                DynamicObjectCollection tpvSelectinvoiceinfo = this.getView().getModel().getEntryEntity("tpv_selectinvoiceinfo");
                for (DynamicObject dynamicObject : tpvSelectinvoiceinfo) {
                    int indexOf = tpvSelectinvoiceinfo.indexOf(dynamicObject);
                    indexOf = indexOf + 1;
                    BigDecimal tpvCurmatchamotax1 = dynamicObject.getBigDecimal("tpv_curmatchamotax1");
                    if (tpvCurmatchamotax1.compareTo(BigDecimal.ZERO) == 0) {

                        evt.setCancel(true);
                        this.getView().showTipNotification("发票选择明细第" + indexOf + "行的本次匹配金额(含税)未填");
                    }


                }
        }

    }
}
