package tpv.tpv_wm.tpv_dt.java.formplugin.dynamic;

import com.itopview.wm.utils.DynamicObjectUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.control.Control;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.events.RowClickEvent;
import kd.bos.form.control.events.RowClickEventListener;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import org.apache.commons.lang3.StringUtils;
import tpv.tpv_wm.tpv_dt.java.utils.PersonnelDepUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName: PurconDeliveryEdit
 * @Description: K采购合同交货情况表
 * @author: zbh
 * @date: 2024-05-15
 */
public class PurconDeliveryEdit extends AbstractFormPlugin implements RowClickEventListener {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        EntryGrid entryGrid = this.getView().getControl("tpv_puededliy");
        entryGrid.addRowClickListener(this);
        this.addClickListeners("tpv_query");
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        puededliy();
    }

    public void puededliy() {
        this.getModel().deleteEntryData("tpv_puededliy");
        String tpvContract = (String) this.getModel().getValue("tpv_contract");
        QFilter qFilter = new QFilter("billstatus", "=", "C");
        if (YEM.isNotEmpty(tpvContract)) {
            qFilter.and("tpv_contractnumber", QCP.ftlike, "%" + tpvContract + "%");
        }

        QFilter filter = PersonnelDepUtils.departmentQFilter(this.getModel(), "tpv_muldepartment", "tpv_department");
        if (filter!=null){
            qFilter.and(filter);
        }
        String tpvDtPurchase = DynamicObjectUtil.getSelectfields("tpv_dt_purchase", false);
        String entrySelectfields = DynamicObjectUtil.getEntrySelectfields(tpvDtPurchase, "tpv_dt_purchase", "tpv_materialentry");
        String selectfields = DynamicObjectUtil.getEntrySelectfields(entrySelectfields, "tpv_dt_purchase", "tpv_dt_purchase_jhqb");
        DynamicObject[] tpvDtPurchases = BusinessDataServiceHelper.load("tpv_dt_purchase", selectfields, qFilter.toArray());

        List<Long> list = Arrays.stream(tpvDtPurchases)
                .map(dtPurchase -> dtPurchase.getLong("id"))
                .collect(Collectors.toList());


        DynamicObject[] imPurinbills = purinbills(list);
        DynamicObject[] tpvDtFreigroambills = freigroamBills(list);

        for (DynamicObject dtPurchase : tpvDtPurchases) {
            DynamicObjectCollection tpvMaterialentry = dtPurchase.getDynamicObjectCollection("tpv_materialentry");
            String tpvMaterial = "";
            BigDecimal tpvQty = BigDecimal.ZERO;
            BigDecimal tpvAmount = dtPurchase.getBigDecimal("tpv_amount");
            String tpvCargostatus = (String) dtPurchase.get("tpv_cargostatus");//货物状态
            for (DynamicObject dynamicObject : tpvMaterialentry) {
                String name = dynamicObject != null ? dynamicObject.getDynamicObject("tpv_material") == null ? "" : dynamicObject.getDynamicObject("tpv_material").getString("name") : "";
                tpvQty = tpvQty.add(dynamicObject.getBigDecimal("tpv_qty"));
                tpvMaterial = builderAppend(tpvMaterial, name);
            }
            int row = this.getModel().createNewEntryRow("tpv_puededliy");
            this.getModel().setValue("tpv_contractstatusx", dtPurchase.get("tpv_contractstatusx"), row);//合同状态
            this.getModel().setValue("tpv_cargostatus", tpvCargostatus, row);//货物状态
            this.getModel().setValue("tpv_contractnumber", dtPurchase.get("tpv_contractnumber"), row);//采购合同
            this.getModel().setValue("tpv_dateofcontracting", dtPurchase.get("tpv_dateofcontracting"), row);//签约日
            this.getModel().setValue("tpv_document", dtPurchase.get("tpv_document.name"), row);//单据类型
            this.getModel().setValue("tpv_tradeproperty", dtPurchase.get("tpv_tradeproperty"), row);//贸易性质
            this.getModel().setValue("tpv_currency", dtPurchase.get("tpv_currency"), row);//结算币别
            this.getModel().setValue("tpv_supplier", dtPurchase.get("tpv_supplier"), row);//供应商
            this.getModel().setValue("tpv_department", dtPurchase.get("tpv_department"), row);//部门
            this.getModel().setValue("tpv_documentarys", dtPurchase.get("tpv_documentarys"), row);//跟单内勤
            this.getModel().setValue("tpv_logisticsfieldserv", dtPurchase.get("tpv_logisticsfieldserv"), row);//物流外勤
            this.getModel().setValue("tpv_material", tpvMaterial, row);//商品
            this.getModel().setValue("tpv_qty", tpvQty, row);//签约数量
            this.getModel().setValue("tpv_contractprice", tpvAmount, row);//签约均价
            this.getModel().setValue("tpv_amount", tpvQty.compareTo(BigDecimal.ZERO) == 0 ? 0 : tpvAmount.divide(tpvQty, 3, BigDecimal.ROUND_HALF_UP), row);//签约金额
            this.getModel().setValue("tpv_ispayamt", dtPurchase.get("tpv_ispayamt"), row);//累计付款金额
            this.getModel().setValue("tpv_combinationnumber", dtPurchase.get("tpv_combinationnumber"), row);//计奖卡
            this.getModel().setValue("tpv_businesstype", dtPurchase.get("tpv_businesstype"), row);//业务类型
            this.getModel().setValue("tpv_paymentcontract", dtPurchase.get("tpv_paymentcontract"), row);//是否预付款合同
            this.getModel().setValue("tpv_deliverydatetype", dtPurchase.get("tpv_deliverydatetype"), row);//交货期类型
            this.getModel().setValue("tpv_daysx", dtPurchase.get("tpv_daysx"), row);//X天

            Map<String, BigDecimal> map = freigroambill(dtPurchase.getLong("id"), "", tpvDtFreigroambills, imPurinbills);
            BigDecimal freigroambill = map.get("tpvDeliverysum");
            DynamicObjectCollection tpvDtPurchaseJhqb = dtPurchase.getDynamicObjectCollection("tpv_dt_purchase_jhqb");
            if (tpvDtPurchaseJhqb.size() == 0) {
                this.getModel().setValue("tpv_daten", null, row);//约定交货期限(当前批次)
            } else if (tpvDtPurchaseJhqb.size() == 1) {
                this.getModel().setValue("tpv_daten", tpvDtPurchaseJhqb.get(0).get("tpv_agreeddeliverydead"), row);//约定交货期限(当前批次)
            } else {
                for (int i = 0; i < tpvDtPurchaseJhqb.size(); i++) {
                    BigDecimal tpvAgreeddeliveryqty = tpvDtPurchaseJhqb.get(i).getBigDecimal("tpv_agreeddeliveryqty");
                    if (tpvAgreeddeliveryqty.compareTo(freigroambill) > 0) {
                        this.getModel().setValue("tpv_daten", tpvDtPurchaseJhqb.get(i).get("tpv_agreeddeliverydead"), row);//约定交货期限(当前批次)
                        break;
                    }
                }
            }

            this.getModel().setValue("tpv_deliverysum", freigroambill, row);//已交货数量
            this.getModel().setValue("tpv_ndeliverysum", "C".equals(tpvCargostatus) ? 0 : tpvQty.subtract(freigroambill), row);//未交货数量
            this.getModel().setValue("tpv_deliveryrate", "C".equals(tpvCargostatus) ? 100 : tpvQty.compareTo(BigDecimal.ZERO) == 0 ? 0 : freigroambill.divide(tpvQty, 3, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)), row);//已交货比例（%）
            this.getModel().setValue("tpv_rknum", map.get("tpvRkTotal"), row);//累计入库数量
            this.getModel().setValue("tpv_remark", dtPurchase.get("tpv_remarks"), row);//备注
            this.getModel().setValue("tpv_comeid", dtPurchase.getLong("id"), row);//单据id
        }
        this.getModel().deleteEntryData("tpv_deliverymx");
        this.getModel().deleteEntryData("tpv_purchase_jhqb");
    }

    private DynamicObject[] purinbills(List<Long> list) {
        QFilter filter = new QFilter("billstatus", "=", "C");
        filter.and("tpv_delivery", "=", "A");//1947214905698430976
        filter.and("tpv_contract", "in", list);
        DynamicObject[] imPurinbills = BusinessDataServiceHelper.load("im_purinbill",
                "billentry,billno,tpv_delivery,biztime,tpv_corebillid,qty,tpv_packnum,tpv_contract,tpv_remarkone", filter.toArray());

        return imPurinbills;
    }

    private DynamicObject[] freigroamBills(List<Long> list) {
        QFilter qFilters = new QFilter("billstatus", "=", "C");
        qFilters.and("tpv_delivery", "=", "A");
        qFilters.and("tpv_corebillid", "in", list);
        DynamicObject[] tpvDtFreigroambills = BusinessDataServiceHelper.load("tpv_dt_freigroambill",
                "billno,tpv_delivery,tpv_corebillid,tpv_bizdate,tpv_materialentry,tpv_qty,tpv_packnum,tpv_roamaction,tpv_carshipname,tpv_comment",
                qFilters.toArray());//货物流转单
        return tpvDtFreigroambills;
    }

    public String builderAppend(String builder, String name) {
        if (builder.length() == 0) {
            builder = name;
        } else {
            if (!builder.contains(name)) {
                builder = builder + "," + name;
            }
        }
        return builder;
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        switch (name) {
            case "tpv_contract":
                puededliy();
                break;
        }
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        Control source = (Control) evt.getSource();
        if (StringUtils.equals("tpv_query", source.getKey())) { //查询
            this.getModel().deleteEntryData("tpv_puededliy");
            puededliy();
        }
    }

    @Override
    public void entryRowClick(RowClickEvent evt) {
        String source = ((Control) evt.getSource()).getKey();
        EntryGrid entryGridHt = this.getControl("tpv_puededliy");//采购合同交货情况
        int[] selectRowsHt = entryGridHt.getSelectRows();//获取选中的行号
        if (selectRowsHt.length != 0) {
            DynamicObject dataEntity = this.getModel().getDataEntity(true);
            DynamicObjectCollection tpvPuededliy = dataEntity.getDynamicObjectCollection("tpv_puededliy");
            long tpvComeid = tpvPuededliy.get(selectRowsHt[0]).getLong("tpv_comeid");
            BigDecimal tpvDeliverysum = tpvPuededliy.get(selectRowsHt[0]).getBigDecimal("tpv_deliverysum");
            List<Long> list = new ArrayList<>();
            list.add(tpvComeid);
            DynamicObject[] imPurinbills = purinbills(list);//采购入库单
            DynamicObject[] tpvDtFreigroambills = freigroamBills(list);//货物流转单
            freigroambill(tpvComeid, "key", tpvDtFreigroambills, imPurinbills);
            purchaseJhqb(tpvComeid, tpvDeliverysum);
        } else {
            this.getModel().deleteEntryData("tpv_deliverymx");
            this.getModel().deleteEntryData("tpv_purchase_jhqb");
        }
    }

    public Map<String, BigDecimal> freigroambill(long tpvComeid, String key, DynamicObject[] tpvDtFreigroambills, DynamicObject[] imPurinbills) {
        this.getModel().deleteEntryData("tpv_deliverymx");
        //采购合同-》货物流转单 核心单据编码 核心单据ID 采购合同号(?);
        BigDecimal tpvDeliverysum = BigDecimal.ZERO;
        BigDecimal tpvRkTotal = BigDecimal.ZERO;
        for (DynamicObject tpvDtFreigroambill : tpvDtFreigroambills) {
            if (tpvComeid == tpvDtFreigroambill.getLong("tpv_corebillid")) {
                DynamicObjectCollection tpvMaterialentry = tpvDtFreigroambill.getDynamicObjectCollection("tpv_materialentry");
                BigDecimal tpvTotalsum = BigDecimal.ZERO;//数量合计
                BigDecimal tpvPagtotalsum = BigDecimal.ZERO;//包装数量合计
                for (DynamicObject dynamicObject : tpvMaterialentry) {
                    tpvTotalsum = tpvTotalsum.add(dynamicObject.getBigDecimal("tpv_qty"));
                    tpvDeliverysum = tpvDeliverysum.add(dynamicObject.getBigDecimal("tpv_qty"));
                    tpvPagtotalsum = tpvPagtotalsum.add(dynamicObject.getBigDecimal("tpv_packnum"));
                }
                if ("key".equals(key)) {
                    int row = this.getModel().createNewEntryRow("tpv_deliverymx");
                    this.getModel().setValue("tpv_srcbillkey", "货物流转单", row);//来源单据
                    this.getModel().setValue("tpv_srcbillno", tpvDtFreigroambill.get("billno"), row);//来源单据好
                    this.getModel().setValue("tpv_deliverymarkw", tpvDtFreigroambill.get("tpv_delivery"), row);//交货标识
                    this.getModel().setValue("tpv_bizdate", tpvDtFreigroambill.get("tpv_bizdate"), row);//业务日期
                    this.getModel().setValue("tpv_totalsum", tpvTotalsum, row);//数量合计
                    this.getModel().setValue("tpv_pagtotalsum", tpvPagtotalsum, row);//包装数量合计
                    this.getModel().setValue("tpv_roamaction", tpvDtFreigroambill.get("tpv_roamaction"), row);//流转动作
                    this.getModel().setValue("tpv_carshipname", tpvDtFreigroambill.get("tpv_carshipname"), row);//车船名
                    this.getModel().setValue("tpv_remarkmx", tpvDtFreigroambill.get("tpv_comment"), row);//备注
                }
            }
        }
        for (DynamicObject imPurinbill : imPurinbills) {
            Long tpvContract = imPurinbill.getLong("tpv_contract");
            if (tpvContract != null) {
                if (tpvContract == tpvComeid) {
                    DynamicObjectCollection tpvMaterialentry = imPurinbill.getDynamicObjectCollection("billentry");
                    BigDecimal tpvTotalsum = BigDecimal.ZERO;//数量合计
                    BigDecimal tpvPagtotalsum = BigDecimal.ZERO;//包装数量合计
                    for (DynamicObject dynamicObject : tpvMaterialentry) {
                        tpvTotalsum = tpvTotalsum.add(dynamicObject.getBigDecimal("qty"));
                        tpvDeliverysum = tpvDeliverysum.add(dynamicObject.getBigDecimal("qty"));
                        tpvRkTotal = tpvRkTotal.add(dynamicObject.getBigDecimal("qty"));
                        tpvPagtotalsum = tpvPagtotalsum.add(dynamicObject.getBigDecimal("tpv_packnum"));
                    }
                    if ("key".equals(key)) {
                        int row = this.getModel().createNewEntryRow("tpv_deliverymx");
                        this.getModel().setValue("tpv_srcbillkey", "采购入库单", row);//来源单据
                        this.getModel().setValue("tpv_srcbillno", imPurinbill.get("billno"), row);//来源单据好
                        this.getModel().setValue("tpv_deliverymarkw", imPurinbill.get("tpv_delivery"), row);//交货标识
                        this.getModel().setValue("tpv_bizdate", imPurinbill.get("biztime"), row);//业务日期
                        this.getModel().setValue("tpv_totalsum", tpvTotalsum, row);//数量合计
                        this.getModel().setValue("tpv_pagtotalsum", tpvPagtotalsum, row);//包装数量合计
//            this.getModel().setValue("tpv_roamaction",imPurinbill.get("tpv_roamaction"),row);//流转动作
//            this.getModel().setValue("tpv_carshipname",imPurinbill.get("tpv_carshipname"),row);//车船名
                        this.getModel().setValue("tpv_remarkmx", imPurinbill.get("tpv_remarkone"), row);//备注
                    }
                }
            }
        }
        Map<String, BigDecimal> map = new HashMap<>();
        map.put("tpvDeliverysum", tpvDeliverysum);
        map.put("tpvRkTotal", tpvRkTotal);
        return map;
    }

    public void purchaseJhqb(long tpvComeid, BigDecimal tpvDeliverysum) {
        this.getModel().deleteEntryData("tpv_purchase_jhqb");
        DynamicObject tpvDtPurchase = BusinessDataServiceHelper.loadSingle(tpvComeid, "tpv_dt_purchase");
        DynamicObjectCollection tpvDtPurchaseJhqbs = tpvDtPurchase.getDynamicObjectCollection("tpv_dt_purchase_jhqb");
        for (DynamicObject tpvDtPurchaseJhqb : tpvDtPurchaseJhqbs) {
            int row = this.getModel().createNewEntryRow("tpv_purchase_jhqb");
            BigDecimal tpvAgreeddeliveryqty = (BigDecimal) tpvDtPurchaseJhqb.get("tpv_agreeddeliveryqty");
            this.getModel().setValue("tpv_agreeddeliverydead", tpvDtPurchaseJhqb.get("tpv_agreeddeliverydead"), row);
            this.getModel().setValue("tpv_agreeddeliveryqty", tpvAgreeddeliveryqty, row);//约定交货数量
            if (tpvDeliverysum.compareTo(BigDecimal.ZERO) != 0) {
                if (tpvAgreeddeliveryqty.compareTo(tpvDeliverysum) > 0) {
                    this.getModel().setValue("tpv_remaining", tpvDeliverysum, row);//冲销数量
                    this.getModel().setValue("tpv_charge", tpvAgreeddeliveryqty.subtract(tpvDeliverysum), row);//剩余数量
                    tpvDeliverysum = BigDecimal.ZERO;
                } else {
                    this.getModel().setValue("tpv_remaining", tpvAgreeddeliveryqty, row);//冲销数量
                    this.getModel().setValue("tpv_charge", 0, row);//剩余数量
                    tpvDeliverysum = tpvDeliverysum.subtract(tpvAgreeddeliveryqty);
                }
            } else {
                this.getModel().setValue("tpv_remaining", 0, row);//冲销数量
                this.getModel().setValue("tpv_charge", tpvAgreeddeliveryqty, row);//剩余数量
            }
            this.getModel().setValue("tpv_paymentorder", tpvDtPurchaseJhqb.get("tpv_paymentorder"), row);
            this.getModel().setValue("tpv_paymentamount", tpvDtPurchaseJhqb.get("tpv_paymentamount"), row);
            this.getModel().setValue("tpv_paydate", tpvDtPurchaseJhqb.get("tpv_paydate"), row);
            this.getModel().setValue("tpv_deliveryremark", tpvDtPurchaseJhqb.get("tpv_deliveryremark"), row);
        }
    }
}
