package tpv.tpv_wm.tpv_dt.java.formplugin.dynamic;

import kd.bos.form.control.Control;
import kd.bos.form.control.events.BeforeClickEvent;
import kd.bos.form.plugin.AbstractFormPlugin;
import tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClient;

import java.util.EventObject;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhkc
 * @Package: com.itopview.wm.dt.formplugin.list.QueryShipListPlugin
 * @Date: 16:53 2023/09/11
 * @Description: 根据动态表单中输入 的信息进行船舶搜索
 * @Version: 7.0
 */
public class QueryShipByCodeEdit extends AbstractFormPlugin {
    private String method;

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners("tbmain");
        this.addClickListeners("btncancel", "btnok");
    }

    @Override
    public void beforeClick(BeforeClickEvent evt) {
        super.beforeClick(evt);
        Map<String, Object> map = this.getView().getFormShowParameter().getCustomParams();
        String key = (String) map.get("key"); // 获取列表电机的按钮标识
        method = "tpv_searchship".equals(key) ? "kw" : "id"; // 如果是船舶查询按钮 参数为 kw ，否则为 id
        this.getSearchCondition(method);
        Control source = (Control) evt.getSource();
        switch (source.getKey()) {
            case "btnok": // 如果点击的是确定按钮
                this.getSearchCondition(method);
                break;
            case "tpv_cancel":
                // 被点击控件为取消则设置返回值为空并关闭页面（在页面关闭回调方法中必须验证返回值不为空，否则会报空指针）
                this.getView().returnDataToParent(null);
                break;
            default:
                break;
        }
    }

    /**
     * @author: Heisenberg
     * @date: 2023/10/11 15:22
     * @description: 获取要搜索的船舶id
     */
    private void getSearchCondition(String method) {
        String errmsg = "";
        String shipindex = (String) this.getModel().getValue("tpv_shipindex");  // 从表单获取要查询的船舶id
        if ("".equals(shipindex)) {
            errmsg = "请输入查询信息！";
            this.getView().returnDataToParent(errmsg);
            return;
        }

        String[] ids = shipindex.split(",");
        List<String> list = Stream.of(ids).collect(Collectors.toList());  // 用流的方式数组转集合
        ShipNewsClient.queryShipInfo(list, method)
                .filter(u -> !u.isEmpty())
                .map(u -> {
                    this.getView().returnDataToParent(u);
                    return u;
                })
                .orElseGet(() -> {
                    this.getView().returnDataToParent("查询数据为空");
                    return null;
                });
    }
}
