package tpv.tpv_wm.tpv_dt.java.formplugin.dynamic;

import kd.bos.form.control.Control;
import kd.bos.form.control.events.BeforeClickEvent;
import kd.bos.form.plugin.AbstractFormPlugin;
import tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClient;

import java.util.*;


/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhkc
 * @Package: com.itopview.wm.dt.formplugin.list.QueryShipTrackListPlugin
 * @Date: 10:27 2023/09/13
 * @Description: 船舶轨迹拆查询动态表单
 * @Version: 7.0
 */
public class QueryShipTrackEdit extends AbstractFormPlugin {
    private static final String SHIPID = "tpv_shipid";

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners("tbmain");
        this.addClickListeners("btncancel", "btnok");
    }

    @Override
    public void beforeClick(BeforeClickEvent evt) {
        super.click(evt);
        Control source = (Control) evt.getSource();
        switch (source.getKey()) {
            case "btnok":
                this.getTimeline();
                break;
            case "tpv_cancel":
                // 被点击控件为取消则设置返回值为空并关闭页面（在页面关闭回调方法中必须验证返回值不为空，否则会报空指针）
                this.getView().returnDataToParent(null);
                break;
            default:
                break;
        }
    }

    /**
     * @author: Heisenberg
     * @date: 2023/10/11 14:43
     * @description: 获取输入的起始时间和结束时间，结束时间为空则默认为起始时间俩天后，将其转为时间戳（单位s）
     */
    private void getTimeline() {
        Date trackBegin = (Date) this.getModel().getValue("tpv_trackbegin"); // 获取长日期船舶起始日期
        Date trackEnd = (Date) this.getModel().getValue("tpv_trackend"); // 获取长日期船舶结束日期

        if (trackBegin == null) {
            this.getView().returnDataToParent("请填写起始日期");
            return;
        }

        List<String> ids = Collections.singletonList((String) this.getModel().getValue(SHIPID)); // 获取被选中单据的船舶ID

        if (trackEnd == null) {
            Calendar cal = Calendar.getInstance();
            cal.setTime(trackBegin);
            cal.add(Calendar.DAY_OF_YEAR, 2);
            trackEnd = cal.getTime(); // 结束时间为空给默认值，起始时间加俩天
        }
        ShipNewsClient.queryShipInfo(ids,
                        String.valueOf(trackBegin.getTime() / 1000L), String.valueOf(trackEnd.getTime() / 1000L))
                .filter(u -> !u.isEmpty())
                .map(u -> {
                    this.getView().returnDataToParent(u);
                    return u;
                })
                .orElseGet(() -> {
                    this.getView().returnDataToParent("船舶轨迹信息为空");
                    return null;
                });
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        Map<String, Object> map = this.getView().getFormShowParameter().getCustomParams();
        this.getModel().setValue("tpv_shipname", map.get("tpv_shipname"));
        this.getModel().setValue(SHIPID, map.get(SHIPID));
    }
}
