package tpv.tpv_wm.tpv_dt.java.formplugin.dynamic;

import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.DateUtil;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.IFormView;
import kd.bos.form.control.Control;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * ClassName: MatchWarehouseFormPlugin
 * Description: 匹配入库信息
 * date: 2023/10/31 20:36
 *
 * @author: Guo jiaChen
 */
public class WarehouseMatchEdit extends AbstractFormPlugin {
    private static final String warehouseEntryDetail = "tpv_wareiformation";


    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners(new String[]{"btnok", "tpv_query", "tpv_deleteentry"});
        this.addItemClickListeners(new String[]{"tpv_advcontoolbarap"});
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        IFormView view = this.getView();
        IDataModel parentModel = view.getParentView().getModel();
        Object tpvDepartment = parentModel.getValue("tpv_department");
        this.getModel().setValue("tpv_org",tpvDepartment);
        Map<String, Object> params = view.getFormShowParameter().getCustomParams();
        JSONObject supplier = (JSONObject) params.get("supplier");
        if (null != supplier) {
            this.getModel().setValue("tpv_invoicedept", supplier.get("id"));
        }
        initData(true);
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        ChangeData[] changeSet = e.getChangeSet();
        int index = changeSet[0].getRowIndex();
        switch (name) {
            case "tpv_rktaxrate1":
            case "tpv_rkcurmatchamo1":
                calculate(index);
                break;
            case "tpv_rkcurmatchqty1":
                showMessage(index);
                break;
            default:
                break;
        }
    }

    /**
     * @param : index
     * @return void
     * @author: chen
     * @date: 2023/12/6 9:33
     * @description: 本次匹配数量大于未匹配数量时提示
     */
    private void showMessage(int index) {
        IDataModel model = this.getModel();
        BigDecimal qty1 = (BigDecimal) model.getValue("tpv_qty1", index);
        BigDecimal rkMatchQty1 = (BigDecimal) model.getValue("tpv_rkmatchqty1", index);
        BigDecimal rkCurMatchQty1 = (BigDecimal) model.getValue("tpv_rkcurmatchqty1", index);
        if (qty1.compareTo(rkMatchQty1.add(rkCurMatchQty1)) < 0) {
            this.getView().showTipNotification(String.format("入库信息分录第%s行,本次匹配数量不能大于未匹配数量!", index + 1));
            this.getModel().beginInit();
            this.getModel().setValue("tpv_rkcurmatchqty1", BigDecimal.ZERO);
            this.getModel().endInit();
            this.getView().updateView("tpv_rkcurmatchqty1", index);
        }

    }

    /**
     * @param index:
     * @return void
     * @author: chen
     * @date: 2023/11/19 16:56
     * @description: 计算本次匹配金额(增值税额) + (不含税)
     */
    private void calculate(int index) {
        BigDecimal taxRate = (BigDecimal) this.getModel().getValue("tpv_rktaxrate1");
        BigDecimal curmatchamo = (BigDecimal) this.getModel().getValue("tpv_rkcurmatchamo1");
        BigDecimal divisor = new BigDecimal(100);
        BigDecimal decimal = taxRate.divide(divisor);
        if (taxRate != null && curmatchamo != null) {
            BigDecimal curmatchamointax = curmatchamo.divide(decimal.add(new BigDecimal(1)), 10, RoundingMode.HALF_UP).multiply(decimal);
            this.getModel().setValue("tpv_rkcurmatchamointax1", curmatchamointax, index);
            this.getModel().setValue("tpv_rkcurmatchamonotax1", curmatchamo.subtract(curmatchamointax), index);
        }
    }

    @Override
    public void click(EventObject evt) {
        super.click(evt);
        String key = ((Control) evt.getSource()).getKey();
        switch (key) {
            case "btnok":
                pickOk();
                break;
            case "tpv_query":
                initData(false);
                break;
            default:
                break;
        }
    }

    /**
     * @param :
     * @return void
     * @author: chen
     * @date: 2023/11/19 16:56
     * @description: 校验 + 回填
     */
    private void pickOk() {
        IFormView view = this.getView();
        IFormView parentView = this.getView().getParentView();

        EntryGrid control = this.getControl(warehouseEntryDetail);
        int[] selectRows = control.getSelectRows();
        Boolean flag = false;
        DynamicObjectCollection collections = this.getModel().getDataEntity(true).getDynamicObjectCollection(warehouseEntryDetail);
        DynamicObjectCollection dynamicObjectCollection = new DynamicObjectCollection();
        //复选框选中
        for (DynamicObject object : collections) {
            if (object.getBoolean("tpv_checkbox")) {
                BigDecimal qty = object.getBigDecimal("tpv_rkcurmatchqty1");
                if (null == qty || qty.compareTo(new BigDecimal("0")) == 0) {
                    view.showErrorNotification("本次匹配数量为空,请检查!");
                    return;
                }
                dynamicObjectCollection.add(object);
                flag = true;
            }
        }
        if (!flag) {
            view.showErrorNotification("未勾选入库信息！");
            return;
        }
        //选中行选中
//        for (int selectRow : selectRows) {
//            DynamicObject object = collections.get(selectRow);
//            BigDecimal qty = object.getBigDecimal("tpv_rkcurmatchqty1");
//            if (null == qty || qty.compareTo(new BigDecimal("0")) == 0) {
//                view.showErrorNotification("本次匹配数量为空,请检查!");
//                return;
//            }
//            dynamicObjectCollection.add(object);
//        }
        HashMap<String, DynamicObjectCollection> collectionHashMap = new HashMap<>();
        collectionHashMap.put("cols", dynamicObjectCollection);
        view.returnDataToParent(collectionHashMap);

        view.sendFormAction(parentView);
        view.close();
    }


    /**
     * @param b:
     * @return void
     * @author: chen
     * @date: 2023/11/19 16:56
     * @description: 初始化数据
     */
    private void initData(boolean b) {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        IDataModel parentModel = view.getParentView().getModel();
        ArrayList<String> entryIDs = new ArrayList<>();
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
        DynamicObject supplier = (DynamicObject) model.getValue("tpv_invoicedept");
        DynamicObject org = (DynamicObject) model.getValue("tpv_org");
        String contractnum = model.getValue("tpv_purcontractnum").toString();//采购合同号
        String purinbillno = (String) model.getValue("tpv_purinbillno");//入库单号
        Date startDate = (Date) model.getValue("tpv_invocestartdate");//起始日期
        Date endDate = (Date) model.getValue("tpv_invoceenddate");//截至日期
        DynamicObject tpvCompany = (DynamicObject) parentModel.getValue("tpv_company");

        if (tpvCompany!=null){
            filters.add(new QFilter("org", QCP.equals, tpvCompany.getPkValue()));//公司
        }
        if (null != supplier) {
            long supID = supplier.getLong("id");//供应商id
            filters.add(new QFilter("supplier", QCP.equals, supID));//供应商
        }
        if (null != purinbillno && purinbillno.length() > 0) {
            filters.add(new QFilter("billno", QCP.like, "%" + purinbillno + "%"));//入库单号
        }
        if (null != org) {
            long supID = org.getLong("id");//部门id
            filters.add(new QFilter("bizdept", QCP.equals, supID));//部门
        }
        if (null != contractnum && contractnum.length() > 0) {
            filters.add(new QFilter("billentry.tpv_contractnumber.tpv_name", QCP.ILIKE, "%" + contractnum + "%"));//采购合同号
        }
        if (null != startDate) {
            filters.add(new QFilter("biztime", QCP.large_equals, DateUtil.getStartOfDay(startDate)));//开票日期从
        }
        if (null != endDate) {
            filters.add(new QFilter("biztime", QCP.less_equals, DateUtil.getEndOfDay(endDate)));//开票日期至
        }
        DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "im_purinbill",
                "id,billno,billstatus,bizdept,biztype.number,billentry,billentry.tpv_fentryid_in,billentry.tpv_contractnumber.tpv_name" +
                        ",billentry.material.masterid,billentry.tpv_prodname,supplier" +//tpv_material
                        ",billentry.tpv_shader,billentry.tpv_origin,billentry.qty,billentry.tpv_specation,billentry.amountandtax,currency,exratetable,exchangerate,billentry.unit"
                , filters.toArray(new QFilter[0]));
        DataSet selected = queryDataSet.select("id", "billno", "billentry.tpv_contractnumber.tpv_name contractnumber", "supplier", "bizdept department"
                , "billentry.material.masterid material", "billentry entryID", "billentry.tpv_prodname prodname"
                , "billentry.tpv_shader shader", "billentry.tpv_origin origin", "billentry.qty qty", "billentry.tpv_specation specation"
                , "billentry.amountandtax taxpramount", "currency", "exratetable", "exchangerate","billentry.unit tpv_rkunit1"
                , "(case when biztype.number != '1101' then billentry else billentry.tpv_fentryid_in end) fentryid");
        //分录填充数据
        if (selected.isEmpty()) {
            this.getModel().deleteEntryData(warehouseEntryDetail);
            return;
        }
        this.getModel().deleteEntryData(warehouseEntryDetail);
        DataSet copy = selected.copy();
        for (Row row : selected) {
            entryIDs.add(row.getString("entryID"));
        }
        setInvoiceData(copy, entryIDs);
    }

    /**
     * @param queryDataSet:
     * @return void
     * @author: chen
     * @date: 2023/11/19 16:57
     * @description: 分录填充数据
     */
    private void setInvoiceData(DataSet queryDataSet, ArrayList<String> entryIDS ) {
        IDataModel model = this.getModel();
        HashMap<String, BigDecimal> matchedQty = getMatchedQty(entryIDS);//获取已匹配数量等
        boolean isVisible = (boolean) model.getValue("tpv_isvisible");//未匹配数量≤0不显示
        for (Row row : queryDataSet) {
            String entryID = row.getString("entryID");//入库分录ID
            BigDecimal matchedNum = null == matchedQty.get(entryID) ? BigDecimal.ZERO : matchedQty.get(entryID);//已匹配数量
            BigDecimal num =  null == row.getBigDecimal("qty") ? BigDecimal.ZERO : row.getBigDecimal("qty");//数量
            //未匹配数量≤0不显示
            if (isVisible && (num.subtract(matchedNum)).compareTo(BigDecimal.ZERO) <= 0) {
                continue;
            }
            int entryRow = this.getModel().createNewEntryRow(warehouseEntryDetail);
            model.setValue("tpv_warehousedept1", row.getLong("department"), entryRow);//部门
            model.setValue("tpv_connum1", row.getString("contractnumber"), entryRow);//采购合同号
            model.setValue("tpv_warenum1", row.getString("billno"), entryRow);//入库单号
            model.setValue("tpv_warehousupplier1", row.getLong("supplier"), entryRow);//供应商
            model.setValue("tpv_warehousmaterial1", row.getLong("material"), entryRow);//物料编码
            model.setValue("tpv_prodname1", row.getString("prodname"), entryRow);//品名
            model.setValue("tpv_shader1", row.getString("shader"), entryRow);//材质
            model.setValue("tpv_origin1", row.getLong("origin"), entryRow);//产地
            model.setValue("tpv_specation1", row.getString("specation"), entryRow);//规格
            model.setValue("tpv_qty1",num, entryRow);//数量
            model.setValue("tpv_warehouseamt1", row.getBigDecimal("taxpramount"), entryRow);//金额
            model.setValue("tpv_rkmatchqty1",matchedNum, entryRow);//已匹配数量
            model.setValue("tpv_rktaxrate1", new BigDecimal("13"), entryRow);//税率%
            model.setValue("tpv_currency1", row.getLong("currency"), entryRow);//币别
            model.setValue("tpv_ratetable1", row.getLong("exratetable"), entryRow);//汇率表
            model.setValue("tpv_rate1", row.getLong("exchangerate"), entryRow);//汇率
            model.setValue("tpv_rkunit1", row.get("tpv_rkunit1"), entryRow);//级联单位
            model.setValue("tpv_purinentryid1", entryID, entryRow);//入库分录id
            model.setValue("tpv_fentryid_in1", row.getString("fentryid"), entryRow);//成本入库分录id
        }
    }


    /**
     * @param entryIDS:
     * @return HashMap<String, BigDecimal>
     * @author: chen
     * @date: 2023/11/19 16:57
     * @description: 获取已匹配数量
     */
    private HashMap<String, BigDecimal> getMatchedQty(ArrayList<String> entryIDS) {
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("tpv_wareiformation.tpv_purinentryid", QCP.in, entryIDS));//单据编号
        HashMap<String, BigDecimal> map = new HashMap<>();
        DataSet load = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_purinvomater",
                "id,tpv_wareiformation,tpv_wareiformation.tpv_purinentryid,tpv_wareiformation.tpv_rkcurmatchqty"
                , filters.toArray(new QFilter[0]));
        DataSet executeSql = load.executeSql("select tpv_wareiformation.tpv_purinentryid ,sum(tpv_wareiformation.tpv_rkcurmatchqty) rkcurmatchqty GROUP BY tpv_wareiformation.tpv_purinentryid");

        DataSet loads = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_wbillcromater",
                "id,tpv_dt_incomingenty,tpv_dt_incomingenty.tpv_bcmonnumrk,tpv_dt_incomingenty.tpv_rkcomid"
                , null);
        DataSet executeSqls = loads.executeSql("select tpv_dt_incomingenty.tpv_rkcomid ,sum(tpv_dt_incomingenty.tpv_bcmonnumrk) rkcurmatchqty GROUP BY tpv_dt_incomingenty.tpv_rkcomid");
        executeSql.print(true);
        executeSqls.print(true);
        for (Row row : executeSql) {
            BigDecimal decimal = row.getBigDecimal("rkcurmatchqty");
            String purinentryid = row.getString("tpv_wareiformation.tpv_purinentryid");
            map.put(purinentryid, decimal);
        }
        for (Row row : executeSqls) {
            BigDecimal decimal = row.getBigDecimal("rkcurmatchqty");
            String purinentryid = row.getString("tpv_dt_incomingenty.tpv_rkcomid");
            map.put(purinentryid, null == map.get(purinentryid) ? decimal : map.get(purinentryid).add(decimal));
        }
        return map;
    }

    public Long toLong(Object longObj) {
        if (longObj == null) {
            return null;
        } else if (longObj instanceof Long) {
            return (Long) longObj;
        } else {
            return Long.valueOf(String.valueOf(longObj));
        }
    }
}
