package tpv.tpv_wm.tpv_dt.java.formplugin.form;

import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.form.control.events.BeforeItemClickEvent;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClient;
import tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClientHelper;

import java.util.*;

import static tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClientHelper.*;


/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhkc
 * @Package: com.itopview.wm.dt.formplugin.form.AttentionFromPerOrDe
 * @Date: 14:24 2023/09/22
 * @Description: 关注船清单 个人关注 部门关注 个+部关注 按钮功能实现
 * @Version: 1.0
 */
@SuppressWarnings("all")
public class AttentionButtonFormPlugin extends AbstractBillPlugIn {
    private String shipid;
    private static boolean flag = false;
    private List<ShipNewsClientHelper.ShipDetail> shiplist;
    private static final String TPV_PEOPLE = "tpv_people"; // 人员
    private static final String TPV_OPERATOR = "tpv_operator"; // 人员
    private static final String TPV_DEPARTMENT = "tpv_department"; // 部门
    private static final int PERSON = 1; // 个人关注按钮
    private static final int DEPARTMENT = 2; // 部门关注按钮
    private static final int ALLATTENTION = 3; // 个人+部门关注按钮
    private static final String TPV_RADIOGROUP = "tpv_radiogroup"; // 按钮组
    private static final long CREATER = YEM.getCurrentUserId(); // 当前创建人
    private static final long ORG = YEM.getCurrentOrgId(); // 当前创建人所在部门

    /**
     * @return void
     * @author: Heisenberg
     * @date: 2023/10/10 14:21
     * @description: 初始化船舶 id 和船舶信息
     */
    private void init() {
        Optional.ofNullable((String) this.getModel().getValue("tpv_shipid")) // 获取当前页面的船舶 id
                .filter(id -> YEM.isNotEmpty(id))
                .ifPresent(id -> shipid = id); // 当前页面船舶 id 赋值给 SHIPID
        SelectShipByNameFormPlugin selectShipByNameFormPlugin = new SelectShipByNameFormPlugin();
        Optional.ofNullable(selectShipByNameFormPlugin.getShiplist())
                .filter(shipDetailList -> !shipDetailList.isEmpty())
                .ifPresent(shipDetailList -> shiplist = shipDetailList);

        if (YEM.isEmpty(shipid)) {
            shipid = selectShipByNameFormPlugin.getId(); // 从搜索控件获取对应的船舶 id
        }
        if (shiplist == null) {
            ShipNewsClient.queryShipInfo(Collections.singletonList(shipid), "id")
                    .ifPresent(list -> shiplist = list);
        }
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addItemClickListeners("tbmain"); // 监听表单的工具栏
    }

    @Override
    public void beforeItemClick(BeforeItemClickEvent evt) {
        super.beforeItemClick(evt); // 按钮事件前初始化船舶 id
        init();
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        this.initDepartment(flag); // 数据更改后通过布尔值 flag 控制部门，因模板中给部门被业务员所干扰,覆盖模板的方法
    }

    /**
     * @param b: 布尔值
     * @return void
     * @author: Heisenberg
     * @date: 2023/10/10 14:29
     * @description: 根据布尔值决定是否给部门赋值，为 true 部门为当前登陆人所在部门，否则为null
     */
    private void initDepartment(boolean b) {
        this.getModel().setValue("tpv_department", b ? ORG : null);
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        switch (evt.getItemKey()) {
            case "tpv_personalized": // 个人关注
                this.personalized();
                break;
            case "tpv_cancelpersonal": // 个人取消
                this.cancelpersonal();
                break;
            case "tpv_departmental": // 部门关注
                this.departmental();
                break;
            case "tpv_canceldepart": // 部门取消
                flag = false;
                this.canceldepart();
                break;
            case "tpv_allattention": // 个+部关注
                this.allattention();
                break;
            default:
                break;
        }
    }

    /**
     * @date: 2023/9/22 17:30
     * @description: 个人关注 存在该条船的数据并且【人员】为“空”进行显示为【个人关注】
     * 反之显示为【个人取消】 将【人员】赋值创建人
     * 没数据则点击后给《关注船清单》添加一条数据，将【人员】赋值创建人
     */
    private void personalized() {
        if (isExistShipId(shipid)) { // 单据列表是否已有该船舶信息
            Optional.ofNullable(this.getModel().getValue(TPV_PEOPLE))
                    .filter(person -> person != null) // 判断人员是否为空
                    .map(person -> {
                        this.getModel().setValue(TPV_RADIOGROUP, null); // 不为空，变为个人取消
                        return person;
                    })
                    .orElseGet(() -> {
                        this.getModel().setValue(TPV_RADIOGROUP, PERSON); // 为空 按钮组选中个人关注
                        return null;
                    });
            this.getModel().setValue(TPV_PEOPLE, CREATER); // 将创建人赋值给人员
        } else {
            DynamicObject[] shipFormList = processShipFormList((List<ShipNewsClientHelper.ShipDetail>) shiplist); // 生成新单据
            Arrays.stream(shipFormList).findFirst().ifPresent(shipForm -> {
                shipForm.set(TPV_PEOPLE, CREATER); // 将创建人赋值给人员
                SaveServiceHelper.save(new DynamicObject[]{shipForm}); // 保存到数据库
                this.getView().showForm(showBillView(shipForm.getLong("id"))); // 打开新生成的单据视图
            });
            this.getView().getParentView().updateView(); // 父视图（列表视图）更新
        }
    }

    /**
     * @date: 2023/9/22 17:30
     * @description: 个人取消 检查【部门】是否为空，为空则删除该船在《关注船清单》中的该记录，不为空则清空【人员】的值；
     */
    private void cancelpersonal() {
        Optional.ofNullable(this.getModel().getValue(TPV_DEPARTMENT))
                .filter(department -> department != null) // 判断部门是否为空
                .map(department -> {
                    flag = true; // 给部门赋值当前业务员部门
                    this.getModel().setValue(TPV_PEOPLE, null); // 清空人员的值
                    this.getView().updateView(); // 更新当前视图
                    return department;
                })
                .orElseGet(() -> {
                    flag = false; // 清空部门字段

                    // 删除当前单据
                    deleteShipForm(new QFilter[]{new QFilter("tpv_shipid", QCP.equals, shipid)});

                    this.getView().getParentView().updateView(); // 父视图（列表视图）更新
                    this.getView().close(); // 关闭当前单据视图
                    return null;
                });
    }

    /**
     * @date: 2023/9/22 17:30
     * @description: 部门关注 根据《关注船清单》是否存在该条船的数据并且【部门】为“空”进行显示为【部门关注】，
     * 反之显示为【部门取消】，
     * 如果《关注船清单》没数据则点击后给《关注船清单》添加一条数据，并且将【部门】赋值创建人；
     * 如果《关注船清单》存在数据则将【部门】赋值创建人所在部门；
     */
    private void departmental() {
        if (isExistShipId(shipid)) {
            Optional.ofNullable(this.getModel().getValue(TPV_DEPARTMENT))
                    .filter(department -> department != null) // 判断部门是否为空
                    .map(department -> {
                        this.getModel().setValue(TPV_RADIOGROUP, null); // 按钮组显示为部门取消
                        return department;
                    })
                    .orElseGet(() -> { // 部门为空
                        this.getModel().setValue(TPV_RADIOGROUP, DEPARTMENT); // 按钮组显示为部门关注
                        return null;
                    });
            this.getModel().setValue(TPV_DEPARTMENT, ORG); // 给部门赋值
        } else {
            flag = true; // 给部门赋值

            // 新建单据
            DynamicObject[] shipFormList = processShipFormList((List<ShipDetail>) shiplist);

            if (shipFormList == null) {
                return;
            }
            Arrays.stream(shipFormList).findFirst().ifPresent(shipForm -> { // 当前对象不为null
                SaveServiceHelper.save(new DynamicObject[]{shipForm}); // 数据库保存
                this.getView().showForm(showBillView(shipForm.getLong("id"))); // 打开新建单据视图
                this.getView().updateView();
            });
        }
    }

    /**
     * @date: 2023/9/22 17:30
     * @description: 部门取消 检查【人员】是否为空，为空则删除该船在《关注船清单》中的该记录，不为空则清空【部门】的值
     */
    private void canceldepart() {
        Optional.ofNullable(this.getModel().getValue(TPV_PEOPLE))
                .filter(person -> person != null)
                .map(person -> {
                    this.getView().updateView();
                    return person;
                })
                .orElseGet(() -> {
                    deleteShipForm(new QFilter[]{new QFilter("tpv_shipid", QCP.equals, shipid)});
                    this.getView().getParentView().updateView();
                    this.getView().close();
                    return null;
                });
    }

    /**
     * @date: 2023/9/22 17:30
     * @description: 个人+部门关注 根据《关注船清单》是否存在该条船的数据并且将【部门】赋值创建人所在部门；【人员】字段赋值创建人；
     */
    private void allattention() {
        if (isExistShipId(shipid)) {
            this.getModel().setValue(TPV_PEOPLE, CREATER);
            this.getModel().setValue(TPV_RADIOGROUP, ALLATTENTION);
        } else {
            flag = true;
            DynamicObject[] shipFormList = processShipFormList((List<ShipDetail>) shiplist);
            Arrays.stream(shipFormList).findFirst().ifPresent(shipForm -> {
                shipForm.set(TPV_PEOPLE, CREATER);
                shipForm.set(TPV_RADIOGROUP, ALLATTENTION);
                SaveServiceHelper.save(new DynamicObject[]{shipForm});
                this.getView().showForm(showBillView(shipForm.getLong("id")));
            });
            this.getView().getParentView().updateView();
        }
    }

}
