package tpv.tpv_wm.tpv_dt.java.formplugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.form.ConfirmCallBackListener;
import kd.bos.form.MessageBoxOptions;
import kd.bos.form.MessageBoxResult;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.MessageBoxClosedEvent;

import java.util.EventObject;

/**
 * @Description 投标申请单 已/未投标按钮 点击赋值
 * @Author LinRui
 * Date 2023/8/16 13:27
 **/
public class BidReqFormPlugin extends AbstractBillPlugIn {


    //工具栏已投标按钮的标识
    private static final String KEY_OUTBID = "tpv_outbid";
    //工具栏未投标按钮的标识
    private static final String KEY_NOTOUTBID = "tpv_notoutbid";
    //单据中的工具栏标识
    private static final String KEY_TOOlBAR = "tbmain";

    /**
     * 注册工具栏监听
     *
     * @param e
     */
    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        //监听工具栏按钮点击事件
        this.addItemClickListeners(KEY_TOOlBAR);
    }

    /**
     * 工具栏按钮点击事件
     *
     * @param evt
     */
    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        //工具栏上的所有按钮的点击都会激活itemClick方法，需要开发人员实现不同按钮的逻辑
        if (StringUtils.equals(KEY_OUTBID, evt.getItemKey())) {
            //作废操作前让用户选择是否继续作废操作，在用户点击确认框上的按钮后，系统会调用confirmCallBack方法
            ConfirmCallBackListener confirmCallBackListener = new ConfirmCallBackListener(KEY_OUTBID, this);
            //设置页面确认框，参数为：标题，选项框类型，回调监听
            this.getView().showConfirm("是否确认已中标？", MessageBoxOptions.YesNo, confirmCallBackListener);
        }
        if (StringUtils.equals(KEY_NOTOUTBID, evt.getItemKey())) {
            //作废操作前让用户选择是否继续作废操作，在用户点击确认框上的按钮后，系统会调用confirmCallBack方法
            ConfirmCallBackListener confirmCallBackListener = new ConfirmCallBackListener(KEY_NOTOUTBID, this);
            //设置页面确认框，参数为：标题，选项框类型，回调监听
            this.getView().showConfirm("是否确认未中标？", MessageBoxOptions.YesNo, confirmCallBackListener);
        }
    }

    /**
     * 选择框回调函数
     *
     * @param messageBoxClosedEvent
     */
    @Override
    public void confirmCallBack(MessageBoxClosedEvent messageBoxClosedEvent) {
        super.confirmCallBack(messageBoxClosedEvent);
        //判断是否是对应确认框的点击回调事件
        if (StringUtils.equals(KEY_OUTBID, messageBoxClosedEvent.getCallBackId())) {
            if (MessageBoxResult.Yes.equals(messageBoxClosedEvent.getResult())) {
                //如果点击确认按钮，则把当前页面相关值清空
                this.getModel().setValue("tpv_bidding", 'A');
            } else if (MessageBoxResult.No.equals(messageBoxClosedEvent.getResult())) {
                // 点击否的相关处理逻辑。。。。
            }
        }
        if (StringUtils.equals(KEY_NOTOUTBID, messageBoxClosedEvent.getCallBackId())) {
            if (MessageBoxResult.Yes.equals(messageBoxClosedEvent.getResult())) {
                //如果点击确认按钮，则把当前页面相关值清空
                this.getModel().setValue("tpv_bidding", 'B');
            } else if (MessageBoxResult.No.equals(messageBoxClosedEvent.getResult())) {
                // 点击否的相关处理逻辑。。。。
            }
        }
    }
}