package tpv.tpv_wm.tpv_dt.java.formplugin.form;

import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.DateUtil;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.clr.DataEntityPropertyCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.IFormView;
import kd.bos.form.control.Control;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * ClassName: InvoiceMatchBillFormPlugin
 * Description: 匹配发票信息弹框
 * date: 2023/10/28 11:35
 *
 * @author: Guo jiaChen
 */
public class InvoiceMatchBillFormPlugin extends AbstractFormPlugin {

    private static final String invoiceEntry = "tpv_invoiceinformation";
    private static final String invoiceEntryDetail = "tpv_invoiceinfoentry";
    private static final String selectInvoiceEntry = "tpv_selectinvoiceinfo";

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addClickListeners(new String[]{"btnok", "tpv_query", "tpv_deleteentry"});
        this.addItemClickListeners(new String[]{"tpv_advcontoolbarap"});
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        IFormView view = this.getView();
        Map<String, Object> params = view.getFormShowParameter().getCustomParams();
        setEntries();
        JSONObject supplier = (JSONObject) params.get("supplier");
        if (null != supplier) {
            this.getModel().setValue("tpv_invoicedept", supplier.get("id"));
        }
        initData();
    }

    /**
     * @param :
     * @return void
     * @author: chen
     * @date: 2023/11/19 16:47
     * @description: 匹配发票信息弹框分录赋值
     */
    private void setEntries() {
        IFormView view = this.getView();
        IFormView parentView = this.getView().getParentView();
        IDataModel parentModel = parentView.getModel();
        DynamicObjectCollection collection = parentModel.getDataEntity(true).getDynamicObjectCollection(invoiceEntryDetail);
        DataEntityPropertyCollection properties = collection.getDynamicObjectType().getProperties();
        DynamicObjectCollection collections = this.getModel().getDataEntity(true).getDynamicObjectCollection(selectInvoiceEntry);
        DataEntityPropertyCollection selectedProperties = collections.getDynamicObjectType().getProperties();

        for (int i = 0; i < collection.size(); i++) {
            DynamicObject addNew = collections.addNew();
            DynamicObject row = collection.get(i);
            for (IDataEntityProperty property : properties) {
                String proKey = property.getName();
                if (proKey.equals("tpv_taxrate")) {
                    addNew.set(proKey, row.get(proKey));
                }
                if (selectedProperties.containsKey(proKey + 1)) {
                    addNew.set(proKey + 1, row.get(proKey));
                }
            }
            addNew.set("seq", i + 1);
        }
        this.getView().updateView(selectInvoiceEntry);
    }


    @Override
    public void click(EventObject evt) {
        super.click(evt);
        String key = ((Control) evt.getSource()).getKey();
        switch (key) {
            case "btnok":
                pickOk();
                break;
            case "tpv_query":
                initData();
                break;
            default:
                break;
        }
    }

    /**
     * @param :
     * @return void
     * @author: chen
     * @date: 2023/11/19 16:48
     * @description: 数据校验 + 回填
     */
    private void pickOk() {
        IFormView view = this.getView();
        IFormView parentView = this.getView().getParentView();
        IDataModel parentModel = parentView.getModel();

        DynamicObject dataEntity = parentModel.getDataEntity(true);
        parentModel.deleteEntryData(invoiceEntryDetail);

        DynamicObjectCollection collections = this.getModel().getDataEntity(true).getDynamicObjectCollection(selectInvoiceEntry);
        DataEntityPropertyCollection selectedProperties = collections.getDynamicObjectType().getProperties();

        HashMap<String, DynamicObjectCollection> collectionHashMap = new HashMap<>();

        collectionHashMap.put("cols", collections);
        view.returnDataToParent(collectionHashMap);
        view.sendFormAction(parentView);
        view.close();
    }

    /**
     * @param proKey:
     * @return String
     * @author: chen
     * @date: 2023/11/19 16:49
     * @description: 截取字符串
     */
    private String subKey(String proKey) {
        if (null != proKey && proKey.length() > 0) {
            return proKey.substring(0, proKey.length() - 1);
        } else {
            return "";
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = "";

        ChangeData[] changeData = e.getChangeSet();
        int rowIndex = changeData[0].getRowIndex();
        String key = e.getProperty().getName();
        switch (key) {
            case "tpv_focus":
                pickFocus(rowIndex);
                break;
            case "tpv_invocestartdate":
            case "tpv_invoceenddate":
                checkDate(key);
            default:
                break;
        }
    }

    /**
     * @param name:
     * @return void
     * @author: chen
     * @date: 2023/11/19 16:49
     * @description: 时间校验
     */
    private void checkDate(String name) {
        Date startDate = (Date) this.getModel().getValue("tpv_invocestartdate");
        Date endDate = (Date) this.getModel().getValue("tpv_invoceenddate");
        String msg = "开票起始日期必须小于截止日期,请重新选择!";
        if (null != startDate && null != endDate) {
            if (startDate.after(endDate)) {
                this.getView().showTipNotification(msg);
                this.getModel().setValue(name, null);
            }
        }
    }


    /**
     * @param rowIndex:
     * @return void
     * @author: chen
     * @date: 2023/11/19 16:50
     * @description: 选中调整发票信息
     */
    private void pickFocus(int rowIndex) {
        IDataModel model = this.getModel();
        DynamicObject row = model.getEntryRowEntity(invoiceEntry, rowIndex);
        DataEntityPropertyCollection properties = row.getDynamicObjectType().getProperties();
        DynamicObjectCollection entry2 = model.getEntryEntity(selectInvoiceEntry);
        DataEntityPropertyCollection properties2 = entry2.getDynamicObjectType().getProperties();
        int seq = row.getInt("seq");
        if (row.getBoolean("tpv_focus")) {
            List<Integer> seqs = entry2.stream().map(x -> x.getInt("tpv_seq")).collect(Collectors.toList());
            if (!seqs.contains(seq)) {
                int index = model.createNewEntryRow(selectInvoiceEntry);
                for (IDataEntityProperty property : properties) {
                    String key = property.getName();
                    if ("tpv_taxrates".equals(key)) {
                        model.setValue(subKey(key), row.get(key), index);
                    }
                    if (properties2.containsKey(key + "1")) {
                        model.setValue(key + "1", row.get(key), index);
                    }
                }
                model.setValue("tpv_seq", seq, index);
            }
        } else {
            for (int i = 0; i < entry2.size(); i++) {
                DynamicObject row2 = entry2.get(i);
                if (seq == row2.getInt("tpv_seq")) {
                    model.deleteEntryRow(selectInvoiceEntry, i);
                }
            }
        }
    }

    /**
     * @param :
     * @return void
     * @author: chen
     * @date: 2023/11/19 16:50
     * @description: 初始化数据
     */
    private void initData() {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("billstatus", QCP.equals, "C"));//已审核
        DynamicObject supplier = (DynamicObject) model.getValue("tpv_invoicedept");
        String invoiceNums = (String) model.getValue("tpv_invocenums");//发票号
        Date startDate = (Date) model.getValue("tpv_invocestartdate");//其实日期
        Date endDate = (Date) model.getValue("tpv_invoceenddate");//截至日期

        if (null != supplier) {
            long supID = supplier.getLong("id");//供应商id
            filters.add(new QFilter("receivablessupp.id", QCP.equals, supID));//供应商
        }
        if (null != invoiceNums && invoiceNums.length() > 0) {
            filters.add(new QFilter("invoiceno", QCP.equals, invoiceNums));//采购发票号
        }
        if (null != startDate) {
            filters.add(new QFilter("issuedate", QCP.large_equals, DateUtil.getStartOfDay(startDate)));//开票日期从
        }
        if (null != endDate) {
            filters.add(new QFilter("issuedate", QCP.less_equals, DateUtil.getEndOfDay(endDate)));//开票日期至
        }
        filters.add(new QFilter("biztype", QCP.equals, "PUR"));//业务类型

        DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "ap_invoice",
                "id,billno,billstatus,invoiceno,receivablessupp.id,entry.quantity,issuedate,pricetaxtotal,amount,tax"
                , filters.toArray(new QFilter[0]));
        DataSet select = queryDataSet.select("id", "billno", "invoiceno", "receivablessupp.id supplier", "entry.quantity quantity", "issuedate", "pricetaxtotal", "amount", "tax");
        DataSet executeSql = select.executeSql("SELECT billno,id, invoiceno, supplier, issuedate,pricetaxtotal, amount, tax, sum(quantity) AS sumqty " +
                "GROUP BY billno,invoiceno, supplier, issuedate, pricetaxtotal, amount, tax,id");
        executeSql.print(true);
        //分录填充数据
        if (executeSql.isEmpty()) {
            this.getModel().deleteEntryData(invoiceEntry);
            return;
        }
        this.getModel().deleteEntryData(invoiceEntry);
        setInvoiceData(executeSql);
    }

    /**
     * @param queryDataSet:
     * @return void
     * @author: chen
     * @date: 2023/11/19 16:50
     * @description: 分录填充数据
     */
    private void setInvoiceData(DataSet queryDataSet) {
        IDataModel model = this.getModel();
        IFormView view = this.getView();
        String billno = (String) view.getParentView().getModel().getValue("billno");
        Map<String, Object> params = view.getFormShowParameter().getCustomParams();
        ArrayList<String> sourceID = getSourceID(String.valueOf(params.get("sourceID")));

        IDataModel parentModel = view.getParentView().getModel();
        Long id = (Long) parentModel.getDataEntity().getPkValue();
        DynamicObjectCollection entity = model.getEntryEntity(invoiceEntry);
        HashMap<String, HashMap<String, BigDecimal>> dataMap = getMatchedQty(id, billno);//获取已匹配数量等
        for (Row row : queryDataSet) {
            int i = model.createNewEntryRow(invoiceEntry);
            String invoiceBillNo = row.getString("billno");
            Long sourceid = row.getLong("id");
            String invoiceno = row.getString("invoiceno");
            Date issuedate = row.getDate("issuedate");
            BigDecimal taxRate = getTaxRate(invoiceno);
            BigDecimal sumqty = row.getBigDecimal("sumqty");
            BigDecimal pricetaxtotal = row.getBigDecimal("pricetaxtotal");//含税金额
            BigDecimal amount = row.getBigDecimal("amount");//不含税金额
            BigDecimal tax = row.getBigDecimal("tax");//增值税额
            Long pkid = row.getLong("id");//来源单id

            BigDecimal matchedQty = BigDecimal.ZERO;
            BigDecimal matchamoalltax = BigDecimal.ZERO;
            BigDecimal matchamonoalltax = BigDecimal.ZERO;
            BigDecimal matchamoinalltax = BigDecimal.ZERO;
            if (dataMap.get(invoiceno) != null) {
                matchedQty = dataMap.get(invoiceno).get("curmactchallqty");//已匹配数量
                matchamoalltax = dataMap.get(invoiceno).get("curmatchamoalltax");//已匹配金额(含税)
                matchamonoalltax = dataMap.get(invoiceno).get("curmatchamonoalltax");//已匹配金额(不含税)
                matchamoinalltax = dataMap.get(invoiceno).get("curmatchamoinalltax");//已匹配金额(增值税额)
            }
            model.setValue("tpv_invocenum", invoiceno, i);//发票号
            model.setValue("tpv_invoicsupplier", toLong(row.getLong("supplier")), i);//供应商
            model.setValue("tpv_invoicqty", sumqty, i);//数量
            model.setValue("tpv_invoicmate", matchedQty, i);//已匹配数量
            model.setValue("tpv_unmatchqty", sumqty.subtract(matchedQty), i);//未匹配数量
            model.setValue("tpv_issuedate", issuedate, i);//开票日期
            model.setValue("tpv_taxamount", pricetaxtotal, i);//含税金额
            model.setValue("tpv_notaxamount", amount, i);//不含税金额
            model.setValue("tpv_amountincrease", tax, i);//增值税额
            model.setValue("tpv_matchamotax", matchamoalltax, i);//已匹配金额(含税)
            model.setValue("tpv_matchamonotax", matchamonoalltax, i);//已匹配金额(不含税)
            model.setValue("tpv_matchamointax", matchamoinalltax, i);//已匹配金额(增值税)
            model.setValue("tpv_unmatchamotax", pricetaxtotal.subtract(matchamoalltax), i);//未匹配金额(含税)
            model.setValue("tpv_numatchamonotax", amount.subtract(matchamonoalltax), i);//未匹配金额(不含税)
            model.setValue("tpv_unmatchamointax", tax.subtract(matchamoinalltax), i);//未匹配金额(增值税额)
            model.setValue("tpv_taxrates", taxRate, i);//税率
            model.setValue("tpv_sourceid", pkid, i);//来源id
            if (sourceID.contains(pkid.toString())) {
                model.setValue("tpv_focus", true, i);//是否选择
                DynamicObjectCollection entity1 = this.getModel().getEntryEntity(selectInvoiceEntry);
                for (DynamicObject object : entity1) {
                    if (object.getString("tpv_sourceid1").equals(pkid.toString())) {
                        object.set("tpv_seq", i + 1);
                    }
                }
            }
        }
    }

    /**
     * @param sourceID:
     * @return ArrayList<String>
     * @author: chen
     * @date: 2023/11/19 16:51
     * @description: 获取父页面数据的ID
     */
    private ArrayList<String> getSourceID(String sourceID) {
        if (sourceID.length() == 0) {
            return new ArrayList<>();
        }
        String substring = sourceID.substring(0, sourceID.length() - 1);
        String[] split = substring.split(",");
        List<String> collect = Arrays.stream(split).collect(Collectors.toList());
        return (ArrayList<String>) collect;
    }

    /**
     * @param invoiceno:
     * @return BigDecimal
     * @author: chen
     * @date: 2023/11/19 16:51
     * @description: 获取税率
     */
    private BigDecimal getTaxRate(String invoiceno) {
        ArrayList<QFilter> filters = new ArrayList<>();
        filters.add(new QFilter("invoiceno", QCP.equals, invoiceno));//合同号相等
        DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "ap_invoice",
                "entry.taxrateid.taxrate"
                , filters.toArray(new QFilter[0]));
        for (Row row : queryDataSet) {
            BigDecimal taxrate = row.getBigDecimal("entry.taxrateid.taxrate");
            return taxrate;
        }

        return BigDecimal.ZERO;
    }

    /**
     * @param pkId:
     * @param billNo:
     * @return HashMap<String, HashMap < String, BigDecimal>>
     * @author: chen
     * @date: 2023/11/19 16:51
     * @description: 获取已匹配数量
     */
    public HashMap<String, HashMap<String, BigDecimal>> getMatchedQty(Long pkId, String billNo) {
        ArrayList<QFilter> filters = new ArrayList<>();
        //filters.add(new QFilter("id", QCP.equals, pkId));//
        filters.add(new QFilter("billno", QCP.not_equals, billNo));//单据编号
        HashMap<String, HashMap<String, BigDecimal>> map = new HashMap<>();
        DataSet load = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_purinvomater",
                "id,billno,tpv_invoiceinfoentry.tpv_invocenum,tpv_invoiceinfoentry.tpv_curmactchqtys" +
                        ",tpv_invoiceinfoentry.tpv_curmatchamotax,tpv_invoiceinfoentry.tpv_curmatchamonotax,tpv_invoiceinfoentry.tpv_curmatchamointax"
                , filters.toArray(new QFilter[0]));
        DataSet selected = load.select("id", "billno", "tpv_invoiceinfoentry.tpv_invocenum invocenum", "tpv_invoiceinfoentry.tpv_curmactchqtys curmactchqty"
                , "tpv_invoiceinfoentry.tpv_curmatchamotax curmatchamotax", "tpv_invoiceinfoentry.tpv_curmatchamonotax curmatchamonotax", "tpv_invoiceinfoentry.tpv_curmatchamointax curmatchamointax");
        DataSet executeSql = selected.executeSql("select invocenum ,sum(curmactchqty) curmactchallqty ,sum(curmatchamotax) curmatchamoalltax" +
                ",sum(curmatchamonotax) curmatchamonoalltax,sum(curmatchamointax) curmatchamoinalltax GROUP BY invocenum");
        for (Row row : executeSql) {
            HashMap<String, BigDecimal> map1 = new HashMap<>();
            map1.put("curmactchallqty", row.getBigDecimal("curmactchallqty"));
            map1.put("curmatchamoalltax", row.getBigDecimal("curmatchamoalltax"));
            map1.put("curmatchamonoalltax", row.getBigDecimal("curmatchamonoalltax"));
            map1.put("curmatchamoinalltax", row.getBigDecimal("curmatchamoinalltax"));
            map.put(row.getString("invocenum"), map1);
        }
        return map;
    }

    public Long toLong(Object longObj) {
        if (longObj == null) {
            return null;
        } else if (longObj instanceof Long) {
            return (Long) longObj;
        } else {
            return Long.valueOf(String.valueOf(longObj));
        }

    }

}
