package tpv.tpv_wm.tpv_dt.java.formplugin.form;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.entity.property.RefBillProp;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.RefBillEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 《销售发货》{收款计划}【收款方式】为信用证时，《收证处理》则自动携带至《发货单》{收款计划}【信用证号】字段
 * @Project: zhck
 * @Package: com.itopview.wm.dt.formplugin.form.LetterCreditCarriedFormPlugin
 * @Date: 2023/11/20 15:59
 * @Author: Heisenberg Wenbei He
 * @Version: 1.0
 */
public class LetterCreditCarriedFormPlugin extends AbstractFormPlugin implements BeforeF7SelectListener {
    private static final String PAYMENT_WAYS = "tpv_paymentmethods"; // {收款计划} 分录 instance of DynamicObjectCollection
    private static final String PAYMENT_TYPE = "tpv_paymenttype"; // {收款计划}【收款方式】instance of Base
    private static final String LC = "tpv_lcf"; // {收款计划}【信用证号】 分录 instance of Bill
    private static final String CONTRACT_NUM = "tpv_contractnumber"; // 合同号 instance of text
    private static final String LC_RECEIPT = "lc_receipt"; // 《收证处理》
    private static final String INTERAL = "JSFS12"; // 国内信用证
    private static final String GLOBAL = "JSFS13"; // 国际信用证
    private static boolean isGlobal; // 是否为国际证

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        BasedataEdit lcEdit = this.getView().getControl(LC);
        lcEdit.addBeforeF7SelectListener(this);
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent beforeF7SelectEvent) {
        if (LC.equals(beforeF7SelectEvent.getProperty().getName())) {
            DynamicObject tpvCustomer = (DynamicObject) this.getModel().getValue("tpv_customer");// 客户
            Long tpvContractnumber = (Long) this.getModel().getValue("tpv_sourcebillid");
            ListShowParameter showParameter = (ListShowParameter) beforeF7SelectEvent.getFormShowParameter();
            DynamicObjectCollection tpvPaymentmethods = this.getModel().getDataEntity(true).getDynamicObjectCollection("tpv_paymentmethods");
            EntryGrid entry = this.getControl("tpv_paymentmethods");
            int[] selectRows = entry.getSelectRows();
            if (selectRows.length==1){
                DynamicObject tpvPaymenttype = tpvPaymentmethods.get(selectRows[0]).getDynamicObject("tpv_paymenttype");
                if (tpvPaymenttype!=null){
                    String name = tpvPaymenttype.getString("number");
                    if ("JSFS12".equals(name)||"JSFS13".equals(name)){

                        List<QFilter> QFilters = new ArrayList<>();
                        QFilter qFilter = new QFilter("benefitertype", "=", "bd_customer");
                        qFilter.and(new QFilter("billstatus", QCP.equals, "C"));
                        qFilter.and(new QFilter("tpv_claimnoticebillno", QCP.not_equals, ""));

                        if (tpvCustomer != null) {
                            qFilter.and(new QFilter("benefiter", QCP.equals, tpvCustomer.getPkValue()));
                        }
                        if ("JSFS12".equals(name)){
                            qFilter.and(new QFilter("isnationalcard", QCP.equals, false));
                        }
                        if ("JSFS13".equals(name)){
                            qFilter.and(new QFilter("isnationalcard", QCP.equals, true));
                        }
                        if (0!=tpvContractnumber){
                            qFilter.and(new QFilter("entrys.tpv_salecontract.id", QCP.equals, tpvContractnumber));
                        }
                        DynamicObject[] lcReceipt = BusinessDataServiceHelper.load("lc_receipt", "benefiter,creditno",
                                qFilter.toArray());
                        List<String> collect = Arrays.stream(lcReceipt).map(tpvTmSnapdebaseedit -> tpvTmSnapdebaseedit.getString("creditno")).collect(Collectors.toList());
                        QFilters.add(new QFilter("number", "in", collect));
                        showParameter.getListFilterParameter().setQFilters(QFilters);
                    }
                }
            }
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        if (e.getProperty().getName().equals(PAYMENT_TYPE)) {
            synchronize(e);
        } else if (e.getProperty().getName().equals("tpv_lcf")) {
            int selectedRowIndex = e.getChangeSet()[0].getRowIndex();
            DynamicObjectCollection paymentWaysCollection = this.getModel().getDataEntity(true).getDynamicObjectCollection(PAYMENT_WAYS);
            DynamicObject selectedPaymentType = paymentWaysCollection.get(selectedRowIndex).getDynamicObject("tpv_lcf");
            if (selectedPaymentType!=null){
                DynamicObject creditReceipts = BusinessDataServiceHelper.loadSingle(LC_RECEIPT,
                        "id,creditno,billstatus,entrys,tpv_salecontract,isnationalcard,tpv_name",
                        new QFilter[]{new QFilter("creditno", QCP.equals, selectedPaymentType.getString("number"))}); // 收款信用证
                this.getModel().setValue("tpv_lc",creditReceipts,selectedRowIndex);
            }else {
                this.getModel().setValue("tpv_lc",null,selectedRowIndex);
            }
        }
    }

    /**
     * @param e: 值更新事件
     * @author: Heisenberg
     * @date: 2023/11/24 11:53
     * @description: 当收款方式为信用证时，若当前单据只关联一个《收证处理》则自动带出，否则手动选择
     */
    private void synchronize(PropertyChangedArgs e) {
        int selectedRowIndex = e.getChangeSet()[0].getRowIndex();
        DynamicObjectCollection paymentWaysCollection = this.getModel().getDataEntity(true).getDynamicObjectCollection(PAYMENT_WAYS);
        DynamicObject selectedPaymentType = paymentWaysCollection.get(selectedRowIndex).getDynamicObject(PAYMENT_TYPE);
        String contractNumber = (String) this.getModel().getValue(CONTRACT_NUM); // 《销售发货》合同号
        if (selectedPaymentType != null) {
            String number = selectedPaymentType.getString("number");
            boolean isCredit = (INTERAL.equals(number) || GLOBAL.equals(number)); // 【收款方式】是否为信用证
            if (isCredit) {
                isGlobal(number);
                RefBillEdit lcEdit = this.getView().getControl(LC);
                lcEdit.setMustInput(true); // 设置必录
                RefBillProp lcProp = (RefBillProp) lcEdit.getProperty();
                lcProp.setMustInput(true); // 设置必录
                DynamicObject[] creditReceipts = BusinessDataServiceHelper.load(LC_RECEIPT,
                        "id,creditno,billstatus,entrys,tpv_salecontract,isnationalcard,tpv_name",
                        new QFilter[]{new QFilter("entrys.tpv_salecontract.tpv_name", QCP.equals, contractNumber),
                                new QFilter("isnationalcard", QCP.equals, isGlobal),
                                new QFilter("billstatus", QCP.equals, "C")}); // 收款信用证
                Optional.ofNullable(creditReceipts)
                        .filter(array -> array.length == 1)
                        .ifPresent(single -> this.getModel().setValue(LC, single[0].getLong("id"), selectedRowIndex));
            }
        }
    }

    /**
     * @param number: 收款方式编码
     * @author: Heisenberg
     * @date: 2023/11/24 11:54
     * @description: 判断信用证为国际还是国内，基于此过滤f7信用证
     */
    private static void isGlobal(String number) {
        isGlobal = GLOBAL.equals(number); // true 为国际证，false 为 国内证
    }
}
