package tpv.tpv_wm.tpv_dt.java.formplugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.events.RowClickEventListener;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.EventObject;

/**
 * @Description: 采购合同【付款信息】默认一行，【付款方式】=“电汇”，【比例】=”100“，增行时，新增行【比例】=100-之前行【比例】之和；
 * @Project: zhck
 * @Package: tpv.tpv_wm.tpv_dt.java.formplugin.form.PaymentDefaultLineFormPlugin
 * @Date: 2023/12/12 15:36
 * @Author: Heisenberg Wenbei He
 * @Version: 1.0
 */
public class PaymentDefaultLineFormPlugin extends AbstractBillPlugIn implements RowClickEventListener {

    private static final String ENTRY = "tpv_dt_purchase_u"; // 付款计划 instance of Entry
    private static final String PAYWAY = "JSFS04"; // 付款方式基础资料编码（电开） instance of BaseData.number
    private static final String PAYMNET = "tpv_payment"; // 付款方式 instance of BaseData
    private static final String HUNTING_MOMENT = "bd_settlementtype"; // 结算方式 instance of BaseData.formsign
    private static final String PROPORTION = "tpv_proportion"; // 付款比例 instance of Decimal-Fraction
    private static final BigDecimal ONE_HUNDRED = new BigDecimal("100"); // instance of Constant
    private static final String ERROR_MESSAGE = "付款计划中比例超过100，已自动校正！"; // 错误提示 instance of Constant

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        EntryGrid entry = this.getView().getControl(ENTRY);
        entry.addRowClickListener(this);
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        this.init();
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        switch (e.getProperty().getName()) {
            case PROPORTION:
                this.isReasonable(e);
                break;
            default:
                break;
        }
    }


    /**
     * @author: Heisenberg
     * @date: 2023/12/12 15:50
     * @description: 判断当前单据是否为新增
     */
    private boolean isOriginal() {
        DynamicObjectCollection collection = this.getModel().getDataEntity(true).getDynamicObjectCollection(ENTRY);
        return collection.isEmpty();
    }

    /**
     * @author: Heisenberg
     * @date: 2023/12/12 20:19
     * @description: 初始化付款计划分录，默认一行，付款方式为电汇，比例为100%
     */
    private void init() {
        if (isOriginal()) {
            DynamicObject huntWay = BusinessDataServiceHelper.loadSingle(HUNTING_MOMENT,
                    new QFilter[]{new QFilter("number", QCP.equals, PAYWAY)});
            int row = this.getModel().createNewEntryRow(ENTRY);
            this.getModel().setValue(PAYMNET, huntWay, row);
            this.getModel().setValue(PROPORTION, ONE_HUNDRED, row);
        }
    }


    /**
     * @param afterDoOperationEventArgs:
     * @return void
     * @author: chen
     * @date: 2023/12/15 14:20
     * @description: 增行自动计算剩余比例并赋值
     */
    @Override
    public void afterDoOperation(AfterDoOperationEventArgs afterDoOperationEventArgs) {
        String key = afterDoOperationEventArgs.getOperateKey();
        if (key.equals("newentry")) {
            // 新增的分录行
            int rowIndex = this.getModel().getEntryCurrentRowIndex(ENTRY);
            this.getModel().setValue(PROPORTION, ONE_HUNDRED.subtract(this.calculateProportion()), rowIndex);
        }
    }

    /**
     * @return BigDecimal
     * @author: Heisenberg
     * @date: 2023/12/12 20:02
     * @description: 计算收款计划分录的比例总和
     */
    private BigDecimal calculateProportion() {
        DynamicObjectCollection collection = this.getModel().getDataEntity(true).getDynamicObjectCollection(ENTRY);
        return collection.stream()
                .map(row -> row.getBigDecimal(PROPORTION))
                .reduce(BigDecimal::add)
                .orElseGet(() -> BigDecimal.ZERO);
    }

    /**
     * @author: Heisenberg
     * @date: 2023/12/12 20:23
     * @description: 只更新比例超过100时，提示并修改最近更改的比例为剩余比例
     */
    private void isReasonable(PropertyChangedArgs e) {
        if (ONE_HUNDRED.compareTo(this.calculateProportion()) < 0) {
            this.getView().showTipNotification(ERROR_MESSAGE);
            this.getModel().setValue(PROPORTION,
                    ONE_HUNDRED.subtract(this.calculateProportion().subtract(
                            (BigDecimal) e.getChangeSet()[0].getNewValue())), e.getChangeSet()[0].getRowIndex());
        }
    }
}
