package tpv.tpv_wm.tpv_dt.java.formplugin.form;

import com.itopview.wm.sys.utils.BothCardHelper;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.events.ChangeData;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.field.RefBillEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

/**
 * @Description 加工指令单 合同号过滤当前组织
 * @Author LinRui
 * Date 2023/8/14 18:15
 **/
public class ProcessFormPlugin extends AbstractBillPlugIn implements BeforeF7SelectListener {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        RefBillEdit tpv_purchasecode = (RefBillEdit) this.getView().getControl("tpv_purchasecode");
        tpv_purchasecode.addBeforeF7SelectListener((BeforeF7SelectListener) this);
        RefBillEdit tpv_salescode = (RefBillEdit) this.getView().getControl("tpv_salescode");
        tpv_salescode.addBeforeF7SelectListener((BeforeF7SelectListener) this);
        RefBillEdit tpv_copackingagreeme = (RefBillEdit) this.getView().getControl("tpv_copackingagreeme");
        tpv_copackingagreeme.addBeforeF7SelectListener((BeforeF7SelectListener) this);
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        String name = e.getProperty().getName();
        switch (name) {
            case "tpv_purchasecode":  //采购合同号
            case "tpv_salescode":  //销售合同号
            case "tpv_copackingagreeme":  //加工协议
                qFilterThisOrgName(e);  //过滤当前组织
                break;
        }
    }

    //过滤当前组织
    private void qFilterThisOrgName(BeforeF7SelectEvent e) {
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        List<QFilter> QFilters = new ArrayList<QFilter>();
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        DynamicObject org = dataEntity.getDynamicObject("org");
        if (org == null) {
            return;
        }
        Long id = (Long) org.get("id");
        QFilters.add(new QFilter("org", "=", id)); //设置过滤条件：组织
        QFilters.add(new QFilter("billstatus", "=", "C")); //单据状态：已审核
        showParameter.getListFilterParameter().setQFilters(QFilters);
    }

    //  银行账户bankaccount  银行信息单据体  bizparther_phone 工商登记电话  linkman 联系人  postal_code 电子邮箱  tx_register_no 纳税人识别号  bizparther_address  注册地址
    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        ChangeData cd = e.getChangeSet()[0];
        ChangeData[] changeData = e.getChangeSet();
        int rowIndex = changeData[0].getRowIndex();
        switch (name) {
            case "tpv_processpurpose":  //加工目的 下拉列表字段
                clearCompanyinfo();  //清楚抬头信息
                break;
            case "tpv_purchasecode":  //采购合同号
                getBillSupplierInfo();  //获取供应商信息
                break;
            case "tpv_salescode":  //销售合同号
                getBillCustomerInfo();  //获取客户信息
                break;
        }
    }

    //获取加工目的选项，清除抬头信息
    private void clearCompanyinfo() {
        this.getModel().setValue("tpv_combinationnumber", null);  //组合号赋空
        clearSupAndCusinfo(); //清楚供应商和客户信息
    }

    //获取选择合同号对应单据里的客户信息，并赋值给当前单据页面抬头客户内容块
    private void getBillCustomerInfo() {
        this.getModel().setValue("tpv_combinationnumber", null);  //组合号赋空
        clearSupAndCusinfo(); //清楚供应商和客户信息
        DynamicObject tpvSalescode = (DynamicObject) this.getModel().getValue("tpv_salescode");
        //合同号为空返回
        if (tpvSalescode == null) {
            return;
        } else {
            DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle("tpv_dt_salescontract", new QFilter[]{new QFilter("billno", QCP.equals, tpvSalescode.getString("billno"))});
            DynamicObject tpvCombinationnumber = dynamicObject.getDynamicObject("tpv_combinationnumber");  //组合号
            DynamicObject tpvCustomer = dynamicObject.getDynamicObject("tpv_customer"); //客户
            if (tpvCustomer == null) {
                this.getView().showMessage("选择的销售合同号里没有填写客户信息");
            } else {
                String tpvCustomername = dynamicObject.getString("tpv_customername");  //客户名称
                String tpvCustomertin = dynamicObject.getString("tpv_customertin");  //纳税人识别号(供应商)
                String tpvCustomerbank = dynamicObject.getString("tpv_customerbank");  //开户行(供应商)
                String tpvCustomeracct = dynamicObject.getString("tpv_customeracct");  //账号(供应商)
                String tpvCustomerlinkman = dynamicObject.getString("tpv_customerlinkman");  //联系人(供应商)
                String tpvCustomertel = dynamicObject.getString("tpv_customertel");  //电话(供应商)
                String tpvCustomeraddr = dynamicObject.getString("tpv_customeraddr");  //地址(供应商)
                String tpvCustomeremail = dynamicObject.getString("tpv_customeremail"); //E-Mail(供应商)
                this.getModel().setValue("tpv_customer", tpvCustomer);
                this.getModel().setValue("tpv_customername", tpvCustomername);
                this.getModel().setValue("tpv_customertin", tpvCustomertin);
                this.getModel().setValue("tpv_customerbank", tpvCustomerbank);
                this.getModel().setValue("tpv_customeracct", tpvCustomeracct);
                this.getModel().setValue("tpv_customerlinkman", tpvCustomerlinkman);
                this.getModel().setValue("tpv_customertel", tpvCustomertel);
                this.getModel().setValue("tpv_customeraddr", tpvCustomeraddr);
                this.getModel().setValue("tpv_customeremail", tpvCustomeremail);
            }
            this.getModel().setValue("tpv_combinationnumber", tpvCombinationnumber); //组合号赋值
            BothCardHelper.setCustomerLabel(this);  //抬头赋值
        }
    }

    //获取选择合同号对应单据里的供应商信息，并赋值给当前单据页面抬头供应商内容块
    private void getBillSupplierInfo() {
        this.getModel().setValue("tpv_combinationnumber", null);  //组合号赋空
        clearSupAndCusinfo(); //清楚供应商和客户信息
        DynamicObject tpvPurchasecode = (DynamicObject) this.getModel().getValue("tpv_purchasecode");
        //合同号为空返回
        if (tpvPurchasecode == null) {
            return;
        } else {
            DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle("tpv_dt_purchase", new QFilter[]{new QFilter("billno", QCP.equals, tpvPurchasecode.getString("billno"))});
            DynamicObject tpvCombinationnumber = dynamicObject.getDynamicObject("tpv_combinationnumber");  //组合号
            DynamicObject tpvSupplier = dynamicObject.getDynamicObject("tpv_supplier"); //供应商
            if (tpvSupplier == null) {
                this.getView().showMessage("选择的采购合同号里没有填写供应商信息");
            } else {
                String tpvSuppliername = dynamicObject.getString("tpv_suppliername");  //供应商名称
                String tpvSuppliertin = dynamicObject.getString("tpv_suppliertin");  //纳税人识别号(供应商)
                String tpvSupplierbank = dynamicObject.getString("tpv_supplierbank");  //开户行(供应商)
                String tpvSupplieracct = dynamicObject.getString("tpv_supplieracct");  //账号(供应商)
                String tpvSupplierlinkman = dynamicObject.getString("tpv_supplierlinkman");  //联系人(供应商)
                String tpvSuppliertel = dynamicObject.getString("tpv_suppliertel");  //电话(供应商)
                String tpvSupplieraddr = dynamicObject.getString("tpv_supplieraddr");  //地址(供应商)
                String tpvSupplieremail = dynamicObject.getString("tpv_supplieremail"); //E-Mail(供应商)
                this.getModel().setValue("tpv_supplier", tpvSupplier);
                this.getModel().setValue("tpv_suppliername", tpvSuppliername);
                this.getModel().setValue("tpv_suppliertin", tpvSuppliertin);
                this.getModel().setValue("tpv_supplierbank", tpvSupplierbank);
                this.getModel().setValue("tpv_supplieracct", tpvSupplieracct);
                this.getModel().setValue("tpv_supplierlinkman", tpvSupplierlinkman);
                this.getModel().setValue("tpv_suppliertel", tpvSuppliertel);
                this.getModel().setValue("tpv_supplieraddr", tpvSupplieraddr);
                this.getModel().setValue("tpv_supplieremail", tpvSupplieremail);
            }
            this.getModel().setValue("tpv_combinationnumber", tpvCombinationnumber);  //组合号赋值
            BothCardHelper.setSupplierLabel(this);  //抬头赋值

        }
    }

    /**
     * Description //清楚供应商和客户抬头信息
     **/
    private void clearSupAndCusinfo() {
        this.getModel().setValue("tpv_customer", null);  //清空抬头信息字段
        this.getModel().setValue("tpv_customername", null);
        this.getModel().setValue("tpv_customertin", null);
        this.getModel().setValue("tpv_customerbank", null);
        this.getModel().setValue("tpv_customeracct", null);
        this.getModel().setValue("tpv_customerlinkman", null);
        this.getModel().setValue("tpv_customertel", null);
        this.getModel().setValue("tpv_customeraddr", null);
        this.getModel().setValue("tpv_customeremail", null);

        this.getModel().setValue("tpv_supplier", null);  //供应商清空字段
        this.getModel().setValue("tpv_suppliername", null);
        this.getModel().setValue("tpv_suppliertin", null);
        this.getModel().setValue("tpv_supplierbank", null);
        this.getModel().setValue("tpv_supplieracct", null);
        this.getModel().setValue("tpv_supplierlinkman", null);
        this.getModel().setValue("tpv_suppliertel", null);
        this.getModel().setValue("tpv_supplieraddr", null);
        this.getModel().setValue("tpv_supplieremail", null);

        BothCardHelper.setCustomerLabel(this);  //抬头客户赋值
        BothCardHelper.setSupplierLabel(this);  //抬头供应商赋值
    }

}
