package tpv.tpv_wm.tpv_dt.java.formplugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.form.ShowType;
import kd.bos.form.control.Search;
import kd.bos.form.control.events.SearchEnterEvent;
import kd.bos.form.control.events.SearchEnterListener;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClient;
import tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClientHelper;
import tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsTypeCodeConverter;

import java.util.Collections;
import java.util.EventObject;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClientHelper.*;


/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhkc
 * @Package: com.itopview.wm.dt.formplugin.form.AutoInsertShipBillPlugin
 * @Date: 11:54 2023/09/20
 * @Description: 关注船清单键入船名自动弹出类似船舶，选择后自动给其他字段赋值 搜索控件
 * @Version: 1.0
 */
public class SelectShipByNameFormPlugin extends AbstractBillPlugIn implements SearchEnterListener {
    private String id;
    private List<ShipNewsClientHelper.ShipDetail> shiplist;

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public List<ShipDetail> getShiplist() {
        return shiplist;
    }

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        Search searchap = this.getControl("tpv_searchap");
        searchap.addEnterListener(this);
    }

    /**
     * @param searchEnterEvent: 搜索控件失去焦点或回车事件
     * @author: Heisenberg
     * @date: 2023/10/10 14:47
     * @description: 截取船舶 id ， 判断是否存在于单据列表，存在打开其视图，否则渲染当前页面给各字段赋值，如需保存需前端手动保存
     */
    @Override
    public void search(SearchEnterEvent searchEnterEvent) {
        id = getSearchId(searchEnterEvent);
        if (isExistShipId(id)) {
            DynamicObject currentShipBill =
                    getCurrentShipBill(new QFilter[]{new QFilter("tpv_shipid", QCP.equals, id)});
            BillShowParameter billShowParameter = new BillShowParameter();
            billShowParameter.setFormId("tpv_wms_gzcqd");
            billShowParameter.setPkId(currentShipBill.getLong("id"));
            billShowParameter.setStatus(OperationStatus.ADDNEW);
            billShowParameter.getOpenStyle().setShowType(ShowType.MainNewTabPage);
            this.getView().showForm(billShowParameter);
        } else {
            Optional<List<ShipDetail>> shipDetails = ShipNewsClient.queryShipInfo(Collections.singletonList(id), "id");
            shipDetails.ifPresent(list -> shiplist = list);
            shipDetails.filter(u -> !u.isEmpty())
                    .flatMap(shipInfoList -> shipInfoList.stream().findFirst())
                    .map(shipInfo -> {
                        this.getModel().setValue("tpv_shipname", shipInfo.getName());
                        this.getModel().setValue("tpv_shipmmssi", shipInfo.getMmsi());
                        this.getModel().setValue("tpv_shipimo", shipInfo.getImo());
                        this.getModel().setValue("tpv_shipid", shipInfo.getShipID());
                        this.getModel().setValue("tpv_destination",
                                ShipNewsTypeCodeConverter.getShipType(Integer.parseInt(shipInfo.getShiptype())));
                        this.getView().showSuccessNotification("船舶查询成功!");
                        return shipInfo;
                    }).orElseGet(() -> {
                        this.getView().showTipNotification("船舶查询为空!");
                        return null;
                    });
        }
    }

    /**
     * @param evt: 搜索控件值改变
     * @return List<String>
     * @author: Heisenberg
     * @date: 2023/10/10 14:45
     * @description: 根据搜索控件输入的值实时跟新下拉列表（船舶名称+船舶id），自动补全，
     */

    @Override
    public List<String> getSearchList(SearchEnterEvent evt) {
        String text = evt.getText(); // 获取搜索框的信息
        List<String> shipDetail = null;
        if (!text.isEmpty()) {
            shipDetail = ShipNewsClient.queryShipInfo(Collections.singletonList(text), "kw")
                    .filter(u -> !u.isEmpty())
                    .map(u -> u.stream().map(s ->
                            s.getName() + "(" + s.getShipID() + ")").collect(Collectors.toList()))
                    .orElse(Collections.emptyList());
        }
        return shipDetail;
    }
}
