package tpv.tpv_wm.tpv_dt.java.formplugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.form.control.Search;
import kd.bos.form.control.events.SearchEnterEvent;
import kd.bos.form.control.events.SearchEnterListener;
import tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClient;

import java.util.Collections;
import java.util.EventObject;
import java.util.List;
import java.util.stream.Collectors;

import static tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClientHelper.getSearchId;


/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhkc
 * @Package: com.itopview.wm.dt.formplugin.form.AutoInsertShipBillPlugin
 * @Date: 11:54 2023/09/20
 * @Description: 关注船清单键入船名自动弹出类似船舶，货物流转单船舶搜索控件调用接口
 * @Version: 1.0
 */
public class SelectShipNameOnlyFormPlugin extends AbstractBillPlugIn implements SearchEnterListener {

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        Search searchap = this.getControl("tpv_searchap");
        searchap.addEnterListener(this);
    }

    /**
     * @param searchEnterEvent: 搜索控件失去焦点或回车事件
     * @author: Heisenberg
     * @date: 2023/10/10 14:47
     * @description: 截取船舶 id ， 判断是否存在于单据列表，
     */
    @Override
    public void search(SearchEnterEvent searchEnterEvent) {
        String id = getSearchId(searchEnterEvent);
        ShipNewsClient.queryShipInfo(Collections.singletonList(id), "id")
                .filter(u -> !u.isEmpty())
                .flatMap(shipInfoList -> shipInfoList.stream().findFirst())
                .map(shipInfo -> {
                    this.getView().showSuccessNotification("船舶查询成功!");
                    return shipInfo;
                }).orElseGet(() -> {
                    this.getView().showTipNotification("查询船舶信息为空!");
                    return null;
                });
    }


    /**
     * @param evt: 搜索控件值改变
     * @return List<String>
     * @author: Heisenberg
     * @date: 2023/10/10 14:45
     * @description: 根据搜索控件输入的值实时跟新下拉列表（船舶名称+船舶id），自动补全，
     */
    @Override
    public List<String> getSearchList(SearchEnterEvent evt) {
        String text = evt.getText(); // 获取搜索框的信息
        return ShipNewsClient.queryShipInfo(Collections.singletonList(text), "kw")
                .filter(u -> !u.isEmpty())
                .map(u -> u.stream().map(s ->
                        s.getName() + "(" + s.getShipID() + ")").collect(Collectors.toList()))
                .orElseGet(() -> Collections.emptyList());
    }
}
