package tpv.tpv_wm.tpv_dt.java.formplugin.form;

import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.bill.BillShowParameter;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.entity.OrmLocaleValue;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.ShowType;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.control.events.RowClickEvent;
import kd.bos.form.control.events.RowClickEventListener;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * ClassName: TransformBenchFromPlugin
 * Description: 转口物流工作台 表单插件
 * date: 2023/9/15 10:00
 * author: Guo jiaChen
 */

public class TransformBenchFromPlugin extends AbstractBillPlugIn implements RowClickEventListener {
    private final static String ENTRY_FIRST = "tpv_entryentity";//工作台分录
    private final static String ENTRY_ARRIVAL = "tpv_unit";//到单
    private final static String ENTRY_HAND = "tpv_unit1";//交单
    private final HashMap<String, Integer> entryIndex = new HashMap<>();//分录下标
    private final ArrayList<HashMap<String, Object>> entryList = new ArrayList<>();//工作台数据

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        String[] listener = {"tpv_addmark", "tpv_query"};
        String[] itemListener = {"tpv_query", "tpv_addmark", "tpv_advconbaritemap"};
        this.addClickListeners(listener);
        this.addItemClickListeners(itemListener);
        EntryGrid entryGrid = this.getView().getControl("tpv_entryentity");
        entryGrid.addRowClickListener(this);
    }

    @Override
    public void entryRowClick(RowClickEvent evt) {
        RowClickEventListener.super.entryRowClick(evt);
        //获取点击行号
        int row = evt.getRow();
        if (row < 0) {
            return;
        }
        entryIndex.put(ENTRY_FIRST, row);
        //采销装运情况
        openContainerFrom(row);
        //到单,交单情况
        arrivalAndHand(row);
        //累计到单,交单数量
        calculateNum(row);
    }

    /**
     * 到单,交单 分录情况填充
     */
    private void arrivalAndHand(int row) {

        ArrayList<String> purNum = new ArrayList<>();//采购
        ArrayList<String> saleNum = new ArrayList<>();//销售
        HashMap<Object, String> map_p = new HashMap<>();
        HashMap<Object, String> map_s = new HashMap<>();
        this.getModel().deleteEntryData(ENTRY_ARRIVAL);
        this.getModel().deleteEntryData(ENTRY_HAND);

        DynamicObject object = this.getModel().getEntryEntity(ENTRY_FIRST).get(row);
        String groupNumber = object.getString("tpv_groupnumber");
        DynamicObject[] purchases = BusinessDataServiceHelper.load("tpv_dt_purchase", "billno", new QFilter[]{new QFilter("tpv_combinationnumber.number", QCP.equals, groupNumber)});
        DynamicObject[] sale = BusinessDataServiceHelper.load("tpv_dt_salescontract", "billno,tpv_contractnos", new QFilter[]{new QFilter("tpv_combinationnumber.number", QCP.equals, groupNumber)});
        if (purchases.length > 0) {
            Arrays.stream(purchases).forEach(pur -> {
                String billNo = pur.getString("billno");
                purNum.add(billNo);
            });
        }
        //到单情况
        arrivalDetail(purNum);
        if (sale.length > 0) {
            Arrays.stream(sale).forEach(sal -> {
                String billno = sal.getString("billno");
                saleNum.add(billno);
            });
        }
        //交单情况
        handDetail(saleNum);
    }

    /**
     * 到单情况分录填充
     *
     * @param billNos
     */
    private void arrivalDetail(ArrayList<String> billNos) {
        BigDecimal count = BigDecimal.ZERO;//数量
        StringBuilder builder = new StringBuilder();
        //货物流转单
        DynamicObject[] freigroamBill = BusinessDataServiceHelper.load("tpv_dt_freigroambill", "billno,tpv_arrpreno,tpv_materialentry,tpv_srcbillno,", new QFilter[]{new QFilter("tpv_materialentry.tpv_srcbillno", QCP.in, billNos)});

        ArrayList<String> freightRoamBill = new ArrayList<>();
        //获取采购合同下的货物流转单
        Arrays.stream(freigroamBill)
                .forEach(freigroam -> {
                    freightRoamBill.add(freigroam.getString("billno"));
                });
        //获取货物流转单下的进口到单
        DynamicObject[] arrivalBill = BusinessDataServiceHelper.load("tpv_is_arrivaldetais", "tpv_corebillno,tpv_supplier,tpv_amount,tpv_unit,tpv_singledate,tpv_materialentry,tpv_srcbillno,",
                new QFilter[]{new QFilter("tpv_materialentry.tpv_srcbillno", QCP.in, freightRoamBill)});
        if (arrivalBill.length <= 0) {
            return;
        }
        int[] ints = this.getModel().batchCreateNewEntryRow(ENTRY_ARRIVAL, arrivalBill.length);
        int i = 0;
        for (DynamicObject object : arrivalBill) {
            DynamicObjectCollection maters = object.getDynamicObjectCollection("tpv_materialentry");
            DynamicObject objects = null;
            OrmLocaleValue o = null;
            if (maters.size() != 0) {
                objects = maters.get(0);
                o = (OrmLocaleValue) objects.get("tpv_unit.name");
                for (DynamicObject mater : arrivalBill) {
                    BigDecimal tpyQty = mater.getBigDecimal("tpv_amount");//
                    if (BigDecimal.ZERO.compareTo(tpyQty) != 0) {
                        count = tpyQty.add(count);
                    }
                }
            }
            DynamicObject supplier = object.getDynamicObject("tpv_supplier");
            BigDecimal amount = object.getBigDecimal("tpv_amount");
            Date singledate = object.getDate("tpv_singledate");
            String coBillNo = object.getString("tpv_corebillno");//核心单据编码
            this.getModel().setValue("tpv_purchasenum", coBillNo, ints[i]);//采购合同号
            this.getModel().setValue("tpv_suppliers", supplier, ints[i]);//供应商
            this.getModel().setValue("tpv_arrivalnum", clearComma(builder), ints[i]);//到单单据
            this.getModel().setValue("tpv_arrivalcount", count, ints[i]);//到单数量
            this.getModel().setValue("tpv_arrivalunits", null == o ? "" : o.getLocaleValue_zh_CN(), ints[i]);//到单
            this.getModel().setValue("tpv_arrivalamount", amount, ints[i]);//到单金额
            this.getModel().setValue("tpv_arrivaldate", singledate, ints[i]);//到单日期
        }
    }


    /**
     * 交单情况分录填充
     * TODO 销售下推 ? 下推 出口交单
     *
     * @param billNos
     */
    private void handDetail(ArrayList<String> billNos) {
        List<String> srcBillNums = new ArrayList<>();

        StringBuilder builder = new StringBuilder();
        //货物流转单编号

        DynamicObject[] freightRoamBill = BusinessDataServiceHelper.load("tpv_dt_freigroambill", "billno,tpv_arrpreno,tpv_materialentry,tpv_srcbillno", new QFilter[]{new QFilter("tpv_materialentry.tpv_srcbillno", QCP.in, billNos)});
        Arrays.stream(freightRoamBill).forEach(bill -> {
            srcBillNums.add(bill.getString("billno"));
        });
        DynamicObject[] arrivalDetails = BusinessDataServiceHelper.load("tpv_es_exportdelivery", "tpv_customer,tpv_amount,tpv_unit,tpv_surrenderdate1,tpv_materialentry,tpv_srcbillno", new QFilter[]{new QFilter("tpv_materialentry.tpv_srcbillno", QCP.in, srcBillNums)});

        int i = 0;
        BigDecimal count = BigDecimal.ZERO;//数量
        if (arrivalDetails.length <= 0) {
            return;
        }
        int[] ints = this.getModel().batchCreateNewEntryRow(ENTRY_ARRIVAL, arrivalDetails.length);
        for (DynamicObject sale : arrivalDetails) {
            DynamicObjectCollection maters = sale.getDynamicObjectCollection("tpv_materialentry");
            DynamicObject object = null;
            OrmLocaleValue unit = null;
            if (maters.size() > 0) {
                object = maters.get(0);
                unit = (OrmLocaleValue) object.get("tpv_unit.name");
                for (DynamicObject mater : arrivalDetails) {
                    BigDecimal tpyQty = mater.getBigDecimal("tpv_amount");//
                    if (BigDecimal.ZERO.compareTo(tpyQty) != 0) {
                        count = tpyQty.add(count);
                    }
                }
            }
            DynamicObject customer = sale.getDynamicObject("tpv_customer");
            BigDecimal amount = sale.getBigDecimal("tpv_amount");
            Date surrenderdate = sale.getDate("tpv_surrenderdate1");
            String coBillNo = object.getString("tpv_corebillno");//核心单据编码
            this.getModel().setValue("tpv_sellnum", coBillNo, ints[i]);//销售合同号
            this.getModel().setValue("tpv_customers", customer, ints[i]);//供应商
            this.getModel().setValue("tpv_submitnum", clearComma(builder), ints[i]);//到单单据
            this.getModel().setValue("tpv_submitcount", count, ints[i]);//到单数量
            this.getModel().setValue("tpv_submitunits",unit == null ? "" : unit.getLocaleValue_zh_CN(), ints[i]);//到单
            this.getModel().setValue("tpv_submitamount1", amount, ints[i]);//到单金额
            this.getModel().setValue("tpv_submitdate", surrenderdate, ints[i]);//到单日期
            ++i;
        }
    }

    /**
     * 累计到单数,交单数
     *
     * @param index
     */
    public void calculateNum(int index) {

        AtomicInteger arrivalNum = new AtomicInteger();
        AtomicInteger handNum = new AtomicInteger();
        DynamicObjectCollection entity = this.getModel().getEntryEntity(ENTRY_ARRIVAL);
        entity.forEach(arrival -> {
            arrivalNum.addAndGet(arrival.getInt("tpv_arrivalcount"));
        });
        DynamicObjectCollection entities = this.getModel().getEntryEntity(ENTRY_HAND);
        entities.forEach(arrival -> {
            handNum.addAndGet(arrival.getInt("tpv_submitcount"));
        });
        //赋值
        this.getModel().setValue("tpv_arrivalordernum", arrivalNum, index);
        this.getModel().setValue("tpv_handordernum", handNum, index);
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        long start = System.currentTimeMillis();
        query();//查询数据
        bindData();//绑定数据
        int rowCount = this.getModel().getEntryEntity(ENTRY_FIRST).getRowCount();
        if (rowCount > 0) {
            //打开工作台分录
            openContainerFrom(0);
            //打开到单交单情况
            arrivalAndHand(0);
            //累计到单,交单数量
            calculateNum(0);
        } else {
            BillShowParameter showParameter = new BillShowParameter();
            showParameter.setStatus(OperationStatus.EDIT);
            showParameter.getOpenStyle().setShowType(ShowType.InContainer);
            showParameter.getOpenStyle().setTargetKey("tpv_flexpanelap1");
            showParameter.setSendToClient(true);
            showParameter.setFormId("tpv_zkwlgztfl");
            this.getView().showForm(showParameter);
        }
        long end = System.currentTimeMillis();
    }

    /**
     * 绑定数据
     */
    private void bindData() {
        IDataModel model = this.getModel();
        DynamicObjectCollection objects = model.getEntryEntity(ENTRY_FIRST);
        if (!YEM.isEmpty(objects)) {
            model.deleteEntryData(ENTRY_FIRST);//清空分录
        }
        if (entryList.size() > 0) {
            int i = 0;
            int[] ints = this.getModel().batchCreateNewEntryRow(ENTRY_FIRST, entryList.size());
            IDataModel object = this.getModel();
            for (HashMap<String, Object> value : entryList) {
                object.setValue("tpv_mark", value.get("tpv_mark"), ints[i]);//组合号
                object.setValue("tpv_groupnum", value.get("tpv_groupnum"), ints[i]);//组合号
                object.setValue("tpv_diversion", value.get("tpv_diversion"), ints[i]);//转采
                object.setValue("tpv_resell", value.get("tpv_resell"), ints[i]);//转销
                object.setValue("tpv_sourcearea", value.get("tpv_sourcearea"), ints[i]);//原产地
                object.setValue("tpv_originpurchase", value.get("tpv_originpurchase"), ints[i]);//原始采购商
                object.setValue("tpv_finalcustomer", value.get("tpv_finalcustomer"), ints[i]);//最终客户
                object.setValue("tpv_priceclause", value.get("tpv_priceclause"), ints[i]);//价格条款
                object.setValue("tpv_commodity", value.get("tpv_commodity"), ints[i]);//商品
                object.setValue("tpv_count", value.get("tpv_count"), ints[i]);//数量
                object.setValue("tpv_currency", value.get("tpv_currency"), ints[i]);//币种
                object.setValue("tpv_repertorynum", value.get("tpv_repertorynum"), ints[i]);//累计入库数
                object.setValue("tpv_allbillnum", value.get("tpv_allbillnum"), ints[i]);//累计开单数
                object.setValue("tpv_residuenum", value.get("tpv_residuenum"), ints[i]);//剩余数量
                object.setValue("tpv_patbackitem", value.get("tpv_patbackitem"), ints[i]);//回款期条款
                object.setValue("tpv_lastpayback", value.get("tpv_lastpayback"), ints[i]);//最迟回款期
                object.setValue("tpv_allrateamount", value.get("tpv_allrateamount"), ints[i]);//累计收汇金额
                object.setValue("tpv_ifamountok", value.get("tpv_ifamountok"), ints[i]);//是否款齐
                object.setValue("tpv_documentwork", value.get("tpv_documentwork"), ints[i]);//跟单内勤
                object.setValue("tpv_shipover", value.get("tpv_shipover"), ints[i]);//装运完结
                object.setValue("tpv_islate", value.get("tpv_islate"), ints[i]);//是否逾期
                object.setValue("tpv_groupnumber", value.get("tpv_groupnumber"), ints[i]);//组合号编码(隐藏)
                i++;
            }
            //更新单据体
            this.getView().updateView(ENTRY_FIRST);
        } else {
            this.getView().showTipNotification("转口物流工作台暂无数据!");
        }
    }

    /**
     * 单据打开初始化界面
     *
     * @param index
     */
    private void openContainerFrom(int index) {
        DynamicObjectCollection entity = this.getModel().getEntryEntity(ENTRY_FIRST);
        String groupNum = entity.get(index).getString("tpv_groupnumber");//获取组合号
        BillShowParameter showParameter = new BillShowParameter();
        showParameter.setStatus(OperationStatus.EDIT);
        showParameter.getOpenStyle().setShowType(ShowType.InContainer);
        showParameter.getOpenStyle().setTargetKey("tpv_flexpanelap1");
        showParameter.setSendToClient(true);
        showParameter.setFormId("tpv_zkwlgztfl");
        showParameter.setCustomParam("groupnumber", groupNum);
        DynamicObject loadSingle = BusinessDataServiceHelper.loadSingle("tpv_bd_combination", "id",
                new QFilter[]{new QFilter("number", QCP.equals, groupNum)});
        showParameter.setPkId(loadSingle.getPkValue());
        this.getView().showForm(showParameter);
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String key = evt.getItemKey();
        switch (key) {
            case "tpv_addmark":
                if (isSelect()) {
                    addMark(); //判断是否有选中数据
                }
                break;
            case "tpv_query"://查询
                query();
                bindData();
                break;
            default:
                break;
        }
    }

    /**
     * 查询数据
     */
    private void query() {
        ArrayList<String> bustype = new ArrayList<>();//业务类型
        ArrayList<String> combinationNum = new ArrayList<>();//组合号

        //业务类型
        DynamicObject[] bustypes = BusinessDataServiceHelper.load("tpv_bd_bustypes", "name", new QFilter[]{new QFilter("group.name", QCP.equals, "纯转口"),//中航,纯转口
                new QFilter("status", QCP.equals, "C")});
        for (DynamicObject dynamicObject : bustypes) {
            bustype.add(dynamicObject.getString("name"));
        }
        //组合号
        DynamicObject[] combinations = BusinessDataServiceHelper.load("tpv_bd_combination", "tpv_remarkree,name,number", new QFilter[]{new QFilter("tpv_businesstype.name", QCP.in, bustype)});

        for (DynamicObject object : combinations) {
            String number = object.getString("number");
            combinationNum.add(number);//组合号编码
        }
        //采购合同过滤字段
        String purchaseFiled = "billno,tpv_combinationnumber,tpv_contractnumber,tpv_tradeproperty.name,tpv_countryoforigin.name,tpv_supplier,tpv_supplier.tpv_sfjtnbqy,tpv_documentarys.name,tpv_priceterms.name,tpv_currency.name,tpv_dt_purchase_yq,tpv_overduedefinition,tpv_paymentstatus,tpv_whetheitisshipped,tpv_bizdate,tpv_durationofthecontr,tpv_materialentry,tpv_prodname,tpv_qty";
        //销售合同过滤字段
        String saleFiled = "billno,tpv_combinationnumber,tpv_customer.name,tpv_customer.tpv_enterprise,tpv_contractnos,tpv_paymentstatus,tpv_tradeproperty.name,tpv_overduedefinition.name,tpv_dt_salescontract_y,tpv_bizdate,tpv_contractnos";
        //采购合同
        DynamicObject[] purchases = BusinessDataServiceHelper.load("tpv_dt_purchase", purchaseFiled,
                new QFilter[]{new QFilter("tpv_combinationnumber.number", QCP.in, combinationNum),
                        new QFilter("billstatus", QCP.equals, "C")});
        //销售合同
        DynamicObject[] sale = BusinessDataServiceHelper.load("tpv_dt_salescontract", saleFiled,
                new QFilter[]{new QFilter("tpv_combinationnumber.number", QCP.in, combinationNum),
                        new QFilter("billstatus", QCP.equals, "C")});

        for (DynamicObject object : combinations) {
            HashMap<String, Object> map = new HashMap<>();//单据体赋值
            String num = object.getString("number");
            String combination = object.getString("name");//组合号名称
            List<DynamicObject> purCollected = Arrays.stream(purchases)
                    .filter(obj -> num.equals(obj.getString("tpv_combinationnumber.number")))
                    .collect(Collectors.toList());
            List<DynamicObject> saleCollected = Arrays.stream(sale)
                    .filter(obj -> num.equals(obj.getString("tpv_combinationnumber.number")))
                    .collect(Collectors.toList());
            //采购合同
            DynamicObject[] purchaseContract = new DynamicObject[purCollected.size()];
            purCollected.toArray(purchaseContract);
            //销售合同
            DynamicObject[] saleContract = new DynamicObject[saleCollected.size()];
            saleCollected.toArray(saleContract);

            map.put("tpv_groupnum", combination);//组合号
            map.put("tpv_groupnumber", num);//组合号编码
            map.put("tpv_mark", object.getString("tpv_remarkree"));//获取备注
            map.put("tpv_diversion", diversion(purchaseContract));//转采 Y
            map.put("tpv_resell", resell(saleContract));//转销 Y
            map.put("tpv_sourcearea", sourceArea(purchaseContract));//原产地 Y
            map.put("tpv_originpurchase", originPurchase(purchaseContract));//原采购商 Y
            map.put("tpv_priceclause", priceClause(purchaseContract));//价格条款 Y
            map.put("tpv_currency", currency(purchaseContract));//币别 Y
            map.put("tpv_finalcustomer", finalCustomer(saleContract));//最终客户
            map.put("tpv_commodity", commodity(purchaseContract));//商品 Y
            map.put("tpv_count", count(purchaseContract));//数量 Y
            BigDecimal reNum = (BigDecimal) repertoryNum(purchaseContract);
            map.put("tpv_repertorynum", reNum);//累计入库数
            BigDecimal allNum = (BigDecimal) allBillNum(saleContract);
            map.put("tpv_allbillnum", allNum);//累计开单数
            map.put("tpv_residuenum", BigDecimalUtils.sub(reNum, allNum, 2));//剩余数量(累计入库数-累计开单数)
            map.put("tpv_patbackitem", patBackItem(saleContract));//回款期条款 Y
            map.put("tpv_lastpayback", lastPayBack(saleContract));//最迟回款期 Y
            map.put("tpv_allrateamount", allRateAmount(saleContract));//累计收汇金额
            map.put("tpv_ifamountok", ifAmountOk(saleContract));//是否款齐
            map.put("tpv_documentwork", documentWork(purchaseContract));//跟单内勤
            map.put("tpv_shipover", shipOver(purchaseContract));//装运完结
            map.put("tpv_islate", isLate(purchaseContract));//是否逾期
            entryList.add(map);
        }
    }

    /**
     * 是否逾期
     * TODO 是否逾期
     *
     * @param purchases
     */
    private Object isLate(DynamicObject[] purchases) {
        return null;
    }

    /**
     * 装运完结
     *
     * @param purchases
     */
    private Object shipOver(DynamicObject[] purchases) {
        int isFlag = 0;
        for (DynamicObject whetheitisshipped : purchases) {
            DynamicObject supplier = whetheitisshipped.getDynamicObject("tpv_supplier");  //供应商
            if (supplier != null) {
                boolean sfjtnbqy = supplier.getBoolean("tpv_sfjtnbqy");
                if (!sfjtnbqy) {
                    boolean aBoolean = whetheitisshipped.getBoolean("tpv_whetheitisshipped");  //是否装运齐
                    if (!aBoolean) {
                        isFlag++;
                    }
                }
            }
        }
        if (isFlag == 0) {
            return "A";
        } else {
            return "B";
        }

    }

    /**
     * 跟单内勤
     *
     * @param purchases
     */
    private Object documentWork(DynamicObject[] purchases) {
        StringBuilder baseData = new StringBuilder();
        for (DynamicObject purchase : purchases) {
            String document = purchase.getString("tpv_documentarys.name");
            if (document != null) {
                baseData.append(document).append(",");
            }
        }
        return clearComma(baseData);
    }

    /**
     * 是否款齐
     *
     * @param sale
     */
    private Object ifAmountOk(DynamicObject[] sale) {
        boolean isFlag = true;
        for (DynamicObject isMoney : sale) {
            DynamicObject customer = isMoney.getDynamicObject("tpv_customer");  //客户
            if (customer != null) {
                boolean tpv_enterprise = customer.getBoolean("tpv_enterprise");
                if (!tpv_enterprise) {
                    String paymentStatus = isMoney.getString("tpv_paymentstatus");
                    if ("A".equals(paymentStatus)) {
                        isFlag = false;
                        break;
                    }
                }
            }
        }
        return isFlag;
    }

    /**
     * 累计收汇金额
     *
     * @param sale
     */
    private Object allRateAmount(DynamicObject[] sale) {
        //TODO
        return null;
    }

    /**
     * 最迟回款期
     *
     * @param sale
     */
    private Object lastPayBack(DynamicObject[] sale) {
        StringBuilder lastPay = new StringBuilder();
        for (DynamicObject last : sale) {
            String contractNumber = last.getString("tpv_contractnos");
            Date bizdate = last.getDate("tpv_bizdate");//TODO 按《销售合同》中【逾期定义】进行计算；
            String lastTime = contractNumber + ": " + DateUtil.formatDate(bizdate);
            lastPay.append(lastTime).append(",");
        }
        return clearComma(lastPay);
    }

    /**
     * 回款期条款
     *
     * @param sale
     */
    private Object patBackItem(DynamicObject[] sale) {
        StringBuilder patBackItemStr = new StringBuilder();
        for (DynamicObject cumSingular : sale) {
            DynamicObject customer = cumSingular.getDynamicObject("tpv_customer");  //客户
            if (customer != null) {
                boolean tpv_enterprise = customer.getBoolean("tpv_enterprise");
                if (!tpv_enterprise) {
                    DynamicObjectCollection entries = cumSingular.getDynamicObjectCollection("tpv_dt_salescontract_y");
                    for (DynamicObject entity : entries) {
                        String overdueDefinition = entity.getString("tpv_overduedefinition.name");
                        patBackItemStr.append(overdueDefinition).append(",");
                    }
                }
            }
        }
        return clearComma(patBackItemStr);
    }

    /**
     * 累计开单数
     */
    private Object allBillNum(DynamicObject[] sale) {
        BigDecimal sunnum = BigDecimal.ZERO;//总数量
        List<String> srcBillNo = new ArrayList<>();

        for (int j = 0; j < sale.length; j++) {
            DynamicObject customer = sale[j].getDynamicObject("tpv_customer");  //客户
            if (customer != null) {
                boolean tpv_enterprise = customer.getBoolean("tpv_enterprise");
                if (!tpv_enterprise) {
                    String id = sale[j].getString("billno");
                    srcBillNo.add(id);
                }
            }
        }

        //销售发货
        DynamicObject[] salesShipping = BusinessDataServiceHelper.load("tpv_dt_salesshipping", "tpv_materialentry,tpv_qty,tpv_customer",
                new QFilter[]{new QFilter("tpv_materialentry.tpv_srcbillno", QCP.in, srcBillNo)});
        for (DynamicObject sales : salesShipping) {
            DynamicObjectCollection materialEntry = sales.getDynamicObjectCollection("tpv_materialentry");
            for (DynamicObject mater : materialEntry) {
                sunnum = sunnum.add(mater.getBigDecimal("tpv_qty"));//数量
            }
        }
        return sunnum;
    }

    /**
     * 累计入库数
     */
    private Object repertoryNum(DynamicObject[] purchases) {
        BigDecimal sunnum = BigDecimal.ZERO;//总数量
        List<String> srcBillNo = new ArrayList<>();
        List<Object> list = new ArrayList<>();
        for (DynamicObject purchase : purchases) {
            DynamicObject supplier = purchase.getDynamicObject("tpv_supplier");  //供应商
            if (supplier != null) {
                boolean sfjtnbqy = supplier.getBoolean("tpv_sfjtnbqy");
                if (!sfjtnbqy) {
                    Object id = purchase.get("billno");
                    list.add(id);
                }
            }
        }
        //货物流转单
        DynamicObject[] freightRoamBill = BusinessDataServiceHelper.load("tpv_dt_freigroambill", "billno,tpv_materialentry,tpv_srcbillno", new QFilter[]{new QFilter("tpv_materialentry.tpv_srcbillno", QCP.in, list)});
        for (DynamicObject sale : freightRoamBill) {
            String tpvSrcBillNo = sale.getString("billno");//
            srcBillNo.add(tpvSrcBillNo);
        }

        //采购入库单
        DynamicObject[] sales = BusinessDataServiceHelper.load("im_purinbill", "billentry,srcbillnumber,qty", new QFilter[]{new QFilter("billentry.srcbillnumber", QCP.in, srcBillNo)});
        for (DynamicObject sale : sales) {
            DynamicObjectCollection materialEntry = sale.getDynamicObjectCollection("billentry");
            if (materialEntry.size() != 0) {
                for (DynamicObject mater : materialEntry) {
                    sunnum = sunnum.add(mater.getBigDecimal("qty"));//数量
                }
            }
        }
        return sunnum;
    }

    /**
     * 数量
     *
     * @param purchases
     */
    private Object count(DynamicObject[] purchases) {
        BigDecimal sum = BigDecimal.ZERO;//总数量
        for (DynamicObject amount : purchases) {
            DynamicObject supplier = amount.getDynamicObject("tpv_supplier");  //供应商
            if (supplier != null) {
                boolean tpvSfjtnbqy = supplier.getBoolean("tpv_sfjtnbqy");
                if (!tpvSfjtnbqy) {
                    DynamicObjectCollection entrys = amount.getDynamicObjectCollection("tpv_materialentry");
                    for (DynamicObject entity : entrys) {
                        sum = sum.add(entity.getBigDecimal("tpv_qty"));
                    }
                }
            }
        }
        return sum;
    }

    /**
     * 商品
     *
     * @return
     */
    private Object commodity(DynamicObject[] purchases) {
        StringBuilder commodity = new StringBuilder();
        for (DynamicObject purchase : purchases) {
            DynamicObject supplier = purchase.getDynamicObject("tpv_supplier");  //供应商
            if (supplier != null) {
                boolean tpvSfjtnbqy = supplier.getBoolean("tpv_sfjtnbqy");
                if (!tpvSfjtnbqy) {
                    DynamicObjectCollection collection = purchase.getDynamicObjectCollection("tpv_materialentry");
                    for (DynamicObject object : collection) {
                        commodity.append(object.getString("tpv_prodname.group.name")).append(",");
                    }
                }
            }
        }
        return clearComma(commodity);
    }

    /**
     * 最终客户
     *
     * @param sale
     */
    private Object finalCustomer(DynamicObject[] sale) {
        StringBuilder customers = new StringBuilder();
        for (DynamicObject bill : sale) {
            DynamicObject customer = bill.getDynamicObject("tpv_customer");  //客户
            if (customer != null) {
                boolean tpv_enterprise = customer.getBoolean("tpv_enterprise");
                if (!tpv_enterprise) {
                    customers.append(bill.getString("tpv_customer.name")).append(",");
                }
            }
        }
        return clearComma(customers);
    }

    /**
     * 币别
     *
     * @param purchases
     */
    private Object currency(DynamicObject[] purchases) {
        StringBuilder currencyName = new StringBuilder();
        for (DynamicObject purchase : purchases) {
            DynamicObject supplier = purchase.getDynamicObject("tpv_supplier");  //供应商
            String currency = purchase.getString("tpv_currency.name");  //币别
            if (supplier != null) {
                boolean tpvSfjtnbqy = supplier.getBoolean("tpv_sfjtnbqy");
                if (!tpvSfjtnbqy) {
                    if (currency != null) {
                        currencyName.append(currency).append(",");
                    }

                }
            }
        }
        return clearComma(currencyName);
    }

    /**
     * 价格条款
     *
     * @param purchases
     */
    private Object priceClause(DynamicObject[] purchases) {
        StringBuilder priceTermsName = new StringBuilder();
        for (DynamicObject purchase : purchases) {
            DynamicObject supplier = purchase.getDynamicObject("tpv_supplier");  //供应商
            String priceTerms = purchase.getString("tpv_priceterms.name");  //价格条款
            if (supplier != null) {
                boolean tpvSfjtnbqy = supplier.getBoolean("tpv_sfjtnbqy");
                if (!tpvSfjtnbqy) {
                    if (priceTerms != null) {
                        priceTermsName.append(priceTerms).append(",");
                    }
                }
            }
        }
        return clearComma(priceTermsName);
    }

    /**
     * 原始供应商
     *
     * @param purchases
     */
    private Object originPurchase(DynamicObject[] purchases) {
        StringBuilder supplierName = new StringBuilder();
        for (DynamicObject purchase : purchases) {
            DynamicObject supplier = purchase.getDynamicObject("tpv_supplier");  //供应商
            if (supplier != null) {
                String sName = supplier.getString("name");
                boolean tpvSfjtnbqy = supplier.getBoolean("tpv_sfjtnbqy");
                if (!tpvSfjtnbqy) {
                    if (sName != null) {
                        supplierName.append(sName).append(",");
                    }
                }
            }
        }
        return clearComma(supplierName);
    }

    /**
     * 原产地
     */
    private Object sourceArea(DynamicObject[] purchases) {
        StringBuilder baseData = new StringBuilder();
        for (DynamicObject purchase : purchases) {
            String country = purchase.getString("tpv_countryoforigin.name");
            if (country != null) {
                baseData.append(country).append(",");
            }
        }
        return clearComma(baseData);
    }

    /**
     * 转销
     *
     * @param sale
     */
    private Object resell(DynamicObject[] sale) {
        StringBuilder baseData = new StringBuilder();
        for (DynamicObject object : sale) {

            DynamicObject tradeProperty = object.getDynamicObject("tpv_tradeproperty");
            String tname = "";
            if (tradeProperty != null) {
                tname = tradeProperty.getString("name");
            }
            String contractNumber = object.getString("tpv_contractnos");
            if (tname != null) {
                if ("转口".equals(tname)) {
                    baseData.append(contractNumber).append(",");
                }
            }
        }
        return clearComma(baseData);
    }

    /**
     * 转采
     */
    private Object diversion(DynamicObject[] purchases) {
        StringBuilder baseData = new StringBuilder();
        for (DynamicObject object : purchases) {
            String name = object.getString("tpv_tradeproperty.name");
            String contractNumber = object.getString("tpv_contractnumber");

            if (name != null) {
                if ("111".equals(name)) {//TODO 转口编码
                    baseData.append(contractNumber).append(",");
                }
            }
        }
        return clearComma(baseData);

    }

    /**
     * 去除字符串最后的,
     *
     * @param builder
     */
    public String clearComma(StringBuilder builder) {
        String string = "";
        if (0 < builder.length()) {
            string = builder.toString();
            string = StringUtils.substring(string, 0, string.length() - 1);
        }
        return string;
    }

    /**
     * 是否已选择数据
     */
    private boolean isSelect() {
        //获取分录选中行
        EntryGrid entryGrid = this.getControl(ENTRY_FIRST);
        int[] selectRows = entryGrid.getSelectRows();
        if (selectRows.length == 0) {
            this.getView().showTipNotification("请先选择一条数据!");
            return false;
        } else {
            entryIndex.put(ENTRY_FIRST, selectRows[0]);
            return true;
        }
    }

    /**
     * 添加备注
     */
    private void addMark() {
        HashMap<String, Object> map = new HashMap<>();
        Integer integer = entryIndex.get(ENTRY_FIRST);
        DynamicObjectCollection entity = this.getModel().getEntryEntity(ENTRY_FIRST);
        String groupNum = entity.get(integer).getString("tpv_groupnumber");//获取组合号
        String beMark = entity.get(integer).getString("tpv_mark");//获去备注
        map.put("index", integer);
        map.put("groupNum", groupNum);
        map.put("beMark", beMark);
        FormShowParameter showParameter = new FormShowParameter();
        showParameter.setStatus(OperationStatus.ADDNEW);
        showParameter.getOpenStyle().setShowType(ShowType.Modal);
        showParameter.setCloseCallBack(new CloseCallBack(this, "mark"));
        showParameter.setCustomParams(map);
        showParameter.setFormId("tpv_wms_marks");
        this.getView().showForm(showParameter);
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
        JSONObject returnData = (JSONObject) closedCallBackEvent.getReturnData();
        if (StringUtils.equals(closedCallBackEvent.getActionId(), "mark") && null != returnData) {
            this.getModel().setValue("tpv_mark", returnData.get("mark"), (Integer) returnData.get("index"));
            //保存数据到数据库
            DynamicObject reMarker = BusinessDataServiceHelper.loadSingle("tpv_bd_combination", "tpv_remarkree",
                    new QFilter[]{new QFilter("number", QCP.equals, returnData.get("groupNum"))});
            reMarker.set("tpv_remarkree", returnData.get("mark"));
            SaveServiceHelper.update(reMarker);
        }
    }
}
