package tpv.tpv_wm.tpv_dt.java.formplugin.list;

import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.ShowType;
import kd.bos.form.control.events.BeforeItemClickEvent;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.list.BillList;
import kd.bos.list.IListView;
import kd.bos.list.events.ListRowClickEvent;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * @Description: 【回填报关数据】按钮，默认锁定，当【贸易性质】=“出口业务”时解锁，点击弹出当前单据商品明细，上述四个字段必填
 *               点击后根据下推规则生成《延期点价申请单》，允许选择多个《销售发货单》但需要控制是同一个《销售合同》下的
 * @Project: zhck
 * @Package: com.itopview.wm.dt.formplugin.list.BackFillSaleListPlugin
 * @Date: 2023/11/23 16:48
 * @Author: Heisenberg Wenbei He
 * @Version: 1.0
 */
public class BackFillSaleListPlugin extends AbstractListPlugin {
    private static final String BACL_FILL = "tpv_backfill";
    private static final String EXPORT_BUSINESS = "003"; // 出口业务
    private static DynamicObject currentBill;

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        addItemClickListeners(BACL_FILL,"toolbar", "toolbarap");
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        if (evt.getItemKey().equals(BACL_FILL)) {
            popupContractForm();
        }
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        this.getView().setEnable(false, BACL_FILL);
    }

    @Override
    public void listRowClick(ListRowClickEvent evt) {
        super.listRowClick(evt);
        lockButton();
        ListSelectedRow selectedRow = evt.getCurrentListSelectedRow();
        if (selectedRow != null) {
            lockButton();
            getCurrentBill(selectedRow.getPrimaryKeyValue());
        }
        Optional.ofNullable(currentBill)
                .filter(single -> single.get("billstatus").equals("C"))
                .filter(single -> single.getDynamicObject("tpv_tradeproperty") != null)
                .filter(single -> EXPORT_BUSINESS.equals(single.getDynamicObject("tpv_tradeproperty").getString("number")))
                .map(single -> {
                    this.getView().setEnable(true, BACL_FILL);
                    return single;
                })
                .orElseGet(() -> {
                    this.getView().setEnable(false, BACL_FILL);
                    return null;
                });
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
        switch (closedCallBackEvent.getActionId()) {
            case "data":
                this.returnDataCarryInfo(closedCallBackEvent);
                break;
            default:
                break;
        }
    }

    /**
     * @param closedCallBackEvent: 回调事件
     * @author: Heisenberg
     * @date: 2023/11/24 16:30
     * @description: 回填数据
     */
    private void returnDataCarryInfo(ClosedCallBackEvent closedCallBackEvent) {
        if (closedCallBackEvent.getReturnData() instanceof HashMap) {
            HashMap<String, Object> map = (HashMap<String, Object>) closedCallBackEvent.getReturnData();
            currentBill.set("tpv_customnum", map.get("tpv_customnum"));
            DynamicObjectCollection data = (DynamicObjectCollection) map.get("dataEntity");
            DynamicObjectCollection collection = currentBill.getDynamicObjectCollection("tpv_materialentry");
            Optional.ofNullable(data)
                    .ifPresent(array -> IntStream.range(0, array.size())
                            .forEach(i -> {
                                collection.get(i).set("tpv_fobprice", array.get(i).getString("tpv_fobprice"));
                                collection.get(i).set("tpv_fobamount", array.get(i).getString("tpv_fobamount"));
                                collection.get(i).set("tpv_customname", array.get(i).getString("tpv_customname"));
                            }));
            SaveServiceHelper.save(new DynamicObject[]{currentBill});
        }
    }

    /**
     * @return List<Long>
     * @author: Heisenberg
     * @date: 2023/11/23 19:08
     * @description: 已选中的单据列表
     */
    private List<Long> getListBillId() {
        IListView view = (IListView) this.getView();
        ListSelectedRowCollection selectedRows = view.getSelectedRows();// 查询列表所勾选的数据
        return selectedRows.stream()
                .map(listSelectedRow -> (Long) listSelectedRow.getPrimaryKeyValue())
                .distinct()
                .collect(Collectors.toList());
    }


    /**
     * @author: Heisenberg
     * @date: 2023/11/24 16:30
     * @description: 弹出商品明细分录动态表单
     */
    private void popupContractForm() {
        FormShowParameter parameter = new FormShowParameter();
        parameter.setFormId("tpv_productdetails");
        parameter.setCustomParams(this.getContractDataForm());
        parameter.setStatus(OperationStatus.ADDNEW);
        parameter.setCloseCallBack(new CloseCallBack(this, "data"));
        parameter.getOpenStyle().setShowType(ShowType.Modal);
        this.getView().showForm(parameter);
    }

    private Map<String, Object> getContractDataForm() {
        HashMap<String, Object> map = new HashMap<>();
        map.put("pkValue", currentBill.getPkValue());
        return map;
    }

    /**
     * @param primaryKeyValue: 单据主键
     * @author: Heisenberg
     * @date: 2023/11/24 16:31
     * @description: 获取当前单
     */
    private static void getCurrentBill(Object primaryKeyValue) {
        currentBill = BusinessDataServiceHelper.loadSingle(primaryKeyValue, "tpv_dt_salesshipping");
    }

    /**
     * @author: Heisenberg
     * @date: 2023/11/24 16:31
     * @description: 锁定【回填报关数据】按钮
     */
    private void lockButton() {
        List<Long> billId = this.getListBillId();
        if (billId.size() != 1) {
            this.getView().setEnable(false, BACL_FILL);
        }
    }

    @Override
    public void beforeItemClick(BeforeItemClickEvent evt) {
        super.beforeItemClick(evt);
        String key = evt.getItemKey();
        switch (key) {
            case "tpv_delaypri":
                push(evt);
                break;
        }
    }

    private void push(BeforeItemClickEvent evt) {
        BillList billList = this.getControl("billlistap");
        ListSelectedRowCollection selectedRows = billList.getSelectedRows();
        if (selectedRows != null && selectedRows.size() > 0) {
            List<DynamicObject> dynamicObjects = new ArrayList<>();
            for (ListSelectedRow tempData : selectedRows) {
                DynamicObject dynamic = BusinessDataServiceHelper.loadSingle(tempData.getPrimaryKeyValue(), "tpv_dt_salesshipping");
                dynamicObjects.add(dynamic);
            }
            List<String> list = new ArrayList<>();
            for (DynamicObject pur : dynamicObjects) {
                String tpvContractnumber = pur.getString("tpv_contractnumber");
                if (!list.contains(tpvContractnumber)){
                    list.add(tpvContractnumber);
                }
            }
            if (list.size()>1){
                this.getView().showTipNotification("需选择为同一张销售合同下的销售发货单进行下推！");
                evt.setCancel(true);
                return;
            }
        }
    }
}
