package tpv.tpv_wm.tpv_dt.java.formplugin.list;

import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.ShowType;
import kd.bos.form.control.events.BeforeItemClickEvent;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.list.IListView;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi.ShipNewsClientHelper;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;


/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhkc
 * @Package: com.itopview.wm.dt.formplugin.list.QueryShipListPlugin
 * @Date: 16:14 2023/09/11
 * @Description: 查询船舶
 * @Version: 7.0
 */
public class QueryShipinfoListPlugin extends AbstractListPlugin {
    private String formsign = "";    // 动态表单标识，根据不同的按钮选择不同的动态表单
    private Map<String, Object> keymap = new HashMap<>();    // 作为列表和动态表单间传递信息的媒介

    @Override
    public void beforeItemClick(BeforeItemClickEvent evt) {
        super.beforeItemClick(evt);
        String key = evt.getItemKey();
        if ("tpv_querytrack".equals(key)) { // 轨迹查询按钮
            List<Long> billIds = ShipNewsClientHelper.getListBillId((IListView) this.getView()); // 被选中的单据id集
            if (billIds.size() != 1) {
                evt.setCancel(true);
                this.getView().showErrorNotification("请选择一条数据进行查询!");
            }
        }
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String key = evt.getItemKey();
        keymap.put("key", key);
        switch (key) {
            case "tpv_searchship":      // 单船查询
            case "tpv_querymanyships":  // 多船查询
            case "tpv_querysingleship": // 如果点击的是这些按钮，弹出动态表单获取id或进行查询
                formsign = "tpv_dt_queryship";
                this.importForm();
                break;
            case "tpv_querytrack":
                DynamicObject ship = BusinessDataServiceHelper.loadSingle(ShipNewsClientHelper.getListBillId((IListView) this.getView()).get(0),
                        "tpv_wms_gzcqd", "tpv_shipname,tpv_shipid"); // 加载被选中的单据
                formsign = "tpv_dt_tracktime";
                keymap.put("tpv_shipname", ship.get("tpv_shipname")); // 获取船舶名称灰显在动态表单中
                keymap.put("tpv_shipid", ship.get("tpv_shipid")); // 获取船舶 id 带入到动态表单
                this.importForm(); // 弹出带有日期的动态表单
                break;
            default:
                break;
        }
    }

    /**
     * 根据点击的按钮弹出不同的动态表单弹出不同的
     */
    private void importForm() {
        FormShowParameter parameter = new FormShowParameter();  // 新建动态表单
        parameter.setFormId(formsign);
        parameter.setStatus(OperationStatus.ADDNEW);
        parameter.getOpenStyle().setShowType(ShowType.Modal);
        parameter.setCloseCallBack(new CloseCallBack(this, formsign));
        parameter.setCustomParams(keymap);
        this.getView().showForm(parameter);

    }

    /**
     * 回调函数
     */
    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        super.closedCallBack(closedCallBackEvent);
        switch (closedCallBackEvent.getActionId()) {
            case "tpv_dt_queryship":    // 返回的标识为查询船舶
                this.createNewShipForm(closedCallBackEvent);
                break;
            case "tpv_dt_tracktime":    // 返回的标识为船舶轨迹查询
                this.updateShipTrack(closedCallBackEvent);
                break;
            default:
                break;
        }
    }

    /**
     * 船舶查询时生成新的单据
     */
    private void createNewShipForm(ClosedCallBackEvent closedCallBackEvent) {
        Optional<Object> optionalReturndata = Optional.ofNullable(closedCallBackEvent.getReturnData());
        optionalReturndata.filter(returnData -> !(returnData instanceof String)) // 如果回调值为String类型，则为错误提示信息
                .map(returnData -> (List<ShipNewsClientHelper.ShipDetail>) returnData)
                .ifPresent(returnData -> {
                    String repeatIds = returnData.stream()
                            .map(ShipNewsClientHelper.ShipDetail::getShipID)
                            .filter(ShipNewsClientHelper::isExistShipId) // 判断列表是否存在当前船舶
                            .collect(Collectors.joining(",")); // 将存在的船舶 id 拼接字符串
                    if (!repeatIds.isEmpty()) {
                        this.getView().showTipNotification("船舶编号为" + repeatIds + "已存在");
                    } else {
                        this.getView().showSuccessNotification("添加成功！");
                    }
                    ShipNewsClientHelper.processShipFormList(returnData); // 新建关注船清单单据
                    ((IListView) this.getView()).refresh();
                });
        optionalReturndata.filter(String.class::isInstance)
                .map(String.class::cast)
                .ifPresent(returnData -> this.getView().showTipNotification(returnData));

    }


    /**
     * 查询船舶轨迹后返回的节点数据，待处理
     */
    private void updateShipTrack(ClosedCallBackEvent closedCallBackEvent) {
        // TODO 等待处理船舶轨迹
        Optional<Object> optionalReturnData = Optional.ofNullable(closedCallBackEvent.getReturnData());
        if (!(closedCallBackEvent.getReturnData() instanceof String)) {
            this.getView().showSuccessNotification("船舶轨迹查询成功！");
        } else {
            String returnData = (String) closedCallBackEvent.getReturnData();
            this.getView().showTipNotification(returnData);
        }
    }
}
