package tpv.tpv_wm.tpv_dt.java.opplugin.convert;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.botp.plugin.AbstractConvertPlugIn;
import kd.bos.entity.botp.plugin.args.AfterConvertEventArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.Arrays;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicReference;

/**
 * @Description: 《销售合同》下推《销售发货》若付款方式为信用证且关联信用证只有一条，自动带出
 * @Project: zhck
 * @Package: com.itopview.wm.dt.opplugin.convert.LcCreditCarryCreditNumConvert
 * @Date: 2023/12/09 16:49
 * @Author: Heisenberg Wenbei He
 * @Version: 1.0
 */
public class LcCreditCarryCreditNumConvert extends AbstractConvertPlugIn {
    private static final String PAYMENT_WAYS = "tpv_paymentmethods"; // {收款计划} 分录 instance of DynamicObjectCollection
    private static final String PAYMENT_TYPE = "tpv_paymenttype"; // {收款计划}【收款方式】instance of Base
    private static final String LC = "tpv_lc"; // {收款计划}【信用证号】 分录 instance of Bill
    private static final String CONTRACT_NUM = "tpv_contractnumber"; // 合同号 instance of text
    private static final String LC_RECEIPT = "lc_receipt"; // 《收证处理》
    private static final String INTERAL = "JSFS12"; // 国内信用证
    private static final String GLOBAL = "JSFS13"; // 国际信用证
    private static boolean isGlobal; // 是否为国际证

    @Override
    public void afterConvert(AfterConvertEventArgs e) {
        super.afterConvert(e);
        String target = this.getTgtMainType().getName();
        ExtendedDataEntity[] billDataEntity = e.getTargetExtDataEntitySet().FindByEntityKey(target);
        this.initlize(billDataEntity);
    }

    private void initlize(ExtendedDataEntity[] billDataEntity) {
        for (ExtendedDataEntity entity : billDataEntity) {
            DynamicObject dataEntity = entity.getDataEntity();
            String tpvContractnumber = dataEntity.getString("tpv_contractnumber");
            DynamicObjectCollection tpvPaymentmethods = dataEntity.getDynamicObjectCollection("tpv_paymentmethods");
            for (DynamicObject tpvPaymentmethod : tpvPaymentmethods) {
                DynamicObject tpvPaymenttype = tpvPaymentmethod.getDynamicObject("tpv_paymenttype");
                if (tpvPaymenttype != null) {
                    String number = tpvPaymenttype.getString("number");
                    boolean isCredit = (INTERAL.equals(number) || GLOBAL.equals(number)); // 【收款方式】是否为信用证
                    if (isCredit) {
                        isGlobal(number);
                        DynamicObject[] creditReceipts = BusinessDataServiceHelper.load(LC_RECEIPT,
                                "id,creditno,billstatus,entrys,tpv_salecontract,isnationalcard,tpv_name",
                                new QFilter[]{new QFilter("entrys.tpv_salecontract.tpv_name", QCP.equals, tpvContractnumber),
                                        new QFilter("isnationalcard", QCP.equals, isGlobal),
                                        new QFilter("billstatus", QCP.equals, "C"),
                                        new QFilter("tpv_claimnoticebillno", QCP.not_equals, "")}); // 收款信用证
                        if (creditReceipts.length != 0 &&creditReceipts.length == 1 ) {
                            tpvPaymentmethod.set("tpv_lc", creditReceipts[0]);//采购合同号
                            tpvPaymentmethod.set("tpv_lcf", BusinessDataServiceHelper.loadSingle("lc_receipt_f7",new QFilter[]{new QFilter("number", QCP.equals, creditReceipts[0].getString("creditno"))}));
                        }
                    }
                }
            }
        }
//        AtomicReference<String> contractNumber = new AtomicReference<>();
//        Optional.ofNullable(billDataEntity)
//                .filter(entity -> entity.length > 0)
//                .ifPresent(array -> Arrays.stream(array).map(single -> {
//                    contractNumber.set((String) single.getValue(CONTRACT_NUM)); // 《销售发货》合同号
//                    DynamicObject entity = single.getDataEntity();
//                    return entity.getDynamicObjectCollection(PAYMENT_WAYS);
//                }).forEach(each -> each.forEach(alone -> {
//                    DynamicObject tpvPaymenttype = alone.getDynamicObject(PAYMENT_TYPE);
//                    if (tpvPaymenttype != null) {
//                        String number = tpvPaymenttype.getString("number");
//                        boolean isCredit = (INTERAL.equals(number) || GLOBAL.equals(number)); // 【收款方式】是否为信用证
//                        if (isCredit) {
//                            isGlobal(number);
//                            DynamicObject[] creditReceipts = BusinessDataServiceHelper.load(LC_RECEIPT,
//                                    "id,creditno,billstatus,entrys,tpv_salecontract,isnationalcard,tpv_name",
//                                    new QFilter[]{new QFilter("tpv_salecontract.tpv_name", QCP.equals, contractNumber.get()),
//                                            new QFilter("isnationalcard", QCP.equals, isGlobal),
//                                            new QFilter("billstatus", QCP.equals, "C")}); // 收款信用证
//                            Optional.ofNullable(creditReceipts)
//                                    .filter(lonely -> lonely.length == 1)
//                                    .ifPresent(credit -> alone.set(LC, creditReceipts[0].getLong("id")));
//                        }
//                    }
//                }
//                )));
    }

    /**
     * @param number: 收款方式编码
     * @author: Heisenberg
     * @date: 2023/11/24 11:54
     * @description: 判断信用证为国际还是国内，基于此过滤f7信用证
     */
    private static void isGlobal(String number) {
        isGlobal = GLOBAL.equals(number); // true 为国际证，false 为 国内证
    }
}
