package tpv.tpv_wm.tpv_dt.java.opplugin.op;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;

import java.math.BigDecimal;
import java.util.List;

/**
 * @Description: 费用结算单 保存后反写分录行对应的来源单据数据
 * @Author LinRui
 * @Date 2023/9/2 14:51
 */
public class FeeSettleSaveOp extends AbstractOperationServicePlugIn implements Plugin {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_dt_statemenbill_di");
        fieldKeys.add("tpv_srcsubentryid");
        fieldKeys.add("tpv_srcbillid");
        fieldKeys.add("tpv_srcbillkeys");
        fieldKeys.add("tpv_settlenumber");
        fieldKeys.add("tpv_settleamount");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        //e.addValidator(new FeeSettleSaveValidator());  //保存时校验 计费数量不能大于对应来源单据的计费数量
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        if ("save".equals(e.getOperationKey()) && null != e.getDataEntities()) {
            DynamicObject[] bills = e.getDataEntities();
            for (int i = 0; i < bills.length; ++i) {
                DynamicObject bill = bills[i];
                saveBackDataCheck(bill);
            }
        }
    }

    /**
     * @Description: //保存校验，反写分录行对应来源单据数据
     * @Date: 15:02 2023/09/02
     * @param: bill - [kd.bos.dataentity.entity.DynamicObject]
     * @return: void
     **/
    public void saveBackDataCheck(DynamicObject bill) {
        //bill = BusinessDataServiceHelper.loadSingle(bill.getPkValue(), bill.getDynamicObjectType().getName());  //获取当前结算费用单的数据
        DynamicObjectCollection tpvDtStatemenbillDi = bill.getDynamicObjectCollection("tpv_dt_statemenbill_di");
        //遍历费用结算单分录
        for (int i = 0; i < tpvDtStatemenbillDi.size(); i++) {
            DynamicObject entityRowDynamicObject = tpvDtStatemenbillDi.get(i);  //费用结算单分录行数据
            Long tpvSrcbillid = entityRowDynamicObject.getLong("tpv_srcbillid");  //来源单据id
            Long tpvSrcsubentryid = entityRowDynamicObject.getLong("tpv_srcsubentryid"); //来源单据子分录id
            String tpvSrcbillkeys = entityRowDynamicObject.getString("tpv_srcbillkeys"); //来源单据标识
            BigDecimal tpvSettlenumber = entityRowDynamicObject.getBigDecimal("tpv_settlenumber"); //结算数量
            BigDecimal tpvSettleamount = entityRowDynamicObject.getBigDecimal("tpv_settleamount"); //结算金额
            if (0 == tpvSrcbillid || 0 == tpvSrcsubentryid) {
                return;
            }
            DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle(tpvSrcbillid, "" + tpvSrcbillkeys + "");
            DynamicObjectCollection dynamicObjectCollection = null;
            if ("tpv_dt_freigroambill".equals(tpvSrcbillkeys) || "tpv_dt_salesshipping".equals(tpvSrcbillkeys)) {
                dynamicObjectCollection = dynamicObject.getDynamicObjectCollection("tpv_materialentry");
            } else if ("im_purinbill".equals(tpvSrcbillkeys) || "im_otherinbill".equals(tpvSrcbillkeys) || "im_transinbill".equals(tpvSrcbillkeys)
                    || "im_transdirbill".equals(tpvSrcbillkeys)) {
                dynamicObjectCollection = dynamicObject.getDynamicObjectCollection("billentry");
            }

            for (DynamicObject entityData : dynamicObjectCollection) {
                setSrcSubEntityData(entityData, tpvSrcbillkeys, tpvSrcsubentryid, tpvSettlenumber, tpvSettleamount); //获取对应单据子分录数据进行反写赋值
            }
            SaveServiceHelper.save(new DynamicObject[]{dynamicObject});
        }
    }

    /**
     * @Description: //获取对应单据的子分录标识及反写字段
     * @param: tpvSrcbillkeys - [java.lang.String]
     * @param: entityData - [kd.bos.dataentity.entity.DynamicObject]
     * @return: void
     **/
    private void setSrcSubEntityData(DynamicObject entityData, String tpvSrcbillkeys, Long tpvSrcsubentryid, BigDecimal tpvSettlenumber, BigDecimal tpvSettleamount) {
        String srcSubEntitykey = "";
        String tpvIsfeechargeqty = "";
        String tpvIsfeeamt = "";
        switch (tpvSrcbillkeys) {
            case "tpv_dt_freigroambill":
                srcSubEntitykey = "tpv_dt_freigroambill_f";  //子分录标识
                tpvIsfeechargeqty = "tpv_isfeechargeqty";  //已结算数量字段标识
                tpvIsfeeamt = "tpv_isfeeamt";  //已结算金额字段标识
                break;
            case "tpv_dt_salesshipping":
                srcSubEntitykey = "tpv_dt_dcostinform_f";  //子分录标识
                tpvIsfeechargeqty = "tpv_isfeechargeqty";  //已结算数量字段标识
                tpvIsfeeamt = "tpv_isfeeamt";  //已结算金额字段标识
                break;
            case "im_purinbill":
                srcSubEntitykey = "tpv_dt_purinbill_f";  //子分录标识
                tpvIsfeechargeqty = "tpv_isfeechargeqty";  //已结算数量字段标识
                tpvIsfeeamt = "tpv_isfeeamt";  //已结算金额字段标识
                break;
            case "im_otherinbill":
                srcSubEntitykey = "tpv_im_otherinbill_f";  //子分录标识
                tpvIsfeechargeqty = "tpv_isfeechargeqty";  //已结算数量字段标识
                tpvIsfeeamt = "tpv_isfeeamt";  //已结算金额字段标识
                break;
            case "im_transinbill":
                srcSubEntitykey = "tpv_dt_transinbill_f";  //子分录标识
                tpvIsfeechargeqty = "tpv_isfeechargeqty";  //已结算数量字段标识
                tpvIsfeeamt = "tpv_isfeeamt";  //已结算金额字段标识
                break;
            case "im_transdirbill":
                srcSubEntitykey = "tpv_dt_transdirbill_f";  //子分录标识
                tpvIsfeechargeqty = "tpv_isfeechargeqty";  //已结算数量字段标识
                tpvIsfeeamt = "tpv_isfeeamt";  //已结算金额字段标识
                break;
            default:
                break;
        }
        setSrcSubEntity(entityData, tpvSrcsubentryid, srcSubEntitykey, tpvSettlenumber, tpvSettleamount, tpvIsfeechargeqty, tpvIsfeeamt); //获取对应单据进行反写赋值
    }

    /**
     * @Description: //获取对应单据进行反写赋值
     * @param: entityData - [kd.bos.dataentity.entity.DynamicObject]
     * @param: tpvSrcsubentryid - [java.lang.String]
     * @param: srcSubEntitykey - [java.lang.String]
     * @return: void
     **/
    private void setSrcSubEntity(DynamicObject entityData, Long tpvSrcsubentryid, String srcSubEntitykey, BigDecimal tpvSettlenumber, BigDecimal tpvSettleamount, String tpvIsfeechargeqty, String tpvIsfeeamt) {
        DynamicObjectCollection srcsubEntityData = entityData.getDynamicObjectCollection("" + srcSubEntitykey + "");
        for (DynamicObject subEntityData : srcsubEntityData) {
            long tpvDtFreigroambillF = subEntityData.getLong("id");  //来源单据子分录id
            if (tpvDtFreigroambillF > 0) {
                if (tpvDtFreigroambillF == tpvSrcsubentryid) {  //核对费用结算单分录来源单据子分录id和对应单据
                    subEntityData.set("" + tpvIsfeechargeqty + "", tpvSettlenumber); //已结算数量
                    subEntityData.set("" + tpvIsfeeamt + "", tpvSettleamount); //已结算金额
                }
            }
        }
    }


}
