package tpv.tpv_wm.tpv_dt.java.opplugin.op;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 费用发票匹配单反写收票单金额校验 -服务插件
 * @Author LinRui
 * @Date 2023/9/22 19:01
 */
public class FeesInvoiceAmountOp extends AbstractOperationServicePlugIn {

    private static final String opValueKey = "tpv_matchinvoicebtn"; //触发弹框按钮标识
    private static final String entry_key1 = "tpv_invoiceinfoentry"; //发票分录

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_invoiceinfoentry");  //分录标识
        fieldKeys.add("tpv_curmatchamotax");  //本次匹配金额(含税)
        fieldKeys.add("tpv_curmatchamonotax");  //本次匹配金额(不含税)
        fieldKeys.add("tpv_curmatchamointax");  //本次匹配金额(增值税额)
        fieldKeys.add("tpv_sourceid");  //来源单id
        fieldKeys.add("tpv_invocenum");  //发票号
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        e.getOperationKey();
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            paramSalePrice(bill);
        }
    }

    private void paramSalePrice(DynamicObject bill) {
        Object pkValue = bill.getPkValue();
        if (YEM.isNotEmpty(pkValue)) {
            JSONObject jsonObject = new JSONObject();
            if (QueryServiceHelper.exists("tpv_dt_costbillmat", bill.getPkValue())) {
                addPriceBackBill(jsonObject, bill); //记录单据界面入库前的临时数据
            }
            OperateOption option = this.getOption();
            option.setVariableValue(opValueKey, jsonObject.toJSONString());
        }
    }

    private void addPriceBackBill(JSONObject jsonObject, DynamicObject bill) {
        DynamicObject tpvDtCostbillmat = BusinessDataServiceHelper.loadSingle(bill.getPkValue(), "tpv_dt_costbillmat");
        List<Long> entryId = new ArrayList<>();
        DynamicObjectCollection tpvMaterialentry = tpvDtCostbillmat.getDynamicObjectCollection("tpv_invoiceinfoentry");
        for (DynamicObject dynamicObject : tpvMaterialentry) {
            Long aLong = Long.valueOf(dynamicObject.getString("tpv_sourceid"));
            entryId.add(aLong);
            jsonObject.put("billIds", entryId);
        }
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            backSalePrice(bill);  //反写收票单金额
        }
    }

    private void backSalePrice(DynamicObject bill) {
        OperateOption option = this.getOption();
        JSONObject jsonObject = new JSONObject();
        if (option.containsVariable(opValueKey)) {
            jsonObject = JSON.parseObject(option.getVariableValue(opValueKey));
        }
        if (QueryServiceHelper.exists("tpv_dt_costbillmat", bill.getPkValue())) {
            addPriceBackBill(jsonObject, bill);//记录单据界面入库后的数据
        }
        List<String> entryid = new ArrayList<>();
        JSONArray object = jsonObject.getJSONArray("billIds");
        if (null != object) {
            if (object.size() > 0) {
                for (Object o : object) {
                    entryid.add(String.valueOf(o));
                }
            }
            Map<String,Map<String,BigDecimal>> map = new HashMap<>(); //存放所有下游金额汇总，反写到对应收票单
            Map<String,BigDecimal> mapAmount = new HashMap<>();
            if (entryid.size() > 0) {
                QFilter qFilter = new QFilter("tpv_invoiceinfoentry.tpv_sourceid",QCP.in,entryid);
                DynamicObjectCollection query = QueryServiceHelper.query("tpv_dt_costbillmat",
                        "id,billno,tpv_invoiceinfoentry,tpv_invoiceinfoentry.tpv_sourceid tpv_sourceid," +
                                "tpv_invoiceinfoentry.tpv_curmatchamotax tpv_curmatchamotax," + //本次匹配金额(含税)
                                "tpv_invoiceinfoentry.tpv_curmatchamonotax tpv_curmatchamonotax," + //本次匹配金额(不含税)
                                "tpv_invoiceinfoentry.tpv_curmatchamointax tpv_curmatchamointax," + //本次匹配金额(增值税)
                                "tpv_invoiceinfoentry.tpv_unmatchamotax tpv_unmatchamotax," + //未匹配金额(含税)
                                "tpv_invoiceinfoentry.tpv_numatchamonotax tpv_numatchamonotax," + //未匹配金额(不含税)
                                "tpv_invoiceinfoentry.tpv_unmatchamointax tpv_unmatchamointax", qFilter.toArray());
                //数据为空，那么收票单反写金额为零
                if (query.size() > 0) {
                    for (DynamicObject dynamicObject : query) {
                        String tpvSourceid = dynamicObject.getString("tpv_sourceid");
                        BigDecimal tpvCurmatchamotax = dynamicObject.getBigDecimal("tpv_curmatchamotax");
                        BigDecimal tpvCurmatchamonotax = dynamicObject.getBigDecimal("tpv_curmatchamonotax");
                        BigDecimal tpvCurmatchamointax = dynamicObject.getBigDecimal("tpv_curmatchamointax");
                        BigDecimal tpvUnmatchamotax = dynamicObject.getBigDecimal("tpv_unmatchamotax");
                        BigDecimal tpvNumatchamonotax = dynamicObject.getBigDecimal("tpv_numatchamonotax");
                        BigDecimal tpvUnmatchamointax = dynamicObject.getBigDecimal("tpv_unmatchamointax");
                        tpvCurmatchamotax = tpvCurmatchamotax != null ? tpvCurmatchamotax : BigDecimal.ZERO;
                        tpvCurmatchamonotax = tpvCurmatchamonotax != null ? tpvCurmatchamonotax : BigDecimal.ZERO;
                        tpvCurmatchamointax = tpvCurmatchamointax != null ? tpvCurmatchamointax : BigDecimal.ZERO;
                        tpvUnmatchamotax = tpvUnmatchamotax != null ? tpvUnmatchamotax : BigDecimal.ZERO;
                        tpvNumatchamonotax = tpvNumatchamonotax != null ? tpvNumatchamonotax : BigDecimal.ZERO;
                        tpvUnmatchamointax = tpvUnmatchamointax != null ? tpvUnmatchamointax : BigDecimal.ZERO;
                        if (!map.containsKey(tpvSourceid)) {
                            mapAmount.put("tpv_curmatchamotax",tpvCurmatchamotax);
                            mapAmount.put("tpv_curmatchamonotax",tpvCurmatchamonotax);
                            mapAmount.put("tpv_curmatchamointax",tpvCurmatchamointax);
                            mapAmount.put("tpv_unmatchamotax",tpvUnmatchamotax);
                            mapAmount.put("tpv_numatchamonotax",tpvNumatchamonotax);
                            mapAmount.put("tpv_unmatchamointax",tpvUnmatchamointax);
                            map.put(tpvSourceid,mapAmount);
                        } else {
                            Map<String, BigDecimal> bigDecimalMap = map.get(tpvSourceid);
                            BigDecimal curmatchamotax = bigDecimalMap.get("tpv_curmatchamotax");
                            BigDecimal curmatchamonotax = bigDecimalMap.get("tpv_curmatchamonotax");
                            BigDecimal curmatchamointax = bigDecimalMap.get("tpv_curmatchamointax");
                            BigDecimal unmatchamotax = bigDecimalMap.get("tpv_unmatchamotax");
                            BigDecimal numatchamonotax = bigDecimalMap.get("tpv_numatchamonotax");
                            BigDecimal unmatchamointax = bigDecimalMap.get("tpv_unmatchamointax");
                            curmatchamotax = curmatchamotax.add(tpvCurmatchamotax);
                            curmatchamonotax = curmatchamonotax.add(tpvCurmatchamonotax);
                            curmatchamointax = curmatchamointax.add(tpvCurmatchamointax);
                            unmatchamotax = unmatchamotax.add(tpvUnmatchamotax);
                            numatchamonotax = numatchamonotax.add(tpvNumatchamonotax);
                            unmatchamointax = unmatchamointax.add(tpvUnmatchamointax);
                            mapAmount.put("tpv_curmatchamotax",curmatchamotax);
                            mapAmount.put("tpv_curmatchamonotax",curmatchamonotax);
                            mapAmount.put("tpv_curmatchamointax",curmatchamointax);
                            mapAmount.put("tpv_unmatchamotax",unmatchamotax);
                            mapAmount.put("tpv_numatchamonotax",numatchamonotax);
                            mapAmount.put("tpv_unmatchamointax",unmatchamointax);
                            map.put(tpvSourceid,mapAmount);
                        }
                    }
                } else {
                    for (String srcIds : entryid) {
                        mapAmount.put("tpv_curmatchamotax",BigDecimal.ZERO);
                        mapAmount.put("tpv_curmatchamonotax",BigDecimal.ZERO);
                        mapAmount.put("tpv_curmatchamointax",BigDecimal.ZERO);
                        mapAmount.put("tpv_unmatchamotax",BigDecimal.ZERO);
                        mapAmount.put("tpv_numatchamonotax",BigDecimal.ZERO);
                        mapAmount.put("tpv_unmatchamointax",BigDecimal.ZERO);
                        map.put(srcIds,mapAmount);
                    }
                }
            }

            if (map.size() > 0) {
                List<Long> idList = new ArrayList<>();
                map.keySet().stream().forEach(x ->idList.add(Long.valueOf(x)));
                QFilter qFilter = new QFilter("id",QCP.in,idList);
                DynamicObject[] query = BusinessDataServiceHelper.load("ap_invoice",
                        "id,billno,shkd_mat_matchamt,shkd_mat_matchtaxamt,shkd_mat_matchnotaxamt", qFilter.toArray());
                if (query.length > 0) {
                    for (DynamicObject dynamicObject : query) {
                        String aLong = String.valueOf(dynamicObject.getLong("id"));
                        if (map.containsKey(aLong)) {
                            Map<String, BigDecimal> bigDecimalMap = map.get(aLong);
                            dynamicObject.set("shkd_mat_matchamt",bigDecimalMap.get("tpv_curmatchamotax")); //含税
                            dynamicObject.set("shkd_mat_matchtaxamt",bigDecimalMap.get("tpv_curmatchamointax")); //税额
                            dynamicObject.set("shkd_mat_matchnotaxamt",bigDecimalMap.get("tpv_curmatchamonotax")); //不含税
                        }
                    }
                }
                SaveServiceHelper.save(query);
            }
        }

    }
}
