package tpv.tpv_wm.tpv_dt.java.opplugin.op;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 费用发票匹配单反写财务应收单金额校验 -服务插件
 * @Author LinRui
 * @Date 2023/9/22 19:01
 */
public class FeesInvoiceFinalAmountOp extends AbstractOperationServicePlugIn {

    private static final String opValueKey = "tpv_chargeofffinarbtn"; //触发弹框按钮标识

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_costitem");  //费用项目
        fieldKeys.add("tpv_pactmessage");  //分录标识
        fieldKeys.add("tpv_writtenoffamt");  //冲销金额
        fieldKeys.add("tpv_finarbillentryid");  //冲销应收单分录id
        fieldKeys.add("tpv_finarbillid");  //冲销应收单id
        fieldKeys.add("tpv_finarbillno");  //冲销应收单号
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        e.getOperationKey();
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            paramSalePrice(bill);
        }
    }

    private void paramSalePrice(DynamicObject bill) {
        Object pkValue = bill.getPkValue();
        if (YEM.isNotEmpty(pkValue)) {
            JSONArray jsonArray = new JSONArray();
            if (QueryServiceHelper.exists("tpv_dt_costbillmat", bill.getPkValue())) {
                addPriceBackBill(jsonArray, bill); //记录单据界面入库前的临时数据
            }
            OperateOption option = this.getOption();
            option.setVariableValue(opValueKey, jsonArray.toJSONString());
        }
    }

    private void addPriceBackBill(JSONArray jsonArray, DynamicObject bill) {
        DynamicObject tpvDtCostbillmat = BusinessDataServiceHelper.loadSingle(bill.getPkValue(), "tpv_dt_costbillmat");
        DynamicObject tpvCostitem = tpvDtCostbillmat.getDynamicObject("tpv_costitem");
        if (YEM.isNotEmpty(tpvCostitem)) {
            DynamicObjectCollection tpvMaterialentry = tpvDtCostbillmat.getDynamicObjectCollection("tpv_pactmessage");
            for (DynamicObject dynamicObject : tpvMaterialentry) {
                JSONObject jsonObject = new JSONObject();
                Long aLong = Long.valueOf(dynamicObject.getString("tpv_finarbillentryid"));
                jsonObject.put(String.valueOf(aLong), tpvCostitem.getString("number"));
                jsonArray.add(jsonObject);
            }
        }
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            backSalePrice(bill);
        }
    }

    private void backSalePrice(DynamicObject bill) {
        OperateOption option = this.getOption();
        JSONArray jsonArray = new JSONArray();
        if (option.containsVariable(opValueKey)) {
            jsonArray = JSON.parseArray(option.getVariableValue(opValueKey));
        }
        if (QueryServiceHelper.exists("tpv_dt_costbillmat", bill.getPkValue())) {
            addPriceBackBill(jsonArray, bill);//记录单据界面入库后的数据
        }
        List<Long> entryid = new ArrayList<>();
        if (jsonArray.size() > 0) {
            String costitemNumber = "";
            for (Object o : jsonArray) {
                JSONObject parseObject = JSONObject.parseObject(o.toString());
                for (String s : parseObject.keySet()) {
                    entryid.add(Long.valueOf(s));
                    costitemNumber = (String) parseObject.get(s);
                }
            }
            Map<Long, Map<String, String>> map = new HashMap<>(); //存放所有下游金额汇总，并且费用项目要一致，反写到财务对应收票单
            if (entryid.size() > 0) {
                QFilter qFilter = new QFilter("tpv_pactmessage.tpv_finarbillentryid", QCP.in,entryid);
                qFilter.and(new QFilter("tpv_costitem.id", QCP.large_than,0));
                qFilter.and(new QFilter("tpv_costitem.number", QCP.not_equals,""));
                if (YEM.isNotEmpty(costitemNumber)) {
                    qFilter.and(new QFilter("tpv_costitem.number", QCP.equals,costitemNumber));
                }
                DynamicObjectCollection query = QueryServiceHelper.query("tpv_dt_costbillmat",
                        "id,billno,tpv_costitem.number tpv_costitemnumber,tpv_pactmessage,tpv_pactmessage.tpv_finarbillentryid tpv_finarbillentryid" //冲销金额
                                , qFilter.toArray());
                //数据为空，那么应收单反写金额为原本金额
                if (query.size() > 0) {
                    for (DynamicObject dynamicObject : query) {
                        Long tpvSourceEntryid = dynamicObject.getLong("tpv_finarbillentryid");
                        String tpvCostitem = dynamicObject.getString("tpv_costitemnumber");
                        if (YEM.isNotEmpty(tpvCostitem)) {
                            Map<String, String> flagWithout = new HashMap<>();
                            flagWithout.put("tpv_costitemnumber", tpvCostitem);
                            flagWithout.put("flagWithout", "YES"); //是否含有其他单包含，费用一致，应收单分录行一致的费用发票匹配单
                            map.put(tpvSourceEntryid, flagWithout);
                        }
                    }
                } else {
                    for (Object o : jsonArray) {
                        JSONObject parseObject = JSONObject.parseObject(o.toString());
                        for (String s : parseObject.keySet()) {
                            Map<String, String> flagWithout = new HashMap<>();
                            String costitem = (String) parseObject.get(s);
                            flagWithout.put("tpv_costitemnumber", costitem);
                            flagWithout.put("flagWithout", "NO");
                            map.put(Long.valueOf(s), flagWithout);
                        }
                    }
                }
            }

            if (map.size() > 0) {
                List<Long> idList = new ArrayList<>();
                map.keySet().stream().forEach(x ->idList.add(x));
                QFilter qFilter = new QFilter("shkd_saleentryentity.id",QCP.in,idList);
                DynamicObject[] query = BusinessDataServiceHelper.load("ar_finarbill",
                        "id,billno,shkd_saleentryentity,shkd_saleentryentity.shkd_freightnotfee," +
                                "shkd_saleentryentity.shkd_insurancenotfee,shkd_saleentryentity.shkd_commissionnotfee," +
                                "shkd_saleentryentity.shkd_famount1 shkd_famount1,shkd_saleentryentity.shkd_famount2 shkd_famount2," +
                                "shkd_saleentryentity.shkd_famount5 shkd_famount5",
                        qFilter.toArray());
                if (query.length > 0) {
                    for (DynamicObject dynamicObject : query) {
                        DynamicObjectCollection saleentryentity = dynamicObject.getDynamicObjectCollection("shkd_saleentryentity");
                        for (DynamicObject entryEntity : saleentryentity) {
                            Long aLong = entryEntity.getLong("id");
                            if (map.containsKey(aLong)) {
                                Map<String, String> bigDecimalMap = map.get(aLong);
                                String tpvCostitem = (String) bigDecimalMap.get("tpv_costitemnumber");
                                String flagWithout = (String) bigDecimalMap.get("flagWithout");
                                if ("FYXM150.1142".equals(tpvCostitem)) {
                                    if ("YES".equals(flagWithout)) {
                                        entryEntity.set("shkd_freightnotfee", BigDecimal.ZERO); //佣金相关-国际运费未冲销金额
                                    } else if ("NO".equals(flagWithout)) {
                                        entryEntity.set("shkd_freightnotfee", entryEntity.getBigDecimal("shkd_famount1")); //佣金相关-国际运费未冲销金额
                                    }
                                }
                                if ("FYXM150.1152".equals(tpvCostitem)) {
                                    if ("YES".equals(flagWithout)) {
                                        entryEntity.set("shkd_insurancenotfee", BigDecimal.ZERO); //佣金相关-国际保险费未冲销金额
                                    } else if ("NO".equals(flagWithout)) {
                                        entryEntity.set("shkd_insurancenotfee", entryEntity.getBigDecimal("shkd_famount2")); //佣金相关-国际保险费未冲销金额
                                    }
                                }
                                if ("FYXM150.1220".equals(tpvCostitem)) {
                                    if ("YES".equals(flagWithout)) {
                                        entryEntity.set("shkd_commissionnotfee", BigDecimal.ZERO); //佣金相关-佣金未冲销金额
                                    } else if ("NO".equals(flagWithout)) {
                                        entryEntity.set("shkd_commissionnotfee", entryEntity.getBigDecimal("shkd_famount5")); //佣金相关-佣金未冲销金额
                                    }
                                }
                            }
                        }

                    }
                }
                SaveServiceHelper.save(query);
            }
        }

    }
}
