package tpv.tpv_wm.tpv_dt.java.opplugin.op;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 采购发票匹配单审核/反审核 -服务插件  审核时生成隐藏【入库/合同信息分录】数据
 * @Author LinRui
 * @Date 2024/6/21 21:15
 */
public class PurchaseInvoiceMatchAuditOp extends AbstractOperationServicePlugIn {

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        String key = e.getOperationKey();
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            switch (key) {
                case "audit":
                    setRKAndHTEntity(bill);
                    break;
                case "unaudit":
                    bill = BusinessDataServiceHelper.loadSingle(bill.getPkValue(), bill.getDynamicObjectType().getName());
                    DynamicObjectCollection tpvContwareinfoentry = bill.getDynamicObjectCollection("tpv_contwareinfoentry");
                    tpvContwareinfoentry.clear();
                    SaveServiceHelper.save(new DynamicObject[]{bill});
                    break;
                default:
                    break;
            }
        }
    }

    private void setRKAndHTEntity(DynamicObject bill) {
        bill = BusinessDataServiceHelper.loadSingle(bill.getPkValue(), bill.getDynamicObjectType().getName());
        DynamicObjectCollection tpvContinformation = bill.getDynamicObjectCollection("tpv_continformation"); //合同信息
        DynamicObjectCollection tpvWareiformation = bill.getDynamicObjectCollection("tpv_wareiformation"); //入库信息
        List<String> tpvConnums = tpvContinformation.stream().filter(x -> !x.getString("tpv_connums").isEmpty()).map(x -> x.getString("tpv_connums")).collect(Collectors.toList());
        List<String> tpvConnum = tpvWareiformation.stream().filter(x -> !x.getString("tpv_connum").isEmpty()).map(x -> x.getString("tpv_connum")).collect(Collectors.toList());
        QFilter qFilter = new QFilter("tpv_contractnumber", QCP.not_equals, "");
        if (tpvConnums.size() > 0) {
            qFilter = new QFilter("tpv_contractnumber", QCP.in, tpvConnums);
        }
        if (tpvConnum.size() > 0) {
            qFilter = new QFilter("tpv_contractnumber", QCP.in, tpvConnum);
        }
        DynamicObject[] objects = BusinessDataServiceHelper.load("tpv_dt_purchase", "id,billno,tpv_contractnumber,tpv_currency", qFilter.toArray());
        Map<String, DynamicObject> map = new HashMap<>();
        if (objects.length > 0) {
            for (DynamicObject object : objects) {
                map.put(object.getString("tpv_contractnumber"), object);
            }
        }
        if (tpvContinformation.size() > 0) {
            for (DynamicObject dynamicObject : tpvContinformation) {
                //获取隐藏分录数据
                DynamicObjectCollection tpvContwareinfoentrys = bill.getDynamicObjectCollection("tpv_contwareinfoentry");
                DynamicObject tpvContwareinfoRow = tpvContwareinfoentrys.addNew();
                tpvContwareinfoRow.set("tpv_cwdept", dynamicObject.getDynamicObject("tpv_condept")); //部门
                tpvContwareinfoRow.set("tpv_cwpurcontrnum", map.containsKey(dynamicObject.getString("tpv_connums"))
                        ? map.get(dynamicObject.getString("tpv_connums")).getLong("id") : 0l); //采购合同号
                tpvContwareinfoRow.set("tpv_cwsupplier", dynamicObject.getDynamicObject("tpv_consupplier")); //供应商
                tpvContwareinfoRow.set("tpv_cwmaterial", dynamicObject.getDynamicObject("tpv_material")); //物料编码
                tpvContwareinfoRow.set("tpv_cwprodname", dynamicObject.getDynamicObject("tpv_conprodname")); //品名
                tpvContwareinfoRow.set("tpv_cwshader", dynamicObject.getString("tpv_conshader")); //材质
                tpvContwareinfoRow.set("tpv_cworigin", dynamicObject.getDynamicObject("tpv_conorigin")); //产地
                tpvContwareinfoRow.set("tpv_cwspecation", dynamicObject.getString("tpv_conspecation")); //规格
                tpvContwareinfoRow.set("tpv_cwqty", dynamicObject.getBigDecimal("tpv_conqty")); //数量
                tpvContwareinfoRow.set("tpv_cwamount", dynamicObject.getBigDecimal("tpv_contractamo")); //金额
                tpvContwareinfoRow.set("tpv_cwmatchqty", dynamicObject.getBigDecimal("tpv_curmatchqty")); //本次匹配数量
                tpvContwareinfoRow.set("tpv_cwmatchamo", dynamicObject.getBigDecimal("tpv_curmatchamo")); //本次匹配金额(含税)
                tpvContwareinfoRow.set("tpv_cwamountnotax", dynamicObject.getBigDecimal("tpv_curamountnotax")); //本次匹配金额(不含税)
                tpvContwareinfoRow.set("tpv_cwamountintax", dynamicObject.getBigDecimal("tpv_curamountintax")); //本次匹配金额(增值税额)
                tpvContwareinfoRow.set("tpv_cwentryid", dynamicObject.getPkValue()); //分录id
                tpvContwareinfoRow.set("tpv_conorwareentryid", dynamicObject.getString("tpv_conentryid")); //合同分录ID
                tpvContwareinfoRow.set("tpv_cwtaxrate", dynamicObject.getBigDecimal("tpv_httaxrate")); //税率%
                tpvContwareinfoRow.set("tpv_cwcurrency", map.containsKey(dynamicObject.getString("tpv_connums"))
                        ? map.get(dynamicObject.getString("tpv_connums")).getDynamicObject("tpv_currency") : 0l); //币别
                tpvContwareinfoRow.set("tpv_matchtype", "A"); //匹配类型
            }
        }
        if (tpvWareiformation.size() > 0) {
            for (DynamicObject dynamicObject : tpvWareiformation) {
                //获取隐藏分录数据
                DynamicObjectCollection tpvContwareinfoentrys = bill.getDynamicObjectCollection("tpv_contwareinfoentry");
                DynamicObject tpvContwareinfoRow = tpvContwareinfoentrys.addNew();
                tpvContwareinfoRow.set("tpv_cwdept", dynamicObject.getDynamicObject("tpv_warehousedept")); //部门
                tpvContwareinfoRow.set("tpv_cwpurcontrnum", map.containsKey(dynamicObject.getString("tpv_connum"))
                        ? map.get(dynamicObject.getString("tpv_connum")).getLong("id") : 0l); //采购合同号
                tpvContwareinfoRow.set("tpv_cwwarenum", dynamicObject.getString("tpv_warenum")); //入库单号
                tpvContwareinfoRow.set("tpv_cwsupplier", dynamicObject.getDynamicObject("tpv_warehousupplier")); //供应商
                tpvContwareinfoRow.set("tpv_cwmaterial", dynamicObject.getDynamicObject("tpv_warehousmaterial")); //物料编码
                tpvContwareinfoRow.set("tpv_cwprodname", dynamicObject.getDynamicObject("tpv_prodname")); //品名
                tpvContwareinfoRow.set("tpv_cwshader", dynamicObject.getString("tpv_shader")); //材质
                tpvContwareinfoRow.set("tpv_cworigin", dynamicObject.getDynamicObject("tpv_origin")); //产地
                tpvContwareinfoRow.set("tpv_cwspecation", dynamicObject.getString("tpv_specation")); //规格
                tpvContwareinfoRow.set("tpv_cwqty", dynamicObject.getBigDecimal("tpv_qty")); //数量
                tpvContwareinfoRow.set("tpv_cwamount", dynamicObject.getBigDecimal("tpv_warehouseamt")); //金额
                tpvContwareinfoRow.set("tpv_cwmatchqty", dynamicObject.getBigDecimal("tpv_rkcurmatchqty")); //本次匹配数量
                tpvContwareinfoRow.set("tpv_cwmatchamo", dynamicObject.getBigDecimal("tpv_rkcurmatchamo")); //本次匹配金额(含税)
                tpvContwareinfoRow.set("tpv_cwamountnotax", dynamicObject.getBigDecimal("tpv_rkcurmatchamonotax")); //本次匹配金额(不含税)
                tpvContwareinfoRow.set("tpv_cwamountintax", dynamicObject.getBigDecimal("tpv_rkcurmatchamointax")); //本次匹配金额(增值税额)
                tpvContwareinfoRow.set("tpv_cwentryid", dynamicObject.getPkValue()); //分录id
                tpvContwareinfoRow.set("tpv_conorwareentryid", dynamicObject.getString("tpv_purinentryid")); //入库分录id
                tpvContwareinfoRow.set("tpv_cwtaxrate", dynamicObject.getBigDecimal("tpv_rktaxrate")); //税率%
                tpvContwareinfoRow.set("tpv_cwrate", dynamicObject.getBigDecimal("tpv_rate")); //汇率
                tpvContwareinfoRow.set("tpv_cwcurrency", dynamicObject.getDynamicObject("tpv_currency")); //币别
                tpvContwareinfoRow.set("tpv_cwratetable", dynamicObject.getDynamicObject("tpv_ratetable")); //汇率表
                tpvContwareinfoRow.set("tpv_cwmatchqtyed", dynamicObject.getBigDecimal("tpv_rkmatchqty")); //已匹配数量
                tpvContwareinfoRow.set("tpv_matchtype", "B"); //匹配类型
            }
        }
        SaveServiceHelper.save(new DynamicObject[]{bill});
    }
}
