package tpv.tpv_wm.tpv_dt.java.opplugin.op;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.List;

/**
 * @ClassName: SalePriceBillAuditOp
 * @Description: 销售价格确认单  审核时：
 * 1、当商品分录【来源】=“销售发货分录”时，根据已价格确认数量的金额+未价格确认数量的金额/对应销售发货分录数量，更新《发货单》对应分录行【含税单价】，
 * 2、当商品分录【来源】=“销售发货拣配库存子分录”时，根据已价格确认数量的金额+未价格确认数量的金额/子分录数量，更新{销售发货拣配库存子分录}对应分录行【含税单价】，并根据销售发货拣配库存子分录含税金额汇总/子分录对应商品分录【数量】更新子分录对应分录【含税单价】。
 * @author: zbh
 * @date: 2024-05-16
 */
public class SalePriceBillAuditOp extends AbstractOperationServicePlugIn {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_materialentry");
        fieldKeys.add("tpv_srcbill");
        fieldKeys.add("tpv_srcbillno");
        fieldKeys.add("tpv_srcentryid");
        fieldKeys.add("tpv_taxprice");
        fieldKeys.add("tpv_taxamt_final");
        fieldKeys.add("tpv_srcsubid");
        fieldKeys.add("tpv_qty");
        fieldKeys.add("tpv_qty_final");
        fieldKeys.add("tpv_taxprice");
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        String key = e.getOperationKey();
        DynamicObject[] entities = e.getDataEntities();
        switch (key) {
            case "audit":
            case "unaudit":
                taxpriceFinal(entities, key);
                break;
        }
    }

    public void taxpriceFinal(DynamicObject[] entities, String key) {
        for (DynamicObject entity : entities) {
            DynamicObjectCollection tpvSrcbills = entity.getDynamicObjectCollection("tpv_materialentry");
            for (DynamicObject tpvSrcbill : tpvSrcbills) {
                String tpvSrcbill1 = tpvSrcbill.getString("tpv_srcbill");
                String tpvSrcbillno = tpvSrcbill.getString("tpv_srcbillno");//来源单据编码
                Long tpvSrcentryid = tpvSrcbill.getLong("tpv_srcentryid");//来源单据分录ID
                BigDecimal tpvTaxamtFinal = tpvSrcbill.getBigDecimal("tpv_taxamt_final");//确认金额（含税）
                BigDecimal tpvQtyP = tpvSrcbill.getBigDecimal("tpv_qty");//发货数量
                BigDecimal tpvQtyFinal = tpvSrcbill.getBigDecimal("tpv_qty_final");//确认数量
                BigDecimal tpvTax = tpvSrcbill.getBigDecimal("tpv_taxprice");//含税单价
                if ("B".equals(tpvSrcbill1)) {
                    DynamicObject dtSalesshipping = BusinessDataServiceHelper.loadSingle("tpv_dt_salesshipping", new QFilter[]{new QFilter("tpv_materialentry", "=", tpvSrcentryid)});
                    DynamicObjectCollection tpvMaterialentry = dtSalesshipping.getDynamicObjectCollection("tpv_materialentry");
                    for (DynamicObject dynamicObject : tpvMaterialentry) {
                        BigDecimal taxprice = dynamicObject.getBigDecimal("tpv_taxprice");
                        BigDecimal c = dynamicObject.getBigDecimal("tpv_qty");
                        if (dynamicObject.getLong("id") == tpvSrcentryid) {
                            BigDecimal tpvTaxprice = BigDecimal.ZERO;
                            if ("audit".equals(key)) {
                                tpvTaxprice=tpvTaxamtFinal.subtract(taxprice.multiply(tpvQtyP.subtract(tpvQtyFinal))).divide(tpvQtyFinal,2,BigDecimal.ROUND_HALF_UP);
                                dynamicObject.set("tpv_taxprice",tpvTaxprice);//含税单价
                                dynamicObject.set("tpv_taxpricerecord",taxprice);//含税单价（记录）
                                dynamicObject.set("tpv_taxpramount",tpvTaxprice.multiply(tpvTaxprice));//含税金额
                            } else {
                                dynamicObject.set("tpv_taxprice",tpvTax);//含税单价（记录）
                                dynamicObject.set("tpv_taxpricerecord",tpvTax);//含税单价（记录）
                                dynamicObject.set("tpv_taxpramount",tpvTaxprice.multiply(tpvTax));//含税金额
                            }
                        }
                    }
                    SaveServiceHelper.save(new DynamicObject[]{dtSalesshipping});
                } else if ("C".equals(tpvSrcbill1)) {
                    Long tpvSrcsubid = tpvSrcbill.getLong("tpv_srcsubid");//来源单据子分录ID
                    DynamicObject dtSalesshipping = BusinessDataServiceHelper.loadSingle("tpv_dt_salesshipping", new QFilter[]{new QFilter("tpv_materialentry.tpv_dt_salepick_in", "=", tpvSrcsubid)});
                    DynamicObjectCollection tpvMaterialentry = dtSalesshipping.getDynamicObjectCollection("tpv_materialentry");
                    for (DynamicObject dynamicObject : tpvMaterialentry) {
                        BigDecimal decimal = BigDecimal.ZERO;
                        DynamicObjectCollection tpvDtSalepickIn = dynamicObject.getDynamicObjectCollection("tpv_dt_salepick_in");
                        for (DynamicObject enty : tpvDtSalepickIn) {
                            BigDecimal tpvTaxpriceIn = enty.getBigDecimal("tpv_taxprice_in");//含税单价
                            BigDecimal tpvPickqtyIn = enty.getBigDecimal("tpv_pickqty_in");//本次配货数量
                            BigDecimal tpvTaxprice = BigDecimal.ZERO;
                            if (enty.getLong("id") == tpvSrcsubid) {
                                if ("audit".equals(key)) {
                                    tpvTaxprice = tpvTaxamtFinal.subtract(tpvTaxpriceIn.multiply(tpvQtyP.subtract(tpvQtyFinal))).divide(tpvQtyFinal,2,BigDecimal.ROUND_HALF_UP);
                                    enty.set("tpv_taxprice_in", tpvTaxprice);//含税单价
                                    enty.set("tpv_taxpricerecord_in", tpvTaxpriceIn);//含税单价
                                    enty.set("tpv_taxpramount_in", tpvPickqtyIn.multiply(tpvTaxprice));//含税金额
                                    decimal=decimal.add(tpvTaxprice);
                                } else {
                                    enty.set("tpv_taxprice_in", tpvTax);//含税单价
                                    enty.set("tpv_taxpricerecord_in", tpvTax);//含税单价（记录）
                                    enty.set("tpv_taxpramount_in", tpvPickqtyIn.multiply(tpvTax));//含税金额
                                    decimal=decimal.add(tpvTaxpriceIn);
                                }

                            } else {
                                decimal=decimal.add(tpvTaxpriceIn);
                            }
                        }
                        dynamicObject.set("tpv_taxprice",decimal);
                    }
                    SaveServiceHelper.save(new DynamicObject[]{dtSalesshipping});
                }
            }
        }
    }
    /*
       BigDecimal decimal = BigDecimal.ZERO;
                        if (dynamicObject.getLong("id") == tpvSrcentryid) {
                            DynamicObjectCollection tpvDtSalepickIn = dynamicObject.getDynamicObjectCollection("tpv_dt_salepick_in");
                            for (DynamicObject enty : tpvDtSalepickIn) {
                                BigDecimal tpvTaxprice = BigDecimal.ZERO;
                                if ("audit".equals(key)) {
                                    tpvTaxprice = enty.getBigDecimal("tpv_taxprice_in");//含税单价
                                } else {
                                    tpvTaxprice = enty.getBigDecimal("tpv_taxpricerecord_in");//含税单价（记录）
                                }
                                if (enty.getLong("id") != tpvSrcsubid) {
                                    decimal = decimal.add(enty.getBigDecimal("tpv_taxpramount_in"));//已价格确认数量
                                }
                                BigDecimal tpvConfirmed = enty.getBigDecimal("tpv_thequantity");//已价格确认数量

                                BigDecimal tpvQty = enty.getBigDecimal("tpv_pickqty_in");//数量
                                BigDecimal add = tpvTaxamtFinal.add(tpvQty.subtract(tpvConfirmed).multiply(tpvTaxprice).divide(tpvQty, 3, BigDecimal.ROUND_HALF_UP));
                                if (enty.getLong("id") == tpvSrcsubid) {
                                    enty.set("tpv_taxprice_in", add);//含税单价
                                    enty.set("tpv_taxpramount_in", add.multiply(tpvQty));//含税金额
                                    decimal = decimal.add(add.multiply(tpvQty));
                                }
                            }
                            BigDecimal tpvQty = dynamicObject.getBigDecimal("tpv_qty");//数量
                            BigDecimal tpvVatrates = dynamicObject.getBigDecimal("tpv_vatrates");//增值税率%
                            BigDecimal divide = decimal.divide(tpvQty, 3, BigDecimal.ROUND_HALF_UP);
                            dynamicObject.set("tpv_taxprice", "audit".equals(key)?divide:BigDecimal.ZERO);//含税单价
                            dynamicObject.set("tpv_taxpramount",  "audit".equals(key)?decimal:BigDecimal.ZERO);//含税金额
                            dynamicObject.set("tpv_taxprice", "audit".equals(key)?divide.multiply(BigDecimal.ONE.add(tpvVatrates.divide(new BigDecimal(100)))):BigDecimal.ZERO);//不含税单价
                        }
     */
}
