package tpv.tpv_wm.tpv_dt.java.opplugin.op;

import com.itopview.wm.utils.DynamicObjectUtil;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author l j w
 * @date 2023/9/26 14:59
 * @className SaleShippingBillPlugIn
 * @description SalesshippingBillPlug 销售发货单提交效验
 *
 */
public class SaleShipingSubmitOp extends AbstractOperationServicePlugIn {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_materialentry");
        keys.add("tpv_dt_salepick_in");
        keys.add("tpv_billno_in");
        keys.add("tpv_customer");
        keys.add("tpv_checked");
        keys.add("tpv_inventorytype");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] bills = this.getDataEntities();
                if ("submit".equals(this.getOperateKey()) && null != e.getDataEntities()) {
                    for (int i = 0; i < bills.length; ++i) {
                        ExtendedDataEntity bill = bills[i];
                        checkoutDeclar(bill);
                        inventory(bill);
                        materialentry(bill);
                        onsubmitDate(bill);
                        proportion(bill);
                    }
                }
            }

            private void proportion(ExtendedDataEntity bill) {
                DynamicObject dy = bill.getDataEntity();
                DynamicObject tpvBilltype = dy.getDynamicObject("tpv_billtype");
                if (tpvBilltype!=null && !"tpv_dt_salesshipping_LS".equals(tpvBilltype.getString("number"))){
                    BigDecimal proport = BigDecimal.ZERO;
                    DynamicObjectCollection tpvPaymentmethods = dy.getDynamicObjectCollection("tpv_paymentmethods");
                    for (DynamicObject tpvPaymentmethod : tpvPaymentmethods) {
                        proport=proport.add(tpvPaymentmethod.getBigDecimal("tpv_proportion"));
                    }
                    if (proport.compareTo(new BigDecimal(100))!=0){
                        this.addMessage(bill, "收款计划比例之和不为100！!");
                    }
                }
            }

            private void onsubmitDate(ExtendedDataEntity bill) {
                DynamicObject dy = bill.getDataEntity();
                BigDecimal overfillrate = BigDecimal.ZERO;//溢装率
                overfillrate=dy.getBigDecimal("tpv_overloadingrates");
                BigDecimal one = BigDecimal.ONE;//1
                BigDecimal million = BigDecimal.valueOf(100);//100
                BigDecimal tpv_billablequantitys = BigDecimal.ZERO;//费用子分录数量
                DynamicObjectCollection materialentry = dy.getDynamicObjectCollection("tpv_materialentry");//商品明细
                if (materialentry.size() > 0) {
                    for (DynamicObject matertrty : materialentry) {
                        DynamicObjectCollection tpv_dt_freigroambill_f = matertrty.getDynamicObjectCollection("tpv_dt_dcostinform_f");//费用信息
                        int row = matertrty.getInt("Seq");
                        BigDecimal tpv_qty = matertrty.getBigDecimal("tpv_qty");//商品分录数量
                        if (tpv_dt_freigroambill_f.size() > 0) {
                            for (DynamicObject freigroambill : tpv_dt_freigroambill_f) {
                                tpv_billablequantitys = freigroambill.getBigDecimal("tpv_billablequantitys");
                            }
                            BigDecimal dive = overfillrate.divide(million);
                            BigDecimal add = dive.add(one);
                            BigDecimal sum = tpv_qty.multiply(add);
                            if (tpv_billablequantitys.compareTo(sum) > 0) {
                                this.addMessage(bill, "商品明细第" + row + "行（{费用信息}子分录【计费数量】之和不能大于{商品分录}【数量】*（1+溢装率）!");
                            }
                        }
                    }
                }
            }

            private void materialentry(ExtendedDataEntity bill) {
                DynamicObject dataEntity = bill.getDataEntity();
                Boolean tpv_nopicking = dataEntity.getBoolean("tpv_nopicking");
                String tpvInventorytype = dataEntity.getString("tpv_inventorytype");
                if ("B".equals(tpvInventorytype)){
                    tpv_nopicking=false;
                }
                if (tpv_nopicking) {
                    return;
                }
                DynamicObjectCollection tpv_materialentry = dataEntity.getDynamicObjectCollection("tpv_materialentry");
                for (DynamicObject dynamicObject : tpv_materialentry) {
                    int row = dynamicObject.getInt("Seq");//可以加减
                    DynamicObjectCollection tpv_dt_salepick_in = dynamicObject.getDynamicObjectCollection("tpv_dt_salepick_in");
                    DynamicObjectCollection tpv_dt_salepick_zy = dynamicObject.getDynamicObjectCollection("tpv_dt_salepick_zy");
                    BigDecimal tpv_qty = dynamicObject.getBigDecimal("tpv_qty");
                    BigDecimal tpv_pickqty_zy = BigDecimal.ZERO;
                    BigDecimal tpv_pickqty_in = BigDecimal.ZERO;
                    if (tpv_dt_salepick_in.size() > 0) {
                        for (DynamicObject tpv_dt_salepick : tpv_dt_salepick_in) {
                            tpv_pickqty_zy = tpv_pickqty_zy.add(tpv_dt_salepick.getBigDecimal("tpv_pickqty_in"));
                        }
                    } else if (tpv_dt_salepick_zy.size() > 0) {
                        for (DynamicObject tpv_dt_salepick : tpv_dt_salepick_zy) {
                            tpv_pickqty_in = tpv_pickqty_in.add(tpv_dt_salepick.getBigDecimal("tpv_pickqty_zy"));
                        }
                    }
                    if (tpv_pickqty_zy.compareTo(BigDecimal.ZERO) == 1) {
                        if (tpv_pickqty_zy.compareTo(tpv_qty) == 1 || tpv_pickqty_zy.compareTo(tpv_qty) == -1) {
                            this.addMessage(bill, "商品分录第" + row + "行数量和已拣配数量不一致，请修改后提交！！！");
                        }
                    }
                    if (tpv_pickqty_in.compareTo(BigDecimal.ZERO) == 1) {
                        if (tpv_pickqty_in.compareTo(tpv_qty) == 1 || tpv_pickqty_in.compareTo(tpv_qty) == -1) {
                            this.addMessage(bill, "商品分录第" + row + "行数量和已拣配数量不一致，请修改后提交！！！");
                        }
                    }
                }
            }

            private void inventory(ExtendedDataEntity dataEntities) {
                DynamicObject dataEntity = dataEntities.getDataEntity();
                DynamicObjectCollection tpv_materialentry = dataEntity.getDynamicObjectCollection("tpv_materialentry");
                Boolean tpv_nopicking = dataEntity.getBoolean("tpv_nopicking");
                String tpvInventorytype = dataEntity.getString("tpv_inventorytype");
                if ("B".equals(tpvInventorytype)){
                    tpv_nopicking=false;
                }
                if (!tpv_nopicking) {
                    if (tpv_materialentry.size() == 0) {
                        this.addMessage(dataEntities, "销售发货-商品信息分录没有数据，不允许保存！！！");
                    }
                    for (DynamicObject dynamicObject : tpv_materialentry) {
                        int row = dynamicObject.getInt("Seq");//可以加减
                        DynamicObjectCollection tpv_dt_salepick_in = dynamicObject.getDynamicObjectCollection("tpv_dt_salepick_in");
                        DynamicObjectCollection tpv_dt_salepick_zy = dynamicObject.getDynamicObjectCollection("tpv_dt_salepick_zy");
                        if (tpv_dt_salepick_in.size() == 0 && tpv_dt_salepick_zy.size() == 0) {
                            if (tpv_dt_salepick_in.size() == 0) {
                                this.addMessage(dataEntities, "销售发货分录第" + row + "行子分录未捡配，请拣配后提交！！！");
                            } else if (tpv_dt_salepick_zy.size() == 0) {
                                this.addMessage(dataEntities, "销售发货分录第" + row + "行子分录未捡配，请拣配后提交！！！");
                            }
                        }
                    }
                }
            }

            private void checkoutDeclar(ExtendedDataEntity dataEntities) {
                DynamicObject dataEntity = dataEntities.getDataEntity();
                Boolean tpvChecked = dataEntity.getBoolean("tpv_checked");//采销主体重合校验
                Boolean tpvWhether = dataEntity.getBoolean("tpv_whether");//是否第三方销售
                String tpvInventorytype = dataEntity.getString("tpv_inventorytype");//直运/库存
                DynamicObjectCollection tpvDtSalepickIn = new DynamicObjectCollection();
                //当第三方销售为否且直运库存为库存
                if (!tpvWhether && "A".equals(tpvInventorytype) && tpvChecked) {
                    DynamicObject tpvCustomer = dataEntity.getDynamicObject("tpv_customer");//销售发货客户
                    if(tpvCustomer!=null){
                        String taxno = tpvCustomer.getString("taxno");//客户税号
                        //客户税号不为空
                        if (taxno != "") {
                            DynamicObjectCollection tpvMaterialentry = dataEntity.getDynamicObjectCollection("tpv_materialentry");
                            for (DynamicObject dynamicObject : tpvMaterialentry) {
                                tpvDtSalepickIn = dynamicObject.getDynamicObjectCollection("tpv_dt_salepick_in");

                            }
                        }
                    }

                }
                String tpvBillnoIn = "";
                if (null != tpvDtSalepickIn) {
                    //遍历子分录获取数据
                    for (DynamicObject salepick : tpvDtSalepickIn) {
                        tpvBillnoIn = salepick.getString("tpv_billno_in");//销售发货分录的采购入库单据编号
                    }
                }
                //和子分录编号一致
                QFilter qFilter = new QFilter("billno", QCP.equals, tpvBillnoIn);
                String selectfields = DynamicObjectUtil.getSelectfields("im_purinbill", false);
                String entrySelectfields = DynamicObjectUtil.getEntrySelectfields(selectfields, "im_purinbill", "billentry", false);
                //通过销售发货查采购入库单
                DynamicObject[] purinbill = BusinessDataServiceHelper.load("im_purinbill", entrySelectfields
                        , new QFilter[]{qFilter});
                if (purinbill.length > 0) {
                    for (int i = 0; i < purinbill.length; i++) {
                        DynamicObject custmain = purinbill[i];
                        Long tpv_sourcebillno = custmain.getLong("tpv_bigintid");//采购合同内码
                        QFilter qFilter1 = new QFilter("id", QCP.equals, tpv_sourcebillno);
                        //通过采购入库查采购合同
                        DynamicObject[] partment = BusinessDataServiceHelper.load("tpv_dt_purchase", "id,billno,tpv_supplier,taxno"
                                , new QFilter[]{qFilter1});
                        if (partment.length > 0) {
                            for (int i1 = 0; i1 < partment.length; i1++) {
                                DynamicObject partments = partment[i1];
                                String Billno = partments.getString("billno");//采购合同的单据编号
                                DynamicObject tpv_supplier = partments.getDynamicObject("tpv_supplier");//采购合同的供应商
//                                String tpv_taxid = tpv_supplier.getString("tpv_taxid");//供应商税号
                                DynamicObject bdSupplier = BusinessDataServiceHelper.loadSingle(tpv_supplier.getPkValue(), "bd_supplier");
                                String taxid = bdSupplier.getString("taxno");
                                if (taxid != "") {
                                    //查询单据编号为采购合同单据编号的
                                    QFilter qFilter2 = new QFilter("tpv_corebillno", QCP.equals, Billno);
                                    String selectfield = DynamicObjectUtil.getSelectfields("im_purinbill", false);
                                    String entrySelectfield = DynamicObjectUtil.getEntrySelectfields(selectfield, "im_purinbill", "billentry", false);
                                    //通过采购合同的单据编号查采购下游所有的采购入库单
                                    DynamicObject[] dt_purinbill = BusinessDataServiceHelper.load("im_purinbill", entrySelectfield
                                            , new QFilter[]{qFilter2});
                                    if (dt_purinbill.length > 0) {
                                        BigDecimal qty = BigDecimal.ZERO;
                                        BigDecimal tpv_packnum = BigDecimal.ZERO;
                                        for (int i2 = 0; i2 < dt_purinbill.length; i2++) {
                                            DynamicObject dt_purinbills = dt_purinbill[i2];
                                            DynamicObjectCollection billentry = dt_purinbills.getDynamicObjectCollection("billentry");
                                            for (DynamicObject bill : billentry) {
                                                BigDecimal qtys = bill.getBigDecimal("qty");//采购入库的数量
                                                BigDecimal tpv_packnums = bill.getBigDecimal("tpv_packnum");//采购合同的包装数量
                                                tpv_packnum = tpv_packnum.add(tpv_packnums);
                                                qty = qty.add(qtys);
                                            }
                                        }
                                        BigDecimal qtsy = BigDecimal.ZERO;
                                        BigDecimal tpv_packnums = BigDecimal.ZERO;
                                        String tpvtaxid = "";
                                        DynamicObjectCollection tpv_materialentry = dataEntity.getDynamicObjectCollection("tpv_materialentry");
                                        DynamicObject tpvsupplier = dataEntity.getDynamicObject("tpv_customer");//销售发货的客户
                                        if (tpvsupplier != null) {
                                            tpvtaxid = tpvsupplier.getString("taxno");//客户税号
                                        }
                                        for (DynamicObject dynamicObject : tpv_materialentry) {
                                            BigDecimal tpv_qty = dynamicObject.getBigDecimal("tpv_qty");
                                            BigDecimal tpv_packnumd = dynamicObject.getBigDecimal("tpv_packnum");
                                            qtsy = qtsy.add(tpv_qty);
                                            tpv_packnums = tpv_packnums.add(tpv_packnumd);
                                        }
                                        if (tpvtaxid != "") {
                                            //不允许销售同税号客户99%以上数量
                                            BigDecimal qtes = qty.multiply(new BigDecimal(99).divide(new BigDecimal(100)));
                                            if (qtsy.compareTo(qtes) > -1) {
                                                this.addMessage(dataEntities, "普通发货单（非第三方、非直运）可拣配采购自发货客户的货物，但不能超过采购合同入库总数量的99%及入库总件数的100%。即采购自该客户的商品不能全量销售回该客户。");
                                            }
                                            if (tpv_packnum.compareTo(tpv_packnums) > -1) {
                                                this.addMessage(dataEntities, "普通发货单（非第三方、非直运）可拣配采购自发货客户的货物，但不能超过采购合同入库总数量的99%及入库总件数的100%。即采购自该客户的商品不能全量销售回该客户。");
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        });
    }
}