package tpv.tpv_wm.tpv_dt.java.opplugin.op;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.dt.report.PickShipsRptQuery;
import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 销售发货单反审核校验，当【库存/直运】为直运时，反审核清除库存子分录内容
 * @Author LinRui
 * @Date 2023/11/8 10:14
 */
public class SaleShippingUnAuditOp extends AbstractOperationServicePlugIn {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_materialentry"); //商品分录
        fieldKeys.add("tpv_dt_salepick_in"); //拣配信息-库存
        fieldKeys.add("tpv_inventorytype"); //库存/直运
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle(bill.getPkValue(), bill.getDataEntityType().getName());
            String tpvInventorytype = dynamicObject.getString("tpv_inventorytype");
            if (YEM.isNotEmpty(tpvInventorytype)) {
                if ("B".equals(tpvInventorytype)) {
                    DynamicObjectCollection tpvMaterialentry = dynamicObject.getDynamicObjectCollection("tpv_materialentry");
                    JSONArray jsonArray = new JSONArray();
                    for (DynamicObject object : tpvMaterialentry) {
                        DynamicObjectCollection tpvDtSalepickIn = object.getDynamicObjectCollection("tpv_dt_salepick_in");
                        for (DynamicObject bill_s : tpvDtSalepickIn) {
                            JSONObject jsonObject = new JSONObject();
                            jsonObject.put("tpv_pickbill1", bill_s.getString("tpv_pickbill_in"));
                            jsonObject.put("tpv_fid1", bill_s.getLong("tpv_fid_in"));
                            jsonArray.add(jsonObject);
                        }
                        tpvDtSalepickIn.clear();
                    }
                    SaveServiceHelper.save(new DynamicObject[]{dynamicObject});

                    if (null != jsonArray) {
                        Map<String, Map<Long, List<DynamicObject>>> pickMap = PickShipsRptQuery.pickDataCols();
                        Map<String, List<JSONObject>> backMap = jsonArray.stream().map(x -> (JSONObject) x)
                                .filter(x -> YEM.isNotEmpty(x.getString("tpv_pickbill1")))
                                .collect(Collectors.groupingBy(x -> x.getString("tpv_pickbill1")));
                        for (String billKey : backMap.keySet()) {
                            List<Long> ids = backMap.get(billKey).stream().map(x -> x.getLong("tpv_fid1")).distinct()
                                    .collect(Collectors.toList());
                            String entryKey;
                            String pickKey;
                            String packnumKey;
                            String subEntryKey = "";
                            String subPickKey = "";
                            boolean von = false;
                            switch (billKey) {
                                case "tpv_dt_purchase":
                                    entryKey = "tpv_materialentry";
                                    pickKey = "tpv_pickedquantity";
                                    packnumKey = "tpv_ispacknumqty";
                                    break;
                                case "tpv_dt_freigroambill":
                                    entryKey = "tpv_materialentry";
                                    pickKey = "tpv_direcroomtqty";
                                    packnumKey = "tpv_ispacknumqty";
                                    break;
                                case "im_purinbill":
                                    entryKey = "billentry";
                                    pickKey = "tpv_quantitypicked";
                                    subEntryKey = "tpv_dt_purinbill_j";
                                    subPickKey = "tpv_quantitypickeds";
                                    packnumKey = "tpv_ispacknumqty";
                                    von = true;
                                    break;
                                case "im_otherinbill":
                                    entryKey = "billentry";
                                    pickKey = "tpv_quantitypicked";
                                    packnumKey = "tpv_ispacknumqty";
                                    break;
                                case "im_transinbill":
                                    entryKey = "billentry";
                                    pickKey = "tpv_quantitypicked";
                                    packnumKey = "tpv_ispacknumqty";
                                    break;
                                case "im_transdirbill":
                                    entryKey = "billentry";
                                    pickKey = "tpv_quantitypicked";
                                    packnumKey = "tpv_ispacknumqty";
                                    break;
                                default:
                                    continue;
                            }
                            String selectFields = "id," + entryKey + "," + pickKey + "," + packnumKey;
                            if (von) {
                                selectFields += "," + subEntryKey + "," + subPickKey;
                            }
                            QFilter qFilter = new QFilter("id", QCP.in, ids);
                            DynamicObject[] pickbills = BusinessDataServiceHelper.load(billKey, selectFields, qFilter.toArray());
                            for (DynamicObject data : pickbills) {
                                DynamicObjectCollection entry = data.getDynamicObjectCollection(entryKey);
                                for (DynamicObject row : entry) {
                                    BigDecimal tpv_pickqty_s = BigDecimal.ZERO;// 已拣配数量
                                    BigDecimal tpv_packnum_s = BigDecimal.ZERO;// 已拣配包装数量
                                    long entryId = row.getLong("id");
                                    Map<Long, List<DynamicObject>> map = pickMap.get(billKey);
                                    if (von) {
                                        DynamicObjectCollection vonEntry = row.getDynamicObjectCollection(subEntryKey);
                                        for (DynamicObject subRow : vonEntry) {
                                            long tpv_vid = subRow.getLong("id");
                                            BigDecimal tpv_pickqty_sub = BigDecimal.ZERO;// 已拣配数量-卷号分录
                                            if (map != null) {
                                                List<DynamicObject> list = map.get(entryId);
                                                if (list != null) {
                                                    for (DynamicObject obj : list) {
                                                        long tpv_vid1 = obj.getLong("tpv_vid");
                                                        if (tpv_vid == tpv_vid1) {
                                                            tpv_pickqty_sub = BigDecimalUtils.add(tpv_pickqty_sub,
                                                                    obj.getBigDecimal("tpv_pickqty_s"), 10);
                                                        }
                                                    }
                                                }
                                            }
                                            subRow.set(subPickKey, tpv_pickqty_sub);
                                        }
                                    }
                                    if (map != null) {
                                        List<DynamicObject> list = map.get(entryId);
                                        if (list != null) {
                                            for (DynamicObject obj : list) {
                                                tpv_pickqty_s = BigDecimalUtils.add(tpv_pickqty_s, obj.getBigDecimal("tpv_pickqty_s"),
                                                        10);
                                                tpv_packnum_s = BigDecimalUtils.add(tpv_packnum_s, obj.getBigDecimal("tpv_packnum_s"),
                                                        10);
                                            }
                                        }
                                    }
                                    row.set(pickKey, tpv_pickqty_s);
                                    row.set(packnumKey, tpv_packnum_s);
                                }
                            }
                            if (pickbills.length > 0) {
                                SaveServiceHelper.save(pickbills);
                            }
                        }
                    }
//                    OperationServiceHelper.executeOperate("save", bill.getDataEntityType().getName(),
//                            new DynamicObject[]{dynamicObject}, YEM.getOption());
                }
            }
        }
    }

}
