package tpv.tpv_wm.tpv_dt.java.opplugin.op;

import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;
import tpv.tpv_wm.tpv_dt.java.opplugin.validator.SaleContractSubmitValidator;

import java.math.BigDecimal;
import java.util.List;

/**
 * @Description: 销售合同提交校检 服务插件
 * @Auther: zbh
 */
public class SalesContractSubmitOp extends AbstractOperationServicePlugIn implements Plugin {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_department");
        keys.add("tpv_amount");
        keys.add("tpv_isselloncredit");
        keys.add("tpv_customer");
        keys.add("tpv_approvaltogether");
        keys.add("tpv_supplier_apt");
        keys.add("tpv_documenttypes");
        keys.add("tpv_supplychain");
        keys.add("tpv_contractnos");
        keys.add("tpv_paymentplans");
        keys.add("tpv_proportion");
        keys.add("tpv_thirdparty");
        keys.add("tpv_contractstatus");
        keys.add("tpv_natureofpayment");
        keys.add("tpv_daytype");
        keys.add("tpv_collectiondate");
        keys.add("tpv_signingdate");
        keys.add("tpv_materialentry");
        keys.add("tpv_material");
        keys.add("tpv_commoditycodj");
        keys.add("tpv_commodityclassifyj");
        keys.add("tpv_strategynum");
        keys.add("tpv_positionnums");
        keys.add("tpv_openposition");
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        String key = e.getOperationKey();
        DynamicObject[] entities = e.getDataEntities();
        switch (key) {
            case "submit":
                submit(entities,key);
                save(entities,key);
                break;
            case "unsubmit":
                submit(entities,key);
                break;
            case "audit":
                submit(entities,key);
                break;
            case "unaudit":
                submit(entities,key);
                break;
            case"save":
                save(entities,key);
                break;
        }
    }

    private void save(DynamicObject[] entities, String key) {
        for (DynamicObject dataEntity : entities) {
            DynamicObjectCollection materialentry = dataEntity.getDynamicObjectCollection("tpv_materialentry");
            if (materialentry == null || materialentry.size() == 0) {
                return;
            }
            DynamicObject material = materialentry.get(0).getDynamicObject("tpv_material");//商品编号
            if (material == null) {
                dataEntity.set("tpv_commoditycodj", null);//实物商品物料编码
                dataEntity.set("tpv_commodityclassifyj", null);//实物商品物料分类
                return;
            }
            dataEntity.set("tpv_commoditycodj", material); //实物商品物料编码
            //查询物料分类
            QFilter qFiltera = new QFilter("material.id", QCP.in, material.getLong("id"));
            qFiltera.and(new QFilter("standard.number", QCP.equals, "JBFLBZ"));
            DynamicObject materialClass = BusinessDataServiceHelper.loadSingle("bd_materialgroupdetail", "id,group,material", new QFilter[]{qFiltera});
            DynamicObject group = materialClass.getDynamicObject("group");//分组
            dataEntity.set("tpv_commodityclassifyj", group);//实物商品物料分类
            SaveServiceHelper.save(new DynamicObject[]{dataEntity});
        }
    }

    private void submit(DynamicObject[] entities, String key) {
        for (DynamicObject entity : entities) {
            if ("submit".equals(key)){
                entity.set("tpv_contractstatus","B");
                SaveServiceHelper.save(new DynamicObject[]{entity});
            }else if ("audit".equals(key)){
                entity.set("tpv_contractstatus","D");
                SaveServiceHelper.save(new DynamicObject[]{entity});
            }else if ("unsubmit".equals(key)||"unaudit".equals(key)){
                entity.set("tpv_contractstatus","A");
                SaveServiceHelper.save(new DynamicObject[]{entity});
            }
        }
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new SaleContractSubmitValidator());
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        String key = e.getOperationKey();
        switch (key) {
            case "submit":
                for (int i = 0; i < bills.length; ++i) {
                    DynamicObject bill = bills[i];
                    setPositionQty("tpv_strategynum", "tpv_positionnums", "tpv_openposition", bill);
                    OperationServiceHelper.executeOperate("save", bill.getDataEntityType().getName(),
                            new DynamicObject[]{bill}, YEM.getOptionNoMsg());
                }
                break;
            default:
                break;
        }
    }

    private void setPositionQty(String name, String positionnumsKey, String openpositionKey, DynamicObject bill) {
        DynamicObject tpvStrategynum = bill.getDynamicObject(name);
        if (YEM.isNotEmpty(tpvStrategynum)) {
            long id = tpvStrategynum.getLong("id");
            DataSet purchase = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_purchase", "id,tpv_contractnumber,tpv_policynumber.id policyid," +
                            "tpv_policynumber.number policynumber,tpv_materialentry,tpv_materialentry.tpv_qty qty",
                    new QFilter[]{new QFilter("tpv_contractstatusx", QCP.in, new String[]{"A", "B", "C", "D", "F", "G"})
                            .and(new QFilter("tpv_policynumber.id", QCP.in, id))}).select("policyid policyid", "qty purqty", "0.0 saleqty");
            DataSet salescontract = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_salescontract", "id,tpv_contractnos,tpv_strategynum.id strategyid," +
                            "tpv_strategynum.number strategynum,tpv_materialentry,tpv_materialentry.tpv_qty qty",
                    new QFilter[]{new QFilter("tpv_contractstatus", QCP.in, new String[]{"A", "B", "C", "D", "G", "H"})
                            .and(new QFilter("tpv_strategynum.id", QCP.in, id))}).select("strategyid policyid", "qty saleqty", "0.0 purqty");
            DataSet dataSet = purchase.union(salescontract).groupBy(new String[]{"policyid"}).sum("saleqty").sum("purqty").finish();
            dataSet = RptUtil.zeroNullNums(dataSet);
            dataSet = dataSet.addField("(purqty - saleqty) qty", "qty");
            BigDecimal qty = BigDecimal.ZERO;
            if (!dataSet.isEmpty()) {
                for (Row row : dataSet) {
                    qty = row.getBigDecimal("qty").setScale(2, BigDecimal.ROUND_HALF_UP).abs();
                }
            }
            BigDecimal tpvReplylimit = BigDecimalUtils.toBigDecimal(tpvStrategynum.getBigDecimal("tpv_replylimit")).setScale(2, BigDecimal.ROUND_HALF_UP);
            bill.set(positionnumsKey, qty);
            bill.set(openpositionKey, tpvReplylimit.subtract(qty));
        } else {
            bill.set(positionnumsKey, BigDecimal.ZERO);
            bill.set(openpositionKey, BigDecimal.ZERO);
        }
    }
}
