package tpv.tpv_wm.tpv_dt.java.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.data.BusinessDataReader;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.IDataEntityType;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.botp.runtime.ConvertOperationResult;
import kd.bos.entity.botp.runtime.PushArgs;
import kd.bos.entity.datamodel.IRefrencedataProvider;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.botp.ConvertServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @ClassName: SalesShippingSaveAuditOp
 * @Description: 1、《零售发货》保存时生成《销售合同）
 * 2、《零售发货》每次保存都需要更新《销售合同》内容；
 * 3、《零售发货》提交和审核也需要同步将和认同进行提交和审核
 * 4.将含税单价复制给含税单价（记录）
 * @author: zbh
 * @date: 2024-04-17
 */
public class SalesShippingSaveAuditOp extends AbstractOperationServicePlugIn {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_sourcebillid"); //商品分录

    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        String key = e.getOperationKey();
        DynamicObject[] entities = e.getDataEntities();
        for (int i = 0; i < entities.length; i++) {
            switch (key) {
                case "save":
                case "submit":
                    tocontractnumber(entities[i]);
                    break;
            }
        }
    }

    private void tocontractnumber(DynamicObject entity) {
        long tpvSourcebillid = entity.getLong("tpv_sourcebillid");
        DynamicObject tpvCustomer = entity.getDynamicObject("tpv_customer");
        String tpvContractnumber = entity.getString("tpv_contractnumber");
        if (0!=tpvSourcebillid){
            DynamicObject tpvDtSalescontract = BusinessDataServiceHelper.loadSingle(tpvSourcebillid, "tpv_dt_salescontract");
            DynamicObject customer = tpvDtSalescontract.getDynamicObject("tpv_customer");
            if (tpvCustomer!=null &&customer!=null){
                long aLong = tpvCustomer.getLong("id");
                long id = customer.getLong("id");
                if (id!=aLong){
                    entity.set("tpv_partysales",tpvContractnumber);
                    entity.set("tpv_customers",tpvCustomer.get("name"));
                }
            }
        }
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        String key = e.getOperationKey();
        DynamicObject[] entities = e.getDataEntities();
        for (int i = 0; i < entities.length; i++) {
            switch (key) {
                case "save":
                case "submit":
                case "unsubmit":
                case "audit":
                    toSalesContract(entities[i], key);
                    break;
            }
        }
    }

    public void toSalesContract(DynamicObject enty, String key) {
        Object value = enty.getPkValue();
        DynamicObject entity = BusinessDataServiceHelper.loadSingle(value, "tpv_dt_salesshipping");
        DynamicObject tpvBilltype = entity.getDynamicObject("tpv_billtype");
        OperateOption optionOne = YEM.getOption();
        if (tpvBilltype == null) {
            return;
        }
        Object number = tpvBilltype.get("number");
        if (!"tpv_dt_salesshipping_LS".equals(tpvBilltype.get("number"))) {
            if("unsubmit".equals(key)){
                taxPriceRecord(entity, key);
            }else if ("submit".equals(key)){
                taxPriceRecord(entity,key);
            }
            return;
        }
        Object pkValue = entity.getPkValue();
        DynamicObject salesContract = BusinessDataServiceHelper.loadSingle("tpv_dt_salescontract", new QFilter[]{new QFilter("tpv_sourcebillid", "=", pkValue)});
        if ("save".equals(key)) {
            if (salesContract == null) {
                savePush(entity);
            } else if (salesContract != null) {
                updateContract(salesContract, entity);
            }
        } else if ("submit".equals(key)) {
            taxPriceRecord(entity,key);
            if (salesContract == null) {
                savePush(entity);
            } else if (salesContract != null) {
               OperationServiceHelper.executeOperate("submit", "tpv_dt_salescontract",new Object[]{salesContract.getPkValue()}, optionOne);

            }
        } else if ("audit".equals(key)) {
            DynamicObjectCollection tpvMaterialentry = salesContract.getDynamicObjectCollection("tpv_materialentry");
            if (tpvMaterialentry.size()!=0){
                DynamicObject tpvMaterial = tpvMaterialentry.get(0).getDynamicObject("tpv_material");
                QFilter qFiltera = new QFilter("material.id", QCP.in, tpvMaterial.getLong("id"));
                qFiltera.and(new QFilter("standard.number", QCP.equals, "JBFLBZ"));
                DynamicObject materialClass = BusinessDataServiceHelper.loadSingle("bd_materialgroupdetail", "id,group,material", new QFilter[]{qFiltera});
                if (materialClass!=null){
                    DynamicObject group = materialClass.getDynamicObject("group");//分组
                    salesContract.set("tpv_commodityclassifyj",group);//实物商品物料分类
                }
                salesContract.set("tpv_commoditycodj",tpvMaterial); //实物商品物料编码
            }
            OperationServiceHelper.executeOperate("audit", "tpv_dt_salescontract",  new Object[]{salesContract.getPkValue()}, optionOne);
//            salesContract.set("billstatus","C");
//            salesContract.set("auditor", YEM.getCurrentUserId());
//            salesContract.set("auditdate",new Date());
//            SaveServiceHelper.save(new DynamicObject[]{salesContract});
        }else if ("unsubmit".equals(key)){
//            taxPriceRecord(entity,key);
//            salesContract.set("billstatus","A");
//            SaveServiceHelper.save(new DynamicObject[]{salesContract});
            OperationServiceHelper.executeOperate("unsubmit", "tpv_dt_salescontract",new Object[]{salesContract.getPkValue()}, optionOne);
        }
    }

    private void taxPriceRecord(DynamicObject entity, String key) {
        DynamicObjectCollection tpvMaterialentrys = entity.getDynamicObjectCollection("tpv_materialentry");
        for (DynamicObject tpvMaterialentry : tpvMaterialentrys) {
            if ("submit".equals(key)) {
                tpvMaterialentry.set("tpv_taxpricerecord", tpvMaterialentry.getBigDecimal("tpv_taxprice"));
            }else {
                tpvMaterialentry.set("tpv_taxpricerecord", 0);
            }
            DynamicObjectCollection tpvDtSalepickZys = tpvMaterialentry.getDynamicObjectCollection("tpv_dt_salepick_zy");
            for (DynamicObject tpvDtSalepickZy : tpvDtSalepickZys) {
                if ("submit".equals(key)){
                    tpvDtSalepickZy.set("tpv_taxpricerecord_zy",tpvDtSalepickZy.getBigDecimal("tpv_taxprice_zy"));
                }else {
                    tpvDtSalepickZy.set("tpv_taxpricerecord_zy",0);
                }
            }
            DynamicObjectCollection tpvDtSalepickIns = tpvMaterialentry.getDynamicObjectCollection("tpv_dt_salepick_in");
            for (DynamicObject tpvDtSalepickIn : tpvDtSalepickIns) {
                if ("submit".equals(key)) {
                    tpvDtSalepickIn.set("tpv_taxpricerecord_in", tpvDtSalepickIn.getBigDecimal("tpv_taxprice_in"));
                }else {
                    tpvDtSalepickIn.set("tpv_taxpricerecord_in",0);
                }
            }
        }
        SaveServiceHelper.save(new DynamicObject[]{entity});
    }

    /*
    salesContract:销售合同
    entity:销售发货
     */
    public void updateContract(DynamicObject salesContract, DynamicObject entity) {
        salesContract.set("tpv_bizdate", entity.get("tpv_bizdate"));//业务日期
        salesContract.set("tpv_operator", entity.get("tpv_operator"));//业务员
        salesContract.set("tpv_company", entity.get("tpv_company"));//公司
        salesContract.set("tpv_department", entity.get("tpv_department"));//部门
        salesContract.set("tpv_customer", entity.get("tpv_customer"));//客户
        salesContract.set("tpv_combinationnumber", entity.get("tpv_combina"));//组合号
        salesContract.set("tpv_businesstype", entity.get("tpv_business"));//业务类型
        salesContract.set("tpv_strategynum", entity.get("tpv_policyber"));//策略号
        salesContract.set("tpv_awardtype", entity.get("tpv_reward"));//提奖类型
        salesContract.set("tpv_overloadingrates", entity.get("tpv_overloadingrates"));//溢装率(%)
        salesContract.set("tpv_shortagerates", entity.get("tpv_shortagerates"));//短装率(%)
        salesContract.set("tpv_inventory", entity.get("tpv_inventorytype"));//库存/直运

        List<Long> materListId = new ArrayList<>();
        DynamicObjectCollection tpvMaterialentry = entity.getDynamicObjectCollection("tpv_materialentry");// 发货商品明细
        for (DynamicObject dynamicObject : tpvMaterialentry) {
            long id = dynamicObject.getLong("id");
            materListId.add(id);
        }
        List<Long> saleListId = new ArrayList<>();
        DynamicObjectCollection saleMaterialentry = salesContract.getDynamicObjectCollection("tpv_materialentry");// 销售 商品明细
        for (DynamicObject dynamicObject : saleMaterialentry) {
            long id = dynamicObject.getLong("tpv_srcentryid");
            if (id != 0) {
                saleListId.add(id);
            }
        }
        //取出销售合同的来源单据分录id等于销售发货的分录id
        List<Long> intersection = materListId.stream().filter(saleListId::contains).collect(Collectors.toList());
        for (Long aLong : intersection) {
            for (DynamicObject sale : saleMaterialentry) {
                long tpvSrcentryid = sale.getLong("tpv_srcentryid");//来源单据分录ID
                if (aLong != tpvSrcentryid) {
                    continue;
                }
                addUpdateEntity(sale, tpvMaterialentry, tpvSrcentryid, entity);
            }
        }

        materListId.removeAll(intersection);// 发货 除去相等
        for (Long aLong : materListId) {
            DynamicObject addNew = saleMaterialentry.addNew();
            addUpdateEntity(addNew, tpvMaterialentry, aLong, entity);
        }
        saleListId.removeAll(intersection);// 销售合同 除去相等
        List<DynamicObject> dyList = new ArrayList<>();
        for (Long aLong : saleListId) {
            for (DynamicObject dynamicObject : saleMaterialentry) {
                long id = dynamicObject.getLong("tpv_srcentryid");
                if (id == aLong) {
                    dyList.add(dynamicObject);
                }
            }
        }
        for (DynamicObject dynamicObject : dyList) {
            saleMaterialentry.remove(dynamicObject);
        }
        OperationServiceHelper.executeOperate("save", "tpv_dt_salescontract", new DynamicObject[]{salesContract});
    }

    /*
    sale：销售分录行实体
    saleMaterialentry：发货商品明细
    tpvSrcentryid：销售分录行实体的来源单据分录ID
    entity：发货实体
     */
    public void addUpdateEntity(DynamicObject sale, DynamicObjectCollection tpvMaterialentry, long tpvSrcentryid, DynamicObject entity) {
        for (DynamicObject ship : tpvMaterialentry) {
            long id = ship.getLong("id");
            if (id != tpvSrcentryid) {
                continue;
            }
            sale.set("tpv_material", ship.get("tpv_material"));//商品编号
            sale.set("tpv_unit", ship.get("tpv_unit"));//计量单位
            sale.set("tpv_qty", ship.get("tpv_qty"));//数量
            sale.set("tpv_price", ship.get("tpv_price"));//不含税单价
            sale.set("tpv_nottaxamount", ship.get("tpv_nottaxamount"));//不含税金额
            sale.set("tpv_taxpramount", ship.get("tpv_taxpramount"));//含税金额
            sale.set("tpv_taxprice", ship.get("tpv_taxprice"));//含税单价
            sale.set("tpv_vatrates", ship.get("tpv_vatrates"));//增值税率%
            sale.set("tpv_vatmoney", ship.get("tpv_vatmoney"));//增值税额
            sale.set("tpv_prodname", ship.get("tpv_prodname"));//品名
            sale.set("tpv_specation", ship.get("tpv_specation"));//规格
            sale.set("tpv_shader", ship.get("tpv_shader"));//材质
            sale.set("tpv_origin", ship.get("tpv_origin"));//产地
            sale.set("tpv_drytonsnum", ship.get("tpv_drytonsnum"));//干吨数量
            sale.set("tpv_packnum", ship.get("tpv_packnum"));//包装数量
            sale.set("tpv_coefficients", ship.get("tpv_coefficients"));//系数
            sale.set("tpv_packagingunit", ship.get("tpv_packagingunit"));//包装单位
            sale.set("tpv_shippedquantity", ship.get("tpv_qty"));//已发货数量
            sale.set("tpv_cobillno", entity.get("billno"));//核心单据编码
            sale.set("tpv_cobillid", entity.get("id"));//核心单据ID
            sale.set("tpv_coentryid", ship.get("id"));//核心单据分录ID
            sale.set("tpv_srcbillkey", "tpv_dt_salesshipping");//来源单据标识
            sale.set("tpv_srcbillno", entity.get("billno"));//来源单据编码
            sale.set("tpv_srcbillid", entity.get("id"));//来源单据ID
            sale.set("tpv_srcentryid", ship.get("id"));//来源单据分录ID
        }
    }

    public void savePush(DynamicObject entity) {
        PushArgs args = new PushArgs();
        args.setSourceEntityNumber("tpv_dt_salesshipping");        // 必选，源单标识
        args.setTargetEntityNumber("tpv_dt_salescontract");      // 必选，目标单标识
        args.setRuleId("1858766716192961536");  // 可选，传入本次下推使用的转换规则id；传入空值，由系统自动寻找合适的转换规则
        args.setBuildConvReport(true);            // 是否输出详细错误报告
        List<ListSelectedRow> selectedRows = new ArrayList<>();
        ListSelectedRow srcBillRows = new ListSelectedRow(entity.getLong("id"));
        selectedRows.add(srcBillRows);
        args.setSelectedRows(selectedRows);        // 必选，设置需要下推的源单及分录内码
        // 调用下推引擎，下推目标单
        ConvertOperationResult pushResult = ConvertServiceHelper.push(args);
        if (pushResult.isSuccess()) {
            // 获取生成的目标单数据包
            MainEntityType targetMainType = EntityMetadataCache.getDataEntityType("tpv_dt_salescontract");
            List<DynamicObject> targetBillObjs = pushResult.loadTargetDataObjects(new IRefrencedataProvider() {
                @Override
                public void fillReferenceData(Object[] objs, IDataEntityType dType) {
                    BusinessDataReader.loadRefence(objs, dType);
                }
            }, targetMainType);
            if (targetBillObjs.size() > 0) {
                DynamicObject targetBillObj = targetBillObjs.get(0);
                OperationServiceHelper.executeOperate("save", "tpv_dt_salescontract", new DynamicObject[]{targetBillObj});
            }
        }
    }
}
