package tpv.tpv_wm.tpv_dt.java.opplugin.op;

import com.itopview.wm.sys.utils.NewVersionHelper;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhck
 * @Package: com.itopview.wm.dt.opplugin.op.SaveReverseWriteContract
 * @Date: 19:14 2023/10/30
 * @Description: 被《采购合同》引用后，合同保存反写【采购合同号】；
 * @Version: 1.0
 */
public class SaveReverseWriteContractOp extends AbstractOperationServicePlugIn {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_applicationform");
        fieldKeys.add("tpv_contractnumber");
        fieldKeys.add("tpv_moneykzed");
        fieldKeys.add("tpv_dt_purchase_u");
        fieldKeys.add("tpv_payment");
        fieldKeys.add("tpv_figure");
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        if ("save".equals(e.getOperationKey()) || "submit".equals(e.getOperationKey()) && null != e.getDataEntities()) {
            DynamicObject[] dynamicObjects = new DynamicObject[0];
            for (DynamicObject dataEntity : e.getDataEntities()) {
                String tpvContractnumber = dataEntity.getString("tpv_contractnumber"); // 采购合同号
                DynamicObject applicationform = dataEntity.getDynamicObject("tpv_applicationform"); // 供应商单票额度

                // 保存后查找《供应商额度》为当前合同号的单据，并清空反写的值
                dynamicObjects = BusinessDataServiceHelper.load
                        ("tpv_bd_supticket", "tpv_proconnum,id",
                                new QFilter[]{new QFilter("tpv_proconnum", QCP.equals, tpvContractnumber)});
                Optional.ofNullable(dynamicObjects)
                        .map(array -> {
                            Arrays.stream(array).forEach(u -> u.set("tpv_proconnum", null));
                            SaveServiceHelper.save(array);
                            return array;
                        });

                // 根据当前采购合同所选的《供应商单票额度维护》反写值
                Optional.ofNullable(applicationform)
                        .map(u -> {
                            DynamicObject bdSupticket = BusinessDataServiceHelper.loadSingle
                                    (u.getPkValue(), "tpv_bd_supticket");
                            bdSupticket.set("tpv_proconnum", tpvContractnumber);
                            SaveServiceHelper.save(new DynamicObject[]{bdSupticket});
                            return u;
                        });
                NewVersionHelper vHelper = new NewVersionHelper(dataEntity.getDataEntityType().getName());
                if (vHelper.isChanged()) {
                    return;
                }
                if ("submit".equals(e.getOperationKey())){
                    DynamicObjectCollection dtPurchaseUs = dataEntity.getDynamicObjectCollection("tpv_dt_purchase_u");
                    BigDecimal decimal = BigDecimal.ZERO;//可开证金额
                    for (DynamicObject dtPurchaseU : dtPurchaseUs) {
                        String payMent = dtPurchaseU.getString("tpv_payment.number");
                        if ("JSFS13".equals(payMent) || "JSFS12".equals(payMent)) {
                            BigDecimal tpvFigure = dtPurchaseU.getBigDecimal("tpv_figure");
                            decimal = decimal.add(tpvFigure);
                        }
                    }
                    dataEntity.set("tpv_moneykzed", decimal);
                    SaveServiceHelper.save(new DynamicObject[]{dataEntity});
                }
            }
        }
    }
}
