package tpv.tpv_wm.tpv_dt.java.opplugin.validator;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.validate.AbstractValidator;

import java.util.Arrays;
import java.util.Date;

/**
 * @Description: 满足条件时，提交校验，关联期限认领分录至少有一行
 * @Project: zhck
 * @Package: tpv.tpv_wm.tpv_dt.java.opplugin.validator.CondictionExistenceSubmitValidator
 * @Date: 2024/04/02 10:12
 * @Author: Heisenberg Wenbei He
 * @Version: 1.0
 */
public class ConditionExistenceSubmitValidator extends AbstractValidator {

    private static final String WARNING_MESSAGE = "点价时，关联期现认领明细分录至少有一行分录";

    @Override
    public void validate() {
        if (!"submit".equals(this.getOperateKey())) {
            return;
        }

        switch (this.getEntityKey()) {

            // 销售价格确认单
            case "tpv_dt_salepricebill":
                this.judge("tpv_isvertex", null, "tpv_settlemententity");
                break;

            // 货物流转单
            case "tpv_dt_freigroambill":
                this.judge("tpv_whetherprice", "tpv_billtype", "tpv_settlemententry");
                break;
            default:
                break;
        }

    }

    /**
     * @param sign:     是否点价标识
     * @param billtype: 单据类型
     * @param entry:    分录标识
     * @author: Heisenberg
     * @date: 2024/4/2 10:34
     * @description: 满足条件时，提交校验，关联期限认领分录至少有一行
     */
    private void judge(String sign, String billtype, String entry) {
        if (billtype == null) {
            Arrays.stream(this.getDataEntities())
                    .filter(single -> Boolean.TRUE.equals(single.getValue(sign)))
                    .filter(single -> single.getDataEntity().getDynamicObjectCollection(entry).isEmpty())
                    .forEach(single -> this.addMessage(single, WARNING_MESSAGE));
        } else {
            Arrays.stream(this.getDataEntities())
                    .filter(single -> Boolean.TRUE.equals(single.getValue(sign)))
                    .filter(single -> "tpv_dt_freigroambill_jg".equals
                            (single.getDataEntity().getDynamicObject(billtype).getString("number")))
                    .filter(single -> single.getDataEntity().getDynamicObjectCollection(entry).isEmpty())
                    .forEach(single -> this.addMessage(single, WARNING_MESSAGE));
        }
        ExtendedDataEntity[] entities = this.getDataEntities();
        String key = this.getEntityKey();
        if("tpv_dt_freigroambill".equals(key)){
            for (ExtendedDataEntity entity : entities) {
                DynamicObject dataEntity = entity.getDataEntity();
                Date tpvBegindate = dataEntity.getDate("tpv_begindate");//开始日期
                Date tpvFinishdate = dataEntity.getDate("tpv_finishdate");//到达日期
                if (tpvFinishdate != null) {
                    if (tpvBegindate.after(tpvFinishdate)) {
                        this.addMessage(entity, "到达日期不能小于开始日期！");
                    }
                }
            }
        }

    }
}
