package tpv.tpv_wm.tpv_dt.java.opplugin.validator;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;

/**
 * @Description: 费用结算单 保存校验，计费数量不能大于上游单计费数量（作废）
 * @Author LinRui
 * @Date 2023/9/2 17:00
 */
public class FeeSettleSaveValidator extends AbstractValidator {
    @Override
    public void validate() {
        ExtendedDataEntity[] dataEntities = this.dataEntities;
        for (int i = 0; i < dataEntities.length; i++) {
            ExtendedDataEntity extend = dataEntities[i];
            DynamicObject entity = extend.getDataEntity();
            check(entity, extend);
        }
    }

    /**
     * @Description: //校验计费数量不能大于当行来源单据中的计费数量
     * @Date: 10:30 2023/09/04
     * @param: entity - [kd.bos.dataentity.entity.DynamicObject]
     * @param: extend - [kd.bos.entity.ExtendedDataEntity]
     * @return: void
     **/
    public void check(DynamicObject entity, ExtendedDataEntity extend) {
        //entity = BusinessDataServiceHelper.loadSingle(entity.getPkValue(),entity.getDynamicObjectType().getName());
        DynamicObjectCollection tpvDtStatemenbillDi = entity.getDynamicObjectCollection("tpv_dt_statemenbill_di");
        for (int i = 0; i < tpvDtStatemenbillDi.size(); i++) {
            BigDecimal tpvBillablequantity = tpvDtStatemenbillDi.get(i).getBigDecimal("tpv_billablequantity");  //计费数量
            long tpvSrcbillid = tpvDtStatemenbillDi.get(i).getLong("tpv_srcbillid"); //费用结算单的分录行来源单据id
            String tpvSrcsubentryid = tpvDtStatemenbillDi.get(i).getString("tpv_srcsubentryid");  //费用结算单的分录行来源单据子分录id
            String tpvSrcbillkeys = tpvDtStatemenbillDi.get(i).getString("tpv_srcbillkeys");  //费用结算单的分录行来源单据标识
            String seq = tpvDtStatemenbillDi.get(i).getString("seq");  //费用结算单的分录行序号
            DynamicObject dynamicObject = BusinessDataServiceHelper.loadSingle(tpvSrcbillid, "" + tpvSrcbillkeys + "");  //查询分录行对应来源单据
            //遍历分录行来源单据父分录及子分录数据
            DynamicObjectCollection dynamicObjectCollection = null;
            if ("tpv_dt_freigroambill".equals(tpvSrcbillkeys)) {
                dynamicObjectCollection = dynamicObject.getDynamicObjectCollection("tpv_materialentry");
            } else if ("im_purinbill".equals(tpvSrcbillkeys) || "im_otherinbill".equals(tpvSrcbillkeys) || "im_transinbill".equals(tpvSrcbillkeys)
                    || "im_transdirbill".equals(tpvSrcbillkeys)) {
                dynamicObjectCollection = dynamicObject.getDynamicObjectCollection("billentry");
            }
            for (DynamicObject entityData : dynamicObjectCollection) {
                //遍历来源单据子分录数据
                setSrcSubEntityData(seq, extend, entityData, tpvSrcbillkeys, tpvSrcsubentryid, tpvBillablequantity); //获取对应单据子分录数据
            }
        }
    }

    /**
     * @Description: //获取对应单据的子分录标识及反写字段
     * @param: tpvSrcbillkeys - [java.lang.String]
     * @param: entityData - [kd.bos.dataentity.entity.DynamicObject]
     * @return: void
     **/
    private void setSrcSubEntityData(String seq, ExtendedDataEntity extend, DynamicObject entityData, String tpvSrcbillkeys, String tpvSrcsubentryid, BigDecimal tpvBillablequantity) {
        switch (tpvSrcbillkeys) {
            case "tpv_dt_freigroambill":  //货物流转单
                String srcSubEntitykeyfre = "tpv_dt_freigroambill_f";  //子分录标识
                String feeQtyFre = "tpv_billablequantitys"; //计费数量字段标识名
                setSrcSubEntity(seq, extend, entityData, tpvSrcsubentryid, srcSubEntitykeyfre, tpvBillablequantity, feeQtyFre); //获取对应单据进行反写赋值
                break;
            case "im_purinbill":  //采购入库单
                String srcSubEntitykeypur = "tpv_dt_purinbill_f";  //子分录标识
                String feeQtyPur = "tpv_chargequantity"; //计费数量字段标识名
                setSrcSubEntity(seq, extend, entityData, tpvSrcsubentryid, srcSubEntitykeypur, tpvBillablequantity, feeQtyPur); //获取对应单据进行反写赋值
                break;
            case "im_otherinbill":  //其他入库单
                String srcSubEntitykeyoth = "tpv_im_otherinbill_f";  //子分录标识
                String feeQtyOther = "tpv_chargequantity"; //计费数量字段标识名
                setSrcSubEntity(seq, extend, entityData, tpvSrcsubentryid, srcSubEntitykeyoth, tpvBillablequantity, feeQtyOther); //获取对应单据进行反写赋值
                break;
            case "im_transinbill":   //分步调入单
                String srcSubEntitykeytrai = "tpv_dt_transinbill_f";  //子分录标识
                String feeQtyTrai = "tpv_chargequantity"; //计费数量字段标识名
                setSrcSubEntity(seq, extend, entityData, tpvSrcsubentryid, srcSubEntitykeytrai, tpvBillablequantity, feeQtyTrai); //获取对应单据进行反写赋值
                break;
            case "im_transdirbill": //直接调拨单
                String srcSubEntitykeytrad = "tpv_dt_transdirbill_f";  //子分录标识
                String feeQtyTrad = "tpv_chargequantity"; //计费数量字段标识名
                setSrcSubEntity(seq, extend, entityData, tpvSrcsubentryid, srcSubEntitykeytrad, tpvBillablequantity, feeQtyTrad); //获取对应单据进行反写赋值
                break;
            default:
                break;
        }
    }

    /**
     * @Description: //获取对应单据费用分录行进行校验
     * @param: entityData - [kd.bos.dataentity.entity.DynamicObject]
     * @param: tpvSrcsubentryid - [java.lang.String]
     * @param: srcSubEntitykey - [java.lang.String]
     * @return: void
     **/
    private void setSrcSubEntity(String seq, ExtendedDataEntity extend, DynamicObject entityData, String tpvSrcsubentryid, String srcSubEntitykey, BigDecimal tpvBillablequantity, String tpvQtyField) {
        DynamicObjectCollection srcsubEntityData = entityData.getDynamicObjectCollection("" + srcSubEntitykey + "");
        for (DynamicObject subEntityData : srcsubEntityData) {
            long tpvDtFreigroambillF = subEntityData.getLong("id");  //来源单据子分录id
            if (tpvDtFreigroambillF > 0) {
                if (tpvDtFreigroambillF == Long.parseLong(tpvSrcsubentryid)) {  //核对费用结算单分录来源单据子分录id和对应单据
                    BigDecimal tpvSrcFeesQty = subEntityData.getBigDecimal("" + tpvQtyField + ""); //获取来源单据子分录的计费数量值
                    if (tpvBillablequantity.compareTo(tpvSrcFeesQty) == 1) {
                        String src = "第 " + seq + " 行" + "结算单的计费数量不能大于源单计费数量！";
                        this.addMessage(extend, src);
                    }
                }
            }
        }
    }
}
