package tpv.tpv_wm.tpv_dt.java.opplugin.validator;

import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import tpv.tpv_wm.tpv_crm.java.report.SupCreditUseRptQuery;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 采购合同提交校验 -当剩余签约额度不足时提示：“xxxxx”;
 * @Author LinRui
 * @Date 2023/10/18 20:53
 */
public class PurChaseSubmitValidator extends AbstractValidator {
    @Override
    public void validate() {
        ExtendedDataEntity[] dataEntities = this.dataEntities;
        for (int i = 0; i < dataEntities.length; i++) {
            ExtendedDataEntity extend = dataEntities[i];
            DynamicObject entity = extend.getDataEntity();
            boolean tpvUseasingleticket = entity.getBoolean("tpv_useasingleticket");
            if (tpvUseasingleticket) {
                check(entity, extend);
            }
        }
    }

    public void check(DynamicObject entity, ExtendedDataEntity extend) {
        //entity = BusinessDataServiceHelper.loadSingle(entity.getPkValue(),entity.getDynamicObjectType().getName());
        DynamicObject tpv_department = entity.getDynamicObject("tpv_department");
        DynamicObject tpv_supplier = entity.getDynamicObject("tpv_supplier");

        BigDecimal tpvAmount = entity.getBigDecimal("tpv_amount");
        if (YEM.isEmpty(tpv_department) & YEM.isEmpty(tpv_supplier)) {
            return;
        }
        QFilter qFilter = new QFilter("enable", QCP.equals, "1");
        if (YEM.isNotEmpty(tpv_department)) {
            qFilter.and(new QFilter("id",QCP.equals,tpv_department.getLong("id")));
        }
        DynamicObject bosAdminorgs = BusinessDataServiceHelper.loadSingle("bos_adminorg", qFilter.toArray());
        if (YEM.isNotEmpty(bosAdminorgs)) {
            DynamicObject orgpattern = bosAdminorgs.getDynamicObject("orgpattern");
            if (YEM.isNotEmpty(orgpattern)) {
                String number = orgpattern.getString("number");
                if ("OF-01".equals(number)) {
                    DynamicObjectCollection dynamicObjects = bosAdminorgs.getDynamicObjectCollection("structure");
                    if (dynamicObjects.size() > 0) {
                        for (DynamicObject dynamicObject : dynamicObjects) {
                            DynamicObject viewparent = dynamicObject.getDynamicObject("viewparent");
                            if (YEM.isNotEmpty(viewparent)) {
                                tpv_department = viewparent;
                            }
                            break;
                        }
                    }
                }
            }
        }
        boolean flag = false;
        if (YEM.isNotEmpty(tpv_department)) {
            QFilter qFilter1 = new QFilter("billstatus",QCP.in,new String[]{"B","C"});
            qFilter1.and(new QFilter("tpv_org.id",QCP.equals,tpv_department.getLong("id")));
            DynamicObject[] load = BusinessDataServiceHelper.load("tpv_bd_supplierlimits", "id", qFilter1.toArray());
            if (load.length > 0) {
                flag = true;
            }
        }
        JSONObject jsonObject = SupCreditUseRptQuery.supadvanceupJsonObject(null, null, null, tpv_department, tpv_supplier);
        String jsonString = jsonObject.toJSONString();
        if (jsonObject.size() > 0) {
            Map<String, BigDecimal> map = new HashMap<>();
            for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
                for (Map.Entry<String, Object> objectEntry : jsonObject.getJSONObject(entry.getKey()).entrySet()) {
                    if ("signingamt".equals(objectEntry.getKey())) {
                        BigDecimal signingamt = (BigDecimal) objectEntry.getValue() == null ? BigDecimal.ZERO : (BigDecimal) objectEntry.getValue();
                        map.put(entry.getKey(), signingamt);
                    }
                }
            }
            BigDecimal sum = BigDecimal.ZERO;
            sum = sum.add(tpvAmount);
            BigDecimal singleOrgAmt = BigDecimal.ZERO;
            BigDecimal totalOrgAmt = BigDecimal.ZERO;
            BigDecimal singleDeptAmt = BigDecimal.ZERO;
            BigDecimal totalDeptAmt = BigDecimal.ZERO;
            for (Map.Entry<String, BigDecimal> stringBigDecimalEntry : map.entrySet()) {
                if ("single_org".equals(stringBigDecimalEntry.getKey())) {
                    singleOrgAmt = singleOrgAmt.add(stringBigDecimalEntry.getValue());
                }
                if ("single_dept".equals(stringBigDecimalEntry.getKey())) {
                    singleDeptAmt = singleDeptAmt.add(stringBigDecimalEntry.getValue());
                }
                if ("total_org".equals(stringBigDecimalEntry.getKey())) {
                    totalOrgAmt = totalOrgAmt.add(stringBigDecimalEntry.getValue());
                }
                if ("total_dept".equals(stringBigDecimalEntry.getKey())) {
                    totalDeptAmt = totalDeptAmt.add(stringBigDecimalEntry.getValue());
                }
            }
            if (flag) {
                if (sum.compareTo(singleDeptAmt) > 0 && sum.compareTo(totalDeptAmt) <= 0) {
                    this.addWarningMessage(extend, "合同金额原币超额，已大于根据【部门】、【供应商】维度关联供应商额度使用情况表数据部门单独授信的剩余签约额度");
                }
                if (sum.compareTo(totalDeptAmt) > 0) {
                    this.addWarningMessage(extend, "合同金额原币超额，已大于根据【部门】、【供应商】维度关联供应商额度使用情况表数据部门总额度的剩余签约额度");
                }
            }
            if (sum.compareTo(singleOrgAmt) > 0 && sum.compareTo(totalOrgAmt) <= 0) {
                this.addWarningMessage(extend, "合同金额原币超额，已大于根据【供应商】维度关联供应商额度使用情况表数据全司单独授信的剩余签约额度");
            }
            if (sum.compareTo(totalOrgAmt) > 0) {
                this.addWarningMessage(extend, "合同金额原币超额，已大于根据【供应商】维度关联供应商额度使用情况表数据全司总额度的剩余签约额度");
            }
        }
    }
}
