package tpv.tpv_wm.tpv_dt.java.opplugin.validator;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.utils.StringUtils;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import tpv.tpv_wm.tpv_crm.java.utils.CusCreditLimitUtils;

import java.math.BigDecimal;

/**
 * @Description: 1）非配供：当【《发货单》 【单据类型】≠“配供业务”：《非配供业务-可放货余额明细表》T列【货款余额】-本单发货金额如果小于-0.02给予提醒，提醒内容:"货款余额（含本单）为xxx元，请注意核对赊销额度！"；
 * 2）配供-框架：当《发货单》【单据类型】=“配供业务”且【框架简称】不为空，根据框架号查《框架配供业务-可放货余额明细表》Y列【额度余额】-本单发货金额小于0给予提醒，提醒内容:“框架额度余额不足，为xxx元，请注意检查！”
 * 3）配供-项目：当《发货单》【单据类型】=“配供业务” &&【框架简称】为空 &&【项目简称】不为空，根据项目号查《项目配供业务-可放货余额明细表》Y列【额度余额】-本单发货金额小于0给予提醒，提醒内容:“项目额度余额不足，为xxx元，请注意检查！”；
 * @Author LinRui
 * @Date 2023/11/8 10:17
 */
public class SaleShippingSubmitValidator extends AbstractValidator {
    @Override
    public void validate() {
        ExtendedDataEntity[] dataEntities = this.dataEntities;
        for (int i = 0; i < dataEntities.length; i++) {
            ExtendedDataEntity extend = dataEntities[i];
            DynamicObject entity = extend.getDataEntity();
            check(entity, extend);
        }
    }

    public void check(DynamicObject entity, ExtendedDataEntity extend) {
        DynamicObject tpvDepartment = entity.getDynamicObject("tpv_department");
        long depId = 0L;
        if (null != tpvDepartment) {
            depId = tpvDepartment.getLong("id");
        }
        DynamicObject tpvCustomer = entity.getDynamicObject("tpv_customer");
        long customerId = 0L;
        if (null != tpvCustomer) {
            customerId = tpvCustomer.getLong("id");
        }
        DynamicObject tpvBilltype = entity.getDynamicObject("tpv_billtype");
        String billtypeString = "";
        if (YEM.isNotEmpty(tpvBilltype)) {
            billtypeString = tpvBilltype.getString("number");
        }
        long tpvAbbreviationId = 0l;
        String tpvAbbreviation = entity.getString("tpv_abbreviation"); //框架简称
        if (YEM.isNotEmpty(tpvAbbreviation)) {
            DynamicObject tpv_pw_filing_KJ = BusinessDataServiceHelper.loadSingle("tpv_pw_filing", new QFilter[]{new QFilter("tpv_frameworkname", QCP.equals, tpvAbbreviation)});
            if (YEM.isNotEmpty(tpv_pw_filing_KJ)) {
                tpvAbbreviationId = tpv_pw_filing_KJ.getLong("id");
            }
        }
        long tpvProjectId = 0l;
        String tpvProject = entity.getString("tpv_project"); //项目简称
        if (YEM.isNotEmpty(tpvProject)) {
            DynamicObject tpv_pw_filing_XM = BusinessDataServiceHelper.loadSingle("tpv_pw_filing", new QFilter[]{new QFilter("tpv_project", QCP.equals, tpvProject)});
            if (YEM.isNotEmpty(tpv_pw_filing_XM)) {
                tpvProjectId = tpv_pw_filing_XM.getLong("id");
            }
        }
        String tpvContractnumber = entity.getString("tpv_contractnumber"); //合同号
        DynamicObjectCollection tpvMaterialentry = entity.getDynamicObjectCollection("tpv_materialentry");
        BigDecimal tpvAmount = BigDecimal.ZERO;
        for (DynamicObject dynamicObject : tpvMaterialentry) {
            BigDecimal tpvTaxpramount = dynamicObject.getBigDecimal("tpv_taxpramount");
            tpvAmount = tpvAmount.add(tpvTaxpramount);
        }
        long contractId = 0l;
        if (YEM.isNotEmpty(tpvContractnumber)) {
            DynamicObject salescontract = BusinessDataServiceHelper.loadSingle("tpv_dt_salescontract", new QFilter[]{new QFilter("tpv_contractnos", QCP.equals, tpvContractnumber)});
            if (YEM.isNotEmpty(salescontract)) {
                contractId = salescontract.getLong("id");
            }
        }

        if ("tpv_dt_salesshipping_BT".equals(billtypeString) && tpvAbbreviationId > 0l) {
            if (StringUtils.isNotEmpty(tpvAbbreviation)) {
                DataSet dataSet = CusCreditLimitUtils.getBalanceKJLimit(depId, customerId, tpvAbbreviationId);
                DynamicObjectCollection balanceKJ = ORM.create().toPlainDynamicObjectCollection(dataSet);
                BigDecimal sum = BigDecimal.ZERO;
                for (DynamicObject dynamicObject : balanceKJ) {
                    BigDecimal tpvAmt15 = dynamicObject.getBigDecimal("tpv_cuscreditlimit");
                    sum = sum.add(tpvAmt15);
                }
                if (sum.subtract(tpvAmount).compareTo(new BigDecimal(0.02).negate()) < 0) {
                    BigDecimal subtract = sum.subtract(tpvAmount);
                    this.addWarningMessage(extend, "框架额度余额不足，为 " + subtract.stripTrailingZeros().toPlainString() + " 元，请注意检查！");
                }
            } else if (StringUtils.isEmpty(tpvAbbreviation) && tpvProjectId > 0l) {
                DataSet dataSet = CusCreditLimitUtils.getBalanceXMLimit(depId, customerId, tpvProjectId);
                DynamicObjectCollection balanceXM = ORM.create().toPlainDynamicObjectCollection(dataSet);
                BigDecimal sum = BigDecimal.ZERO;
                for (DynamicObject dynamicObject : balanceXM) {
                    BigDecimal tpvAmt15 = dynamicObject.getBigDecimal("tpv_cuscreditlimit");
                    sum = sum.add(tpvAmt15);
                }
                if (sum.subtract(tpvAmount).compareTo(BigDecimal.ZERO) < 0) {
                    BigDecimal subtract = sum.subtract(tpvAmount);
                    this.addWarningMessage(extend, "项目额度余额不足，为 " + subtract.stripTrailingZeros().toPlainString() + " 元，请注意检查！");
                }
            }
        } else if (!"tpv_dt_salesshipping_BT".equals(tpvBilltype) && contractId > 0l) {
            DataSet dataSet = CusCreditLimitUtils.getBalanceConDataLimit(depId, customerId, contractId);
            DynamicObjectCollection balanceCon = ORM.create().toPlainDynamicObjectCollection(dataSet);
            BigDecimal sum = BigDecimal.ZERO;
            for (DynamicObject dynamicObject : balanceCon) {
                BigDecimal tpvAmt15 = dynamicObject.getBigDecimal("tpv_paymentbalance");
                sum = sum.add(tpvAmt15);
            }
            if (sum.subtract(tpvAmount).compareTo(BigDecimal.ZERO) < 0) {
                BigDecimal subtract = sum.subtract(tpvAmount);
                this.addWarningMessage(extend, "可放货余额表为" + sum.stripTrailingZeros().toPlainString() + " 元，本单金额" + tpvAmount.stripTrailingZeros().toPlainString() + "元，货款不足，请注意核对! ");
            }
        }

    }
}
