package tpv.tpv_wm.tpv_dt.java.report;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.DispatchServiceHelper;
import kd.fi.gl.service.BalanceQueryParam;
import tpv.tpv_wm.tpv_crm.java.utils.CusCreditLimitUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import tpv.tpv_wm.tpv_dt.java.utils.PersonnelDepUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @ClassName: ComparisonRptQuery
 * @Description: 客户赊销余额对账表
 * @author: zbh
 * @date: 2024-05-17
 */
public class ComparisonRptQuery extends AbstractReportListDataPlugin {

    private static final String algoKey = ComparisonRptQuery.class.getName();

    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        String whereFilter = createWhereStr(param);
        FilterInfo filterInfo = param.getFilter();
        DynamicObject tpvCompany = filterInfo.getDynamicObject("tpv_company");
        DynamicObject tpvPeriod = filterInfo.getDynamicObject("tpv_period");
        DataSet instockDataSet = query(whereFilter, tpvCompany, tpvPeriod);
        return instockDataSet;
    }

    public static DataSet query(String whereFilter, DynamicObject tpvCompany, DynamicObject tpvPeriod) {
        BigDecimal amountbase = BigDecimal.valueOf(inovkeBiz("2205010102", tpvCompany, tpvPeriod));//合同负债-业务款-国内账款-销售货款
        BigDecimal unpayment = BigDecimal.valueOf(inovkeBiz("2205010103", tpvCompany, tpvPeriod));//合同负债-业务款-国内账款-无归属货款
        BigDecimal depositcontracts = BigDecimal.valueOf(inovkeBiz("2205010101", tpvCompany, tpvPeriod));//合同负债-业务款-国内账款-合同保证金


        DataSet mainDataSet = mainDataSet(whereFilter, tpvCompany).where(whereFilter);

        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        for (DynamicObject col : col_s) {
            BigDecimal tpvOccpayment = col.getBigDecimal("tpv_occpayment");//归属客户损益占用货款
            BigDecimal tpvSalesnotachieved = col.getBigDecimal("tpv_salesnotachieved");//销售未达
            BigDecimal tpvLossmargin = col.getBigDecimal("tpv_lossmargin");//归属客户损益占用保证金
            BigDecimal tpvPaymentgoods = col.getBigDecimal("tpv_paymentgoods");//业务余额 货款合计
            BigDecimal tpvTotalmargins = col.getBigDecimal("tpv_totalmargins");//业务余额 保证金合计
            col.set("tpv_amountbase", amountbase);//合同负债-业务款-国内账款-销售货款
            col.set("tpv_unpayment", unpayment);//合同负债-业务款-国内账款-无归属货款
            col.set("tpv_totalpayment", amountbase.add(unpayment).subtract(tpvOccpayment).subtract(tpvSalesnotachieved));//货款合计
            col.set("tpv_depositcontracts", depositcontracts);//合同负债-业务款-国内账款-合同保证金
            col.set("tpv_totalmargin", depositcontracts.subtract(tpvLossmargin));//财务余额	 保证金合计
            col.set("tpv_totalpaymentgood", amountbase.add(unpayment).subtract(tpvOccpayment).subtract(tpvSalesnotachieved).subtract(tpvPaymentgoods));//差额 货款合计
            col.set("tpv_margin", depositcontracts.subtract(tpvLossmargin).subtract(tpvTotalmargins));//差额 保证金合计
        }
        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
        return createDataSet;
    }


    private static DataSet mainDataSet(String whereFilter, DynamicObject tpvCompany) {

        QFilter purQFilter = new QFilter("billstatus", "=", "C");
//        String tpvSalenum = filterInfo.getString("tpv_contractnumber");
//        if (YEM.isNotEmpty(tpvSalenum)) {
//            purQFilter.and("tpv_contractnumber", QCP.ftlike, "%" + tpvSalenum + "%");
//        }
        //采购合同
//        DataSet tpvDtPurchase = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_purchase", "id,tpv_company tpv_companys,tpv_department tpv_departments,tpv_customer tpv_customers,tpv_contractnumber tpv_contractnumbers", purQFilter.toArray());
        QFilter planQFilter = new QFilter("tpv_invalid", "=", false);
        planQFilter.and("tpv_settlement.tpv_programme.tpv_documenttype", "=", "A");
        //高低计规划单
//        DataSet tpvCmPlannedbills = ORM.create().queryDataSet(this.getClass().getName(), "tpv_cm_plannedbills", "tpv_settlement.tpv_purconums tpv_purconumsh,tpv_settlement.tpv_programme.tpv_documenttype tpv_documenttype,tpv_settlement.tpv_programme.tpv_amount tpv_occpayment",
//                planQFilter.toArray()).groupBy(new String[]{"tpv_occpayment", "tpv_purconumsh"}).sum("tpv_occpayment").finish()
//                .where("tpv_documenttype = 'A'")
        ;

//        DataSet purCmPlann = tpvDtPurchase.leftJoin(tpvCmPlannedbills).on("id", "tpv_purconumsh").select("id", "tpv_companys", "tpv_departments", "tpv_customers", "tpv_contractnumbers", "tpv_occpayment").finish();

        //高低计单
//        DataSet tpvCmUneven = ORM.create().queryDataSet(this.getClass().getName(), "tpv_cm_uneven", "billno,tpv_advconap.tpv_purconums tpv_purconumsg,tpv_advconap.tpv_unplanqty tpv_unplanqty,tpv_exchangeratermb,tpv_advconap.tpv_planqty tpv_planqty",
//                        new QFilter[]{new QFilter("tpv_unevened", "=", "B")})
//                .groupBy(new String[]{"tpv_purconumsg", "billno"}).sum("tpv_unplanqty").finish();//高低计单 低计 未规划金额>0
//        DataSet purCmUneven = purCmPlann.leftJoin(tpvCmUneven).on("id", "tpv_purconumsg").select("id", "tpv_companys", "tpv_departments", "tpv_customers", "tpv_contractnumbers", "tpv_occpayment+tpv_unplanqty  tpv_occpayment").finish();


//        DataSet tpvCmUnevenG = ORM.create().queryDataSet(this.getClass().getName(), "tpv_cm_uneven", "billno billnos,tpv_advconap.tpv_purconums tpv_purconums,tpv_advconap.tpv_unplanqty tpv_unplanqtyG,tpv_exchangeratermb,tpv_advconap.tpv_planqty tpv_planqty",
//                        new QFilter[]{new QFilter("tpv_unevened", "=", "A")})
//                .groupBy(new String[]{"tpv_purconums", "billnos"}).sum("tpv_unplanqtyG").finish();//高低计单 高计 未规划金额>0

//        DataSet purUnevenG = purCmUneven.leftJoin(tpvCmUnevenG).on("id", "tpv_purconums").select("id", "tpv_companys", "tpv_departments", "tpv_customers", "tpv_contractnumbers", "tpv_occpayment-tpv_unplanqtyG  tpv_occpayment").finish();


        QFilter saleQFilter = new QFilter("billstatus", "=", "C");
//        if (YEM.isNotEmpty(tpvSalenum)) {
//            saleQFilter.and("tpv_contractnos", QCP.ftlike, "%" + tpvSalenum + "%");
//        }
        DataSet tpvDtContract = ORM.create().queryDataSet(algoKey, "tpv_dt_salescontract", "id,tpv_company tpv_companys,tpv_department tpv_departments,tpv_customer tpv_customers,tpv_contractnos tpv_contractnumbers", saleQFilter.toArray());

        DataSet saleShipping = ORM.create().queryDataSet(algoKey, "tpv_dt_salesshipping", "tpv_sourcebillid,tpv_materialentry.tpv_taxpramount tpv_taxpramount,tpv_materialentry.tpv_dt_salepick_in.tpv_invoiceamt tpv_invoiceamt", null)
                .groupBy(new String[]{"tpv_sourcebillid", "tpv_taxpramount"}).sum("tpv_invoiceamt").finish().select("tpv_taxpramount-tpv_invoiceamt tpv_salesnotachieved", "tpv_sourcebillid").groupBy(new String[]{"tpv_sourcebillid"}).sum("tpv_salesnotachieved").finish();

        DataSet tpvDtSalescontract = tpvDtContract.leftJoin(saleShipping).on("id", "tpv_sourcebillid").select("id", "tpv_companys", "tpv_departments", "tpv_customers", "tpv_contractnumbers", "tpv_salesnotachieved").finish();
        //高低计规划单
        DataSet saleCmPlannedbills = ORM.create().queryDataSet(algoKey, "tpv_cm_plannedbills", "tpv_settlement.tpv_saleconums tpv_purconumsh,tpv_settlement.tpv_programme.tpv_documenttype tpv_documenttype,tpv_settlement.tpv_programme.tpv_amount tpv_occpayment",
                planQFilter.toArray()).groupBy(new String[]{"tpv_occpayment", "tpv_purconumsh"}).sum("tpv_occpayment").finish();

        DataSet salenneBills = tpvDtSalescontract.leftJoin(saleCmPlannedbills).on("id", "tpv_purconumsh").select("tpv_salesnotachieved", "id", "tpv_companys", "tpv_departments", "tpv_customers", "tpv_contractnumbers", "tpv_occpayment").finish();

        QFilter plansaleFilter = new QFilter("tpv_invalid", "=", false);
        plansaleFilter.and("tpv_settlement.tpv_programme.tpv_documenttype", "=", "B");
        //高低计规划单
        DataSet tpvsalennedbills = ORM.create().queryDataSet(algoKey, "tpv_cm_plannedbills", "tpv_settlement.tpv_saleconums tpv_saleconums,tpv_settlement.tpv_programme.tpv_documenttype tpv_documenttype,tpv_settlement.tpv_programme.tpv_amount tpv_lossmargin",
                plansaleFilter.toArray()).groupBy(new String[]{"tpv_lossmargin", "tpv_saleconums"}).sum("tpv_lossmargin").finish();

        DataSet saleCmPlann = salenneBills.leftJoin(tpvsalennedbills).on("id", "tpv_saleconums").select("tpv_salesnotachieved", "id", "tpv_companys", "tpv_departments", "tpv_customers", "tpv_contractnumbers", "tpv_occpayment", "tpv_lossmargin").finish();
        //高低计单
        DataSet saleCmUneven = ORM.create().queryDataSet(algoKey, "tpv_cm_uneven", "billno,tpv_advconap.tpv_saleconums tpv_purconumsg,tpv_advconap.tpv_unplanqty tpv_unplanqty,tpv_exchangeratermb,tpv_advconap.tpv_planqty tpv_planqty",
                        new QFilter[]{new QFilter("tpv_unevened", "=", "B")})
                .groupBy(new String[]{"tpv_purconumsg", "tpv_purconumsg"}).sum("tpv_unplanqty").finish();//高低计单 低计 未规划金额>0
        DataSet sale = saleCmPlann.leftJoin(saleCmUneven).on("id", "tpv_purconumsg").select("tpv_lossmargin", "tpv_salesnotachieved", "id", "tpv_companys", "tpv_departments", "tpv_customers", "tpv_contractnumbers", "tpv_occpayment+tpv_unplanqty  tpv_occpayment").finish();


        DataSet saleCmUnevenG = ORM.create().queryDataSet(algoKey, "tpv_cm_uneven", "billno billnos,tpv_advconap.tpv_saleconums tpv_purconums,tpv_advconap.tpv_unplanqty tpv_unplanqtyG,tpv_exchangeratermb,tpv_advconap.tpv_planqty tpv_planqty",
                        new QFilter[]{new QFilter("tpv_unevened", "=", "A")})
                .groupBy(new String[]{"tpv_purconums", "tpv_purconums"}).sum("tpv_unplanqtyG").finish();//高低计单 高计 未规划金额>0

        DataSet saleUnevenG = sale.leftJoin(saleCmUnevenG).on("id", "tpv_purconums").select("tpv_lossmargin", "tpv_salesnotachieved", "id", "tpv_companys", "tpv_departments", "tpv_customers", "tpv_contractnumbers", "tpv_unplanqtyG","tpv_occpayment").finish();
//        return alescontract.union(finished);
        DataSet tpvPaymentbalance = CusCreditLimitUtils.getDeliverableBalanceCons(0L, 0L, 0L).select("tpv_contractno,tpv_paymentbalance tpv_paymentgoods,tpv_totalrcvdeposit tpv_totalmargins").groupBy(new String[]{"tpv_contractno", "tpv_paymentgoods", "tpv_totalmargins"}).sum("tpv_paymentgoods").sum("tpv_totalmargins").finish();
        DataSet finished = saleUnevenG.leftJoin(tpvPaymentbalance).on("tpv_contractnumbers", "tpv_contractno")
                .select("tpv_lossmargin", "tpv_lossmargin tpv_depositcontracts", "tpv_lossmargin tpv_totalpayment", "tpv_lossmargin tpv_unpayment", "tpv_lossmargin tpv_amountbase", "tpv_salesnotachieved", "id", "tpv_companys", "tpv_departments","tpv_occpayment-tpv_unplanqtyG tpv_occpayment",
                        "tpv_customers", "tpv_contractnumbers", "tpv_paymentgoods", "tpv_lossmargin tpv_totalmargin", "tpv_lossmargin tpv_totalpaymentgood", "tpv_lossmargin tpv_margin", "tpv_totalmargins").finish();
        return finished;
    }

    public static double inovkeBiz(String number, DynamicObject tpvCompany, DynamicObject tpvPeriod) {
        double sum = 0;
        if (tpvCompany != null && tpvPeriod != null) {
            BalanceQueryParam param = new BalanceQueryParam();//gl_rpt_accountbalance
            //科目   会计科目
            DynamicObject accountview = BusinessDataServiceHelper.loadSingle("bd_accountview", new QFilter[]{new QFilter("number", "=", number)});//
            //账簿类型  账簿类型
            DynamicObject stype = BusinessDataServiceHelper.loadSingle("bd_accountbookstype", new QFilter[]{new QFilter("number", "=", "100001")});
            //科目表 科目表
            DynamicObject table = BusinessDataServiceHelper.loadSingle("bd_accounttable", new QFilter[]{new QFilter("number", "=", "AVICCOA-0001")});

            param.addSelector("beginfor");
            param.addSelector("beginlocal");
            param.addSelector("beginqty");
            param.addSelector("debitfor");
            param.addSelector("debitlocal");
            param.addSelector("creditfor");
            param.addSelector("creditlocal");
            param.addSelector("debitqty");
            param.addSelector("creditqty");
            param.addSelector("yeardebitfor");
            param.addSelector("yeardebitlocal");
            param.addSelector("yearcreditfor");
            param.addSelector("yearcreditlocal");
            param.addSelector("yeardebitqty");
            param.addSelector("yearcreditqty");
            param.addSelector("endfor");
            param.addSelector("endlocal");
            param.addSelector("endqty");

//        param.addGroupBy("f000101");
            param.setOrgId(tpvCompany.getLong("id"));//核算组织编码
            param.setBookTypeId(stype.getLong("id"));//账簿类型编码
            param.setAccountTableId(table.getLong("id"));//科目表
            param.setPeriodId(tpvPeriod.getLong("id"));//期间编码
            List<Map<String, Long>> list = new ArrayList<>();
            HashMap<String, List<Long>> hashMap = new HashMap<>();//[] {"yeardebitlocal":0.00} 1344142747925218304
            Map<String, Long> flex = new HashMap<>();
//            flex.put("f000101", null);//部门
////        flex.put("f000301",null);//商务伙伴
////        flex.put("f000502",null);//销售合同号
////        flex.put("f000503",null);//采购合同号
            list.add(flex);
            param.setAccountAssgrp(accountview.getLong("id"), null);//科目维度
            String result = DispatchServiceHelper.invokeBizService("fi", "gl", "BalanceService", "getBalance", param.toString());

            try {
                if (result.length() > 2) {
                    JSONArray jsonArray = new JSONArray(result);
                    for (int i = 0; i < jsonArray.length(); i++) {
                        JSONObject jsonObject = jsonArray.getJSONObject(i);
                        if (jsonObject.has("endfor")) {
                            double value = jsonObject.getDouble("endfor");
                            sum += value;
                        }
                    }
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }
//            try {
//                if (result.length() > 2) {
//                    ObjectMapper objectMapper = new ObjectMapper();
//                    List<Object> data = objectMapper.readValue(result, List.class);
//                    for (Object obj : data) {
//                        if (obj instanceof java.util.Map) {
//                            Map<String, Double> map = (Map<String, Double>) obj;
//                            if (map.containsKey("endfor")) {
//                                sum += map.get("endfor");
//                            }
//                        }
//                    }
//                }
//            } catch (Exception e) {
//                e.printStackTrace();
//            }
        }//endfor-7.773881052499999E8   endlocal-7.773881052499999E8  beginfor-7.773881052499999E8 debitfor
        return sum;
    }

    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        DynamicObject tpvCompany = filterInfo.getDynamicObject("tpv_company");
        DynamicObject tpvDepartment = filterInfo.getDynamicObject("tpv_department");
        String tpvContractnumber = filterInfo.getString("tpv_contractnumber");
        DynamicObject tpvCustomer = filterInfo.getDynamicObject("tpv_customer");
        if (tpvCompany != null) {
            whereFilter = String.format("%s and tpv_companys = %s", whereFilter, tpvCompany.getPkValue());
        }
        if (tpvDepartment != null) {
            whereFilter = String.format("%s and tpv_departments = %s", whereFilter, tpvDepartment.getPkValue());
        }
        if (YEM.isNotEmpty(tpvContractnumber)) {
            whereFilter = String.format("%s and tpv_contractnumbers like '%%%s%%'", whereFilter, tpvContractnumber);
        }
        if (tpvCustomer != null) {
            whereFilter = String.format("%s and tpv_customers = %s", whereFilter, tpvCustomer.getPkValue());
        }
        String whereStr = PersonnelDepUtils.createWhereStr(filterInfo,"tpv_departments", "tpv_muldepartment");
        if (!"".equals(whereStr)){
            whereFilter=String.format("%s%s", whereFilter, whereStr);
        }
        return whereFilter;
    }
}

