package tpv.tpv_wm.tpv_dt.java.report;

import kd.bos.algo.*;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import tpv.tpv_wm.tpv_dt.java.utils.PersonnelDepUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static kd.fi.er.report.query.TripOrderRpt.algoKey;

/**
 * @author l j w
 * @date 2023/8/24 11:23
 * 保管方库存表报表插件
 * @className Custodytable
 * @description
 */
public class CustodytableRptQuery extends AbstractReportListDataPlugin {
    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        FilterInfo filterInfo = param.getFilter();//获取所有的过滤条件
        String contractnumber = filterInfo.getString("tpv_contractnumber");//合同号
        String movement = filterInfo.getString("tpv_movement");//货物流转单号
        DynamicObject custodiancar = filterInfo.getDynamicObject("tpv_custodiancar");//保管/承运方
        BigDecimal quantity = filterInfo.getBigDecimal("tpv_quantity");
        QFilter QFilter = new QFilter("billstatus", QCP.equals, "C");//查询单据状态为已审核的
        if (null != custodiancar) {
            QFilter.and("tpv_custcarrier.id", QCP.equals, custodiancar.getLong("id"));//查询保管/承运方等于保管/承运方的
        }
        if (null != contractnumber) {
            QFilter.and("tpv_purcontnum", QCP.ILIKE, "%" + contractnumber + "%");
        }
        if (null != movement) {
            QFilter.and("billno", QCP.like, "%" + movement + "%");//查询单据编号中包含货物流转单单号的
        }
        QFilter whereStrQF = PersonnelDepUtils.createWhereStrQF(filterInfo, "tpv_department", "tpv_muldepartment");
        if (whereStrQF!=null){
            QFilter.and(whereStrQF);
        }
        String selelctFiles = "tpv_custcarrier  tpv_custodiancarrier," +
                "tpv_purcontnum tpv_contract," +
                "tpv_roamaction tpv_flowaction," +
                "tpv_materialentry.tpv_material tpv_productid," +
                "tpv_materialentry.tpv_unit tpv_unitofmeasure," +
                "tpv_materialentry.tpv_qty tpv_qty," +
                "tpv_materialentry.tpv_alreroamnum tpv_flowed," +
                "tpv_materialentry.tpv_tinnum tpv_inbound," +
                "tpv_materialentry.tpv_notroamnum tpv_uncirculated"; // 查询字段
        DataSet dataSet = QueryServiceHelper.queryDataSet(this.getClass().getName(), "tpv_dt_freigroambill", selelctFiles,
                QFilter.toArray(), null);//根据单据编号查询
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(dataSet);
        List<Field> fields = new ArrayList<>();
        List<Object[]> allrows = new ArrayList<>();
        fields.add(new Field("tpv_custodiancarrier", DataType.LongType));// 保管/承运方
        fields.add(new Field("tpv_contract", DataType.StringType));// 合同号
        fields.add(new Field("tpv_flowaction", DataType.StringType));// 流转动作
        fields.add(new Field("tpv_productid", DataType.LongType));// 商品编码
        fields.add(new Field("tpv_unitofmeasure", DataType.LongType));// 计量单位
        fields.add(new Field("tpv_qty", DataType.BigDecimalType));// 数量
        fields.add(new Field("tpv_uncirculated", DataType.BigDecimalType));// 未流转数量
        fields.add(new Field("tpv_flowed", DataType.BigDecimalType));// 已流转数量
        fields.add(new Field("tpv_inbound", DataType.BigDecimalType));// 已入库数量
        int size = fields.size();// DataSet字段数
        if (size > 0) {
                allrows = toallrows(col_s, size);
        }
        //创建dataset
        CollectionInput collectionInput = new CollectionInput(new RowMeta(fields.toArray(new Field[0])), allrows);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(collectionInput);
        return createDataSet;
//        return dataSet;
    }

    private List<Object[]> toallrows(DynamicObjectCollection col_s, int size) {
        List<Object[]> allrows = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject map = col_s.get(i);
            Object[] array = new Object[size];
            array[0] = map.getLong("tpv_custodiancarrier");//保管/承运方
            array[1] = map.getString("tpv_contract");// 合同号
            array[2] = map.getString("tpv_flowaction");// 流转动作
            array[3] = map.getLong("tpv_productid");// 商品编码
            array[4] = map.getLong("tpv_unitofmeasure");// 计量单位
            BigDecimal qty = map.getBigDecimal("tpv_qty");// 数量
            BigDecimal flowed = map.getBigDecimal("tpv_flowed");// 已流转数量
            BigDecimal inbound = map.getBigDecimal("tpv_inbound");//已入库数量
            BigDecimal uncirculated = Uncirculated(qty, flowed, inbound);//计算未流转数量
            array[5] = qty;
            array[6] = uncirculated;//未流转数量
            array[7] = flowed;
            array[8] = inbound;
            if (uncirculated.compareTo(BigDecimal.ZERO) == 1){
                allrows.add(array);
            }
        }
            return allrows;

    }
    public BigDecimal Uncirculated(BigDecimal qty, BigDecimal flowed, BigDecimal inbound) {
        BigDecimal comfa = qty.subtract(flowed).subtract(inbound);
    return comfa;
    }
}


