package tpv.tpv_wm.tpv_dt.java.report;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.ArrayList;
import java.util.Collection;

/**
 * @ClassName: DetaileGenTableRptQuery
 * @Description:  按销售量计服务费总表
 * @author: zbh
 * @date: 2024-07-29
 */
public class DetaileGenTableRptQuery extends AbstractReportListDataPlugin {

    private static final String algoKey = DetaileGenTableRptQuery.class.getName();
    
    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        String whereFilter = createWhereStr(param);
        DataSet instockDataSet = query(whereFilter, param);
        return instockDataSet;
    }
    
    public static DataSet query(String whereFilter, ReportQueryParam param) {
        DataSet mainDataSet = mainDataSet().where(whereFilter);

        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
        return createDataSet;
    }
    public static DataSet mainDataSet() {
        DynamicObject transferType = BusinessDataServiceHelper.loadSingle("shkd_transfertype", new QFilter[]{new QFilter("number", "=", "1010")});
        QFilter qFilter = new QFilter("entryentity.shkd_transfertype", "=", transferType == null ? 0L : transferType.getPkValue());
        qFilter.and("billstatus", "=", "C");
        DataSet shkdCrosdeparfunproallo = ORM.create().queryDataSet(algoKey, "shkd_crosdeparfunproallo", "shkd_businessiunit,entryentity.shkd_transferamount transferamount,entryentity.shkd_outsalescontract outsalescontract,entryentity.shkd_outprocurecontract outprocurecontract", qFilter.toArray());
        DataSet tpvDtPurchase = ORM.create().queryDataSet(algoKey, "tpv_dt_purchase", "id,tpv_combinationnumber", null);
        shkdCrosdeparfunproallo = shkdCrosdeparfunproallo.leftJoin(tpvDtPurchase).on("outprocurecontract", "id").select(new String[]{"shkd_businessiunit", "transferamount tpv_conamounted", "outprocurecontract", "outsalescontract", "tpv_combinationnumber"}).finish();
        DataSet tpvBdCombination = ORM.create().queryDataSet(algoKey, "tpv_bd_combination", "id,name tpv_comstring", null);
        shkdCrosdeparfunproallo = shkdCrosdeparfunproallo.leftJoin(tpvBdCombination).on("tpv_combinationnumber", "id").select(new String[]{"shkd_businessiunit", "tpv_conamounted", "tpv_comstring"}).finish();
        shkdCrosdeparfunproallo = shkdCrosdeparfunproallo.groupBy(new String[]{"tpv_comstring", "shkd_businessiunit"}).sum("tpv_conamounted").finish();


        DataSet simOriginalBill = DetailedSerRptQuery.mainDataSet().select(new String[]{"tpv_department","tpv_comstring","tpv_num","tpv_condepartment","tpv_conpro","tpv_conamount"}).groupBy(new String[]{"tpv_comstring","tpv_department", "tpv_condepartment","tpv_conpro","tpv_conamount"}).sum("tpv_num").sum("tpv_conamount").finish();
        simOriginalBill.leftJoin(shkdCrosdeparfunproallo).on("tpv_comstring","tpv_comstring").select(new String[]{"tpv_comstring","tpv_department", "tpv_condepartment","tpv_conpro","tpv_conamounted tpv_fcamount","tpv_conamount-tpv_conamounted tpv_residueamt"}).finish();
        return simOriginalBill;
    }
    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        DynamicObject tpvOrg = filterInfo.getDynamicObject("tpv_org");//业务部门
        if (tpvOrg!=null){
            whereFilter = String.format("%s and tpv_department = %s", whereFilter, tpvOrg.getPkValue());
        }
        DynamicObject tpvDepartmentl = filterInfo.getDynamicObject("tpv_departmentl");//分成部门
        if (tpvDepartmentl!=null){
            whereFilter = String.format("%s and tpv_condepartment = %s", whereFilter, tpvOrg.getPkValue());
        }
        String tpvName = filterInfo.getString("tpv_name");//计奖卡号
        if (YEM.isNotEmpty(tpvName)){
            whereFilter = String.format("%s and tpv_comstring like '%%%s%%'", whereFilter, tpvName);
        }
        boolean tpvCtmon = filterInfo.getBoolean("tpv_ctmon");//剩余金额>0
        if (tpvCtmon){
            whereFilter = String.format("%s and tpv_residueamt > %s", whereFilter, "0");
        }
        boolean tpvOndep = filterInfo.getBoolean("tpv_ondep");//仅看需跨部门分成
        if (tpvOndep){
            whereFilter = String.format("%s and tpv_department <> %s", whereFilter,
                    String.join(",", "tpv_condepartment"));
        }
        return whereFilter;
    }
}
