package tpv.tpv_wm.tpv_dt.java.report;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

/**
 * @ClassName: DetailedSerRptQuery
 * @Description: 按销售量计服务费明细表
 * 取数源为：《开票申请单》明细，且按该明细所属计奖卡号上定义的【分成方式】=“按销售量计服务费”的行产生（即允许有多个部门对同一笔发票明细多次计费）。
 * 【服务费单价(元/每吨)】计奖卡号上定义的【分成方式】=“按销售量计服务费”的数据行的服务费单价
 * 【应分成金额】=【服务费单价(元/每吨)】*【数量】
 * @author: zbh
 * @date: 2024-07-22
 */
public class DetailedSerRptQuery extends AbstractReportListDataPlugin {

    private static final String algoKey = DetailedSerRptQuery.class.getName();

    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        String whereFilter = createWhereStr(param);
        DataSet instockDataSet = query(whereFilter, param);
        return instockDataSet;
    }

    public static DataSet query(String whereFilter, ReportQueryParam param) {
        DataSet mainDataSet = mainDataSet().where(whereFilter);

        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
        return createDataSet;
    }

    public static DataSet mainDataSet() {
        DataSet simOriginalBill = ORM.create().queryDataSet(algoKey, "sim_original_bill",
                "tpv_reqdepartment tpv_department,sim_original_bill_item.tpv_salescontract tpv_salescontract,sim_original_bill_item.tpv_delivernumd iddelivernumd,sim_original_bill_item.tpv_prodname tpv_prodname,sim_original_bill_item.tpv_specation tpv_specation," +
                        "sim_original_bill_item.tpv_shader tpv_shader,sim_original_bill_item.tpv_origin tpv_origin,sim_original_bill_item.num tpv_num,billno tpv_salesim,billdate tpv_signingsim", null);
        DataSet tpvDtSalesshipping = ORM.create().queryDataSet(algoKey, "tpv_dt_salesshipping", "billno tpv_delivernumd,tpv_bizdate tpv_delivedate,id", null);
        simOriginalBill=simOriginalBill.leftJoin(tpvDtSalesshipping).on("iddelivernumd","id").select("tpv_delivernumd","tpv_delivedate","tpv_department","tpv_salescontract","tpv_prodname","tpv_specation","tpv_shader","tpv_origin","tpv_num","tpv_salesim","tpv_signingsim").finish();
        DataSet tpvDtSalescontract = ORM.create().queryDataSet(algoKey, "tpv_dt_salescontract", "tpv_combinationnumber.name combiname,tpv_contractnos tpv_saleconumhb,tpv_customer tpv_clienthb,id,tpv_signingdate", null);
        simOriginalBill=simOriginalBill.leftJoin(tpvDtSalescontract).on("tpv_salescontract","id").select("combiname","tpv_delivernumd","tpv_delivedate","tpv_department","tpv_prodname","tpv_specation","tpv_shader","tpv_origin","tpv_num","tpv_salesim","tpv_signingsim","tpv_signingdate",
                "tpv_saleconumhb","tpv_clienthb").finish();
        DataSet tpvBdCombination = ORM.create().queryDataSet(algoKey, "tpv_bd_combination", "id,tpv_splitdepartment.tpv_departmententry tpv_condepartment,tpv_splitdepartment.tpv_apercentage tpv_conpro,name tpv_comstring,tpv_splitdepartment.tpv_allocationmethod tpv_allocationmethod", null).where("tpv_allocationmethod='B'");
        simOriginalBill=simOriginalBill.leftJoin(tpvBdCombination).on("combiname","tpv_comstring").select("tpv_comstring","combiname","tpv_delivernumd","tpv_delivedate","tpv_department","tpv_prodname","tpv_specation","tpv_shader","tpv_origin","tpv_num","tpv_salesim","tpv_signingsim","tpv_signingdate",
                "tpv_saleconumhb","tpv_clienthb","tpv_condepartment","tpv_conpro","tpv_conpro*tpv_num tpv_conamount").finish();
        return simOriginalBill;
    }

    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        DynamicObject tpvRepdep = filterInfo.getDynamicObject("tpv_repdep");//业务部门
        if (tpvRepdep!=null){
            whereFilter = String.format("%s and tpv_department = %s", whereFilter, tpvRepdep.getPkValue());
        }
        DynamicObject tpvCondep = filterInfo.getDynamicObject("tpv_condep");//分成部门
        if (tpvRepdep!=null){
            whereFilter = String.format("%s and tpv_condepartment = %s", whereFilter, tpvCondep.getPkValue());
        }
        String tpvConbation = filterInfo.getString("tpv_conbation");//计奖卡号
        if (YEM.isNotEmpty(tpvConbation)){
            whereFilter = String.format("%s and tpv_comstring like '%%%s%%'", whereFilter, tpvConbation);
        }
        String tpvSalenumber = filterInfo.getString("tpv_salenumber");//销售合同
        if (YEM.isNotEmpty(tpvSalenumber)){
            whereFilter = String.format("%s and tpv_saleconumhb like '%%%s%%'", whereFilter, tpvSalenumber);
        }
        DynamicObject tpvCustom = filterInfo.getDynamicObject("tpv_custom");//销售客户
        if (tpvCustom!=null){
            whereFilter = String.format("%s and tpv_clienthb = %s", whereFilter, tpvCustom.getPkValue());
        }
        Date tpvStart = filterInfo.getDate("tpv_start");//签约日期从
        if (tpvStart!=null){
            SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String format = str.format(tpvStart);
            whereFilter = String.format("%s and tpv_signingdate >= to_date('" + format + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        Date tpvEnd = filterInfo.getDate("tpv_end");//签约日期到
        if (tpvStart!=null){
            SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String format = str.format(tpvEnd);
            whereFilter = String.format("%s and tpv_signingdate >= to_date('" + format + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        String tpvRec = filterInfo.getString("tpv_rec");//销售发票
        if (YEM.isNotEmpty(tpvRec)){
            whereFilter = String.format("%s and tpv_salesim like '%%%s%%'", whereFilter, tpvRec);
        }
        Date tpvRecstart = filterInfo.getDate("tpv_recstart");//开票日期从
        if (tpvStart!=null){
            SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String format = str.format(tpvRecstart);
            whereFilter = String.format("%s and tpv_signingsim >= to_date('" + format + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        Date tpvRecend = filterInfo.getDate("tpv_recend");//开票日期到
        if (tpvStart!=null){
            SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String format = str.format(tpvRecend);
            whereFilter = String.format("%s and tpv_signingsim >= to_date('" + format + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        String tpvShipnum = filterInfo.getString("tpv_shipnum");//发货单号
        if (YEM.isNotEmpty(tpvShipnum)){
            whereFilter = String.format("%s and tpv_delivernumd like '%%%s%%'", whereFilter, tpvRec);
        }
        Date tpvShipstart = filterInfo.getDate("tpv_shipstart");//发货日期从
        if (tpvStart!=null){
            SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String format = str.format(tpvShipstart);
            whereFilter = String.format("%s and tpv_delivedate >= to_date('" + format + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        Date tpvShipend = filterInfo.getDate("tpv_shipend");//发货日期到
        if (tpvStart!=null){
            SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String format = str.format(tpvShipend);
            whereFilter = String.format("%s and tpv_delivedate >= to_date('" + format + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        return whereFilter;
    }
}
