package tpv.tpv_wm.tpv_dt.java.report;

import com.itopview.wm.utils.DynamicObjectUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 采购价格确认工作台
 * @description 查询插件
 * @author GX
 */
public class PurPriceRptQuery extends AbstractReportListDataPlugin{
	private static List<String> fieldKeys_pur = new ArrayList<>();// 采购合同
	private static List<String> fieldKeys_car = new ArrayList<>();// 货物流转
	private static List<String> fieldKeys_purin = new ArrayList<>();// 采购入库

	@Override
	public DataSet query(ReportQueryParam param, Object obj) throws Throwable {
		String whereFilter = createWhereStr(param);
		DataSet instockDataSet = query(whereFilter);
		return instockDataSet;
	}
	
	public static DataSet query(String whereFilter) {
		DataSet mainDataSet = mainDataSet(whereFilter);
		DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
		String selectFk=DynamicObjectUtil.getSelectfields("tpv_dt_purchase", false);
		DynamicObject[] fkcl = BusinessDataServiceHelper.load("tpv_dt_purchase",selectFk,
				new QFilter[] {});// 查询同一合同下数据；
		Map<String, List<DynamicObject>> billMap = Stream.of(fkcl).collect(Collectors.groupingBy(x -> x.getString("id")));
		for (DynamicObject col : col_s) {
			String tpv_id = col.getString("tpv_id");//单据id
			Long policynumber=null;
			String pricingtype=null;
			List<DynamicObject> billList = billMap.get(tpv_id);
			if(billList!=null){
				for(DynamicObject fk:billList) {
					policynumber=fk.getLong("tpv_policynumber.id");//策略号
					pricingtype=fk.getString("tpv_pricingtype");//定价类型
				}
			}
			col.set("tpv_policynumber", policynumber);
			col.set("tpv_pricingtype", pricingtype);
		}

		RowMeta rowMeta = mainDataSet.getRowMeta();
		int fieldCount = rowMeta.getFieldCount();
		Collection<Object[]> collection = new ArrayList<>();
		for (int i = 0; i < col_s.size(); i++) {
			DynamicObject col = col_s.get(i);
			Object[] obj = new Object[fieldCount];
			for (int j = 0; j < fieldCount; j++) {
				obj[j] = col.get(j);
			}
			collection.add(obj);
		}
		CollectionInput inputs = new CollectionInput(rowMeta, collection);
		DataSet createDataSet = Algo.create(PurPriceRptQuery.class.getName()).createDataSet(inputs);
		return createDataSet.where("tpv_qty > 0 and tpv_billstatus = 'C'").orderBy(new String[] { "tpv_bizdate desc" });
	}
	
	
	private static DataSet mainDataSet(String whereFilter) {
		initAllFields();
		QFilter[] qFilters = null;
		DataSet tpv_dt_purchase = ORM.create().queryDataSet(PurPriceRptQuery.class.getName(), "tpv_dt_purchase",
				String.join(",", fieldKeys_pur), qFilters);
		DataSet tpv_dt_freigroambill = ORM.create().queryDataSet(PurPriceRptQuery.class.getName(), "tpv_dt_freigroambill",
				String.join(",", fieldKeys_car),qFilters);
		DataSet im_purinbill = ORM.create().queryDataSet(PurPriceRptQuery.class.getName(), "im_purinbill",
				String.join(",", fieldKeys_purin), qFilters);
		DataSet dataSet = tpv_dt_purchase.union(tpv_dt_freigroambill,im_purinbill);
		if (YEM.isNotEmpty(whereFilter)) {
			dataSet = dataSet.where(whereFilter);
		}
		return dataSet;
	}
	
	private String createWhereStr(ReportQueryParam param) {
		FilterInfo filterInfo = param.getFilter();
		String whereFilter = "1 = 1";
		// 公司
		DynamicObject tpv_orgs = filterInfo.getDynamicObject("tpv_orgs");
		if (YEM.isNotEmpty(tpv_orgs)) {
			whereFilter = String.format("%s and tpv_org = %s", whereFilter, tpv_orgs.getPkValue());
		}
		// 组合号
		DynamicObject tpv_combinationnumbers = filterInfo.getDynamicObject("tpv_combinationnumbers");
		if (YEM.isNotEmpty(tpv_combinationnumbers)) {
			whereFilter = String.format("%s and tpv_combinationnumber = %s", whereFilter, tpv_combinationnumbers.getPkValue());
		}
		return whereFilter;
	}
	
	
	private static void initAllFields() {
		fieldKeys_pur.clear();
		fieldKeys_car.clear();
		fieldKeys_purin.clear();
		addFields("tpv_source", "'A'", "'B'", "'C'");// 来源单据
		addFields("tpv_srcbilltype", "tpv_document", "tpv_billtype", "billtype");// 单据类型
		addFields("tpv_srcbillno", "billno");// 来源单据编号
		addFields("tpv_srcbillid","id");// 来源单据ID
		addFields("tpv_srcentryid","tpv_materialentry","tpv_materialentry","billentry");// 来源单据分录ID
		addFields("tpv_org", "org", "org","bizorg");//公司
		addFields("tpv_department", "tpv_department", "tpv_department","bizdept");//部门
		addFields("tpv_customer", "tpv_supplier","tpv_supplier","supplier");//供应商
		addFields("tpv_policynumber", "tpv_policynumber","null","null");// 策略号
		addFields("tpv_combinationnumber", "tpv_combinationnumber");// 组合号
		addFields("tpv_billno", "tpv_contractnumber","tpv_purcontnum","tpv_purcontnum");// 合同号
		addFields("tpv_whetherprice", "tpv_whetherprice");// 是否点价
		addFields("tpv_pricingtype", "tpv_pricingtype", "tpv_pricingtype", "null");// 定价类型
		addFields("tpv_postpricingrules", "tpv_postpricingrules");// 后定价规则
		addFields("tpv_pointpriceperiod", "tpv_pointpriceperiod");//点价期限
		addFields("tpv_pointpricecontract", "tpv_pointpricecontract");// 点价合约
		addFields("tpv_literdiscount", "tpv_literdiscount");// 升贴水
		addFields("tpv_material", "tpv_materialentry.tpv_material.name", "tpv_materialentry.tpv_material.name","billentry.materialname");// 商品编码
		addFields("tpv_materialname", "tpv_materialentry.tpv_material.name", "tpv_materialentry.tpv_material.name","billentry.materialname");// 商品名称
		addFields("tpv_prodname","tpv_materialentry.tpv_prodname","tpv_materialentry.tpv_prodname","billentry.tpv_prodname");// 品名
		addFields("tpv_origin","tpv_materialentry.tpv_origin","tpv_materialentry.tpv_origin","billentry.tpv_origin");// 产地
		addFields("tpv_shader","tpv_materialentry.tpv_shader","tpv_materialentry.tpv_shader","billentry.tpv_shader");// 材质
		addFields("tpv_specation", "tpv_materialentry.tpv_specation","tpv_materialentry.tpv_specation","billentry.tpv_specation");// 规格
		addFields("tpv_unit", "tpv_materialentry.tpv_unit","tpv_materialentry.tpv_unit","billentry.unit");//计量单位
		addFields("tpv_qty", "tpv_materialentry.tpv_effectiveqty - tpv_materialentry.tpv_flowedquantity",
				"tpv_materialentry.tpv_qty - tpv_materialentry.tpv_alreroamnum - tpv_materialentry.tpv_tinnum",
				"billentry.qty - billentry.tpv_alreroamnum");// 数量
		addFields("tpv_taxprice", "tpv_materialentry.tpv_taxprice","tpv_materialentry.tpv_taxprice","billentry.priceandtax");// 原单含税单价
		addFields("tpv_taxpramount", "tpv_materialentry.tpv_taxpramount","tpv_materialentry.tpv_taxpramount","billentry.amountandtax");// 含税金额
		addFields("tpv_priceconfirm", "tpv_materialentry.tpv_priceconfirm","tpv_materialentry.tpv_priceconfirm","billentry.tpv_priceconfirmation");// 价格确认
		addFields("tpv_bizdate", "tpv_bizdate","tpv_bizdate","biztime");// 业务日期
		addFields("tpv_id","id","tpv_corebillid","tpv_corebillid");//核心单据id
		addFields("tpv_billstatus", "billstatus");// 单据状态
	}

	private static void addFields(String fieldKey, String fieldKey_bill) {
		fieldKeys_pur.add(fieldKey_bill + " " + fieldKey);
		fieldKeys_car.add(fieldKey_bill + " " + fieldKey);
		fieldKeys_purin.add(fieldKey_bill + " " + fieldKey);
	}

	private static void addFields(String fieldKey, String fieldKey_pur, String fieldKey_car, String fieldKey_purin) {
		fieldKeys_pur.add(fieldKey_pur+" "+fieldKey);
		fieldKeys_car.add(fieldKey_car+" "+fieldKey);
		fieldKeys_purin.add(fieldKey_purin+" "+fieldKey);
	}

}
