package tpv.tpv_wm.tpv_dt.java.report;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

/**
 * @ClassName: ScheduleRptQuery
 * @Description: 销项税明细表
 * @author: zbh
 * @date: 2024-05-29
 */
public class ScheduleRptQuery extends AbstractReportListDataPlugin {

    private static final String algoKey = ScheduleRptQuery.class.getName();

    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        String whereFilter = createWhereStr(param);
        DataSet instockDataSet = query(whereFilter, param);
        return instockDataSet;
    }

    public static DataSet query(String whereFilter, ReportQueryParam param) {
        DataSet mainDataSet = mainDataSet().where(whereFilter);

        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
        return createDataSet;
    }

    public static DataSet mainDataSet() {
        String selelctFiles = "tpv_reqdepartment tpv_department,billno tpv_billno,billdate tpv_billdate,sim_original_bill_item.expenseitem tpv_costitem,sim_original_bill_item.tpv_combonum tpv_coms,sim_original_bill_item.tpv_salescontract tpv_saleconum," +
                "sim_original_bill_item.tpv_delivernumd tpv_deliver,sim_original_bill_item.tpv_prodname tpv_prodname,sim_original_bill_item.tpv_origin tpv_origin,sim_original_bill_item.tpv_shader tpv_shader,sim_original_bill_item.tpv_specation tpv_specation," +
                "sim_original_bill_item.num tpv_num,sim_original_bill_item.taxunitprice tpv_taxprice,sim_original_bill_item.taxamount tpv_taxpramount,sim_original_bill_item.amount tpv_nottaxamount,sim_original_bill_item.taxrate tpv_taxrate,sim_original_bill_item.tax tpv_common";
        DataSet imOriginalBill = ORM.create().queryDataSet(algoKey, "sim_original_bill", selelctFiles, null);

        DataSet tpvDtSalescontract = ORM.create().queryDataSet(algoKey, "tpv_dt_salescontract", "id,tpv_customer tpv_clienthb,tpv_contractnos tpv_saleconumhb,tpv_signingdate", null);
        imOriginalBill = imOriginalBill.leftJoin(tpvDtSalescontract).on("tpv_saleconum", "id").select(new String[]{"tpv_department", "tpv_billno", "tpv_billdate", "tpv_costitem", "tpv_coms", "tpv_saleconum",
                "tpv_deliver", "tpv_prodname", "tpv_origin", "tpv_shader", "tpv_specation", "tpv_num", "tpv_taxprice", "tpv_taxpramount", "tpv_nottaxamount", "tpv_taxrate", "tpv_common", "tpv_saleconumhb", "tpv_clienthb","tpv_signingdate"}).finish();
        DataSet tpvDtSalesshipping = ORM.create().queryDataSet(algoKey, "tpv_dt_salesshipping", "billno tpv_delivernumd,id", null);
        imOriginalBill = imOriginalBill.leftJoin(tpvDtSalesshipping).on("tpv_deliver", "id").select(new String[]{"tpv_department", "tpv_billno", "tpv_billdate", "tpv_costitem", "tpv_coms", "tpv_saleconum",
                "tpv_deliver", "tpv_prodname", "tpv_origin", "tpv_shader", "tpv_specation", "tpv_num", "tpv_taxprice", "tpv_taxpramount", "tpv_nottaxamount", "tpv_taxrate", "tpv_common", "tpv_saleconumhb", "tpv_clienthb",
                "tpv_delivernumd","tpv_signingdate"}).finish();
        DataSet tpvBdCombination = ORM.create().queryDataSet(algoKey, "tpv_bd_combination", "id,tpv_splitdepartment.tpv_departmententry tpv_condepartment,tpv_splitdepartment.tpv_apercentage tpv_conpro,name tpv_comstring", null);
        imOriginalBill = imOriginalBill.leftJoin(tpvBdCombination).on("tpv_coms","id").select(new String[]{"tpv_department", "tpv_billno", "tpv_billdate", "tpv_costitem", "tpv_comstring", "tpv_saleconum",
                "tpv_deliver", "tpv_prodname", "tpv_origin", "tpv_shader", "tpv_specation", "tpv_num", "tpv_taxprice", "tpv_taxpramount", "tpv_nottaxamount", "tpv_taxrate", "tpv_common", "tpv_saleconumhb", "tpv_clienthb",
                "tpv_delivernumd","tpv_signingdate","tpv_conpro","tpv_condepartment","tpv_common*tpv_conpro tpv_conamount"}).finish();
        return imOriginalBill;
    }

    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        DynamicObject tpvOrg = filterInfo.getDynamicObject("tpv_org");//业务部门
        DynamicObject tpvDep = filterInfo.getDynamicObject("tpv_dep");//分成部门
        String tpvName = filterInfo.getString("tpv_name");//计奖卡
        String tpvContractnums = filterInfo.getString("tpv_contractnums");//销售合同
        DynamicObject tpvCustomerrep = filterInfo.getDynamicObject("tpv_customerrep");//客户
        Date tpvStartsm = filterInfo.getDate("tpv_startsm");//发票日期开始
        Date tpvEndsm = filterInfo.getDate("tpv_endsm");//发票日期结束
        boolean tpvOndep = filterInfo.getBoolean("tpv_ondep");//仅看需跨部门分成
        if (tpvOrg != null) {
            whereFilter = String.format("%s and tpv_department = %s", whereFilter, tpvOrg.getPkValue());
        }
        if (tpvDep != null) {
            whereFilter = String.format("%s and tpv_condepartment = %s", whereFilter, tpvDep.getPkValue());
        }
        if (YEM.isNotEmpty(tpvName)) {
            whereFilter = String.format("%s and tpv_comstring like '%%%s%%'", whereFilter, tpvName);
        }
        if (YEM.isNotEmpty(tpvContractnums)) {
            whereFilter = String.format("%s and tpv_saleconumhb like '%%%s%%'", whereFilter, tpvContractnums);
        }
        if (tpvCustomerrep != null) {
            whereFilter = String.format("%s and tpv_clienthb = %s", whereFilter, tpvCustomerrep.getPkValue());
        }
        if (tpvStartsm != null) {
            SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String format = str.format(tpvStartsm);
            whereFilter = String.format("%s and tpv_billdate >= to_date('" + format + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        if (tpvEndsm != null) {
            SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String format = str.format(tpvEndsm);
            whereFilter = String.format("%s and tpv_billdate <= to_date('" + format + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        if (tpvOndep) {
            whereFilter = String.format("%s and tpv_department <> %s", whereFilter,
                    String.join(",", "tpv_condepartment"));
        }
        return whereFilter;
    }
}
