package tpv.tpv_wm.tpv_dt.java.report;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

/**
 * @ClassName: SummaryFunRptQuery
 * @Description: 资金总表
 * @author: zbh
 * @date: 2024-07-01
 */
public class SummaryFunRptQuery extends AbstractReportListDataPlugin {


    private static final String algoKey = SummaryFunRptQuery.class.getName();

    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        String whereFilter = createWhereStr(param);
        DataSet instockDataSet = query(whereFilter, param);
        return instockDataSet;
    }

    public static DataSet query(String whereFilter, ReportQueryParam param) {
        DataSet mainDataSet = mainDataSet().where(whereFilter);

        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
        return createDataSet;
    }

    public static DataSet mainDataSet() {

        DynamicObject transferType = BusinessDataServiceHelper.loadSingle("shkd_transfertype", new QFilter[]{new QFilter("number", "=", "1010")});
        QFilter qFilter = new QFilter("entryentity.shkd_transfertype", "=", transferType == null ? 0L : transferType.getPkValue());
        qFilter.and("billstatus", "=", "C");
        DataSet shkdCrosdeparfunproallo = ORM.create().queryDataSet(algoKey, "shkd_crosdeparfunproallo", "shkd_businessiunit,entryentity.shkd_transferamount transferamount,entryentity.shkd_outsalescontract outsalescontract,entryentity.shkd_outprocurecontract outprocurecontract", qFilter.toArray());
        DataSet tpvDtPurchase = ORM.create().queryDataSet(algoKey, "tpv_dt_purchase", "id,tpv_combinationnumber", null);
        shkdCrosdeparfunproallo = shkdCrosdeparfunproallo.leftJoin(tpvDtPurchase).on("outprocurecontract", "id").select(new String[]{"shkd_businessiunit", "transferamount tpv_conamounted", "outprocurecontract", "outsalescontract", "tpv_combinationnumber"}).finish();
        DataSet tpvBdCombination = ORM.create().queryDataSet(algoKey, "tpv_bd_combination", "id,name tpv_comstring", null);
        shkdCrosdeparfunproallo = shkdCrosdeparfunproallo.leftJoin(tpvBdCombination).on("tpv_combinationnumber", "id").select(new String[]{"shkd_businessiunit", "tpv_conamounted", "tpv_comstring"}).finish();
        shkdCrosdeparfunproallo = shkdCrosdeparfunproallo.groupBy(new String[]{"tpv_comstring", "shkd_businessiunit"}).sum("tpv_conamounted").finish();

        DataSet statementFundA = StatemenFundsRptQuery.mainDataSet().select("tpv_department", "tpv_comstring", "tpv_condepartment", "tpv_conpro", "tpv_countamount tpv_totalrep","tpv_cerep").groupBy(new String[]{"tpv_comstring", "tpv_condepartment", "tpv_department","tpv_conpro","tpv_cerep"}).sum("tpv_totalrep").finish().where("tpv_cerep='A'");
        DataSet statementFundB = StatemenFundsRptQuery.mainDataSet().select("tpv_department", "tpv_comstring", "tpv_condepartment", "tpv_conpro", "tpv_countamount tpv_totalpay","tpv_cerep").groupBy(new String[]{"tpv_comstring", "tpv_condepartment", "tpv_department","tpv_conpro","tpv_cerep"}).sum("tpv_totalpay").finish().where("tpv_cerep='B'");
        statementFundA=statementFundA.leftJoin(statementFundB).on("tpv_department","tpv_department").on("tpv_comstring","tpv_comstring").on("tpv_condepartment","tpv_condepartment").select("tpv_comstring", "tpv_condepartment", "tpv_department","tpv_conpro","tpv_cerep","tpv_totalrep","tpv_totalpay").finish();
        statementFundA=statementFundA.leftJoin(shkdCrosdeparfunproallo).on("tpv_comstring","tpv_comstring").select("tpv_comstring", "tpv_condepartment", "tpv_department","tpv_conpro","tpv_cerep","tpv_totalrep","tpv_totalpay","tpv_conamounted tpv_conamount","(case when tpv_totalpay=0 then (tpv_totalrep)*(-1) else tpv_totalpay-tpv_totalrep end) tpv_totalsub").finish();
        return statementFundA;
    }

    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        DynamicObject tpvRepdep = filterInfo.getDynamicObject("tpv_repdep");//业务部门
        if (tpvRepdep!=null){
            whereFilter = String.format("%s and tpv_department = %s", whereFilter, tpvRepdep.getPkValue());
        }
        DynamicObject tpvCondep = filterInfo.getDynamicObject("tpv_condep");//分成部门
        if (tpvCondep!=null){
            whereFilter = String.format("%s and tpv_condepartment = %s", whereFilter, tpvCondep.getPkValue());
        }
        String tpvConbation = filterInfo.getString("tpv_conbation");//计奖卡号
        if (YEM.isNotEmpty(tpvConbation)){
            whereFilter = String.format("%s and tpv_comstring like '%%%s%%'", whereFilter, tpvConbation);
        }
        boolean tpvCtmon = filterInfo.getBoolean("tpv_ctmon");//剩余金额>0
        if (tpvCtmon){
            whereFilter = String.format("%s and tpv_totalsub > %s", whereFilter, "0");
        }
        boolean tpvOndep = filterInfo.getBoolean("tpv_ondep");//仅看需跨部门分成
        if (tpvOndep){
            whereFilter = String.format("%s and tpv_department <> %s", whereFilter,
                    String.join(",", "tpv_condepartment"));
        }
        return whereFilter;
    }
}
