package tpv.tpv_wm.tpv_iscb.java.api.sealstewardapi;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.sys.utils.ApiHelper;
import com.itopview.wm.sys.utils.ApiLog;
import com.itopview.wm.utils.HttpHelper;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.MainEntityType;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.*;

import static tpv.tpv_wm.tpv_iscb.java.api.sealstewardapi.SealStewardClientHelper.*;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhck
 * @Package: com.itopview.wm.dt.task.SealStewardTask
 * @Date: 10:48 2023/10/25
 * @Description: 章管家接口
 * @Version: 1.0
 */
public final class SealStewardClient {
    private static String accessToken; // token
    private static final String ERRCODE = "errcode";
    private static final String API = "api.seal";
    private static String clientId;
    private static String clientSecret;
    private static String grantType;
    private static String responseType;

    private SealStewardClient() {
        super();
    }

    /*
      @author: Heisenberg
     * @date: 2023/10/28 15:07
     * @description: 静态代码快，初始化参数
     */
    static {
        ApiHelper apiHelper = new ApiHelper(API);
        if (apiHelper.isExist()) {
            clientId = apiHelper.getParam("client_id"); // 授权id
            clientSecret = apiHelper.getParam("client_secret"); // 授权secret
            grantType = apiHelper.getParam("grant_type"); // 授权类型
            responseType = apiHelper.getParam("response_type"); // 返回类型
        }
    }

    /**
     * @param limitStartTime: 限制开始时间，终止时间为60天后
     * @param sealIds:        印章ID
     * @param applicationId:  申请用印ID
     * @param name:           申请单的文件名称或流程名称
     * @param phoneNumber:    address 申请人手机号
     * @return String
     * @author: Heisenberg
     * @date: 2023/10/28 15:12
     * @description: 申请用印
     */
    public static List<SealStewardClientHelper.SealSteward>
    applicationForSeal(Date limitStartTime, String sealIds, String applicationId, String name, String phoneNumber, BottomTableModel bottomTableModel) {
        DynamicObject steward = BusinessDataServiceHelper.newDynamicObject("tpv_iscb_steward");
        steward.set("tpv_sourcebillscode", bottomTableModel.getTpvSourcebillscode());
        steward.set("tpv_serialnumber", applicationId);
        String tpvAppid = ((MainEntityType) bottomTableModel.getSrcEntity().getDataEntityType()).getAppId();
        steward.set("tpv_appid", tpvAppid);
        Date currentDate = new Date();
        steward.set("tpv_bizdate", currentDate);
        steward.set("tpv_attacname", bottomTableModel.getFileName());
        steward.set("tpv_sourcebills", bottomTableModel.getSrcEntity().getDataEntityType().getName());
        steward.set("tpv_billsid", bottomTableModel.getSrcEntity().getLong("id"));
        SaveServiceHelper.save(new DynamicObject[]{steward});//保存

        Map<Object, Object> seal = getApiLog("applicationForSeal");
        ApiLog log = (ApiLog) seal.get("log");
        accessToken = getAccessTokenForSeal();
        String requestEntity = getApplicationToSealEntityForSeal(limitStartTime, sealIds, applicationId, name, phoneNumber);
        Map<String, String> header = getHeaderForSeal();
        String response;
        List<SealStewardClientHelper.SealSteward> sealStewards = null;
        String url = (String) seal.get("url"); // 申请用印接口地址
        int errcode = -1;
        try {
            sealStewards = new ArrayList<>();
            response = HttpHelper.httpPost(url, header, requestEntity);
            JSONObject jsonObject = JSONObject.parseObject(response);
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            if (errcode != 0) {
                throw new SealStewardException(errcode);
            }
            JSONArray data = jsonObject.getJSONArray("data");
            // 反射生成SealSteward对象并添加到sealStewards中
            data.stream().map(u -> JSONObject.parseObject(String.valueOf(u),
                    SealStewardClientHelper.SealSteward.class)).forEach(sealStewards::add);
            log.setRequestData(requestEntity);
            log.setResponseData(response);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return sealStewards;
    }

    /**
     * @param sealapplyIds: 撤销用印ID
     * @return int
     * @author: Heisenberg
     * @date: 2023/10/28 14:33
     * @description: 删除用印（撤销用印）
     */
    public static int withdrawOfApplicationForSeal(String sealapplyIds) {
        Map<Object, Object> seal = getApiLog("withdrawOfApplicationForSeal");
        ApiLog log = (ApiLog) seal.get("log");
        accessToken = getAccessTokenForSeal();
        try {
            sealapplyIds = URLEncoder.encode(sealapplyIds, StandardCharsets.UTF_8.toString());
        } catch (UnsupportedEncodingException ignored) {
        }
        String url = seal.get("url") + sealapplyIds;
        int errcode = -1;
        try {
            Map<String, String> header = getHeaderForSeal();
            String result = HttpHelper.httpDelete(url, header);
            JSONObject jsonObject = JSONObject.parseObject(result);
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            if (errcode != 0) {
                throw new SealStewardException(errcode);
            }
            log.setRequestData(requestEntity(url));
            log.setResponseData(result);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return errcode;
    }

    /**
     * @param uids: 用户ids或编码,多个id之间用逗号隔开
     * @return int
     * @author: Heisenberg
     * @date: 2023/10/28 15:54
     * @description: 批量删除用户
     */
    public static int batchDeleteUsersForSeal(String uids) {
        Map<Object, Object> seal = getApiLog("batchDeleteUsersForSeal");
        ApiLog log = (ApiLog) seal.get("log");
        accessToken = getAccessTokenForSeal();
        try {
            uids = URLEncoder.encode(uids, StandardCharsets.UTF_8.toString());
        } catch (UnsupportedEncodingException ignored) {
        }
        String url = seal.get("url") + uids;
        int errcode = -1;
        try {
            Map<String, String> header = getHeaderForSeal();
            String result = HttpHelper.httpDelete(url, header);
            JSONObject jsonObject = JSONObject.parseObject(result);
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            if (errcode != 0) {
                throw new SealStewardException(errcode);
            }
            log.setRequestData(requestEntity(url));
            log.setResponseData(result);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return errcode;
    }

    /**
     * @param userForSeals: 封装 user 的集合
     * @return int
     * @author: Heisenberg
     * @date: 2023/10/28 18:13
     * @description: 批量添加或修改用户
     */
    public static int batchAddOrModifyUsersForSeal(List<SealStewardClientHelper.UserForSeal> userForSeals) {
        Map<Object, Object> seal = getApiLog("batchAddOrModifyUsersForSeal");
        ApiLog log = (ApiLog) seal.get("log");
        accessToken = getAccessTokenForSeal();
        String url = (String) seal.get("url");
        int errcode = -1;
        try {
            Map<String, String> header = getHeaderForSeal();
            String requestEntity = getBatchAddOrModifyToUserEntityForSeal(userForSeals);
            String result = HttpHelper.httpPost(url, header, requestEntity);
            JSONObject jsonObject = JSONObject.parseObject(result);
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            if (errcode != 0) {
                throw new SealStewardException(errcode);
            }
            log.setRequestData(requestEntity);
            log.setResponseData(result);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return errcode;
    }

    /**
     * @param uids: 用户 id
     * @return int
     * @author: Heisenberg
     * @date: 2023/11/3 14:08
     * @description: 批量启用用户
     */
    public static int batchEnableUsersForSeal(String uids) {
        Map<Object, Object> seal = getApiLog("batchEnableUsersForSeal");
        ApiLog log = (ApiLog) seal.get("log");
        accessToken = getAccessTokenForSeal();
        try {
            uids = URLEncoder.encode(uids, StandardCharsets.UTF_8.toString());
        } catch (UnsupportedEncodingException ignored) {
        }
        String url = seal.get("url") + uids;
        int errcode = -1;
        try {
            String result = HttpHelper.httpPut(url, getHeaderForSeal());
            JSONObject jsonObject = JSONObject.parseObject(result);
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            if (errcode != 0) {
                throw new SealStewardException(errcode);
            }
            log.setRequestData(requestEntity(url));
            log.setResponseData(result);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return errcode;
    }

    /**
     * @param uids: 用户 id
     * @return int
     * @author: Heisenberg
     * @date: 2023/11/3 14:09
     * @description: 批量停用用户
     */
    public static int batchDisableUsersForSeal(String uids) {
        Map<Object, Object> seal = getApiLog("batchDisableUsersForSeal");
        ApiLog log = (ApiLog) seal.get("log");
        accessToken = getAccessTokenForSeal();
        try {
            uids = URLEncoder.encode(uids, StandardCharsets.UTF_8.toString());
        } catch (UnsupportedEncodingException ignored) {
        }
        String url = seal.get("url") + uids;
        int errcode = -1;
        try {
            String result = HttpHelper.httpPut(url, getHeaderForSeal());
            JSONObject jsonObject = JSONObject.parseObject(result);
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            if (errcode != 0) {
                throw new SealStewardException(errcode);
            }
            log.setRequestData(url);
            log.setResponseData(result);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return errcode;
    }

    /**
     * @param departmentForSeal: 封装 department 的集合
     * @return int
     * @author: Heisenberg
     * @date: 2023/10/31 14:05
     * @description: 批量添加或修改部门
     */
    public static int batchAddOrModifyDepartmentsForSeal(List<SealStewardClientHelper.DepartmentForSeal> departmentForSeal) {
        Map<Object, Object> seal = getApiLog("batchAddOrModifyDepartments");
        ApiLog log = (ApiLog) seal.get("log");
        accessToken = getAccessTokenForSeal();
        String url = (String) seal.get("url");
        int errcode = -1;
        try {
            Map<String, String> header = getHeaderForSeal();
            String requestEntity = getBatchAddOrModifyToDepartmentEntityForSeal(departmentForSeal);
            String result = HttpHelper.httpPost(url, header, requestEntity);
            JSONObject jsonObject = JSONObject.parseObject(result);
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            if (errcode != 0) {
                throw new SealStewardException(errcode);
            }
            log.setRequestData(requestEntity);
            log.setResponseData(result);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return errcode;
    }

    /**
     * @param departmentIds: 部门 ID
     * @return int
     * @author: Heisenberg
     * @date: 2023/10/31 14:26
     * @description: 如果部门下存在用户，则不能删除
     */
    public static int batchDeleteDepartmentsForSeal(String departmentIds) {
        Map<Object, Object> seal = getApiLog("batchDeleteDepartmentsForSeal");
        ApiLog log = (ApiLog) seal.get("log");
        accessToken = getAccessTokenForSeal();
        try {
            departmentIds = URLEncoder.encode(departmentIds, StandardCharsets.UTF_8.toString());
        } catch (UnsupportedEncodingException ignored) {
        }
        String url = seal.get("url") + departmentIds;
        int errcode = -1;
        try {
            String result = HttpHelper.httpDelete(url, getHeaderForSeal());
            JSONObject jsonObject = JSONObject.parseObject(result);
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            if (errcode != 0) {
                throw new SealStewardException(errcode);
            }
            log.setRequestData(requestEntity(url));
            log.setResponseData(result);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return errcode;
    }

    /**
     * @param departmentIds: 部门 id 或 编码（多个 id 之间用逗号隔开）
     * @return int
     * @author: Heisenberg
     * @date: 2023/11/3 9:34
     * @description: 批量启用部门
     */

    public static int batchEnableDepartmentsForSeal(String departmentIds) {
        Map<Object, Object> seal = getApiLog("batchEnableDepartmentsForSeal");
        ApiLog log = (ApiLog) seal.get("log");
        accessToken = getAccessTokenForSeal();
        try {
            departmentIds = URLEncoder.encode(departmentIds, StandardCharsets.UTF_8.toString());
        } catch (UnsupportedEncodingException ignored) {
        }
        String url = seal.get("url") + departmentIds;
        int errcode = -1;
        try {
            String result = HttpHelper.httpPut(url, getHeaderForSeal());
            JSONObject jsonObject = JSONObject.parseObject(result);
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            if (errcode != 0) {
                throw new SealStewardException(errcode);
            }
            log.setRequestData(requestEntity(url));
            log.setResponseData(result);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return errcode;
    }

    /**
     * @param departmentIds: 部门 id 或 编码
     * @return int
     * @author: Heisenberg
     * @date: 2023/11/3 9:56
     * @description: 批量停用部门
     */
    public static int batchDisableDepartmentsForSeal(String departmentIds) {
        accessToken = getAccessTokenForSeal();
        Map<Object, Object> seal = getApiLog("batchDisableDepartmentsForSeal");
        ApiLog log = (ApiLog) seal.get("log");
        try {
            departmentIds = URLEncoder.encode(departmentIds, StandardCharsets.UTF_8.toString());
        } catch (UnsupportedEncodingException ignored) {
        }
        String url = seal.get("url") + departmentIds;
        int errcode = -1;
        try {
            String result = HttpHelper.httpPut(url, getHeaderForSeal());
            JSONObject jsonObject = JSONObject.parseObject(result);
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            if (errcode != 0) {
                throw new SealStewardException(errcode);
            }
            log.setRequestData(requestEntity(url));
            log.setResponseData(result);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return errcode;
    }


    /**
     * @param limitStartTime: 限制开始时间，终止时间为60天后
     * @param sealIds:        印章ID
     * @param applicationId:  申请用印ID
     * @param name:           申请单的文件名称或流程名称
     * @param phoneNumber:    address 申请人手机号
     * @return String
     * @author: Heisenberg
     * @date: 2023/10/28 15:12
     * @description: 申请用印请求体
     */
    private static String getApplicationToSealEntityForSeal(
            Date limitStartTime, String sealIds, String applicationId, String name, String phoneNumber) {
        Date date = new Date();
        String startTime = dateFormat(date); // 开始取当前时间，limitStartTime作废，可优化  -longh
        String endTime = getLimitEndTime(date);
//        String startTime = dateFormat(limitStartTime);
//        String endTime = getLimitEndTime(limitStartTime);
        JSONArray jsonArray = new JSONArray();
        JSONObject params = new JSONObject();
        params.put("sealapply_id", applicationId);
        params.put("name", name);
        params.put("file_count", 1);
        params.put("type_id", "APITEST");
        params.put("apply_status", 6);
//        params.put("uid", "15664888536"); // 申请用印人的手机号
        params.put("uid", phoneNumber); // 申请用印人的手机号
        params.put("seal_ids1", sealIds); // 印章id
        params.put("use_count1", 20); // 盖章次数
        params.put("is_sealed_bid1", "false"); // 骑缝盖章
        params.put("is_auto_batchs1", 1); // 是否批量
        params.put("code", applicationId); // 选填（不填由章管家默认生成），code是指第三方系统的单据编码
        params.put("operateTime", startTime); // 创建时间
        params.put("is_out", startTime); // 预计开始时间
        params.put("outTime", endTime); // 预计结束时间
        params.put("returnTime", "false"); // 是否外带
        params.put("limit_time", "1"); // 是否限时盖章
        params.put("limitStartTime", startTime); // 限时开始时间
        params.put("limitEndTime", endTime); // 限时结束时间 = 限时开始时间 + 60
        jsonArray.add(params);
        return jsonArray.toJSONString();
    }

    /**
     * @param userForSeals: 添加或修改的用户（包含用户信息的）集合
     * @return String
     * @author: Heisenberg
     * @date: 2023/10/30 9:15
     * @description: 批量添加或修改用户请求头
     */
    private static String getBatchAddOrModifyToUserEntityForSeal(List<SealStewardClientHelper.UserForSeal> userForSeals) {
        JSONArray users = new JSONArray();
        Optional.ofNullable(userForSeals)
                .map(u -> {
                    u.forEach(userForSeal -> {
                        Map<String, Object> params = new LinkedHashMap<>();

                        // required = true
                        params.put("mobile", userForSeal.getMobile());
                        params.put("name", userForSeal.getName());
                        params.put("uid", userForSeal.getUid());
                        params.put("department_id", userForSeal.getDepartment_id());

                        // required = false
                        Optional.ofNullable(userForSeal.getCode())
                                .filter(Objects::nonNull)
                                .map(code -> params.put("code", code));
                        Optional.ofNullable(userForSeal.getMail_address())
                                .map(mail_address -> params.put("mail_address", mail_address))
                                .filter(Objects::nonNull);
                        Optional.ofNullable(userForSeal.getPassword())
                                .filter(Objects::nonNull)
                                .map(password -> params.put("password", password));
                        Optional.ofNullable(userForSeal.getPosition())
                                .filter(Objects::nonNull)
                                .map(position -> params.put("position", position));
                        Optional.ofNullable(userForSeal.getIs_enable())
                                .filter(Objects::nonNull)
                                .map(is_enable -> params.put("is_enable", is_enable));
                        Optional.ofNullable(userForSeal.getCellphone())
                                .filter(Objects::nonNull)
                                .map(cellphone -> params.put("cellphone", cellphone));
                        Optional.ofNullable(userForSeal.getQyWechat_id())
                                .filter(Objects::nonNull)
                                .map(qyWechat_id -> params.put("qyWechat_id", qyWechat_id));
                        Optional.ofNullable(userForSeal.getDingtalk_id())
                                .filter(Objects::nonNull)
                                .map(dingtalk_id -> params.put("dingtalk_id", dingtalk_id));

                        users.add(new JSONObject(params));
                    });
                    return u;
                });
        return users.toString();
    }

    /**
     * @param departmentForSeal: 添加或修改的部门（包含部门信息的）集合
     * @return String
     * @author: Heisenberg
     * @date: 2023/10/31 14:03
     * @description: 批量添加或修改部门请求头
     */
    private static String getBatchAddOrModifyToDepartmentEntityForSeal(List<SealStewardClientHelper.DepartmentForSeal> departmentForSeal) {
        JSONArray department = new JSONArray();
        Optional.ofNullable(departmentForSeal)
                .map(u -> {
                    u.forEach(userForSeal -> {
                        Map<String, Object> params = new LinkedHashMap<>();

                        // required = true
                        params.put("name", userForSeal.getName());
                        params.put("department_id", userForSeal.getDepartment_id());

                        // required = false
                        Optional.ofNullable(userForSeal.getCode())
                                .filter(Objects::nonNull)
                                .map(code -> params.put("code", code));
                        Optional.ofNullable(userForSeal.getIs_enable())
                                .filter(Objects::nonNull)
                                .map(is_enable -> params.put("is_enable", is_enable));
                        Optional.ofNullable(userForSeal.getParent_id())
                                .filter(Objects::nonNull)
                                .map(parent_id -> params.put("parent_id", parent_id));
                        Optional.ofNullable(userForSeal.getRemark())
                                .filter(Objects::nonNull)
                                .map(remark -> params.put("remark", remark));

                        department.add(new JSONObject(params));
                    });
                    return u;
                });
        return department.toString();
    }

    /**
     * @return String
     * @author: Heisenberg
     * @date: 2023/10/28 14:33
     * @description: REST API 需要权限才能访问，权限通过发送 HTTP 请求时携带 token 来体现，获取 token
     */
    public static String getAccessTokenForSeal() {
        Map<Object, Object> seal = getApiLog("getAccessTokenForSeal");
        ApiLog log = (ApiLog) seal.get("log");
        JSONObject jsonObject = null;
        int errcode = -1;
        try {
            String url = String.format((String) seal.get("url"), clientId, clientSecret, grantType, responseType);
            String response = HttpHelper.httpGet(url);
            // 解析json数据
            jsonObject = JSONObject.parseObject(response);
            // 获取状态码
            errcode = Integer.parseInt(jsonObject.getString(ERRCODE));
            // 检查状态码是否为成功状态
            if (0 != errcode) {
                throw new SealStewardException(errcode);
            }
            log.setRequestData(requestEntity(url));
            log.setResponseData(response);
        } catch (Exception e) {
            log.setErrorMsg(SealStewardException.getErrMessage(errcode));
        }
        log.setSuccess(errcode == 0);
        log.write();
        return jsonObject.getString("accessToken");
    }

    /**
     * @return Map<String, String>
     * @author: Heisenberg
     * @date: 2023/10/28 14:36
     * @description: 默认请求头 加上
     */
    private static Map<String, String> getHeaderForSeal() {
        Map<String, String> header = new HashMap<>();
        header.put("Authorization", "Bearer " + accessToken);
        header.put("Content-Type", "application/json");
        header.put("Accept-Encoding", "gzip, deflate, br");
        header.put("Connection", "keep-alive");
        header.put("User-Agent",
                "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML,"
                        + " like Gecko) Chrome/91.0.4472.124 Safari/537.36"); // chrome浏览器
        return header;
    }

}
