package tpv.tpv_wm.tpv_iscb.java.api.sealstewardapi;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhck
 * @Package: com.itopview.wm.dt.api.sealstewardapi.SealStewardException
 * @Date: 10:13 2023/10/18
 * @Description: 章管家http请求中状态码和异常信息的转换
 * @Version: 1.0
 */
public class SealStewardException extends Exception {
    private int errCode; // 状态码
    private String errMsg; // 自定义异常类，当状态码不为0时，抛出异常 异常信息
    private static final Map<Integer, String> errorMessages;

    static {
        errorMessages = new HashMap<>();
        errorMessages.put(-1, "请求失败");
        errorMessages.put(0, "请求成功");
        errorMessages.put(201, "accesstoken无效或超时");
        errorMessages.put(404, "请求的URI地址不存在");
        errorMessages.put(11000, "批量操作最大数量不能超过50");
        errorMessages.put(11001, "client_id和client_secret无效");
        errorMessages.put(11002, "企业服务时间已到期");
        errorMessages.put(11003, "用户id未注册");
        errorMessages.put(11004, "企业不存在");
        errorMessages.put(11005, "必填参数为空");
        errorMessages.put(11006, "手机号未注册");
        errorMessages.put(11007, "parent_id不存在");
        errorMessages.put(11008, "编码已存在");
        errorMessages.put(11009, "id已存在");
        errorMessages.put(11010, "部门下存在用户，不允许删除");
        errorMessages.put(11011, "部门id不存在");
        errorMessages.put(11012, "用户手机号已存在");
        errorMessages.put(11013, "status参数错误[0:启用；1：停用]");
        errorMessages.put(11014, "印章类别下存在印章，不允许删除");
        errorMessages.put(11015, "保管人id不存在");
        errorMessages.put(11016, "印章类型id不存在");
        errorMessages.put(11017, "apply_status参数错误，只允许为0、2、3或10");
        errorMessages.put(11018, "Id不存在");
        errorMessages.put(11019, "印章未入库");
        errorMessages.put(1101901, "印章简称重复");
        errorMessages.put(1101902, "印章全称重复");
        errorMessages.put(11020, "印章未创建、未通过审批或已销毁停用");
        errorMessages.put(110201, "传递的印章为空");
        errorMessages.put(11021, "文件类型id不存在");
        errorMessages.put(1102101, "文件不存在");
        errorMessages.put(1102102, "水印设置没有配置");
        errorMessages.put(11022, "apply_status参数错误，只允许为0（未送审）、1（自动送审）或6（审批完成）");
        errorMessages.put(11023, "往来单位id不存在");
        errorMessages.put(11024, "用印申请不存在");
        errorMessages.put(110241, "用印申请单在智能用印的进行中或者已完成用印的不能删除");
        errorMessages.put(110242, "用印申请单在归档进行中或已归档的不能删除");
        errorMessages.put(11025, "重复的id");
        errorMessages.put(11026, "重复的编码");
        errorMessages.put(11027, "在审批中或已绑定智能硬件的印章不允许删除");
        errorMessages.put(11028, "parent_id有误");
        errorMessages.put(11029, "uid不存在");
        errorMessages.put(11030, "超出最大用户数");
        errorMessages.put(11031, "当前盖章申请已被确认");
        errorMessages.put(11032, "当前用印申请单已被重置");
        errorMessages.put(11033, "本次确认侦测到印章移动，已被自动拒绝");
        errorMessages.put(11034, "远程确认单据不存在");
        errorMessages.put(11035, "普通印章和一代印章无法使用封标盖章");
        errorMessages.put(11036, "二代印章无法选择长盖章的文件类型");
        errorMessages.put(11040, "部门不存在");
        errorMessages.put(25001, "该级下存在项目名称相同的部门");
        errorMessages.put(25002, "确认人ids不存在");
        errorMessages.put(25003, "长盖章不允许限制盖章次数");
        errorMessages.put(25004, "管理员账号不能删除");
        errorMessages.put(25005, "上级部门不存在");
        errorMessages.put(25006, "您的IP地址未授权访问接口，请联系管理员进行授权");
        errorMessages.put(25007, "转办人ids不存在");
    }

    public void setErrCode(int errCode) {
        this.errCode = errCode;
    }

    public void setErrMsg(String errMsg) {
        this.errMsg = errMsg;
    }

    public int getErrCode() {
        return errCode;
    }

    public String getErrMsg() {
        return errMsg;
    }

    public SealStewardException(int errCode) {
        this.errCode = errCode;
        this.errMsg = errorMessages.getOrDefault(errCode, "未知错误");
    }


    public static String getErrMessage(int errcode) {
        return errorMessages.getOrDefault(errcode, "未知错误");
    }

    @Override
    public String toString() {
        return "SealStewardException{" +
                "errCode=" + errCode +
                ", errMsg='" + errMsg + '\'' +
                '}';
    }
}
