package tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhkc
 * @Package: com.itopview.wm.dt.task.StatusCodeConverter
 * @Date: 15:52 2023/09/14
 * @Description: 船讯网http请求中状态码和异常信息的转换
 * @Version: 7.0
 */
public class ShipNewsApiException extends Exception {
    private final int statusCode;     // 状态码
    private final String errorMessage;    // 自定义异常类，当状态码不为0时，抛出异常 异常信息
    private static final Map<Integer, String> ERROR_MESSAGES;

    static {
        ERROR_MESSAGES = new HashMap<>();
        ERROR_MESSAGES.put(0, "成功");
        ERROR_MESSAGES.put(1, "失败(具体原因不明)");
        ERROR_MESSAGES.put(3, "操作对象不存在(船舶、船队等)");
        ERROR_MESSAGES.put(6, "Key 过期");
        ERROR_MESSAGES.put(7, "key 被锁定");
        ERROR_MESSAGES.put(9, "key 不存在");
        ERROR_MESSAGES.put(12, "请求数据量过大，拒绝执行");
        ERROR_MESSAGES.put(13, "服务器繁忙");
        ERROR_MESSAGES.put(14, "请求来自非绑定域名");
        ERROR_MESSAGES.put(15, "多船请求数量超过限制");
        ERROR_MESSAGES.put(16, "区域船接口超出区域外");
        ERROR_MESSAGES.put(17, "写授权码错误");
        ERROR_MESSAGES.put(19, "程序内部错误");
        ERROR_MESSAGES.put(21, "不具备服务使用权限");
        ERROR_MESSAGES.put(22, "查询船舶数量已超限");
        ERROR_MESSAGES.put(23, "查询船舶次数已超限");
        ERROR_MESSAGES.put(24, "查询船舶超出限定船舶集合");
        ERROR_MESSAGES.put(25, "查询船舶档案数量已超限");
        ERROR_MESSAGES.put(26, "查询船舶当前状态数量已超限");
        ERROR_MESSAGES.put(27, "查询港口超出限定港口集合");
        ERROR_MESSAGES.put(28, "轨迹查询时间超限");
        ERROR_MESSAGES.put(29, "接口单位时间内请求频率太高，请稍后再尝试");
        ERROR_MESSAGES.put(100, "参数错误(缺少参数，或参数不合法)");
        ERROR_MESSAGES.put(10000, "网络连接失败");
        ERROR_MESSAGES.put(10001, "不支持的操作");
    }

    @Override
    public String getMessage() {
        return errorMessage;
    }

    @Override
    public String toString() {
        return "API Exception: " + errorMessage + " (Status Code: " + statusCode + ")";
    }

    public ShipNewsApiException(int statusCode) {
        this(statusCode, ERROR_MESSAGES.getOrDefault(statusCode, "未知错误"));
    }

    public static String getErrMessage(int errcode) {
        return ERROR_MESSAGES.getOrDefault(errcode, "未知错误");
    }

    public ShipNewsApiException(int statusCode, String message) {
        super(message);
        this.statusCode = statusCode;
        this.errorMessage = message;
    }
}
