package tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi;

import com.itopview.wm.utils.YEM;
import kd.bos.bill.BillShowParameter;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.form.ShowType;
import kd.bos.form.control.events.SearchEnterEvent;
import kd.bos.list.IListView;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.DeleteServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhkc
 * @Package: com.itopview.wm.dt.utils.ShipListHelper
 * @Date: 11:05 2023/09/25
 * @Description:
 * @Version: 1.0
 */
public final class ShipNewsClientHelper {
    public static final String TPV_WMS_GZCQD = "tpv_wms_gzcqd"; // 关注船清单 单据标识

    /**
     * @author: Heisenberg
     * @date: 2023/10/11 11:00
     * @description: 工具类私有构造器
     */
    private ShipNewsClientHelper() {
        super();
    }

    /**
     * @param shipDetailList: 传入船舶集合
     * @author: Heisenberg
     * @date: 2023/9/25 9:39
     * @description: 将船舶信息添加到单据列表中
     */
    public static DynamicObject[] processShipFormList(List<ShipDetail> shipDetailList) {
        if (shipDetailList == null) {
            return new DynamicObject[0];
        }
        DynamicObject loaded = BusinessDataServiceHelper.loadSingle(YEM.getCurrentOrgId(),
                "bos_adminorg", "id,name"); // 获取当前登录人的所属部门

        // 创建一个只读(不可修改)的列表视图
        List<DynamicObject> shipFormList = Collections.unmodifiableList(
                shipDetailList.stream()
                        .filter(shipDetail -> !isExistShipId(shipDetail.getShipID())) // 过滤列表已存在的关注船清单单据
                        .map(shipDetail -> {

                            // 新建关注船清单单据
                            DynamicObject shipBill = BusinessDataServiceHelper.newDynamicObject(TPV_WMS_GZCQD);

                            shipBill.set("creator", YEM.getCurrentUserId()); // 赋值业务员为当前登陆人，用于带出当前部门
                            shipBill.set("tpv_operator", YEM.getCurrentUserId()); // 赋值创建人为当前登陆人
                            shipBill.set("org", loaded); // 赋值组织为当前部门
                            shipBill.set("billstatus", "C"); // 赋值单据状态为已审核

                            // 赋值单据编号为船舶名称+船舶id
                            shipBill.set("billno", shipDetail.getName() + "(" + shipDetail.getShipID() + ")");

                            shipBill.set("tpv_shipname", shipDetail.getName()); // 赋值船名
                            shipBill.set("tpv_shipmmssi", shipDetail.getMmsi()); // 赋值MMSI
                            shipBill.set("tpv_shipimo", shipDetail.getImo()); // 赋值船 IMO
                            shipBill.set("tpv_shipid", shipDetail.getShipID()); // 赋值船舶id 唯一标识 隐藏字段

                            // 将获取到的船舶类型编号转为对应船舶类型
                            shipBill.set("tpv_destination",
                                    ShipNewsTypeCodeConverter.getShipType(Integer.parseInt(shipDetail.getShiptype())));
                            return shipBill;

                        })
                        .collect(Collectors.toList())
        );
        SaveServiceHelper.save(shipFormList.toArray(new DynamicObject[0])); // 保存到数据库
        return shipFormList.toArray(new DynamicObject[0]);
    }

    /**
     * @param searchEnterEvent: 回车事件
     * @return String
     * @author: Heisenberg
     * @date: 2023/9/25 18:03
     * @description: 获取搜索框内的id
     */
    public static String getSearchId(SearchEnterEvent searchEnterEvent) {
        return Optional.ofNullable(searchEnterEvent.getText())
                .filter(u -> !u.isEmpty())
                .map(u -> {
                    int start = u.indexOf("(") + 1; // 使用indexOf方法找到文本中第一个出现的"("字符，并将其索引加1赋给start变量
                    int end = u.indexOf(")"); // 使用indexOf方法找到文本中第一个出现的")"字符，并将其索引赋给end变量。
                    return u.substring(start, end); // 使用substring方法从文本中截取出start和end之间的子字符串
                })
                .orElse(null);
    }

    /**
     * @param id: 船舶id
     * @return boolean
     * @author: Heisenberg
     * @date: 2023/9/25 11:20
     * @description: 判断当前列表是否存在当前船舶ID，存在返回true,反之返回false
     */
    public static boolean isExistShipId(String id) {

        // 加载关注船清单
        DynamicObject[] ships = BusinessDataServiceHelper
                .load(TPV_WMS_GZCQD, "tpv_shipid", null);

        List<String> shipIds = Optional.ofNullable(ships) // 判断列表是否有数据
                .map(Arrays::stream)
                .orElseGet(Stream::empty) // 初始列表无数据返回一个空对像
                .map(ship -> ship.get("tpv_shipid").toString()) // 获取关注船清单列表中所有船舶 id 集合
                .collect(Collectors.toList());
        return shipIds.contains(id); // 集合包含该 id 返回true,反之返回false
    }

    /**
     * @param qFilters: 过滤器
     * @return DynamicObject
     * @author: Heisenberg
     * @date: 2023/9/25 11:27
     * @description: 过滤指定指定船舶ID，返回对应单据
     */
    public static DynamicObject getCurrentShipBill(QFilter[] qFilters) {
        return BusinessDataServiceHelper
                .loadSingle(TPV_WMS_GZCQD, qFilters);
    }

    /**
     * @param pkid: 单据id = getLong("id")
     * @return BillShowParameter
     * @author: Heisenberg
     * @date: 2023/9/28 11:47
     * @description: 打开已知id的船舶单据页面视图
     */
    public static BillShowParameter showBillView(Long pkid) {
        BillShowParameter billShowParameter = new BillShowParameter();
        billShowParameter.getOpenStyle().setShowType(ShowType.Modal);
        billShowParameter.setFormId(TPV_WMS_GZCQD);
        billShowParameter.setPkId(pkid);
        return billShowParameter;
    }

    /**
     * @param qFilters: 过滤器
     * @return int
     * @author: Heisenberg
     * @date: 2023/9/25 15:13
     * @description: 删除指定关注船清单单据
     */
    public static int deleteShipForm(QFilter[] qFilters) {
        return DeleteServiceHelper.delete(TPV_WMS_GZCQD, qFilters);
    }

    /**
     * @param view: 列表视图
     * @return List<Long>
     * @author: Heisenberg
     * @date: 2023/9/25 11:28
     * @description: 获取列表选中的单据
     */
    public static List<Long> getListBillId(IListView view) {
        return view.getSelectedRows().stream()
                .map(selectedRow -> (Long) selectedRow.getPrimaryKeyValue())
                .distinct()
                .collect(Collectors.toList());
    }

    /**
     * @author: Heisenberg
     * @date: 2023/9/20 16:29
     * @description: ShipDeTail 封装了返回的所有数据，
     */
    public static class ShipDetail {
        private String ShipID; // 船舶ID
        private String From; // 数据来源 0代表AIS 1代表卫星
        private String utc;
        private String datatype;
        private String MatchType; // 匹配类型 1：船名； 2：呼号  3：mmsi 5:imo
        private String name; // 船舶名称
        private String mmsi; // 船舶MMSI 9位数字
        private String shiptype; //船舶类型
        private String imo; // 0 = Not available（default） 2147483647 = Null
        private String callsign; // 船舶呼号
        private String length; // 船长 分米 （0-10220）
        private String width; // 船宽 分米 （0-1260]
        private String left; // 左舷距 分米 （0-630]
        private String trail; //尾距 分米 （0-5110）
        private String draught; // 吃水 毫米 （0-25500]
        private String dest; // 目的地
        private String dest_std; // 标准化的目的地（只有返回 json 格式时包含此信息） 根据dest内容匹配到船讯网港口库
        private String destcode; // 标准化后的目的地港口编码 (只有返回 json 格式时包含此信息)
        private String distance; // 已航行的距离，单位海里。需要开通相应权限才能获取该值
        private String eta; // 预到时间 [MM][DD][HH][MM]
        private String eta_std; // 标准化后的预到时间 (只有返回 json 格式时包含此信息) ：对 eta 标准化处理， 格式为[YYYY] [MM][DD][HH][MM]
        private String navistat; // 船舶航行状态
        private String lat;  // 维度 1/1000000度 [90000000,90000000]
        private String lon; //经度 ，1/1000000 度,[-180000000,180000000]
        private String hdg; // 船首向 ，1/100 度, [0,35900]
        private String cog; // 航迹向 ，1/100 度, [0,35990]
        private String sog; // 速度 ，毫米/秒, [0,52576]
        private String rot; // 转向率 ，1/100   度/秒, [-1200,1200]
        private String lasttime; // 更新时间， unix 时间戳

        public String getDatatype() {
            return datatype;
        }

        public void setDatatype(String datatype) {
            this.datatype = datatype;
        }

        public String getDistance() {
            return distance;
        }

        public void setDistance(String distance) {
            this.distance = distance;
        }

        public String getShipID() {
            return ShipID;
        }

        public void setShipID(String shipID) {
            ShipID = shipID;
        }

        public String getFrom() {
            return From;
        }

        public void setFrom(String from) {
            From = from;
        }

        public String getMatchType() {
            return MatchType;
        }

        public void setMatchType(String matchType) {
            MatchType = matchType;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getMmsi() {
            return mmsi;
        }

        public void setMmsi(String mmsi) {
            this.mmsi = mmsi;
        }

        public String getShiptype() {
            return shiptype;
        }

        public void setShiptype(String shiptype) {
            this.shiptype = shiptype;
        }

        public String getImo() {
            return imo;
        }

        public void setImo(String imo) {
            this.imo = imo;
        }

        public String getCallsign() {
            return callsign;
        }

        public void setCallsign(String callsign) {
            this.callsign = callsign;
        }

        public String getLength() {
            return length;
        }

        public void setLength(String length) {
            this.length = length;
        }

        public String getWidth() {
            return width;
        }

        public void setWidth(String width) {
            this.width = width;
        }

        public String getLeft() {
            return left;
        }

        public void setLeft(String left) {
            this.left = left;
        }

        public String getTrail() {
            return trail;
        }

        public void setTrail(String trail) {
            this.trail = trail;
        }

        public String getDraught() {
            return draught;
        }

        public void setDraught(String draught) {
            this.draught = draught;
        }

        public String getDest() {
            return dest;
        }

        public void setDest(String dest) {
            this.dest = dest;
        }

        public String getDest_std() {
            return dest_std;
        }

        public void setDest_std(String dest_std) {
            this.dest_std = dest_std;
        }

        public String getDestcode() {
            return destcode;
        }

        public void setDestcode(String destcode) {
            this.destcode = destcode;
        }

        public String getEta() {
            return eta;
        }

        public void setEta(String eta) {
            this.eta = eta;
        }

        public String getEta_std() {
            return eta_std;
        }

        public void setEta_std(String eta_std) {
            this.eta_std = eta_std;
        }

        public String getNavistat() {
            return navistat;
        }

        public void setNavistat(String navistat) {
            this.navistat = navistat;
        }

        public String getLat() {
            return lat;
        }

        public void setLat(String lat) {
            this.lat = lat;
        }

        public String getLon() {
            return lon;
        }

        public void setLon(String lon) {
            this.lon = lon;
        }

        public String getHdg() {
            return hdg;
        }

        public void setHdg(String hdg) {
            this.hdg = hdg;
        }

        public String getCog() {
            return cog;
        }

        public void setCog(String cog) {
            this.cog = cog;
        }

        public String getSog() {
            return sog;
        }

        public void setSog(String sog) {
            this.sog = sog;
        }

        public String getRot() {
            return rot;
        }

        public void setRot(String rot) {
            this.rot = rot;
        }

        public String getLasttime() {
            return lasttime;
        }

        public void setLasttime(String lasttime) {
            this.lasttime = lasttime;
        }

        public String getUtc() {
            return utc;
        }

        public void setUtc(String utc) {
            this.utc = utc;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            ShipDetail detail = (ShipDetail) o;
            return Objects.equals(ShipID, detail.ShipID) && Objects.equals(From, detail.From) && Objects.equals(utc, detail.utc) && Objects.equals(datatype, detail.datatype) && Objects.equals(MatchType, detail.MatchType) && Objects.equals(name, detail.name) && Objects.equals(mmsi, detail.mmsi) && Objects.equals(shiptype, detail.shiptype) && Objects.equals(imo, detail.imo) && Objects.equals(callsign, detail.callsign) && Objects.equals(length, detail.length) && Objects.equals(width, detail.width) && Objects.equals(left, detail.left) && Objects.equals(trail, detail.trail) && Objects.equals(draught, detail.draught) && Objects.equals(dest, detail.dest) && Objects.equals(dest_std, detail.dest_std) && Objects.equals(destcode, detail.destcode) && Objects.equals(distance, detail.distance) && Objects.equals(eta, detail.eta) && Objects.equals(eta_std, detail.eta_std) && Objects.equals(navistat, detail.navistat) && Objects.equals(lat, detail.lat) && Objects.equals(lon, detail.lon) && Objects.equals(hdg, detail.hdg) && Objects.equals(cog, detail.cog) && Objects.equals(sog, detail.sog) && Objects.equals(rot, detail.rot) && Objects.equals(lasttime, detail.lasttime);
        }

        @Override
        public int hashCode() {
            return Objects.hash(ShipID, From, utc, datatype, MatchType, name, mmsi, shiptype, imo, callsign, length, width, left, trail, draught, dest, dest_std, destcode, distance, eta, eta_std, navistat, lat, lon, hdg, cog, sog, rot, lasttime);
        }

        @Override
        public String toString() {
            return "ShipDetail{" + "ShipID='" + ShipID + '\'' + ", From='" + From + '\'' + ", utc='" + utc + '\'' + ", datatype='" + datatype + '\'' + ", MatchType='" + MatchType + '\'' + ", name='" + name + '\'' + ", mmsi='" + mmsi + '\'' + ", shiptype='" + shiptype + '\'' + ", imo='" + imo + '\'' + ", callsign='" + callsign + '\'' + ", length='" + length + '\'' + ", width='" + width + '\'' + ", left='" + left + '\'' + ", trail='" + trail + '\'' + ", draught='" + draught + '\'' + ", dest='" + dest + '\'' + ", dest_std='" + dest_std + '\'' + ", destcode='" + destcode + '\'' + ", distance='" + distance + '\'' + ", eta='" + eta + '\'' + ", eta_std='" + eta_std + '\'' + ", navistat='" + navistat + '\'' + ", lat='" + lat + '\'' + ", lon='" + lon + '\'' + ", hdg='" + hdg + '\'' + ", cog='" + cog + '\'' + ", sog='" + sog + '\'' + ", rot='" + rot + '\'' + ", lasttime='" + lasttime + '\'' + '}';
        }
    }
}
