package tpv.tpv_wm.tpv_iscb.java.formplugin.form;

import com.itopview.wm.sys.formplugin.form.AbstractVersionBillPlugIn;
import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.IFormView;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListShowParameter;
import kd.bos.mvc.bill.BillView;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import tpv.tpv_wm.tpv_iscb.java.utils.PrintHelper;

import java.io.IOException;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

/**
 * 外贸通用模板1基类
 * 
 * @description:1、继承了版本变更功能 2、业务员部门携带、过滤
 * @author longh
 * @date 2022年6月16日 下午2:30:41
 */
public class Tpl1BillPlugIn extends AbstractVersionBillPlugIn implements BeforeF7SelectListener {

	@Override
	public void afterBindData(EventObject e) {
		super.afterBindData(e);
		OperationStatus status = ((BillView) e.getSource()).getFormShowParameter().getStatus();
		boolean addnew = OperationStatus.ADDNEW.equals(status);
		initDept(addnew);
		this.getModel().setDataChanged(false);
	}

	@Override
	public void registerListener(EventObject e) {
		super.registerListener(e);
		((BasedataEdit) this.getControl("tpv_department")).addBeforeF7SelectListener(this);
	}
	
	@Override
	public void itemClick(ItemClickEvent evt) {
		super.itemClick(evt);
		String itemKey = evt.getItemKey();
		switch (itemKey) {
		case "tpv_print":
			try {
				PrintHelper.print(this.getView());
			} catch (IOException e) {
				throw new RuntimeException(e);
			}
			break;
		default:
			break;
		}
	}

	@Override
	public void beforeF7Select(BeforeF7SelectEvent e) {
		String name = e.getProperty().getName();
		switch (name) {
		case "tpv_department":
//			deftF7(e);
			break;
		default:
			break;
		}
	}

	/**
	 * 部门过滤
	 * 
	 * @param e
	 */
	@SuppressWarnings("unused")
	private void deftF7(BeforeF7SelectEvent e) {
		IDataModel model = this.getModel();
		DynamicObject tpv_operator = (DynamicObject) model.getValue("tpv_operator");
		if (tpv_operator != null) {
			long userId = tpv_operator.getLong("id");
			List<Long> deptIds = getDeptIds(userId);
			QFilter qFilter = new QFilter("id", QCP.in, deptIds);
			ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
			showParameter.getTreeFilterParameter().getQFilters().add(qFilter);
			showParameter.getListFilterParameter().getQFilters().add(qFilter);
		}
	}

	@Override
	public void propertyChanged(PropertyChangedArgs e) {
		super.propertyChanged(e);
		String key = e.getProperty().getName();
		switch (key) {
			case "tpv_operator":
				initDept(true);
				break;
			case "tpv_collaborative":
			case "tpv_department_aps":
				setValues();
			default:
				break;
		}
	}

	/**
	 * 业务员带出部门,并设置部门锁定性
	 * 
	 * @param setDeptValue
	 */
	private void initDept(boolean setDeptValue) {
		IDataModel model = this.getModel();
		IFormView view = this.getView();
		boolean enableDept = false;
		DynamicObject tpv_operator = (DynamicObject) model.getValue("tpv_operator");
		Long deptId = null;
		List<Long> deptIds = new ArrayList<>();
		if (tpv_operator != null) {
			long userId = tpv_operator.getLong("id");
			deptIds = getDeptIds(userId);
			if (deptIds.size() > 0) {
				deptId = deptIds.get(0);
			}
			if (deptIds.size() > 1) {
				enableDept = true;
			}
			if (deptIds.size() == 0) {
				//部门为空
			}
		} else {
			model.setValue("tpv_department", null);
		}
		if (setDeptValue && deptIds.size() == 1) {
			model.setValue("tpv_department", deptId);
		}
		view.setEnable(enableDept, "tpv_department");
		model.setValue("tpv_approvaldepts","无");//协同审批字段初始赋值
	}

	/**
	 * 获取可选部门ID
	 * 
	 * @param userId
	 * @return
	 */
	public static List<Long> getDeptIds(long userId) {
		List<Long> deptIds = new ArrayList<>();
		// List<Long> allDepartmentByUserId =
		// UserServiceHelper.getAllDepartmentByUserId(userId);
		// List<Long> allDepartmentByUserId = YEM.getUserChangeAbleOrgIds(userId);
		// QFilter qFilter = new QFilter("id", QCP.in, allDepartmentByUserId);
		// qFilter.and(new QFilter("orgpattern.number", QCP.equals, "Orgform06"));
		// DynamicObjectCollection query = QueryServiceHelper.query("bos_org", "id",
		// qFilter.toArray());
		// for (DynamicObject org : query) {
		// deptIds.add(org.getLong("id"));
		// }
		QFilter qFilter = new QFilter("user", QCP.equals, userId);
		qFilter.and(new QFilter("org.orgpattern.number", QCP.equals, "Orgform06"));
		DynamicObjectCollection query = QueryServiceHelper.query("bos_userposition", "id,org", qFilter.toArray());
		for (DynamicObject org : query) {
			deptIds.add(org.getLong("org"));
		}
		return deptIds;
	}


	/**
	 * 协同审批赋值
	 */
	private void setValues(){
		ArrayList<String> approvaldepts = new ArrayList<>();
		DynamicObjectCollection entity = this.getModel().getEntryEntity("tpv_approvalsetting");
		for (DynamicObject object : entity) {
			boolean collaborative = object.getBoolean("tpv_collaborative");
			if (collaborative){
				approvaldepts.add(object.getString("tpv_department_aps.name") == null ? "无" :object.getString("tpv_department_aps.name") );
			}
		}
		if (approvaldepts.size() > 0){
			this.getModel().setValue("tpv_approvaldepts",String.join(",",approvaldepts));
		}else {
			this.getModel().setValue("tpv_approvaldepts","无");
		}
	}

}
