package tpv.tpv_wm.tpv_sc.java.formplugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.Date;

/**
 * ClassName: DeptFundLimitFromPlugin
 * Description:  部门资金临时额度表单插件
 * date: 2023/9/4 16:26
 *
 * @author: Guo jiaChen
 * @version:
 * @since JDK 1.8
 */
public class DeptFundLimitFromPlugin extends AbstractBillPlugIn {
    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String key = e.getProperty().getName();
        switch (key) {
            case "tpv_quiotastdate":
            case "tpv_quotaenddate":
                judgeTime(key);
            case"tpv_department":
                tpvDepartment();
                break;
            case"tpv_applystart":
            case"tpv_applyend":
                ApplyTime(key);
                break;
        }
    }

    private void ApplyTime(String key) {
        Date endDate = (Date) this.getModel().getValue("tpv_applyend");//截至日期
        Date startDate = (Date) this.getModel().getValue("tpv_applystart");//起始日期
        if (startDate != null && endDate != null) {
            if (startDate.compareTo(endDate) >= 0) {
                this.getView().showTipNotification("额度使用申请截至日期不能小于起始日期,请重新选择!");
                if ("tpv_applystart".equals(key)) {
                    this.getModel().setValue("tpv_applystart", null);
                } else {
                    this.getModel().setValue("tpv_applystart", null);
                }

            }
        }
    }

    /**
     * 按照【部门】取最新的《部门资金额度（测算）》【编制月综合资金额度】+已审核未过期《部门资金临时额度》【批复增加资金额度】
     */
    private void tpvDepartment() {
        DynamicObject tpvDepartment = (DynamicObject) this.getModel().getValue("tpv_department");
        if (tpvDepartment!=null){
            long departmentLong = tpvDepartment.getLong("id");
            BigDecimal decimal = BigDecimal.ZERO;
            QFilter qFilter = new QFilter("tpv_entryentity.tpv_departments", "=", departmentLong);
            DynamicObject[] scFundslimtmeas = BusinessDataServiceHelper.load("tpv_sc_fundslimtmeas", "tpv_departments,tpv_orgmonthcamt,tpv_entryentity", qFilter.toArray(), "createtime desc");
            if (scFundslimtmeas.length!=0){
                DynamicObjectCollection tpvEntryentitys = scFundslimtmeas[0].getDynamicObjectCollection("tpv_entryentity");
                for (DynamicObject tpvEntryentity : tpvEntryentitys) {
                    DynamicObject tpvDepartments = tpvEntryentity.getDynamicObject("tpv_departments");
                    if (tpvDepartments!=null){
                        long aLong = tpvDepartments.getLong("id");
                        if (aLong==departmentLong){
                            decimal=decimal.add(tpvEntryentity.getBigDecimal("tpv_orgmonthcamt"));
                        }
                    }
                }
            }
            Date date = new Date();
            Object pkValue = this.getModel().getDataEntity(true).getPkValue();
            QFilter qFilterDep = new QFilter("billstatus","=", "C");
            qFilterDep.and(new QFilter("tpv_department","=", tpvDepartment.getPkValue()));
            qFilterDep.and(new QFilter("tpv_applystart",QCP.less_equals,date));
            qFilterDep.and(new QFilter("tpv_applyend",QCP.large_equals,date));
            DynamicObject[] tpvScDeptfundlimits = BusinessDataServiceHelper.load("tpv_sc_deptfundlimit", "tpv_replyamount", qFilterDep.toArray());
            for (DynamicObject tpvScDeptfundlimit : tpvScDeptfundlimits) {
                decimal=decimal.add( tpvScDeptfundlimit.getBigDecimal("tpv_replyamount"));
            }
            this.getModel().setValue("tpv_deptamountlimit",decimal);
        }else{
            this.getModel().setValue("tpv_deptamountlimit",null);
        }


    }

    /**
     * @param key:
     * @return void
     * @author: chen
     * @date: 2023/11/19 17:00
     * @description: 判断起始日期是否小于截至日期
     */
    private void judgeTime(String key) {
        Date endDate = (Date) this.getModel().getValue("tpv_quotaenddate");//截至日期
        Date startDate = (Date) this.getModel().getValue("tpv_quiotastdate");//起始日期
        if (startDate != null && endDate != null) {
            if (startDate.compareTo(endDate) >= 0) {
                this.getView().showTipNotification("额度使用截至日期不能小于起始日期,请重新选择!");
                if ("tpv_quiotastdate".equals(key)) {
                    this.getModel().setValue("tpv_quiotastdate", null);
                } else {
                    this.getModel().setValue("tpv_quotaenddate", null);
                }

            }
        }
    }
}