package tpv.tpv_wm.tpv_sc.java.formplugin.form;

import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.clr.DataEntityPropertyCollection;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.form.IFormView;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.permission.api.HasPermOrgResult;
import tpv.tpv_wm.tpv_sc.java.report.GuaraFeeShareQueryRpt;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 担保费分摊 -表单插件
 * @Author LinRui
 * @Date 2024/2/5 11:21
 */
public class GuaraFeeShareFormPlugin extends AbstractBillPlugIn {
    private static final String entryKey_1 = "tpv_entryentity";

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addItemClickListeners("tpv_advcontoolbarap");
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String itemKey = evt.getItemKey().toLowerCase();
        switch (itemKey) {
            case "tpv_sharefee":
                sharefeeClick();
                break;
            default:
                break;
        }
    }

    private void sharefeeClick() {
        IFormView view = this.getView();
        IDataModel model = this.getModel();
        DynamicObject dataEntity = model.getDataEntity(true);
        List<String> notInputs = new ArrayList<>();
        Map<String, Object> param = new HashMap<>();
        YEM.getNotNull(view, notInputs, dataEntity, "tpv_company");
        YEM.getNotNull(view, notInputs, dataEntity, "tpv_year");
        if (notInputs.size() > 0) {
            view.showTipNotification(String.format("请先填写【%s】。", String.join("、", notInputs)));
        } else {
            sharefeeData();
        }
    }

    private void sharefeeData() {
        IFormView view = this.getView();
        IDataModel model = view.getModel();
        model.deleteEntryData(entryKey_1);   //清除分录
        DynamicObject dataEntity = model.getDataEntity(true);
        String whereFilter = "1=1";
        // 公司
        DynamicObject tpvCompany = dataEntity.getDynamicObject("tpv_company");
        if (YEM.isNotEmpty(tpvCompany)) {
            whereFilter = String.format("%s and tpv_entryorgs = %s", whereFilter, tpvCompany.getPkValue());
        } else {
            HasPermOrgResult userChangeAbleOrg = YEM.getUserChangeAbleOrg();
            if (!userChangeAbleOrg.hasAllOrgPerm()) {
                List<String> ids = YEM.getUserChangeAbleOrgIds().stream().map(x -> String.valueOf(x))
                        .collect(Collectors.toList());
                whereFilter = String.format("%s and tpv_entryorgs in (%s)", whereFilter, String.join(",", ids));
            }
        }
        Date tpvYear = dataEntity.getDate("tpv_year");
        SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        if (null != tpvYear) {
            Date yearFirst = DateUtil.getYearFirst(tpvYear); //当前 年的第一天
            Date dayStartTime = getDayStartTime(yearFirst);
            Date yearLast = DateUtil.getYearLast(tpvYear); //当前 年的最后一天
            Date dayEndTime = getDayEndTime(yearLast);
            String formatStart =  str.format(dayStartTime);
            String formatEnd =  str.format(dayEndTime);
            whereFilter = String.format("%s and tpv_endtime >= to_date('" + formatStart + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
            whereFilter = String.format("%s and tpv_endtime <= to_date('" + formatEnd + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        DataSet query = GuaraFeeShareQueryRpt.query(whereFilter);
        DynamicObjectType enrtyType = model.getEntryEntity(entryKey_1).getDynamicObjectType();
        DataEntityPropertyCollection properties = enrtyType.getProperties();
        List<String> dataSetAliasList = RptUtil.getDataSetAliasList(query);
        for (Row row : query) {
            int index = model.createNewEntryRow(entryKey_1);
            for (IDataEntityProperty property : properties) {
                String key = property.getName();
                if (dataSetAliasList.contains(key)) {
                    model.setValue(key, row.get(key), index);
                }
            }
        }
    }

    /**
     * 获取日期的结束时间
     */
    public static Date getDayEndTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 23, 59, 59);
            calendar.set(14, 999);
            return calendar.getTime();
        }
    }

    /**
     * 获取该日期的开始时间
     */
    public static Date getDayStartTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 0, 0, 0);
            calendar.set(14, 0);
            return calendar.getTime();
        }
    }
}
