package tpv.tpv_wm.tpv_sc.java.formplugin.form;

import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.dataentity.metadata.clr.DataEntityPropertyCollection;
import kd.bos.dataentity.metadata.dynamicobject.DynamicObjectType;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.form.IFormView;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.permission.api.HasPermOrgResult;
import tpv.tpv_wm.tpv_sc.java.report.MoreDeptfundsUseQueryRpt;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 多部门资金占用 -表单插件
 * @Author LinRui
 * @Date 2024/2/27 14:52
 */
public class MoreDeptFundsUse extends AbstractBillPlugIn {
    private static final String entryKey_1 = "tpv_entryentity";

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addItemClickListeners("tpv_advcontoolbarap");
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String itemKey = evt.getItemKey().toLowerCase();
        switch (itemKey) {
            case "tpv_depfundsuse":
                depfundsuseClick();
                break;
            default:
                break;
        }
    }

    private void depfundsuseClick() {
        IFormView view = this.getView();
        IDataModel model = this.getModel();
        DynamicObject dataEntity = model.getDataEntity(true);
        List<String> notInputs = new ArrayList<>();
        YEM.getNotNull(view, notInputs, dataEntity, "tpv_companys");
        YEM.getNotNull(view, notInputs, dataEntity, "tpv_year");
        YEM.getNotNull(view, notInputs, dataEntity, "tpv_month");
        if (notInputs.size() > 0) {
            view.showTipNotification(String.format("请先填写【%s】。", String.join("、", notInputs)));
        } else {
            depfundsuseData();
        }
    }

    private void depfundsuseData() {
        IFormView view = this.getView();
        IDataModel model = view.getModel();
        model.deleteEntryData(entryKey_1);   //清除分录
        DynamicObject dataEntity = model.getDataEntity(true);
        String whereFilter = "1=1";
        DynamicObjectCollection tpvCompanys = dataEntity.getDynamicObjectCollection("tpv_companys");
        if (tpvCompanys.size() > 0) {
            List<String> ids = tpvCompanys.stream().map(x -> String.valueOf(x.getDynamicObject("fbasedataid")
                    .getPkValue())).collect(Collectors.toList());
            whereFilter = String.format("%s and tpv_companys in (%s)", whereFilter, String.join(",", ids));
        } else {
            HasPermOrgResult userChangeAbleOrg = YEM.getUserChangeAbleOrg();
            if (!userChangeAbleOrg.hasAllOrgPerm()) {
                List<String> ids = YEM.getUserChangeAbleOrgIds().stream().map(x -> String.valueOf(x))
                        .collect(Collectors.toList());
                whereFilter = String.format("%s and tpv_companys in (%s)", whereFilter, String.join(",", ids));
            }
        }
        Date tpvYear = dataEntity.getDate("tpv_year");

        SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String tpvMonth = dataEntity.getString("tpv_month");

        int i = monthConvertInt(tpvMonth);
        Calendar ca = Calendar.getInstance();
        ca.setTime(tpvYear);
        ca.set(2, i - 1);
        Date monthFirstDay = DateUtil.getMonthFirstDay(ca.getTime()); //当前 年 当前 月的第一天
        Date monthToEndDay = DateUtil.getMonthEndDay(ca.getTime()); //当前 年 当前 月的最后一天
        Date dayStartTime = getDayStartTime(monthFirstDay);
        Date dayEndTime = getDayEndTime(monthToEndDay);
        String formatStart = str.format(dayStartTime);
        String formatEnd = str.format(dayEndTime);
        whereFilter = String.format("%s and tpv_businessdate >= to_date('" + formatStart + "','yyyy-MM-dd HH:mm:ss')", whereFilter);
        whereFilter = String.format("%s and tpv_businessdate <= to_date('" + formatEnd + "','yyyy-MM-dd HH:mm:ss')", whereFilter);
        DataSet query = MoreDeptfundsUseQueryRpt.query(whereFilter,dayStartTime,dayEndTime);
        DynamicObjectType enrtyType = model.getEntryEntity(entryKey_1).getDynamicObjectType();
        DataEntityPropertyCollection properties = enrtyType.getProperties();
        List<String> dataSetAliasList = RptUtil.getDataSetAliasList(query);
        for (Row row : query) {
            int index = model.createNewEntryRow(entryKey_1);
            for (IDataEntityProperty property : properties) {
                String key = property.getName();
                if (dataSetAliasList.contains(key)) {
                    model.setValue(key, row.get(key), index);
                }
            }
        }
        this.getView().updateView(entryKey_1);
    }

    /**
     * @Description: //将月份数值转为整数
     * @param: month
     **/
    public static int monthConvertInt(String month) {
        if (YEM.isNotEmpty(month)) {
            int i = 0;
            switch (month) {
                case "Jan":
                    i = 1;
                    break;
                case "Feb":
                    i = 2;
                    break;
                case "Mar":
                    i = 3;
                    break;
                case "Apr":
                    i = 4;
                    break;
                case "May":
                    i = 5;
                    break;
                case "Jun":
                    i = 6;
                    break;
                case "Jul":
                    i = 7;
                    break;
                case "Aug":
                    i = 8;
                    break;
                case "Sep":
                    i = 9;
                    break;
                case "Oct":
                    i = 10;
                    break;
                case "Nov":
                    i = 11;
                    break;
                case "Dec":
                    i = 12;
                    break;
                default:
                    break;
            }
            return i;
        }
        return 0;
    }

    /**
     * 获取日期的结束时间
     */
    public static Date getDayEndTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 23, 59, 59);
            calendar.set(14, 999);
            return calendar.getTime();
        }
    }

    /**
     * 获取该日期的开始时间
     */
    public static Date getDayStartTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 0, 0, 0);
            calendar.set(14, 0);
            return calendar.getTime();
        }
    }
}
