package tpv.tpv_wm.tpv_sc.java.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @ClassName: FundsUsedsSaveOp
 * @Description: 资金占用明细（财务） 单据保存时，检查公司+年+月，已经存在资金占用，提醒“已经存在XX公司XX年XX月的资金占用，编号为XXXX，不可重复生成！”,不可继续。"
 * @author: LinRui
 * @date: 2024-07-16
 */
public class FundsUsedsSaveOp extends AbstractOperationServicePlugIn {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> keys = e.getFieldKeys();
        keys.add("tpv_year");
        keys.add("tpv_month");
        keys.add("tpv_company");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {
            @Override
            public void validate() {
                ExtendedDataEntity[] dataEntities = this.getDataEntities();
                for (ExtendedDataEntity dataEntity : dataEntities) {
                    saveOrgValidators(dataEntity);
                }
            }
            public void saveOrgValidators(ExtendedDataEntity dataEntity) {
                DynamicObject entity = dataEntity.getDataEntity();
                DynamicObject tpvCompany = entity.getDynamicObject("tpv_company");
                Date tpvYear = entity.getDate("tpv_year");
                if (tpvYear==null || YEM.isNotEmpty(tpvCompany)){
                    return;
                }
                Calendar ca = Calendar.getInstance();
                ca.setTime(tpvYear);
                int year = ca.get(2);
                String tpvMonth = entity.getString("tpv_month");
                DynamicObject[] tpvScFundsused = BusinessDataServiceHelper.load("tpv_sc_fundsused", "tpv_company,billno",
                        new QFilter[]{new QFilter("tpv_year", "=", tpvYear),
                                new QFilter("tpv_month", "=", tpvMonth),
                                new QFilter("tpv_company.id", "=", tpvCompany.getLong("id")),
                                new QFilter("id", QCP.not_equals, entity.getPkValue())});
                if (tpvScFundsused.length > 0) {
                    for (DynamicObject dynamicObject : tpvScFundsused) {
                        this.addErrorMessage(dataEntity, "已经存在"+tpvCompany.getString("name")+""+year+"年"+ tpvMonth +"的资金占用，编号为"+dynamicObject.getString("billno")+"，不可重复生成！");
                    }
                }
            }
        });
    }
}
