package tpv.tpv_wm.tpv_sc.java.opplugin.validator;

import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @Description: 资金占用明细（组合） -反审核校验
 * @Author LinRui
 * @Date 2024/3/8 19:19
 */
public class FundsUsedInfoUnAuditValidator extends AbstractValidator {
    @Override
    public void validate() {
        ExtendedDataEntity[] dataEntities = this.dataEntities;
        for (int i = 0; i < dataEntities.length; i++) {
            ExtendedDataEntity extend = dataEntities[i];
            DynamicObject entity = extend.getDataEntity();
            check(entity, extend);
        }
    }

    public void check(DynamicObject entity, ExtendedDataEntity extend) {
        entity = BusinessDataServiceHelper.loadSingle(entity.getPkValue(),entity.getDynamicObjectType().getName());
        Date tpvYear = entity.getDate("tpv_year");
        String tpvMonth = entity.getString("tpv_month");
        DynamicObject tpvCompany = entity.getDynamicObject("tpv_company");
        DynamicObject tpvDepartmentf = entity.getDynamicObject("tpv_departmentf");
        Date dayStartTime = null;
        Date dayEndTime = null;
        if (YEM.isNotEmpty(tpvMonth) && null != tpvYear) {
            int i = monthConvertInt(tpvMonth);
            tpvYear.setMonth(i - 1);
            Date monthFirstDay = DateUtil.getMonthFirstDay(tpvYear); //当前 年 当前 月的第一天
            Date monthToEndDay = DateUtil.getMonthEndDay(tpvYear); //当前 年 当前 月的最后一天
            dayStartTime = getDayStartTime(monthFirstDay);
            dayEndTime = getDayEndTime(monthToEndDay);
        }
        DynamicObjectCollection dynamicObjects = entity.getDynamicObjectCollection("tpv_sc_fundsused_info");
        List<Long> deptIdList = new ArrayList<>();
        if (dynamicObjects.size() > 0) {
            for (DynamicObject dynamicObject : dynamicObjects) {
                DynamicObject tpvDepartments = dynamicObject.getDynamicObject("tpv_departments");
                if (YEM.isNotEmpty(tpvDepartments)) {
                    deptIdList.add(tpvDepartments.getLong("id"));
                }
            }
        }
        DataSet dataSet = ORM.create().queryDataSet("tpv_sc_deptfundspool", "tpv_sc_deptfundspool", "id,tpv_sc_fundspool_info",
                new QFilter[]{ new QFilter("billstatus", QCP.equals, "C"),
                        new QFilter("tpv_sc_fundspool_info", QCP.large_than, 0),
                        new QFilter("tpv_sc_fundspool_info.tpv_businessdate", QCP.large_than, dayStartTime),
                        new QFilter("tpv_sc_fundspool_info.tpv_businessdate", QCP.less_equals, dayEndTime),
                        new QFilter("tpv_sc_fundspool_info.tpv_departments.id", QCP.large_than, 0),
                        new QFilter("tpv_sc_fundspool_info.tpv_departments", QCP.in, deptIdList.size() > 0 ? deptIdList : 0)});
        DynamicObjectCollection dynamicObjectCollection = ORM.create().toPlainDynamicObjectCollection(dataSet);
        if (dynamicObjectCollection.size() > 0) {
            this.addMessage(extend,"该时间段内，本单中已有部门生成部门资金池，不允许反审核，请先删除对应部门资金池数据！");
        }
    }

    /**
     * @Description: //将月份数值转为整数
     * @param: month
     **/
    private int monthConvertInt(String month) {
        if (YEM.isNotEmpty(month)) {
            int i = 0;
            switch (month) {
                case "Jan": i = 1;break; case "Feb": i = 2;break; case "Mar": i = 3;break;
                case "Apr": i = 4;break; case "May": i = 5;break; case "Jun": i = 6;break;
                case "Jul": i = 7;break; case "Aug": i = 8;break; case "Sep": i = 9;break;
                case "Oct": i = 10;break; case "Nov": i = 11;break; case "Dec": i = 12;break;
                default:
                    break;
            }
            return i;
        }
        return 0;
    }

    /**
     * 获取日期的结束时间
     */
    public static Date getDayEndTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 23, 59, 59);
            calendar.set(14, 999);
            return calendar.getTime();
        }
    }

    /**
     * 获取该日期的开始时间
     */
    public static Date getDayStartTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 0, 0, 0);
            calendar.set(14, 0);
            return calendar.getTime();
        }
    }
}
