package tpv.tpv_wm.tpv_sc.java.opplugin.validator;

import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.Calendar;
import java.util.Date;

/**
 * @Description: 资金占用明细（财务） -反审核校验
 * @Author LinRui
 * @Date 2024/3/8 19:19
 */
public class FundsUsedsUnAuditValidator extends AbstractValidator {
    @Override
    public void validate() {
        ExtendedDataEntity[] dataEntities = this.dataEntities;
        for (int i = 0; i < dataEntities.length; i++) {
            ExtendedDataEntity extend = dataEntities[i];
            DynamicObject entity = extend.getDataEntity();
            check(entity, extend);
        }
    }

    public void check(DynamicObject entity, ExtendedDataEntity extend) {
        entity = BusinessDataServiceHelper.loadSingle(entity.getPkValue(),entity.getDynamicObjectType().getName());
        Date tpvYear = entity.getDate("tpv_year");
        DynamicObject tpvCompany = entity.getDynamicObject("tpv_company");
        String tpvMonth = entity.getString("tpv_month");
        int monthed = monthConvertInt(tpvMonth);
        if (monthed == 12) {
            monthed = 1;
            tpvYear.setYear(tpvYear.getYear() + 1);
        } else {
            monthed = monthed + 1;
        }
        String convertmonth = IntConvertmonth(monthed);
        QFilter qFilter = new QFilter("tpv_company", QCP.equals, tpvCompany.getLong("id"));
        qFilter.and(new QFilter("tpv_year", QCP.equals, tpvYear));
        qFilter.and(new QFilter("tpv_month", QCP.equals, convertmonth));
        DynamicObject[] load = BusinessDataServiceHelper.load(entity.getDynamicObjectType().getName(), "id,tpv_year,tpv_company,tpv_month", qFilter.toArray());
        if (load.length > 0) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(tpvYear);
            this.addMessage(extend,"公司为 " + calendar.get(Calendar.YEAR)  + "年" + monthed + "月的资金占用未删除，本月资金占用不能反审核，请检查！");
        }
    }

    /**
     * @Description: //将月份数值转为整数
     * @param: month
     **/
    private int monthConvertInt(String month) {
        if (YEM.isNotEmpty(month)) {
            int i = 0;
            switch (month) {
                case "Jan": i = 1;break; case "Feb": i = 2;break; case "Mar": i = 3;break;
                case "Apr": i = 4;break; case "May": i = 5;break; case "Jun": i = 6;break;
                case "Jul": i = 7;break; case "Aug": i = 8;break; case "Sep": i = 9;break;
                case "Oct": i = 10;break; case "Nov": i = 11;break; case "Dec": i = 12;break;
                default:
                    break;
            }
            return i;
        }
        return 0;
    }

    /**
     * @Description: //将月份数值转为整数
     * @param: month
     **/
    private static String IntConvertmonth(int month) {
        if (YEM.isNotEmpty(month)) {
            String monthValue = "";
            switch (month) {
                case 1: monthValue = "Jan";break; case 2: monthValue = "Feb";break; case 3: monthValue = "Mar";break;
                case 4: monthValue = "Apr";break; case 5: monthValue = "May";break; case 6: monthValue = "Jun";break;
                case 7: monthValue = "Jul";break; case 8: monthValue = "Aug";break; case 9: monthValue = "Sep";break;
                case 10: monthValue = "Oct";break; case 11: monthValue = "Nov";break; case 12: monthValue = "Dec";break;
                default:
                    break;
            }
            return monthValue;
        }
        return null;
    }

    /**
     * 获取日期的结束时间
     */
    public static Date getDayEndTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 23, 59, 59);
            calendar.set(14, 999);
            return calendar.getTime();
        }
    }

    /**
     * 获取该日期的开始时间
     */
    public static Date getDayStartTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 0, 0, 0);
            calendar.set(14, 0);
            return calendar.getTime();
        }
    }
}
