package tpv.tpv_wm.tpv_sc.java.report;

import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @ClassName: ActualReceiptQueryRpt
 * @Description: 实收付明细查询
 * @author: zbh
 * @date: 2024-07-25
 */
public class ActualReceiptQueryRpt extends AbstractReportListDataPlugin {

    private static final String algoKey = ActualReceiptQueryRpt.class.getName();

    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        FilterInfo filterInfo = param.getFilter();
        DynamicObject tpv_org_f = filterInfo.getDynamicObject("tpv_org_f");// 公司
        DynamicObject tpv_dept_f = filterInfo.getDynamicObject("tpv_dept_f"); // 部门
        DynamicObject tpv_groupdept_f = filterInfo.getDynamicObject("tpv_groupdept_f"); // 部门组
        Date tpv_startdate_f = filterInfo.getDate("tpv_startdate_f");// 开始日期
        Date tpv_enddate_f = filterInfo.getDate("tpv_enddate_f");// 结束日期
        String tpvTypecom = filterInfo.getString("tpv_typecom");//类别
        Long orgid = YEM.isNotEmpty(tpv_org_f) ? tpv_org_f.getLong("id") : 0L;
        Long deptid = YEM.isNotEmpty(tpv_dept_f) ? tpv_dept_f.getLong("id") : 0L;
        Long groupdeptid = YEM.isNotEmpty(tpv_groupdept_f) ? tpv_groupdept_f.getLong("id") : 0L;
        DataSet query = query(orgid, deptid, groupdeptid, tpv_startdate_f, tpv_enddate_f, tpvTypecom);
        return query;
    }

    public static DataSet query(Long orgid, Long depid, Long groupdeptid, Date startdate, Date enddate, String tpvTypecom) {
        QFilter qFilter = new QFilter("tpv_bigdept.id", QCP.large_than, 0l);
        List list = new ArrayList();
        list.add("A");
        list.add("B");
        list.add("C");
        list.add("D");
        list.add("E");
        list.add("F");
        list.add("G");
        qFilter.and(new QFilter("tpv_realpreocp", QCP.equals, "A"));
        qFilter.and(new QFilter("tpv_type", QCP.in, list));
        if (YEM.isNotEmpty(orgid)) {
            qFilter.and(new QFilter("tpv_org.id", QCP.equals, orgid));
        }
        if (YEM.isNotEmpty(depid)) {
            qFilter.and(new QFilter("tpv_bigdept.id", QCP.equals, depid));
        }
        if (YEM.isNotEmpty(groupdeptid)) {
            qFilter.and(new QFilter("tpv_depertment.id", QCP.equals, groupdeptid));
        }
        if (null != startdate) {
            Date dayStartTime = DateUtil.getStartOrEndDate(startdate, false);
//            Date dayStartTime = DeptDayNetFlowQueryRpt.getDayStartTime(startdate);
            qFilter.and(new QFilter("tpv_datetime", QCP.large_equals, dayStartTime));
        }
        if (null != enddate) {
            Date dayEndTime = DateUtil.getStartOrEndDate(enddate, true);
//            Date dayEndTime = DeptDayNetFlowQueryRpt.getDayEndTime(enddate);
            qFilter.and(new QFilter("tpv_datetime", QCP.less_equals, dayEndTime));
        }
        if (null != startdate & null != enddate) {
            qFilter.and(new QFilter("tpv_datetime", QCP.not_equals, null));
        }
        if (YEM.isNotEmpty(tpvTypecom)) {
            qFilter.and(new QFilter("tpv_type", QCP.equals, tpvTypecom));
        }
        String selectfields = "tpv_org.name tpv_companyname,tpv_bigdept.name tpv_deptname,tpv_depertment.name tpv_groupdeptname,tpv_datetime tpv_businessdate," +
                "tpv_type,tpv_addorsub,tpv_currency,tpv_amount,tpv_orignalamount,tpv_usdamount,tpv_tousdrate,tpv_tormbrate,tpv_triggerbill,tpv_triggerbillno,tpv_org.id tpv_companyid," +
                "tpv_bigdept.id tpv_deptid,tpv_depertment.id tpv_groupdeptid";
        //查询部门资金额度底表数据
        DataSet tpv_sc_capitallimit = ORM.create().queryDataSet(algoKey, "tpv_sc_capitallimit", selectfields, qFilter.toArray());
        tpv_sc_capitallimit = tpv_sc_capitallimit.select("tpv_companyname", "tpv_deptname", "tpv_groupdeptname", "tpv_businessdate", "tpv_type", "tpv_addorsub",
                "tpv_currency", "tpv_amount", "tpv_orignalamount", "tpv_usdamount", "tpv_tousdrate", "tpv_tormbrate", "tpv_triggerbill", "tpv_triggerbillno", "tpv_companyid",
                "tpv_deptid", "tpv_groupdeptid", "(case when tpv_addorsub =='A' then tpv_orignalamount else (-1)*tpv_orignalamount end) tpv_orignala",
                "(case when tpv_addorsub =='A' then tpv_usdamount else (-1)*tpv_usdamount end) tpv_usdarsub",
                "(case when tpv_addorsub =='A' then tpv_amount else (-1)*tpv_amount end) tpv_amountala");

        return tpv_sc_capitallimit;
    }
}
