package tpv.tpv_wm.tpv_sc.java.report;

import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.*;

/**
 * @Description: 部门日均资金占用明细 -报表查询插件
 * @Author LinRui
 * @Date 2024/2/1 11:40
 */
public class DepDayfundsUsedQueryRpt extends AbstractReportListDataPlugin {

    private static final String algoKey = DepDayfundsUsedQueryRpt.class.getName();

    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        return null;
    }

    public static DataSet query(String whereFilter) {
        DataSet mainDataSet = mainDataSet(whereFilter);
        String whereFilters = splitWhereFilter(whereFilter,0);
        String dayStartTime = splitWhereFilter(whereFilter,1);
        String dayEndTime = splitWhereFilter(whereFilter,2);
        Date startTime = DateUtil.parseDateHms(dayStartTime);
        Date endTime = DateUtil.parseDateHms(dayEndTime);
        Map<Long, Long> parentDeptIds = mapParentDeptIds();
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        for (DynamicObject col : col_s) {
            Date tpvCreatetime = col.getDate("tpv_createtime");
            if (tpvCreatetime.compareTo(startTime) > 0) {
                col.set("tpv_starttime",startTime);
            } else {
                col.set("tpv_starttime",tpvCreatetime);
            }
            col.set("tpv_endtime",endTime);
            Date tpvStarttime = col.getDate("tpv_starttime");
            Date tpvEndtime = col.getDate("tpv_endtime");
            int days = DateUtil.countActualDays(tpvStarttime, tpvEndtime); //相差天数
            col.set("tpv_dateocc",days);

            //切换大部门
            if (parentDeptIds.containsKey(col.getLong("tpv_depertment"))) {
                col.set("tpv_depertment", parentDeptIds.get(col.getLong("tpv_depertment")));
            }
        }

        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
        if (YEM.isNotEmpty(whereFilters)) {
            createDataSet = createDataSet.where(whereFilters);
        }
        createDataSet = createDataSet.groupBy(new String[]{"tpv_companys","tpv_depertment","tpv_starttime","tpv_endtime","tpv_dateocc"})
                .sum("tpv_occupied").sum("tpv_interest").sum("tpv_applyamt").sum("tpv_profit").sum("tpv_byamt")
                .sum("tpv_sharp").sum("tpv_accamt").sum("tpv_favourable").sum("tpv_supamount").sum("tpv_supinterest").finish();
        createDataSet = createDataSet.addField("(tpv_occupied + tpv_applyamt + tpv_byamt + tpv_accamt + tpv_supamount) / tpv_dateocc tpv_totalday","tpv_totalday");
        createDataSet = createDataSet.addField("tpv_companys tpv_entryorgs","tpv_entryorgs");

        return createDataSet.orderBy(new String[]{"tpv_starttime desc"});
    }

    private static DataSet mainDataSet(String whereFilter) {
        QFilter orgdutyQfilter = new QFilter("enable", QCP.equals, "1");
        orgdutyQfilter.and(new QFilter("orgduty.number", QCP.in, new String[]{"ZHKC002"}));
        DataSet orgduty = ORM.create().queryDataSet(algoKey, "bos_org_dutyrelation", "org.id tpv_depid",
                orgdutyQfilter.toArray());
        DataSet copyOrgduty = orgduty.copy();
        List<Long> orgdutyIds = new ArrayList<>();
        for (Row row : copyOrgduty) {
            if (!orgdutyIds.contains(row.getLong("tpv_depid"))) {
                orgdutyIds.add(row.getLong("tpv_depid"));
            }
        }
        String selectfields = "id,tpv_year,tpv_month,tpv_company,tpv_sc_fundsused_aa,tpv_sc_fundsused_aa.tpv_businessdate tpv_businessdate," +
                "tpv_sc_fundsused_aa.tpv_companys tpv_companys,tpv_sc_fundsused_aa.tpv_depertment tpv_depertment," +
                "tpv_sc_fundsused_aa.tpv_depertment.createtime tpv_createtime," +
                "tpv_sc_fundsused_aa.tpv_occupied tpv_occupied," + //现款(占用金额)
                "tpv_sc_fundsused_aa.tpv_interest tpv_interest," + //现款(利息)
                "tpv_sc_fundsused_aa.tpv_applyamt tpv_applyamt," + //银承(占用金额)
                "tpv_sc_fundsused_aa.tpv_profit tpv_profit," + //银承(利息)
                "tpv_sc_fundsused_aa.tpv_byamt tpv_byamt," + //商票(占用金额)
                "tpv_sc_fundsused_aa.tpv_sharp tpv_sharp," + //商票(利息)
                "tpv_sc_fundsused_aa.tpv_accamt tpv_accamt," + //供应链票据(占用金额)
                "tpv_sc_fundsused_aa.tpv_favourable tpv_favourable"; //供应链票据(利息)
        List<QFilter> qFilter1 = new ArrayList<>();
        qFilter1.add(new QFilter("billstatus", QCP.equals, "C"));
        if (orgdutyIds.size() > 0) {
            qFilter1.add(new QFilter("tpv_sc_fundsused_aa.tpv_depertment.id", QCP.in, orgdutyIds));
        } else {
            qFilter1.add(new QFilter("tpv_sc_fundsused_aa.tpv_depertment.id", QCP.less_than, 0));
        }
        DataSet dataSet = ORM.create().queryDataSet(algoKey, "tpv_sc_fundsused", selectfields, qFilter1.toArray(new QFilter[qFilter1.size()]));
        dataSet = dataSet.addField("0.0 tpv_supamount","tpv_supamount");
        dataSet = dataSet.addField("0.0 tpv_supinterest","tpv_supinterest");
        dataSet = dataSet.select("tpv_companys","tpv_depertment","tpv_createtime","tpv_businessdate",
                "tpv_occupied", "tpv_interest","tpv_applyamt", "tpv_profit","tpv_byamt","tpv_sharp","tpv_accamt",
                "tpv_favourable","tpv_supamount","tpv_supinterest");

        String selectfields2 = "id,tpv_deptfundyear tpv_year,tpv_deptfundmonth tpv_month,tpv_company,tpv_entryentity," +
                "tpv_entryentity.tpv_businessdates tpv_businessdate," +
                "tpv_entryentity.tpv_companys.id tpv_companys," +
                "tpv_entryentity.tpv_dept.id tpv_depertment," +
                "tpv_entryentity.tpv_dept.createtime tpv_createtime," +
                "tpv_entryentity.tpv_supamount tpv_supamount," +
                "tpv_entryentity.tpv_supinterest tpv_supinterest";
        List<QFilter> qFilter2 = new ArrayList<>();
        qFilter2.add(new QFilter("billstatus", QCP.equals, "C"));
        if (orgdutyIds.size() > 0) {
            qFilter2.add(new QFilter("tpv_sc_fundsused_aa.tpv_depertment.id", QCP.in, orgdutyIds));
        } else {
            qFilter2.add(new QFilter("tpv_sc_fundsused_aa.tpv_depertment.id", QCP.less_than, 0));
        }
        DataSet tpv_sc_supfunddetails = ORM.create().queryDataSet(algoKey, "tpv_sc_supfunddetails", selectfields2, qFilter2.toArray(new QFilter[qFilter2.size()]));
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.addField("0.0 tpv_occupied","tpv_occupied");
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.addField("0.0 tpv_interest","tpv_interest");
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.addField("0.0 tpv_applyamt","tpv_applyamt");
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.addField("0.0 tpv_profit","tpv_profit");
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.addField("0.0 tpv_byamt","tpv_byamt");
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.addField("0.0 tpv_sharp","tpv_sharp");
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.addField("0.0 tpv_accamt","tpv_accamt");
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.addField("0.0 tpv_favourable","tpv_favourable");
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.select("tpv_companys","tpv_depertment","tpv_createtime","tpv_businessdate",
                "tpv_occupied", "tpv_interest","tpv_applyamt", "tpv_profit","tpv_byamt","tpv_sharp","tpv_accamt",
                "tpv_favourable","tpv_supamount","tpv_supinterest");
        dataSet = dataSet.union(tpv_sc_supfunddetails);
        dataSet = dataSet.addField("null tpv_starttime","tpv_starttime");
        dataSet = dataSet.addField("null tpv_endtime","tpv_endtime");
        dataSet = dataSet.addField("1 tpv_dateocc","tpv_dateocc");
        return dataSet;
    }

    private static String splitWhereFilter(String whereFilter, int value) {
        String[] split = whereFilter.split("》");
        if (split.length > 0) {
            int i = 0;
            for (String object : split) {
                if (i == 0 && value == 0) whereFilter = object; //where条件
                else if (i == 1 && value == 1) whereFilter = object; //开始日期
                else if (i == 2 && value == 2) whereFilter = object; //结束日期
                i++;
            }
        }
        return whereFilter;
    }

    private static Map<Long,Long> mapParentDeptIds(){
        Map<Long,Long> mapParentId = new HashMap<>();
        QFilter qFilter = new QFilter("enable", QCP.equals, "1");
//        qFilter.and(new QFilter("number",QCP.equals,"Org-01-001-019-001"));
        qFilter.and(new QFilter("orgpattern.number",QCP.in,new String[]{"OF-01","Orgform06"}));
        DynamicObject[] bosAdminorgs = BusinessDataServiceHelper.load("bos_adminorg","id,orgpattern,structure,structure.viewparent", qFilter.toArray());
        if (bosAdminorgs.length > 0) {
            for (DynamicObject bosAdminorg : bosAdminorgs) {
                DynamicObject orgpattern = bosAdminorg.getDynamicObject("orgpattern");
                if (YEM.isNotEmpty(orgpattern)) {
                    String number = orgpattern.getString("number");
                    if ("OF-01".equals(number)) { //小组只会对应一个部门
                        DynamicObjectCollection dynamicObjects = bosAdminorg.getDynamicObjectCollection("structure");
                        if (dynamicObjects.size() > 0) {
                            for (DynamicObject dynamicObject : dynamicObjects) {
                                DynamicObject viewparent = dynamicObject.getDynamicObject("viewparent");
                                if (YEM.isNotEmpty(viewparent)) {
                                    mapParentId.put(bosAdminorg.getLong("id"),viewparent.getLong("id"));
                                }
                                break;
                            }
                        }
                    } else if ("Orgform06".equals(number)) {
                        mapParentId.put(bosAdminorg.getLong("id"),bosAdminorg.getLong("id"));
                    }
                }
            }
        }
        return mapParentId;
    }

}
