package tpv.tpv_wm.tpv_sc.java.report;

import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.util.Calendar;
import java.util.Date;

/**
 * @Description: 部门每日净流量表 -报表查询
 * @Author LinRui
 * @Date 2024/5/8 10:11
 */
public class DeptDayNetFlowQueryRpt extends AbstractReportListDataPlugin {
    private static final String algoKey = DeptDayNetFlowQueryRpt.class.getName();

    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        FilterInfo filterInfo = param.getFilter();
        DynamicObject tpv_org_f = filterInfo.getDynamicObject("tpv_org_f");// 公司
        DynamicObject tpv_dept_f = filterInfo.getDynamicObject("tpv_dept_f"); // 部门
        DynamicObject tpv_groupdept_f = filterInfo.getDynamicObject("tpv_groupdept_f"); // 部门组
        Date tpv_startdate_f = filterInfo.getDate("tpv_startdate_f");// 开始日期
        Date tpv_enddate_f = filterInfo.getDate("tpv_enddate_f");// 结束日期
        Long orgid = YEM.isNotEmpty(tpv_org_f) ? tpv_org_f.getLong("id") : 0L;
        Long deptid = YEM.isNotEmpty(tpv_dept_f) ? tpv_dept_f.getLong("id") : 0L;
        Long groupdeptid = YEM.isNotEmpty(tpv_groupdept_f) ? tpv_groupdept_f.getLong("id") : 0L;
        DataSet query = query(orgid, deptid, groupdeptid, tpv_startdate_f, tpv_enddate_f);
        return query;
    }

    public static DataSet query(Long orgid, Long depid, Long groupdeptid, Date startdate, Date enddate) {
        QFilter qFilter = new QFilter("tpv_bigdept.id", QCP.large_than, 0l);
        if (YEM.isNotEmpty(orgid)) {
            qFilter.and(new QFilter("tpv_org.id", QCP.equals, orgid));
        }
        if (YEM.isNotEmpty(depid)) {
            qFilter.and(new QFilter("tpv_bigdept.id", QCP.equals, depid));
        }
        if (YEM.isNotEmpty(groupdeptid)) {
            qFilter.and(new QFilter("tpv_depertment.id", QCP.equals, groupdeptid));
        }
        if (null != startdate) {
//            Date dayStartTime = getDayStartTime(startdate);
            Date dayStartTime = DateUtil.getStartOrEndDate(startdate, false);
            qFilter.and(new QFilter("tpv_datetime", QCP.large_equals, dayStartTime));
        }
        if (null != enddate) {
//            Date dayEndTime = getDayEndTime(enddate);
            Date dayEndTime = DateUtil.getStartOrEndDate(enddate, true);
            qFilter.and(new QFilter("tpv_datetime", QCP.less_equals, dayEndTime));
        }
        if (null != startdate & null != enddate) {
            qFilter.and(new QFilter("tpv_datetime", QCP.not_equals, null));
        }
        String selectfields = "billno,tpv_org.id,tpv_org.name,tpv_bigdept.id,tpv_bigdept.name," +
                "tpv_depertment.id,tpv_depertment.name,tpv_type,tpv_addorsub,tpv_datetime,tpv_realpreocp,tpv_amount";
        //查询部门资金额度底表数据
        DataSet tpv_sc_capitallimit = ORM.create().queryDataSet(algoKey, "tpv_sc_capitallimit", selectfields, qFilter.toArray());
        tpv_sc_capitallimit = tpv_sc_capitallimit.select("billno", "tpv_org.name tpv_companyname", "tpv_org.id tpv_companyid", "tpv_bigdept.name tpv_deptname", "tpv_bigdept.id tpv_deptid", "tpv_datetime",
                "tpv_depertment.id tpv_groupdeptid", "tpv_depertment.name tpv_groupdeptname", "to_char(tpv_datetime,'yyyy-MM-dd') tpv_day",
                "(case when tpv_type == 'A' then tpv_amount else 0.0 end) tpv_putcash",//收现款
                "(case when tpv_type == 'B' then tpv_amount else 0.0 end) tpv_paycash", //现款
                "(case when tpv_type == 'C' and tpv_realpreocp=='A' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_notereceivableA", "(case when tpv_type == 'C' and tpv_realpreocp='A' and tpv_addorsub=='B' then tpv_amount else 0.0 end) tpv_notereceivableB",//收银承
                "(case when tpv_type == 'D' and tpv_realpreocp=='A' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_notepayableA", "(case when tpv_type == 'D' and tpv_realpreocp='A' and tpv_addorsub=='B' then tpv_amount else 0.0 end) tpv_notepayableB",//付银承
                "(case when tpv_type == 'G' and tpv_realpreocp=='A' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_scmpropay", "(case when tpv_type == 'G' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_scmprorec", //供应链票据
                "(case when tpv_type == 'L' and tpv_realpreocp=='A' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_scmfinadd", "(case when tpv_type == 'L' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_scmfinless"); //供应链金融
        DataSet dataSet = tpv_sc_capitallimit.select(new String[]{"tpv_notereceivableA-tpv_notereceivableB tpv_notereceivable", "tpv_notepayableA-tpv_notepayableB tpv_notepayable", "tpv_deptname", "tpv_deptid", "tpv_companyid", "tpv_companyname",
                        "tpv_groupdeptid", "tpv_groupdeptname", "tpv_day", "tpv_putcash", "tpv_paycash", "tpv_scmpropay", "tpv_scmprorec",
                        "tpv_scmfinless", "tpv_scmfinadd"}).groupBy(new String[]{"tpv_deptname", "tpv_deptid", "tpv_companyid", "tpv_companyname",
                        "tpv_groupdeptid", "tpv_groupdeptname", "tpv_day"})
                .sum("tpv_putcash").sum("tpv_paycash").sum("tpv_notereceivable").sum("tpv_notepayable")
                .sum("tpv_scmpropay").sum("tpv_scmprorec").sum("tpv_scmfinless").sum("tpv_scmfinadd").finish();
        dataSet = RptUtil.zeroNullNums(dataSet);
        dataSet = dataSet.addField("to_date(tpv_day,'yyyy-MM-dd') tpv_businessdate", "tpv_businessdate");
        dataSet = dataSet.addField("(tpv_putcash-tpv_paycash+tpv_notereceivable-tpv_notepayable)  tpv_netflowdaysum", "tpv_netflowdaysum");
        return dataSet.orderBy(new String[]{"tpv_businessdate"});
    }

    /**
     * 获取日期的结束时间
     */
    public static Date getDayEndTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 23, 59, 59);
            calendar.set(14, 999);
            return calendar.getTime();
        }
    }

    /**
     * 获取该日期的开始时间
     */
    public static Date getDayStartTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 0, 0, 0);
            calendar.set(14, 0);
            return calendar.getTime();
        }
    }
}
