package tpv.tpv_wm.tpv_sc.java.report;

import com.itopview.wm.utils.YEM;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import tpv.tpv_wm.tpv_dt.java.utils.PersonnelDepUtils;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.time.temporal.Temporal;
import java.util.*;

/**
 * @Description: 暂估进仓表 -报表查询
 * @Author LinRui
 * @Date 2024/3/22 10:37
 */
public class EstInboundRptQuery extends AbstractReportListDataPlugin {
    private static final String algoKey = EstInboundRptQuery.class.getName();

    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        String whereFilter = createWhereStr(param);
        DataSet instockDataSet = query(whereFilter);
        return instockDataSet;
    }

    public static Date getLastMonthDay(Date date) {
        Calendar ca = Calendar.getInstance();
        ca.setTime(date);
        ca.set(2, ca.get(2) + 1);
        ca.set(5, ca.getActualMaximum(5));
        ca.set(11, 23);
        ca.set(12, 59);
        ca.set(13, 59);
        ca.set(14, 99);
        return ca.getTime();
    }

    public static DataSet query(String whereFilter) {
        DataSet mainDataSet = mainDataSet(whereFilter);
        DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(mainDataSet);
        for (DynamicObject col : col_s) {
            Date tpvPiartime = col.getDate("tpv_piartime");
            BigDecimal tpvDiscamount = col.getBigDecimal("tpv_discamount");
            if (tpvPiartime == null) {
                Date tpvBiztime = col.getDate("tpv_biztime");
                tpvPiartime = getLastMonthDay(tpvBiztime);
                col.set("tpv_piartime", tpvPiartime);
            }
            Instant dateTant = new Date().toInstant();
            LocalDateTime dateTime = dateTant.atZone(ZoneId.systemDefault()).toLocalDateTime();
            Instant piarTimeTant = tpvPiartime.toInstant();
            LocalDateTime piarTime = piarTimeTant.atZone(ZoneId.systemDefault()).toLocalDateTime();
            long monthsBetween = ChronoUnit.MONTHS.between(piarTime, dateTime);
            BigDecimal decimal = BigDecimal.ONE;
            if (monthsBetween <= 2) {
                decimal = new BigDecimal(0.1).divide(new BigDecimal(100));
            } else if (monthsBetween == 3) {
                decimal = new BigDecimal(0.2).divide(new BigDecimal(100));
            } else {
                decimal = new BigDecimal(0.3).divide(new BigDecimal(100));
            }
            col.set("tpv_penaltyinterest", tpvDiscamount.multiply(new BigDecimal(0.13).multiply(decimal)));
        }
        RowMeta rowMeta = mainDataSet.getRowMeta();
        int fieldCount = rowMeta.getFieldCount();
        Collection<Object[]> collection = new ArrayList<>();
        for (int i = 0; i < col_s.size(); i++) {
            DynamicObject col = col_s.get(i);
            Object[] obj = new Object[fieldCount];
            for (int j = 0; j < fieldCount; j++) {
                obj[j] = col.get(j);
            }
            collection.add(obj);
        }
        CollectionInput inputs = new CollectionInput(rowMeta, collection);
        DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
        return createDataSet;
    }

    private static DataSet mainDataSet(String whereFilter) {
//        QFilter qFilters_1 = new QFilter("billstatus", QCP.equals, "C");
        String selectfields = "id,billno tpv_billno,billstatus tpv_billstatus,bizorg tpv_company,bizdept tpv_department,biztime tpv_biztime,tpv_tradeproperty,supplier tpv_supplier,tpv_warehouseentry tpv_entrepot," +
                "billentry.id entryid,tpv_purcontnum,tpv_contract,billentry.tpv_prodname tpv_prodname,billentry.material material,billentry.tpv_specation tpv_specation,billentry.tpv_shader tpv_shader,billentry.tpv_origin.name tpv_origin," +
                "billentry.qty tpv_inqty,settlecurrency tpv_incurrency,tpv_tradeproperty tpv_billtype";
        DataSet supImPurinbill = ORM.create().queryDataSet(algoKey, "im_purinbill", selectfields, null)
                .groupBy(new String[]{"tpv_inqty", "tpv_incurrency", "entryid", "tpv_prodname", "tpv_contract", "material", "tpv_billno", "tpv_billstatus", "tpv_company", "tpv_department", "tpv_biztime", "tpv_tradeproperty", "tpv_supplier", "tpv_entrepot", "tpv_billtype"})
                .groupConcat("concat(tpv_shader,tpv_origin,tpv_specation)", "tpv_materialdes").finish();
        DataSet copy = supImPurinbill.copy();
        DataSet bdSuppliergroupdetail = ORM.create().queryDataSet(algoKey, "bd_suppliergroupdetail", "group tpv_supplysupplier,supplier", null);
        DataSet materImPurinbill = supImPurinbill.leftJoin(bdSuppliergroupdetail).on("tpv_supplier", "supplier")
                .select("tpv_billtype", "tpv_inqty", "tpv_incurrency", "entryid", "tpv_materialdes", "tpv_prodname", "tpv_contract", "tpv_supplysupplier", "material", "tpv_billno", "tpv_billstatus", "tpv_company", "tpv_department", "tpv_biztime", "tpv_tradeproperty", "tpv_supplier", "tpv_entrepot").finish();
        DataSet materialgroupdetail = ORM.create().queryDataSet(algoKey, "bd_materialgroupdetail", "material,group tpv_linetype",
                null);//物料分类标准页签
        DataSet imPurinbill = materImPurinbill.leftJoin(materialgroupdetail).on("material", "material").select("tpv_billtype", "tpv_inqty", "tpv_incurrency", "entryid", "tpv_materialdes", "tpv_prodname", "tpv_contract", "tpv_supplysupplier", "tpv_billno", "tpv_billstatus", "tpv_company", "tpv_department", "tpv_biztime", "tpv_tradeproperty", "tpv_supplier", "tpv_entrepot", "tpv_linetype").finish();
        List<Long> purinEntrySrcIds = new ArrayList<>();
        List<String> purinEntryIds = new ArrayList<>();
        List<Long> purinEntryLongs = new ArrayList<>();
        for (Row row : copy) {
            if (YEM.isNotEmpty(row.getLong("tpv_contract"))) {
                if (!purinEntrySrcIds.contains(row.getLong("tpv_contract"))) {
                    purinEntrySrcIds.add(row.getLong("tpv_contract"));
                }
            }
            purinEntryIds.add(String.valueOf(row.getLong("entryid")));
            purinEntryLongs.add(row.getLong("entryid"));
        }

        if (purinEntrySrcIds.size() > 0) {
            QFilter qFilters_2 = new QFilter("billstatus", QCP.equals, "C");
            qFilters_2.and(new QFilter("id", QCP.in, purinEntrySrcIds));
            DataSet tpvDtPurchase = ORM.create().queryDataSet(algoKey, "tpv_dt_purchase",
                    "id tpv_contractnumber,tpv_contractnumber tpv_contractnu,tpv_contractstatusx tpv_combo,tpv_operator tpv_conoperator,creator tpv_concreator,tpv_piartime",
                    qFilters_2.toArray());
            imPurinbill = imPurinbill.leftJoin(tpvDtPurchase).on("tpv_contract", "tpv_contractnumber").select(
                    new String[]{"tpv_billtype", "tpv_inqty", "tpv_incurrency", "entryid", "tpv_materialdes", "tpv_prodname", "tpv_contractnumber","tpv_contractnu", "tpv_combo", "tpv_conoperator", "tpv_concreator"
                            , "tpv_billstatus", "tpv_company", "tpv_department", "tpv_biztime", "tpv_billno", "tpv_supplier", "tpv_entrepot", "tpv_supplysupplier", "tpv_linetype",
                            "(case when tpv_piartime is not null then tpv_piartime end) tpv_piartime", "tpv_inqty tpv_penaltyinterest"
                    }).finish();
        } else {
            imPurinbill = imPurinbill.select();
        }
        if (purinEntryIds.size() > 0) {
            QFilter qFilters_4 = new QFilter("tpv_proof", QCP.equals, true);
            qFilters_4.and("tpv_storageid", QCP.in, purinEntryIds);//"Cast(expr as DataType) as 新子段","新子段"
            DataSet cmCostdisposeProof = ORM.create().queryDataSet(algoKey, "tpv_cm_costdispose", "tpv_proof,id,tpv_storageid,tpv_origampnotax,tpv_soudocumtype",
                    qFilters_4.toArray());//成本处理单 是否凭证等于是
            cmCostdisposeProof = cmCostdisposeProof.addField("Cast(tpv_storageid as Long) tpv_storage", "tpv_storage").groupBy(new String[]{"tpv_storage", "id", "tpv_soudocumtype", "tpv_proof"}).sum("tpv_origampnotax").finish();//成本处理生成凭证=是

            imPurinbill = imPurinbill.leftJoin(cmCostdisposeProof).on("entryid", "tpv_storage").select("tpv_inqty", "tpv_incurrency", "entryid", "tpv_materialdes", "tpv_prodname", "tpv_contractnumber","tpv_contractnu", "tpv_combo", "tpv_conoperator", "tpv_concreator"
                    , "tpv_billstatus", "tpv_billtype", "tpv_company", "tpv_department", "tpv_biztime", "tpv_billno", "tpv_supplier", "tpv_entrepot", "tpv_supplysupplier", "tpv_linetype",
                    "(case when id=0 then 'A' else 'B' end) tpv_combobiz", "tpv_piartime", "tpv_penaltyinterest"
            ).finish();

            DataSet cmCostdispose = ORM.create().queryDataSet(algoKey, "tpv_cm_costdispose", "tpv_proof,id,tpv_storageid,tpv_origampnotax,tpv_soudocumtype",
                    new QFilter[]{new QFilter("tpv_storageid", QCP.in, purinEntryIds)});//成本处理单

            cmCostdispose = cmCostdispose.addField("Cast(tpv_storageid as Long) tpv_storage", "tpv_storage").
                    groupBy(new String[]{"tpv_storage", "id", "tpv_soudocumtype"}).sum("tpv_origampnotax").finish();

            DataSet cmCostdisMents = cmCostdispose.copy().where("tpv_soudocumtype='tpv_cm_costadjustments'");//来源等于成本调整单

            imPurinbill = imPurinbill.leftJoin(cmCostdispose).on("entryid", "tpv_storage").select("tpv_inqty", "tpv_incurrency", "entryid", "tpv_materialdes", "tpv_prodname", "tpv_contractnumber","tpv_contractnu", "tpv_combo", "tpv_conoperator", "tpv_concreator"
                    , "tpv_billstatus", "tpv_company", "tpv_department", "tpv_biztime", "tpv_billno", "tpv_supplier", "tpv_entrepot", "tpv_supplysupplier", "tpv_linetype",
                    "tpv_combobiz", "tpv_billtype", "tpv_origampnotax tpv_inamount", "tpv_piartime", "tpv_penaltyinterest").finish();

            imPurinbill = imPurinbill.leftJoin(cmCostdisMents).on("entryid", "tpv_storage").select("tpv_inqty", "tpv_incurrency", "entryid", "tpv_materialdes", "tpv_prodname", "tpv_contractnumber","tpv_contractnu", "tpv_combo", "tpv_conoperator", "tpv_concreator"
                    , "tpv_billstatus", "tpv_company", "tpv_department", "tpv_biztime", "tpv_billno", "tpv_supplier", "tpv_entrepot", "tpv_supplysupplier", "tpv_linetype",
                    "tpv_combobiz", "tpv_billtype", "tpv_inamount+tpv_origampnotax tpv_inamount", "tpv_piartime", "tpv_penaltyinterest").finish();

            QFilter qFilters_5 = new QFilter("biztype", QCP.equals, 1101);
            qFilters_5.and("billentry.tpv_purinfentryids", QCP.in, purinEntryLongs);

            DataSet tpvDtPurinbill = ORM.create().queryDataSet(algoKey, "im_purinbill", "billentry.id billid,billentry.qty tpv_outqty,settlecurrency tpv_outcurrency,billentry.tpv_purinfentryids tpv_purinfentryids,billentry.tpv_fentryid_in tpv_fentryid",
                    qFilters_5.toArray());//采购退货单

            DataSet cmCostdisposeFent = ORM.create().queryDataSet(algoKey, "tpv_cm_costdispose", "tpv_proof,id,tpv_storageid,tpv_origampnotax tpv_outamount,tpv_soudocumtype,tpv_soudocumid",
                    null).groupBy(new String[]{"tpv_soudocumid"}).sum("tpv_outamount").finish();//成本处理单
            cmCostdisposeFent = cmCostdisposeFent.addField("Cast(tpv_soudocumid as Long) tpv_soudocum", "tpv_soudocum");
            DataSet tpvDtPurinbillFent = tpvDtPurinbill.copy();
            tpvDtPurinbillFent = tpvDtPurinbillFent.leftJoin(cmCostdisposeFent).on("billid", "tpv_soudocum").select("tpv_outamount", "tpv_fentryid").finish();
            imPurinbill = imPurinbill.leftJoin(tpvDtPurinbillFent).on("entryid", "tpv_fentryid").select("tpv_inqty", "tpv_incurrency", "entryid", "tpv_materialdes", "tpv_prodname", "tpv_contractnumber","tpv_contractnu", "tpv_combo", "tpv_conoperator", "tpv_concreator"
                    , "tpv_billstatus", "tpv_company", "tpv_department", "tpv_biztime", "tpv_billno", "tpv_supplier", "tpv_entrepot", "tpv_supplysupplier", "tpv_linetype",
                    "tpv_combobiz", "tpv_billtype", "tpv_inamount", "tpv_piartime", "tpv_penaltyinterest", "tpv_fentryid", "tpv_outamount").finish();
            imPurinbill = imPurinbill.leftJoin(tpvDtPurinbill).on("entryid", "tpv_purinfentryids").select("tpv_inqty", "tpv_incurrency", "entryid", "tpv_materialdes", "tpv_prodname", "tpv_contractnumber","tpv_contractnu", "tpv_combo", "tpv_conoperator", "tpv_concreator"
                    , "tpv_billstatus", "tpv_company", "tpv_department", "tpv_biztime", "tpv_billno", "tpv_supplier", "tpv_entrepot", "tpv_supplysupplier", "tpv_linetype",
                    "tpv_combobiz", "tpv_billtype", "tpv_inamount", "tpv_outqty", "tpv_outcurrency", "tpv_piartime", "tpv_penaltyinterest", "tpv_fentryid", "tpv_outamount").finish();

            QFilter qFilters_3 = new QFilter("billstatus", QCP.equals, "C");
            qFilters_3.and(new QFilter("tpv_wareiformation.tpv_purinentryid", QCP.in, purinEntryIds));
            DataSet tpvDtPurinvomater = ORM.create().queryDataSet(algoKey, "tpv_dt_purinvomater", "tpv_wareiformation.tpv_qty tpv_authqty,tpv_wareiformation.tpv_currency tpv_authcurrency,tpv_wareiformation.tpv_rkcurmatchamonotax tpv_authamount,tpv_wareiformation.tpv_purinentryid tpv_purinentryid",
                    qFilters_3.toArray());
            tpvDtPurinvomater = tpvDtPurinvomater.addField("Cast(tpv_purinentryid as Long) tpv_purinentry", "tpv_purinentry").
                    groupBy(new String[]{"tpv_authcurrency", "tpv_purinentry"}).sum("tpv_authqty").sum("tpv_authamount").finish();
            imPurinbill = imPurinbill.leftJoin(tpvDtPurinvomater).on("entryid", "tpv_purinentry").select("tpv_inqty", "tpv_incurrency", "entryid", "tpv_materialdes", "tpv_prodname", "tpv_contractnumber","tpv_contractnu", "tpv_combo", "tpv_conoperator", "tpv_concreator"
                    , "tpv_billstatus", "tpv_company", "tpv_department", "tpv_biztime", "tpv_billno", "tpv_supplier", "tpv_entrepot", "tpv_supplysupplier", "tpv_linetype",
                    "tpv_combobiz", "tpv_billtype", "tpv_inamount", "tpv_outqty", "tpv_outcurrency", "tpv_authqty", "tpv_authamount", "tpv_authcurrency", "tpv_outamount", "tpv_inqty+tpv_outqty-tpv_authqty tpv_discqty", "tpv_incurrency tpv_disccurrency", "tpv_inamount+tpv_outamount-tpv_authamount tpv_discamount", "tpv_piartime", "tpv_penaltyinterest").finish();
        } else {
            imPurinbill = imPurinbill.select();
        }

        DataSet dataSet = imPurinbill;
        if (YEM.isNotEmpty(whereFilter)) {
            dataSet = dataSet.where(whereFilter);
        }
        return dataSet;
    }

    private String createWhereStr(ReportQueryParam param) {
        FilterInfo filterInfo = param.getFilter();
        String whereFilter = "1 = 1";
        String tpvPurnum = filterInfo.getString("tpv_purnum");//采购合同号
        String tpvRknum = filterInfo.getString("tpv_rknum");//入库单号
        DynamicObject tpvOrg = filterInfo.getDynamicObject("tpv_org");//公司
        String tpvBillate = filterInfo.getString("tpv_billate");//状态
        Date tpvBegin = filterInfo.getDate("tpv_begin");//开始
        Date tpvEnd = filterInfo.getDate("tpv_end");
        DynamicObject tpvWorse = filterInfo.getDynamicObject("tpv_worse");//仓库
        DynamicObject tpvMaterial = filterInfo.getDynamicObject("tpv_material");//商品
        if (YEM.isNotEmpty(tpvPurnum)) {
            whereFilter = String.format("%s and tpv_contractnu like '%%%s%%'", whereFilter, tpvPurnum);
        }
        if (YEM.isNotEmpty(tpvRknum)) {
            whereFilter = String.format("%s and tpv_billno like '%%%s%%'", whereFilter, tpvRknum);
        }
        if (tpvOrg != null) {
            whereFilter = String.format("%s and tpv_company = %s", whereFilter, tpvOrg.getPkValue());
        }
        if (YEM.isNotEmpty(tpvBillate)) {
            whereFilter = String.format("%s and tpv_billstatus ='%s'", whereFilter, String.join(",", tpvBillate));
        }
        if (tpvBegin != null) {
            SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String format = str.format(tpvBegin);
            whereFilter = String.format("%s and tpv_biztime >= to_date('" + format + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        if (tpvEnd != null) {
            SimpleDateFormat str = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String format = str.format(tpvEnd);
            whereFilter = String.format("%s and tpv_biztime <= to_date('" + format + "','yyyy-MM-dd HH:mm:ss') ", whereFilter);
        }
        if (tpvWorse != null) {
            whereFilter = String.format("%s and tpv_entrepot = %s", whereFilter, tpvWorse.getPkValue());
        }
        if (tpvMaterial != null) {
            whereFilter = String.format("%s and tpv_prodname = %s", whereFilter, tpvMaterial.getPkValue());
        }
        String whereStr = PersonnelDepUtils.createWhereStr(filterInfo,"tpv_department", "tpv_muldepartment");
        if (!"".equals(whereStr)){
            whereFilter=String.format("%s%s", whereFilter, whereStr);
        }
        return whereFilter;
    }

}
