package tpv.tpv_wm.tpv_sc.java.report;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.DateUtil;
import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.JoinDataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.*;

/**
 * @Description: 资金占用实时查询表 -报表查询
 * @Author LinRui
 * @Date 2024/2/21 16:11
 */
public class FundsUsedTimeQueryRpt extends AbstractReportListDataPlugin {
    private static final String algoKey = FundsUsedTimeQueryRpt.class.getName();

    @Override
    public DataSet query(ReportQueryParam param, Object o) throws Throwable {
        FilterInfo filterInfo = param.getFilter();
        DynamicObject tpv_org_f = filterInfo.getDynamicObject("tpv_org_f");// 公司
        DynamicObject tpv_dept_f = filterInfo.getDynamicObject("tpv_dept_f"); // 部门
        Date tpv_date_f = filterInfo.getDate("tpv_date_f");// 日期
        Long orgid = YEM.isNotEmpty(tpv_org_f) ? tpv_org_f.getLong("id") : 0L;
        Long deptid = YEM.isNotEmpty(tpv_dept_f) ? tpv_dept_f.getLong("id") : 0L;
        DataSet query = query(orgid,deptid,tpv_date_f);
        return query;
    }

    public static DataSet query(Long orgid, Long depid, Date date) {
        QFilter qFiltero = new QFilter("orgduty.number", QCP.equals, "ZHKC002"); //部门属性为业务部门
        qFiltero.and(new QFilter("enable", QCP.equals, "1"));
        qFiltero.and(new QFilter("org.orgpattern.number", QCP.equals, "Orgform06")); //形态：部门
        if (YEM.isNotEmpty(depid)) {
            qFiltero.and(new QFilter("org.id", QCP.equals, depid));
        }
        //查询部门属性管理
        DataSet bos_org_dutyrelation = ORM.create().queryDataSet(algoKey,"bos_org_dutyrelation","org.number deptnumber",qFiltero.toArray());

        QFilter qFilterd = new QFilter("orgpattern.number", QCP.equals, "Orgform06"); //形态：部门
        qFilterd.and(new QFilter("enable", QCP.equals, "1"));
        if (YEM.isNotEmpty(depid)) {
            qFilterd.and(new QFilter("id", QCP.equals, depid));
        }
        DataSet dataSetbos = ORM.create().queryDataSet(algoKey,"bos_adminorg","id did,number dnumber,name dname,structure.viewparent.id porgid",qFilterd.toArray());

        dataSetbos = bos_org_dutyrelation.leftJoin(dataSetbos).on("deptnumber","dnumber")
                .select(RptUtil.getDataSetFiledAlias(dataSetbos)).finish();
//        DataSet dataSetbos = bos_org.leftJoin(bos_adminorg).on("oid", "porgid").select("oid","did","onumber","dnumber","oname","dname").finish();
        if (null != date) {
            String dateHms = DateUtil.formatDateHms(date);
            dataSetbos = dataSetbos.addField("to_date('" + dateHms + "','yyyy-MM-dd HH:mm:ss') tpv_businessdate","tpv_businessdate");
            dataSetbos = dataSetbos.addField("'" + dateHms.substring(0,7) + "' tpv_day","tpv_day");
        } else {
            dataSetbos = dataSetbos.addField("null tpv_businessdate","tpv_businessdate"); //日期类型
            dataSetbos = dataSetbos.addField("'' businessdate","businessdate"); //字符串 ，只为在未输入查询日期是，清空右表数据
        }

//        QFilter qFilter = new QFilter("tpv_org.id", QCP.large_than, 0l);
//        qFilter.and(new QFilter("tpv_bigdept.id", QCP.large_than, 0l));
        QFilter qFilter = new QFilter("tpv_bigdept.id", QCP.large_than, 0l);
        if (null != date) {
            Date monthFirstDay = DateUtil.getMonthFirstDay(date);
            Date dayEndTime = getDayEndTime(date);
            qFilter.and(new QFilter("tpv_datetime", QCP.large_than, monthFirstDay));
            qFilter.and(new QFilter("tpv_datetime", QCP.less_equals, dayEndTime));
        } else {
            qFilter.and(new QFilter("tpv_datetime", QCP.not_equals, null));
        }
        String selectfields = "billno,tpv_org.id,tpv_org.name,tpv_bigdept.id,tpv_bigdept.name,tpv_type,tpv_addorsub,tpv_datetime,tpv_realpreocp,tpv_amount";
        //查询部门资金额度底表数据
        DataSet tpv_sc_capitallimit = ORM.create().queryDataSet(algoKey,"tpv_sc_capitallimit",selectfields,qFilter.toArray());
        tpv_sc_capitallimit = tpv_sc_capitallimit.select("billno","tpv_bigdept.name tpv_deptname","tpv_bigdept.id tpv_deptid","tpv_datetime","to_char(tpv_datetime,'yyyy-MM') tpv_days",
                //现款
                "(case when tpv_type == 'A' then tpv_amount else 0.0 end) tpv_putcash","(case when tpv_type == 'B' then tpv_amount else 0.0 end) tpv_paycash",
                //银承
                "(case when tpv_type == 'C' and tpv_addorsub == 'A' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_notereceivableadd",
                "(case when tpv_type == 'C' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_notereceivableless",
                "(case when tpv_type == 'D' and tpv_addorsub == 'A' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_notepayableadd",
                "(case when tpv_type == 'D' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_notepayableless",
                //供应链票据
                "(case when tpv_type == 'G' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_scmprorec","(case when tpv_type == 'G' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_scmpropay",
                //供应链金融
                "(case when tpv_type == 'L' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_scmfinadd","(case when tpv_type == 'L' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_scmfinless",
                //国际证余额
                "(case when tpv_type == 'I' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_interamtadd",
                "(case when tpv_type == 'I' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_interamtless",
                //国内证余额
                "(case when tpv_type == 'J' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_chinaamtadd",
                "(case when tpv_type == 'J' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_chinaamtless",
                //应收票据(商票)
                "(case when tpv_type == 'E' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_bareceiveadd",
                "(case when tpv_type == 'E' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_bareceiveless",
                //应付票据(商票)
                "(case when tpv_type == 'F' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_bapayadd",
                "(case when tpv_type == 'F' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_bapayless",
                //贸易融资
                "(case when tpv_type == 'H' and tpv_addorsub == 'A' then tpv_amount else 0.0 end) tpv_trloanadd",
                "(case when tpv_type == 'H' and tpv_addorsub == 'B' and tpv_realpreocp == 'A' then tpv_amount else 0.0 end) tpv_trloanless");
        tpv_sc_capitallimit = tpv_sc_capitallimit.groupBy(new String[]{"tpv_deptname","tpv_deptid","tpv_days"})
                .sum("tpv_putcash").sum("tpv_paycash").sum("tpv_notereceivableadd").sum("tpv_notereceivableless")
                .sum("tpv_notepayableadd").sum("tpv_notepayableless").sum("tpv_scmpropay").sum("tpv_scmprorec")
                .sum("tpv_scmfinless").sum("tpv_scmfinadd").sum("tpv_chinaamtless").sum("tpv_chinaamtadd")
                .sum("tpv_interamtless").sum("tpv_interamtadd").sum("tpv_bareceiveadd").sum("tpv_bareceiveless")
                .sum("tpv_bapayadd").sum("tpv_bapayless").sum("tpv_trloanadd").sum("tpv_trloanless").finish();
        tpv_sc_capitallimit = tpv_sc_capitallimit.select("tpv_deptname","tpv_deptid","tpv_days",
                "tpv_putcash","tpv_paycash","tpv_notereceivableadd - tpv_notereceivableless tpv_notereceivable",
                "tpv_notepayableadd - tpv_notepayableless tpv_notepayable","tpv_scmpropay","tpv_scmprorec","tpv_scmfinadd","tpv_scmfinless",
                "tpv_bareceiveadd","tpv_bareceiveless","tpv_bapayadd","tpv_bapayless","tpv_chinaamtadd - tpv_chinaamtless tpv_chinaamt",
                "tpv_interamtadd - tpv_interamtless tpv_interamt","tpv_trloanadd - tpv_trloanless tpv_trloanamt");

        JoinDataSet joinDataSet = dataSetbos.leftJoin(tpv_sc_capitallimit);
        if (null != date) {
            joinDataSet = joinDataSet.on("did", "tpv_deptid");
        } else {
            joinDataSet = joinDataSet.on("did", "tpv_deptid").on("businessdate","tpv_days");
        }
        DataSet dataSet = joinDataSet.select("dname tpv_deptname","did tpv_deptid","tpv_day","tpv_businessdate",
                        "tpv_putcash","tpv_paycash", "tpv_notereceivable","tpv_notepayable", "tpv_scmpropay","tpv_scmprorec",
                        "tpv_scmfinless","tpv_scmfinadd", "tpv_bareceiveadd","tpv_bareceiveless","tpv_bapayadd","tpv_bapayless", "tpv_chinaamt",
                        "tpv_interamt", "tpv_trloanamt", "tpv_chinaamt + tpv_interamt + tpv_trloanamt tpv_tradesum") //贸易金融合计
                .finish();
        QFilter qFilter1 = new QFilter("tpv_bigdept.id", QCP.large_than, 0l);
        if (null != date) {
            Date dayEndTime = getDayEndTime(date);
            qFilter1.and(new QFilter("tpv_datetime", QCP.less_equals, dayEndTime));
        } else {
            qFilter1.and(new QFilter("tpv_datetime", QCP.not_equals, null));
        }
        String selectfields1 = "billno,tpv_org.id,tpv_org.name,tpv_bigdept.id,tpv_bigdept.name,tpv_type,tpv_addorsub,tpv_datetime,tpv_realpreocp,tpv_amount";
        //查询部门资金额度底表数据(预占用数据)
        DataSet capitallimitAdvance = ORM.create().queryDataSet(algoKey,"tpv_sc_capitallimit",selectfields1,qFilter1.toArray());
        capitallimitAdvance = capitallimitAdvance.select("tpv_bigdept.name tpv_deptname", "tpv_bigdept.id tpv_deptid",
                //银承应收(预占用)
                "(case when tpv_type == 'C' and tpv_addorsub == 'A' and tpv_realpreocp != 'A' then tpv_amount else 0.0 end) tpv_bankreceiveadd",
                "(case when tpv_type == 'C' and tpv_addorsub == 'B' and tpv_realpreocp != 'A' then tpv_amount else 0.0 end) tpv_bankreceiveless",
                //银承应付(预占用)
                "(case when tpv_type == 'D' and tpv_addorsub == 'A' and tpv_realpreocp != 'A' then tpv_amount else 0.0 end) tpv_bankpayadd",
                "(case when tpv_type == 'D' and tpv_addorsub == 'B' and tpv_realpreocp != 'A' then tpv_amount else 0.0 end) tpv_bankpayless",
                //贸易融资(预占用)
                "(case when tpv_type == 'H' and tpv_addorsub == 'A' and tpv_realpreocp != 'A' then tpv_amount else 0.0 end) tpv_trloanadd",
                "(case when tpv_type == 'H' and tpv_addorsub == 'B' and tpv_realpreocp != 'A' then tpv_amount else 0.0 end) tpv_trloanlessup",
                //国内证余额(预占用)
                "(case when tpv_type == 'J' and tpv_addorsub == 'A' and tpv_realpreocp != 'A' then tpv_amount else 0.0 end) tpv_chinaamtadd",
                "(case when tpv_type == 'J' and tpv_addorsub == 'B' and tpv_realpreocp != 'A' then tpv_amount else 0.0 end) tpv_chinaamtlessup",
                //国际证余额(预占用)
                "(case when tpv_type == 'I' and tpv_addorsub == 'A' and tpv_realpreocp != 'A' then tpv_amount else 0.0 end) tpv_interamtadd",
                "(case when tpv_type == 'I' and tpv_addorsub == 'B' and tpv_realpreocp != 'A' then tpv_amount else 0.0 end) tpv_interamtlessup");
        capitallimitAdvance = capitallimitAdvance.groupBy(new String[]{"tpv_deptname","tpv_deptid"})
                .sum("tpv_chinaamtadd").sum("tpv_interamtadd").sum("tpv_trloanadd").sum("tpv_bankreceiveadd")
                .sum("tpv_bankreceiveless").sum("tpv_bankpayadd").sum("tpv_bankpayless").sum("tpv_trloanlessup")
                .sum("tpv_interamtlessup").sum("tpv_chinaamtlessup").finish();
        capitallimitAdvance = capitallimitAdvance.select("tpv_deptname","tpv_deptid",
                "tpv_bankreceiveless - tpv_bankreceiveadd + tpv_bankpayadd - tpv_bankpayless tpv_bankadvance",
                "tpv_chinaamtadd - tpv_chinaamtlessup tpv_chinaamtadvance","tpv_interamtadd - tpv_interamtlessup tpv_interamtadvance",
                "tpv_trloanadd - tpv_trloanlessup tpv_trloanadvance");

        JoinDataSet joinDataSet1 = dataSet.leftJoin(capitallimitAdvance);
        joinDataSet1 = joinDataSet1.on("tpv_deptid", "tpv_deptid");
        dataSet = joinDataSet1.select("tpv_deptname","tpv_deptid","tpv_day","tpv_businessdate",
                        "tpv_putcash","tpv_paycash", "tpv_notereceivable","tpv_notepayable", "tpv_scmpropay","tpv_scmprorec",
                        "tpv_scmfinless","tpv_scmfinadd", "tpv_bareceiveadd","tpv_bareceiveless","tpv_bapayadd","tpv_bapayless",
                        "tpv_chinaamt", "tpv_interamt", "tpv_trloanamt","tpv_tradesum",
                        "tpv_bankadvance","tpv_chinaamtadvance","tpv_interamtadvance","tpv_trloanadvance") //预占用合计
                .finish();
        dataSet = RptUtil.zeroNullNums(dataSet);
        dataSet = dataSet.addField("tpv_bankadvance + tpv_chinaamtadvance + tpv_interamtadvance + tpv_trloanadvance tpv_advancesum","tpv_advancesum");//预占用合计

        QFilter qFilter2 = new QFilter("billstatus", QCP.equals, "C");
        if (YEM.isNotEmpty(depid)) {
            qFilter2.and(new QFilter("tpv_depertment.id", QCP.equals, depid));
        }
//        if (YEM.isNotEmpty(orgid)) {
//            qFilter2.and(new QFilter("tpv_companys.id", QCP.equals, orgid));
//        }
        if (null != date) {
            Date lastMonthEndDay = DateUtil.getLastMonthEndDay(date);
            Date dayStartTime = getDayStartTime(lastMonthEndDay);
            qFilter2.and(new QFilter("tpv_sc_fundsused_aa.tpv_businessdate", QCP.large_equals, dayStartTime));
            qFilter2.and(new QFilter("tpv_sc_fundsused_aa.tpv_businessdate", QCP.less_equals, lastMonthEndDay));
        }
        String selectfields2 = "id,billno,tpv_sc_fundsused_aa,tpv_sc_fundsused_aa.tpv_businessdate tpv_businessdate," +
                "tpv_sc_fundsused_aa.tpv_companys tpv_companys,tpv_sc_fundsused_aa.tpv_depertment tpv_depertment," +
                "tpv_sc_fundsused_aa.tpv_occupied tpv_occupied,tpv_sc_fundsused_aa.tpv_applyamt tpv_applyamt," +
                "tpv_sc_fundsused_aa.tpv_accamt tpv_accamt,tpv_sc_fundsused_aa.tpv_needamt tpv_needamt," +
                "tpv_sc_fundsused_aa.tpv_useamt tpv_useamt";
        //查询资金占用明细(财务)数据
        DataSet tpv_sc_fundsused = ORM.create().queryDataSet(algoKey,"tpv_sc_fundsused",selectfields2,qFilter2.toArray());
        tpv_sc_fundsused = tpv_sc_fundsused.groupBy(new String[]{"tpv_depertment","tpv_businessdate"})
                .sum("tpv_occupied").sum("tpv_applyamt").sum("tpv_accamt").sum("tpv_needamt").sum("tpv_useamt")
                .finish();
        tpv_sc_fundsused = tpv_sc_fundsused.addField("to_char(DATEADD(DAY,1,tpv_businessdate),'yyyy-MM') days","days");
        dataSet = dataSet.leftJoin(tpv_sc_fundsused).on("tpv_deptid","tpv_depertment").on("tpv_day","days")
                .select("tpv_deptname","tpv_deptid","tpv_businessdate","tpv_putcash","tpv_paycash","tpv_notereceivable","tpv_notepayable",
                        "tpv_scmpropay","tpv_scmprorec","tpv_scmfinless","tpv_scmfinadd","tpv_bareceiveadd","tpv_bareceiveless","tpv_bapayadd","tpv_bapayless",
                        "tpv_chinaamt","tpv_interamt","tpv_trloanamt","tpv_bankadvance","tpv_chinaamtadvance","tpv_interamtadvance","tpv_trloanadvance",
                        "tpv_tradesum","tpv_advancesum","tpv_occupied tpv_cashinitoccup","tpv_applyamt tpv_bankinitoccup","tpv_accamt tpv_scmproinitoccup",
                        "tpv_needamt tpv_bapayinitoccup","tpv_useamt tpv_bareceiveinitoccup","tpv_day")
                .finish();
        QFilter qFilter3 = new QFilter("billstatus", QCP.equals, "C");
        if (YEM.isNotEmpty(depid)) {
            qFilter3.and(new QFilter("tpv_dept.id", QCP.equals, depid));
        }
//        if (YEM.isNotEmpty(orgid)) {
//            qFilter3.and(new QFilter("tpv_companys.id", QCP.equals, orgid));
//        }
        if (null != date) {
            Date lastMonthEndDay = DateUtil.getLastMonthEndDay(date);
            Date dayStartTime = getDayStartTime(lastMonthEndDay);
            qFilter3.and(new QFilter("tpv_entryentity.tpv_businessdate", QCP.large_equals, dayStartTime));
            qFilter3.and(new QFilter("tpv_entryentity.tpv_businessdate", QCP.less_equals, lastMonthEndDay));
        }
        String selectfields3 = "id,billno,tpv_entryentity,tpv_entryentity.tpv_businessdates tpv_businessdate," +
                "tpv_entryentity.tpv_companys tpv_companys,tpv_entryentity.tpv_dept tpv_depertment," +
                "tpv_entryentity.tpv_supamount tpv_supamount";
        //查询资金占用明细（资金）供应链金融及国内证数据
        DataSet tpv_sc_supfunddetails = ORM.create().queryDataSet(algoKey,"tpv_sc_supfunddetails",selectfields3,qFilter3.toArray());
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.groupBy(new String[]{"tpv_depertment","tpv_businessdate"})
                .sum("tpv_supamount").finish();
        tpv_sc_supfunddetails = tpv_sc_supfunddetails.addField("to_char(DATEADD(DAY,1,tpv_businessdate),'yyyy-MM') days","days");
        List<String> aliasList1 = RptUtil.getDataSetAliasList(dataSet);
        aliasList1.add("tpv_supamount tpv_scmfininitoccup");
        dataSet = dataSet.leftJoin(tpv_sc_supfunddetails).on("tpv_deptid","tpv_depertment").on("tpv_day","days")
                .select(aliasList1.toArray(new String[0])).finish();

        QFilter qFilter4 = new QFilter("enable", QCP.equals, "1");
        if (YEM.isNotEmpty(depid)) {
            qFilter4.and(new QFilter("tpv_department.id", QCP.equals, depid));
        }
        if (null != date) {
            date = getDayEndTime(date);
            qFilter4.and(new QFilter("tpv_effectivedate", QCP.less_equals, date));
        }
        String selectfields4 = "tpv_effectivedate,tpv_department,tpv_caplimat,tpv_deptproport / 100 tpv_deptproport";
        //查询部门比例
        DataSet tpv_sc_deptproportion = ORM.create().queryDataSet(algoKey,"tpv_sc_deptproportion",selectfields4,qFilter4.toArray(),"tpv_effectivedate desc");
        if (null != date) {
            tpv_sc_deptproportion = tpv_sc_deptproportion.top(1);
//                    .groupBy(new String[]{"tpv_department"})
//                    .sum("tpv_deptproport").finish();
        } else {
            tpv_sc_deptproportion = tpv_sc_deptproportion.top(1).select("tpv_department","0.0 tpv_deptproport");
        }
        JoinDataSet joinDataSet2 = dataSet.leftJoin(tpv_sc_deptproportion);
        joinDataSet2 = joinDataSet2.on("tpv_deptid", "tpv_department");
        List<String> aliasList2 = RptUtil.getDataSetAliasList(dataSet);
        aliasList2.add("(case when tpv_deptproport != null then tpv_deptproport else 0.0 end) tpv_deptproport");
        dataSet = joinDataSet2.select(aliasList2.toArray(new String[0])).finish();

        QFilter qFilter5 = new QFilter("tpv_forbidden", QCP.equals, true);
        qFilter5.and(new QFilter("tpv_usetype", QCP.equals, "A"));
        if (YEM.isNotEmpty(depid)) {
            qFilter5.and(new QFilter("tpv_department.id", QCP.equals, depid));
        }
        if (null != date) {
            Date dayEndTime = getDayEndTime(date);
            Date dayStartTime = getDayStartTime(date);
            qFilter5.and(new QFilter("tpv_quotaenddate", QCP.large_equals, dayStartTime));
            qFilter5.and(new QFilter("tpv_quiotastdate", QCP.less_equals, dayEndTime));
        }
        String selectfields5 = "tpv_department,tpv_quiotastdate,tpv_quotaenddate,tpv_replyamount";
        //查询部门资金临时额度
        DataSet tpv_sc_deptfundlimit = ORM.create().queryDataSet(algoKey,"tpv_sc_deptfundlimit",selectfields5,qFilter5.toArray());
        if (null != date) {
            tpv_sc_deptfundlimit = tpv_sc_deptfundlimit.groupBy(new String[]{"tpv_department"}).sum("tpv_replyamount").finish();
        } else {
            tpv_sc_deptfundlimit = tpv_sc_deptfundlimit.select("tpv_department","0.0 tpv_replyamount");
        }
        List<String> aliasList3 = RptUtil.getDataSetAliasList(dataSet);
        aliasList3.add("(case when tpv_replyamount != null then tpv_replyamount else 0.0 end) tpv_replyamount");
        dataSet = dataSet.leftJoin(tpv_sc_deptfundlimit).on("tpv_deptid","tpv_department")
                .select(aliasList3.toArray(new String[0])).finish();

        QFilter qFilter6 = new QFilter("billstatus", QCP.equals, "C");
        if (YEM.isNotEmpty(depid)) {
            qFilter6.and(new QFilter("tpv_entryentity.tpv_departments.id", QCP.equals, depid));
        }
        String formatDate = "";
        if (null != date) {
            formatDate = DateUtil.formatDate(date);
            String month = "0".equals(formatDate.split("-")[1].substring(0))
                    ? formatDate.split("-")[1].substring(1) : formatDate.split("-")[1];
            Integer integer = Integer.valueOf(month);
            List<String> strbegin = new ArrayList<>();
            for (Integer i = 1; i <= integer; i++) {
                strbegin.add(IntConvertmonth(i));
            }
            List<String> strend = new ArrayList<>();
            for (Integer i = integer; i <= 12; i++) {
                strend.add(IntConvertmonth(i));
            }
            qFilter6.and(new QFilter("tpv_monthorgbegin", QCP.in, strbegin));
            qFilter6.and(new QFilter("tpv_monthorgend", QCP.in, strend));
        }
        String selectfields6 = "tpv_yearorg,tpv_monthorgbegin,tpv_monthorgend,tpv_entryentity,tpv_entryentity.tpv_departments tpv_departments," +
                "tpv_entryentity.tpv_orgmonthcamt tpv_orgmonthcamt";
        //查询部门资金额度(测算)
        DataSet tpv_sc_fundslimtmeas = ORM.create().queryDataSet(algoKey,"tpv_sc_fundslimtmeas",selectfields6,qFilter6.toArray());
        if (null != date) {
            tpv_sc_fundslimtmeas = tpv_sc_fundslimtmeas.addField("to_char(tpv_yearorg,'yyyy') year", "year");
            tpv_sc_fundslimtmeas = tpv_sc_fundslimtmeas.where("year == '" + formatDate.split("-")[0] + "'");
            tpv_sc_fundslimtmeas = tpv_sc_fundslimtmeas.groupBy(new String[]{"tpv_departments"}).sum("tpv_orgmonthcamt").finish();
        } else {
            tpv_sc_fundslimtmeas = tpv_sc_fundslimtmeas.select("tpv_departments","0.0 tpv_orgmonthcamt");
        }
        List<String> aliasList4 = RptUtil.getDataSetAliasList(dataSet);
        aliasList4.add("(case when tpv_orgmonthcamt != null then tpv_orgmonthcamt else 0.0 end) tpv_orgmonthcamt");
        dataSet = dataSet.leftJoin(tpv_sc_fundslimtmeas).on("tpv_deptid","tpv_departments")
                .select(aliasList4.toArray(new String[0])).finish();
        dataSet = RptUtil.zeroNullNums(dataSet);
        List<String> listfield1 = RptUtil.getDataSetAliasList(dataSet);
        listfield1.add("tpv_cashinitoccup + tpv_paycash - tpv_putcash tpv_cashbalanceoccup");
        listfield1.add("tpv_bankinitoccup + tpv_notepayable - tpv_notereceivable tpv_bankbalanceoccup");
        listfield1.add("tpv_scmproinitoccup - tpv_scmprorec + tpv_scmpropay tpv_scmprobalanceoccup");
        listfield1.add("tpv_scmfininitoccup + tpv_scmfinadd - tpv_scmfinless tpv_scmfinbalanceoccup");
        listfield1.add("tpv_bareceiveinitoccup - tpv_bareceiveadd + tpv_bareceiveless tpv_bareceiveoccup");
        listfield1.add("tpv_bapayinitoccup + tpv_bapayadd - tpv_bapayless tpv_bapayoccup");
        listfield1.add("tpv_replyamount + tpv_orgmonthcamt tpv_nowfundsamt");
        String[] selectfield1 = listfield1.toArray(new String[0]);
        dataSet = dataSet.select(selectfield1);

        List<String> listfield2 = RptUtil.getDataSetAliasList(dataSet);
        listfield2.add("tpv_cashbalanceoccup + tpv_bankbalanceoccup tpv_onesumamt");
        String[] selectfield2 = listfield2.toArray(new String[0]);
        dataSet = dataSet.select(selectfield2);

        List<String> listfield3 = RptUtil.getDataSetAliasList(dataSet);
        listfield3.add("tpv_onesumamt + tpv_tradesum tpv_twosumamt");
        String[] selectfield3 = listfield3.toArray(new String[0]);
        dataSet = dataSet.select(selectfield3);

        List<String> listfield4 = RptUtil.getDataSetAliasList(dataSet);
        listfield4.add("tpv_twosumamt * tpv_deptproport tpv_threesumamt");
        String[] selectfield4 = listfield4.toArray(new String[0]);
        dataSet = dataSet.select(selectfield4);

        List<String> listfield5 = RptUtil.getDataSetAliasList(dataSet);
        listfield5.add("tpv_nowfundsamt - tpv_threesumamt tpv_leaveableamt");
        String[] selectfield5 = listfield5.toArray(new String[0]);
        dataSet = dataSet.select(selectfield5);

        List<String> listfield6 = RptUtil.getDataSetAliasList(dataSet);
        listfield6.add("tpv_threesumamt + (tpv_advancesum * tpv_deptproport) tpv_foursumamt");
        String[] selectfield6 = listfield6.toArray(new String[0]);
        dataSet = dataSet.select(selectfield6);

        List<String> listfield7 = RptUtil.getDataSetAliasList(dataSet);
        listfield7.add("tpv_nowfundsamt - tpv_foursumamt tpv_leaveablesum");
        String[] selectfield7 = listfield7.toArray(new String[0]);
        dataSet = dataSet.select(selectfield7);
        return dataSet;
    }

    public static JSONArray queryDatas(String orgName,String depName, String time) {
        JSONArray result = new JSONArray();
        QFilter qFiltero = new QFilter("orgpattern.number", QCP.in, new String[]{"Orgform02","Orgform03"}); //形态：公司、分公司
        qFiltero.and(new QFilter("enable", QCP.equals, "1"));
        DynamicObject[] bosOrgs = BusinessDataServiceHelper.load("bos_org", "id,name,number", qFiltero.toArray());
        Map<Long, String> orgMap = new HashMap<>();
        Map<String, Long> orgNameMap = new HashMap<>();
        Arrays.stream(bosOrgs).filter(x -> bosOrgs != null).forEach(x -> {
            Long id = x.getLong("id");
            if (!orgMap.containsKey(id)) {
                String name = x.getString("name");
                orgMap.put(id, name);
                orgNameMap.put(name, id);
            }
        });
        QFilter qFilterd = new QFilter("orgpattern.number", QCP.equals, "Orgform06"); //形态：部门
        qFilterd.and(new QFilter("enable", QCP.equals, "1"));
        DynamicObject[] bosAdminorgs = BusinessDataServiceHelper.load("bos_adminorg", "id,name,number", qFilterd.toArray());
        Map<Long, String> deptMap = new HashMap<>();
        Map<String, Long> deptNameMap = new HashMap<>();
//        Arrays.stream(bosAdminorgs).filter(x -> bosAdminorgs != null).forEach(x -> {
//            Long id = x.getLong("id");
//            if (!deptMap.containsKey(id)) {
//                String name = x.getString("name");
//                deptMap.put(id, name);
//                deptNameMap.put(name, id);
//            }
//        });
        if (bosAdminorgs.length > 0) {
            for (DynamicObject bosAdminorg : bosAdminorgs) {
                Long id = bosAdminorg.getLong("id");
                if (!deptMap.containsKey(id)) {
                    String name = bosAdminorg.getString("name");
                    deptMap.put(id, name);
                    deptNameMap.put(name, id);
                }
            }
        }
        Long depId = 0L;
        Long orgId = 0L;
        Date date = null;
        String whereFilter = "1 = 1";
        // 公司
        if (YEM.isNotEmpty(orgName)) {
            orgId = orgNameMap.get(orgName) != null ? orgNameMap.get(orgName) : 0L;
        }
        // 部门
        if (YEM.isNotEmpty(depName)) {
            if (!deptNameMap.containsKey(depName)) {
                return result;
            }
            depId = deptNameMap.get(depName) != null ? deptNameMap.get(depName) : 0L;
        }
        //查询日期
        if (YEM.isNotEmpty(time)) {
            date = DateUtil.parseDate(time);
        }
        for (Row row : query(orgId,depId,date)) {
            JSONObject jsonObject = new JSONObject(new LinkedHashMap<>());
            jsonObject.put("orgname", "");  //公司名称
            jsonObject.put("deptname", row.getString("tpv_deptname"));  //部门名称
            jsonObject.put("deptid", row.getString("tpv_deptid"));  //部门ID
            jsonObject.put("orgid", 0L);  //公司ID
            jsonObject.put("cashinitoccup", row.getBigDecimal("tpv_cashinitoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_cashinitoccup").doubleValue(), 4) : BigDecimal.ZERO);  //期初占用(现款)
            jsonObject.put("putcash", row.getBigDecimal("tpv_putcash") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_putcash").doubleValue(), 4) : BigDecimal.ZERO);  //收现款(现款)
            jsonObject.put("paycash", row.getBigDecimal("tpv_paycash") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_paycash").doubleValue(), 4) : BigDecimal.ZERO);  //付现款(现款)

            jsonObject.put("cashbalanceoccup", row.getBigDecimal("tpv_cashbalanceoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_cashbalanceoccup").doubleValue(), 4) : BigDecimal.ZERO);  //占用余额(现款)

            jsonObject.put("bankinitoccup", row.getBigDecimal("tpv_bankinitoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bankinitoccup").doubleValue(), 4) : BigDecimal.ZERO);  //期初占用(银承)
            jsonObject.put("notereceivable", row.getBigDecimal("tpv_notereceivable") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_notereceivable").doubleValue(), 4) : BigDecimal.ZERO);  //应收票据(银承)
            jsonObject.put("notepayable", row.getBigDecimal("tpv_notepayable") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_notepayable").doubleValue(), 4) : BigDecimal.ZERO);  //应付票据(银承)

            jsonObject.put("bankbalanceoccup", row.getBigDecimal("tpv_bankbalanceoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bankbalanceoccup").doubleValue(), 4) : BigDecimal.ZERO);  //占用余额(银承)

            jsonObject.put("scmproinitoccup", row.getBigDecimal("tpv_scmproinitoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_scmproinitoccup").doubleValue(), 4) : BigDecimal.ZERO);  //期初占用(供应链产品)
            jsonObject.put("scmprorec", row.getBigDecimal("tpv_scmprorec") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_scmprorec").doubleValue(), 4) : BigDecimal.ZERO);  //增加(供应链产品)
            jsonObject.put("scmpropay", row.getBigDecimal("tpv_scmpropay") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_scmpropay").doubleValue(), 4) : BigDecimal.ZERO);  //减少(供应链产品)

            jsonObject.put("scmprobalanceoccup", row.getBigDecimal("tpv_scmprobalanceoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_scmprobalanceoccup").doubleValue(), 4) : BigDecimal.ZERO);  //占用余额(供应链产品)

            jsonObject.put("scmfininitoccup", row.getBigDecimal("tpv_scmfininitoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_scmfininitoccup").doubleValue(), 4) : BigDecimal.ZERO);  //期初占用(供应链金融)
            jsonObject.put("scmfinadd", row.getBigDecimal("tpv_scmfinadd") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_scmfinadd").doubleValue(), 4) : BigDecimal.ZERO);  //增加(供应链金融)
            jsonObject.put("scmfinless", row.getBigDecimal("tpv_scmfinless") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_scmfinless").doubleValue(), 4) : BigDecimal.ZERO);  //减少(供应链金融)

            jsonObject.put("scmfinbalanceoccup", row.getBigDecimal("tpv_scmfinbalanceoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_scmfinbalanceoccup").doubleValue(), 4) : BigDecimal.ZERO);  //占用余额(供应链金融)
            jsonObject.put("chinaamt", row.getBigDecimal("tpv_chinaamt") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_chinaamt").doubleValue(), 4) : BigDecimal.ZERO);  //国内证余额
            jsonObject.put("interamt", row.getBigDecimal("tpv_interamt") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_interamt").doubleValue(), 4) : BigDecimal.ZERO);  //国际证余额
            jsonObject.put("trloanamt", row.getBigDecimal("tpv_trloanamt") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_trloanamt").doubleValue(), 4) : BigDecimal.ZERO);  //押汇余额

            jsonObject.put("bareceiveinitoccup", row.getBigDecimal("tpv_bareceiveinitoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bareceiveinitoccup").doubleValue(), 4) : BigDecimal.ZERO);  //期初占用(商业承兑汇票（应收）)
            jsonObject.put("bareceiveadd", row.getBigDecimal("tpv_bareceiveadd") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bareceiveadd").doubleValue(), 4) : BigDecimal.ZERO);  //增加(商业承兑汇票（应收）)
            jsonObject.put("bareceiveless", row.getBigDecimal("tpv_bareceiveless") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bareceiveless").doubleValue(), 4) : BigDecimal.ZERO);  //减少(商业承兑汇票（应收）)

            jsonObject.put("bareceiveoccup", row.getBigDecimal("tpv_bareceiveoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bareceiveoccup").doubleValue(), 4) : BigDecimal.ZERO);  //占用余额(商业承兑汇票（应收）)

            jsonObject.put("bapayinitoccup", row.getBigDecimal("tpv_bapayinitoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bapayinitoccup").doubleValue(), 4) : BigDecimal.ZERO);  //期初占用(商业承兑汇票（应付）)
            jsonObject.put("bapayadd", row.getBigDecimal("tpv_bapayadd") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bapayadd").doubleValue(), 4) : BigDecimal.ZERO);  //增加(商业承兑汇票（应付）)
            jsonObject.put("bapayless", row.getBigDecimal("tpv_bapayless") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bapayless").doubleValue(), 4) : BigDecimal.ZERO);  //减少(商业承兑汇票（应付）)

            jsonObject.put("bapayoccup", row.getBigDecimal("tpv_bapayoccup") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bapayoccup").doubleValue(), 4) : BigDecimal.ZERO);  //占用余额(商业承兑汇票（应付）)
            jsonObject.put("bankadvance", row.getBigDecimal("tpv_bankadvance") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_bankadvance").doubleValue(), 4) : BigDecimal.ZERO);  //预占用(银承)
            jsonObject.put("chinaamtadvance", row.getBigDecimal("tpv_chinaamtadvance") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_chinaamtadvance").doubleValue(), 4) : BigDecimal.ZERO);  //预占用(国内证)
            jsonObject.put("interamtadvance", row.getBigDecimal("tpv_interamtadvance") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_interamtadvance").doubleValue(), 4) : BigDecimal.ZERO);  //预占用(国际证)
            jsonObject.put("trloanadvance", row.getBigDecimal("tpv_trloanadvance") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_trloanadvance").doubleValue(), 4) : BigDecimal.ZERO);  //预占用(押汇)
            jsonObject.put("onesumamt", row.getBigDecimal("tpv_onesumamt") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_onesumamt").doubleValue(), 4) : BigDecimal.ZERO);  //占用合计1
            jsonObject.put("twosumamt", row.getBigDecimal("tpv_twosumamt") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_twosumamt").doubleValue(), 4) : BigDecimal.ZERO);  //占用合计2
            jsonObject.put("threesumamt", row.getBigDecimal("tpv_threesumamt") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_threesumamt").doubleValue(), 4) : BigDecimal.ZERO);  //占用合计3
            jsonObject.put("foursumamt", row.getBigDecimal("tpv_foursumamt") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_foursumamt").doubleValue(), 4) : BigDecimal.ZERO);  //占用合计4
            jsonObject.put("nowfundsamt", row.getBigDecimal("tpv_nowfundsamt") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_nowfundsamt").doubleValue(), 4) : BigDecimal.ZERO);  //当前资金额度
            jsonObject.put("leaveableamt", row.getBigDecimal("tpv_leaveableamt") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_leaveableamt").doubleValue(), 4) : BigDecimal.ZERO);  //剩余可用额度
            jsonObject.put("leaveablesum", row.getBigDecimal("tpv_leaveablesum") != null ?
                    BigDecimalUtils.ceil(row.getBigDecimal("tpv_leaveablesum").doubleValue(), 4) : BigDecimal.ZERO);  //剩余可用额度(总)
            result.add(jsonObject);
        }
        return result;
    }

    /**
     * @Description: //将月份数值转为整数
     * @param: month
     **/
    private static String IntConvertmonth(int month) {
        if (YEM.isNotEmpty(month)) {
            String monthValue = "";
            switch (month) {
                case 1: monthValue = "Jan";break; case 2: monthValue = "Feb";break; case 3: monthValue = "Mar";break;
                case 4: monthValue = "Apr";break; case 5: monthValue = "May";break; case 6: monthValue = "Jun";break;
                case 7: monthValue = "Jul";break; case 8: monthValue = "Aug";break; case 9: monthValue = "Sep";break;
                case 10: monthValue = "Oct";break; case 11: monthValue = "Nov";break; case 12: monthValue = "Dec";break;
                default:
                    break;
            }
            return monthValue;
        }
        return null;
    }

    /**
     * 获取日期的结束时间
     */
    public static Date getDayEndTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 23, 59, 59);
            calendar.set(14, 999);
            return calendar.getTime();
        }
    }

    /**
     * 获取该日期的开始时间
     */
    public static Date getDayStartTime(Date date) {
        if (date == null) {
            return null;
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.set(calendar.get(1), calendar.get(2), calendar.get(5), 0, 0, 0);
            calendar.set(14, 0);
            return calendar.getTime();
        }
    }
}
