package tpv.tpv_wm.tpv_sc.java.report;

import com.alibaba.druid.util.StringUtils;
import kd.bos.algo.Algo;
import kd.bos.algo.DataSet;
import kd.bos.algo.RowMeta;
import kd.bos.algo.input.CollectionInput;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.report.AbstractReportListDataPlugin;
import kd.bos.entity.report.FilterInfo;
import kd.bos.entity.report.ReportQueryParam;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import static kd.macc.aca.algox.costcalc.action.AbstractActSingleCheckAction.algoKey;

/**
 * 周计划汇总报表
 * WangShuGuang
 */
public class WeelPlanQueryRpt extends AbstractReportListDataPlugin {

    public final static Integer ZERO = 0;

    @Override
    public DataSet query(ReportQueryParam reportQueryParam, Object o) throws Throwable {
        // 初始化过滤条件
        //年份
        Date years = null;
        //月份
        String money = null;
        //周数
        String week = null;
        //公司
        DynamicObjectCollection company = null;
        FilterInfo filterInfo = reportQueryParam.getFilter();
        years = filterInfo.getDate("tpv_datayear");
        // 将日期对象转换为LocalDateTime
        LocalDateTime DateTime = years.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
        // 获取BigDecimal对象
        BigDecimal year = new BigDecimal(DateTime.getYear());
        money = filterInfo.getString("tpv_month");
        week = filterInfo.getString("tpv_week");
        company = filterInfo.getDynamicObjectCollection("tpv_company");
        // 过滤条件集合
        List<QFilter> list = new ArrayList<>();
        if (year.compareTo(BigDecimal.ZERO) != 0 && !StringUtils.isEmpty(money) && !StringUtils.isEmpty(week) && company.size() > ZERO) {
            //拿到当前年份的第一天的日期
            LocalDate firstDayOfYear = LocalDate.of(year.intValue(), 1, 1);
            //拿到当前年份的最后一天日期
            LocalDate endDayOfYear = LocalDate.of(year.intValue(), 12, 31);
            LocalDateTime localDateTime = firstDayOfYear.atStartOfDay(); // 将 LocalDate 转换为 LocalDateTime，时间部分设为零点
            Date startdate = Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant()); // 将 LocalDateTime 转换为 Date
            LocalDateTime endDateTime = endDayOfYear.atStartOfDay(); // 将 LocalDate 转换为 LocalDateTime，时间部分设为零点
            Date enddate = Date.from(endDateTime.atZone(ZoneId.systemDefault()).toInstant()); // 将 LocalDateTime 转换为 Date
            //list.add(new QFilter("billstatus", QCP.equals, "C"));//查询单据状态为已审核的
            list.add(new QFilter("tpv_bizdate", QCP.large_equals, startdate));
            list.add(new QFilter("tpv_bizdate", QCP.less_equals, enddate));
            //月份过滤
            list.add(new QFilter("tpv_month", QCP.equals, money));
            //周数过滤
            list.add(new QFilter("tpv_week", QCP.equals, week));
            List<Long> idS = new ArrayList<>();
            for (DynamicObject row : company) {
                //DynamicObject dy = row.getDynamicObject("tpv_company.number");
                Long a = row.getLong("id");
                idS.add(a);
            }
            list.add(new QFilter("tpv_sc_detail.tpv_corporation.id", QCP.in, idS));
            DataSet work = this.queryWork(list).select("year(tpv_bizdate) tpv_year", "tpv_monthdata", "tpv_weekdata", "tpv_collectiondays", "tpv_rmbmoneygathering",
                    "tpv_daomoneygathering", "tpv_rmbmoneypayment", "tpv_daomoneypayment", "tpv_sc_detail.tpv_incomeexpenses", "tpv_sc_detail.tpv_currencyfield", "tpv_basedatafield", "tpv_rmbcurrency", "tpv_daocurrency");
            DynamicObjectCollection col_s = ORM.create().toPlainDynamicObjectCollection(work);
            RowMeta rowMeta = work.getRowMeta();
            int fieldCount = rowMeta.getFieldCount();
            Collection<Object[]> collection = new ArrayList<>();
            for (int i = 0; i < col_s.size(); i++) {
                DynamicObject col = col_s.get(i);
                Long tpvCurrencyfield = col.getLong("tpv_sc_detail.tpv_currencyfield");
                //币种1是人民币 6是美元
                String tpvIncomeexpenses = col.getString("tpv_sc_detail.tpv_incomeexpenses");
                if ("A".equals(tpvIncomeexpenses)) {
                    if (tpvCurrencyfield.toString().equals("6")) {
                        //清空除美元收款外其他字段
                        col.set("tpv_rmbmoneygathering", null);
                        col.set("tpv_rmbmoneypayment", null);
                        col.set("tpv_daomoneypayment", null);
                        //col.set("tpv_currencyfield2", tpvCurrencyfield);
                    } else {
                        //清空除人民币收款外其他字段
                        col.set("tpv_daomoneygathering", null);
                        col.set("tpv_rmbmoneypayment", null);
                        col.set("tpv_daomoneypayment", null);
                        //col.set("tpv_currencyfield1", tpvCurrencyfield);
                    }
                } else {
                    if (tpvCurrencyfield.toString().equals("6")) {
                        //清空除美元付款外其他字段
                        col.set("tpv_rmbmoneygathering", null);
                        col.set("tpv_daomoneygathering", null);
                        col.set("tpv_rmbmoneypayment", null);
                        //col.set("tpv_currencyfield2", tpvCurrencyfield);
                    } else {
                        //清空除人民币付款外其他字段
                        col.set("tpv_daomoneypayment", null);
                        col.set("tpv_rmbmoneygathering", null);
                        col.set("tpv_daomoneygathering", null);
                        //col.set("tpv_currencyfield1", tpvCurrencyfield);
                    }
                }
                Object[] obj = new Object[fieldCount];
                for (int j = 0; j < fieldCount; j++) {
                    obj[j] = col.get(j);
                }
                collection.add(obj);
            }
            CollectionInput inputs = new CollectionInput(rowMeta, collection);
            DataSet createDataSet = Algo.create(algoKey).createDataSet(inputs);
            createDataSet = createDataSet.groupBy(new String[]{"tpv_year", "tpv_monthdata", "tpv_weekdata", "tpv_collectiondays", "tpv_basedatafield"})
                    .sum("tpv_rmbmoneygathering")
                    .sum("tpv_daomoneygathering")
                    .sum("tpv_rmbmoneypayment")
                    .sum("tpv_daomoneypayment").finish();
            createDataSet = createDataSet.select("tpv_year", "tpv_monthdata", "tpv_weekdata", "tpv_collectiondays", "tpv_rmbmoneygathering",
                    "tpv_daomoneygathering", "tpv_rmbmoneypayment", "tpv_daomoneypayment", "1 tpv_rmbcurrency", "6 tpv_daocurrency", "tpv_basedatafield");
            return createDataSet;
        }
        return null;
    }

    private DataSet queryWork(List<QFilter> list) {
        // 定义查询字段名
        String selectFiles = "tpv_bizdate,"//申请日期对应年份
                + "tpv_month tpv_monthdata,"//月份
                + "tpv_week tpv_weekdata,"//周数
                + "tpv_sc_detail.tpv_receiptpayment tpv_collectiondays,"//收款日期
                + "tpv_sc_detail.tpv_original tpv_rmbmoneygathering,"//人民币收款
                + "tpv_sc_detail.tpv_original tpv_daomoneygathering,"//美元收款
                + "tpv_sc_detail.tpv_original tpv_rmbmoneypayment,"//人民币付款
                + "tpv_sc_detail.tpv_original tpv_daomoneypayment,"//美元付款
                + "tpv_sc_detail.tpv_incomeexpenses,"//收支方向
                + "tpv_sc_detail.tpv_currencyfield,"//币种
                + "tpv_sc_detail.tpv_currencyfield tpv_rmbcurrency,"//币种
                + "tpv_sc_detail.tpv_currencyfield tpv_daocurrency,"//币种
                + "tpv_sc_detail.tpv_corporation.id tpv_basedatafield";//公司
        DataSet queryDataSet = ORM.create().queryDataSet(this.getClass().getName(), "tpv_sc_capitalweek", selectFiles, list.toArray(new QFilter[0])).select(new String[]{"tpv_bizdate", "tpv_monthdata", "tpv_weekdata", "tpv_collectiondays", "tpv_rmbmoneygathering",
                "tpv_basedatafield", "tpv_sc_detail.tpv_incomeexpenses", "tpv_sc_detail.tpv_currencyfield", "tpv_daomoneygathering", "tpv_rmbmoneypayment", "tpv_daomoneypayment", "tpv_rmbcurrency", "tpv_daocurrency"});
        return queryDataSet;
    }

}
