package tpv.tpv_wm.tpv_tm.java.api.openapi;

import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.openapi.common.custom.annotation.*;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import tpv.tpv_wm.tpv_tm.java.api.entitys.DealrecordModel;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.*;

/**
 * @Description: 开放查询接口，将参数传入成交记录单，返回ResultData
 * @Author LinRui
 * @Date 2024/3/11 18:14
 */

@ApiController(value = "Dealrecord", desc = "成交记录单查询表")
@ApiMapping("Dealrecord")
@ApiErrorCodes({
        @ApiErrorCode(code = "404", desc = "路径访问错误")
})
public class DealrecordResultController implements Serializable {
    @ApiPostMapping(value = "saveDealrecords", desc = "批量添加成交记录单数据")
    public CustomApiResult<String> saveDealrecords(@Valid @NotNull @ApiParam(
            value = "批量创建成交记录单数据，不能超过3000条数据",
            required = true
    ) @Size(
            max = 3000
    ) List<DealrecordModel> dealrecordModels) {
        if (dealrecordModels.size() > 3000) {
            return CustomApiResult.fail(null,"入参数据，不能超过3000条数据");
        }
        DynamicObject[] tpv_tm_contractool = BusinessDataServiceHelper.load("tpv_tm_contractool", "id,number,name",
                new QFilter[]{new QFilter("enable", QCP.equals, "1")});
        Map<String,Long> mapTool = new HashMap<>();
        if (null != tpv_tm_contractool && tpv_tm_contractool.length > 0) {
            for (DynamicObject dynamicObject : tpv_tm_contractool) {
                mapTool.put(dynamicObject.getString("name"),dynamicObject.getLong("id"));
            }
        }

        DynamicObject[] tpv_tm_futureaccount = BusinessDataServiceHelper.load("tpv_tm_futureaccount",
                "id,tpv_futures,tpv_administrative",
                new QFilter[]{new QFilter("billstatus", QCP.equals, "C")});
        Map<String, DynamicObject> mapFuture = new HashMap<>();
        if (null != tpv_tm_futureaccount && tpv_tm_futureaccount.length > 0) {
            for (DynamicObject dynamicObject : tpv_tm_futureaccount) {
                mapFuture.put(dynamicObject.getString("tpv_futures"), dynamicObject);
            }
        }
        List<DynamicObject> dynamicObjectList = new ArrayList<>();

        try {
            for (int i = 0; i < dealrecordModels.size(); i++) {
                DealrecordModel dealrecordModel = dealrecordModels.get(i);
                DynamicObject dynamicObject = BusinessDataServiceHelper.newDynamicObject("tpv_tm_dealrecord");
                dynamicObject.set("tpv_nsid",dealrecordModel.getFid()); //外来id
                dynamicObject.set("billno",dealrecordModel.getFbillno()); //单据编号
                dynamicObject.set("billstatus",dealrecordModel.getStatus()); //单据状态
                dynamicObject.set("tpv_account",YEM.isNotEmpty(dealrecordModel.getUserid()) ?mapFuture.containsKey(dealrecordModel.getUserid())
                        ? mapFuture.get(dealrecordModel.getUserid()).getLong("id") : null : null); //账户
                DynamicObjectCollection material = mapFuture.get(dealrecordModel.getUserid()).getDynamicObjectCollection("tpv_administrative");
                DynamicObjectCollection tpv_material = new DynamicObjectCollection();
                for (DynamicObject entry : material) {
                    DynamicObject fbasedataId = entry.getDynamicObject("fbasedataId");
                    DynamicObject newMaterial = new DynamicObject(material.getDynamicObjectType());
                    newMaterial.set("fbasedataId", fbasedataId ); //集合或数组
                    tpv_material.add(newMaterial);
                }
                if (tpv_material.size() > 0) {
                    dynamicObject.set("tpv_amcunit",YEM.isNotEmpty(dealrecordModel.getUserid()) ?mapFuture.containsKey(dealrecordModel.getUserid())
                            ? tpv_material : null : null); //账户归属管理单元
                    dynamicObject.set("tpv_claimablesnapin",YEM.isNotEmpty(dealrecordModel.getUserid()) ?mapFuture.containsKey(dealrecordModel.getUserid())
                            ? tpv_material : null : null); //可认领管理单元
                } else {
                    dynamicObject.set("tpv_amcunit", null); //账户归属管理单元
                    dynamicObject.set("tpv_claimablesnapin", null); //可认领管理单元
                }
                dynamicObject.set("tpv_contractool",dealrecordModel.getInstrumentid() != null ? mapTool.containsKey(dealrecordModel.getInstrumentid())
                        ? mapTool.get(dealrecordModel.getInstrumentid()) : null : null); //合约池
                dynamicObject.set("tpv_tradeway",dealrecordModel.getDirection()); //买卖方向
                dynamicObject.set("tpv_openclosesign",dealrecordModel.getOffset()); //开平标志
                dynamicObject.set("tpv_deallots", dealrecordModel.getVolume() != null ? BigDecimalUtils.toBigDecimal(dealrecordModel.getVolume()) : BigDecimal.ZERO); //成交手数
                dynamicObject.set("tpv_dealton",dealrecordModel.getWeight() != null ? BigDecimalUtils.toBigDecimal(dealrecordModel.getWeight()) : BigDecimal.ZERO); //成交吨数
                dynamicObject.set("tpv_dealprice",dealrecordModel.getPrice()); //成交价格
                dynamicObject.set("tpv_dealdate", dealrecordModel.getTradedatetime() != null ? dealrecordModel.getTradedatetime() : null); //成交时间
                dynamicObject.set("tpv_processingfee",dealrecordModel.getCommission()); //手续费
                dynamicObject.set("tpv_varietyname",dealrecordModel.getProductid()); //品种名
                dynamicObject.set("tpv_contracmul",dealrecordModel.getVolumemultiple() != null ? BigDecimalUtils.toBigDecimal(dealrecordModel.getVolumemultiple()) : BigDecimal.ZERO); //合约乘数
                dynamicObject.set("tpv_recorddate", dealrecordModel.getTimestamp1() != null ? dealrecordModel.getTimestamp1() : null); //记录时间
                dynamicObject.set("tpv_srcbillway","A"); //数据来源
                dynamicObject.set("tpv_insuresign",dealrecordModel.getHedge_type()); //投保标志
                dynamicObject.set("creator",YEM.getCurrentUserId()); //创建人
                dynamicObject.set("createtime",new Date()); //创建时间
                dynamicObject.set("modifier",YEM.getCurrentUserId()); //修改人
                dynamicObject.set("modifytime", new Date()); //修改时间
                dynamicObjectList.add(dynamicObject);
            }
            DynamicObject[] dynamicObjects = dynamicObjectList.toArray(new DynamicObject[0]);
            SaveServiceHelper.save(dynamicObjects);
            return CustomApiResult.success("created successfully!");
        } catch (Exception e) {
            RuntimeException runtimeException = new RuntimeException(e);
            return CustomApiResult.fail("500", "批量添加失败! " + runtimeException.getMessage());
        }

    }

    /**
     * @deprecated 方法作废
     * */
    @ApiPostMapping(value = "saveDealrecord", desc = "添加成交记录单数据")
    public CustomApiResult<String> saveDealrecord(@Valid @NotNull @ApiRequestBody DealrecordModel dealrecordModel) {
        DynamicObject[] tpv_tm_contractool = BusinessDataServiceHelper.load("tpv_tm_contractool", "id,number,name",
                new QFilter[]{new QFilter("enable", QCP.equals, "1")});
        Map<String,Long> mapTool = new HashMap<>();
        if (null != tpv_tm_contractool && tpv_tm_contractool.length > 0) {
            for (DynamicObject dynamicObject : tpv_tm_contractool) {
                mapTool.put(dynamicObject.getString("name"),dynamicObject.getLong("id"));
            }
        }

        DynamicObject[] tpv_tm_futureaccount = BusinessDataServiceHelper.load("tpv_tm_futureaccount",
                "id,tpv_futures,tpv_administrative",
                new QFilter[]{new QFilter("enable", QCP.equals, "1")});
        Map<String, DynamicObject> mapFuture = new HashMap<>();
        if (null != tpv_tm_futureaccount && tpv_tm_futureaccount.length > 0) {
            for (DynamicObject dynamicObject : tpv_tm_futureaccount) {
                mapFuture.put(dynamicObject.getString("tpv_futures"), dynamicObject);
            }
        }
        try {
            DynamicObject dynamicObject = BusinessDataServiceHelper.newDynamicObject("tpv_tm_dealrecord");
            dynamicObject.set("tpv_nsid",dealrecordModel.getFid()); //外来id
            dynamicObject.set("billno",dealrecordModel.getFbillno()); //单据编号
            dynamicObject.set("billstatus",dealrecordModel.getStatus()); //单据状态
            dynamicObject.set("tpv_account",YEM.isNotEmpty(dealrecordModel.getUserid()) ?mapFuture.containsKey(dealrecordModel.getUserid())
                    ? mapFuture.get(dealrecordModel.getUserid()).getLong("id") : null : null); //账户
            DynamicObjectCollection material = mapFuture.get(dealrecordModel.getUserid()).getDynamicObjectCollection("tpv_administrative");
            DynamicObjectCollection tpv_material = new DynamicObjectCollection();
            for (DynamicObject entry : material) {
                DynamicObject fbasedataId = entry.getDynamicObject("fbasedataId");
                DynamicObject newMaterial = new DynamicObject(material.getDynamicObjectType());
                newMaterial.set("fbasedataId", fbasedataId ); //集合或数组
                tpv_material.add(newMaterial);
            }
            if (tpv_material.size() > 0) {
                dynamicObject.set("tpv_amcunit",YEM.isNotEmpty(dealrecordModel.getUserid()) ?mapFuture.containsKey(dealrecordModel.getUserid())
                        ? tpv_material : null : null); //账户归属管理单元
                dynamicObject.set("tpv_claimablesnapin",YEM.isNotEmpty(dealrecordModel.getUserid()) ?mapFuture.containsKey(dealrecordModel.getUserid())
                        ? tpv_material : null : null); //可认领管理单元
            } else {
                dynamicObject.set("tpv_amcunit", null); //账户归属管理单元
                dynamicObject.set("tpv_claimablesnapin", null); //可认领管理单元
            }
            dynamicObject.set("tpv_contractool",dealrecordModel.getInstrumentid() != null ? mapTool.containsKey(dealrecordModel.getInstrumentid())
                    ? mapTool.get(dealrecordModel.getInstrumentid()) : null : null); //合约池
            dynamicObject.set("tpv_tradeway",dealrecordModel.getDirection()); //买卖方向
            dynamicObject.set("tpv_openclosesign",dealrecordModel.getOffset()); //开平标志
            dynamicObject.set("tpv_deallots", dealrecordModel.getVolume() != null ? BigDecimalUtils.toBigDecimal(dealrecordModel.getVolume()) : BigDecimal.ZERO); //成交手数
            dynamicObject.set("tpv_dealton",dealrecordModel.getWeight() != null ? BigDecimalUtils.toBigDecimal(dealrecordModel.getWeight()) : BigDecimal.ZERO); //成交吨数
            dynamicObject.set("tpv_dealprice",dealrecordModel.getPrice()); //成交价格
            dynamicObject.set("tpv_dealdate", dealrecordModel.getTradedatetime() != null ? dealrecordModel.getTradedatetime() : null); //成交时间
            dynamicObject.set("tpv_processingfee",dealrecordModel.getCommission()); //手续费
            dynamicObject.set("tpv_varietyname",dealrecordModel.getProductid()); //品种名
            dynamicObject.set("tpv_contracmul",dealrecordModel.getVolumemultiple() != null ? BigDecimalUtils.toBigDecimal(dealrecordModel.getVolumemultiple()) : BigDecimal.ZERO); //合约乘数
            dynamicObject.set("tpv_recorddate", dealrecordModel.getTimestamp1() != null ? dealrecordModel.getTimestamp1() : null); //记录时间
            dynamicObject.set("tpv_srcbillway","A"); //数据来源
            dynamicObject.set("tpv_insuresign",dealrecordModel.getHedge_type()); //投保标志
            dynamicObject.set("creator",YEM.getCurrentUserId()); //创建人
            dynamicObject.set("createtime",new Date()); //创建时间
            dynamicObject.set("modifier",YEM.getCurrentUserId()); //修改人
            dynamicObject.set("modifytime", new Date()); //修改时间
            SaveServiceHelper.save(new DynamicObject[]{dynamicObject});
            return CustomApiResult.success("created successfully!");
        } catch (Exception e) {
            RuntimeException runtimeException = new RuntimeException(e);
            return CustomApiResult.fail("500", "添加失败! " + runtimeException.getMessage());
        }
    }
}
