package tpv.tpv_wm.tpv_tm.java.formplugin.form;

import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.RptUtil;
import com.itopview.wm.utils.YEM;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.EventObject;

/**
 * @ClassName: ConttranFormPlugin
 * @Description: 合同结转单
 * @author: zbh
 * @date: 2024-07-30
 */
public class ConttranFormPlugin extends AbstractBillPlugIn {

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        IDataModel model = this.getModel();
        switch (name){
            case "tpv_baseprices"://合同基价
            case "tpv_purbasis"://采购 基价调整
            case "tpv_salebasis"://销售 基价调整
                baseprices(model);
                break;
            case "tpv_strategynum": //采购策略号
                setPositionQty(name, "tpv_positionnumspur", "tpv_openpositionpur");
                break;
            case "tpv_salestrategynum": //销售策略号
                setPositionQty(name, "tpv_positionnumssale", "tpv_openpositionsale");
                break;
            default:
                break;
        }
    }

    private void setPositionQty(String name, String positionnumsKey, String openpositionKey) {
        DynamicObject tpvStrategynum = (DynamicObject) this.getModel().getValue(name);
        if (YEM.isNotEmpty(tpvStrategynum)) {
            long id = tpvStrategynum.getLong("id");
            DataSet purchase = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_purchase", "id,tpv_contractnumber,tpv_policynumber.id policyid," +
                            "tpv_policynumber.number policynumber,tpv_materialentry,tpv_materialentry.tpv_qty qty",
                    new QFilter[]{new QFilter("tpv_contractstatusx", QCP.in, new String[]{"A", "B", "C", "D", "F", "G"})
                            .and(new QFilter("tpv_policynumber.id", QCP.in, id))}).select("policyid policyid","qty purqty","0.0 saleqty");
            DataSet salescontract = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_salescontract", "id,tpv_contractnos,tpv_strategynum.id strategyid," +
                            "tpv_strategynum.number strategynum,tpv_materialentry,tpv_materialentry.tpv_qty qty",
                    new QFilter[]{new QFilter("tpv_contractstatus", QCP.in, new String[]{"A","B","C","D","G","H"})
                            .and(new QFilter("tpv_strategynum.id", QCP.in, id))}).select("strategyid policyid","qty saleqty","0.0 purqty");
            DataSet dataSet = purchase.union(salescontract).groupBy(new String[]{"policyid"}).sum("saleqty").sum("purqty").finish();
            dataSet = RptUtil.zeroNullNums(dataSet);
            dataSet = dataSet.addField("(purqty - saleqty) qty", "qty");
            BigDecimal qty = BigDecimal.ZERO;
            if (!dataSet.isEmpty()) {
                for (Row row : dataSet) {
                    qty = row.getBigDecimal("qty").setScale(2, BigDecimal.ROUND_HALF_UP).abs();
                }
            }
            BigDecimal tpvReplylimit = BigDecimalUtils.toBigDecimal(tpvStrategynum.getBigDecimal("tpv_replylimit")).setScale(2, BigDecimal.ROUND_HALF_UP);
            this.getModel().setValue(positionnumsKey,qty);
            this.getModel().setValue(openpositionKey,tpvReplylimit.subtract(qty));
        } else {
            this.getModel().setValue(positionnumsKey, BigDecimal.ZERO);
            this.getModel().setValue(openpositionKey, BigDecimal.ZERO);
        }
    }

    public static void baseprices(IDataModel model) {
        BigDecimal tpvBaseprices = (BigDecimal) model.getValue("tpv_baseprices");//合同基价
        BigDecimal tpvPurbasis = (BigDecimal) model.getValue("tpv_purbasis");//采购 基价调整
        BigDecimal tpvSalebasis = (BigDecimal) model.getValue("tpv_salebasis");//销售 基价调整

        DynamicObjectCollection tpvTmIniactivehed =model.getDataEntity(true).getDynamicObjectCollection("tpv_tm_iniactivehed");//采购
        DynamicObjectCollection tpvTmActivehed = model.getDataEntity(true).getDynamicObjectCollection("tpv_tm_activehed");//销售

        if (tpvTmIniactivehed.size()==0){//采购
            if (tpvPurbasis==null){
                model.setValue("tpv_basis",tpvBaseprices);
            }else {
                model.setValue("tpv_basis",tpvBaseprices.subtract(tpvPurbasis));
            }
        }else {
            BigDecimal decimal = BigDecimal.ZERO;
            for (DynamicObject dynamicObject : tpvTmIniactivehed) {
                decimal= decimal.add(dynamicObject.getBigDecimal("tpv_initranprice"));
            }
            decimal=decimal.divide(new BigDecimal(tpvTmIniactivehed.size()),4, RoundingMode.HALF_UP);
            if (tpvPurbasis==null){
                model.setValue("tpv_basis",tpvBaseprices.subtract(decimal));
            }else {
                model.setValue("tpv_basis",tpvBaseprices.add(tpvPurbasis).subtract(decimal));
            }
        }


        if (tpvTmActivehed.size()==0){//销售
            if (tpvSalebasis==null){
                model.setValue("tpv_basissale",tpvBaseprices);
            }else {
                model.setValue("tpv_basissale",tpvBaseprices.add(tpvSalebasis));
            }
        }else {
            BigDecimal decimal = BigDecimal.ZERO;
            for (DynamicObject dynamicObject : tpvTmActivehed) {
                decimal= decimal.add(dynamicObject.getBigDecimal("tpv_tranprice"));
            }
            decimal=decimal.divide(new BigDecimal(tpvTmActivehed.size()),4, RoundingMode.HALF_UP);
            if (tpvSalebasis==null){
                model.setValue("tpv_basissale",tpvBaseprices.subtract(decimal));
            }else {
                model.setValue("tpv_basissale",tpvBaseprices.add(tpvSalebasis).subtract(decimal));
            }
        }
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        IDataModel model = this.getModel();
//        baseprices(model);
    }
}
