package tpv.tpv_wm.tpv_tm.java.formplugin.form;

import kd.bos.bill.OperationStatus;
import kd.bos.dataentity.entity.DataEntityBase;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.CloseCallBack;
import kd.bos.form.FormShowParameter;
import kd.bos.form.IFormView;
import kd.bos.form.ShowType;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.events.BeforeDoOperationEventArgs;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.form.operate.FormOperate;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.ksql.util.StringUtil;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 基础资料 《策略号》表单插件
 * @Project: zhck
 * @Package: tpv.tpv_wm.tpv_tm.java.formplugin.form.StrategyFormPlugin
 * @Date: 2024/04/08 11:10
 * @Author: Heisenberg Wenbei He
 * @Version: 1.0
 */
public class StrategyFormPlugin extends AbstractFormPlugin implements BeforeF7SelectListener {

    // 分成部门 - 分录标识
    private static final String SPLIT_DEPARTMENT_ENTRY = "tpv_splitdepartment";

    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        BasedataEdit tpvContract = this.getControl("tpv_contract");
        tpvContract.addBeforeF7SelectListener(this);
        BasedataEdit tpvOperator = this.getControl("tpv_operator");
        tpvOperator.addBeforeF7SelectListener(this);
        BasedataEdit tpv_termdefinition = this.getControl("tpv_termdefinition");
        tpv_termdefinition.addBeforeF7SelectListener(this);
        BasedataEdit tpv_departmententry = this.getControl("tpv_departmententry");
        tpv_departmententry.addBeforeF7SelectListener(this);
        BasedataEdit tpv_contractcounts = this.getControl("tpv_contractcounts");
        tpv_contractcounts.addBeforeF7SelectListener(this);
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent e) {
        switch (e.getProperty().getName()) {
            case "tpv_contract":
                tpvContract(e);
                break;
            case "tpv_operator":
                ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
                List<QFilter> filters = new ArrayList<>();
                CloseCallBack closeCallBack = new CloseCallBack(this, e.getProperty().getName());
                showParameter.setCloseCallBack(closeCallBack);
                showParameter.getListFilterParameter().setQFilters(filters);
                break;
            case"tpv_termdefinition"://协同部门 期现管理单元定义
                tpvTermdefinition(e);
                break;
            case"tpv_departmententry":
                tpvDepartmententry(e);
                break;
            case"tpv_contractcounts":
                tpvContractcounts(e);
                break;
            default:
                break;
        }
    }

    private void tpvContractcounts(BeforeF7SelectEvent e) {
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        List<QFilter> filters = new ArrayList<>();
        DynamicObjectCollection tpvStrategys = this.getModel().getDataEntity(true).getDynamicObjectCollection("tpv_bd_strategy_hy");
        List<Long> list = new ArrayList<>();
        for (DynamicObject tpvStrategy : tpvStrategys) {
            DynamicObjectCollection tpvContractcounts = tpvStrategy.getDynamicObjectCollection("tpv_contractcounts");
            for (DynamicObject tpvContractcount : tpvContractcounts) {
                list.add(tpvContractcount.getLong("fbasedataId.id"));
            }
        }
        filters.add(new QFilter("id",QCP.not_in,list));
        filters.add(new QFilter("level",QCP.equals,1));
        showParameter.getListFilterParameter().setQFilters(filters);
    }

    private void tpvDepartmententry(BeforeF7SelectEvent e) {
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        List<QFilter> filters = new ArrayList<>();
        DynamicObjectCollection tpvSplitdepartment = this.getModel().getDataEntity(true).getDynamicObjectCollection("tpv_splitdepartment");
        List<Long> list = new ArrayList<>();
        for (DynamicObject dynamicObject : tpvSplitdepartment) {
            DynamicObject tpvTermdefinition = dynamicObject.getDynamicObject("tpv_departmententry");
            if (tpvTermdefinition!=null){
                list.add(tpvTermdefinition.getLong("id"));
            }
        }
       List<String> strings = new ArrayList<>();
        strings.add("Orgform01");
        strings.add("Orgform02");
        strings.add("Orgform03");
        strings.add("Orgform04");
        strings.add("Orgform05");
        filters.add(new QFilter("id",QCP.not_in,list));
        filters.add(new QFilter("orgpattern.number",QCP.not_in,strings));
        showParameter.getListFilterParameter().setQFilters(filters);
    }

    private void tpvTermdefinition(BeforeF7SelectEvent e) {
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        List<QFilter> filters = new ArrayList<>();
        DynamicObjectCollection tpvBdStrategyXt = this.getModel().getDataEntity(true).getDynamicObjectCollection("tpv_bd_strategy_xt");
        List<Long> list = new ArrayList<>();
        for (DynamicObject dynamicObject : tpvBdStrategyXt) {
            DynamicObject tpvTermdefinition = dynamicObject.getDynamicObject("tpv_termdefinition");
            if (tpvTermdefinition!=null){
                list.add(tpvTermdefinition.getLong("id"));
            }
        }
        filters.add(new QFilter("id",QCP.not_in,list));
        filters.add(new QFilter("tpv_billtype.number",QCP.equals,"tpv_tm_snapdefinition_BT"));
        showParameter.getListFilterParameter().setQFilters(filters);
    }

    public void tpvContract(BeforeF7SelectEvent e){
        ListShowParameter showParameter = (ListShowParameter) e.getFormShowParameter();
        List<QFilter> filters = new ArrayList<>();
        DynamicObject[] all = BusinessDataServiceHelper.load("tpv_tm_contractool", "parent", null);
        DynamicObject[] parent = BusinessDataServiceHelper.load("tpv_tm_contractool"
                , "parent"
                , new QFilter[]{new QFilter("parent", "!=", 0L)});
        List<Object> ids = Arrays.stream(all).map(DataEntityBase::getPkValue).collect(Collectors.toList());
        List<Object> parents = Arrays.stream(parent)
                .map(single -> single.getDynamicObject("parent").getPkValue()).collect(Collectors.toList());
        ids.removeAll(parents);

        LocalDate currentDate = LocalDate.now();
        LocalTime midnight = LocalTime.MIDNIGHT;
        LocalDateTime localDateTime = LocalDateTime.of(currentDate, midnight);

        filters.add(new QFilter("enable", "=", "1"));
        filters.add(new QFilter("id", "in", ids));
        filters.add(new QFilter("tpv_effectivedate", ">=", localDateTime));
        showParameter.getListFilterParameter().setQFilters(filters);
    }
    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        this.setDefaultValueForFirstRow();
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        this.setDefaultValueForFirstRow();
    }


    @Override
    public void beforeDoOperation(BeforeDoOperationEventArgs e) {
        super.beforeDoOperation(e);
        String operateKey = ((FormOperate) e.getSource()).getOperateKey();
        IFormView view = this.getView();
        if ("donothing_wh".equals(operateKey)) {
            FormShowParameter parameter = new FormShowParameter();
            parameter.setStatus(OperationStatus.ADDNEW);
            parameter.getOpenStyle().setShowType(ShowType.Modal);
            parameter.setFormId("tpv_bd_databackfill");
            HashMap<String, Object> map = new HashMap<>();

            map.put("tpv_replylimit", this.getModel().getValue("tpv_quotaqty"));
            map.put("tpv_pearlywarningvalue", this.getModel().getValue("tpv_earlywarningvalue"));
            map.put("tpv_pstoplossvalue", this.getModel().getValue("tpv_stoplossvalue"));
            map.put("tpv_validuntilfrom", this.getModel().getValue("tpv_validfrom"));
            map.put("tpv_validuntilapp", this.getModel().getValue("tpv_validuntil"));
            parameter.setCustomParams(map);

            parameter.setCloseCallBack(new CloseCallBack(this, "donothing_wh"));
            view.showForm(parameter);
        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent e) {
        String actionId = e.getActionId();
        Object returnData = e.getReturnData();

        if (StringUtil.equals("donothing_wh", actionId) && null != returnData) {
            Map returnDataStr = (Map) returnData;
            Object tpv_replylimit = returnDataStr.get("tpv_replylimit");
            Object tpv_pearlywarningvalue = returnDataStr.get("tpv_pearlywarningvalue");
            Object tpv_pstoplossvalue = returnDataStr.get("tpv_pstoplossvalue");
            Object tpv_validuntilfrom = returnDataStr.get("tpv_validuntilfrom");
            Object tpv_validuntilapp = returnDataStr.get("tpv_validuntilapp");
            this.getModel().setValue("tpv_replylimit", tpv_replylimit);
            this.getModel().setValue("tpv_pearlywarningvalue", tpv_pearlywarningvalue);
            this.getModel().setValue("tpv_pstoplossvalue", tpv_pstoplossvalue);
            this.getModel().setValue("tpv_validuntilfrom", tpv_validuntilfrom);
            this.getModel().setValue("tpv_validuntilapp", tpv_validuntilapp);
            // 保存
            DynamicObject dataEntity = this.getModel().getDataEntity(true);
            SaveServiceHelper.save(new DynamicObject[]{dataEntity});

        } else if (StringUtil.equals("tpv_operator", actionId) && null != returnData) {
            IDataModel model = this.getModel();
            ListSelectedRowCollection selectedRows = (ListSelectedRowCollection) e.getReturnData();
            if (selectedRows != null && selectedRows.size() > 0) {
                ListSelectedRow selectedRow = selectedRows.get(0);
                long userId = (long) selectedRow.getPrimaryKeyValue();//人员id
                long userEntryId = (long) selectedRow.getEntryPrimaryKeyValue();//部门分录id
                model.setValue("tpv_operator", userId);
                DynamicObjectCollection entry = getUserDepts(userId);
                for (DynamicObject row : entry) {
                    if (userEntryId == row.getLong("id")) {
                        model.setValue("tpv_departmentorg", row.get("org"));
                    }
                }
            }
        }
        setDefaultValueForFirstRow();
    }

    private static DynamicObjectCollection getUserDepts(long userId) {
        QFilter qFilter = new QFilter("user", QCP.equals, userId);
        return QueryServiceHelper.query("bos_userposition", "id,org", qFilter.toArray());
    }

    /**
     * @author: Heisenberg
     * @date: 2024/4/8 11:15
     * @description: 分录-分成部门 默认存在一行数据，部门=当前部门，分成方式=按比例分成，分成比例=100%
     */
    private void setDefaultValueForFirstRow() {
        DynamicObjectCollection collection = this.getModel().getDataEntity(true).getDynamicObjectCollection(SPLIT_DEPARTMENT_ENTRY);
        if (collection.isEmpty()) {
            int row = this.getModel().createNewEntryRow(SPLIT_DEPARTMENT_ENTRY);
            DynamicObject department = (DynamicObject) this.getModel().getValue("tpv_departmentorg");
            this.getModel().setValue("tpv_allocationmethod", "C", row);
            this.getModel().setValue("tpv_apercentage", 100, row);
            this.getModel().setValue("tpv_departmententry", department, row);
        } else {
            DynamicObject tpvDepartmententry = collection.get(0).getDynamicObject("tpv_departmententry");
            if (tpvDepartmententry == null) {
                DynamicObject department = (DynamicObject) this.getModel().getValue("tpv_departmentorg");
                this.getModel().setValue("tpv_departmententry", department, 0);
            }
        }
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
        switch (name) {
            case "tpv_validfrom":
            case "tpv_validuntil":
                extracted();
                break;
            case "tpv_swaplegdate":
            case "tpv_swaplegdatato":
                extract();
                break;
            case "tpv_departmentorg":
                setDefaultValueForFirstRow();
                break;
            case "tpv_allocationmethod":
                allocationMethod();
                break;
        }
    }

    private void allocationMethod() {
        DynamicObject dataEntity = this.getModel().getDataEntity(true);
        DynamicObjectCollection tpvSplitdepartments = dataEntity.getDynamicObjectCollection("tpv_splitdepartment");
        if (tpvSplitdepartments.size() == 0 && tpvSplitdepartments.size() == 1) {
            return;
        }

        List<String> list = new ArrayList<>();
        for (DynamicObject tpvSplitdepartment : tpvSplitdepartments) {
            list.add(tpvSplitdepartment.getString("tpv_allocationmethod"));
        }
        if (list.contains("B")&&list.contains("C")){
            EntryGrid entryGrid = this.getControl("tpv_splitdepartment");
            int selectRows[] = entryGrid.getSelectRows();//获取选中的行号
            if (selectRows.length!=0){
                this.getView().showTipNotification("分成部门：分成方式“按利润比例分成”、“按销售了计服务费”两种方式控制不能同时存在！");
                this.getModel().setValue("tpv_allocationmethod",null,selectRows[0]);
            }
        }
        this.getView().updateView("tpv_splitdepartment");
    }

    private void extracted() {
        //有效期从
        Date tpv_validfrom = (Date) this.getModel().getValue("tpv_validfrom");
        //有效期至
        Date tpv_validuntil = (Date) this.getModel().getValue("tpv_validuntil");
        if (tpv_validfrom != null && tpv_validuntil != null) {
            if (tpv_validuntil.compareTo(tpv_validfrom) <= 0) {
                this.getModel().beginInit();
//                this.getModel().setValue("tpv_validuntil", null);
                this.getModel().endInit();
            }
        }
    }

    private void extract() {
        //有效期从
        Date tpv_validfrom = (Date) this.getModel().getValue("tpv_swaplegdate");
        //有效期至
        Date tpv_validuntil = (Date) this.getModel().getValue("tpv_swaplegdatato");
        if (tpv_validfrom != null && tpv_validuntil != null) {
            if (tpv_validuntil.compareTo(tpv_validfrom) <= 0) {
                this.getModel().beginInit();
//                this.getModel().setValue("tpv_swaplegdatato", null);
                this.getModel().endInit();
            }
        }
    }
}
