package tpv.tpv_wm.tpv_tm.java.formplugin.list;

import com.itopview.wm.utils.BigDecimalUtils;
import com.itopview.wm.utils.RptUtil;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.events.BeforeCreateListDataProviderArgs;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.mvc.list.ListDataProvider;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;

import java.math.BigDecimal;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 策略号 -列表插件
 * @Author LinRui
 * @Date 2024/8/26 16:16
 */
public class StrategyListPlugin extends AbstractListPlugin {

    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
    }

    @Override
    public void beforeCreateListDataProvider(BeforeCreateListDataProviderArgs args) {
        super.beforeCreateListDataProvider(args);
        args.setListDataProvider(new ListDataProvider() {
            @Override
            public DynamicObjectCollection getData(int start, int limit) {
                DynamicObjectCollection rows = super.getData(start,limit);
                if(rows.isEmpty()){
                    return rows;
                }
                List<Long> ids = rows.stream().map(x -> x.getLong("id")).collect(Collectors.toList());
                DataSet purchase = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_purchase", "id,tpv_contractnumber,tpv_policynumber.id policyid," +
                                "tpv_policynumber.number policynumber,tpv_materialentry,tpv_materialentry.tpv_qty qty",
                        new QFilter[]{new QFilter("tpv_contractstatusx", QCP.in, new String[]{"A", "B", "C", "D", "F", "G"})
                                .and(new QFilter("tpv_policynumber.id", QCP.in, ids))}).select("policyid policyid","qty purqty","0.0 saleqty");
                DataSet salescontract = ORM.create().queryDataSet(this.getClass().getName(), "tpv_dt_salescontract", "id,tpv_contractnos,tpv_strategynum.id strategyid," +
                                "tpv_strategynum.number strategynum,tpv_materialentry,tpv_materialentry.tpv_qty qty",
                        new QFilter[]{new QFilter("tpv_contractstatus", QCP.in, new String[]{"A","B","C","D","G","H"})
                                .and(new QFilter("tpv_strategynum.id", QCP.in, ids))}).select("strategyid policyid","qty saleqty","0.0 purqty");
                DataSet dataSet = purchase.union(salescontract).groupBy(new String[]{"policyid"}).sum("saleqty").sum("purqty").finish();
                dataSet = RptUtil.zeroNullNums(dataSet);
                dataSet = dataSet.addField("(purqty - saleqty) qty", "qty");
                Map<Long, BigDecimal> map = new HashMap<>();
                if (!dataSet.isEmpty()) {
                    for (Row row : dataSet) {
                        if (!map.containsKey(row.getLong("policyid"))) {
                            map.put(row.getLong("policyid"), row.getBigDecimal("qty").setScale(2, BigDecimal.ROUND_HALF_UP).abs());
                        }
                    }
                }

                for (DynamicObject row : rows) {
                    Long id = row.getLong("id");
                    BigDecimal tpvReplylimit = BigDecimalUtils.toBigDecimal(row.getBigDecimal("tpv_replylimit")).setScale(2, BigDecimal.ROUND_HALF_UP);
                    if (map.containsKey(id)) {
                        BigDecimal qty = BigDecimalUtils.toBigDecimal(map.get(id));
                        row.set("tpv_openposition", tpvReplylimit.subtract(qty));//可开仓量
                        row.set("tpv_positionnums", qty);//持仓量
                    }
                }
                return rows;
            }
        });
    }

}
