package tpv.tpv_wm.tpv_tm.java.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @ClassName: FuturesrevocationSaveOp
 * @Description: 期货交易下单人撤销申请
 * 2、如果《期货交易下单人撤销、关闭申请（单据）》中【关闭/撤销】=“撤销”保存后，反写《期货交易下单人申请（单据）》{分录-下单人}【状态】=“撤销”，注意反向；
 * 3、如果《期货交易下单人撤销、关闭申请（单据）》中【关闭/撤销】=“关闭”保存后，反写《期货交易下单人申请（单据）》{分录-下单人}【状态】=“关闭”，且更新【关闭时间】为反写时间注意反向；
 * @author: zbh
 * @date: 2024-07-23
 */
public class FuturesrevocationSaveOp extends AbstractOperationServicePlugIn {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_clunsub");
        fieldKeys.add("tpv_sourceid");
        fieldKeys.add("tpv_tm_personnel");
        fieldKeys.add("tpv_sourcebillid");
        fieldKeys.add("tpv_bizdate");
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        String operationKey = e.getOperationKey();
        switch (operationKey) {
            //监听点击保存按钮
            case "save":
            case "submit":
                saveSubmit(e);
                break;
            case "delete":
                delete(e);
                break;
            default:
                break;
        }
    }

    private void delete(BeforeOperationArgs e) {
        DynamicObject[] entities = e.getDataEntities();
        for (DynamicObject entity : entities) {
            Long tpvSourcebillid = entity.getLong("tpv_sourcebillid");
            DynamicObject tpvTmFuturesdeal = BusinessDataServiceHelper.loadSingle(tpvSourcebillid, "tpv_tm_futuresdeal");
            DynamicObjectCollection tpvTmPersonnel = entity.getDynamicObjectCollection("tpv_tm_personnel");//人员
            List<Long> arrayList = new ArrayList<>();
            for (DynamicObject dynamicObject : tpvTmPersonnel) {
                arrayList.add(dynamicObject.getLong("tpv_sourceid"));
            }
            DynamicObjectCollection tpvTmPlacepeoples = tpvTmFuturesdeal.getDynamicObjectCollection("tpv_tm_placepeople");
            for (DynamicObject tpvTmPlacepeople : tpvTmPlacepeoples) {
                long id = tpvTmPlacepeople.getLong("id");
                if (arrayList.contains(id)) {
                    tpvTmPlacepeople.set("tpv_state", "A");
                    tpvTmPlacepeople.set("tpv_closedates", null);
                }
            }
            SaveServiceHelper.save(new DynamicObject[]{tpvTmFuturesdeal});
        }
    }

    private void saveSubmit(BeforeOperationArgs e) {
        DynamicObject[] entities = e.getDataEntities();
        for (DynamicObject entity : entities) {
            String tpvClunsub = entity.getString("tpv_clunsub");
            if (YEM.isNotEmpty(tpvClunsub)){
                Long tpvSourcebillid = entity.getLong("tpv_sourcebillid");
                Date tpvBizdate = entity.getDate("tpv_bizdate");
                DynamicObject tpvTmFuturesdeal = BusinessDataServiceHelper.loadSingle(tpvSourcebillid, "tpv_tm_futuresdeal");
                DynamicObjectCollection tpvTmPersonnel = entity.getDynamicObjectCollection("tpv_tm_personnel");//人员
                List<Long> arrayList = new ArrayList<>();
                for (DynamicObject dynamicObject : tpvTmPersonnel) {
                    arrayList.add(dynamicObject.getLong("tpv_sourceid"));
                }
                DynamicObjectCollection tpvTmPlacepeoples = tpvTmFuturesdeal.getDynamicObjectCollection("tpv_tm_placepeople");
                for (DynamicObject tpvTmPlacepeople : tpvTmPlacepeoples) {
                    long id = tpvTmPlacepeople.getLong("id");
                    if (arrayList.contains(id)) {
                        tpvTmPlacepeople.set("tpv_state", tpvClunsub);
                        if ("C".equals(tpvClunsub)) {
                            tpvTmPlacepeople.set("tpv_closedates", tpvBizdate);
                        }else {
                            tpvTmPlacepeople.set("tpv_closedates", null);
                        }
                    }
                }
                SaveServiceHelper.save(new DynamicObject[]{tpvTmFuturesdeal});
            }
        }
    }
}
