package tpv.tpv_wm.tpv_tm.java.opplugin.op;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: PassivityhedSaveOp
 * @Description: 被动套保申请单保存反写对应合同【多头买入被动套保单号】、【空头卖出被动套保单号】
 * @author: zbh
 * @date: 2024-05-09
 */
public class PassivityhedSaveOp extends AbstractOperationServicePlugIn {
    private static final String opValueKey1 = "tpv_purchase"; //触发弹框按钮标识
    private static final String opValueKey2 = "tpv_salescontract"; //触发弹框按钮标识
    private static final String opValueKey3 = "tpv_maxhomesales"; //触发弹框按钮标识

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_tm_agentcontract");  //分录标识
        fieldKeys.add("tpv_contractnumber");  //合同号
        fieldKeys.add("tpv_contractbillid");  //合同id
        fieldKeys.add("tpv_contractkey");  //合同类型(销售/采购合同)
        fieldKeys.add("tpv_homesales");  //本家销售合同(第三方销售原合同)
        fieldKeys.add("tpv_tm_contract");  //分录标识
        fieldKeys.add("tpv_tm_bear");  //分录标识
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        super.beforeExecuteOperationTransaction(e);
        e.getOperationKey();
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            DynamicObjectCollection tpvTmBear = bill.getDynamicObjectCollection("tpv_tm_bear");
            DynamicObjectCollection tpvTmContract = bill.getDynamicObjectCollection("tpv_tm_contract");
            if (tpvTmBear.size() > 0  || tpvTmContract.size() > 0) {
                paramContratNo(bill);
            }
        }
    }

    private void paramContratNo(DynamicObject bill) {
        Object pkValue = bill.getPkValue();
        if (YEM.isNotEmpty(pkValue)) {
            JSONObject jsonObject = new JSONObject();
            if (QueryServiceHelper.exists("tpv_tm_passivityhedging", bill.getPkValue())) {
                addPriceBackBill(jsonObject, bill); //记录单据界面入库前的临时数据
            }
            OperateOption option = this.getOption();
            if (option.getVariables().containsKey(opValueKey1)) {
                option.setVariableValue(opValueKey1, jsonObject.toJSONString());
            }
            if (option.getVariables().containsKey(opValueKey2)) {
                option.setVariableValue(opValueKey2, jsonObject.toJSONString());
            }
            if (option.getVariables().containsKey(opValueKey3)) {
                option.setVariableValue(opValueKey3, jsonObject.toJSONString());
            }
        }
    }

    private void addPriceBackBill(JSONObject jsonObject, DynamicObject bill) {
        DynamicObject tpvDtCostbillmat = BusinessDataServiceHelper.loadSingle(bill.getPkValue(), "tpv_tm_passivityhedging");
        DynamicObjectCollection tpvTmAgentcontract = tpvDtCostbillmat.getDynamicObjectCollection("tpv_tm_agentcontract");
        List<Long> purIds = new ArrayList<>();
        List<Long> saleIds = new ArrayList<>();
        for (DynamicObject dynamicObject : tpvTmAgentcontract) {
            String tpvContractkey = dynamicObject.getString("tpv_contractkey");
            if ("pur".equals(tpvContractkey)) {
                purIds.add(dynamicObject.getLong("tpv_contractbillid"));
            }
            if ("sale".equals(tpvContractkey)) {
                saleIds.add(dynamicObject.getLong("tpv_contractbillid"));
            }
        }
        jsonObject.put("pur", purIds);
        jsonObject.put("sale", saleIds);
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            DynamicObject bill = bills[i];
            DynamicObjectCollection tpvTmBear = bill.getDynamicObjectCollection("tpv_tm_bear");
            DynamicObjectCollection tpvTmContract = bill.getDynamicObjectCollection("tpv_tm_contract");
            Map<String, Object> value = new HashMap<>();
            if (tpvTmBear.size() > 0) {
                value.put("tpv_bear",true);
            } else {
                value.put("tpv_bear",false);
            }
            if (tpvTmContract.size() > 0) {
                value.put("tpv_multipoint",true);
            } else {
                value.put("tpv_multipoint",false);
            }
            backContratNo(bill, value);  //反写对应合同的【多头买入被动套保单号】、【空头卖出被动套保单号】字段
        }
    }

    private void backContratNo(DynamicObject bill, Map<String, Object> value) {
        OperateOption option = this.getOption();
        JSONObject jsonObject = new JSONObject();
        if (option.containsVariable(opValueKey1)) {
            jsonObject = JSON.parseObject(option.getVariableValue(opValueKey1));
        }
        if (option.containsVariable(opValueKey2)) {
            jsonObject = JSON.parseObject(option.getVariableValue(opValueKey2));
        }
        if (option.containsVariable(opValueKey3)) {
            jsonObject = JSON.parseObject(option.getVariableValue(opValueKey3));
        }
        if (QueryServiceHelper.exists("tpv_tm_passivityhedging", bill.getPkValue())) {
            addPriceBackBill(jsonObject, bill);//记录单据界面入库后的数据
        }
        DynamicObject tpvDtCostbillmat = BusinessDataServiceHelper.loadSingle(bill.getPkValue(), "tpv_tm_passivityhedging");
        DynamicObjectCollection tpvTmAgentcontract = tpvDtCostbillmat.getDynamicObjectCollection("tpv_tm_agentcontract");
        Map<String, List<Long>> conMap = new HashMap<>();
        conMap.put("pur", new ArrayList<>());
        conMap.put("sale", new ArrayList<>());
        for (DynamicObject dynamicObject : tpvTmAgentcontract) {
            String tpvContractkey = dynamicObject.getString("tpv_contractkey");
            if ("pur".equals(tpvContractkey)) {
                if (!conMap.get("pur").contains(dynamicObject.getLong("tpv_contractbillid"))) {
                    conMap.get("pur").add(dynamicObject.getLong("tpv_contractbillid"));
                }
            }
            if ("sale".equals(tpvContractkey)) {
                if (!conMap.get("sale").contains(dynamicObject.getLong("tpv_contractbillid"))) {
                    conMap.get("sale").add(dynamicObject.getLong("tpv_contractbillid"));
                }
            }
        }
        JSONArray purObject = jsonObject.getJSONArray("pur");
        List<Long> purIds = new ArrayList<>();
        if (null != purObject) {
            if (purObject.size() > 0) {
                for (Object o : purObject) {
                    purIds.add(Long.valueOf(o.toString()));
                }
                QFilter qFilter = new QFilter("id",QCP.in,purIds);
                DynamicObject[] query = BusinessDataServiceHelper.load("tpv_dt_purchase",
                        "id,billno,tpv_contractnumber,tpv_multipoint,tpv_bear", qFilter.toArray());
                if (query.length > 0) {
                    for (DynamicObject dynamicObject : query) {
                        Long aLong = dynamicObject.getLong("id");
                        if (conMap.get("pur").contains(aLong)) {
                            boolean tpvMultipoint = (boolean) value.get("tpv_multipoint");
                            if (tpvMultipoint) {
                                dynamicObject.set("tpv_multipoint",bill.getString("billno")); //多头买入被动套保单号
                            } else {
                                dynamicObject.set("tpv_multipoint",""); //多头买入被动套保单号
                            }
                            boolean tpvBear = (boolean) value.get("tpv_bear");
                            if (tpvBear) {
                                dynamicObject.set("tpv_bear",bill.getString("billno")); //空头卖出被动套保单号
                            } else {
                                dynamicObject.set("tpv_bear",""); //空头卖出被动套保单号
                            }
                        } else {
                            dynamicObject.set("tpv_multipoint",""); //多头买入被动套保单号
                            dynamicObject.set("tpv_bear",""); //空头卖出被动套保单号
                        }
                    }
                }
                SaveServiceHelper.save(query);
            }

        }
        JSONArray saleObject = jsonObject.getJSONArray("sale");
        List<Long> saleIds = new ArrayList<>();
        if (null != saleObject) {
            if (saleObject.size() > 0) {
                for (Object o : saleObject) {
                    saleIds.add(Long.valueOf(o.toString()));
                }
            }
            QFilter qFilter = new QFilter("id",QCP.in,saleIds);
            DynamicObject[] query = BusinessDataServiceHelper.load("tpv_dt_salescontract",
                    "id,billno,tpv_contractnos,tpv_multipoint,tpv_bear", qFilter.toArray());
            if (query.length > 0) {
                for (DynamicObject dynamicObject : query) {
                    Long aLong = dynamicObject.getLong("id");
                    if (conMap.get("sale").contains(aLong)) {
                        boolean tpvMultipoint = (boolean) value.get("tpv_multipoint");
                        if (tpvMultipoint) {
                            dynamicObject.set("tpv_multipoint",bill.getString("billno")); //多头买入被动套保单号
                        } else {
                            dynamicObject.set("tpv_multipoint",""); //多头买入被动套保单号
                        }
                        boolean tpvBear = (boolean) value.get("tpv_bear");
                        if (tpvBear) {
                            dynamicObject.set("tpv_bear",bill.getString("billno")); //空头卖出被动套保单号
                        } else {
                            dynamicObject.set("tpv_bear",""); //空头卖出被动套保单号
                        }
                    } else {
                        dynamicObject.set("tpv_multipoint",""); //多头买入被动套保单号
                        dynamicObject.set("tpv_bear",""); //空头卖出被动套保单号
                    }
                }
            }
            SaveServiceHelper.save(query);
        }
    }
}
