package tpv.tpv_wm.tpv_tm.java.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.servicehelper.operation.OperationServiceHelper;

import java.util.List;

/**
 * @ClassName: PassivityhedSubmitOp
 * @Description: 被动套保申请单提交 -校验合约分录或空头卖出分录至少有一条数据
 * @author: zbh
 * @date: 2024-05-09
 */
public class PassivityhedSubmitOp extends AbstractOperationServicePlugIn {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_tm_contract");  //分录标识
        fieldKeys.add("tpv_tm_bear");  //分录标识
        fieldKeys.add("tpv_attachmentdetail");  //分录标识
        fieldKeys.add("tpv_attachmenttype");  //附件状态
        fieldKeys.add("tpv_uploadstamped");  //是否已上传客户盖章件
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new AbstractValidator() {

            @Override
            public void validate() {
                ExtendedDataEntity[] bills = this.getDataEntities();
                for (int i = 0; i < bills.length; i++) {
                    ExtendedDataEntity bill = bills[i];
                    DynamicObject dynamicObject = bill.getDataEntity();
                    DynamicObjectCollection tpvTmBear = dynamicObject.getDynamicObjectCollection("tpv_tm_bear");
                    DynamicObjectCollection tpvTmContract = dynamicObject.getDynamicObjectCollection("tpv_tm_contract");
                    if (tpvTmBear.size() == 0  && tpvTmContract.size() == 0) {
                        this.addMessage(bill,"【合约】或【空头卖出】分录至少存在一条数据，才能允许提交，请检查!");
                    }
                }
            }
        });
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        DynamicObject[] bills = e.getDataEntities();
        for (int i = 0; i < bills.length; ++i) {
            Boolean flag = false;
            DynamicObject bill = bills[i];
            DynamicObjectCollection entryRows = bill.getDynamicObjectCollection("tpv_attachmentdetail");
            for (DynamicObject entryRow : entryRows) {
                DynamicObject tpvAttachmenttype = entryRow.getDynamicObject("tpv_attachmenttype");
                if (YEM.isNotEmpty(tpvAttachmenttype)) {
                    if ("009".equals(tpvAttachmenttype.getString("number"))) {
                        flag = true;
                        break;
                    }
                }
            }
            if (flag) {
                bill.set("tpv_uploadstamped", flag);
            } else {
                bill.set("tpv_uploadstamped", flag);
            }
            OperationServiceHelper.executeOperate("save", bill.getDataEntityType().getName(),
                    new DynamicObject[]{bill}, YEM.getOptionNoMsg());
        }
    }
}
