package tpv.tpv_wm.tpv_tm.java.opplugin.op;

import com.itopview.wm.utils.YEM;
import kd.bos.data.BusinessDataReader;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.IDataEntityType;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.MainEntityType;
import kd.bos.entity.botp.runtime.ConvertOperationResult;
import kd.bos.entity.botp.runtime.PushArgs;
import kd.bos.entity.datamodel.IRefrencedataProvider;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.form.operate.botp.Push;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.botp.ConvertServiceHelper;
import kd.bos.servicehelper.operation.DeleteServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: UnClaimdetailsAuditOp
 * @Description: 取消认领审批单
 * 《取消认领审批单》审核后，再将原《认领明细》【红冲标志】修改为【被红冲】，同时生成新的《认领明细单》（同原认领明细单一模一样）且【红冲标志】为空，
 * 如果《认领明细》为“暂存”状态，则直接删除；
 * 【取消认领】在任何状态下都可以编辑；
 * 另外《取消认领审批单》【单据状态】增加“作废”，增加按钮【作废】，点击【作废】按钮后，将【单据状态】变为作废。同时将原《认领明细》【红冲标志】变为空。
 * 《取消认领审批单》反审核时，将【单据状态】变为作废。需将生成新的《认领明细》删除，同时将原《认领明细》【红冲标志】变为空。
 * @author: zbh
 * @date: 2024-07-29
 */
public class UnClaimdetailsAuditOp extends AbstractOperationServicePlugIn {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("tpv_sourcebillid");
        fieldKeys.add("tpv_sourcebillkey");
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);
        String key = e.getOperationKey();
        switch (key) {
            case "audit":
                audit(e);
                break;
            case "cancellation":
                cancellation(e);
                break;
            case "unaudit":
                unaudit(e);
                break;
        }
    }

    private void unaudit(AfterOperationArgs e) {
        DynamicObject[] entities = e.getDataEntities();
        for (DynamicObject entity : entities) {
            String tpvSourcebillkey = entity.getString("tpv_sourcebillkey");//来源单据标识
            long tpvSourcebillid = entity.getLong("tpv_sourcebillid");//来源单据ID
            if (YEM.isNotEmpty(tpvSourcebillid) && tpvSourcebillid != 0 && YEM.isNotEmpty(tpvSourcebillkey)) {
                DynamicObject tpvTmClaimdetails = BusinessDataServiceHelper.loadSingle(tpvSourcebillid, "tpv_tm_claimdetails");
                tpvTmClaimdetails.set("tpv_redflush", null);
                SaveServiceHelper.save(new DynamicObject[]{tpvTmClaimdetails});
                DeleteServiceHelper.delete("tpv_tm_claimdetails",new QFilter[]{new QFilter("tpv_sourcebillid","=",tpvSourcebillid)});
            }
        }
    }

    private void cancellation(AfterOperationArgs e) {
        DynamicObject[] entities = e.getDataEntities();
        for (DynamicObject entity : entities) {
            long tpvSourcebillid = entity.getLong("tpv_sourcebillid");//来源单据ID
            if (YEM.isNotEmpty(tpvSourcebillid) && tpvSourcebillid != 0) {
                DynamicObject tpvTmClaimdetails = BusinessDataServiceHelper.loadSingle(tpvSourcebillid, "tpv_tm_claimdetails");
                tpvTmClaimdetails.set("tpv_redflush", null);
                SaveServiceHelper.save(new DynamicObject[]{tpvTmClaimdetails});
                entity.set("billstatus", "D");
            }
        }
        SaveServiceHelper.save(entities);
    }

    private void audit(AfterOperationArgs e) {
        DynamicObject[] entities = e.getDataEntities();
        for (DynamicObject entity : entities) {
            long tpvSourcebillid = entity.getLong("tpv_sourcebillid");//来源单据ID
            if (YEM.isNotEmpty(tpvSourcebillid)) {
                DynamicObject tpvTmClaimdetails = BusinessDataServiceHelper.loadSingle(tpvSourcebillid, "tpv_tm_claimdetails");
                tpvTmClaimdetails.set("tpv_redflush", "B");
                SaveServiceHelper.save(new DynamicObject[]{tpvTmClaimdetails});

                List<ListSelectedRow> selectedRowsTwo = new ArrayList<>();
                ListSelectedRow listSelectedRowTwo = new ListSelectedRow(tpvSourcebillid);//来源单据ID
                selectedRowsTwo.add(listSelectedRowTwo);
                PushArgs pushArgsTwo = new PushArgs();
                pushArgsTwo.setSourceEntityNumber("tpv_tm_claimdetails");
                pushArgsTwo.setTargetEntityNumber("tpv_tm_claimdetails");
                pushArgsTwo.setAutoSave(true);//是否保存
                pushArgsTwo.setRuleId("2005224221759775744"); // 可选，传入本次下推使用的转换规则id；传入空值，由系统自动寻找合适的转换规则
                pushArgsTwo.setBuildConvReport(true); // 是否输出详细错误报告
                pushArgsTwo.setSelectedRows(selectedRowsTwo); // 必选，设置需要下推的源单及分录内码
                ConvertOperationResult pushResultTwo = ConvertServiceHelper.push(pushArgsTwo);

                if (pushResultTwo.isSuccess()) {
                    MainEntityType targetMainTypeOne = EntityMetadataCache
                            .getDataEntityType(pushArgsTwo.getTargetEntityNumber());
                    List<DynamicObject> targetBillObjsOne = pushResultTwo
                            .loadTargetDataObjects(new IRefrencedataProvider() {
                                @Override
                                public void fillReferenceData(Object[] objs, IDataEntityType dType) {
                                    BusinessDataReader.loadRefence(objs, dType);
                                }
                            }, targetMainTypeOne);
                    if (targetBillObjsOne.size() > 0) {
                        DynamicObject targetBillObjOne = targetBillObjsOne.get(0);
                        OperationServiceHelper.executeOperate("save", "tpv_tm_claimdetails",
                                new DynamicObject[]{targetBillObjOne}, YEM.getOption());
                    }
                }
            }

        }
    }
}
