package tpv.tpv_wm.tpv_tm.java.opplugin.validator;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.validate.AbstractValidator;

import java.math.BigDecimal;
import java.util.Arrays;

/**
 * @Description: 基础资料《策略号》提交时校验 分录- 分成部门 中 分成比例 为 100%
 * @Project: zhck
 * @Package: tpv.tpv_wm.tpv_tm.java.opplugin.validator.StrategyValidator
 * @Date: 2024/04/08 11:21
 * @Author: Heisenberg Wenbei He
 * @Version: 1.0
 */
public class StrategyValidator extends AbstractValidator {

    @Override
    public void validate() {
        if (!"submit".equals(this.getOperateKey())) {
            return;
        }

        Arrays.stream(this.getDataEntities())
                .filter(single -> single.getDataEntity().getDate("tpv_swaplegdate") != null
                        && single.getDataEntity().getDate("tpv_swaplegdatato") != null)
                .filter(single -> single.getDataEntity().getDate("tpv_swaplegdate")
                        .after(single.getDataEntity().getDate("tpv_swaplegdatato")))
                .forEach(single -> this.addMessage(single, "【套保期现从】日期必须小于【套保期现到】日期！"));

        Arrays.stream(this.getDataEntities())
                .filter(single -> single.getDataEntity().getDate("tpv_validfrom") != null
                        && single.getDataEntity().getDate("tpv_validuntil") != null)
                .filter(single -> single.getDataEntity().getDate("tpv_validfrom")
                        .after(single.getDataEntity().getDate("tpv_validuntil")))
                .forEach(single -> this.addMessage(single, "【有效期从】日期必须小于【有效期至】日期！"));

        Arrays.stream(this.getDataEntities())
                .filter(single -> single.getDataEntity().getDate("tpv_validuntilfrom") != null
                        && single.getDataEntity().getDate("tpv_validuntilapp") != null)
                .filter(single -> single.getDataEntity().getDate("tpv_validuntilfrom")
                        .after(single.getDataEntity().getDate("tpv_validuntilapp")))
                .forEach(single -> this.addMessage(single, "【批复有效期从】日期必须小于【批复有效期至】日期！"));

        Arrays.stream(this.getDataEntities())
                .filter(single -> single.getDataEntity().getDynamicObjectCollection("tpv_splitdepartment").stream()
                        .filter(row -> "C".equals(row.getString("tpv_allocationmethod")))
                        .map(row -> row.getBigDecimal("tpv_apercentage"))
                        .reduce(BigDecimal::add)
                        .filter(sum -> sum.compareTo(new BigDecimal(100)) != 0)
                        .isPresent()
                )
                .forEach(single -> addMessage(single, "分成部门中分成比例总和需为100%！"));

        Arrays.stream(this.getDataEntities())
                .filter(single -> single.getDataEntity().getDynamicObjectCollection("tpv_bd_strategy_xt").stream()
                        .map(row -> row.getBigDecimal("tpv_proportion"))
                        .reduce(BigDecimal::add)
                        .filter(sum -> sum.compareTo(new BigDecimal(100)) != 0)
                        .isPresent()
                )
                .forEach(single -> addMessage(single, "协同部门中协同比例和需为100%！"));

        ExtendedDataEntity[] entities = this.getDataEntities();
        for (ExtendedDataEntity entity : entities) {
            DynamicObject dataEntity = entity.getDataEntity();
            Boolean tpvCooperate = dataEntity.getBoolean("tpv_cooperate");
            DynamicObjectCollection strategySt = dataEntity.getDynamicObjectCollection("tpv_bd_strategy_xt");
            if (tpvCooperate){
                if (strategySt.size()==0){
                    this.addMessage(entity, "协同部门未空");
                }
            }
        }
    }
}
