package tpv.tpv_wm.tpv_wms.java.formplugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.IDataModel;
import kd.bos.form.FormShowParameter;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.EventObject;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @Description: 转口物流工作台采购装运情况分录(PC布局)
 * @Auther: Guo jiaChen
 */
public class TransformBenchPCFormPlugin extends AbstractBillPlugIn {
    private static final String ENTRY_PURCHASE = "tpv_entryentity";//采购装运情况分录
    private static final String ENTRY_SALE = "tpv_entryentity1";//采购装运情况分录 


    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);
        String[] itemListener = {"tpv_advcontoolbarap31", "tpv_advcontoolbarap4"};
        this.addItemClickListeners(itemListener);
    }

    @Override
    public void afterBindData(EventObject e) {
        super.afterCreateNewData(e);
        FormShowParameter parameter = this.getView().getFormShowParameter();
        String number = parameter.getCustomParam("groupnumber");
        if (null != number) {
            DynamicObjectCollection purchase = this.getModel().getEntryEntity(ENTRY_PURCHASE);
            DynamicObjectCollection sale = this.getModel().getEntryEntity(ENTRY_SALE);
            if (purchase.size() == 0) {
                purchaseQuery(number);
            }
            if (sale.size() == 0) {
                saleQuery(number);
            }
        }
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        super.itemClick(evt);
        String key = evt.getItemKey();
        long start = System.currentTimeMillis();
        switch (key) {
            case "tpv_addentry"://采购增行
                purchaseQuery(null);
                break;
            case "tpv_addentrys"://销售增行
                saleQuery(null);
                break;
            default:
                break;
        }
        long end = System.currentTimeMillis();
        //this.getView().showTipNotification("查询时间:" + (end - start));
    }


    /**
     * 销售合同
     */
    private void saleQuery(String number) {
        String combinationNumber = "";
        if (number == null || number.length() == 0) {
            combinationNumber = (String) this.getModel().getValue("number");
        } else {
            combinationNumber = number;
        }
        QFilter[] qFilter = new QFilter[]{new QFilter("billstatus", QCP.equals, "C"),
                new QFilter("tpv_combinationnumber.number", QCP.equals, combinationNumber),
                new QFilter("tpv_customer.tpv_enterprise", QCP.equals, false)};
        DynamicObject[] queryCG = BusinessDataServiceHelper.load("tpv_dt_salescontract", "tpv_materialentry,tpv_combinationnumber.number,tpv_qty,tpv_customer,tpv_contractnos,tpv_portofloading,tpv_destinationport,tpv_unit", qFilter);
        IDataModel model = this.getModel();
        if (queryCG.length > 0) {
            int[] ints = this.getModel().batchCreateNewEntryRow(ENTRY_SALE, queryCG.length);//货物流转分录
            int i = 0;
            for (DynamicObject obj : queryCG) {
                DynamicObjectCollection collection = obj.getDynamicObjectCollection("tpv_materialentry");
                Integer sum = calculate(collection, "tpv_qty");//计算总数
                DynamicObject customer = obj.getDynamicObject("tpv_customer");
                DynamicObject portofloading = obj.getDynamicObject("tpv_portofloading");//起运港
                DynamicObject destinationport = obj.getDynamicObject("tpv_destinationport");//目的港
                String contractnos = obj.getString("tpv_contractnos");
                model.setValue("tpv_salenumber", contractnos, ints[i]);//销售合同号
                model.setValue("tpv_customer", customer, ints[i]);//客户
                model.setValue("tpv_startports", portofloading, ints[i]);//起运港
                model.setValue("tpv_endports", destinationport, ints[i]);//目的港
                model.setValue("tpv_units", customer, ints[i]);//单位
                model.setValue("tpv_startnums", sum, ints[i]);//起运数量
                ++i;
            }
        } else {
            this.getView().showTipNotification("销售装运情况分录,暂无数据可以添加!");
        }

    }

    private Integer calculate(DynamicObjectCollection collection, String name) {
        AtomicInteger sum = new AtomicInteger();
        collection.stream().forEach(dyObj -> {
            sum.addAndGet(dyObj.getInt(name));
        });
        return sum.get();
    }

    /**
     * 采购合同
     */
    private void purchaseQuery(String number) {
        String combinationNumber = "";
        if (number == null || number.length() == 0) {
            combinationNumber = (String) this.getModel().getValue("number");
        } else {
            combinationNumber = number;
        }
        QFilter[] qFilter = new QFilter[]{new QFilter("billstatus", QCP.equals, "C"),
                new QFilter("tpv_combinationnumber.number", QCP.equals, combinationNumber),
                new QFilter("tpv_supplier.tpv_sfjtnbqy", QCP.equals, false)};
        DynamicObject[] queryCG = BusinessDataServiceHelper.load("tpv_dt_purchase", "tpv_supplier,tpv_contractnumber,tpv_whetheitisshipped,createtime", qFilter);
        IDataModel model = this.getModel();
        if (queryCG.length > 0) {
            int[] ints = this.getModel().batchCreateNewEntryRow(ENTRY_PURCHASE, queryCG.length);//货物流转分录
            int i = 0;
            for (DynamicObject obj : queryCG) {
                DynamicObject supplier = obj.getDynamicObject("tpv_supplier");
                String contractnumber = obj.getString("tpv_contractnumber");
                //model.setValue("tpv_startdate", createtime, ints[i]);//起运日期
                model.setValue("tpv_purnumber", contractnumber, ints[i]);//采购合同号
                model.setValue("tpv_psupplier", supplier, ints[i]);//供应商
                ++i;
            }
        } else {
            this.getView().showTipNotification("采购装运情况分录,暂无数据可以添加!");
        }
    }
}
