package com.itopview.wm.cm.customplugin;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName: Poc3edYemInterface
 * @Description: 已收金额：按本销售合同，取收款记录（费用项目见下图），作为已收项目来源1
 * 按本销售合同下属的第三方销售合同，取收款记录（费用项目见下图），但金额需要处理为“回款金额*（本销售合同签约均价）/（第三方销售合同签约均价）”，作为已收项目来源2
 * @author: zbh
 * @date: 2024-05-23
 */
@YemImplementation("Poc3edYemInterface")
public class Poc3edYemInterface implements YemInterface {
    @Override
    public BigDecimal getBigDecimal(Map map) {
        BigDecimal amt = BigDecimal.ZERO;
        long contract_id = (long) map.get("contract_id");
        if (0L != contract_id) {
            DynamicObject contract = BusinessDataServiceHelper.loadSingle(contract_id, "tpv_dt_salescontract");
            BigDecimal amount = contract.getBigDecimal("tpv_amount");
            BigDecimal decimal = BigDecimal.ZERO;
            DynamicObject[] salesContract = BusinessDataServiceHelper.load("tpv_dt_salescontract", "id", new QFilter[]{new QFilter("tpv_materialentry.tpv_srcbillid", "=", contract.getPkValue()), new QFilter("tpv_thirdparty", "=", true)});
            List<Long> saleIds = Arrays.stream(salesContract).map(x -> x.getLong("id")).collect(Collectors.toList());
            saleIds.add(contract.getLong("id"));
            List<String> listName = new ArrayList<>();
            listName.add("收销售合同保证金");
            listName.add("国外销售合同货款（预收）");
            listName.add("国外销售合同货款（应收）");
            listName.add("国内销售合同货款");
            DynamicObject[] expenseitemedit = BusinessDataServiceHelper.load("er_expenseitemedit", "id", new QFilter[]{new QFilter("name", "in", listName)});
            List<Long> expenIds = Arrays.stream(expenseitemedit).map(x -> x.getLong("id")).collect(Collectors.toList());
            DynamicObject[] casRecbills = BusinessDataServiceHelper.load("cas_recbill", "shkd_contractamount,entry,shkd_salecontract,e_expenseitem", new QFilter[]{new QFilter("entry.shkd_salecontract", "in", saleIds), new QFilter("entry.e_expenseitem", "in", expenIds)});
            for (DynamicObject casRecbill : casRecbills) {
                DynamicObjectCollection entrys = casRecbill.getDynamicObjectCollection("entry");
                for (DynamicObject entry : entrys) {
                    DynamicObject eExpenseitem = entry.getDynamicObject("e_expenseitem");
                    DynamicObject shkdSalecontract = entry.getDynamicObject("shkd_salecontract");
                    if (eExpenseitem == null && shkdSalecontract == null) {
                        continue;
                    }
                    if (expenIds.contains(eExpenseitem.getLong("id")) && saleIds.contains(shkdSalecontract.getLong("id"))){
                        if (shkdSalecontract.getLong("id")==contract.getLong("id")){
                            decimal=decimal.add(entry.getBigDecimal("shkd_contractamount"));
                        }else {
                            amt=amt.add(entry.getBigDecimal("shkd_contractamount"));
                        }
                    }
                }
            }
            if (amt.compareTo(BigDecimal.ZERO)!=0){
                amt=amount.divide(amt,2, BigDecimal.ROUND_HALF_UP);
            }
            amt=amount.add(decimal);
        }

        return amt;
    }

    @Override
    public boolean getBoolean(Map map) {
        return false;
    }

    @Override
    public String getString(Map map) {
        return null;
    }

    @Override
    public Date getDate(Map map) {
        return null;
    }

    @Override
    public long getLong(Map map) {
        return 0;
    }

    @Override
    public Map getMap(Map map) {
        return null;
    }

    @Override
    public List getList(Map map) {
        return null;
    }
}
